@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2022 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,16 +32,16 @@
 #include "Disassembler.h"
 #include "JITCode.h"
 #include "Options.h"
-
-#if OS(LINUX)
 #include "PerfLog.h"
-#endif
+#include <wtf/TZoneMallocInlines.h>
 
 namespace JSC {
 
 size_t LinkBuffer::s_profileCummulativeLinkedSizes[LinkBuffer::numberOfProfiles];
 size_t LinkBuffer::s_profileCummulativeLinkedCounts[LinkBuffer::numberOfProfiles];
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(LinkBuffer);
+
 LinkBuffer::CodeRef<LinkBufferPtrTag> LinkBuffer::finalizeCodeWithoutDisassemblyImpl()
 {
     performFinalization();
@@ -57,18 +57,20 @@ LinkBuffer::CodeRef<LinkBufferPtrTag> LinkBuffer::finalizeCodeWithDisassemblyImp
 {
     CodeRef<LinkBufferPtrTag> result = finalizeCodeWithoutDisassemblyImpl();
 
-#if OS(LINUX)
+#if OS(LINUX) || OS(DARWIN)
     if (Options::logJITCodeForPerf()) {
         StringPrintStream out;
         va_list argList;
         va_start(argList, format);
-        va_start(argList, format);
         out.vprintf(format, argList);
         va_end(argList);
         PerfLog::log(out.toCString(), result.code().untaggedPtr<const uint8_t*>(), result.size());
     }
 #endif
 
+    if (!dumpDisassembly && !Options::logJIT())
+        return result;
+
     bool justDumpingHeader = !dumpDisassembly || m_alreadyDisassembled;
 
     StringPrintStream out;
@@ -95,11 +97,10 @@ LinkBuffer::CodeRef<LinkBufferPtrTag> LinkBuffer::finalizeCodeWithDisassemblyImp
         out.vprintf(format, argList);
 
     va_end(argList);
-    out.printf(":\n");
 
     uint8_t* executableAddress = result.code().untaggedPtr<uint8_t*>();
-    out.printf("    Code at [%p, %p)%s\n", executableAddress, executableAddress + result.size(), justDumpingHeader ? "." : ":");
-    
+    out.printf(": [%p, %p) %zu bytes%s\n", executableAddress, executableAddress + result.size(), result.size(), justDumpingHeader ? "." : ":");
+
     CString header = out.toCString();
     
     if (justDumpingHeader) {
@@ -142,7 +143,7 @@ static std::once_flag flag;
     return *threadSpecificBranchCompactionLinkBufferPtr;
 }
 
-DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER(BranchCompactionLinkBuffer);
+DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER_AND_EXPORT(BranchCompactionLinkBuffer, WTF_INTERNAL);
 DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(BranchCompactionLinkBuffer);
 
 class BranchCompactionLinkBuffer {
@@ -216,18 +217,15 @@ private:
 
 static ALWAYS_INLINE void recordLinkOffsets(AssemblerData& assemblerData, int32_t regionStart, int32_t regionEnd, int32_t offset)
 {
+#if OS(DARWIN)
+    memset_pattern4(bitwise_cast<uint8_t*>(assemblerData.buffer()) + regionStart, &offset, regionEnd - regionStart);
+#else
     int32_t ptr = regionStart / sizeof(int32_t);
     const int32_t end = regionEnd / sizeof(int32_t);
     int32_t* offsets = reinterpret_cast_ptr<int32_t*>(assemblerData.buffer());
     while (ptr < end)
         offsets[ptr++] = offset;
-}
-
-// We use this to prevent compile errors on some platforms that are unhappy
-// about the signature of the system's memcpy.
-ALWAYS_INLINE void* memcpyWrapper(void* dst, const void* src, size_t bytes)
-{
-    return memcpy(dst, src, bytes);
+#endif
 }
 
 template <typename InstructionType>
@@ -238,7 +236,7 @@ void LinkBuffer::copyCompactAndLinkCode(MacroAssembler& macroAssembler, JITCompi
     if (didFailToAllocate())
         return;
 
-    Vector<LinkRecord, 0, UnsafeVectorOverflow>& jumpsToLink = macroAssembler.jumpsToLink();
+    auto& jumpsToLink = macroAssembler.jumpsToLink();
     m_assemblerStorage = macroAssembler.m_assembler.buffer().releaseAssemblerData();
     uint8_t* inData = bitwise_cast<uint8_t*>(m_assemblerStorage.buffer());
 #if CPU(ARM64E)
@@ -276,11 +274,12 @@ void LinkBuffer::copyCompactAndLinkCode(MacroAssembler& macroAssembler, JITCompi
 
     if (m_shouldPerformBranchCompaction) {
         for (unsigned i = 0; i < jumpCount; ++i) {
+            auto& linkRecord = jumpsToLink[i];
             int offset = readPtr - writePtr;
             ASSERT(!(offset & 1));
-                
+
             // Copy the instructions from the last jump to the current one.
-            size_t regionSize = jumpsToLink[i].from() - readPtr;
+            size_t regionSize = linkRecord.from() - readPtr;
             InstructionType* copySource = reinterpret_cast_ptr<InstructionType*>(inData + readPtr);
             InstructionType* copyEnd = reinterpret_cast_ptr<InstructionType*>(inData + readPtr + regionSize);
             InstructionType* copyDst = reinterpret_cast_ptr<InstructionType*>(outData + writePtr);
@@ -291,38 +290,32 @@ void LinkBuffer::copyCompactAndLinkCode(MacroAssembler& macroAssembler, JITCompi
                 InstructionType insn = read(copySource++);
                 *copyDst++ = insn;
             }
-            recordLinkOffsets(m_assemblerStorage, readPtr, jumpsToLink[i].from(), offset);
+            recordLinkOffsets(m_assemblerStorage, readPtr, linkRecord.from(), offset);
             readPtr += regionSize;
             writePtr += regionSize;
                 
             // Calculate absolute address of the jump target, in the case of backwards
             // branches we need to be precise, forward branches we are pessimistic
             const uint8_t* target;
-#if CPU(ARM64)
-            const intptr_t to = jumpsToLink[i].to(&macroAssembler.m_assembler);
-#else
-            const intptr_t to = jumpsToLink[i].to();
-#endif
-            if (to >= jumpsToLink[i].from())
+            const intptr_t to = linkRecord.to(&macroAssembler.m_assembler);
+            if (linkRecord.isThunk())
+                target = bitwise_cast<uint8_t*>(to);
+            else if (to >= linkRecord.from())
                 target = codeOutData + to - offset; // Compensate for what we have collapsed so far
             else
                 target = codeOutData + to - executableOffsetFor(to);
 
-            JumpLinkType jumpLinkType = MacroAssembler::computeJumpType(jumpsToLink[i], codeOutData + writePtr, target);
+            JumpLinkType jumpLinkType = MacroAssembler::computeJumpType(linkRecord, codeOutData + writePtr, target);
             // Compact branch if we can...
-            if (MacroAssembler::canCompact(jumpsToLink[i].type())) {
+            if (MacroAssembler::canCompact(linkRecord.type())) {
                 // Step back in the write stream
-                int32_t delta = MacroAssembler::jumpSizeDelta(jumpsToLink[i].type(), jumpLinkType);
+                int32_t delta = MacroAssembler::jumpSizeDelta(linkRecord.type(), jumpLinkType);
                 if (delta) {
                     writePtr -= delta;
-                    recordLinkOffsets(m_assemblerStorage, jumpsToLink[i].from() - delta, readPtr, readPtr - writePtr);
+                    recordLinkOffsets(m_assemblerStorage, linkRecord.from() - delta, readPtr, readPtr - writePtr);
                 }
             }
-#if CPU(ARM64)
-            jumpsToLink[i].setFrom(&macroAssembler.m_assembler, writePtr);
-#else
-            jumpsToLink[i].setFrom(writePtr);
-#endif
+            linkRecord.setFrom(&macroAssembler.m_assembler, writePtr);
         }
     } else {
         if (ASSERT_ENABLED) {
@@ -351,17 +344,18 @@ void LinkBuffer::copyCompactAndLinkCode(MacroAssembler& macroAssembler, JITCompi
     recordLinkOffsets(m_assemblerStorage, readPtr, initialSize, readPtr - writePtr);
         
     for (unsigned i = 0; i < jumpCount; ++i) {
-        uint8_t* location = codeOutData + jumpsToLink[i].from();
-#if CPU(ARM64)
-        const intptr_t to = jumpsToLink[i].to(&macroAssembler.m_assembler);
-#else
-        const intptr_t to = jumpsToLink[i].to();
-#endif
-        uint8_t* target = codeOutData + to - executableOffsetFor(to);
-        if (g_jscConfig.useFastJITPermissions)
-            MacroAssembler::link<memcpyWrapper>(jumpsToLink[i], outData + jumpsToLink[i].from(), location, target);
+        auto& linkRecord = jumpsToLink[i];
+        uint8_t* location = codeOutData + linkRecord.from();
+        const intptr_t to = linkRecord.to(&macroAssembler.m_assembler);
+        uint8_t* target = nullptr;
+        if (linkRecord.isThunk())
+            target = bitwise_cast<uint8_t*>(to);
         else
-            MacroAssembler::link<performJITMemcpy>(jumpsToLink[i], outData + jumpsToLink[i].from(), location, target);
+            target = codeOutData + to - executableOffsetFor(to);
+        if (g_jscConfig.useFastJITPermissions)
+            MacroAssembler::link<memcpyWrapper>(linkRecord, outData + linkRecord.from(), location, target);
+        else
+            MacroAssembler::link<performJITMemcpy>(linkRecord, outData + linkRecord.from(), location, target);
     }
 
     size_t compactSize = writePtr + initialSize - readPtr;
