@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,11 +28,15 @@
 
 #if ENABLE(ASSEMBLER)
 
+#include <wtf/TZoneMallocInlines.h>
+
 namespace JSC {
 namespace Probe {
 
 static void flushDirtyStackPages(State*);
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(Context);
+
 void executeJSCJITProbe(State* state)
 {
     Context context(state);
