@@ -1642,6 +1642,11 @@ public:
         cacheFlush(location, sizeof(uint32_t) * 2);
     }
 
+    static void relinkTailCall(void* from, void* to)
+    {
+        relinkJump(from, to);
+    }
+
     static void replaceWithVMHalt(void* where)
     {
         uint32_t* location = reinterpret_cast<uint32_t*>(where);
@@ -1668,6 +1673,12 @@ public:
         cacheFlush(from, sizeof(uint32_t) * 2);
     }
 
+    static void replaceWithNops(void* from, size_t memoryToFillWithNopsInBytes)
+    {
+        fillNops<memcpy>(from, memoryToFillWithNopsInBytes);
+        cacheFlush(from, memoryToFillWithNopsInBytes);
+    }
+
     static void revertJumpReplacementToPatch(void* from, void* valuePtr)
     {
         uint32_t* location = reinterpret_cast<uint32_t*>(from);
