@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,12 +30,14 @@
 #include "DFGFinalizer.h"
 #include "DFGJITCode.h"
 #include "LinkBuffer.h"
+#include <wtf/TZoneMalloc.h>
 
 namespace JSC { namespace DFG {
 
 class JITFinalizer final : public Finalizer {
+    WTF_MAKE_TZONE_ALLOCATED(JITFinalizer);
 public:
-    JITFinalizer(Plan&, Ref<JITCode>&&, std::unique_ptr<LinkBuffer>, CodePtr<JSEntryPtrTag> withArityCheck = CodePtr<JSEntryPtrTag>(CodePtr<JSEntryPtrTag>::EmptyValue));
+    JITFinalizer(Plan&, Ref<DFG::JITCode>&&, std::unique_ptr<LinkBuffer>, CodePtr<JSEntryPtrTag> withArityCheck = CodePtr<JSEntryPtrTag>(CodePtr<JSEntryPtrTag>::EmptyValue));
     ~JITFinalizer() final;
     
     size_t codeSize() final;
@@ -44,7 +46,7 @@ public:
 
 private:
     
-    Ref<JITCode> m_jitCode;
+    Ref<DFG::JITCode> m_jitCode;
     std::unique_ptr<LinkBuffer> m_linkBuffer;
     CodePtr<JSEntryPtrTag> m_withArityCheck;
 };
