@@ -59,8 +59,8 @@ void BlockDirectory::setSubspace(Subspace* subspace)
 
 void BlockDirectory::updatePercentageOfPagedOutPages(SimpleStats& stats)
 {
-    // FIXME: We should figure out a solution for Windows.
-#if OS(UNIX)
+    // FIXME: We should figure out a solution for Windows and PlayStation.
+#if OS(UNIX) && !PLATFORM(PLAYSTATION)
     size_t pageSize = WTF::pageSize();
     ASSERT(!(MarkedBlock::blockSize % pageSize));
     auto numberOfPagesInMarkedBlock = MarkedBlock::blockSize / pageSize;
@@ -83,6 +83,8 @@ void BlockDirectory::updatePercentageOfPagedOutPages(SimpleStats& stats)
         for (unsigned i = 0; i < numberOfPagesInMarkedBlock; ++i)
             stats.add(!(pagedBits[i] & pageIsResidentAndNotCompressed));
     }
+#else
+    UNUSED_PARAM(stats);
 #endif
 }
 
@@ -108,7 +110,7 @@ MarkedBlock::Handle* BlockDirectory::findBlockForAllocation(LocalAllocator& allo
     }
 }
 
-MarkedBlock::Handle* BlockDirectory::tryAllocateBlock(Heap& heap)
+MarkedBlock::Handle* BlockDirectory::tryAllocateBlock(JSC::Heap& heap)
 {
     SuperSamplerScope superSamplerScope(false);
     
