@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,6 +31,7 @@
 #include "AbstractSlotVisitor.h"
 #include "CodeBlock.h"
 #include "HeapInlines.h"
+#include "JSCellInlines.h"
 #include "VMInlines.h"
 #include <wtf/CompilationThread.h>
 
@@ -77,6 +78,7 @@ auto JITPlan::tier() const -> Tier
     case JITCompilationMode::Baseline:
         return Tier::Baseline;
     case JITCompilationMode::DFG:
+    case JITCompilationMode::UnlinkedDFG:
         return Tier::DFG;
     case JITCompilationMode::FTL:
     case JITCompilationMode::FTLForOSREntry:
@@ -87,7 +89,12 @@ auto JITPlan::tier() const -> Tier
 
 JITCompilationKey JITPlan::key()
 {
-    return JITCompilationKey(m_codeBlock->baselineAlternative(), m_mode);
+    JSCell* codeBlock;
+    if (m_mode == JITCompilationMode::Baseline)
+        codeBlock = m_codeBlock->unlinkedCodeBlock();
+    else
+        codeBlock = m_codeBlock->baselineAlternative();
+    return JITCompilationKey(codeBlock, m_mode);
 }
 
 bool JITPlan::isKnownToBeLiveAfterGC()
@@ -140,7 +147,7 @@ bool JITPlan::reportCompileTimes() const
 {
     return Options::reportCompileTimes()
         || (Options::reportBaselineCompileTimes() && m_mode == JITCompilationMode::Baseline)
-        || (Options::reportDFGCompileTimes() && m_mode == JITCompilationMode::DFG)
+        || (Options::reportDFGCompileTimes() && isDFG())
         || (Options::reportFTLCompileTimes() && isFTL());
 }
 
@@ -150,15 +157,18 @@ void JITPlan::compileInThread(JITWorklistThread* thread)
 
     MonotonicTime before;
     CString codeBlockName;
-    if (UNLIKELY(computeCompileTimes()))
+
+    bool computeCompileTimes = this->computeCompileTimes();
+    if (UNLIKELY(computeCompileTimes)) {
         before = MonotonicTime::now();
-    if (UNLIKELY(reportCompileTimes()))
-        codeBlockName = toCString(*m_codeBlock);
+        if (reportCompileTimes())
+            codeBlockName = toCString(*m_codeBlock);
+    }
 
     CompilationScope compilationScope;
 
 #if ENABLE(DFG_JIT)
-    if (DFG::logCompilationChanges(m_mode) || Options::logPhaseTimes())
+    if (UNLIKELY(DFG::logCompilationChanges(m_mode) || Options::logPhaseTimes()))
         dataLog("DFG(Plan) compiling ", *m_codeBlock, " with ", m_mode, ", instructions size = ", m_codeBlock->instructionsSize(), "\n");
 #endif // ENABLE(DFG_JIT)
 
@@ -166,21 +176,22 @@ void JITPlan::compileInThread(JITWorklistThread* thread)
 
     RELEASE_ASSERT((path == CancelPath) == (m_stage == JITPlanStage::Canceled));
 
-    MonotonicTime after;
-    if (UNLIKELY(computeCompileTimes())) {
-        after = MonotonicTime::now();
+    if (LIKELY(!computeCompileTimes))
+        return;
 
-        if (Options::reportTotalCompileTimes()) {
-            if (isFTL()) {
-                totalFTLCompileTime += after - before;
-                totalFTLDFGCompileTime += m_timeBeforeFTL - before;
-                totalFTLB3CompileTime += after - m_timeBeforeFTL;
-            } else if (mode() == JITCompilationMode::Baseline)
-                totalBaselineCompileTime += after - before;
-            else
-                totalDFGCompileTime += after - before;
-        }
+    MonotonicTime after = MonotonicTime::now();
+
+    if (Options::reportTotalCompileTimes()) {
+        if (isFTL()) {
+            totalFTLCompileTime += after - before;
+            totalFTLDFGCompileTime += m_timeBeforeFTL - before;
+            totalFTLB3CompileTime += after - m_timeBeforeFTL;
+        } else if (mode() == JITCompilationMode::Baseline)
+            totalBaselineCompileTime += after - before;
+        else
+            totalDFGCompileTime += after - before;
     }
+
     const char* pathName = nullptr;
     switch (path) {
     case FailPath:
@@ -203,10 +214,23 @@ void JITPlan::compileInThread(JITWorklistThread* thread)
         break;
     }
     if (m_codeBlock) { // m_codeBlock will be null if the compilation was cancelled.
-        if (path == FTLPath)
+        switch (path) {
+        case FTLPath:
             CODEBLOCK_LOG_EVENT(m_codeBlock, "ftlCompile", ("took ", (after - before).milliseconds(), " ms (DFG: ", (m_timeBeforeFTL - before).milliseconds(), ", B3: ", (after - m_timeBeforeFTL).milliseconds(), ") with ", pathName));
-        else
+            break;
+        case DFGPath:
             CODEBLOCK_LOG_EVENT(m_codeBlock, "dfgCompile", ("took ", (after - before).milliseconds(), " ms with ", pathName));
+            break;
+        case BaselinePath:
+            CODEBLOCK_LOG_EVENT(m_codeBlock, "baselineCompile", ("took ", (after - before).milliseconds(), " ms with ", pathName));
+            break;
+        case FailPath:
+            CODEBLOCK_LOG_EVENT(m_codeBlock, "failed compilation", ("took ", (after - before).milliseconds(), " ms with ", pathName));
+            break;
+        case CancelPath:
+            CODEBLOCK_LOG_EVENT(m_codeBlock, "cancelled compilation", ("took ", (after - before).milliseconds(), " ms with ", pathName));
+            break;
+        }
     }
     if (UNLIKELY(reportCompileTimes())) {
         dataLog("Optimized ", codeBlockName, " using ", m_mode, " with ", pathName, " into ", codeSize(), " bytes in ", (after - before).milliseconds(), " ms");
