@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2020 Apple Inc. All rights reserved.
+# Copyright (C) 2011-2022 Apple Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions
@@ -59,6 +59,7 @@ MACRO_INSTRUCTIONS =
      "rrotateq",
      "subi",
      "xori",
+     "load2ia",
      "loadi",
      "loadis",
      "loadb",
@@ -67,14 +68,17 @@ MACRO_INSTRUCTIONS =
      "loadh",
      "loadhsi",
      "loadhsq",
+     "store2ia",
      "storei",
      "storeh",
      "storeb",
      "loadf",
      "loadd",
+     "loadv",
      "moved",
      "storef",
      "stored",
+     "storev",
      "addf",
      "addd",
      "divf",
@@ -137,7 +141,9 @@ MACRO_INSTRUCTIONS =
      "bcd2i",
      "movdz",
      "pop",
+     "popv",
      "push",
+     "pushv",
      "move",
      "sxi2q",
      "zxi2q",
@@ -375,11 +381,27 @@ X86_INSTRUCTIONS =
      "fence",
     ]
 
+X86_SIMD_INSTRUCTIONS =
+    [
+    ]
+
 ARM_INSTRUCTIONS =
     [
+     "adci",
+     "bcs",
      "clrbp",
      "mvlbl",
-     "globaladdr"
+     "globaladdr",
+     "sbci",
+     "moveii",
+     "loadlinkb",
+     "loadlinkh",
+     "loadlinki",
+     "loadlink2i",
+     "storecondb",
+     "storecondh",
+     "storecondi",
+     "storecond2i",
     ]
 
 ARM64_INSTRUCTIONS =
@@ -387,10 +409,6 @@ ARM64_INSTRUCTIONS =
      "bfiq", # Bit field insert <source reg> <last bit written> <width immediate> <dest reg>
      "pcrtoaddr",   # Address from PC relative offset - adr instruction
      "globaladdr",
-     "divi",
-     "divis",
-     "divq",
-     "divqs",
      "notq",
      "loadlinkacqb",
      "loadlinkacqh",
@@ -430,15 +448,38 @@ ARM64_INSTRUCTIONS =
      "atomicloadh",
      "atomicloadi",
      "atomicloadq",
+     "loadpairq",
+     "loadpairi",
+     "storepairq",
+     "storepairi",
+     "loadpaird",
+     "storepaird",
+    ]
+
+ARM64_SIMD_INSTRUCTIONS =
+    [
+     "umovb",
+     "umovh",
+     "umovi",
+     "umovq",
     ]
 
 RISC_INSTRUCTIONS =
     [
      "smulli",  # Multiply two 32-bit words and produce a 64-bit word
+     "umulli",  # Multiply two 32-bit words and produce a 64-bit word
      "addis",   # Add integers and set a flag.
      "subis",   # Same, but for subtraction.
      "oris",    # Same, but for bitwise or.
-     "addps"    # addis but for pointers.
+     "addps",   # addis but for pointers.
+     "divi",
+     "divis",
+     "divq",
+     "divqs",
+     "remi",
+     "remis",
+     "remq",
+     "remqs"
     ]
 
 MIPS_INSTRUCTIONS =
@@ -459,6 +500,8 @@ CXX_INSTRUCTIONS =
      "cloopCallNative",         # operands: callee
      "cloopCallSlowPath",       # operands: callTarget, currentFrame, currentPC
      "cloopCallSlowPathVoid",   # operands: callTarget, currentFrame, currentPC
+     "cloopCallSlowPath3",      # operands: callTarget, a0, a1, a2
+     "cloopCallSlowPath4",      # operands: callTarget, a0, a1, a2, a3
 
      # For debugging only:
      # Takes no operands but simply emits whatever follows in // comments as
@@ -469,7 +512,7 @@ CXX_INSTRUCTIONS =
      "cloopDo",              # no operands
     ]
 
-INSTRUCTIONS = MACRO_INSTRUCTIONS + X86_INSTRUCTIONS + ARM_INSTRUCTIONS + ARM64_INSTRUCTIONS + RISC_INSTRUCTIONS + MIPS_INSTRUCTIONS + CXX_INSTRUCTIONS
+INSTRUCTIONS = MACRO_INSTRUCTIONS + X86_INSTRUCTIONS + X86_SIMD_INSTRUCTIONS + ARM_INSTRUCTIONS + ARM64_INSTRUCTIONS + ARM64_SIMD_INSTRUCTIONS + RISC_INSTRUCTIONS + MIPS_INSTRUCTIONS + CXX_INSTRUCTIONS
 
 INSTRUCTION_SET = INSTRUCTIONS.to_set
 
