@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2009-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -208,6 +208,14 @@ Ref<ArrayBuffer> ArrayBuffer::create(const void* source, size_t byteLength)
     return buffer.releaseNonNull();
 }
 
+Ref<ArrayBuffer> ArrayBuffer::create(std::span<uint8_t> span)
+{
+    auto buffer = tryCreate(span);
+    if (!buffer)
+        CRASH();
+    return buffer.releaseNonNull();
+}
+
 Ref<ArrayBuffer> ArrayBuffer::create(ArrayBufferContents&& contents)
 {
     return adoptRef(*new ArrayBuffer(WTFMove(contents)));
@@ -268,6 +276,11 @@ RefPtr<ArrayBuffer> ArrayBuffer::tryCreate(const void* source, size_t byteLength
     return createInternal(WTFMove(contents), source, byteLength);
 }
 
+RefPtr<ArrayBuffer> ArrayBuffer::tryCreate(std::span<uint8_t> span)
+{
+    return tryCreate(span.data(), span.size_bytes());
+}
+
 Ref<ArrayBuffer> ArrayBuffer::createUninitialized(size_t numElements, unsigned elementByteSize)
 {
     return create(numElements, elementByteSize, ArrayBufferContents::InitializationPolicy::DontInitialize);
@@ -412,9 +425,7 @@ bool ArrayBuffer::transferTo(VM& vm, ArrayBufferContents& result)
         return true;
     }
 
-    bool isDetachable = !m_pinCount && !m_locked;
-
-    if (!isDetachable) {
+    if (!isDetachable()) {
         m_contents.copyTo(result);
         if (!result.m_data)
             return false;
@@ -451,7 +462,7 @@ Expected<int64_t, GrowFailReason> ArrayBuffer::grow(VM& vm, size_t newByteLength
         return makeUnexpected(GrowFailReason::GrowSharedUnavailable);
     auto result = shared->grow(vm, newByteLength);
     if (result && result.value() > 0)
-        vm.heap.reportExtraMemoryAllocated(result.value());
+        vm.heap.reportExtraMemoryAllocated(static_cast<JSCell*>(nullptr), result.value());
     return result;
 }
 
@@ -545,7 +556,7 @@ Expected<int64_t, GrowFailReason> ArrayBuffer::resize(VM& vm, size_t newByteLeng
     }
 
     if (deltaByteLength > 0)
-        vm.heap.reportExtraMemoryAllocated(deltaByteLength);
+        vm.heap.reportExtraMemoryAllocated(static_cast<JSCell*>(nullptr), deltaByteLength);
 
     return deltaByteLength;
 }
@@ -641,5 +652,16 @@ ASCIILiteral errorMessageForTransfer(ArrayBuffer* buffer)
     return "Cannot transfer an ArrayBuffer whose backing store has been accessed by the JavaScriptCore C API"_s;
 }
 
+std::optional<ArrayBufferContents> ArrayBufferContents::fromDataSpan(std::span<const uint8_t> data)
+{
+    void* buffer = Gigacage::tryMalloc(Gigacage::Primitive, data.size_bytes());
+    if (!buffer)
+        return std::nullopt;
+
+    memcpy(buffer, data.data(), data.size_bytes());
+
+    return ArrayBufferContents { buffer, data.size_bytes(), std::nullopt, ArrayBuffer::primitiveGigacageDestructor() };
+}
+
 } // namespace JSC
 
