@@ -339,7 +339,7 @@ JSC_DEFINE_HOST_FUNCTION(objectConstructorAssign, (JSGlobalObject* globalObject,
                 break;
             }
             JSObject* source = asObject(sourceValue);
-            if (!source->staticPropertiesReified() || !source->structure()->canPerformFastPropertyEnumeration()) {
+            if (!source->staticPropertiesReified() || !source->structure()->canPerformFastPropertyEnumerationCommon() || source->canHaveExistingOwnIndexedProperties()) {
                 willBatch = false;
                 break;
             }
@@ -387,7 +387,9 @@ JSC_DEFINE_HOST_FUNCTION(objectConstructorAssign, (JSGlobalObject* globalObject,
                 RETURN_IF_EXCEPTION(scope, { });
             }
 
-            if (objectAssignFast(vm, target, source, properties, values))
+            bool objectAssignFastSucceeded = objectAssignFast(globalObject, target, source, properties, values);
+            RETURN_IF_EXCEPTION(scope, { });
+            if (objectAssignFastSucceeded)
                 continue;
         }
 
@@ -417,18 +419,21 @@ JSC_DEFINE_HOST_FUNCTION(objectConstructorEntries, (JSGlobalObject* globalObject
     {
         Vector<RefPtr<UniquedStringImpl>, 8> properties;
         MarkedArgumentBuffer values;
-        bool canUseFastPath = target->fastForEachPropertyWithSideEffectFreeFunctor(vm, [&](const PropertyTableEntry& entry) -> bool {
-            if (entry.attributes() & PropertyAttribute::DontEnum)
+        bool canUseFastPath = false;
+        if (!target->canHaveExistingOwnIndexedProperties()) {
+            canUseFastPath = target->fastForEachPropertyWithSideEffectFreeFunctor(vm, [&](const PropertyTableEntry& entry) -> bool {
+                if (entry.attributes() & PropertyAttribute::DontEnum)
+                    return true;
+
+                if (entry.key()->isSymbol())
+                    return true;
+
+                properties.append(entry.key());
+                values.appendWithCrashOnOverflow(target->getDirect(entry.offset()));
+
                 return true;
-
-            if (entry.key()->isSymbol())
-                return true;
-
-            properties.append(entry.key());
-            values.appendWithCrashOnOverflow(target->getDirect(entry.offset()));
-
-            return true;
-        });
+            });
+        }
 
         if (canUseFastPath) {
             Structure* arrayStructure = globalObject->arrayStructureForIndexingTypeDuringAllocation(ArrayWithContiguous);
@@ -561,26 +566,40 @@ JSC_DEFINE_HOST_FUNCTION(objectConstructorValues, (JSGlobalObject* globalObject,
     }
 
     {
-        MarkedArgumentBuffer values;
-        bool canUseFastPath = target->fastForEachPropertyWithSideEffectFreeFunctor(vm, [&](const PropertyTableEntry& entry) -> bool {
-            if (entry.attributes() & PropertyAttribute::DontEnum)
-                return true;
+        MarkedArgumentBuffer namedPropertyValues;
+        bool canUseFastPath = false;
+        if (!target->canHaveExistingOwnIndexedGetterSetterProperties()) {
+            canUseFastPath = target->fastForEachPropertyWithSideEffectFreeFunctor(vm, [&](const PropertyTableEntry& entry) -> bool {
+                if (entry.attributes() & PropertyAttribute::DontEnum)
+                    return true;
 
-            if (entry.key()->isSymbol())
-                return true;
+                if (entry.key()->isSymbol())
+                    return true;
 
-            values.appendWithCrashOnOverflow(target->getDirect(entry.offset()));
-            return true;
-        });
+                namedPropertyValues.appendWithCrashOnOverflow(target->getDirect(entry.offset()));
+                return true;
+            });
+        }
 
         if (canUseFastPath) {
             Structure* arrayStructure = globalObject->arrayStructureForIndexingTypeDuringAllocation(ArrayWithContiguous);
+            MarkedArgumentBuffer indexedPropertyValues;
+            if (target->canHaveExistingOwnIndexedProperties()) {
+                target->forEachOwnIndexedProperty<JSObject::SortMode::Ascending>(globalObject, [&](unsigned, JSValue value) {
+                    indexedPropertyValues.appendWithCrashOnOverflow(value);
+                    return IterationStatus::Continue;
+                });
+            }
+            RETURN_IF_EXCEPTION(scope, { });
+
             {
                 ObjectInitializationScope initializationScope(vm);
                 JSArray* result = nullptr;
-                if (LIKELY(result = JSArray::tryCreateUninitializedRestricted(initializationScope, nullptr, arrayStructure, values.size()))) {
-                    for (unsigned i = 0; i < values.size(); ++i)
-                        result->initializeIndex(initializationScope, i, values.at(i));
+                if (LIKELY(result = JSArray::tryCreateUninitializedRestricted(initializationScope, nullptr, arrayStructure, indexedPropertyValues.size() + namedPropertyValues.size()))) {
+                    for (unsigned i = 0; i < indexedPropertyValues.size(); ++i)
+                        result->initializeIndex(initializationScope, i, indexedPropertyValues.at(i));
+                    for (unsigned i = 0; i < namedPropertyValues.size(); ++i)
+                        result->initializeIndex(initializationScope, indexedPropertyValues.size() + i, namedPropertyValues.at(i));
                     return JSValue::encode(result);
                 }
             }
@@ -625,7 +644,7 @@ JSC_DEFINE_HOST_FUNCTION(objectConstructorValues, (JSGlobalObject* globalObject,
 }
 
 // https://tc39.github.io/ecma262/#sec-topropertydescriptor
-bool toPropertyDescriptor(JSGlobalObject* globalObject, JSValue in, PropertyDescriptor& desc)
+inline bool toPropertyDescriptor(JSGlobalObject* globalObject, JSValue in, PropertyDescriptor& desc, bool& withoutSideEffect)
 {
     ASSERT(desc.isEmpty());
     VM& vm = globalObject->vm();
@@ -637,44 +656,102 @@ bool toPropertyDescriptor(JSGlobalObject* globalObject, JSValue in, PropertyDesc
     }
     JSObject* description = asObject(in);
 
-    JSValue enumerable = description->getIfPropertyExists(globalObject, vm.propertyNames->enumerable);
-    RETURN_IF_EXCEPTION(scope, false);
-    if (enumerable)
-        desc.setEnumerable(enumerable.toBoolean(globalObject));
+    JSValue enumerable;
+    JSValue configurable;
+    JSValue value;
+    JSValue writable;
+    JSValue get;
+    JSValue set;
 
-    JSValue configurable = description->getIfPropertyExists(globalObject, vm.propertyNames->configurable);
-    RETURN_IF_EXCEPTION(scope, false);
-    if (configurable)
-        desc.setConfigurable(configurable.toBoolean(globalObject));
+    if (globalObject->propertyDescriptorFastPathWatchpointSet().state() == ClearWatchpoint)
+        globalObject->tryInstallPropertyDescriptorFastPathWatchpoint();
 
-    JSValue value = description->getIfPropertyExists(globalObject, vm.propertyNames->value);
-    RETURN_IF_EXCEPTION(scope, false);
-    if (value)
-        desc.setValue(value);
 
-    JSValue writable = description->getIfPropertyExists(globalObject, vm.propertyNames->writable);
-    RETURN_IF_EXCEPTION(scope, false);
-    if (writable)
-        desc.setWritable(writable.toBoolean(globalObject));
-
-    JSValue get = description->getIfPropertyExists(globalObject, vm.propertyNames->get);
-    RETURN_IF_EXCEPTION(scope, false);
-    if (get) {
-        if (!get.isUndefined() && !get.isCallable()) {
-            throwTypeError(globalObject, scope, "Getter must be a function."_s);
-            return false;
+    bool canUseFastPath = false;
+    if (globalObject->propertyDescriptorFastPathWatchpointSet().isStillValid() && globalObject->objectPrototypeChainIsSane() && description->inherits<JSFinalObject>() && description->getPrototypeDirect() == globalObject->objectPrototype() && description->structure()->canPerformFastPropertyEnumeration()) {
+        canUseFastPath = description->fastForEachPropertyWithSideEffectFreeFunctor(vm, [&](const PropertyTableEntry& entry) -> bool {
+            PropertyName propertyName(entry.key());
+            if (propertyName == vm.propertyNames->enumerable)
+                enumerable = description->getDirect(entry.offset());
+            else if (propertyName == vm.propertyNames->configurable)
+                configurable = description->getDirect(entry.offset());
+            else if (propertyName == vm.propertyNames->value)
+                value = description->getDirect(entry.offset());
+            else if (propertyName == vm.propertyNames->writable)
+                writable = description->getDirect(entry.offset());
+            else if (propertyName == vm.propertyNames->get)
+                get = description->getDirect(entry.offset());
+            else if (propertyName == vm.propertyNames->set)
+                set = description->getDirect(entry.offset());
+            return true;
+        });
+        if (canUseFastPath) {
+            if (enumerable)
+                desc.setEnumerable(enumerable.toBoolean(globalObject));
+            if (configurable)
+                desc.setConfigurable(configurable.toBoolean(globalObject));
+            if (value)
+                desc.setValue(value);
+            if (writable)
+                desc.setWritable(writable.toBoolean(globalObject));
+            if (get) {
+                if (!get.isUndefined() && !get.isCallable()) {
+                    throwTypeError(globalObject, scope, "Getter must be a function."_s);
+                    return false;
+                }
+                desc.setGetter(get);
+            }
+            if (set) {
+                if (!set.isUndefined() && !set.isCallable()) {
+                    throwTypeError(globalObject, scope, "Setter must be a function."_s);
+                    return false;
+                }
+                desc.setSetter(set);
+            }
+            withoutSideEffect = true;
         }
-        desc.setGetter(get);
     }
 
-    JSValue set = description->getIfPropertyExists(globalObject, vm.propertyNames->set);
-    RETURN_IF_EXCEPTION(scope, false);
-    if (set) {
-        if (!set.isUndefined() && !set.isCallable()) {
-            throwTypeError(globalObject, scope, "Setter must be a function."_s);
-            return false;
+    if (!canUseFastPath) {
+        enumerable = description->getIfPropertyExists(globalObject, vm.propertyNames->enumerable);
+        RETURN_IF_EXCEPTION(scope, false);
+        if (enumerable)
+            desc.setEnumerable(enumerable.toBoolean(globalObject));
+
+        configurable = description->getIfPropertyExists(globalObject, vm.propertyNames->configurable);
+        RETURN_IF_EXCEPTION(scope, false);
+        if (configurable)
+            desc.setConfigurable(configurable.toBoolean(globalObject));
+
+        value = description->getIfPropertyExists(globalObject, vm.propertyNames->value);
+        RETURN_IF_EXCEPTION(scope, false);
+        if (value)
+            desc.setValue(value);
+
+        writable = description->getIfPropertyExists(globalObject, vm.propertyNames->writable);
+        RETURN_IF_EXCEPTION(scope, false);
+        if (writable)
+            desc.setWritable(writable.toBoolean(globalObject));
+
+        get = description->getIfPropertyExists(globalObject, vm.propertyNames->get);
+        RETURN_IF_EXCEPTION(scope, false);
+        if (get) {
+            if (!get.isUndefined() && !get.isCallable()) {
+                throwTypeError(globalObject, scope, "Getter must be a function."_s);
+                return false;
+            }
+            desc.setGetter(get);
+        }
+
+        set = description->getIfPropertyExists(globalObject, vm.propertyNames->set);
+        RETURN_IF_EXCEPTION(scope, false);
+        if (set) {
+            if (!set.isUndefined() && !set.isCallable()) {
+                throwTypeError(globalObject, scope, "Setter must be a function."_s);
+                return false;
+            }
+            desc.setSetter(set);
         }
-        desc.setSetter(set);
     }
 
     if (!desc.isAccessorDescriptor())
@@ -693,6 +770,12 @@ bool toPropertyDescriptor(JSGlobalObject* globalObject, JSValue in, PropertyDesc
     return true;
 }
 
+bool toPropertyDescriptor(JSGlobalObject* globalObject, JSValue value, PropertyDescriptor& desc)
+{
+    bool withoutSideEffect = false;
+    return toPropertyDescriptor(globalObject, value, desc, withoutSideEffect);
+}
+
 JSC_DEFINE_HOST_FUNCTION(objectConstructorDefineProperty, (JSGlobalObject* globalObject, CallFrame* callFrame))
 {
     VM& vm = globalObject->vm();
@@ -714,7 +797,7 @@ JSC_DEFINE_HOST_FUNCTION(objectConstructorDefineProperty, (JSGlobalObject* globa
     RELEASE_AND_RETURN(scope, JSValue::encode(obj));
 }
 
-static JSValue defineProperties(JSGlobalObject* globalObject, JSObject* object, JSObject* properties)
+static JSValue definePropertiesSlow(JSGlobalObject* globalObject, JSObject* object, JSObject* properties)
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
@@ -760,6 +843,92 @@ static JSValue defineProperties(JSGlobalObject* globalObject, JSObject* object,
     return object;
 }
 
+static JSValue defineProperties(JSGlobalObject* globalObject, JSObject* object, JSObject* properties)
+{
+    VM& vm = globalObject->vm();
+    auto scope = DECLARE_THROW_SCOPE(vm);
+
+    Vector<RefPtr<UniquedStringImpl>, 8> propertyNames;
+    MarkedArgumentBuffer values;
+    bool canUseFastPath = !hasIndexedProperties(properties->indexingType()) && properties->fastForEachPropertyWithSideEffectFreeFunctor(vm, [&](const PropertyTableEntry& entry) -> bool {
+        if (entry.attributes() & PropertyAttribute::DontEnum)
+            return true;
+
+        PropertyName propertyName(entry.key());
+        if (propertyName.isPrivateName())
+            return true;
+
+        propertyNames.append(entry.key());
+        values.appendWithCrashOnOverflow(properties->getDirect(entry.offset()));
+
+        return true;
+    });
+    if (UNLIKELY(!canUseFastPath))
+        RELEASE_AND_RETURN(scope, definePropertiesSlow(globalObject, object, properties));
+
+    unsigned index = 0;
+    unsigned numProperties = propertyNames.size();
+    Vector<PropertyDescriptor> descriptors;
+    MarkedArgumentBuffer markBuffer;
+#define RETURN_IF_EXCEPTION_CLEARING_OVERFLOW(value) do { \
+    if (scope.exception()) { \
+        markBuffer.overflowCheckNotNeeded(); \
+        return value; \
+    } \
+} while (false)
+
+    for (; index < numProperties; ++index) {
+        JSValue prop = values.at(index);
+        bool withoutSideEffect = false;
+        PropertyDescriptor descriptor;
+        toPropertyDescriptor(globalObject, prop, descriptor, withoutSideEffect);
+        RETURN_IF_EXCEPTION_CLEARING_OVERFLOW({ });
+        descriptors.append(descriptor);
+        // Ensure we mark all the values that we're accumulating
+        if (descriptor.isDataDescriptor() && descriptor.value())
+            markBuffer.append(descriptor.value());
+        if (descriptor.isAccessorDescriptor()) {
+            if (descriptor.getter())
+                markBuffer.append(descriptor.getter());
+            if (descriptor.setter())
+                markBuffer.append(descriptor.setter());
+        }
+        if (UNLIKELY(!withoutSideEffect)) // Bail out to the slow code.
+            break;
+    }
+
+    if (UNLIKELY(index < numProperties)) {
+        for (; index < numProperties; ++index) {
+            JSValue prop = properties->get(globalObject, propertyNames[index].get());
+            RETURN_IF_EXCEPTION_CLEARING_OVERFLOW({ });
+            PropertyDescriptor descriptor;
+            toPropertyDescriptor(globalObject, prop, descriptor);
+            RETURN_IF_EXCEPTION_CLEARING_OVERFLOW({ });
+            descriptors.append(descriptor);
+            // Ensure we mark all the values that we're accumulating
+            if (descriptor.isDataDescriptor() && descriptor.value())
+                markBuffer.append(descriptor.value());
+            if (descriptor.isAccessorDescriptor()) {
+                if (descriptor.getter())
+                    markBuffer.append(descriptor.getter());
+                if (descriptor.setter())
+                    markBuffer.append(descriptor.setter());
+            }
+        }
+    }
+
+    RELEASE_ASSERT(!markBuffer.hasOverflowed());
+#undef RETURN_IF_EXCEPTION_CLEARING_OVERFLOW
+
+    for (unsigned index = 0; index < numProperties; index++) {
+        PropertyName propertyName(propertyNames[index].get());
+        ASSERT(!propertyName.isPrivateName());
+        object->methodTable()->defineOwnProperty(object, globalObject, propertyName, descriptors[index], true);
+        RETURN_IF_EXCEPTION(scope, { });
+    }
+    return object;
+}
+
 JSC_DEFINE_HOST_FUNCTION(objectConstructorDefineProperties, (JSGlobalObject* globalObject, CallFrame* callFrame))
 {
     VM& vm = globalObject->vm();
