@@ -29,36 +29,41 @@
 
 namespace JSC {
 
-ALWAYS_INLINE bool canPerformFastPropertyEnumerationForObjectAssign(Structure* structure)
+inline Structure* ObjectConstructor::createStructure(VM& vm, JSGlobalObject* globalObject, JSValue prototype)
 {
+    return Structure::create(vm, globalObject, prototype, TypeInfo(InternalFunctionType, StructureFlags), info());
+}
+
+ALWAYS_INLINE bool canPerformFastPropertyNameEnumerationForJSONStringifyWithSideEffect(Structure* structure)
+{
+    // We do not check GetterSetter, CustomGetterSetter. GetterSetter and CustomGetterSetter will be invoked, and we fall back to the a bit more generic mode when we detect structure transition.
     if (structure->typeInfo().overridesGetOwnPropertySlot())
         return false;
     if (structure->typeInfo().overridesAnyFormOfGetOwnPropertyNames())
         return false;
-    // FIXME: Indexed properties can be handled.
-    // https://bugs.webkit.org/show_bug.cgi?id=185358
     if (hasIndexedProperties(structure->indexingType()))
         return false;
-    if (structure->hasGetterSetterProperties())
-        return false;
-    if (structure->hasReadOnlyOrGetterSetterPropertiesExcludingProto())
-        return false;
-    if (structure->hasCustomGetterSetterProperties())
-        return false;
     if (structure->isUncacheableDictionary())
         return false;
-    // Cannot perform fast [[Put]] to |target| if the property names of the |source| contain "__proto__".
-    if (structure->hasUnderscoreProtoPropertyExcludingOriginalProto())
+    if (structure->hasNonReifiedStaticProperties())
         return false;
     return true;
 }
 
-ALWAYS_INLINE bool canPerformFastPropertyEnumerationForJSONStringify(Structure* structure)
+ALWAYS_INLINE void objectAssignIndexedPropertiesFast(JSGlobalObject* globalObject, JSObject* target, JSObject* source)
 {
-    return canPerformFastPropertyEnumerationForObjectAssign(structure);
+    VM& vm = globalObject->vm();
+    auto scope = DECLARE_THROW_SCOPE(vm);
+
+    source->forEachOwnIndexedProperty<>(globalObject, [&](unsigned index, JSValue value) {
+        target->putDirectIndex(globalObject, index, value);
+        RETURN_IF_EXCEPTION(scope, IterationStatus::Done);
+        return IterationStatus::Continue;
+    });
+    RETURN_IF_EXCEPTION(scope, void());
 }
 
-ALWAYS_INLINE void objectAssignFast(VM& vm, JSObject* target, JSObject* source, Vector<RefPtr<UniquedStringImpl>, 8>& properties, MarkedArgumentBuffer& values)
+ALWAYS_INLINE bool objectAssignFast(JSGlobalObject* globalObject, JSObject* target, JSObject* source, Vector<RefPtr<UniquedStringImpl>, 8>& properties, MarkedArgumentBuffer& values)
 {
     // |source| Structure does not have any getters. And target can perform fast put.
     // So enumerating properties and putting properties are non observable.
@@ -74,28 +79,38 @@ ALWAYS_INLINE void objectAssignFast(VM& vm, JSObject* target, JSObject* source,
     // regression in object-assign-replace. Since the code is small and fast path, we keep both.
 
     // Do not clear since Vector::clear shrinks the backing store.
-    properties.resize(0);
+    VM& vm = globalObject->vm();
+    auto scope = DECLARE_THROW_SCOPE(vm);
+    properties.shrink(0);
     values.clear();
-    source->structure(vm)->forEachProperty(vm, [&] (const PropertyMapEntry& entry) -> bool {
-        if (entry.attributes & PropertyAttribute::DontEnum)
+
+    if (source->canHaveExistingOwnIndexedGetterSetterProperties())
+        return false;
+    bool canUseFastPath = source->fastForEachPropertyWithSideEffectFreeFunctor(vm, [&](const PropertyTableEntry& entry) -> bool {
+        if (entry.attributes() & PropertyAttribute::DontEnum)
             return true;
 
-        PropertyName propertyName(entry.key);
+        PropertyName propertyName(entry.key());
         if (propertyName.isPrivateName())
             return true;
 
-        properties.append(entry.key);
-        values.appendWithCrashOnOverflow(source->getDirect(entry.offset));
+        properties.append(entry.key());
+        values.appendWithCrashOnOverflow(source->getDirect(entry.offset()));
 
         return true;
     });
+    if (!canUseFastPath)
+        return false;
 
-    for (size_t i = 0; i < properties.size(); ++i) {
-        // FIXME: We could put properties in a batching manner to accelerate Object.assign more.
-        // https://bugs.webkit.org/show_bug.cgi?id=185358
-        PutPropertySlot putPropertySlot(target, true);
-        target->putOwnDataProperty(vm, properties[i].get(), values.at(i), putPropertySlot);
+    if (source->canHaveExistingOwnIndexedProperties()) {
+        objectAssignIndexedPropertiesFast(globalObject, target, source);
+        RETURN_IF_EXCEPTION(scope, { });
     }
+
+    // Actually, assigning with empty object (option for example) is common. (`Object.assign(defaultOptions, passedOptions)` where `passedOptions` is empty object.)
+    if (properties.size())
+        target->putOwnDataPropertyBatching(vm, properties.data(), values.data(), properties.size());
+    return true;
 }
 
 
