@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004, 2008, 2009 Apple Inc. All rights reserved.
+ * Copyright (C) 2004-2023 Apple Inc. All rights reserved.
  * Copyright (C) 2008 Collabora Ltd.
  * Copyright (C) 2011 Peter Varga (pvarga@webkit.org), University of Szeged
  *
@@ -32,9 +32,12 @@
 #include "YarrInterpreter.h"
 #include <wtf/Assertions.h>
 #include <wtf/BumpPointerAllocator.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace JSC { namespace Yarr {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(RegularExpression);
+
 class RegularExpression::Private : public RefCounted<RegularExpression::Private> {
 public:
     static Ref<Private> create(StringView pattern, OptionSet<Flags> flags)
