@@ -36,12 +36,14 @@
 #include "compiler/translator/tree_ops/spirv/EmulateFramebufferFetch.h"
 #include "compiler/translator/tree_ops/spirv/EmulateYUVBuiltIns.h"
 #include "compiler/translator/tree_ops/spirv/FlagSamplersWithTexelFetch.h"
+#include "compiler/translator/tree_ops/spirv/ReswizzleYUVOps.h"
 #include "compiler/translator/tree_ops/spirv/RewriteInterpolateAtOffset.h"
 #include "compiler/translator/tree_ops/spirv/RewriteR32fImages.h"
 #include "compiler/translator/tree_util/BuiltIn.h"
 #include "compiler/translator/tree_util/DriverUniform.h"
 #include "compiler/translator/tree_util/FindFunction.h"
 #include "compiler/translator/tree_util/FindMain.h"
+#include "compiler/translator/tree_util/FindSymbolNode.h"
 #include "compiler/translator/tree_util/IntermNode_util.h"
 #include "compiler/translator/tree_util/ReplaceClipCullDistanceVariable.h"
 #include "compiler/translator/tree_util/ReplaceVariable.h"
@@ -1026,8 +1028,9 @@ bool TranslatorSPIRV::translateImpl(TIntermBlock *root,
                 }
             }
 
-            bool hasGLSampleMask        = false;
-            bool hasGLSecondaryFragData = false;
+            bool hasGLSampleMask           = false;
+            bool hasGLSecondaryFragData    = false;
+            const TIntermSymbol *yuvOutput = nullptr;
 
             for (const ShaderVariable &outputVar : mOutputVariables)
             {
@@ -1043,6 +1046,13 @@ bool TranslatorSPIRV::translateImpl(TIntermBlock *root,
                     hasGLSecondaryFragData = true;
                     continue;
                 }
+                if (outputVar.yuv)
+                {
+                    // We can only have one yuv output
+                    ASSERT(yuvOutput == nullptr);
+                    yuvOutput = FindSymbolNode(root, ImmutableString(outputVar.name));
+                    continue;
+                }
             }
 
             if (usesPointCoord)
@@ -1164,6 +1174,11 @@ bool TranslatorSPIRV::translateImpl(TIntermBlock *root,
                 {
                     return false;
                 }
+
+                if (!ReswizzleYUVOps(this, root, &getSymbolTable(), yuvOutput))
+                {
+                    return false;
+                }
             }
 
             if (!EmulateDithering(this, compileOptions, root, &getSymbolTable(), specConst,
@@ -1378,7 +1393,8 @@ void TranslatorSPIRV::assignSpirvIds(TIntermBlock *root)
             // webgl_FragColor, webgl_FragData, webgl_SecondaryFragColor and webgl_SecondaryFragData
             // are recorded with their original names (starting with gl_)
             ImmutableString name(symbol->getName());
-            if (angle::BeginsWith(name.data(), "webgl_"))
+            if (angle::BeginsWith(name.data(), "webgl_") &&
+                symbol->variable().symbolType() == SymbolType::AngleInternal)
             {
                 name = ImmutableString(name.data() + 3, name.length() - 3);
             }
