@@ -126,9 +126,6 @@ struct Limitations
     // Unable to support different values for front and back faces for stencil refs and masks
     bool noSeparateStencilRefsAndMasks = false;
 
-    // Renderer doesn't support non-constant indexing loops in fragment shader
-    bool shadersRequireIndexedLoopValidation = false;
-
     // Renderer doesn't support Simultaneous use of GL_CONSTANT_ALPHA/GL_ONE_MINUS_CONSTANT_ALPHA
     // and GL_CONSTANT_COLOR/GL_ONE_MINUS_CONSTANT_COLOR blend functions.
     bool noSimultaneousConstantColorAndAlphaBlendFunc = false;
@@ -625,6 +622,15 @@ struct DisplayExtensions
     // EGL_ANGLE_colorspace_attribute_passthrough
     bool eglColorspaceAttributePassthroughANGLE = false;
 
+    // EGL_EXT_gl_colorspace_bt2020_linear
+    bool glColorspaceBt2020Linear = false;
+
+    // EGL_EXT_gl_colorspace_bt2020_pq
+    bool glColorspaceBt2020Pq = false;
+
+    // EGL_EXT_gl_colorspace_bt2020_hlg
+    bool glColorspaceBt2020Hlg = false;
+
     // EGL_ANDROID_framebuffer_target
     bool framebufferTargetANDROID = false;
 
@@ -732,6 +738,9 @@ struct ClientExtensions
     // EGL_EXT_platform_wayland
     bool platformWaylandEXT = false;
 
+    // EGL_MESA_platform_surfaceless
+    bool platformSurfacelessMESA = false;
+
     // EGL_ANGLE_platform_angle
     bool platformANGLE = false;
 
