@@ -15,6 +15,7 @@
 #include "common/Color.h"
 #include "common/angleutils.h"
 #include "common/bitset_utils.h"
+#include "libANGLE/ContextMutex.h"
 #include "libANGLE/Debug.h"
 #include "libANGLE/GLES1State.h"
 #include "libANGLE/Overlay.h"
@@ -33,8 +34,6 @@
 namespace egl
 {
 class ShareGroup;
-class ContextMutex;
-class SingleContextMutex;
 }  // namespace egl
 
 namespace gl
@@ -482,8 +481,6 @@ class PrivateState : angle::NonCopyable
     // Hint setters
     void setGenerateMipmapHint(GLenum hint);
     GLenum getGenerateMipmapHint() const { return mGenerateMipmapHint; }
-    void setTextureFilteringHint(GLenum hint);
-    GLenum getTextureFilteringHint() const { return mTextureFilteringHint; }
     GLenum getFragmentShaderDerivativeHint() const { return mFragmentShaderDerivativeHint; }
     void setFragmentShaderDerivativeHint(GLenum hint);
 
@@ -583,7 +580,7 @@ class PrivateState : angle::NonCopyable
     {
         mDirtyBits.set();
         mExtendedDirtyBits.set();
-        mDirtyCurrentValues.set();
+        mDirtyCurrentValues = mAllAttribsMask;
     }
 
     const state::ExtendedDirtyBits &getExtendedDirtyBits() const { return mExtendedDirtyBits; }
@@ -640,7 +637,6 @@ class PrivateState : angle::NonCopyable
     GLfloat mLineWidth;
 
     GLenum mGenerateMipmapHint;
-    GLenum mTextureFilteringHint;
     GLenum mFragmentShaderDerivativeHint;
 
     Rectangle mViewport;
@@ -657,6 +653,9 @@ class PrivateState : angle::NonCopyable
     VertexAttribVector mVertexAttribCurrentValues;  // From glVertexAttrib
     ComponentTypeMask mCurrentValuesTypeMask;
 
+    // Mask of all attributes that are available to this context: [0, maxVertexAttributes)
+    AttributesMask mAllAttribsMask;
+
     // Texture and sampler bindings
     GLint mActiveSampler;  // Active texture unit selector - GL_TEXTURE0
 
@@ -740,8 +739,7 @@ class State : angle::NonCopyable
           egl::ShareGroup *shareGroup,
           TextureManager *shareTextures,
           SemaphoreManager *shareSemaphores,
-          egl::ContextMutex *sharedContextMutex,
-          egl::SingleContextMutex *singleContextMutex,
+          egl::ContextMutex *contextMutex,
           const OverlayType *overlay,
           const EGLenum clientType,
           const Version &clientVersion,
@@ -854,8 +852,8 @@ class State : angle::NonCopyable
 
     // If both a Program and a ProgramPipeline are bound, the Program will
     // always override the ProgramPipeline.
-    ProgramExecutable *getProgramExecutable() const { return mExecutable; }
-    ProgramExecutable *getLinkedProgramExecutable(const Context *context) const
+    ProgramExecutable *getProgramExecutable() const { return mExecutable.get(); }
+    void ensureNoPendingLink(const Context *context) const
     {
         if (mProgram)
         {
@@ -865,7 +863,11 @@ class State : angle::NonCopyable
         {
             mProgramPipeline->resolveLink(context);
         }
-        return mExecutable;
+    }
+    ProgramExecutable *getLinkedProgramExecutable(const Context *context) const
+    {
+        ensureNoPendingLink(context);
+        return mExecutable.get();
     }
 
     // Program binding manipulation
@@ -1051,9 +1053,9 @@ class State : angle::NonCopyable
     }
 
     // Sets the dirty bit for the program executable.
-    angle::Result onProgramExecutableChange(const Context *context, Program *program);
+    angle::Result installProgramExecutable(const Context *context);
     // Sets the dirty bit for the program pipeline executable.
-    angle::Result onProgramPipelineExecutableChange(const Context *context);
+    angle::Result installProgramPipelineExecutable(const Context *context);
 
     const state::DirtyBits getDirtyBits() const
     {
@@ -1342,7 +1344,6 @@ class State : angle::NonCopyable
     float getLineWidth() const { return mPrivateState.getLineWidth(); }
     unsigned int getActiveSampler() const { return mPrivateState.getActiveSampler(); }
     GLenum getGenerateMipmapHint() const { return mPrivateState.getGenerateMipmapHint(); }
-    GLenum getTextureFilteringHint() const { return mPrivateState.getTextureFilteringHint(); }
     GLenum getFragmentShaderDerivativeHint() const
     {
         return mPrivateState.getFragmentShaderDerivativeHint();
@@ -1414,6 +1415,9 @@ class State : angle::NonCopyable
     PrivateState *getMutablePrivateState() { return &mPrivateState; }
     GLES1State *getMutableGLES1State() { return mPrivateState.getMutableGLES1State(); }
 
+    angle::Result installProgramPipelineExecutableIfNotAlready(const Context *context);
+    angle::Result onExecutableChange(const Context *context);
+
     void unsetActiveTextures(const ActiveTextureMask &textureMask);
     void setActiveTextureDirty(size_t textureIndex, Texture *texture);
     void updateTextureBinding(const Context *context, size_t textureIndex, Texture *texture);
@@ -1498,10 +1502,7 @@ class State : angle::NonCopyable
     bool mIsDebugContext;
 
     egl::ShareGroup *mShareGroup;
-    egl::ContextMutex *const mSharedContextMutex;
-    egl::SingleContextMutex *const mSingleContextMutex;
-    std::atomic<egl::ContextMutex *> mContextMutex;  // Simple pointer without reference counting
-    bool mIsSharedContextMutexActive;
+    mutable egl::ContextMutex mContextMutex;
 
     // Resource managers.
     BufferManager *mBufferManager;
@@ -1520,7 +1521,9 @@ class State : angle::NonCopyable
     BindingPointer<Renderbuffer> mRenderbuffer;
     Program *mProgram;
     BindingPointer<ProgramPipeline> mProgramPipeline;
-    ProgramExecutable *mExecutable;
+    // The _installed_ executable.  Note that this may be different from the program's (or the
+    // program pipeline's) executable, as they may have been unsuccessfully relinked.
+    SharedProgramExecutable mExecutable;
 
     VertexArray *mVertexArray;
 
