@@ -20,9 +20,8 @@ class Program11 : public ProgramD3D
     Program11(const gl::ProgramState &programState, Renderer11 *renderer11);
     ~Program11() override;
 
-    angle::Result syncState(const gl::Context *context,
-                            const gl::Program::DirtyBits &dirtyBits) override;
+    angle::Result syncState(const gl::Context *context) override;
 };
 }  // namespace rx
 
-#endif  // LIBANGLE_RENDERER_D3D_D3D11_PROGRAM11_H_
\ No newline at end of file
+#endif  // LIBANGLE_RENDERER_D3D_D3D11_PROGRAM11_H_
