@@ -85,7 +85,9 @@ VkMemoryPropertyFlags GetPreferredMemoryType(RendererVk *renderer,
         case gl::BufferUsage::StreamDraw:
             // For non-static usage where the CPU performs a write-only access, request
             // a host uncached memory
-            return kHostUncachedFlags;
+            return renderer->getFeatures().preferHostCachedForNonStaticBufferUsage.enabled
+                       ? kHostCachedFlags
+                       : kHostUncachedFlags;
         case gl::BufferUsage::DynamicCopy:
         case gl::BufferUsage::DynamicRead:
         case gl::BufferUsage::StreamCopy:
@@ -297,15 +299,15 @@ void BufferVk::destroy(const gl::Context *context)
 {
     ContextVk *contextVk = vk::GetImpl(context);
 
-    release(contextVk);
+    (void)release(contextVk);
 }
 
-void BufferVk::release(ContextVk *contextVk)
+angle::Result BufferVk::release(ContextVk *contextVk)
 {
     RendererVk *renderer = contextVk->getRenderer();
     if (mBuffer.valid())
     {
-        mBuffer.releaseBufferAndDescriptorSetCache(contextVk);
+        ANGLE_TRY(contextVk->releaseBufferAllocation(&mBuffer));
     }
     if (mStagingBuffer.valid())
     {
@@ -317,6 +319,8 @@ void BufferVk::release(ContextVk *contextVk)
         buffer.data->release(renderer);
     }
     mVertexConversionBuffers.clear();
+
+    return angle::Result::Continue;
 }
 
 angle::Result BufferVk::setExternalBufferData(const gl::Context *context,
@@ -328,7 +332,7 @@ angle::Result BufferVk::setExternalBufferData(const gl::Context *context,
     ContextVk *contextVk = vk::GetImpl(context);
 
     // Release and re-create the memory and buffer.
-    release(contextVk);
+    ANGLE_TRY(release(contextVk));
 
     VkBufferCreateInfo createInfo    = {};
     createInfo.sType                 = VK_STRUCTURE_TYPE_BUFFER_CREATE_INFO;
@@ -435,6 +439,19 @@ angle::Result BufferVk::setDataWithMemoryType(const gl::Context *context,
         ANGLE_TRY(GetMemoryTypeIndex(contextVk, size, memoryPropertyFlags, &mMemoryTypeIndex));
         ANGLE_TRY(acquireBufferHelper(contextVk, size, mUsageType));
     }
+    else if (size != static_cast<size_t>(mState.getSize()))
+    {
+        if (mBuffer.onBufferUserSizeChange(renderer))
+        {
+            // If we have a dedicated VkBuffer created with user size, even if the storage is
+            // reused, we have to recreate that VkBuffer with user size when user size changes.
+            // When this happens, we must notify other objects that observing this buffer, such as
+            // vertex array. The reason vertex array is observing the buffer's storage change is
+            // because they uses VkBuffer. Now VkBuffer have changed, vertex array needs to
+            // re-process it just like storage has been reallocated.
+            onStateChange(angle::SubjectMessage::InternalMemoryAllocationChanged);
+        }
+    }
 
     if (data != nullptr)
     {
@@ -495,7 +512,8 @@ angle::Result BufferVk::allocStagingBuffer(ContextVk *contextVk,
     if (mStagingBuffer.valid())
     {
         if (size <= mStagingBuffer.getSize() &&
-            (coherency == vk::MemoryCoherency::Coherent) == mStagingBuffer.isCoherent() &&
+            IsCoherent(coherency) == mStagingBuffer.isCoherent() &&
+            IsCached(coherency) == mStagingBuffer.isCached() &&
             contextVk->getRenderer()->hasResourceUseFinished(mStagingBuffer.getResourceUse()))
         {
             // If size is big enough and it is idle, then just reuse the existing staging buffer
@@ -507,7 +525,7 @@ angle::Result BufferVk::allocStagingBuffer(ContextVk *contextVk,
     }
 
     ANGLE_TRY(
-        mStagingBuffer.allocateForCopyBuffer(contextVk, static_cast<size_t>(size), coherency));
+        contextVk->initBufferForBufferCopy(&mStagingBuffer, static_cast<size_t>(size), coherency));
     *mapPtr                = mStagingBuffer.getMappedMemory();
     mIsStagingBufferMapped = true;
 
@@ -540,7 +558,7 @@ angle::Result BufferVk::handleDeviceLocalBufferMap(ContextVk *contextVk,
                                                    VkDeviceSize size,
                                                    uint8_t **mapPtr)
 {
-    ANGLE_TRY(allocStagingBuffer(contextVk, vk::MemoryCoherency::Coherent, size, mapPtr));
+    ANGLE_TRY(allocStagingBuffer(contextVk, vk::MemoryCoherency::CachedCoherent, size, mapPtr));
 
     // Copy data from device local buffer to host visible staging buffer.
     VkBufferCopy copyRegion = {mBuffer.getOffset() + offset, mStagingBuffer.getOffset(), size};
@@ -623,7 +641,7 @@ angle::Result BufferVk::ghostMappedBuffer(ContextVk *contextVk,
         memcpy(dstMapPtr, srcMapPtr, static_cast<size_t>(mState.getSize()));
     }
 
-    src.releaseBufferAndDescriptorSetCache(contextVk);
+    ANGLE_TRY(contextVk->releaseBufferAllocation(&src));
 
     // Return the already mapped pointer with the offset adjustment to avoid the call to unmap().
     *mapPtr = dstMapPtr + offset;
@@ -716,7 +734,7 @@ angle::Result BufferVk::mapRangeImpl(ContextVk *contextVk,
 
     if (smallMapRange && rangeInvalidate)
     {
-        ANGLE_TRY(allocStagingBuffer(contextVk, vk::MemoryCoherency::NonCoherent,
+        ANGLE_TRY(allocStagingBuffer(contextVk, vk::MemoryCoherency::CachedNonCoherent,
                                      static_cast<size_t>(length), mapPtrBytes));
         return angle::Result::Continue;
     }
@@ -904,8 +922,8 @@ angle::Result BufferVk::stagedUpdate(ContextVk *contextVk,
     if (dataSource.data != nullptr)
     {
         uint8_t *mapPointer = nullptr;
-        ANGLE_TRY(
-            allocStagingBuffer(contextVk, vk::MemoryCoherency::NonCoherent, size, &mapPointer));
+        ANGLE_TRY(allocStagingBuffer(contextVk, vk::MemoryCoherency::CachedNonCoherent, size,
+                                     &mapPointer));
         memcpy(mapPointer, dataSource.data, size);
         ANGLE_TRY(flushStagingBuffer(contextVk, offset, size));
         mIsStagingBufferMapped = false;
@@ -1029,7 +1047,7 @@ angle::Result BufferVk::acquireAndUpdate(ContextVk *contextVk,
 
     if (prevBuffer.valid())
     {
-        prevBuffer.releaseBufferAndDescriptorSetCache(contextVk);
+        ANGLE_TRY(contextVk->releaseBufferAllocation(&prevBuffer));
     }
 
     return angle::Result::Continue;
@@ -1151,11 +1169,12 @@ angle::Result BufferVk::acquireBufferHelper(ContextVk *contextVk,
 
     if (mBuffer.valid())
     {
-        mBuffer.releaseBufferAndDescriptorSetCache(contextVk);
+        ANGLE_TRY(contextVk->releaseBufferAllocation(&mBuffer));
     }
 
     // Allocate the buffer directly
-    ANGLE_TRY(mBuffer.initSuballocation(contextVk, mMemoryTypeIndex, size, alignment, usageType));
+    ANGLE_TRY(
+        contextVk->initBufferAllocation(&mBuffer, mMemoryTypeIndex, size, alignment, usageType));
 
     // Tell the observers (front end) that a new buffer was created, so the necessary
     // dirty bits can be set. This allows the buffer views pointing to the old buffer to
