@@ -227,10 +227,19 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
     {
         return mShareGroupVk->getDescriptorSetLayoutCache();
     }
+    vk::DescriptorSetArray<vk::MetaDescriptorPool> &getMetaDescriptorPools()
+    {
+        return mShareGroupVk->getMetaDescriptorPools();
+    }
 
     // Device loss
     gl::GraphicsResetStatus getResetStatus() override;
 
+    bool isDebugEnabled()
+    {
+        return mRenderer->enableDebugUtils() || mRenderer->angleDebuggerMode();
+    }
+
     // EXT_debug_marker
     angle::Result insertEventMarker(GLsizei length, const char *marker) override;
     angle::Result pushGroupMarker(GLsizei length, const char *marker) override;
@@ -304,6 +313,8 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
     CompilerImpl *createCompiler() override;
     ShaderImpl *createShader(const gl::ShaderState &state) override;
     ProgramImpl *createProgram(const gl::ProgramState &state) override;
+    ProgramExecutableImpl *createProgramExecutable(
+        const gl::ProgramExecutable *executable) override;
 
     // Framebuffer creation
     FramebufferImpl *createFramebuffer(const gl::FramebufferState &state) override;
@@ -668,11 +679,8 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
     size_t getVkIndexTypeSize(gl::DrawElementsType glIndexType) const;
     bool shouldConvertUint8VkIndexType(gl::DrawElementsType glIndexType) const;
 
-    ProgramExecutableVk *getExecutable() const;
-
     bool isRobustResourceInitEnabled() const;
-
-    uint32_t getDriverUniformSize(PipelineType pipelineType) const;
+    bool hasRobustAccess() const { return mState.hasRobustAccess(); }
 
     // Queries that begin and end automatically with render pass start and end
     angle::Result beginRenderPassQuery(QueryVk *queryVk);
@@ -739,12 +747,6 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
         return angle::Result::Continue;
     }
 
-    angle::Result bindCachedDescriptorPool(
-        DescriptorSetIndex descriptorSetIndex,
-        const vk::DescriptorSetLayoutDesc &descriptorSetLayoutDesc,
-        uint32_t descriptorCountMultiplier,
-        vk::DescriptorPoolPointer *poolPointerOut);
-
     // Put the context in framebuffer fetch mode.  If the permanentlySwitchToFramebufferFetchMode
     // feature is enabled, this is done on first encounter of framebuffer fetch, and makes the
     // context use framebuffer-fetch-enabled render passes from here on.
@@ -765,17 +767,20 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
 
     // Whether VK_EXT_pipeline_robustness should be used to enable robust buffer access in the
     // pipeline.
-    bool shouldUsePipelineRobustness() const
+    vk::PipelineRobustness pipelineRobustness() const
     {
-        return getFeatures().supportsPipelineRobustness.enabled && mState.hasRobustAccess();
+        return getFeatures().supportsPipelineRobustness.enabled && mState.hasRobustAccess()
+                   ? vk::PipelineRobustness::Robust
+                   : vk::PipelineRobustness::NonRobust;
     }
     // Whether VK_EXT_pipeline_protected_access should be used to restrict the pipeline to protected
     // command buffers.  Note that when false, if the extension is supported, the pipeline can be
     // restricted to unprotected command buffers.
-    bool shouldRestrictPipelineToProtectedAccess() const
+    vk::PipelineProtectedAccess pipelineProtectedAccess() const
     {
-        return getFeatures().supportsPipelineProtectedAccess.enabled &&
-               mState.hasProtectedContent();
+        return getFeatures().supportsPipelineProtectedAccess.enabled && mState.hasProtectedContent()
+                   ? vk::PipelineProtectedAccess::Protected
+                   : vk::PipelineProtectedAccess::Unprotected;
     }
 
     vk::ComputePipelineFlags getComputePipelineFlags() const;
@@ -809,6 +814,43 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
 
     bool isDitherEnabled() { return mState.isDitherEnabled(); }
 
+    // The following functions try to allocate memory for buffers and images. If they fail due to
+    // OOM errors, they will try other options for memory allocation.
+    angle::Result initBufferAllocation(vk::BufferHelper *bufferHelper,
+                                       uint32_t memoryTypeIndex,
+                                       size_t allocationSize,
+                                       size_t alignment,
+                                       BufferUsageType bufferUsageType);
+    angle::Result initImageAllocation(vk::ImageHelper *imageHelper,
+                                      bool hasProtectedContent,
+                                      const vk::MemoryProperties &memoryProperties,
+                                      VkMemoryPropertyFlags flags,
+                                      vk::MemoryAllocationType allocationType);
+
+    angle::Result releaseBufferAllocation(vk::BufferHelper *bufferHelper);
+
+    // Helper functions to initialize a buffer for a specific usage
+    // Suballocate a host visible buffer with alignment good for copyBuffer.
+    angle::Result initBufferForBufferCopy(vk::BufferHelper *bufferHelper,
+                                          size_t size,
+                                          vk::MemoryCoherency coherency);
+    // Suballocate a host visible buffer with alignment good for copyImage.
+    angle::Result initBufferForImageCopy(vk::BufferHelper *bufferHelper,
+                                         size_t size,
+                                         vk::MemoryCoherency coherency,
+                                         angle::FormatID formatId,
+                                         VkDeviceSize *offset,
+                                         uint8_t **dataPtr);
+    // Suballocate a buffer with alignment good for shader storage or copyBuffer.
+    angle::Result initBufferForVertexConversion(vk::BufferHelper *bufferHelper,
+                                                size_t size,
+                                                vk::MemoryHostVisibility hostVisibility);
+
+    // In the event of collecting too much garbage, we should flush the garbage so it can be freed.
+    void addToPendingImageGarbage(vk::ResourceUse use, VkDeviceSize size);
+
+    bool hasExcessPendingGarbage() const;
+
   private:
     // Dirty bits.
     enum DirtyBitType : size_t
@@ -1273,8 +1315,6 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
 
     VertexArrayVk *getVertexArray() const;
     FramebufferVk *getDrawFramebuffer() const;
-    ProgramVk *getProgram() const;
-    ProgramPipelineVk *getProgramPipeline() const;
 
     // Read-after-write hazards are generally handled with |glMemoryBarrier| when the source of
     // write is storage output.  When the write is outside render pass, the natural placement of the
@@ -1438,6 +1478,14 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
                                                                   DIRTY_BIT_DESCRIPTOR_SETS};
     static constexpr DirtyBits kXfbBuffersAndDescSetDirtyBits{DIRTY_BIT_TRANSFORM_FEEDBACK_BUFFERS,
                                                               DIRTY_BIT_DESCRIPTOR_SETS};
+    static constexpr DirtyBits kDepthDynamicStateDirtyBits =
+        DirtyBits{DIRTY_BIT_DYNAMIC_DEPTH_BIAS, DIRTY_BIT_DYNAMIC_DEPTH_TEST_ENABLE,
+                  DIRTY_BIT_DYNAMIC_DEPTH_WRITE_ENABLE, DIRTY_BIT_DYNAMIC_DEPTH_COMPARE_OP,
+                  DIRTY_BIT_DYNAMIC_DEPTH_BIAS_ENABLE};
+    static constexpr DirtyBits kStencilDynamicStateDirtyBits =
+        DirtyBits{DIRTY_BIT_DYNAMIC_STENCIL_COMPARE_MASK, DIRTY_BIT_DYNAMIC_STENCIL_WRITE_MASK,
+                  DIRTY_BIT_DYNAMIC_STENCIL_REFERENCE, DIRTY_BIT_DYNAMIC_STENCIL_TEST_ENABLE,
+                  DIRTY_BIT_DYNAMIC_STENCIL_OP};
 
     // The offset we had the last time we bound the index buffer.
     const GLvoid *mLastIndexBufferOffset;
@@ -1511,8 +1559,8 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
 
     // The garbage list for single context use objects. The list will be GPU tracked by next
     // submission queueSerial. Note: Resource based shared object should always be added to
-    // renderer's mSharedGarbage.
-    vk::GarbageList mCurrentGarbage;
+    // renderer's mSharedGarbageList.
+    vk::GarbageObjects mCurrentGarbage;
 
     RenderPassCache mRenderPassCache;
 
@@ -1579,6 +1627,7 @@ class ContextVk : public ContextImpl, public vk::Context, public MultisampleText
     // The size of copy commands issued between buffers and images. Used to submit the command
     // buffer for the outside render pass.
     VkDeviceSize mTotalBufferToImageCopySize;
+    VkDeviceSize mEstimatedPendingImageGarbageSize;
 
     // Semaphores that must be flushed before the current commands. Flushed semaphores will be
     // waited on in the next submission.
@@ -1689,17 +1738,21 @@ ANGLE_INLINE bool UseLineRaster(const ContextVk *contextVk, gl::PrimitiveMode mo
 {
     return gl::IsLineMode(mode);
 }
+
+uint32_t GetDriverUniformSize(vk::Context *context, PipelineType pipelineType);
 }  // namespace rx
 
 // Generate a perf warning, and insert an event marker in the command buffer.
-#define ANGLE_VK_PERF_WARNING(contextVk, severity, ...)                         \
-    do                                                                          \
-    {                                                                           \
-        char ANGLE_MESSAGE[200];                                                \
-        snprintf(ANGLE_MESSAGE, sizeof(ANGLE_MESSAGE), __VA_ARGS__);            \
-        ANGLE_PERF_WARNING(contextVk->getDebug(), severity, ANGLE_MESSAGE);     \
-                                                                                \
-        contextVk->insertEventMarkerImpl(GL_DEBUG_SOURCE_OTHER, ANGLE_MESSAGE); \
+#define ANGLE_VK_PERF_WARNING(contextVk, severity, ...)                             \
+    do                                                                              \
+    {                                                                               \
+        ANGLE_PERF_WARNING(contextVk->getDebug(), severity, __VA_ARGS__);           \
+        if (contextVk->isDebugEnabled())                                            \
+        {                                                                           \
+            char ANGLE_MESSAGE[200];                                                \
+            snprintf(ANGLE_MESSAGE, sizeof(ANGLE_MESSAGE), __VA_ARGS__);            \
+            contextVk->insertEventMarkerImpl(GL_DEBUG_SOURCE_OTHER, ANGLE_MESSAGE); \
+        }                                                                           \
     } while (0)
 
 // Generate a trace event for graphics profiler, and insert an event marker in the command buffer.
