@@ -15,6 +15,8 @@
 #include "common/utilities.h"
 #include "libANGLE/Context.h"
 #include "libANGLE/InfoLog.h"
+#include "libANGLE/ProgramExecutable.h"
+#include "libANGLE/renderer/ProgramExecutableImpl.h"
 #include "libANGLE/renderer/vulkan/ContextVk.h"
 #include "libANGLE/renderer/vulkan/ShaderInterfaceVariableInfoMap.h"
 #include "libANGLE/renderer/vulkan/spv_utils.h"
@@ -30,10 +32,11 @@ class ShaderInfo final : angle::NonCopyable
     ShaderInfo();
     ~ShaderInfo();
 
-    angle::Result initShaders(ContextVk *contextVk,
+    angle::Result initShaders(vk::Context *context,
                               const gl::ShaderBitSet &linkedShaderStages,
                               const gl::ShaderMap<const angle::spirv::Blob *> &spirvBlobs,
-                              const ShaderInterfaceVariableInfoMap &variableInfoMap);
+                              const ShaderInterfaceVariableInfoMap &variableInfoMap,
+                              bool isGLES1);
     void initShaderFromProgram(gl::ShaderType shaderType, const ShaderInfo &programShaderInfo);
     void clear();
 
@@ -68,7 +71,7 @@ class ProgramInfo final : angle::NonCopyable
     ProgramInfo();
     ~ProgramInfo();
 
-    angle::Result initProgram(ContextVk *contextVk,
+    angle::Result initProgram(vk::Context *context,
                               gl::ShaderType shaderType,
                               bool isLastPreFragmentStage,
                               bool isTransformFeedbackProgram,
@@ -90,10 +93,10 @@ class ProgramInfo final : angle::NonCopyable
 };
 
 // State for the default uniform blocks.
-struct DefaultUniformBlock final : private angle::NonCopyable
+struct DefaultUniformBlockVk final : private angle::NonCopyable
 {
-    DefaultUniformBlock();
-    ~DefaultUniformBlock();
+    DefaultUniformBlockVk();
+    ~DefaultUniformBlockVk();
 
     // Shadow copies of the shader uniform data.
     angle::MemoryBuffer uniformData;
@@ -107,21 +110,74 @@ struct DefaultUniformBlock final : private angle::NonCopyable
 using DescriptorSetCountList   = angle::PackedEnumMap<DescriptorSetIndex, uint32_t>;
 using ImmutableSamplerIndexMap = angle::HashMap<vk::YcbcrConversionDesc, uint32_t>;
 
-using DefaultUniformBlockMap = gl::ShaderMap<std::shared_ptr<DefaultUniformBlock>>;
+using DefaultUniformBlockMap = gl::ShaderMap<std::shared_ptr<DefaultUniformBlockVk>>;
 
-class ProgramExecutableVk
+class ProgramExecutableVk : public ProgramExecutableImpl
 {
   public:
-    ProgramExecutableVk();
-    virtual ~ProgramExecutableVk();
+    ProgramExecutableVk(const gl::ProgramExecutable *executable);
+    ~ProgramExecutableVk() override;
 
-    void reset(ContextVk *contextVk);
+    void destroy(const gl::Context *context) override;
 
     void save(ContextVk *contextVk, bool isSeparable, gl::BinaryOutputStream *stream);
-    std::unique_ptr<rx::LinkEvent> load(ContextVk *contextVk,
-                                        const gl::ProgramExecutable &glExecutable,
-                                        bool isSeparable,
-                                        gl::BinaryInputStream *stream);
+    angle::Result load(ContextVk *contextVk,
+                       bool isSeparable,
+                       gl::BinaryInputStream *stream,
+                       bool *successOut);
+
+    void setUniform1fv(GLint location, GLsizei count, const GLfloat *v) override;
+    void setUniform2fv(GLint location, GLsizei count, const GLfloat *v) override;
+    void setUniform3fv(GLint location, GLsizei count, const GLfloat *v) override;
+    void setUniform4fv(GLint location, GLsizei count, const GLfloat *v) override;
+    void setUniform1iv(GLint location, GLsizei count, const GLint *v) override;
+    void setUniform2iv(GLint location, GLsizei count, const GLint *v) override;
+    void setUniform3iv(GLint location, GLsizei count, const GLint *v) override;
+    void setUniform4iv(GLint location, GLsizei count, const GLint *v) override;
+    void setUniform1uiv(GLint location, GLsizei count, const GLuint *v) override;
+    void setUniform2uiv(GLint location, GLsizei count, const GLuint *v) override;
+    void setUniform3uiv(GLint location, GLsizei count, const GLuint *v) override;
+    void setUniform4uiv(GLint location, GLsizei count, const GLuint *v) override;
+    void setUniformMatrix2fv(GLint location,
+                             GLsizei count,
+                             GLboolean transpose,
+                             const GLfloat *value) override;
+    void setUniformMatrix3fv(GLint location,
+                             GLsizei count,
+                             GLboolean transpose,
+                             const GLfloat *value) override;
+    void setUniformMatrix4fv(GLint location,
+                             GLsizei count,
+                             GLboolean transpose,
+                             const GLfloat *value) override;
+    void setUniformMatrix2x3fv(GLint location,
+                               GLsizei count,
+                               GLboolean transpose,
+                               const GLfloat *value) override;
+    void setUniformMatrix3x2fv(GLint location,
+                               GLsizei count,
+                               GLboolean transpose,
+                               const GLfloat *value) override;
+    void setUniformMatrix2x4fv(GLint location,
+                               GLsizei count,
+                               GLboolean transpose,
+                               const GLfloat *value) override;
+    void setUniformMatrix4x2fv(GLint location,
+                               GLsizei count,
+                               GLboolean transpose,
+                               const GLfloat *value) override;
+    void setUniformMatrix3x4fv(GLint location,
+                               GLsizei count,
+                               GLboolean transpose,
+                               const GLfloat *value) override;
+    void setUniformMatrix4x3fv(GLint location,
+                               GLsizei count,
+                               GLboolean transpose,
+                               const GLfloat *value) override;
+
+    void getUniformfv(const gl::Context *context, GLint location, GLfloat *params) const override;
+    void getUniformiv(const gl::Context *context, GLint location, GLint *params) const override;
+    void getUniformuiv(const gl::Context *context, GLint location, GLuint *params) const override;
 
     void clearVariableInfoMap();
 
@@ -134,7 +190,6 @@ class ProgramExecutableVk
     angle::Result getGraphicsPipeline(ContextVk *contextVk,
                                       vk::GraphicsPipelineSubset pipelineSubset,
                                       const vk::GraphicsPipelineDesc &desc,
-                                      const gl::ProgramExecutable &glExecutable,
                                       const vk::GraphicsPipelineDesc **descPtrOut,
                                       vk::PipelineHelper **pipelineOut);
 
@@ -143,33 +198,37 @@ class ProgramExecutableVk
                                          vk::PipelineCacheAccess *pipelineCache,
                                          PipelineSource source,
                                          const vk::GraphicsPipelineDesc &desc,
-                                         const gl::ProgramExecutable &glExecutable,
                                          const vk::GraphicsPipelineDesc **descPtrOut,
                                          vk::PipelineHelper **pipelineOut);
 
     angle::Result linkGraphicsPipelineLibraries(ContextVk *contextVk,
                                                 vk::PipelineCacheAccess *pipelineCache,
                                                 const vk::GraphicsPipelineDesc &desc,
-                                                const gl::ProgramExecutable &glExecutable,
                                                 vk::PipelineHelper *vertexInputPipeline,
                                                 vk::PipelineHelper *shadersPipeline,
                                                 vk::PipelineHelper *fragmentOutputPipeline,
                                                 const vk::GraphicsPipelineDesc **descPtrOut,
                                                 vk::PipelineHelper **pipelineOut);
 
-    angle::Result getOrCreateComputePipeline(ContextVk *contextVk,
+    angle::Result getOrCreateComputePipeline(vk::Context *context,
                                              vk::PipelineCacheAccess *pipelineCache,
                                              PipelineSource source,
-                                             const gl::ProgramExecutable &glExecutable,
+                                             vk::PipelineRobustness pipelineRobustness,
+                                             vk::PipelineProtectedAccess pipelineProtectedAccess,
                                              vk::PipelineHelper **pipelineOut);
 
     const vk::PipelineLayout &getPipelineLayout() const { return mPipelineLayout.get(); }
-    angle::Result createPipelineLayout(ContextVk *contextVk,
-                                       const gl::ProgramExecutable &glExecutable,
+    void resetLayout(ContextVk *contextVk);
+    angle::Result createPipelineLayout(vk::Context *context,
+                                       PipelineLayoutCache *pipelineLayoutCache,
+                                       DescriptorSetLayoutCache *descriptorSetLayoutCache,
                                        gl::ActiveTextureArray<TextureVk *> *activeTextures);
+    angle::Result initializeDescriptorPools(
+        vk::Context *context,
+        DescriptorSetLayoutCache *descriptorSetLayoutCache,
+        vk::DescriptorSetArray<vk::MetaDescriptorPool> *metaDescriptorPools);
 
     angle::Result updateTexturesDescriptorSet(vk::Context *context,
-                                              const gl::ProgramExecutable &executable,
                                               const gl::ActiveTextureArray<TextureVk *> &textures,
                                               const gl::SamplerBindingVector &samplers,
                                               bool emulateSeamfulCubeMapSampling,
@@ -231,28 +290,29 @@ class ProgramExecutableVk
         return roundUp(mDefaultUniformBlocks[shaderType]->uniformData.size(), alignment);
     }
 
-    std::shared_ptr<DefaultUniformBlock> &getSharedDefaultUniformBlock(gl::ShaderType shaderType)
+    std::shared_ptr<DefaultUniformBlockVk> &getSharedDefaultUniformBlock(gl::ShaderType shaderType)
     {
         return mDefaultUniformBlocks[shaderType];
     }
 
     bool hasDirtyUniforms() const { return mDefaultUniformBlocksDirty.any(); }
 
-    void setAllDefaultUniformsDirty(const gl::ProgramExecutable &executable);
+    void setAllDefaultUniformsDirty();
     angle::Result updateUniforms(vk::Context *context,
                                  UpdateDescriptorSetsBuilder *updateBuilder,
                                  vk::CommandBufferHelperCommon *commandBufferHelper,
                                  vk::BufferHelper *emptyBuffer,
-                                 const gl::ProgramExecutable &glExecutable,
                                  vk::DynamicBuffer *defaultUniformStorage,
                                  bool isTransformFeedbackActiveUnpaused,
                                  TransformFeedbackVk *transformFeedbackVk);
-    void onProgramBind(const gl::ProgramExecutable &glExecutable);
+    void onProgramBind();
 
     const ShaderInterfaceVariableInfoMap &getVariableInfoMap() const { return mVariableInfoMap; }
 
-    angle::Result warmUpPipelineCache(ContextVk *contextVk,
-                                      const gl::ProgramExecutable &glExecutable);
+    angle::Result warmUpPipelineCache(vk::Context *context,
+                                      vk::PipelineRobustness pipelineRobustness,
+                                      vk::PipelineProtectedAccess pipelineProtectedAccess,
+                                      vk::RenderPass *temporaryCompatibleRenderPassOut);
 
     const vk::WriteDescriptorDescs &getShaderResourceWriteDescriptorDescs() const
     {
@@ -269,13 +329,47 @@ class ProgramExecutableVk
     {
         return mTextureWriteDescriptorDescs;
     }
-    const gl::Program::DirtyBits &getDirtyBits() const { return mDirtyBits; }
+    const gl::ProgramExecutable::DirtyBits &getDirtyBits() const { return mDirtyBits; }
     void resetUniformBufferDirtyBits() { mDirtyBits.reset(); }
 
+    // The following functions are for internal use of programs, including from a threaded link job:
+    angle::Result resizeUniformBlockMemory(vk::Context *context,
+                                           const gl::ShaderMap<size_t> &requiredBufferSize);
+    void resolvePrecisionMismatch(const gl::ProgramMergedVaryings &mergedVaryings);
+    angle::Result initShaders(vk::Context *context,
+                              const gl::ShaderBitSet &linkedShaderStages,
+                              const gl::ShaderMap<const angle::spirv::Blob *> &spirvBlobs,
+                              bool isGLES1)
+    {
+        return mOriginalShaderInfo.initShaders(context, linkedShaderStages, spirvBlobs,
+                                               mVariableInfoMap, isGLES1);
+    }
+    void assignAllSpvLocations(vk::Context *context,
+                               const gl::ProgramState &programState,
+                               const gl::ProgramLinkedResources &resources)
+    {
+        SpvSourceOptions options = SpvCreateSourceOptions(context->getFeatures());
+        SpvAssignAllLocations(options, programState, resources, &mVariableInfoMap);
+    }
+
   private:
     friend class ProgramVk;
     friend class ProgramPipelineVk;
 
+    void reset(ContextVk *contextVk);
+
+    template <int cols, int rows>
+    void setUniformMatrixfv(GLint location,
+                            GLsizei count,
+                            GLboolean transpose,
+                            const GLfloat *value);
+
+    template <class T>
+    void getUniformImpl(GLint location, T *v, GLenum entryPointType) const;
+
+    template <typename T>
+    void setUniformImpl(GLint location, GLsizei count, const T *v, GLenum entryPointType);
+
     void addInterfaceBlockDescriptorSetDesc(const std::vector<gl::InterfaceBlock> &blocks,
                                             gl::ShaderBitSet shaderTypes,
                                             VkDescriptorType descType,
@@ -283,23 +377,17 @@ class ProgramExecutableVk
     void addAtomicCounterBufferDescriptorSetDesc(
         const std::vector<gl::AtomicCounterBuffer> &atomicCounterBuffers,
         vk::DescriptorSetLayoutDesc *descOut);
-    void addImageDescriptorSetDesc(const gl::ProgramExecutable &executable,
-                                   vk::DescriptorSetLayoutDesc *descOut);
-    void addInputAttachmentDescriptorSetDesc(const gl::ProgramExecutable &executable,
-                                             vk::DescriptorSetLayoutDesc *descOut);
+    void addImageDescriptorSetDesc(vk::DescriptorSetLayoutDesc *descOut);
+    void addInputAttachmentDescriptorSetDesc(vk::DescriptorSetLayoutDesc *descOut);
     angle::Result addTextureDescriptorSetDesc(
-        ContextVk *contextVk,
-        const gl::ProgramExecutable &executable,
+        vk::Context *context,
         const gl::ActiveTextureArray<TextureVk *> *activeTextures,
         vk::DescriptorSetLayoutDesc *descOut);
 
-    void resolvePrecisionMismatch(const gl::ProgramMergedVaryings &mergedVaryings);
-
     size_t calcUniformUpdateRequiredSpace(vk::Context *context,
-                                          const gl::ProgramExecutable &glExecutable,
                                           gl::ShaderMap<VkDeviceSize> *uniformOffsets) const;
 
-    ANGLE_INLINE angle::Result initProgram(ContextVk *contextVk,
+    ANGLE_INLINE angle::Result initProgram(vk::Context *context,
                                            gl::ShaderType shaderType,
                                            bool isLastPreFragmentStage,
                                            bool isTransformFeedbackProgram,
@@ -313,7 +401,7 @@ class ProgramExecutableVk
         // specialization constants.
         if (!programInfo->valid(shaderType))
         {
-            ANGLE_TRY(programInfo->initProgram(contextVk, shaderType, isLastPreFragmentStage,
+            ANGLE_TRY(programInfo->initProgram(context, shaderType, isLastPreFragmentStage,
                                                isTransformFeedbackProgram, mOriginalShaderInfo,
                                                optionBits, variableInfoMap));
         }
@@ -323,7 +411,7 @@ class ProgramExecutableVk
     }
 
     ANGLE_INLINE angle::Result initGraphicsShaderProgram(
-        ContextVk *contextVk,
+        vk::Context *context,
         gl::ShaderType shaderType,
         bool isLastPreFragmentStage,
         bool isTransformFeedbackProgram,
@@ -331,41 +419,35 @@ class ProgramExecutableVk
         ProgramInfo *programInfo,
         const ShaderInterfaceVariableInfoMap &variableInfoMap)
     {
-        return initProgram(contextVk, shaderType, isLastPreFragmentStage,
-                           isTransformFeedbackProgram, optionBits, programInfo, variableInfoMap);
+        return initProgram(context, shaderType, isLastPreFragmentStage, isTransformFeedbackProgram,
+                           optionBits, programInfo, variableInfoMap);
     }
 
     ANGLE_INLINE angle::Result initComputeProgram(
-        ContextVk *contextVk,
+        vk::Context *context,
         ProgramInfo *programInfo,
         const ShaderInterfaceVariableInfoMap &variableInfoMap)
     {
         ProgramTransformOptions optionBits = {};
-        return initProgram(contextVk, gl::ShaderType::Compute, false, false, optionBits,
-                           programInfo, variableInfoMap);
+        return initProgram(context, gl::ShaderType::Compute, false, false, optionBits, programInfo,
+                           variableInfoMap);
     }
 
     ProgramTransformOptions getTransformOptions(ContextVk *contextVk,
-                                                const vk::GraphicsPipelineDesc &desc,
-                                                const gl::ProgramExecutable &glExecutable);
-    angle::Result initGraphicsShaderPrograms(ContextVk *contextVk,
+                                                const vk::GraphicsPipelineDesc &desc);
+    angle::Result initGraphicsShaderPrograms(vk::Context *context,
                                              ProgramTransformOptions transformOptions,
-                                             const gl::ProgramExecutable &glExecutable,
                                              vk::ShaderProgramHelper **shaderProgramOut);
-    angle::Result createGraphicsPipelineImpl(ContextVk *contextVk,
+    angle::Result createGraphicsPipelineImpl(vk::Context *context,
                                              ProgramTransformOptions transformOptions,
                                              vk::GraphicsPipelineSubset pipelineSubset,
                                              vk::PipelineCacheAccess *pipelineCache,
                                              PipelineSource source,
                                              const vk::GraphicsPipelineDesc &desc,
-                                             const gl::ProgramExecutable &glExecutable,
+                                             const vk::RenderPass &compatibleRenderPass,
                                              const vk::GraphicsPipelineDesc **descPtrOut,
                                              vk::PipelineHelper **pipelineOut);
 
-    angle::Result resizeUniformBlockMemory(ContextVk *contextVk,
-                                           const gl::ProgramExecutable &glExecutable,
-                                           const gl::ShaderMap<size_t> &requiredBufferSize);
-
     angle::Result getOrAllocateDescriptorSet(vk::Context *context,
                                              UpdateDescriptorSetsBuilder *updateBuilder,
                                              vk::CommandBufferHelperCommon *commandBufferHelper,
@@ -376,14 +458,12 @@ class ProgramExecutableVk
 
     // When loading from cache / binary, initialize the pipeline cache with given data.  Otherwise
     // the cache is lazily created as needed.
-    angle::Result initializePipelineCache(ContextVk *contextVk,
+    angle::Result initializePipelineCache(vk::Context *context,
                                           bool compressed,
                                           const std::vector<uint8_t> &pipelineData);
-    angle::Result ensurePipelineCacheInitialized(ContextVk *contextVk);
+    angle::Result ensurePipelineCacheInitialized(vk::Context *context);
 
-    void resetLayout(ContextVk *contextVk);
-    void initializeWriteDescriptorDesc(ContextVk *contextVk,
-                                       const gl::ProgramExecutable &glExecutable);
+    void initializeWriteDescriptorDesc(vk::Context *context);
 
     // Descriptor sets and pools for shader resources for this program.
     vk::DescriptorSetArray<VkDescriptorSet> mDescriptorSets;
@@ -396,7 +476,7 @@ class ProgramExecutableVk
     // deleted while this program is in use.
     uint32_t mImmutableSamplersMaxDescriptorCount;
     ImmutableSamplerIndexMap mImmutableSamplerIndexMap;
-    vk::BindingPointer<vk::PipelineLayout> mPipelineLayout;
+    vk::AtomicBindingPointer<vk::PipelineLayout> mPipelineLayout;
     vk::DescriptorSetLayoutPointerArray mDescriptorSetLayouts;
 
     // A set of dynamic offsets used with vkCmdBindDescriptorSets for the default uniform buffers.
@@ -448,7 +528,11 @@ class ProgramExecutableVk
     vk::WriteDescriptorDescs mDefaultUniformWriteDescriptorDescs;
     vk::WriteDescriptorDescs mDefaultUniformAndXfbWriteDescriptorDescs;
 
-    gl::Program::DirtyBits mDirtyBits;
+    vk::DescriptorSetLayoutDesc mShaderResourceSetDesc;
+    vk::DescriptorSetLayoutDesc mTextureSetDesc;
+    vk::DescriptorSetLayoutDesc mDefaultUniformAndXfbSetDesc;
+
+    gl::ProgramExecutable::DirtyBits mDirtyBits;
 };
 
 }  // namespace rx
