@@ -21,15 +21,24 @@ VK_DEFINE_HANDLE(VmaVirtualBlock)
 namespace vma
 {
 typedef VkFlags VirtualBlockCreateFlags;
+#if ANGLE_VMA_VERSION < 3000000
 typedef enum VirtualBlockCreateFlagBits
 {
-    GENERAL = 0x0000000,
+    GENERAL = 0x00000000,
     LINEAR  = 0x00000001,
     BUDDY   = 0x00000002
 } VirtualBlockCreateFlagBits;
+#else
+typedef enum VirtualBlockCreateFlagBits
+{
+    GENERAL = 0x00000000,
+    LINEAR  = 0x00000001,
+} VirtualBlockCreateFlagBits;
+#endif
 
 typedef struct StatInfo
 {
+#if ANGLE_VMA_VERSION < 3000000
     // Number of VkDeviceMemory Vulkan memory blocks allocated.
     uint32_t blockCount;
     // Number of VmaAllocation allocation objects allocated.
@@ -42,6 +51,27 @@ typedef struct StatInfo
     VkDeviceSize unusedBytes;
     VkDeviceSize allocationSizeMin, allocationSizeAvg, allocationSizeMax;
     VkDeviceSize unusedRangeSizeMin, unusedRangeSizeAvg, unusedRangeSizeMax;
+#else
+    struct BasicInfo
+    {
+        // Number of VkDeviceMemory Vulkan memory blocks allocated.
+        uint32_t blockCount;
+        // Number of VmaAllocation allocation objects allocated.
+        uint32_t allocationCount;
+        VkDeviceSize blockBytes;
+        VkDeviceSize allocationBytes;
+    } basicInfo;
+    /// Number of free ranges of memory between allocations.
+    uint32_t unusedRangeCount;
+    /// Smallest allocation size. `VK_WHOLE_SIZE` if there are 0 allocations.
+    VkDeviceSize allocationSizeMin;
+    /// Largest allocation size. 0 if there are 0 allocations.
+    VkDeviceSize allocationSizeMax;
+    /// Smallest empty range size. `VK_WHOLE_SIZE` if there are 0 empty ranges.
+    VkDeviceSize unusedRangeSizeMin;
+    /// Largest empty range size. 0 if there are 0 empty ranges.
+    VkDeviceSize unusedRangeSizeMax;
+#endif
 } StatInfo;
 
 VkResult InitAllocator(VkPhysicalDevice physicalDevice,
@@ -77,6 +107,7 @@ VkResult AllocateAndBindMemoryForImage(VmaAllocator allocator,
                                        VkImage *pImage,
                                        VkMemoryPropertyFlags requiredFlags,
                                        VkMemoryPropertyFlags preferredFlags,
+                                       uint32_t memoryTypeBits,
                                        bool allocateDedicatedMemory,
                                        VmaAllocation *pAllocationOut,
                                        uint32_t *pMemoryTypeIndexOut,
