@@ -10,7 +10,7 @@
 namespace angle
 {
 
-class CopyTexImageTest : public ANGLETest
+class CopyTexImageTest : public ANGLETest<>
 {
   protected:
     CopyTexImageTest()
@@ -40,13 +40,13 @@ class CopyTexImageTest : public ANGLETest
 
     void testTearDown() override { glDeleteProgram(mTextureProgram); }
 
-    void initializeResources(GLenum format, GLenum type)
+    void initializeResources(GLenum internalFormat, GLenum format, GLenum type)
     {
         for (size_t i = 0; i < kFboCount; ++i)
         {
             glBindTexture(GL_TEXTURE_2D, mFboTextures[i]);
-            glTexImage2D(GL_TEXTURE_2D, 0, format, kFboSizes[i], kFboSizes[i], 0, format, type,
-                         nullptr);
+            glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, kFboSizes[i], kFboSizes[i], 0, format,
+                         type, nullptr);
 
             // Disable mipmapping
             glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
@@ -63,6 +63,11 @@ class CopyTexImageTest : public ANGLETest
         ASSERT_GL_NO_ERROR();
     }
 
+    void initializeResources(GLenum format, GLenum type)
+    {
+        initializeResources(format, format, type);
+    }
+
     void verifyResults(GLuint texture,
                        const GLubyte data[4],
                        GLint fboSize,
@@ -90,6 +95,33 @@ class CopyTexImageTest : public ANGLETest
         EXPECT_PIXEL_NEAR((xs + xe) / 2, (ys + ye) / 2, data[0], data[1], data[2], data[3], 1.0);
     }
 
+    void verifyCheckeredResults(GLuint texture,
+                                const GLubyte data0[4],
+                                const GLubyte data1[4],
+                                const GLubyte data2[4],
+                                const GLubyte data3[4],
+                                GLint fboWidth,
+                                GLint fboHeight)
+    {
+        glViewport(0, 0, fboWidth, fboHeight);
+
+        glBindFramebuffer(GL_FRAMEBUFFER, 0);
+
+        // Draw a quad with the target texture
+        glUseProgram(mTextureProgram);
+        glBindTexture(GL_TEXTURE_2D, texture);
+        glUniform1i(mTextureUniformLocation, 0);
+
+        drawQuad(mTextureProgram, essl1_shaders::PositionAttrib(), 0.5f);
+
+        // Expect that the rendered quad has the same color as the source texture
+        EXPECT_PIXEL_EQ(fboWidth / 4, fboHeight / 4, data0[0], data0[1], data0[2], data0[3]);
+        EXPECT_PIXEL_EQ(fboWidth / 4, 3 * fboHeight / 4, data1[0], data1[1], data1[2], data1[3]);
+        EXPECT_PIXEL_EQ(3 * fboWidth / 4, fboHeight / 4, data2[0], data2[1], data2[2], data2[3]);
+        EXPECT_PIXEL_EQ(3 * fboWidth / 4, 3 * fboHeight / 4, data3[0], data3[1], data3[2],
+                        data3[3]);
+    }
+
     void runCopyTexImageTest(GLenum format, GLubyte expected[3][4])
     {
         GLTexture tex;
@@ -116,6 +148,47 @@ class CopyTexImageTest : public ANGLETest
         }
     }
 
+    // x, y, width, height specify the portion of fbo to be copied into tex.
+    // flip_y specifies if the glCopyTextImage must be done from y-flipped fbo.
+    void runCopyTexImageTestCheckered(GLenum format,
+                                      const uint32_t x[3],
+                                      const uint32_t y[3],
+                                      const uint32_t width[3],
+                                      const uint32_t height[3],
+                                      const GLubyte expectedData0[4],
+                                      const GLubyte expectedData1[4],
+                                      const GLubyte expectedData2[4],
+                                      const GLubyte expectedData3[4],
+                                      bool mesaFlipY)
+    {
+        GLTexture tex;
+        glBindTexture(GL_TEXTURE_2D, tex);
+
+        // Disable mipmapping
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+        glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+
+        // Perform the copy multiple times.
+        for (size_t i = 0; i < kFboCount; ++i)
+        {
+            glViewport(0, 0, kFboSizes[i], kFboSizes[i]);
+            glBindFramebuffer(GL_FRAMEBUFFER, mFbos[i]);
+
+            ANGLE_GL_PROGRAM(checkerProgram, essl1_shaders::vs::Passthrough(),
+                             essl1_shaders::fs::Checkered());
+            drawQuad(checkerProgram.get(), essl1_shaders::PositionAttrib(), 0.5f);
+            EXPECT_GL_NO_ERROR();
+
+            if (mesaFlipY)
+                glFramebufferParameteriMESA(GL_FRAMEBUFFER, GL_FRAMEBUFFER_FLIP_Y_MESA, 1);
+            glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, x[i], y[i], width[i], height[i], 0);
+            ASSERT_GL_NO_ERROR();
+
+            verifyCheckeredResults(tex, expectedData0, expectedData1, expectedData2, expectedData3,
+                                   kFboSizes[i], kFboSizes[i]);
+        }
+    }
+
     void runCopyTexSubImageTest(GLenum format, GLubyte expected[3][4])
     {
         GLTexture tex;
@@ -206,11 +279,6 @@ class CopyTexImageTest : public ANGLETest
                                                          {0.5f, 0.25f, 1.0f, 0.75f}};
 };
 
-// Until C++17, need to redundantly declare the constexpr members outside the class (only the
-// arrays, because the others are already const-propagated and not needed by the linker).
-constexpr uint32_t CopyTexImageTest::kFboSizes[];
-constexpr GLfloat CopyTexImageTest::kFboColors[][4];
-
 TEST_P(CopyTexImageTest, RGBAToRGB)
 {
     GLubyte expected[3][4] = {
@@ -319,6 +387,20 @@ TEST_P(CopyTexImageTest, SubImageRGBToL)
     runCopyTexSubImageTest(GL_LUMINANCE, expected);
 }
 
+TEST_P(CopyTexImageTest, RGBXToL)
+{
+    ANGLE_SKIP_TEST_IF(!IsGLExtensionEnabled("GL_ANGLE_rgbx_internal_format"));
+
+    GLubyte expected[3][4] = {
+        {64, 64, 64, 255},
+        {255, 255, 255, 255},
+        {127, 127, 127, 255},
+    };
+
+    initializeResources(GL_RGBX8_ANGLE, GL_RGB, GL_UNSIGNED_BYTE);
+    runCopyTexImageTest(GL_LUMINANCE, expected);
+}
+
 // Read default framebuffer with glCopyTexImage2D().
 TEST_P(CopyTexImageTest, DefaultFramebuffer)
 {
@@ -358,7 +440,7 @@ TEST_P(CopyTexImageTest, CopyTexSubImageFromCubeMap)
     // glCopyTexSubImage2D will take one face of this image to copy over a pixel in a 1x6
     // framebuffer.
     GLColor fboPixels[kCubeMapFaceCount]   = {GLColor::red,  GLColor::yellow, GLColor::green,
-                                            GLColor::cyan, GLColor::blue,   GLColor::magenta};
+                                              GLColor::cyan, GLColor::blue,   GLColor::magenta};
     GLColor whitePixels[kCubeMapFaceCount] = {GLColor::white, GLColor::white, GLColor::white,
                                               GLColor::white, GLColor::white, GLColor::white};
 
@@ -409,9 +491,6 @@ TEST_P(CopyTexImageTest, CopyTexSubImageFromCubeMap)
 // Calling CopyTexSubImage to a non-cube-complete texture.
 TEST_P(CopyTexImageTest, CopyTexSubImageToNonCubeCompleteDestination)
 {
-    // TODO(hqle): Find what wrong with NVIDIA GPU. http://anglebug.com/4137
-    ANGLE_SKIP_TEST_IF(IsNVIDIA() && IsMetal());
-
     constexpr GLsizei kCubeMapFaceCount = 6;
 
     // The framebuffer will be a 1x6 image with 6 different colors.  Each glCopyTexSubImage2D will
@@ -466,9 +545,6 @@ TEST_P(CopyTexImageTest, CopyTexSubImageToNonCubeCompleteDestination)
 // Deleting textures after copying to them. http://anglebug.com/4267
 TEST_P(CopyTexImageTest, DeleteAfterCopyingToTextures)
 {
-    // TODO(crbug.com/1132295): Failing on Apple DTK.
-    ANGLE_SKIP_TEST_IF(IsOSX() && IsARM64() && IsDesktopOpenGL());
-
     GLTexture texture;
     glBindTexture(GL_TEXTURE_2D, texture);
     glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 2, 2, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
@@ -509,6 +585,7 @@ TEST_P(CopyTexImageTest, DeleteAfterCopyingToTextures)
     // Crashes on Intel GPUs on macOS.
     texture2.reset();
 }
+
 // Test if glCopyTexImage2D() implementation performs conversions well from GL_TEXTURE_3D to
 // GL_TEXTURE_2D.
 // This is similar to CopyTexImageTestES3.CopyTexSubImageFromTexture3D but for GL_OES_texture_3D
@@ -518,10 +595,10 @@ TEST_P(CopyTexImageTest, CopyTexSubImageFrom3DTexureOES)
     ANGLE_SKIP_TEST_IF(!IsGLExtensionEnabled("GL_OES_texture_3D"));
     // TODO(anglebug.com/3801)
     // Seems to fail on D3D11 Windows.
-    ANGLE_SKIP_TEST_IF(IsD3D11() & IsWindows());
+    ANGLE_SKIP_TEST_IF(IsD3D11() && IsWindows());
 
     // http://anglebug.com/4927
-    ANGLE_SKIP_TEST_IF(IsPixel2() || IsOpenGLES());
+    ANGLE_SKIP_TEST_IF((IsPixel2() || IsNexus5X()) && IsOpenGLES());
 
     constexpr GLsizei kDepth = 6;
 
@@ -529,7 +606,7 @@ TEST_P(CopyTexImageTest, CopyTexSubImageFrom3DTexureOES)
     // glCopyTexSubImage2D will take one face of this image to copy over a pixel in a 1x6
     // framebuffer.
     GLColor fboPixels[kDepth]   = {GLColor::red,  GLColor::yellow, GLColor::green,
-                                 GLColor::cyan, GLColor::blue,   GLColor::magenta};
+                                   GLColor::cyan, GLColor::blue,   GLColor::magenta};
     GLColor whitePixels[kDepth] = {GLColor::white, GLColor::white, GLColor::white,
                                    GLColor::white, GLColor::white, GLColor::white};
 
@@ -569,6 +646,150 @@ TEST_P(CopyTexImageTest, CopyTexSubImageFrom3DTexureOES)
     }
 }
 
+// Tests image copy from y-flipped fbo works.
+TEST_P(CopyTexImageTest, CopyTexImageMesaYFlip)
+{
+    ANGLE_SKIP_TEST_IF(!IsGLExtensionEnabled("GL_MESA_framebuffer_flip_y"));
+
+    std::array<uint32_t, 3> copyOrigin{0};
+    std::array<uint32_t, 3> copySize;
+    for (size_t i = 0; i < kFboCount; i++)
+        copySize[i] = kFboSizes[i];
+
+    initializeResources(GL_RGBA, GL_UNSIGNED_BYTE);
+    runCopyTexImageTestCheckered(GL_RGBA, copyOrigin.data(), copyOrigin.data(), copySize.data(),
+                                 copySize.data(), GLColor::green.data(), GLColor::red.data(),
+                                 GLColor::yellow.data(), GLColor::blue.data(),
+                                 true /* mesaFlipY */);
+}
+
+// Tests image partial copy from y-flipped fbo works.
+TEST_P(CopyTexImageTest, CopyTexImageMesaYFlipPartial)
+{
+    ANGLE_SKIP_TEST_IF(!IsGLExtensionEnabled("GL_MESA_framebuffer_flip_y"));
+
+    std::array<uint32_t, kFboCount> copyX;
+    std::array<uint32_t, kFboCount> copyY{0};
+    std::array<uint32_t, kFboCount> copyWidth;
+    std::array<uint32_t, kFboCount> copyHeight;
+
+    for (size_t i = 0; i < kFboCount; i++)
+    {
+        copyX[i]      = kFboSizes[i] / 2;
+        copyHeight[i] = kFboSizes[i];
+    }
+    copyWidth = copyX;
+
+    initializeResources(GL_RGBA, GL_UNSIGNED_BYTE);
+    runCopyTexImageTestCheckered(GL_RGBA, copyX.data(), copyY.data(), copyWidth.data(),
+                                 copyHeight.data(), GLColor::yellow.data(), GLColor::blue.data(),
+                                 GLColor::yellow.data(), GLColor::blue.data(),
+                                 true /* mesaFlipY */);
+}
+
+// Tests subimage copy from y-flipped fbo works.
+TEST_P(CopyTexImageTest, CopyTexSubImageMesaYFlip)
+{
+    ANGLE_SKIP_TEST_IF(!IsGLExtensionEnabled("GL_MESA_framebuffer_flip_y"));
+
+    GLuint format = GL_RGBA;
+    initializeResources(format, GL_UNSIGNED_BYTE);
+
+    glViewport(0, 0, kFboSizes[0], kFboSizes[0]);
+    glBindFramebuffer(GL_FRAMEBUFFER, mFbos[0]);
+
+    ANGLE_GL_PROGRAM(checkerProgram, essl1_shaders::vs::Passthrough(),
+                     essl1_shaders::fs::Checkered());
+    drawQuad(checkerProgram.get(), essl1_shaders::PositionAttrib(), 0.5f);
+    EXPECT_GL_NO_ERROR();
+
+    GLTexture tex;
+    glBindTexture(GL_TEXTURE_2D, tex);
+
+    // Disable mipmapping
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
+    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
+
+    // Create the texture with copy of the first fbo.
+    glBindFramebuffer(GL_FRAMEBUFFER, mFbos[0]);
+    glFramebufferParameteriMESA(GL_FRAMEBUFFER, GL_FRAMEBUFFER_FLIP_Y_MESA, 1);
+    glCopyTexImage2D(GL_TEXTURE_2D, 0, format, 0, 0, kFboSizes[0], kFboSizes[0], 0);
+    ASSERT_GL_NO_ERROR();
+
+    // Make sure out-of-bound writes to the texture return invalid value.
+    glBindFramebuffer(GL_FRAMEBUFFER, mFbos[1]);
+    drawQuad(checkerProgram.get(), essl1_shaders::PositionAttrib(), 0.5f);
+    EXPECT_GL_NO_ERROR();
+
+    glFramebufferParameteriMESA(GL_FRAMEBUFFER, GL_FRAMEBUFFER_FLIP_Y_MESA, 1);
+
+    // xoffset < 0 and yoffset < 0
+    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, -1, -1, 0, 0, kFboSizes[0], kFboSizes[0]);
+    ASSERT_GL_ERROR(GL_INVALID_VALUE);
+
+    // xoffset + width > w and yoffset + height > h
+    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 1, 1, 0, 0, kFboSizes[0], kFboSizes[0]);
+    ASSERT_GL_ERROR(GL_INVALID_VALUE);
+
+    // xoffset + width > w and yoffset + height > h, out of bounds
+    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, -1, -1, 1 + kFboSizes[0], 1 + kFboSizes[0]);
+    ASSERT_GL_ERROR(GL_INVALID_VALUE);
+
+    // Copy the second fbo over a portion of the image.
+    GLint offset = kFboSizes[0] / 2;
+    GLint extent = kFboSizes[0] - offset;
+    glCopyTexSubImage2D(GL_TEXTURE_2D, 0, offset, offset, kFboSizes[1] / 2, kFboSizes[1] / 2,
+                        extent, extent);
+    ASSERT_GL_NO_ERROR();
+
+    // Only part of the image is changed.
+    verifyCheckeredResults(tex, GLColor::green.data(), GLColor::red.data(), GLColor::yellow.data(),
+                           GLColor::blue.data(), kFboSizes[0], kFboSizes[0]);
+}
+
+// Tests that set RobustResourceInit to true, so that code path with
+// RobustResourceInit == true can be checked
+class CopyTexImageTestRobustResourceInit : public CopyTexImageTest
+{
+  protected:
+    CopyTexImageTestRobustResourceInit() : CopyTexImageTest() { setRobustResourceInit(true); }
+};
+
+// Adapted from the fuzz test with invalid input
+TEST_P(CopyTexImageTestRobustResourceInit, InvalidInputParam)
+{
+    glClear(GL_COLOR_BUFFER_BIT);
+
+    const GLint w = getWindowWidth(), h = getWindowHeight();
+    GLTexture tex;
+    glBindTexture(GL_TEXTURE_2D, tex);
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, w, h, 0, GL_RGBA, GL_UNSIGNED_BYTE, nullptr);
+
+    // pass y that is greater than max2DTextureSize
+    GLenum target         = GL_TEXTURE_2D;
+    GLint level           = 0;
+    GLenum internalFormat = GL_LUMINANCE_ALPHA;
+    GLint x               = 0;
+    GLint y               = 13434880;
+    GLsizei width         = 0;
+    GLsizei height        = 65830;
+    GLint border          = 0;
+    glCopyTexImage2D(target, level, internalFormat, x, y, width, height, border);
+    EXPECT_GL_ERROR(GL_INVALID_VALUE);
+
+    // pass x and width that will result in integer overflow when we apply x+width
+    target         = GL_TEXTURE_2D;
+    level          = 0;
+    internalFormat = GL_LUMINANCE_ALPHA;
+    x              = std::numeric_limits<GLint>::max();
+    y              = 0;
+    width          = 253;
+    height         = 1;
+    border         = 0;
+    glCopyTexImage2D(target, level, internalFormat, x, y, width, height, border);
+    EXPECT_GL_ERROR(GL_INVALID_VALUE);
+}
+
 // specialization of CopyTexImageTest is added so that some tests can be explicitly run with an ES3
 // context
 class CopyTexImageTestES3 : public CopyTexImageTest
@@ -640,7 +861,7 @@ TEST_P(CopyTexImageTestES3, ReadBufferIsNone)
 TEST_P(CopyTexImageTestES3, 2DArraySubImage)
 {
     // Seems to fail on AMD OpenGL Windows.
-    ANGLE_SKIP_TEST_IF(IsAMD() && IsOpenGL() & IsWindows());
+    ANGLE_SKIP_TEST_IF(IsAMD() && IsOpenGL() && IsWindows());
 
     GLTexture tex;
     glBindTexture(GL_TEXTURE_2D_ARRAY, tex);
@@ -682,7 +903,7 @@ TEST_P(CopyTexImageTestES3, CopyTexSubImageFromTexture3D)
 {
     // TODO(anglebug.com/3801)
     // Seems to fail on D3D11 Windows.
-    ANGLE_SKIP_TEST_IF(IsD3D11() & IsWindows());
+    ANGLE_SKIP_TEST_IF(IsD3D11() && IsWindows());
 
     constexpr GLsizei kTexSize = 4;
     constexpr GLsizei kLayers  = 2;
@@ -1001,6 +1222,9 @@ TEST_P(CopyTexImageTestES3, 3DSubImageDrawMismatchedTextureTypes)
     // TODO(anglebug.com/3801)
     ANGLE_SKIP_TEST_IF(IsWindows() && IsD3D11());
 
+    // TODO(anglebug.com/5491)
+    ANGLE_SKIP_TEST_IF(IsIOS() && IsOpenGLES());
+
     GLFramebuffer fbo;
     glBindFramebuffer(GL_FRAMEBUFFER, fbo);
 
@@ -1039,17 +1263,78 @@ TEST_P(CopyTexImageTestES3, 3DSubImageDrawMismatchedTextureTypes)
     glBindTexture(GL_TEXTURE_3D, 0);
 }
 
-// Use this to select which configurations (e.g. which renderer, which GLES major version) these
-// tests should be run against.
+// Make sure a single-level texture can be redefined through glCopyTexImage2D from a framebuffer
+// bound to the same texture.  Regression test for a bug in the Vulkan backend where the texture was
+// released before the copy.
+TEST_P(CopyTexImageTestES3, RedefineSameLevel)
+{
+    constexpr GLsizei kSize     = 32;
+    constexpr GLsizei kHalfSize = kSize / 2;
+
+    // Create a single-level texture with four colors in different regions.
+    std::vector<GLColor> initData(kSize * kSize);
+    for (GLsizei y = 0; y < kSize; ++y)
+    {
+        const bool isTop = y < kHalfSize;
+        for (GLsizei x = 0; x < kSize; ++x)
+        {
+            const bool isLeft = x < kHalfSize;
+
+            GLColor color           = isLeft && isTop    ? GLColor::red
+                                      : isLeft && !isTop ? GLColor::green
+                                      : !isLeft && isTop ? GLColor::blue
+                                                         : GLColor::yellow;
+            color.A                 = 123;
+            initData[y * kSize + x] = color;
+        }
+    }
+
+    GLTexture tex;
+    glBindTexture(GL_TEXTURE_2D, tex);
+    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, kSize, kSize, 0, GL_RGBA, GL_UNSIGNED_BYTE,
+                 initData.data());
+
+    // Bind the framebuffer to the same texture
+    GLFramebuffer framebuffer;
+    glBindFramebuffer(GL_FRAMEBUFFER, framebuffer);
+    glFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D, tex, 0);
+
+    // Redefine the texture
+    glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, kHalfSize / 2, kHalfSize / 2, kHalfSize, kHalfSize,
+                     0);
+
+    // Verify copy is done correctly.
+    ASSERT_GL_FRAMEBUFFER_COMPLETE(GL_FRAMEBUFFER);
+
+    EXPECT_PIXEL_RECT_EQ(0, 0, kHalfSize / 2, kHalfSize / 2, GLColor::red);
+    EXPECT_PIXEL_RECT_EQ(kHalfSize / 2, 0, kHalfSize / 2, kHalfSize / 2, GLColor::blue);
+    EXPECT_PIXEL_RECT_EQ(0, kHalfSize / 2, kHalfSize / 2, kHalfSize / 2, GLColor::green);
+    EXPECT_PIXEL_RECT_EQ(kHalfSize / 2, kHalfSize / 2, kHalfSize / 2, kHalfSize / 2,
+                         GLColor::yellow);
+}
+
 ANGLE_INSTANTIATE_TEST(CopyTexImageTest,
                        ANGLE_ALL_TEST_PLATFORMS_ES2,
                        ES2_D3D11_PRESENT_PATH_FAST(),
                        ES3_VULKAN(),
-                       WithEmulateCopyTexImage2DFromRenderbuffers(ES2_OPENGL()),
-                       WithEmulateCopyTexImage2DFromRenderbuffers(ES2_OPENGLES()));
+                       ES2_OPENGL().enable(Feature::EmulateCopyTexImage2D),
+                       ES2_OPENGLES().enable(Feature::EmulateCopyTexImage2D),
+                       ES2_OPENGL().enable(Feature::EmulateCopyTexImage2DFromRenderbuffers),
+                       ES2_OPENGLES().enable(Feature::EmulateCopyTexImage2DFromRenderbuffers));
 
+GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(CopyTexImageTestES3);
 ANGLE_INSTANTIATE_TEST(CopyTexImageTestES3,
                        ANGLE_ALL_TEST_PLATFORMS_ES3,
-                       WithEmulateCopyTexImage2DFromRenderbuffers(ES3_OPENGL()),
-                       WithEmulateCopyTexImage2DFromRenderbuffers(ES3_OPENGLES()));
+                       ES3_OPENGL().enable(Feature::EmulateCopyTexImage2D),
+                       ES3_OPENGLES().enable(Feature::EmulateCopyTexImage2D),
+                       ES3_OPENGL().enable(Feature::EmulateCopyTexImage2DFromRenderbuffers),
+                       ES3_OPENGLES().enable(Feature::EmulateCopyTexImage2DFromRenderbuffers));
+ANGLE_INSTANTIATE_TEST(CopyTexImageTestRobustResourceInit,
+                       ANGLE_ALL_TEST_PLATFORMS_ES2,
+                       ES2_D3D11_PRESENT_PATH_FAST(),
+                       ES3_VULKAN(),
+                       ES2_OPENGL().enable(Feature::EmulateCopyTexImage2D),
+                       ES2_OPENGLES().enable(Feature::EmulateCopyTexImage2D),
+                       ES2_OPENGL().enable(Feature::EmulateCopyTexImage2DFromRenderbuffers),
+                       ES2_OPENGLES().enable(Feature::EmulateCopyTexImage2DFromRenderbuffers));
 }  // namespace angle
