@@ -14,6 +14,7 @@ import posixpath
 import random
 import re
 import subprocess
+import sys
 import tarfile
 import tempfile
 import threading
@@ -53,7 +54,7 @@ def _FindAapt():
 
 
 def _RemovePrefix(str, prefix):
-    assert str.startswith(prefix)
+    assert str.startswith(prefix), 'Expected prefix %s, got: %s' % (prefix, str)
     return str[len(prefix):]
 
 
@@ -185,21 +186,43 @@ def _RemoveDeviceFile(device_path):
     _AdbShell('rm -f ' + device_path + ' || true')  # ignore errors
 
 
-def _AddRestrictedTracesJson():
-    def add(tar, fn):
-        assert (fn.startswith('../../'))
-        tar.add(fn, arcname=fn.replace('../../', ''))
-
+def _MakeTar(path, patterns):
     with _TempLocalFile() as tempfile_path:
         with tarfile.open(tempfile_path, 'w', format=tarfile.GNU_FORMAT) as tar:
-            for f in glob.glob('../../src/tests/restricted_traces/*/*.json', recursive=True):
-                add(tar, f)
-            add(tar, '../../src/tests/restricted_traces/restricted_traces.json')
-        _AdbRun(['push', tempfile_path, '/sdcard/chromium_tests_root/t.tar'])
+            for p in patterns:
+                for f in glob.glob(p, recursive=True):
+                    tar.add(f, arcname=f.replace('../../', ''))
+        _AdbRun(['push', tempfile_path, path])
 
+
+def _AddRestrictedTracesJson():
+    _MakeTar('/sdcard/chromium_tests_root/t.tar', [
+        '../../src/tests/restricted_traces/*/*.json',
+        '../../src/tests/restricted_traces/restricted_traces.json'
+    ])
     _AdbShell('r=/sdcard/chromium_tests_root; tar -xf $r/t.tar -C $r/ && rm $r/t.tar')
 
 
+def _AddDeqpFiles(suite_name):
+    patterns = [
+        '../../third_party/VK-GL-CTS/src/external/openglcts/data/mustpass/*/*/main/*.txt',
+        '../../src/tests/deqp_support/*.txt'
+    ]
+    if '_gles2_' in suite_name:
+        patterns.append('gen/vk_gl_cts_data/data/gles2/**')
+    if '_gles3_' in suite_name:
+        patterns.append('gen/vk_gl_cts_data/data/gles3/**')
+        patterns.append('gen/vk_gl_cts_data/data/gl_cts/data/gles3/**')
+    if '_gles31_' in suite_name:
+        patterns.append('gen/vk_gl_cts_data/data/gles31/**')
+        patterns.append('gen/vk_gl_cts_data/data/gl_cts/data/gles31/**')
+    if '_gles32_' in suite_name:
+        patterns.append('gen/vk_gl_cts_data/data/gl_cts/data/gles32/**')
+
+    _MakeTar('/sdcard/chromium_tests_root/deqp.tar', patterns)
+    _AdbShell('r=/sdcard/chromium_tests_root; tar -xf $r/deqp.tar -C $r/ && rm $r/deqp.tar')
+
+
 def _GetDeviceApkPath():
     pm_path = _AdbShell('pm path %s || true' % TEST_PACKAGE_NAME).decode().strip()
     if not pm_path:
@@ -210,34 +233,60 @@ def _GetDeviceApkPath():
     return device_apk_path
 
 
+def _LocalFileHash(local_path, gz_tail_size):
+    h = hashlib.sha256()
+    with open(local_path, 'rb') as f:
+        if local_path.endswith('.gz'):
+            # equivalent of tail -c {gz_tail_size}
+            offset = os.path.getsize(local_path) - gz_tail_size
+            if offset > 0:
+                f.seek(offset)
+        for data in iter(lambda: f.read(65536), b''):
+            h.update(data)
+    return h.hexdigest()
+
+
 def _CompareHashes(local_path, device_path):
+    # The last 8 bytes of gzip contain CRC-32 and the initial file size and the preceding
+    # bytes should be affected by changes in the middle if we happen to run into a collision
+    gz_tail_size = 4096
+
+    if local_path.endswith('.gz'):
+        cmd = 'test -f {path} && tail -c {gz_tail_size} {path} | sha256sum -b || true'.format(
+            path=device_path, gz_tail_size=gz_tail_size)
+    else:
+        cmd = 'test -f {path} && sha256sum -b {path} || true'.format(path=device_path)
+
     if device_path.startswith('/data'):
         # Use run-as for files that reside on /data, which aren't accessible without root
-        device_hash = _AdbShell('run-as ' + TEST_PACKAGE_NAME + ' sha256sum -b ' + device_path +
-                                ' 2> /dev/null || true').decode().strip()
-    else:
-        device_hash = _AdbShell('sha256sum -b ' + device_path +
-                                ' 2> /dev/null || true').decode().strip()
+        cmd = "run-as {TEST_PACKAGE_NAME} sh -c '{cmd}'".format(
+            TEST_PACKAGE_NAME=TEST_PACKAGE_NAME, cmd=cmd)
+
+    device_hash = _AdbShell(cmd).decode().strip()
     if not device_hash:
         logging.debug('_CompareHashes: File not found on device')
         return False  # file not on device
 
-    h = hashlib.sha256()
-    try:
-        with open(local_path, 'rb') as f:
-            for data in iter(lambda: f.read(65536), b''):
-                h.update(data)
-    except Exception as e:
-        logging.error('An error occurred in _CompareHashes: %s' % e)
+    return _LocalFileHash(local_path, gz_tail_size) == device_hash
 
-    return h.hexdigest() == device_hash
+
+def _CheckSameApkInstalled(apk_path):
+    device_apk_path = _GetDeviceApkPath()
+
+    try:
+        if device_apk_path and _CompareHashes(apk_path, device_apk_path):
+            return True
+    except subprocess.CalledProcessError as e:
+        # non-debuggable test apk installed on device breaks run-as
+        logging.warning('_CompareHashes of apk failed: %s' % e)
+
+    return False
 
 
 def _PrepareTestSuite(suite_name):
     apk_path = _ApkPath(suite_name)
-    device_apk_path = _GetDeviceApkPath()
 
-    if device_apk_path and _CompareHashes(apk_path, device_apk_path):
+    if _CheckSameApkInstalled(apk_path):
         logging.info('Skipping APK install because host and device hashes match')
     else:
         logging.info('Installing apk path=%s size=%s' % (apk_path, os.path.getsize(apk_path)))
@@ -260,6 +309,9 @@ def _PrepareTestSuite(suite_name):
     if suite_name == ANGLE_TRACE_TEST_SUITE:
         _AddRestrictedTracesJson()
 
+    if '_deqp_' in suite_name:
+        _AddDeqpFiles(suite_name)
+
     if suite_name == 'angle_end2end_tests':
         _AdbRun([
             'push', '../../src/tests/angle_end2end_tests_expectations.txt',
@@ -294,6 +346,11 @@ def PrepareRestrictedTraces(traces):
 
     def _PushLibToAppDir(lib_name):
         local_path = lib_name
+        if not os.path.exists(local_path):
+            print('Error: missing library: ' + local_path)
+            print('Is angle_restricted_traces set in gn args?')  # b/294861737
+            sys.exit(1)
+
         device_path = '/data/user/0/com.android.angle.test/angle_traces/' + lib_name
         if _HashesMatch(local_path, device_path):
             return
@@ -304,8 +361,6 @@ def PrepareRestrictedTraces(traces):
             _AdbRun(['push', local_path, tmp_path])
             _AdbShell('run-as ' + TEST_PACKAGE_NAME + ' cp ' + tmp_path + ' ./angle_traces/')
             _AdbShell('rm ' + tmp_path)
-        except Exception as e:
-            logging.error('An error occurred in _PushToAppDir: %s' % e)
         finally:
             _RemoveDeviceFile(tmp_path)
 
@@ -320,13 +375,13 @@ def PrepareRestrictedTraces(traces):
         path_from_root = 'src/tests/restricted_traces/' + trace + '/' + trace + '.angledata.gz'
         _Push('../../' + path_from_root, path_from_root)
 
-        tracegz = 'gen/tracegz_' + trace + '.gz'
-        _Push(tracegz, tracegz)
-
         if _Global.traces_outside_of_apk:
             lib_name = 'libangle_restricted_traces_' + trace + _Global.lib_extension
             _PushLibToAppDir(lib_name)
 
+        tracegz = 'gen/tracegz_' + trace + '.gz'
+        _Push(tracegz, tracegz)
+
     # Push one additional file when running outside the APK
     if _Global.traces_outside_of_apk:
         _PushLibToAppDir('libangle_trace_interpreter' + _Global.lib_extension)
@@ -369,55 +424,59 @@ def _TempLocalFile():
         os.remove(path)
 
 
-def _RunInstrumentation(flags):
-    with _TempDeviceFile() as temp_device_file:
-        cmd = ' '.join([
-            'p=%s;' % TEST_PACKAGE_NAME,
-            'ntr=org.chromium.native_test.NativeTestInstrumentationTestRunner;',
-            'am instrument -w',
-            '-e $ntr.NativeTestActivity "$p".AngleUnitTestActivity',
-            '-e $ntr.ShardNanoTimeout 2400000000000',
-            '-e org.chromium.native_test.NativeTest.CommandLineFlags "%s"' % ' '.join(flags),
-            '-e $ntr.StdoutFile ' + temp_device_file,
-            '"$p"/org.chromium.build.gtest_apk.NativeTestInstrumentationTestRunner',
-        ])
-
-        _AdbShell(cmd)
-        return _ReadDeviceFile(temp_device_file)
-
-
-def _DumpDebugInfo(since_time):
-    logcat_output = _AdbRun(['logcat', '-t', since_time]).decode()
-    logging.info('logcat:\n%s', logcat_output)
-
-    pid_lines = [
-        ln for ln in logcat_output.split('\n')
-        if 'org.chromium.native_test.NativeTest.StdoutFile' in ln
+def _SetCaptureProps(env, device_out_dir):
+    capture_var_map = {  # src/libANGLE/capture/FrameCapture.cpp
+        'ANGLE_CAPTURE_ENABLED': 'debug.angle.capture.enabled',
+        'ANGLE_CAPTURE_FRAME_START': 'debug.angle.capture.frame_start',
+        'ANGLE_CAPTURE_FRAME_END': 'debug.angle.capture.frame_end',
+        'ANGLE_CAPTURE_TRIGGER': 'debug.angle.capture.trigger',
+        'ANGLE_CAPTURE_LABEL': 'debug.angle.capture.label',
+        'ANGLE_CAPTURE_COMPRESSION': 'debug.angle.capture.compression',
+        'ANGLE_CAPTURE_VALIDATION': 'debug.angle.capture.validation',
+        'ANGLE_CAPTURE_VALIDATION_EXPR': 'debug.angle.capture.validation_expr',
+        'ANGLE_CAPTURE_SOURCE_EXT': 'debug.angle.capture.source_ext',
+        'ANGLE_CAPTURE_SOURCE_SIZE': 'debug.angle.capture.source_size',
+        'ANGLE_CAPTURE_FORCE_SHADOW': 'debug.angle.capture.force_shadow',
+    }
+    empty_value = '""'
+    shell_cmds = [
+        # out_dir is special because the corresponding env var is a host path not a device path
+        'setprop debug.angle.capture.out_dir ' + (device_out_dir or empty_value),
+    ] + [
+        'setprop %s %s' % (v, env.get(k, empty_value)) for k, v in sorted(capture_var_map.items())
     ]
-    if pid_lines:
-        debuggerd_output = _AdbShell('debuggerd %s' % pid_lines[-1].split(' ')[2]).decode()
-        logging.warning('debuggerd output:\n%s', debuggerd_output)
+
+    _AdbShell('\n'.join(shell_cmds))
 
 
-def _RunInstrumentationWithTimeout(flags, timeout):
-    initial_time = _AdbShell('date +"%F %T.%3N"').decode().strip()
+def _RunInstrumentation(flags):
+    assert TEST_PACKAGE_NAME == 'com.android.angle.test'  # inlined below for readability
 
-    results = []
+    with _TempDeviceFile() as temp_device_file:
+        cmd = r'''
+am instrument -w \
+    -e org.chromium.native_test.NativeTestInstrumentationTestRunner.StdoutFile {out} \
+    -e org.chromium.native_test.NativeTest.CommandLineFlags "{flags}" \
+    -e org.chromium.native_test.NativeTestInstrumentationTestRunner.ShardNanoTimeout "1000000000000000000" \
+    -e org.chromium.native_test.NativeTestInstrumentationTestRunner.NativeTestActivity \
+    com.android.angle.test.AngleUnitTestActivity \
+    com.android.angle.test/org.chromium.build.gtest_apk.NativeTestInstrumentationTestRunner
+        '''.format(
+            out=temp_device_file, flags=r' '.join(flags)).strip()
 
-    def run():
-        results.append(_RunInstrumentation(flags))
-
-    t = threading.Thread(target=run)
-    t.daemon = True
-    t.start()
-    t.join(timeout=timeout)
-
-    if t.is_alive():  # join timed out
-        logging.warning('Timed out, dumping debug info')
-        _DumpDebugInfo(since_time=initial_time)
-        raise TimeoutError('Test run did not finish in %s seconds' % timeout)
-
-    return results[0]
+        capture_out_dir = os.environ.get('ANGLE_CAPTURE_OUT_DIR')
+        if capture_out_dir:
+            assert os.path.isdir(capture_out_dir)
+            with _TempDeviceDir() as device_out_dir:
+                _SetCaptureProps(os.environ, device_out_dir)
+                try:
+                    _AdbShell(cmd)
+                finally:
+                    _SetCaptureProps({}, None)  # reset
+                _PullDir(device_out_dir, capture_out_dir)
+        else:
+            _AdbShell(cmd)
+        return _ReadDeviceFile(temp_device_file)
 
 
 def AngleSystemInfo(args):
@@ -453,40 +512,6 @@ def _RemoveFlag(args, f):
     return original_value
 
 
-def RunSmokeTest():
-    _EnsureTestSuite(ANGLE_TRACE_TEST_SUITE)
-
-    test_name = 'TraceTest.words_with_friends_2'
-    run_instrumentation_timeout = 60
-
-    logging.info('Running smoke test (%s)', test_name)
-
-    trace_name = GetTraceFromTestName(test_name)
-    if not trace_name:
-        raise Exception('Cannot find trace name from %s.' % test_name)
-
-    PrepareRestrictedTraces([trace_name])
-
-    with _TempDeviceFile() as device_test_output_path:
-        flags = [
-            '--gtest_filter=' + test_name, '--no-warmup', '--steps-per-trial', '1', '--trials',
-            '1', '--isolated-script-test-output=' + device_test_output_path
-        ]
-        try:
-            output = _RunInstrumentationWithTimeout(flags, run_instrumentation_timeout)
-        except TimeoutError:
-            raise Exception('Smoke test did not finish in %s seconds' %
-                            run_instrumentation_timeout)
-
-        test_output = _ReadDeviceFile(device_test_output_path)
-
-    output_json = json.loads(test_output)
-    if output_json['tests'][test_name]['actual'] != 'PASS':
-        raise Exception('Smoke test (%s) failed. Output:\n%s' % (test_name, output))
-
-    logging.info('Smoke test passed')
-
-
 def RunTests(test_suite, args, stdoutfile=None, log_output=True):
     _EnsureTestSuite(test_suite)
 
@@ -511,7 +536,7 @@ def RunTests(test_suite, args, stdoutfile=None, log_output=True):
                 device_output_dir = stack.enter_context(_TempDeviceDir())
                 args.append('--render-test-output-dir=' + device_output_dir)
 
-            output = _RunInstrumentationWithTimeout(args, timeout=10 * 60)
+            output = _RunInstrumentation(args)
 
             if '--list-tests' in args:
                 # When listing tests, there may be no output file. We parse stdout anyways.
