@@ -97,7 +97,8 @@ template<typename, typename = void> class StringTypeAdapter;
 template<typename> class UniqueRef;
 template<typename T, class... Args> UniqueRef<T> makeUniqueRef(Args&&...);
 template<typename, size_t = 0, typename = CrashOnOverflow, size_t = 16, typename = VectorBufferMalloc> class Vector;
-template<typename, typename = DefaultWeakPtrImpl> class WeakPtr;
+template<typename, typename WeakPtrImpl = DefaultWeakPtrImpl, typename = RawPtrTraits<WeakPtrImpl>> class WeakPtr;
+template<typename, typename = DefaultWeakPtrImpl> class WeakRef;
 
 template<typename> struct DefaultHash;
 template<> struct DefaultHash<AtomString>;
@@ -122,7 +123,10 @@ template<typename Key, typename Value, typename Extractor, typename HashFunction
 template<typename Value, typename = DefaultHash<Value>, typename = HashTraits<Value>> class HashCountedSet;
 template<typename KeyArg, typename MappedArg, typename = DefaultHash<KeyArg>, typename = HashTraits<KeyArg>, typename = HashTraits<MappedArg>, typename = HashTableTraits> class HashMap;
 template<typename ValueArg, typename = DefaultHash<ValueArg>, typename = HashTraits<ValueArg>, typename = HashTableTraits> class HashSet;
-
+template<typename ResolveValueT, typename RejectValueT, unsigned options = 0> class NativePromise;
+using GenericPromise = NativePromise<void, void>;
+using GenericNonExclusivePromise = NativePromise<void, void, 1>;
+template<typename T> class NativePromiseRequest;
 }
 
 namespace std {
@@ -145,6 +149,7 @@ using WTF::EnumeratedArray;
 using WTF::FixedVector;
 using WTF::Function;
 using WTF::FunctionDispatcher;
+using WTF::GenericPromise;
 using WTF::HashCountedSet;
 using WTF::HashMap;
 using WTF::HashSet;
@@ -155,6 +160,8 @@ using WTF::Logger;
 using WTF::MachSendRight;
 using WTF::makeUniqueRef;
 using WTF::MonotonicTime;
+using WTF::NativePromise;
+using WTF::NativePromiseRequest;
 using WTF::NeverDestroyed;
 using WTF::ObjectIdentifier;
 using WTF::ObjectIdentifierGeneric;
@@ -181,6 +188,7 @@ using WTF::URL;
 using WTF::UniqueRef;
 using WTF::Vector;
 using WTF::WeakPtr;
+using WTF::WeakRef;
 
 template<class T, class E> using Expected = std::experimental::expected<T, E>;
 template<class E> using Unexpected = std::experimental::unexpected<E>;
