@@ -95,30 +95,55 @@
 #include <wtf/PlatformEnablePlayStation.h>
 #endif
 
+/* --------- GTK/WPE port --------- */
+#if PLATFORM(GTK) || PLATFORM(WPE)
+#include <wtf/PlatformEnableGlib.h>
+#endif
+
 /* ---------  ENABLE macro defaults --------- */
 
 /* Do not use PLATFORM() tests in this section ! */
 
-#if !defined(ENABLE_WEBPROCESS_NSRUNLOOP)
-#define ENABLE_WEBPROCESS_NSRUNLOOP 0
+#if !defined(ENABLE_3D_TRANSFORMS)
+#define ENABLE_3D_TRANSFORMS 0
 #endif
 
-#if !defined(ENABLE_MAC_GESTURE_EVENTS)
-#define ENABLE_MAC_GESTURE_EVENTS 0
+#if !defined(ENABLE_ACCESSIBILITY)
+#define ENABLE_ACCESSIBILITY 1
 #endif
 
-#if !defined(ENABLE_CURSOR_VISIBILITY)
-#define ENABLE_CURSOR_VISIBILITY 0
+#if !defined(ENABLE_ACCESSIBILITY_ANIMATION_CONTROL)
+#define ENABLE_ACCESSIBILITY_ANIMATION_CONTROL 0
+#endif
+
+#if !defined(ENABLE_ADVANCED_PRIVACY_PROTECTIONS)
+#define ENABLE_ADVANCED_PRIVACY_PROTECTIONS 0
 #endif
 
 #if !defined(ENABLE_AIRPLAY_PICKER)
 #define ENABLE_AIRPLAY_PICKER 0
 #endif
 
+#if !defined(ENABLE_ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION)
+#define ENABLE_ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION 0
+#endif
+
+#if !defined(ENABLE_ALTERNATE_WEBM_PLAYER)
+#define ENABLE_ALTERNATE_WEBM_PLAYER 0
+#endif
+
+#if !defined(ENABLE_APP_HIGHLIGHTS)
+#define ENABLE_APP_HIGHLIGHTS 0
+#endif
+
 #if !defined(ENABLE_APPLE_PAY_REMOTE_UI)
 #define ENABLE_APPLE_PAY_REMOTE_UI 0
 #endif
 
+#if !defined(ENABLE_ASYNC_SCROLLING)
+#define ENABLE_ASYNC_SCROLLING 0
+#endif
+
 #if !defined(ENABLE_AUTOCORRECT)
 #define ENABLE_AUTOCORRECT 0
 #endif
@@ -127,60 +152,8 @@
 #define ENABLE_AUTOCAPITALIZE 0
 #endif
 
-#if !defined(ENABLE_TEXT_AUTOSIZING)
-#define ENABLE_TEXT_AUTOSIZING 0
-#endif
-
-#if !defined(ENABLE_IOS_GESTURE_EVENTS)
-#define ENABLE_IOS_GESTURE_EVENTS 0
-#endif
-
-#if !defined(ENABLE_IOS_TOUCH_EVENTS)
-#define ENABLE_IOS_TOUCH_EVENTS 0
-#endif
-
-#if !defined(ENABLE_PRIMARY_SNAPSHOTTED_PLUGIN_HEURISTIC)
-#define ENABLE_PRIMARY_SNAPSHOTTED_PLUGIN_HEURISTIC 0
-#endif
-
-#if !defined(ENABLE_WKPDFVIEW)
-#define ENABLE_WKPDFVIEW 0
-#endif
-
-#if !defined(ENABLE_PREVIEW_CONVERTER)
-#define ENABLE_PREVIEW_CONVERTER 0
-#endif
-
-#if !defined(ENABLE_META_VIEWPORT)
-#define ENABLE_META_VIEWPORT 0
-#endif
-
-#if !defined(ENABLE_FILE_REPLACEMENT)
-#define ENABLE_FILE_REPLACEMENT 0
-#endif
-
-#if !defined(ENABLE_UI_SIDE_COMPOSITING)
-#define ENABLE_UI_SIDE_COMPOSITING 0
-#endif
-
-#if !defined(ENABLE_3D_TRANSFORMS)
-#define ENABLE_3D_TRANSFORMS 0
-#endif
-
-#if !defined(ENABLE_ACCESSIBILITY)
-#define ENABLE_ACCESSIBILITY 1
-#endif
-
-#if !defined(ENABLE_OVERFLOW_SCROLLING_TOUCH)
-#define ENABLE_OVERFLOW_SCROLLING_TOUCH 0
-#endif
-
-#if !defined(ENABLE_APNG)
-#define ENABLE_APNG 1
-#endif
-
-#if !defined(ENABLE_CHANNEL_MESSAGING)
-#define ENABLE_CHANNEL_MESSAGING 1
+#if !defined(ENABLE_BADGING)
+#define ENABLE_BADGING 1
 #endif
 
 #if !defined(ENABLE_CONTENT_CHANGE_OBSERVER)
@@ -199,18 +172,10 @@
 #define ENABLE_CONTEXT_MENU_EVENT 1
 #endif
 
-#if !defined(ENABLE_CSS_BOX_DECORATION_BREAK)
-#define ENABLE_CSS_BOX_DECORATION_BREAK 1
-#endif
-
 #if !defined(ENABLE_CSS_COMPOSITING)
 #define ENABLE_CSS_COMPOSITING 0
 #endif
 
-#if !defined(ENABLE_CSS_CONIC_GRADIENTS)
-#define ENABLE_CSS_CONIC_GRADIENTS 0
-#endif
-
 #if !defined(ENABLE_CSS_TRANSFORM_STYLE_OPTIMIZED_3D)
 #define ENABLE_CSS_TRANSFORM_STYLE_OPTIMIZED_3D 0
 #endif
@@ -219,14 +184,36 @@
 #define ENABLE_CUSTOM_CURSOR_SUPPORT 1
 #endif
 
+#if !defined(ENABLE_CURSOR_VISIBILITY)
+#define ENABLE_CURSOR_VISIBILITY 0
+#endif
+
 #if !defined(ENABLE_DARK_MODE_CSS)
 #define ENABLE_DARK_MODE_CSS 0
 #endif
 
+#if !defined(ENABLE_DATA_DETECTION)
+#define ENABLE_DATA_DETECTION 0
+#endif
+
+#if !defined(ENABLE_DATACUE_VALUE)
+#define ENABLE_DATACUE_VALUE 0
+#endif
+
 #if !defined(ENABLE_DATALIST_ELEMENT)
 #define ENABLE_DATALIST_ELEMENT 0
 #endif
 
+#if ENABLE(INPUT_TYPE_DATE) || ENABLE(INPUT_TYPE_DATETIMELOCAL) || ENABLE(INPUT_TYPE_MONTH) || ENABLE(INPUT_TYPE_TIME) || ENABLE(INPUT_TYPE_WEEK)
+#if !defined(ENABLE_DATE_AND_TIME_INPUT_TYPES)
+#define ENABLE_DATE_AND_TIME_INPUT_TYPES 1
+#endif
+#endif
+
+#if !defined(ENABLE_DECLARATIVE_WEB_PUSH)
+#define ENABLE_DECLARATIVE_WEB_PUSH 0
+#endif
+
 #if !defined(ENABLE_DEVICE_ORIENTATION)
 #define ENABLE_DEVICE_ORIENTATION 0
 #endif
@@ -239,10 +226,6 @@
 #define ENABLE_DESTINATION_COLOR_SPACE_LINEAR_SRGB 0
 #endif
 
-#if !defined(ENABLE_DOWNLOAD_ATTRIBUTE)
-#define ENABLE_DOWNLOAD_ATTRIBUTE 1
-#endif
-
 #if !defined(ENABLE_DRAG_SUPPORT)
 #define ENABLE_DRAG_SUPPORT 1
 #endif
@@ -251,22 +234,20 @@
 #define ENABLE_ENCRYPTED_MEDIA 0
 #endif
 
-#if !defined(ENABLE_FILTERS_LEVEL_2)
-#define ENABLE_FILTERS_LEVEL_2 0
+#if !defined(ENABLE_FILE_REPLACEMENT)
+#define ENABLE_FILE_REPLACEMENT 0
 #endif
 
 #if !defined(ENABLE_FTPDIR)
 #define ENABLE_FTPDIR 1
 #endif
 
-#if !defined(ENABLE_FULLSCREEN_API)
-#define ENABLE_FULLSCREEN_API 0
+#if !defined(ENABLE_FULL_KEYBOARD_ACCESS)
+#define ENABLE_FULL_KEYBOARD_ACCESS 0
 #endif
 
-#if ((PLATFORM(IOS) || PLATFORM(WATCHOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION)) && HAVE(AVKIT)) || PLATFORM(MAC)
-#if !defined(ENABLE_VIDEO_PRESENTATION_MODE)
-#define ENABLE_VIDEO_PRESENTATION_MODE 1
-#endif
+#if !defined(ENABLE_FULLSCREEN_API)
+#define ENABLE_FULLSCREEN_API 0
 #endif
 
 #if !defined(ENABLE_GAMEPAD)
@@ -277,6 +258,30 @@
 #define ENABLE_GEOLOCATION 0
 #endif
 
+#if !defined(ENABLE_GPU_PROCESS_DOM_RENDERING_BY_DEFAULT)
+#define ENABLE_GPU_PROCESS_DOM_RENDERING_BY_DEFAULT 0
+#endif
+
+#if !defined(ENABLE_GPU_PROCESS_WEBGL_BY_DEFAULT)
+#define ENABLE_GPU_PROCESS_WEBGL_BY_DEFAULT 0
+#endif
+
+#if !defined(ENABLE_IMAGE_ANALYSIS)
+#define ENABLE_IMAGE_ANALYSIS 0
+#endif
+
+#if !defined(ENABLE_IMAGE_ANALYSIS_ENHANCEMENTS)
+#define ENABLE_IMAGE_ANALYSIS_ENHANCEMENTS 0
+#endif
+
+#if !defined(ENABLE_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES)
+#define ENABLE_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES 0
+#endif
+
+#if !defined(ENABLE_INLINE_PATH_DATA)
+#define ENABLE_INLINE_PATH_DATA 0
+#endif
+
 #if !defined(ENABLE_INPUT_TYPE_COLOR)
 #define ENABLE_INPUT_TYPE_COLOR 1
 #endif
@@ -301,8 +306,12 @@
 #define ENABLE_INPUT_TYPE_WEEK 0
 #endif
 
-#if !defined(ENABLE_IOS_FORM_CONTROL_REFRESH)
-#define ENABLE_IOS_FORM_CONTROL_REFRESH 0
+#if !defined(ENABLE_IOS_GESTURE_EVENTS)
+#define ENABLE_IOS_GESTURE_EVENTS 0
+#endif
+
+#if !defined(ENABLE_IOS_TOUCH_EVENTS)
+#define ENABLE_IOS_TOUCH_EVENTS 0
 #endif
 
 #if !defined(ENABLE_IPC_TESTING_API)
@@ -312,12 +321,6 @@
 #endif
 #endif
 
-#if ENABLE(INPUT_TYPE_DATE) || ENABLE(INPUT_TYPE_DATETIMELOCAL) || ENABLE(INPUT_TYPE_MONTH) || ENABLE(INPUT_TYPE_TIME) || ENABLE(INPUT_TYPE_WEEK)
-#if !defined(ENABLE_DATE_AND_TIME_INPUT_TYPES)
-#define ENABLE_DATE_AND_TIME_INPUT_TYPES 1
-#endif
-#endif
-
 #if !defined(ENABLE_INSPECTOR_ALTERNATE_DISPATCHERS)
 #define ENABLE_INSPECTOR_ALTERNATE_DISPATCHERS 0
 #endif
@@ -330,6 +333,10 @@
 #define ENABLE_INSPECTOR_TELEMETRY 0
 #endif
 
+#if !defined(ENABLE_KINETIC_SCROLLING)
+#define ENABLE_KINETIC_SCROLLING 0
+#endif
+
 #if !defined(ENABLE_LAYER_BASED_SVG_ENGINE)
 #define ENABLE_LAYER_BASED_SVG_ENGINE 0
 #endif
@@ -338,6 +345,10 @@
 #define ENABLE_LLVM_PROFILE_GENERATION 0
 #endif
 
+#if !defined(ENABLE_MAC_GESTURE_EVENTS)
+#define ENABLE_MAC_GESTURE_EVENTS 0
+#endif
+
 #if !defined(ENABLE_MATHML)
 #define ENABLE_MATHML 1
 #endif
@@ -358,10 +369,6 @@
 #define ENABLE_MEDIA_SOURCE 0
 #endif
 
-#if !defined(ENABLE_MANAGED_MEDIA_SOURCE)
-#define ENABLE_MANAGED_MEDIA_SOURCE 0
-#endif
-
 #if !defined(ENABLE_MEDIA_STATISTICS)
 #define ENABLE_MEDIA_STATISTICS 0
 #endif
@@ -370,6 +377,10 @@
 #define ENABLE_MEDIA_STREAM 0
 #endif
 
+#if !defined(ENABLE_META_VIEWPORT)
+#define ENABLE_META_VIEWPORT 0
+#endif
+
 #if !defined(ENABLE_MHTML)
 #define ENABLE_MHTML 0
 #endif
@@ -378,6 +389,10 @@
 #define ENABLE_MODERN_MEDIA_CONTROLS 0
 #endif
 
+#if !defined(ENABLE_MONOSPACE_FONT_EXCEPTION)
+#define ENABLE_MONOSPACE_FONT_EXCEPTION 0
+#endif
+
 #if !defined(ENABLE_MOUSE_CURSOR_SCALE)
 #define ENABLE_MOUSE_CURSOR_SCALE 0
 #endif
@@ -386,8 +401,8 @@
 #define ENABLE_MOUSE_FORCE_EVENTS 1
 #endif
 
-#if !defined(ENABLE_NETSCAPE_PLUGIN_METADATA_CACHE)
-#define ENABLE_NETSCAPE_PLUGIN_METADATA_CACHE 0
+#if !defined(ENABLE_NOTIFICATION_EVENT)
+#define ENABLE_NOTIFICATION_EVENT 0
 #endif
 
 #if !defined(ENABLE_NOTIFICATIONS)
@@ -402,8 +417,12 @@
 #define ENABLE_OFFSCREEN_CANVAS_IN_WORKERS 1
 #endif
 
-#if !defined(ENABLE_THUNDER)
-#define ENABLE_THUNDER 0
+#if !defined(ENABLE_OPENTYPE_MATH)
+#define ENABLE_OPENTYPE_MATH 0
+#endif
+
+#if !defined(ENABLE_OPENTYPE_VERTICAL)
+#define ENABLE_OPENTYPE_VERTICAL 0
 #endif
 
 // ORIENTATION_EVENTS should never get enabled on Desktop, only Mobile.
@@ -411,6 +430,10 @@
 #define ENABLE_ORIENTATION_EVENTS 0
 #endif
 
+#if !defined(ENABLE_OVERFLOW_SCROLLING_TOUCH)
+#define ENABLE_OVERFLOW_SCROLLING_TOUCH 0
+#endif
+
 #if OS(WINDOWS)
 #if !defined(ENABLE_PAN_SCROLLING)
 #define ENABLE_PAN_SCROLLING 1
@@ -421,24 +444,52 @@
 #define ENABLE_PAYMENT_REQUEST 0
 #endif
 
-#if !defined(ENABLE_PERIODIC_MEMORY_MONITOR)
-#define ENABLE_PERIODIC_MEMORY_MONITOR 0
-#endif
-
-#if !defined(ENABLE_POINTER_LOCK)
-#define ENABLE_POINTER_LOCK 1
+#if !defined(ENABLE_PDFJS)
+#define ENABLE_PDFJS 0
 #endif
 
 #if !defined(ENABLE_PENCIL_HOVER)
 #define ENABLE_PENCIL_HOVER 0
 #endif
 
+#if !defined(ENABLE_PERIODIC_MEMORY_MONITOR)
+#define ENABLE_PERIODIC_MEMORY_MONITOR 0
+#endif
+
+#if !defined(ENABLE_PLATFORM_DRIVEN_TEXT_CHECKING)
+#define ENABLE_PLATFORM_DRIVEN_TEXT_CHECKING 0
+#endif
+
+#if !defined(ENABLE_POINTER_LOCK)
+#define ENABLE_POINTER_LOCK 1
+#endif
+
+#if !defined(ENABLE_PREDEFINED_COLOR_SPACE_DISPLAY_P3)
+#define ENABLE_PREDEFINED_COLOR_SPACE_DISPLAY_P3 0
+#endif
+
+#if !defined(ENABLE_PREVIEW_CONVERTER)
+#define ENABLE_PREVIEW_CONVERTER 0
+#endif
+
 #if !defined(ENABLE_REMOTE_INSPECTOR)
 #define ENABLE_REMOTE_INSPECTOR 0
 #endif
 
-#if !defined(ENABLE_RUBBER_BANDING)
-#define ENABLE_RUBBER_BANDING 0
+#if !defined(ENABLE_REMOTE_LAYER_TREE_ON_MAC_BY_DEFAULT)
+#define ENABLE_REMOTE_LAYER_TREE_ON_MAC_BY_DEFAULT 0
+#endif
+
+#if !defined(ENABLE_RESOURCE_USAGE)
+#define ENABLE_RESOURCE_USAGE 0
+#endif
+
+#if !defined(ENABLE_SCROLLING_THREAD)
+#define ENABLE_SCROLLING_THREAD 0
+#endif
+
+#if !defined(ENABLE_SEC_ITEM_SHIM)
+#define ENABLE_SEC_ITEM_SHIM 0
 #endif
 
 #if !defined(ENABLE_SECURITY_ASSERTIONS)
@@ -460,6 +511,14 @@
 #define ENABLE_SPELLCHECK 0
 #endif
 
+#if !defined(ENABLE_STATIC_IPAD_USER_AGENT_VALUE)
+#define ENABLE_STATIC_IPAD_USER_AGENT_VALUE 0
+#endif
+
+#if !defined(ENABLE_TEXT_AUTOSIZING)
+#define ENABLE_TEXT_AUTOSIZING 0
+#endif
+
 #if !defined(ENABLE_TEXT_CARET)
 #define ENABLE_TEXT_CARET 1
 #endif
@@ -468,8 +527,8 @@
 #define ENABLE_TEXT_SELECTION 1
 #endif
 
-#if !defined(ENABLE_ASYNC_SCROLLING)
-#define ENABLE_ASYNC_SCROLLING 0
+#if !defined(ENABLE_THUNDER)
+#define ENABLE_THUNDER 0
 #endif
 
 #if !defined(ENABLE_TOUCH_EVENTS)
@@ -480,22 +539,26 @@
 #define ENABLE_TOUCH_ACTION_REGIONS 0
 #endif
 
-#if !defined(ENABLE_WHEEL_EVENT_REGIONS)
-#define ENABLE_WHEEL_EVENT_REGIONS 0
+#if !defined(ENABLE_UI_SIDE_COMPOSITING)
+#define ENABLE_UI_SIDE_COMPOSITING 0
 #endif
 
 #if !defined(ENABLE_VIDEO)
 #define ENABLE_VIDEO 0
 #endif
 
-#if !defined(ENABLE_DATACUE_VALUE)
-#define ENABLE_DATACUE_VALUE 0
+#if !defined(ENABLE_VIDEO_PRESENTATION_MODE)
+#define ENABLE_VIDEO_PRESENTATION_MODE 0
 #endif
 
 #if !defined(ENABLE_WEBGL)
 #define ENABLE_WEBGL 0
 #endif
 
+#if !defined(ENABLE_WEBPROCESS_NSRUNLOOP)
+#define ENABLE_WEBPROCESS_NSRUNLOOP 0
+#endif
+
 #if !defined(ENABLE_WEB_ARCHIVE)
 #define ENABLE_WEB_ARCHIVE 0
 #endif
@@ -504,50 +567,10 @@
 #define ENABLE_WEB_AUDIO 0
 #endif
 
-#if !defined(ENABLE_WK_WEB_EXTENSIONS)
-#define ENABLE_WK_WEB_EXTENSIONS 0
-#endif
-
-#if !defined(ENABLE_XSLT)
-#define ENABLE_XSLT 1
-#endif
-
-#if !defined(ENABLE_SERVICE_WORKER)
-#define ENABLE_SERVICE_WORKER 1
-#endif
-
-#if !defined(ENABLE_MONOSPACE_FONT_EXCEPTION)
-#define ENABLE_MONOSPACE_FONT_EXCEPTION 0
-#endif
-
-#if !defined(ENABLE_FULL_KEYBOARD_ACCESS)
-#define ENABLE_FULL_KEYBOARD_ACCESS 0
-#endif
-
-#if !defined(ENABLE_PLATFORM_DRIVEN_TEXT_CHECKING)
-#define ENABLE_PLATFORM_DRIVEN_TEXT_CHECKING 0
-#endif
-
 #if !defined(ENABLE_WEB_PLAYBACK_CONTROLS_MANAGER)
 #define ENABLE_WEB_PLAYBACK_CONTROLS_MANAGER 0
 #endif
 
-#if !defined(ENABLE_RESOURCE_USAGE)
-#define ENABLE_RESOURCE_USAGE 0
-#endif
-
-#if !defined(ENABLE_SEC_ITEM_SHIM)
-#define ENABLE_SEC_ITEM_SHIM 0
-#endif
-
-#if !defined(ENABLE_DATA_DETECTION)
-#define ENABLE_DATA_DETECTION 0
-#endif
-
-#if !defined(ENABLE_FILE_SHARE)
-#define ENABLE_FILE_SHARE 1
-#endif
-
 #if !defined(ENABLE_WEBXR)
 #define ENABLE_WEBXR 0
 #endif
@@ -556,8 +579,24 @@
 #define ENABLE_WEBXR_HANDS 0
 #endif
 
-#if !defined(ENABLE_BADGING)
-#define ENABLE_BADGING 1
+#if !defined(ENABLE_WHEEL_EVENT_LATCHING)
+#define ENABLE_WHEEL_EVENT_LATCHING 0
+#endif
+
+#if !defined(ENABLE_WHEEL_EVENT_REGIONS)
+#define ENABLE_WHEEL_EVENT_REGIONS 0
+#endif
+
+#if !defined(ENABLE_WKPDFVIEW)
+#define ENABLE_WKPDFVIEW 0
+#endif
+
+#if !defined(ENABLE_WK_WEB_EXTENSIONS)
+#define ENABLE_WK_WEB_EXTENSIONS 0
+#endif
+
+#if !defined(ENABLE_XSLT)
+#define ENABLE_XSLT 1
 #endif
 
 /*
@@ -572,6 +611,9 @@
 #define ENABLE_CFPREFS_DIRECT_MODE 0
 #endif
 
+#if !defined(ENABLE_UNPREFIXED_BACKDROP_FILTER)
+#define ENABLE_UNPREFIXED_BACKDROP_FILTER 0
+#endif
 
 
 /* FIXME: This section of the file has not been cleaned up yet and needs major work. */
@@ -592,8 +634,8 @@
 #if CPU(MIPS)
 #undef ENABLE_WEBASSEMBLY
 #define ENABLE_WEBASSEMBLY 0
-#undef ENABLE_WEBASSEMBLY_B3JIT
-#define ENABLE_WEBASSEMBLY_B3JIT 0
+#undef ENABLE_WEBASSEMBLY_OMGJIT
+#define ENABLE_WEBASSEMBLY_OMGJIT 0
 #undef ENABLE_WEBASSEMBLY_BBQJIT
 #define ENABLE_WEBASSEMBLY_BBQJIT 0
 #endif
@@ -612,8 +654,8 @@
 #if CPU(RISCV64)
 #undef ENABLE_WEBASSEMBLY
 #define ENABLE_WEBASSEMBLY 1
-#undef ENABLE_WEBASSEMBLY_B3JIT
-#define ENABLE_WEBASSEMBLY_B3JIT 1
+#undef ENABLE_WEBASSEMBLY_OMGJIT
+#define ENABLE_WEBASSEMBLY_OMGJIT 0
 #undef ENABLE_WEBASSEMBLY_BBQJIT
 #define ENABLE_WEBASSEMBLY_BBQJIT 0
 #endif
@@ -718,16 +760,16 @@
 
 #if ENABLE(WEBASSEMBLY) && ENABLE(JIT) && CPU(ARM)
 #undef ENABLE_B3_JIT
-#define ENABLE_B3_JIT 1
-#undef ENABLE_WEBASSEMBLY_B3JIT
-#define ENABLE_WEBASSEMBLY_B3JIT 1
+#define ENABLE_B3_JIT 0
+#undef ENABLE_WEBASSEMBLY_OMGJIT
+#define ENABLE_WEBASSEMBLY_OMGJIT 0
 #undef ENABLE_WEBASSEMBLY_BBQJIT
 #define ENABLE_WEBASSEMBLY_BBQJIT 0
 #endif
 
 #if !defined(ENABLE_WEBASSEMBLY) && (ENABLE(B3_JIT) && PLATFORM(COCOA) && CPU(ADDRESS64))
 #define ENABLE_WEBASSEMBLY 1
-#define ENABLE_WEBASSEMBLY_B3JIT 1
+#define ENABLE_WEBASSEMBLY_OMGJIT 1
 #define ENABLE_WEBASSEMBLY_BBQJIT 1
 #endif
 
@@ -844,7 +886,7 @@
 #endif
 
 /* CSS Selector JIT Compiler */
-#if !defined(ENABLE_CSS_SELECTOR_JIT) && ((CPU(X86_64) || CPU(ARM64) || (CPU(ARM_THUMB2) && OS(DARWIN))) && ENABLE(JIT) && (OS(DARWIN) || PLATFORM(GTK) || PLATFORM(WPE)))
+#if !defined(ENABLE_CSS_SELECTOR_JIT) && ((CPU(X86_64) || CPU(ARM64) || (CPU(ARM_THUMB2) && OS(DARWIN))) && ENABLE(JIT) && (OS(DARWIN) || OS(WINDOWS) || PLATFORM(GTK) || PLATFORM(WPE)))
 #define ENABLE_CSS_SELECTOR_JIT 1
 #endif
 
@@ -884,44 +926,6 @@
 #define ENABLE_TREE_DEBUGGING 1
 #endif
 
-#if !defined(ENABLE_OPENTYPE_VERTICAL) && PLATFORM(GTK) || PLATFORM(WPE)
-#define ENABLE_OPENTYPE_VERTICAL 1
-#endif
-
-#if !defined(ENABLE_OPENTYPE_MATH) && (OS(DARWIN) && USE(CG)) || PLATFORM(WIN) || PLATFORM(PLAYSTATION)
-#define ENABLE_OPENTYPE_MATH 1
-#endif
-
-#if !defined(ENABLE_INLINE_PATH_DATA) && USE(CG)
-#define ENABLE_INLINE_PATH_DATA 1
-#endif
-
-#if ((PLATFORM(COCOA) || PLATFORM(PLAYSTATION) || PLATFORM(WPE)) && ENABLE(ASYNC_SCROLLING)) || PLATFORM(GTK)
-#define ENABLE_KINETIC_SCROLLING 1
-#endif
-
-#if PLATFORM(MAC)
-// FIXME: Maybe this can be combined with ENABLE_KINETIC_SCROLLING.
-#define ENABLE_WHEEL_EVENT_LATCHING 1
-#endif
-
-#if PLATFORM(MAC)
-#define ENABLE_MOMENTUM_EVENT_DISPATCHER 1
-#endif
-
-#if !defined(ENABLE_SCROLLING_THREAD)
-#if USE(NICOSIA)
-#define ENABLE_SCROLLING_THREAD 1
-#else
-#define ENABLE_SCROLLING_THREAD 0
-#endif
-#endif
-
-// FIXME: Reenable PDFJS by default for Cocoa (https://bugs.webkit.org/show_bug.cgi?id=242263).
-#if !defined(ENABLE_PDFJS) && ( /* PLATFORM(COCOA) || */ PLATFORM(GTK) || PLATFORM(WPE))
-#define ENABLE_PDFJS 1
-#endif
-
 /* This feature works by embedding the OpcodeID in the 32 bit just before the generated LLint code
    that executes each opcode. It cannot be supported by the CLoop since there's no way to embed the
    OpcodeID word in the CLoop's switch statement cases. It is also currently not implemented for MSVC.
@@ -930,26 +934,14 @@
 #define ENABLE_LLINT_EMBEDDED_OPCODE_ID 1
 #endif
 
-#if !defined(ENABLE_PREDEFINED_COLOR_SPACE_DISPLAY_P3)
-#define ENABLE_PREDEFINED_COLOR_SPACE_DISPLAY_P3 0
-#endif
-
-#if !defined(ENABLE_GPU_PROCESS_DOM_RENDERING_BY_DEFAULT) && (PLATFORM(IOS_FAMILY) && !PLATFORM(WATCHOS) && !PLATFORM(VISION) || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000))
-#define ENABLE_GPU_PROCESS_DOM_RENDERING_BY_DEFAULT 1
-#endif
-
-#if !defined(ENABLE_GPU_PROCESS_WEBGL_BY_DEFAULT) && (PLATFORM(IOS_FAMILY) || PLATFORM(MAC)) && !PLATFORM(VISION)
-#define ENABLE_GPU_PROCESS_WEBGL_BY_DEFAULT 1
-#endif
-
-#if !defined(ENABLE_REMOTE_LAYER_TREE_ON_MAC_BY_DEFAULT) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000
-#define ENABLE_REMOTE_LAYER_TREE_ON_MAC_BY_DEFAULT 1
-#endif
-
 /* Asserts, invariants for macro definitions */
 
-#if ENABLE(MEDIA_CONTROLS_SCRIPT) && !ENABLE(VIDEO)
-#error "ENABLE(MEDIA_CONTROLS_SCRIPT) requires ENABLE(VIDEO)"
+#if ENABLE(ALTERNATE_WEBM_PLAYER) && !ENABLE(MEDIA_SOURCE)
+#error "ENABLE(ALTERNATE_WEBM_PLAYER) requires ENABLE(MEDIA_SOURCE)"
+#endif
+
+#if ENABLE(DECLARATIVE_WEB_PUSH) && !ENABLE(NOTIFICATION_EVENT)
+#error "ENABLE(DECLARATIVE_WEB_PUSH) requires ENABLE(NOTIFICATION_EVENT)"
 #endif
 
 #if ENABLE(INSPECTOR_ALTERNATE_DISPATCHERS) && !ENABLE(REMOTE_INSPECTOR)
@@ -964,10 +956,18 @@
 #error "ENABLE(OFFSCREEN_CANVAS_IN_WORKERS) requires ENABLE(OFFSCREEN_CANVAS)"
 #endif
 
+#if ENABLE(MEDIA_CONTROLS_SCRIPT) && !ENABLE(VIDEO)
+#error "ENABLE(MEDIA_CONTROLS_SCRIPT) requires ENABLE(VIDEO)"
+#endif
+
 #if ENABLE(MEDIA_RECORDER) && !ENABLE(MEDIA_STREAM)
 #error "ENABLE(MEDIA_RECORDER) requires ENABLE(MEDIA_STREAM)"
 #endif
 
+#if ENABLE(NOTIFICATION_EVENT) && !ENABLE(NOTIFICATIONS)
+#error "ENABLE(NOTIFICATION_EVENT) requires ENABLE(NOTIFICATIONS)"
+#endif
+
 #if USE(CG)
 
 #if ENABLE(DESTINATION_COLOR_SPACE_DISPLAY_P3) && !HAVE(CORE_GRAPHICS_DISPLAY_P3_COLOR_SPACE)
@@ -988,35 +988,18 @@
 #error "ENABLE(WEBXR_HANDS) requires ENABLE(WEBXR)"
 #endif
 
-#if ENABLE(SERVICE_WORKER) && ENABLE(NOTIFICATIONS) \
-    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || (PLATFORM(IOS) || PLATFORM(VISION) || PLATFORM(GTK) || PLATFORM(WPE)))
-#if !defined(ENABLE_NOTIFICATION_EVENT)
-#define ENABLE_NOTIFICATION_EVENT 1
-#endif
+#if !defined(ENABLE_WEBPROCESS_CACHE)
+#define ENABLE_WEBPROCESS_CACHE 0
 #endif
 
-#if !defined(ENABLE_IMAGE_ANALYSIS_ENHANCEMENTS) \
-    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || (PLATFORM(IOS) || PLATFORM(MACCATALYST)))
-#define ENABLE_IMAGE_ANALYSIS_ENHANCEMENTS 1
+#if !defined(ENABLE_FEATURE_DEFAULT_VALIDATION) \
+    && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
+// FIXME: Check feature flag default values on other platforms once it's
+// possible to make feature status conditional.
+#define ENABLE_FEATURE_DEFAULT_VALIDATION 1
 #endif
 
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
-    || PLATFORM(VISION)
-#define ENABLE_ACCESSIBILITY_ANIMATION_CONTROL 1
-#endif
-
-#if !defined(ENABLE_ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION) \
-    && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 140000)
-#define ENABLE_ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION 1
-#endif
-
-#if !defined(ENABLE_ADVANCED_PRIVACY_PROTECTIONS) \
-    && (PLATFORM(MAC) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
-    || PLATFORM(VISION) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000))
-#define ENABLE_ADVANCED_PRIVACY_PROTECTIONS 1
+#if !defined(ENABLE_EXTENSION_CAPABILITIES) \
+    && USE(EXTENSIONKIT)
+#define ENABLE_EXTENSION_CAPABILITIES 1
 #endif
