@@ -43,10 +43,30 @@
 #define ENABLE_3D_TRANSFORMS 1
 #endif
 
+#if !defined(ENABLE_ACCESSIBILITY_ANIMATION_CONTROL) \
+    && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION)
+#define ENABLE_ACCESSIBILITY_ANIMATION_CONTROL 1
+#endif
+
 #if !defined(ENABLE_ACCESSIBILITY_ISOLATED_TREE) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000
 #define ENABLE_ACCESSIBILITY_ISOLATED_TREE 1
 #endif
 
+#if !defined(ENABLE_ADDITIONAL_PRECONNECT_ON_HTTP_1X) && PLATFORM(MAC)
+#define ENABLE_ADDITIONAL_PRECONNECT_ON_HTTP_1X 1
+#endif
+
+#if !defined(ENABLE_ADVANCED_PRIVACY_PROTECTIONS) \
+    && (PLATFORM(MAC) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000))
+#define ENABLE_ADVANCED_PRIVACY_PROTECTIONS 1
+#endif
+
 #if !defined(ENABLE_AIRPLAY_PICKER) && !PLATFORM(MAC) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 #define ENABLE_AIRPLAY_PICKER 1
 #endif
@@ -55,8 +75,22 @@
 #define ENABLE_ALTERNATE_WEBM_PLAYER 1
 #endif
 
-#if !defined(ENABLE_ANIMATED_KEYBOARD_SCROLLING) && PLATFORM(IOS_FAMILY)
-#define ENABLE_ANIMATED_KEYBOARD_SCROLLING 1
+#if !defined(ENABLE_ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION) \
+    && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 140000)
+#define ENABLE_ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION 1
+#endif
+
+#if !defined(ENABLE_APP_BOUND_DOMAINS) && PLATFORM(IOS_FAMILY)
+#define ENABLE_APP_BOUND_DOMAINS 1
+#endif
+
+#if !defined(ENABLE_APP_HIGHLIGHTS) && (PLATFORM(MAC) || PLATFORM(IOS))
+#define ENABLE_APP_HIGHLIGHTS 1
+#endif
+
+// FIXME: Should this be disabled on tvOS and watchOS?
+#if !defined(ENABLE_APP_PRIVACY_REPORT) && PLATFORM(IOS_FAMILY) && !PLATFORM(APPLETV) && !PLATFORM(WATCHOS)
+#define ENABLE_APP_PRIVACY_REPORT 1
 #endif
 
 #if !defined(ENABLE_APPLE_PAY) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
@@ -79,7 +113,10 @@
 #define ENABLE_APPLE_PAY_DEFERRED_LINE_ITEM 1
 #endif
 
-#if !defined(ENABLE_APPLE_PAY_FEATURES) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
+#if !defined(ENABLE_APPLE_PAY_FEATURES) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
+    || PLATFORM(IOS) \
+    || PLATFORM(VISION))
 #define ENABLE_APPLE_PAY_FEATURES 1
 #endif
 
@@ -87,7 +124,7 @@
 #define ENABLE_APPLE_PAY_INSTALLMENTS 1
 #endif
 
-#if !defined(ENABLE_APPLE_PAY_LATER) && (PLATFORM(IOS)  || PLATFORM(VISION))
+#if !defined(ENABLE_APPLE_PAY_LATER) && (PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_APPLE_PAY_LATER 1
 #endif
 
@@ -159,12 +196,23 @@
 #define ENABLE_APPLICATION_MANIFEST 1
 #endif
 
-#if !defined(ENABLE_APP_BOUND_DOMAINS) && PLATFORM(IOS_FAMILY)
-#define ENABLE_APP_BOUND_DOMAINS 1
+#if !defined(ENABLE_SEPARATED_MODEL) && PLATFORM(VISION) && HAVE(CORE_ANIMATION_SEPARATED_LAYERS)
+#define ENABLE_SEPARATED_MODEL 1
 #endif
 
-#if !defined(ENABLE_MANAGED_DOMAINS)
-#define ENABLE_MANAGED_DOMAINS 1
+#if PLATFORM(IOS_FAMILY) && HAVE(ASV_INLINE_PREVIEW) && !ENABLE(SEPARATED_MODEL)
+#define ENABLE_ARKIT_INLINE_PREVIEW_IOS 1
+#endif
+
+#if PLATFORM(MAC) && HAVE(ASV_INLINE_PREVIEW)
+#define ENABLE_ARKIT_INLINE_PREVIEW_MAC 1
+#endif
+
+#if ENABLE(ARKIT_INLINE_PREVIEW_MAC) || ENABLE(ARKIT_INLINE_PREVIEW_IOS)
+#define ENABLE_ARKIT_INLINE_PREVIEW 1
+#define ENABLE_ARKIT_INLINE_PREVIEW_CAMERA_TRANSFORM 1
+#define ENABLE_ARKIT_INLINE_PREVIEW_ANIMATIONS_CONTROL 1
+#define ENABLE_ARKIT_INLINE_PREVIEW_AUDIO_CONTROL 1
 #endif
 
 #if !defined(ENABLE_ASYNC_SCROLLING)
@@ -187,18 +235,26 @@
 #define ENABLE_AVF_CAPTIONS 1
 #endif
 
+#if !defined(ENABLE_AX_THREAD_TEXT_APIS) && PLATFORM(MAC)
+#define ENABLE_AX_THREAD_TEXT_APIS 0
+#endif
+
 #if !defined(ENABLE_BUILT_IN_NOTIFICATIONS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_BUILT_IN_NOTIFICATIONS 1
 #endif
 
-#if !defined(ENABLE_INSTALL_COORDINATION_BUNDLES) && (PLATFORM(IOS) || PLATFORM(VISION))
-#define ENABLE_INSTALL_COORDINATION_BUNDLES 1
-#endif
-
 #if !defined(ENABLE_CACHE_PARTITIONING)
 #define ENABLE_CACHE_PARTITIONING 1
 #endif
 
+#if !defined(ENABLE_CARENDERSERVER_BLOCKING_IN_WEBCONTENT) \
+    && (((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) && !PLATFORM(VISION) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define ENABLE_CARENDERSERVER_BLOCKING_IN_WEBCONTENT 1
+#endif
+
 #if !defined(ENABLE_CFPREFS_DIRECT_MODE)
 #define ENABLE_CFPREFS_DIRECT_MODE 1
 #endif
@@ -231,10 +287,6 @@
 #define ENABLE_CSS_COMPOSITING 1
 #endif
 
-#if !defined(ENABLE_CSS_CONIC_GRADIENTS)
-#define ENABLE_CSS_CONIC_GRADIENTS 1
-#endif
-
 #if !defined(ENABLE_CSS_PAINTING_API)
 #define ENABLE_CSS_PAINTING_API 1
 #endif
@@ -263,6 +315,20 @@
 #define ENABLE_DATA_DETECTION 1
 #endif
 
+#if !defined(ENABLE_DECLARATIVE_WEB_PUSH) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
+    || PLATFORM(IOS) || PLATFORM(VISION))
+#define ENABLE_DECLARATIVE_WEB_PUSH 1
+#endif
+
+#if !defined(ENABLE_DESTINATION_COLOR_SPACE_DISPLAY_P3)
+#define ENABLE_DESTINATION_COLOR_SPACE_DISPLAY_P3 1
+#endif
+
+#if !defined(ENABLE_DESTINATION_COLOR_SPACE_LINEAR_SRGB)
+#define ENABLE_DESTINATION_COLOR_SPACE_LINEAR_SRGB 1
+#endif
+
 #if !defined(ENABLE_DEVICE_ORIENTATION)
 #define ENABLE_DEVICE_ORIENTATION 1
 #endif
@@ -279,6 +345,14 @@
 #define ENABLE_ENCRYPTED_MEDIA 1
 #endif
 
+// Used in internal build
+#if !defined(ENABLE_ENTERPRISE_PASSKEYS_WK) \
+    && ((PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION) \
+    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000))
+#define ENABLE_ENTERPRISE_PASSKEYS_WK 1
+#endif
+
 #if !defined(ENABLE_FILE_REPLACEMENT)
 #define ENABLE_FILE_REPLACEMENT 1
 #endif
@@ -287,20 +361,24 @@
 #define ENABLE_FILESYSTEM_POSIX_FAST_PATH 1
 #endif
 
-#if !defined(ENABLE_FILTERS_LEVEL_2)
-#define ENABLE_FILTERS_LEVEL_2 1
+#if !defined(ENABLE_FIXED_IOS_TOUCH_POINT_RADIUS) && PLATFORM(VISION)
+#define ENABLE_FIXED_IOS_TOUCH_POINT_RADIUS 1
 #endif
 
 #if !defined(ENABLE_FTL_JIT) && !USE(JSVALUE32_64)
 #define ENABLE_FTL_JIT 1
 #endif
 
-#if !defined(ENABLE_FULLSCREEN_API) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
+#if !defined(ENABLE_FULL_KEYBOARD_ACCESS) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
+#define ENABLE_FULL_KEYBOARD_ACCESS 1
+#endif
+
+#if !defined(ENABLE_FULLSCREEN_API) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !((PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED < 170000))
 #define ENABLE_FULLSCREEN_API 1
 #endif
 
-#if !defined(ENABLE_FULL_KEYBOARD_ACCESS) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
-#define ENABLE_FULL_KEYBOARD_ACCESS 1
+#if !defined(ENABLE_FULLSCREEN_DISMISSAL_GESTURES) && (PLATFORM(IOS_FAMILY) && !PLATFORM(VISION))
+#define ENABLE_FULLSCREEN_DISMISSAL_GESTURES 1
 #endif
 
 #if !defined(ENABLE_GAMEPAD) && !PLATFORM(WATCHOS)
@@ -319,6 +397,16 @@
 #define ENABLE_GPU_PROCESS_BY_DEFAULT 1
 #endif
 
+#if !defined(ENABLE_GPU_PROCESS_DOM_RENDERING_BY_DEFAULT) && (PLATFORM(IOS_FAMILY) && !PLATFORM(WATCHOS) \
+    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000))
+#define ENABLE_GPU_PROCESS_DOM_RENDERING_BY_DEFAULT 1
+#endif
+
+#if !defined(ENABLE_GPU_PROCESS_WEBGL_BY_DEFAULT) && (PLATFORM(IOS_FAMILY) || PLATFORM(MAC))
+#define ENABLE_GPU_PROCESS_WEBGL_BY_DEFAULT 1
+#endif
+
 #if !defined(ENABLE_GPU_DRIVER_PREWARMING) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 140000
 #define ENABLE_GPU_DRIVER_PREWARMING 1
 #endif
@@ -327,6 +415,24 @@
 #define ENABLE_HARDWARE_JPEG 1
 #endif
 
+#if !defined(ENABLE_IMAGE_ANALYSIS) && HAVE(VK_IMAGE_ANALYSIS) && !PLATFORM(VISION)
+#define ENABLE_IMAGE_ANALYSIS 1
+#endif
+
+#if !defined(ENABLE_IMAGE_ANALYSIS_ENHANCEMENTS) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
+    || (PLATFORM(IOS) || PLATFORM(MACCATALYST)))
+#define ENABLE_IMAGE_ANALYSIS_ENHANCEMENTS 1
+#endif
+
+#if !defined(ENABLE_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES) && HAVE(VK_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES) && !PLATFORM(VISION)
+#define ENABLE_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES 1
+#endif
+
+#if !defined(ENABLE_INLINE_PATH_DATA) && USE(CG)
+#define ENABLE_INLINE_PATH_DATA 1
+#endif
+
 #if !defined(ENABLE_INPUT_TYPE_DATE)
 #define ENABLE_INPUT_TYPE_DATE 1
 #endif
@@ -363,8 +469,8 @@
 #define ENABLE_INSPECTOR_TELEMETRY 1
 #endif
 
-#if !defined(ENABLE_IOS_FORM_CONTROL_REFRESH) && PLATFORM(IOS_FAMILY)
-#define ENABLE_IOS_FORM_CONTROL_REFRESH 1
+#if !defined(ENABLE_INTERACTION_REGIONS_IN_EVENT_REGION) && PLATFORM(VISION)
+#define ENABLE_INTERACTION_REGIONS_IN_EVENT_REGION 1
 #endif
 
 #if !defined(ENABLE_IOS_GESTURE_EVENTS) && PLATFORM(IOS_FAMILY) && USE(APPLE_INTERNAL_SDK)
@@ -375,6 +481,23 @@
 #define ENABLE_IOS_TOUCH_EVENTS 1
 #endif
 
+#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
+    || PLATFORM(IOS) \
+    || PLATFORM(MACCATALYST) \
+    || PLATFORM(VISION))
+#define ENABLE_JS_COOKIE_CHECKING 1
+#endif
+
+#if !defined(ENABLE_KINETIC_SCROLLING) && ENABLE(ASYNC_SCROLLING)
+#define ENABLE_KINETIC_SCROLLING 1
+#endif
+
+#if !defined(ENABLE_LAUNCHD_BLOCKING_IN_WEBCONTENT) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || PLATFORM(IOS_FAMILY))
+#define ENABLE_LAUNCHD_BLOCKING_IN_WEBCONTENT 1
+#endif
+
 #if !defined(ENABLE_LAYER_BASED_SVG_ENGINE)
 #define ENABLE_LAYER_BASED_SVG_ENGINE 1
 #endif
@@ -387,10 +510,18 @@
 #define ENABLE_LEGACY_ENCRYPTED_MEDIA 1
 #endif
 
+#if !defined(ENABLE_LOGD_BLOCKING_IN_WEBCONTENT) && PLATFORM(IOS_FAMILY) && USE(APPLE_INTERNAL_SDK)
+#define ENABLE_LOGD_BLOCKING_IN_WEBCONTENT 1
+#endif
+
 #if !defined(ENABLE_MAC_GESTURE_EVENTS) && PLATFORM(MAC) && USE(APPLE_INTERNAL_SDK)
 #define ENABLE_MAC_GESTURE_EVENTS 1
 #endif
 
+#if !defined(ENABLE_MANAGED_DOMAINS)
+#define ENABLE_MANAGED_DOMAINS 1
+#endif
+
 #if !defined(ENABLE_MEDIA_CAPTURE) && !PLATFORM(MAC) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 #define ENABLE_MEDIA_CAPTURE 1
 #endif
@@ -411,22 +542,27 @@
 #define ENABLE_MEDIA_SESSION_PLAYLIST 1
 #endif
 
-#if !defined(ENABLE_MEDIA_SESSION) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
+#if !defined(ENABLE_MEDIA_SESSION) && !PLATFORM(WATCHOS)
 #define ENABLE_MEDIA_SESSION 1
 #endif
 
-#if !defined(ENABLE_DOM_AUDIO_SESSION) && PLATFORM(COCOA) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
+#if !defined(ENABLE_DOM_AUDIO_SESSION) && PLATFORM(COCOA) && !PLATFORM(WATCHOS)
 #define ENABLE_DOM_AUDIO_SESSION 1
 #endif
 
+#if !defined(ENABLE_LOCKDOWN_MODE_API) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000))
+#define ENABLE_LOCKDOWN_MODE_API 1
+#endif
+
+#if !defined(ENABLE_LOCKDOWN_MODE_TELEMETRY) && PLATFORM(MAC)
+#define ENABLE_LOCKDOWN_MODE_TELEMETRY 1
+#endif
+
 #if !defined(ENABLE_MEDIA_SOURCE) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 #define ENABLE_MEDIA_SOURCE 1
 #endif
 
-#if !defined(MANAGED_MEDIA_SOURCE) && ENABLE(MEDIA_SOURCE)
-#define ENABLE_MANAGED_MEDIA_SOURCE 1
-#endif
-
 #if !defined(ENABLE_MEDIA_STREAM) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 #define ENABLE_MEDIA_STREAM 1
 #endif
@@ -447,6 +583,10 @@
 #define ENABLE_META_VIEWPORT 1
 #endif
 
+#if !defined(ENABLE_MOBILE_GESTALT_DEVICE_NAME) && PLATFORM(IOS)
+#define ENABLE_MOBILE_GESTALT_DEVICE_NAME 1
+#endif
+
 #if !defined(ENABLE_MODEL_ELEMENT)
 #define ENABLE_MODEL_ELEMENT 1
 #endif
@@ -455,6 +595,10 @@
 #define ENABLE_MODERN_MEDIA_CONTROLS 1
 #endif
 
+#if PLATFORM(MAC)
+#define ENABLE_MOMENTUM_EVENT_DISPATCHER 1
+#endif
+
 // FIXME: We almost certainly do not need this monospace font exception for watchOS and tvOS.
 #if !defined(ENABLE_MONOSPACE_FONT_EXCEPTION) && (PLATFORM(WATCHOS) || PLATFORM(APPLETV))
 #define ENABLE_MONOSPACE_FONT_EXCEPTION 1
@@ -476,18 +620,54 @@
 #define ENABLE_NETWORK_CACHE_STALE_WHILE_REVALIDATE 1
 #endif
 
+#if !defined(ENABLE_NETWORK_ISSUE_REPORTING) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000))
+#define ENABLE_NETWORK_ISSUE_REPORTING 1
+#endif
+
 #if !defined(ENABLE_NON_VISIBLE_WEBPROCESS_MEMORY_CLEANUP_TIMER) && (PLATFORM(MAC) || PLATFORM(MACCATALYST))
 #define ENABLE_NON_VISIBLE_WEBPROCESS_MEMORY_CLEANUP_TIMER 1
 #endif
 
+#if !defined(ENABLE_NOTIFICATION_EVENT) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
+    || (PLATFORM(IOS) || PLATFORM(VISION)))
+#define ENABLE_NOTIFICATION_EVENT 1
+#endif
+
 #if !defined(ENABLE_NOTIFICATIONS)
 #define ENABLE_NOTIFICATIONS 1
 #endif
 
+#if !defined(ENABLE_NOTIFY_FILTERING) && PLATFORM(MAC)
+#define ENABLE_NOTIFY_FILTERING 1
+#endif
+
 #if !defined(ENABLE_OFFSCREEN_CANVAS) && (PLATFORM(MAC) || PLATFORM(IOS_FAMILY))
 #define ENABLE_OFFSCREEN_CANVAS 1
 #endif
 
+// Used in internal build
+#if !defined(ENABLE_ONGOING_CREDENTIAL_SHARING_WEBKIT_SPI) \
+    && ((PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION) \
+    || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000))
+#define ENABLE_ONGOING_CREDENTIAL_SHARING_WEBKIT_SPI 1
+#endif
+
+#if !defined(ENABLE_OPENTYPE_MATH) && (OS(DARWIN) && USE(CG))
+#define ENABLE_OPENTYPE_MATH 1
+#endif
+
+// FIXME: Is it intentional that tvOS and macCatalyst are omitted?
+#if !defined(ENABLE_OPUS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
+#define ENABLE_OPUS 1
+#endif
+
 // ORIENTATION_EVENTS should never get enabled on Desktop, only Mobile.
 #if !defined(ENABLE_ORIENTATION_EVENTS) && PLATFORM(IOS_FAMILY)
 #define ENABLE_ORIENTATION_EVENTS 1
@@ -497,6 +677,10 @@
 #define ENABLE_OVERFLOW_SCROLLING_TOUCH 1
 #endif
 
+#if !defined(ENABLE_OVERLAY_REGIONS_IN_EVENT_REGION) && PLATFORM(VISION)
+#define ENABLE_OVERLAY_REGIONS_IN_EVENT_REGION 0
+#endif
+
 #if !defined(ENABLE_PAYMENT_REQUEST)
 #define ENABLE_PAYMENT_REQUEST 1
 #endif
@@ -505,18 +689,31 @@
 #define ENABLE_PAYMENT_REQUEST_SELECTED_SHIPPING_OPTION 1
 #endif
 
-#if !defined(ENABLE_PICTURE_IN_PICTURE_API) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
-#define ENABLE_PICTURE_IN_PICTURE_API 1
+// FIXME: Reenable PDFJS by default for Cocoa (https://bugs.webkit.org/show_bug.cgi?id=242263).
+#if !defined(ENABLE_PDFJS)
+#define ENABLE_PDFJS 0
 #endif
 
-#if !defined(ENABLE_PDFKIT_PLUGIN) && PLATFORM(MAC)
-#define ENABLE_PDFKIT_PLUGIN 1
+#if !defined(ENABLE_LEGACY_PDFKIT_PLUGIN) && PLATFORM(MAC)
+#define ENABLE_LEGACY_PDFKIT_PLUGIN 1
+#endif
+
+#if !defined(ENABLE_PDF_PLUGIN) && (ENABLE(LEGACY_PDFKIT_PLUGIN) || ENABLE(UNIFIED_PDF))
+#define ENABLE_PDF_PLUGIN 1
+#endif
+
+#if !defined(ENABLE_PDF_HUD) && ENABLE(PDF_PLUGIN) && PLATFORM(MAC)
+#define ENABLE_PDF_HUD 1
 #endif
 
 #if !defined(ENABLE_PERIODIC_MEMORY_MONITOR) && PLATFORM(MAC)
 #define ENABLE_PERIODIC_MEMORY_MONITOR 1
 #endif
 
+#if !defined(ENABLE_PICTURE_IN_PICTURE_API) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
+#define ENABLE_PICTURE_IN_PICTURE_API 1
+#endif
+
 #if !defined(ENABLE_PLATFORM_DRIVEN_TEXT_CHECKING) && PLATFORM(MACCATALYST)
 #define ENABLE_PLATFORM_DRIVEN_TEXT_CHECKING 1
 #endif
@@ -529,17 +726,20 @@
 #define ENABLE_PENCIL_HOVER 1
 #endif
 
-
 #if !defined(ENABLE_POST_EDITING_GRAMMAR_CHECKING) && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) || PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_POST_EDITING_GRAMMAR_CHECKING 1
 #endif
 
+#if !defined(ENABLE_PREDEFINED_COLOR_SPACE_DISPLAY_P3) && HAVE(CORE_ANIMATION_FIX_FOR_RADAR_78402035)
+#define ENABLE_PREDEFINED_COLOR_SPACE_DISPLAY_P3 1
+#endif
+
 #if !defined(ENABLE_PREVIEW_CONVERTER) && (PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_PREVIEW_CONVERTER 1
 #endif
 
-#if !defined(ENABLE_PRIMARY_SNAPSHOTTED_PLUGIN_HEURISTIC)
-#define ENABLE_PRIMARY_SNAPSHOTTED_PLUGIN_HEURISTIC 1
+#if !defined(ENABLE_PREWARM_WEBPROCESS_ON_PROVISIONAL_LOAD)
+#define ENABLE_PREWARM_WEBPROCESS_ON_PROVISIONAL_LOAD 1
 #endif
 
 #if !defined(ENABLE_PUBLIC_SUFFIX_LIST)
@@ -550,19 +750,8 @@
 #define ENABLE_REMOTE_INSPECTOR 1
 #endif
 
-#if !defined(ENABLE_TRACKING_PREVENTION)
-#define ENABLE_TRACKING_PREVENTION 1
-#endif
-
-#if ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || PLATFORM(IOS) \
-    || PLATFORM(MACCATALYST) \
-    || PLATFORM(VISION))
-#define ENABLE_JS_COOKIE_CHECKING 1
-#endif
-
-#if !defined(ENABLE_SPEECH_SYNTHESIS) && !PLATFORM(MACCATALYST)
-#define ENABLE_SPEECH_SYNTHESIS 1
+#if !defined(ENABLE_REMOTE_LAYER_TREE_ON_MAC_BY_DEFAULT) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000
+#define ENABLE_REMOTE_LAYER_TREE_ON_MAC_BY_DEFAULT 1
 #endif
 
 #if !defined(ENABLE_RESOURCE_USAGE)
@@ -577,10 +766,6 @@
 #define ENABLE_ROUTING_ARBITRATION 1
 #endif
 
-#if !defined(ENABLE_RUBBER_BANDING) && PLATFORM(MAC)
-#define ENABLE_RUBBER_BANDING 1
-#endif
-
 #if !defined(ENABLE_SANDBOX_EXTENSIONS)
 #define ENABLE_SANDBOX_EXTENSIONS 1
 #endif
@@ -593,10 +778,6 @@
 #define ENABLE_SEC_ITEM_SHIM 1
 #endif
 
-#if !defined(ENABLE_SEPARATED_WX_HEAP) && CPU(ARM64)
-#define ENABLE_SEPARATED_WX_HEAP 1
-#endif
-
 #if !defined(ENABLE_SERVER_PRECONNECT)
 #define ENABLE_SERVER_PRECONNECT 1
 #endif
@@ -605,10 +786,31 @@
 #define ENABLE_SERVICE_CONTROLS 1
 #endif
 
+#if !defined(ENABLE_SEPARATED_WX_HEAP) && CPU(ARM64)
+#define ENABLE_SEPARATED_WX_HEAP 1
+#endif
+
+#if !defined(ENABLE_SET_WEBCONTENT_PROCESS_INFORMATION_IN_NETWORK_PROCESS) && PLATFORM(MAC)
+#define ENABLE_SET_WEBCONTENT_PROCESS_INFORMATION_IN_NETWORK_PROCESS 1
+#endif
+
 #if !defined(ENABLE_SHAREABLE_RESOURCE)
 #define ENABLE_SHAREABLE_RESOURCE 1
 #endif
 
+#if !defined(ENABLE_SPEECH_SYNTHESIS) && !PLATFORM(MACCATALYST)
+#define ENABLE_SPEECH_SYNTHESIS 1
+#endif
+
+#if !defined(ENABLE_STATIC_IPAD_USER_AGENT_VALUE) && PLATFORM(VISION)
+#define ENABLE_STATIC_IPAD_USER_AGENT_VALUE 1
+#endif
+
+// Used in internal build
+#if PLATFORM(IOS) || PLATFORM(MAC) || PLATFORM(VISION)
+#define ENABLE_SYNCED_CREDENTIALS 1
+#endif
+
 // FIXME: Should this be enabled for watchOS and tvOS?
 #if !defined(ENABLE_TAKE_UNBOUNDED_NETWORKING_ASSERTION) && !PLATFORM(MAC) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
 #define ENABLE_TAKE_UNBOUNDED_NETWORKING_ASSERTION 1
@@ -630,6 +832,14 @@
 #define ENABLE_TEXT_SELECTION 0
 #endif
 
+#if !defined(ENABLE_THREADED_ANIMATION_RESOLUTION)
+#define ENABLE_THREADED_ANIMATION_RESOLUTION 1
+#endif
+
+#if !defined(ENABLE_TIMER_DRIVEN_DISPLAY_REFRESH_FOR_TESTING) && PLATFORM(VISION)
+#define ENABLE_TIMER_DRIVEN_DISPLAY_REFRESH_FOR_TESTING 1
+#endif
+
 #if !defined(ENABLE_TOUCH_EVENTS) && PLATFORM(IOS_FAMILY) && USE(APPLE_INTERNAL_SDK)
 #define ENABLE_TOUCH_EVENTS 1
 #endif
@@ -638,15 +848,15 @@
 #define ENABLE_TOUCH_ACTION_REGIONS 1
 #endif
 
+#if !defined(ENABLE_TRACKER_DISPOSITION)
+#define ENABLE_TRACKER_DISPOSITION 1
+#endif
+
 // FIXME (rdar://49961589): Enable on macCatalyst.
 #if !defined(ENABLE_TRANSCODE_UIIMAGEPICKERCONTROLLER_VIDEO) && (PLATFORM(IOS) || PLATFORM(VISION))
 #define ENABLE_TRANSCODE_UIIMAGEPICKERCONTROLLER_VIDEO 1
 #endif
 
-#if !defined(ENABLE_WHEEL_EVENT_REGIONS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
-#define ENABLE_WHEEL_EVENT_REGIONS 1
-#endif
-
 #if !defined(ENABLE_UI_SIDE_COMPOSITING)
 #define ENABLE_UI_SIDE_COMPOSITING 1
 #endif
@@ -663,7 +873,9 @@
 #define ENABLE_VIDEO 1
 #endif
 
-#if !defined(ENABLE_VIDEO_PRESENTATION_MODE) && !PLATFORM(IOS_FAMILY_SIMULATOR) && !PLATFORM(APPLETV)
+#if !defined(ENABLE_VIDEO_PRESENTATION_MODE) \
+    && (PLATFORM(IOS_FAMILY) && HAVE(AVKIT)) \
+    || PLATFORM(MAC)
 #define ENABLE_VIDEO_PRESENTATION_MODE 1
 #endif
 
@@ -671,14 +883,50 @@
 #define ENABLE_VIDEO_USES_ELEMENT_FULLSCREEN 1
 #endif
 
-#if PLATFORM(VISION)
-#define HAVE_PIP_CONTROLLER 1
+#if !defined(ENABLE_VORBIS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
+#define ENABLE_VORBIS 1
 #endif
 
 #if !defined(ENABLE_VP9) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
 #define ENABLE_VP9 1
 #endif
 
+#if !defined(ENABLE_WEB_API_STATISTICS)
+#define ENABLE_WEB_API_STATISTICS 1
+#endif
+
+#if !defined(ENABLE_WEB_ARCHIVE)
+#define ENABLE_WEB_ARCHIVE 1
+#endif
+
+#if !defined(ENABLE_WEB_AUDIO)
+#define ENABLE_WEB_AUDIO 1
+#endif
+
+#if !defined(ENABLE_WEB_AUTHN) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
+#define ENABLE_WEB_AUTHN 1
+#endif
+
+#if !defined(ENABLE_WEB_CODECS) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
+#define ENABLE_WEB_CODECS 1
+#endif
+
+#if !defined(ENABLE_WEB_CRYPTO)
+#define ENABLE_WEB_CRYPTO 1
+#endif
+
+#if !defined(ENABLE_WEB_PLAYBACK_CONTROLS_MANAGER) && PLATFORM(MAC)
+#define ENABLE_WEB_PLAYBACK_CONTROLS_MANAGER 1
+#endif
+
+#if !defined(ENABLE_WEB_RTC) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
+#define ENABLE_WEB_RTC 1
+#endif
+
+#if !defined(ENABLE_WEBCONTENT_GPU_SANDBOX_EXTENSIONS_BLOCKING) && (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000)
+#define ENABLE_WEBCONTENT_GPU_SANDBOX_EXTENSIONS_BLOCKING 1
+#endif
+
 #if !defined(ENABLE_WEBDRIVER_KEYBOARD_GRAPHEME_CLUSTERS)
 #define ENABLE_WEBDRIVER_KEYBOARD_GRAPHEME_CLUSTERS 1
 #endif
@@ -703,44 +951,42 @@
 #define ENABLE_WEBGL 1
 #endif
 
+#if !defined(ENABLE_WEBGL_IN_WORKERS) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION))
+#define ENABLE_WEBGL_IN_WORKERS 1
+#endif
+
+#if !defined(ENABLE_WEBM_FORMAT_READER) && PLATFORM(MAC)
+#define ENABLE_WEBM_FORMAT_READER 1
+#endif
+
+#if !defined(ENABLE_WEBPROCESS_CACHE) && (PLATFORM(MAC) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 170400))
+#define ENABLE_WEBPROCESS_CACHE 1
+#endif
+
 #if !defined(ENABLE_WEBPROCESS_NSRUNLOOP) && PLATFORM(MAC)
 #define ENABLE_WEBPROCESS_NSRUNLOOP 1
 #endif
 
-#if !defined(ENABLE_WEB_API_STATISTICS)
-#define ENABLE_WEB_API_STATISTICS 1
+#if !defined(ENABLE_WEBXR) && PLATFORM(VISION)
+#define ENABLE_WEBXR 1
 #endif
 
-#if !defined(ENABLE_WEB_ARCHIVE)
-#define ENABLE_WEB_ARCHIVE 1
+#if !defined(ENABLE_WEBXR_HANDS) && PLATFORM(VISION)
+#define ENABLE_WEBXR_HANDS 1
 #endif
 
-#if !defined(ENABLE_WEB_AUDIO)
-#define ENABLE_WEB_AUDIO 1
+#if !defined(ENABLE_WHEEL_EVENT_LATCHING) && PLATFORM(MAC)
+// FIXME: Maybe this can be combined with ENABLE_KINETIC_SCROLLING.
+#define ENABLE_WHEEL_EVENT_LATCHING 1
 #endif
 
-#if !defined(ENABLE_WEB_AUTHN) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
-#define ENABLE_WEB_AUTHN 1
-#endif
-
-#if !defined(ENABLE_WEB_RTC) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
-#define ENABLE_WEB_RTC 1
-#endif
-
-#if !defined(ENABLE_WEB_CODECS) && !PLATFORM(MACCATALYST) && !PLATFORM(WATCHOS) && !PLATFORM(APPLETV)
-#define ENABLE_WEB_CODECS 1
-#endif
-
-#if !defined(ENABLE_WK_WEB_EXTENSIONS) && (PLATFORM(MAC) || PLATFORM(MACCATALYST) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
-#define ENABLE_WK_WEB_EXTENSIONS 1
-#endif
-
-#if !defined(ENABLE_WEB_CRYPTO)
-#define ENABLE_WEB_CRYPTO 1
-#endif
-
-#if !defined(ENABLE_WEB_PLAYBACK_CONTROLS_MANAGER) && PLATFORM(MAC)
-#define ENABLE_WEB_PLAYBACK_CONTROLS_MANAGER 1
+#if !defined(ENABLE_WHEEL_EVENT_REGIONS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
+#define ENABLE_WHEEL_EVENT_REGIONS 1
 #endif
 
 #if !defined(ENABLE_WIRELESS_PLAYBACK_TARGET)
@@ -751,155 +997,22 @@
 #define ENABLE_WIRELESS_PLAYBACK_TARGET_AVAILABILITY_API 1
 #endif
 
+#if !defined(ENABLE_WK_WEB_EXTENSIONS) && (PLATFORM(MAC) || PLATFORM(MACCATALYST) || PLATFORM(IOS) || PLATFORM(IOS_SIMULATOR) || PLATFORM(VISION))
+#define ENABLE_WK_WEB_EXTENSIONS 1
+#endif
+
 #if !defined(ENABLE_WKPDFVIEW) && (PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION))
 #define ENABLE_WKPDFVIEW 1
 #endif
 
-#if !defined(ENABLE_VORBIS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
-#define ENABLE_VORBIS 1
-#endif
-
-#if !defined(ENABLE_SET_WEBCONTENT_PROCESS_INFORMATION_IN_NETWORK_PROCESS) && PLATFORM(MAC)
-#define ENABLE_SET_WEBCONTENT_PROCESS_INFORMATION_IN_NETWORK_PROCESS 1
-#endif
-
-#if !defined(ENABLE_WEBM_FORMAT_READER) && PLATFORM(MAC)
-#define ENABLE_WEBM_FORMAT_READER 1
-#endif
-
-// FIXME: Is it intentional that tvOS and macCatalyst are omitted?
-#if !defined(ENABLE_OPUS) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
-#define ENABLE_OPUS 1
-#endif
-
-#if !defined(ENABLE_SEPARATED_MODEL) && PLATFORM(VISION) && HAVE(CORE_ANIMATION_SEPARATED_LAYERS)
-#define ENABLE_SEPARATED_MODEL 1
-#endif
-
-#if !defined(ENABLE_NOTIFY_FILTERING) && PLATFORM(MAC)
-#define ENABLE_NOTIFY_FILTERING 1
-#endif
-
-// FIXME: Should this be disabled on tvOS and watchOS?
-#if !defined(ENABLE_APP_PRIVACY_REPORT) && PLATFORM(IOS_FAMILY) && !PLATFORM(APPLETV) && !PLATFORM(WATCHOS)
-#define ENABLE_APP_PRIVACY_REPORT 1
-#endif
-
-#if !defined(ENABLE_APP_HIGHLIGHTS) \
-    && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION))
-#define ENABLE_APP_HIGHLIGHTS 1
-#endif
-
-#if !defined(ENABLE_DESTINATION_COLOR_SPACE_DISPLAY_P3)
-#define ENABLE_DESTINATION_COLOR_SPACE_DISPLAY_P3 1
-#endif
-
-#if !defined(ENABLE_DESTINATION_COLOR_SPACE_LINEAR_SRGB)
-#define ENABLE_DESTINATION_COLOR_SPACE_LINEAR_SRGB 1
-#endif
-
-#if !defined(ENABLE_IMAGE_ANALYSIS) && HAVE(VK_IMAGE_ANALYSIS)
-#define ENABLE_IMAGE_ANALYSIS 1
-#endif
-
-#if !defined(ENABLE_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES) && HAVE(VK_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES)
-#define ENABLE_IMAGE_ANALYSIS_FOR_MACHINE_READABLE_CODES 1
-#endif
-
-#if !defined(ENABLE_PREDEFINED_COLOR_SPACE_DISPLAY_P3) && HAVE(CORE_ANIMATION_FIX_FOR_RADAR_78402035)
-#define ENABLE_PREDEFINED_COLOR_SPACE_DISPLAY_P3 1
-#endif
-
-#if PLATFORM(IOS_FAMILY) && HAVE(ASV_INLINE_PREVIEW) && !ENABLE(SEPARATED_MODEL)
-#define ENABLE_ARKIT_INLINE_PREVIEW_IOS 1
-#endif
-
-#if PLATFORM(MAC) && HAVE(ASV_INLINE_PREVIEW)
-#define ENABLE_ARKIT_INLINE_PREVIEW_MAC 1
-#endif
-
-#if ENABLE(ARKIT_INLINE_PREVIEW_IOS) || ENABLE(ARKIT_INLINE_PREVIEW_MAC)
-#define ENABLE_ARKIT_INLINE_PREVIEW 1
-#endif
-
-#if ENABLE(ARKIT_INLINE_PREVIEW_MAC) || ENABLE(ARKIT_INLINE_PREVIEW_IOS)
-#define ENABLE_ARKIT_INLINE_PREVIEW_CAMERA_TRANSFORM 1
-#define ENABLE_ARKIT_INLINE_PREVIEW_ANIMATIONS_CONTROL 1
-#define ENABLE_ARKIT_INLINE_PREVIEW_AUDIO_CONTROL 1
-#endif
-
-#if !defined(ENABLE_ADDITIONAL_PRECONNECT_ON_HTTP_1X) && PLATFORM(MAC)
-#define ENABLE_ADDITIONAL_PRECONNECT_ON_HTTP_1X 1
-#endif
-
-#if !defined(ENABLE_FULLSCREEN_DISMISSAL_GESTURES) && PLATFORM(IOS_FAMILY)
-#define ENABLE_FULLSCREEN_DISMISSAL_GESTURES 1
-#endif
-
-#if (!defined(ENABLE_LOCKDOWN_MODE_API) && (PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION)))
-#define ENABLE_LOCKDOWN_MODE_API 1
-#endif
-
-#if !defined(ENABLE_WEBCONTENT_CRASH_TESTING) && USE(APPLE_INTERNAL_SDK) \
-    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-        || PLATFORM(IOS) \
-        || PLATFORM(VISION)) \
-    && HAVE(CHIRP_SPI)
-#define ENABLE_WEBCONTENT_CRASH_TESTING 1
-#endif
-
-#if !defined(ENABLE_WEBCONTENT_GPU_SANDBOX_EXTENSIONS_BLOCKING) && ((PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) || PLATFORM(VISION))
-#define ENABLE_WEBCONTENT_GPU_SANDBOX_EXTENSIONS_BLOCKING 1
-#endif
-
-#if !defined(ENABLE_MOBILE_GESTALT_DEVICE_NAME) && PLATFORM(IOS)
-#define ENABLE_MOBILE_GESTALT_DEVICE_NAME 1
-#endif
-
-#if !defined(ENABLE_LOCKDOWN_MODE_TELEMETRY) && PLATFORM(MAC)
-#define ENABLE_LOCKDOWN_MODE_TELEMETRY 1
-#endif
-
-#if !defined(ENABLE_THREADED_ANIMATION_RESOLUTION)
-#define ENABLE_THREADED_ANIMATION_RESOLUTION 1
-#endif
-
-#if !defined(ENABLE_WEBGL_IN_WORKERS) \
-    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+#if !defined(ENABLE_AV1) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED >= 130300) \
     || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
     || PLATFORM(VISION))
-#define ENABLE_WEBGL_IN_WORKERS 1
+#define ENABLE_AV1 1
 #endif
 
-#if !defined(ENABLE_CARENDERSERVER_BLOCKING_IN_WEBCONTENT) \
-    && (((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) && !PLATFORM(VISION) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
-    || PLATFORM(VISION))
-#define ENABLE_CARENDERSERVER_BLOCKING_IN_WEBCONTENT 1
+#if !defined(ENABLE_WINDOW_PROXY_PROPERTY_ACCESS_NOTIFICATION)
+#define ENABLE_WINDOW_PROXY_PROPERTY_ACCESS_NOTIFICATION 1
 #endif
 
-#if !defined(ENABLE_LAUNCHD_BLOCKING_IN_WEBCONTENT) \
-    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
-    || PLATFORM(IOS_FAMILY))
-#define ENABLE_LAUNCHD_BLOCKING_IN_WEBCONTENT 1
-#endif
-
-#if !defined(ENABLE_LOGD_BLOCKING_IN_WEBCONTENT) && PLATFORM(IOS) && !PLATFORM(VISION) && USE(APPLE_INTERNAL_SDK)
-#define ENABLE_LOGD_BLOCKING_IN_WEBCONTENT 1
-#endif
-
-#if !defined(ENABLE_SAMPLE_BUFFER_CONTENT_KEY_SESSION_SUPPORT) \
-    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
-    || PLATFORM(VISION))
-#define ENABLE_SAMPLE_BUFFER_CONTENT_KEY_SESSION_SUPPORT 1
-#endif
-
-#if !defined(ENABLE_PREWARM_WEBPROCESS_ON_PROVISIONAL_LOAD) && PLATFORM(IOS)
-#define ENABLE_PREWARM_WEBPROCESS_ON_PROVISIONAL_LOAD 1
-#endif
