@@ -30,6 +30,11 @@
 #define HAVE_KDEBUG_H 1
 #endif
 
+#if OS(DARWIN)
+#include <cinttypes>
+#include <sys/types.h>
+#endif
+
 // No namespaces because this file has to be includable from C and Objective-C.
 
 // Reserved kdebug codes. Do not change these.
@@ -195,72 +200,109 @@ using WTF::tracePoint;
 
 #endif // __cplusplus
 
-#if HAVE(OS_SIGNPOST) && HAVE(KDEBUG_H)
+#if HAVE(OS_SIGNPOST)
 
-#import <os/signpost.h>
+#include <os/signpost.h>
+#include <wtf/Seconds.h>
 
 WTF_EXTERN_C_BEGIN
+
+WTF_EXPORT_PRIVATE extern bool WTFSignpostIndirectLoggingEnabled;
+
 WTF_EXPORT_PRIVATE os_log_t WTFSignpostLogHandle();
+WTF_EXPORT_PRIVATE bool WTFSignpostHandleIndirectLog(os_log_t, pid_t, const char* line);
+
+WTF_EXPORT_PRIVATE uint64_t WTFCurrentContinuousTime(Seconds deltaFromNow);
+
 WTF_EXTERN_C_END
 
-#define WTF_OS_SIGNPOST_ANIMATION_INTERVAL_TAG "isAnimation=YES"
+#define FOR_EACH_WTF_SIGNPOST_NAME(M) \
+    M(DataTask) \
+    M(NavigationAndPaintTiming) \
+    M(ExecuteScriptElement) \
+    M(RegisterImportMap) \
 
-// The first argument to these signpost APIs is a pointer that can be used to disambiguite nested
-// intervals with the same name (i.e. used to create an os_signpost_id). If you don't care about
-// handling nested intervals, then pass `nullptr` as the pointer argument.
+#define DECLARE_WTF_SIGNPOST_NAME_ENUM(name) WTFOSSignpostName ## name,
 
-// These macros emit signposts into logd's buffer only when tracing is enabled. You should generally
-// prefer using these macros instead of the SignpostAlways variants to avoid unnecessary log bloat.
-#define WTFEmitSignpost(pointer, name, ...) \
-    WTFEmitSignpostWithFunction(os_signpost_event_emit, (pointer), name, ##__VA_ARGS__)
+enum WTFOSSignpostName {
+    FOR_EACH_WTF_SIGNPOST_NAME(DECLARE_WTF_SIGNPOST_NAME_ENUM)
+    WTFOSSignpostNameCount,
+};
 
-#define WTFBeginSignpost(pointer, name, ...) \
-    WTFEmitSignpostWithFunction(os_signpost_interval_begin, (pointer), name, ##__VA_ARGS__)
+enum WTFOSSignpostType {
+    WTFOSSignpostTypeEmitEvent,
+    WTFOSSignpostTypeBeginInterval,
+    WTFOSSignpostTypeEndInterval,
+    WTFOSSignpostTypeCount,
+};
 
-#define WTFBeginAnimationSignpost(pointer, name, format, ...) \
-    WTFEmitSignpostWithFunction(os_signpost_interval_begin, (pointer), name, format " " WTF_OS_SIGNPOST_ANIMATION_INTERVAL_TAG, ##__VA_ARGS__)
+#endif
 
-#define WTFEndSignpost(pointer, name, ...) \
-    WTFEmitSignpostWithFunction(os_signpost_interval_end, (pointer), name, ##__VA_ARGS__)
+#if HAVE(KDEBUG_H)
 
-#define WTFEmitSignpostWithFunction(emitFunc, pointer, name, ...) \
-do { \
-    if (UNLIKELY(kdebug_is_enabled(KDBG_EVENTID(DBG_APPS, DBG_APPS_WEBKIT_MISC, 0)))) { \
-        WTFEmitSignpostAlwaysWithFunction(emitFunc, pointer, name, ##__VA_ARGS__); \
-    } \
-} while (0)
+// The first argument to WTF{Emit,Begin,End}Signpost is a pointer that can be used to disambiguate
+// nested intervals with the same name (i.e. used to create an os_signpost_id). If you don't care
+// about handling nested intervals, then pass `nullptr` as the pointer argument.
+//
+// The second argument to these signpost APIs is a signpost name that must be listed in the
+// FOR_EACH_WTF_SIGNPOST_NAME macro above.
+//
+// The third (or more) arguments are an optional os_log-compatible format string followed by
+// arguments for the format string.
+//
+// These macros will emit signposts into logd's buffer only when the DBG_APPS kdebug class is
+// enabled. The DBG_APPS kdebug class is enabled automatically by certain internal profiling tools,
+// but is not enabled by tailspin. This means we generally emit signposts only if a developer is
+// actively profiling their system.
+#define WTFEmitSignpost(pointer, name, ...) WTFEmitSignpostWithTimeDelta((pointer), name, Seconds(), ##__VA_ARGS__)
+#define WTFBeginSignpost(pointer, name, ...) WTFBeginSignpostWithTimeDelta((pointer), name, Seconds(), ##__VA_ARGS__)
+#define WTFEndSignpost(pointer, name, ...) WTFEndSignpostWithTimeDelta((pointer), name, Seconds(), ##__VA_ARGS__)
 
-// These macros always emit signposts into logd's buffer.
-#define WTFEmitSignpostAlways(pointer, name, ...) \
-    WTFEmitSignpostAlwaysWithFunction(os_signpost_event_emit, (pointer), name, ##__VA_ARGS__)
+// These work like WTF{Emit,Begin,End}Signpost, but offset the timestamp associated with the event
+// by timeDelta (which should of type WTF::Seconds).
+#define WTFEmitSignpostWithTimeDelta(pointer, name, timeDelta, ...) \
+    WTFEmitSignpostWithType(WTFOSSignpostTypeEmitEvent, os_signpost_event_emit, (pointer), name, (timeDelta), " %{signpost.description:event_time}llu", "" __VA_ARGS__)
 
-#define WTFBeginSignpostAlways(pointer, name, ...) \
-    WTFEmitSignpostAlwaysWithFunction(os_signpost_interval_begin, (pointer), name, ##__VA_ARGS__)
+#define WTFBeginSignpostWithTimeDelta(pointer, name, timeDelta, ...) \
+    WTFEmitSignpostWithType(WTFOSSignpostTypeBeginInterval, os_signpost_interval_begin, (pointer), name, (timeDelta), " %{signpost.description:begin_time}llu", "" __VA_ARGS__)
 
-#define WTFBeginAnimationSignpostAlways(pointer, name, format, ...) \
-    WTFEmitSignpostAlwaysWithFunction(os_signpost_interval_begin, (pointer), name, format " " WTF_OS_SIGNPOST_ANIMATION_INTERVAL_TAG, ##__VA_ARGS__)
+#define WTFEndSignpostWithTimeDelta(pointer, name, timeDelta, ...) \
+    WTFEmitSignpostWithType(WTFOSSignpostTypeEndInterval, os_signpost_interval_end, (pointer), name, (timeDelta), " %{signpost.description:end_time}llu", "" __VA_ARGS__)
 
-#define WTFEndSignpostAlways(pointer, name, ...) \
-    WTFEmitSignpostAlwaysWithFunction(os_signpost_interval_end, (pointer), name, ##__VA_ARGS__)
+#define WTFEmitSignpostWithType(type, emitMacro, pointer, name, timeDelta, timeFormat, format, ...) \
+    do { \
+        if (UNLIKELY(kdebug_is_enabled(KDBG_EVENTID(DBG_APPS, DBG_APPS_WEBKIT_MISC, 0)))) { \
+            if (WTFSignpostIndirectLoggingEnabled) \
+                WTFEmitSignpostIndirectlyWithType(type, pointer, name, timeDelta, format, ##__VA_ARGS__); \
+            else { \
+                Seconds delta = (timeDelta); \
+                if (delta) \
+                    WTFEmitSignpostDirectlyWithType(emitMacro, pointer, name, format timeFormat, ##__VA_ARGS__, WTFCurrentContinuousTime(delta)); \
+                else \
+                    WTFEmitSignpostDirectlyWithType(emitMacro, pointer, name, format, ##__VA_ARGS__); \
+            } \
+        } \
+    } while (0)
 
-#define WTFEmitSignpostAlwaysWithFunction(emitFunc, pointer, name, ...) \
-do { \
-    os_log_t wtfHandle = WTFSignpostLogHandle(); \
-    const void *wtfPointer = (pointer); \
-    os_signpost_id_t wtfSignpostID = wtfPointer ? os_signpost_id_make_with_pointer(wtfHandle, wtfPointer) : OS_SIGNPOST_ID_EXCLUSIVE; \
-    emitFunc(wtfHandle, wtfSignpostID, name, ##__VA_ARGS__); \
-} while (0)
+#define WTFEmitSignpostDirectlyWithType(emitMacro, pointer, name, format, ...) \
+    do { \
+        os_log_t wtfHandle = WTFSignpostLogHandle(); \
+        const void *wtfPointer = (pointer); \
+        os_signpost_id_t wtfSignpostID = wtfPointer ? os_signpost_id_make_with_pointer(wtfHandle, wtfPointer) : OS_SIGNPOST_ID_EXCLUSIVE; \
+        emitMacro(wtfHandle, wtfSignpostID, #name, format, ##__VA_ARGS__); \
+    } while (0)
+
+#define WTFEmitSignpostIndirectlyWithType(type, pointer, name, timeDelta, format, ...) \
+    os_log(WTFSignpostLogHandle(), "type=%d name=%d p=%" PRIuPTR " ts=%llu " format, type, WTFOSSignpostName ## name, reinterpret_cast<uintptr_t>(pointer), WTFCurrentContinuousTime(timeDelta), ##__VA_ARGS__)
 
 #else
 
 #define WTFEmitSignpost(pointer, name, ...) do { } while (0)
 #define WTFBeginSignpost(pointer, name, ...) do { } while (0)
-#define WTFBeginAnimationSignpost(pointer, name, format, ...) do { } while (0)
 #define WTFEndSignpost(pointer, name, ...) do { } while (0)
 
-#define WTFEmitSignpostAlways(pointer, name, ...) do { } while (0)
-#define WTFBeginSignpostAlways(pointer, name, ...) do { } while (0)
-#define WTFBeginAnimationSignpostAlways(pointer, name, format, ...) do { } while (0)
-#define WTFEndSignpostAlways(pointer, name, ...) do { } while (0)
+#define WTFEmitSignpostWithTimeDelta(pointer, name, ...) do { } while (0)
+#define WTFBeginSignpostWithTimeDelta(pointer, name, ...) do { } while (0)
+#define WTFEndSignpostWithTimeDelta(pointer, name, ...) do { } while (0)
 
 #endif
