@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2005-2022 Apple Inc. All rights reserved.
+ *  Copyright (C) 2005-2023 Apple Inc. All rights reserved.
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Library General Public
@@ -39,8 +39,8 @@
 #include <wtf/ValueCheck.h>
 #include <wtf/VectorTraits.h>
 
-#if ASAN_ENABLED
-extern "C" void __sanitizer_annotate_contiguous_container(const void* begin, const void* end, const void* old_mid, const void* new_mid);
+#if ASAN_ENABLED && __has_include(<sanitizer/asan_interface.h>)
+#include <sanitizer/asan_interface.h>
 #endif
 
 namespace JSC {
@@ -298,6 +298,8 @@ struct VectorTypeOperations
 template<typename T>
 constexpr inline bool isValidCapacityForVector(size_t capacity) { return capacity <= std::numeric_limits<unsigned>::max() / sizeof(T); }
 
+template<typename Collection> struct CopyOrMoveToVectorResult;
+
 template<typename T, typename Malloc>
 class VectorBufferBase {
     WTF_MAKE_NONCOPYABLE(VectorBufferBase);
@@ -561,7 +563,7 @@ public:
 #if ASAN_ENABLED
     void* endOfBuffer()
     {
-        ASSERT(buffer());
+        ASSERT_WITH_SECURITY_IMPLICATION(buffer());
 
         IGNORE_GCC_WARNINGS_BEGIN("invalid-offsetof")
         static_assert((offsetof(VectorBuffer, m_inlineBuffer) + sizeof(m_inlineBuffer)) % 8 == 0, "Inline buffer end needs to be on 8 byte boundary for ASan annotations to work.");
@@ -632,8 +634,8 @@ private:
         if (left == right)
             return;
         
-        ASSERT(leftSize <= inlineCapacity);
-        ASSERT(rightSize <= inlineCapacity);
+        ASSERT_WITH_SECURITY_IMPLICATION(leftSize <= inlineCapacity);
+        ASSERT_WITH_SECURITY_IMPLICATION(rightSize <= inlineCapacity);
         
         size_t swapBound = std::min(leftSize, rightSize);
         for (unsigned i = 0; i < swapBound; ++i)
@@ -657,9 +659,9 @@ private:
 };
 
 struct UnsafeVectorOverflow {
-    static NO_RETURN_DUE_TO_ASSERT void overflowed()
+    static NO_RETURN_DUE_TO_ASSERT_WITH_SECURITY_IMPLICATION void overflowed()
     {
-        ASSERT_NOT_REACHED();
+        ASSERT_NOT_REACHED_WITH_SECURITY_IMPLICATION();
     }
 };
 
@@ -705,13 +707,34 @@ public:
             TypeOperations::uninitializedFill(begin(), end(), val);
     }
 
-    Vector(const T* data, size_t dataSize)
+    template<typename Functor, typename = typename std::enable_if_t<std::is_invocable_v<Functor, size_t>>>
+    Vector(size_t size, const Functor& valueGenerator)
+    {
+        reserveInitialCapacity(size);
+
+        asanSetInitialBufferSizeTo(size);
+
+        if constexpr (std::is_same_v<std::invoke_result_t<Functor, size_t>, std::optional<T>>) {
+            for (size_t i = 0; i < size; ++i) {
+                if (auto item = valueGenerator(i))
+                    unsafeAppendWithoutCapacityCheck(WTFMove(*item));
+                else
+                    return;
+            }
+        } else {
+            for (size_t i = 0; i < size; ++i)
+                unsafeAppendWithoutCapacityCheck(valueGenerator(i));
+        }
+    }
+
+    template<typename U = T>
+    Vector(const U* data, size_t dataSize)
         : Base(dataSize, dataSize)
     {
         asanSetInitialBufferSizeTo(dataSize);
 
         if (begin())
-            TypeOperations::uninitializedCopy(data, data + dataSize, begin());
+            VectorCopier<std::is_trivial<T>::value, U>::uninitializedCopy(data, data + dataSize, begin());
     }
 
     Vector(std::span<const T> span)
@@ -724,7 +747,7 @@ public:
         asanSetInitialBufferSizeTo(initializerList.size());
 
         for (const auto& element : initializerList)
-            uncheckedAppend(element);
+            unsafeAppendWithoutCapacityCheck(element);
     }
 
     template<typename... Items>
@@ -824,7 +847,8 @@ public:
     template<typename U> bool appendIfNotContains(const U&);
 
     void shrink(size_t size);
-    void grow(size_t size);
+    ALWAYS_INLINE void grow(size_t size) { growImpl<FailureAction::Crash>(size); }
+    ALWAYS_INLINE bool tryGrow(size_t size) { return growImpl<FailureAction::Report>(size); }
     void resize(size_t size);
     void resizeToFit(size_t size);
     ALWAYS_INLINE void reserveCapacity(size_t newCapacity) { reserveCapacity<FailureAction::Crash>(newCapacity); }
@@ -834,6 +858,9 @@ public:
     void shrinkCapacity(size_t newCapacity);
     void shrinkToFit() { shrinkCapacity(size()); }
 
+    void growCapacityBy(size_t increment) { growCapacityBy<FailureAction::Crash>(increment); }
+    bool tryGrowCapacityBy(size_t increment) { return growCapacityBy<FailureAction::Report>(increment); }
+
     void clear() { shrinkCapacity(0); }
 
     ALWAYS_INLINE void append(ValueType&& value) { append<ValueType>(std::forward<ValueType>(value)); }
@@ -842,17 +869,16 @@ public:
     template<typename... Args> ALWAYS_INLINE void constructAndAppend(Args&&... args) { constructAndAppend<FailureAction::Crash>(std::forward<Args>(args)...); }
     template<typename... Args> ALWAYS_INLINE bool tryConstructAndAppend(Args&&... args) { return constructAndAppend<FailureAction::Report>(std::forward<Args>(args)...); }
 
-    void uncheckedAppend(ValueType&& value) { uncheckedAppend<ValueType>(std::forward<ValueType>(value)); }
-    template<typename U> void uncheckedAppend(U&&);
-    template<typename... Args> void uncheckedConstructAndAppend(Args&&...);
-
     template<typename U> ALWAYS_INLINE void append(const U* u, size_t size) { append<FailureAction::Crash>(u, size); }
     template<typename U> ALWAYS_INLINE bool tryAppend(const U* u, size_t size) { return append<FailureAction::Report>(u, size); }
     template<typename U> ALWAYS_INLINE void append(std::span<const U> span) { append(span.data(), span.size()); }
-    template<typename U> ALWAYS_INLINE void uncheckedAppend(std::span<const U> span) { uncheckedAppend<FailureAction::Crash>(span.data(), span.size()); }
+    template<typename U> ALWAYS_INLINE void appendList(std::initializer_list<U> initializerList) { append(std::data(initializerList), initializerList.size()); }
     template<typename U, size_t otherCapacity, typename OtherOverflowHandler, size_t otherMinCapacity, typename OtherMalloc> void appendVector(const Vector<U, otherCapacity, OtherOverflowHandler, otherMinCapacity, OtherMalloc>&);
     template<typename U, size_t otherCapacity, typename OtherOverflowHandler, size_t otherMinCapacity, typename OtherMalloc> void appendVector(Vector<U, otherCapacity, OtherOverflowHandler, otherMinCapacity, OtherMalloc>&&);
 
+    template<typename Functor, typename = typename std::enable_if_t<std::is_invocable_v<Functor, size_t>>>
+    void appendUsingFunctor(size_t, const Functor&);
+
     void insert(size_t position, ValueType&& value) { insert<ValueType>(position, std::forward<ValueType>(value)); }
     template<typename U> void insert(size_t position, const U*, size_t);
     template<typename U> void insert(size_t position, U&&);
@@ -879,6 +905,7 @@ public:
     void fill(const T& val) { fill(val, size()); }
 
     template<typename Iterator> void appendRange(Iterator start, Iterator end);
+    template<typename ContainerType, typename MapFunction> void appendContainerWithMapping(ContainerType&&, const MapFunction&);
 
     MallocPtr<T, Malloc> releaseBuffer();
 
@@ -912,9 +939,15 @@ public:
 
     bool isHashTableDeletedValue() const { return m_size == std::numeric_limits<decltype(m_size)>::max(); }
 
+    void unsafeAppendWithoutCapacityCheck(ValueType&& value) { unsafeAppendWithoutCapacityCheck<ValueType>(std::forward<ValueType>(value)); }
+    template<typename U> void unsafeAppendWithoutCapacityCheck(U&&);
+    template<typename U> bool unsafeAppendWithoutCapacityCheck(const U*, size_t);
+
 private:
+    template<FailureAction> bool growImpl(size_t);
     template<FailureAction> bool reserveCapacity(size_t newCapacity);
     template<FailureAction> bool reserveInitialCapacity(size_t initialCapacity);
+    template<FailureAction> bool growCapacityBy(size_t increment);
 
     template<FailureAction> bool expandCapacity(size_t newMinCapacity);
     template<FailureAction> T* expandCapacity(size_t newMinCapacity, T*);
@@ -925,7 +958,13 @@ private:
 
     template<FailureAction, typename U> bool append(U&&);
     template<FailureAction, typename U> bool append(const U*, size_t);
-    template<FailureAction, typename U> bool uncheckedAppend(const U*, size_t);
+
+    template<typename MapFunction, typename DestinationVectorType, typename SourceType, typename Enable> friend struct Mapper;
+    template<typename MapFunction, typename DestinationVectorType, typename SourceType, typename Enable> friend struct CompactMapper;
+    template<typename DestinationItemType, typename Collection> friend Vector<DestinationItemType> copyToVectorOf(const Collection&);
+    template<typename Collection> friend Vector<typename CopyOrMoveToVectorResult<Collection>::Type> copyToVector(const Collection&);
+    template<typename U, size_t otherInlineCapacity, typename OtherOverflowHandler, size_t otherMinCapacity, typename OtherMalloc> friend class Vector;
+    template<typename DestinationVector, typename Collection> friend DestinationVector copyToVectorSpecialization(const Collection&);
 
     template<size_t position, typename U, typename... Items>
     void uncheckedInitialize(U&& item, Items&&... items)
@@ -936,8 +975,8 @@ private:
     template<size_t position, typename U>
     void uncheckedInitialize(U&& value)
     {
-        ASSERT(position < size());
-        ASSERT(position < capacity());
+        ASSERT_WITH_SECURITY_IMPLICATION(position < size());
+        ASSERT_WITH_SECURITY_IMPLICATION(position < capacity());
         new (NotNull, begin() + position) T(std::forward<U>(value));
     }
 
@@ -1141,8 +1180,27 @@ template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t min
 template<typename Iterator>
 void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::appendRange(Iterator start, Iterator end)
 {
-    for (Iterator it = start; it != end; ++it)
-        append(*it);
+    using category = typename std::iterator_traits<Iterator>::iterator_category;
+    static_assert(std::is_base_of_v<std::input_iterator_tag, category>);
+
+    if constexpr (std::is_base_of_v<std::random_access_iterator_tag, category>)
+        reserveCapacity(size() + (end - start));
+
+    for (Iterator it = start; it != end; ++it) {
+        if constexpr (std::is_base_of_v<std::random_access_iterator_tag, category>)
+            unsafeAppendWithoutCapacityCheck(*it);
+        else
+            append(*it);
+    }
+}
+
+template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
+template<typename Functor, typename>
+void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::appendUsingFunctor(size_t size, const Functor& valueGenerator)
+{
+    reserveCapacity(this->size() + size);
+    for (size_t i = 0; i < size; ++i)
+        unsafeAppendWithoutCapacityCheck(valueGenerator(i));
 }
 
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
@@ -1217,22 +1275,29 @@ void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::resizeToFi
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
 void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::shrink(size_t size)
 {
-    ASSERT(size <= m_size);
+    ASSERT_WITH_SECURITY_IMPLICATION(size <= m_size);
     TypeOperations::destruct(begin() + size, end());
     asanBufferSizeWillChangeTo(size);
     m_size = size;
 }
 
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
-void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::grow(size_t size)
+template<FailureAction failureAction>
+bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::growImpl(size_t size)
 {
-    ASSERT(size >= m_size);
-    if (size > capacity())
-        expandCapacity<FailureAction::Crash>(size);
+    ASSERT_WITH_SECURITY_IMPLICATION(size >= m_size);
+    if (size > capacity()) {
+        bool success = expandCapacity<failureAction>(size);
+        if constexpr (failureAction == FailureAction::Report) {
+            if (UNLIKELY(!success))
+                return false;
+        }
+    }
     asanBufferSizeWillChangeTo(size);
     if (begin())
         TypeOperations::initializeIfNonPOD(end(), begin() + size);
     m_size = size;
+    return true;
 }
 
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
@@ -1315,13 +1380,30 @@ template<FailureAction action>
 inline bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::reserveInitialCapacity(size_t initialCapacity)
 {
     static_assert(action == FailureAction::Crash || action == FailureAction::Report);
-    ASSERT(!m_size);
+    ASSERT_WITH_SECURITY_IMPLICATION(!m_size);
     ASSERT(capacity() == inlineCapacity);
     if (initialCapacity <= inlineCapacity)
         return true;
     return Base::template allocateBuffer<action>(initialCapacity);
 }
 
+template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
+template<FailureAction action>
+bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::growCapacityBy(size_t increment)
+{
+    static_assert(action == FailureAction::Crash || action == FailureAction::Report);
+    unsigned increment32 = static_cast<unsigned>(increment);
+    unsigned capacity32 = static_cast<unsigned>(capacity());
+    unsigned newCapacity = increment32 + capacity32;
+    if (increment32 < increment || newCapacity < capacity32) {
+        if constexpr (action == FailureAction::Crash)
+            CRASH();
+        else
+            return false;
+    }
+    return reserveCapacity<action>(newCapacity);
+}
+
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
 void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::shrinkCapacity(size_t newCapacity)
 {
@@ -1384,14 +1466,13 @@ ALWAYS_INLINE bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Mallo
 }
 
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
-template<FailureAction action, typename U>
-ALWAYS_INLINE bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::uncheckedAppend(const U* data, size_t dataSize)
+template<typename U>
+ALWAYS_INLINE bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::unsafeAppendWithoutCapacityCheck(const U* data, size_t dataSize)
 {
-    static_assert(action == FailureAction::Crash || action == FailureAction::Report);
     if (!dataSize)
         return true;
 
-    ASSERT_WITH_MESSAGE((Checked<size_t>(size()) + dataSize) <= capacity(), "uncheckedAppend() without expanding capacity");
+    ASSERT_WITH_SECURITY_IMPLICATION((Checked<size_t>(size()) + dataSize) <= capacity());
 
     size_t newSize = m_size + dataSize;
     asanBufferSizeWillChangeTo(newSize);
@@ -1434,7 +1515,7 @@ template<FailureAction action, typename U>
 bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::appendSlowCase(U&& value)
 {
     static_assert(action == FailureAction::Crash || action == FailureAction::Report);
-    ASSERT(size() == capacity());
+    ASSERT_WITH_SECURITY_IMPLICATION(size() == capacity());
 
     auto ptr = const_cast<std::remove_cvref_t<U>*>(std::addressof(value));
     ptr = expandCapacity<action>(size() + 1, ptr);
@@ -1455,7 +1536,7 @@ template<FailureAction action, typename... Args>
 bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::constructAndAppendSlowCase(Args&&... args)
 {
     static_assert(action == FailureAction::Crash || action == FailureAction::Report);
-    ASSERT(size() == capacity());
+    ASSERT_WITH_SECURITY_IMPLICATION(size() == capacity());
 
     bool success = expandCapacity<action>(size() + 1);
     if constexpr (action == FailureAction::Report) {
@@ -1476,9 +1557,9 @@ bool Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::constructA
 
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
 template<typename U>
-ALWAYS_INLINE void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::uncheckedAppend(U&& value)
+ALWAYS_INLINE void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::unsafeAppendWithoutCapacityCheck(U&& value)
 {
-    ASSERT(size() < capacity());
+    ASSERT_WITH_SECURITY_IMPLICATION(size() < capacity());
 
     asanBufferSizeWillChangeTo(m_size + 1);
 
@@ -1486,18 +1567,6 @@ ALWAYS_INLINE void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Mallo
     ++m_size;
 }
 
-template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
-template<typename... Args>
-ALWAYS_INLINE void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::uncheckedConstructAndAppend(Args&&... args)
-{
-    ASSERT(size() < capacity());
-
-    asanBufferSizeWillChangeTo(m_size + 1);
-
-    new (NotNull, end()) T(std::forward<Args>(args)...);
-    ++m_size;
-}
-
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
 template<typename U, size_t otherCapacity, typename OtherOverflowHandler, size_t otherMinCapacity, typename OtherMalloc>
 inline void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::appendVector(const Vector<U, otherCapacity, OtherOverflowHandler, otherMinCapacity, OtherMalloc>& val)
@@ -1513,7 +1582,7 @@ inline void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::app
     if (newSize > capacity())
         expandCapacity<FailureAction::Crash>(newSize);
     for (auto& item : val)
-        uncheckedAppend(WTFMove(item));
+        unsafeAppendWithoutCapacityCheck(WTFMove(item));
 }
 
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
@@ -1687,10 +1756,13 @@ inline auto Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::map
     ResultVector result;
     result.reserveInitialCapacity(size());
     for (size_t i = 0; i < size(); ++i)
-        result.uncheckedAppend(mapFunction(at(i)));
+        result.unsafeAppendWithoutCapacityCheck(mapFunction(at(i)));
     return result;
 }
 
+template <typename ContainerType>
+size_t containerSize(const ContainerType& container) { return std::size(container); }
+
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
 template<typename MapFunction>
 inline auto Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::map(MapFunction&& mapFunction) const -> std::enable_if_t<std::is_invocable_v<MapFunction, const T&>, Vector<typename std::invoke_result_t<MapFunction, const T&>>>
@@ -1698,6 +1770,15 @@ inline auto Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::map
     return map<Vector<typename std::invoke_result_t<MapFunction, const T&>>, MapFunction>(std::forward<MapFunction>(mapFunction));
 }
 
+template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
+template<typename ContainerType, typename MapFunction>
+void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::appendContainerWithMapping(ContainerType&& container, const MapFunction& mapFunction)
+{
+    reserveCapacity(size() + containerSize(container));
+    for (auto&& item : container)
+        unsafeAppendWithoutCapacityCheck(mapFunction(std::forward<decltype(item)>(item)));
+}
+
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
 inline MallocPtr<T, Malloc> Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::releaseBuffer()
 {
@@ -1723,7 +1804,7 @@ inline MallocPtr<T, Malloc> Vector<T, inlineCapacity, OverflowHandler, minCapaci
 template<typename T, size_t inlineCapacity, typename OverflowHandler, size_t minCapacity, typename Malloc>
 inline void Vector<T, inlineCapacity, OverflowHandler, minCapacity, Malloc>::checkConsistency()
 {
-#if ASSERT_ENABLED
+#if ENABLE(SECURITY_ASSERTIONS)
     for (size_t i = 0; i < size(); ++i)
         ValueCheck<T>::checkConsistency(at(i));
 #endif
@@ -1744,7 +1825,7 @@ bool operator==(const Vector<T, inlineCapacityA, OverflowHandlerA, minCapacityA,
     return VectorTypeOperations<T>::compare(a.data(), b.data(), a.size());
 }
 
-#if ASSERT_ENABLED
+#if ENABLE(SECURITY_ASSERTIONS)
 template<typename T> struct ValueCheck<Vector<T>> {
     typedef Vector<T> TraitType;
     static void checkConsistency(const Vector<T>& v)
@@ -1752,7 +1833,7 @@ template<typename T> struct ValueCheck<Vector<T>> {
         v.checkConsistency();
     }
 };
-#endif // ASSERT_ENABLED
+#endif // ENABLE(SECURITY_ASSERTIONS)
 
 template<typename VectorType, typename Func>
 size_t removeRepeatedElements(VectorType& vector, const Func& func)
@@ -1776,42 +1857,48 @@ struct CollectionInspector {
     using SourceItemType = typename std::iterator_traits<IteratorType>::value_type;
 };
 
-template<typename MapFunction, typename SourceType, typename Enable = void>
+template<typename MapFunction, typename DestinationVectorType, typename SourceType, typename Enable = void>
 struct Mapper {
-    using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
-    using DestinationItemType = typename std::invoke_result<MapFunction, SourceItemType&>::type;
-
-    static Vector<DestinationItemType> map(const SourceType& source, const MapFunction& mapFunction)
+    static void map(DestinationVectorType& result, const SourceType& source, const MapFunction& mapFunction)
     {
-        Vector<DestinationItemType> result;
-        // FIXME: Use std::size when available on all compilers.
-        result.reserveInitialCapacity(source.size());
+        result.reserveInitialCapacity(containerSize(source));
         for (auto&& item : source)
-            result.uncheckedAppend(mapFunction(item));
-        return result;
+            result.unsafeAppendWithoutCapacityCheck(mapFunction(item));
     }
 };
 
-template<typename MapFunction, typename SourceType>
-struct Mapper<MapFunction, SourceType, typename std::enable_if<std::is_rvalue_reference<SourceType&&>::value>::type> {
+template<typename MapFunction, typename DestinationVectorType, typename SourceType>
+struct Mapper<MapFunction, DestinationVectorType, SourceType, typename std::enable_if<std::is_rvalue_reference<SourceType&&>::value>::type> {
+    static void map(DestinationVectorType& result, SourceType&& source, const MapFunction& mapFunction)
+    {
+        result.reserveInitialCapacity(containerSize(source));
+        for (auto&& item : source)
+            result.unsafeAppendWithoutCapacityCheck(mapFunction(WTFMove(item)));
+    }
+};
+
+template<size_t inlineCapacity = 0, typename OverflowHandler = CrashOnOverflow, size_t minCapacity = 16, typename MapFunction, typename SourceType>
+Vector<typename std::invoke_result<MapFunction, typename CollectionInspector<SourceType>::SourceItemType&&>::type, inlineCapacity, OverflowHandler, minCapacity> map(SourceType&& source, MapFunction&& mapFunction)
+{
     using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
     using DestinationItemType = typename std::invoke_result<MapFunction, SourceItemType&&>::type;
+    using DestinationVectorType = Vector<DestinationItemType, inlineCapacity, OverflowHandler, minCapacity>;
 
-    static Vector<DestinationItemType> map(SourceType&& source, const MapFunction& mapFunction)
-    {
-        Vector<DestinationItemType> result;
-        // FIXME: Use std::size when available on all compilers.
-        result.reserveInitialCapacity(source.size());
-        for (auto&& item : source)
-            result.uncheckedAppend(mapFunction(WTFMove(item)));
-        return result;
-    }
-};
+    DestinationVectorType result;
+    Mapper<MapFunction, DestinationVectorType, SourceType>::map(result, std::forward<SourceType>(source), std::forward<MapFunction>(mapFunction));
+    return result;
+}
 
-template<typename MapFunction, typename SourceType>
-Vector<typename Mapper<MapFunction, SourceType>::DestinationItemType> map(SourceType&& source, MapFunction&& mapFunction)
+template<size_t inlineCapacity = 0, typename OverflowHandler = CrashOnOverflow, size_t minCapacity = 16, typename MapFunction, typename SourceType>
+Vector<typename std::invoke_result<MapFunction, typename CollectionInspector<SourceType>::SourceItemType&>::type, inlineCapacity, OverflowHandler, minCapacity> map(SourceType& source, MapFunction&& mapFunction)
 {
-    return Mapper<MapFunction, SourceType>::map(std::forward<SourceType>(source), std::forward<MapFunction>(mapFunction));
+    using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
+    using DestinationItemType = typename std::invoke_result<MapFunction, SourceItemType&>::type;
+    using DestinationVectorType = Vector<DestinationItemType, inlineCapacity, OverflowHandler, minCapacity>;
+
+    DestinationVectorType result;
+    Mapper<MapFunction, DestinationVectorType, SourceType&>::map(result, source, std::forward<MapFunction>(mapFunction));
+    return result;
 }
 
 template<typename MapFunctionReturnType>
@@ -1835,48 +1922,107 @@ struct CompactMapTraits<RefPtr<T>> {
     static ItemType extractValue(RefPtr<T>&& returnValue) { return returnValue.releaseNonNull(); }
 };
 
-template<typename MapFunction, typename SourceType, typename Enable = void>
+template<typename T>
+struct CompactMapTraits<RetainPtr<T>> {
+    using ItemType = RetainPtr<T>;
+    static bool hasValue(const RetainPtr<T>& returnValue) { return !!returnValue; }
+    static ItemType extractValue(RetainPtr<T>&& returnValue) { return WTFMove(returnValue); }
+};
+
+template<typename MapFunction, typename DestinationVectorType, typename SourceType, typename Enable = void>
 struct CompactMapper {
     using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
     using ResultItemType = typename std::invoke_result<MapFunction, SourceItemType&>::type;
-    using DestinationItemType = typename CompactMapTraits<ResultItemType>::ItemType;
 
-    static Vector<DestinationItemType> compactMap(const SourceType& source, const MapFunction& mapFunction)
+    static void compactMap(DestinationVectorType& result, const SourceType& source, const MapFunction& mapFunction)
     {
-        Vector<DestinationItemType> result;
         for (auto&& item : source) {
             auto itemResult = mapFunction(item);
             if (CompactMapTraits<ResultItemType>::hasValue(itemResult))
                 result.append(CompactMapTraits<ResultItemType>::extractValue(WTFMove(itemResult)));
         }
         result.shrinkToFit();
+    }
+};
+
+template<typename MapFunction, typename DestinationVectorType, typename SourceType>
+struct CompactMapper<MapFunction, DestinationVectorType, SourceType, typename std::enable_if<std::is_rvalue_reference<SourceType&&>::value>::type> {
+    using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
+    using ResultItemType = typename std::invoke_result<MapFunction, SourceItemType&&>::type;
+
+    static void compactMap(DestinationVectorType& result, SourceType&& source, const MapFunction& mapFunction)
+    {
+        for (auto&& item : source) {
+            auto itemResult = mapFunction(WTFMove(item));
+            if (CompactMapTraits<ResultItemType>::hasValue(itemResult))
+                result.unsafeAppendWithoutCapacityCheck(CompactMapTraits<ResultItemType>::extractValue(WTFMove(itemResult)));
+        }
+        result.shrinkToFit();
+    }
+};
+
+template<size_t inlineCapacity = 0, typename OverflowHandler = CrashOnOverflow, size_t minCapacity = 16, typename MapFunction, typename SourceType>
+Vector<typename CompactMapTraits<typename std::invoke_result<MapFunction, typename CollectionInspector<SourceType>::SourceItemType&&>::type>::ItemType, inlineCapacity, OverflowHandler, minCapacity> compactMap(SourceType&& source, MapFunction&& mapFunction)
+{
+    using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
+    using ResultItemType = typename std::invoke_result<MapFunction, SourceItemType&&>::type;
+    using DestinationItemType = typename CompactMapTraits<ResultItemType>::ItemType;
+    using DestinationVectorType = Vector<DestinationItemType, inlineCapacity, OverflowHandler, minCapacity>;
+
+    DestinationVectorType result;
+    result.reserveInitialCapacity(containerSize(source));
+    CompactMapper<MapFunction, DestinationVectorType, SourceType>::compactMap(result, std::forward<SourceType>(source), std::forward<MapFunction>(mapFunction));
+    return result;
+}
+
+template<size_t inlineCapacity = 0, typename OverflowHandler = CrashOnOverflow, size_t minCapacity = 16, typename MapFunction, typename SourceType>
+Vector<typename CompactMapTraits<typename std::invoke_result<MapFunction, typename CollectionInspector<SourceType>::SourceItemType&>::type>::ItemType, inlineCapacity, OverflowHandler, minCapacity> compactMap(SourceType& source, MapFunction&& mapFunction)
+{
+    using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
+    using ResultItemType = typename std::invoke_result<MapFunction, SourceItemType&>::type;
+    using DestinationItemType = typename CompactMapTraits<ResultItemType>::ItemType;
+    using DestinationVectorType = Vector<DestinationItemType, inlineCapacity, OverflowHandler, minCapacity>;
+
+    DestinationVectorType result;
+    result.reserveInitialCapacity(containerSize(source));
+    CompactMapper<MapFunction, DestinationVectorType, SourceType&>::compactMap(result, source, std::forward<MapFunction>(mapFunction));
+    return result;
+}
+
+template<typename MapFunction, typename SourceType>
+struct FlatMapper {
+    using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
+    using DestinationItemType = typename CollectionInspector<typename std::invoke_result<MapFunction, SourceItemType&>::type>::SourceItemType;
+
+    static Vector<DestinationItemType> flatMap(const SourceType& source, const MapFunction& mapFunction)
+    {
+        Vector<DestinationItemType> result;
+        for (auto&& item : source)
+            result.appendVector(mapFunction(item));
+        result.shrinkToFit();
         return result;
     }
 };
 
-template<typename MapFunction, typename SourceType>
-struct CompactMapper<MapFunction, SourceType, typename std::enable_if<std::is_rvalue_reference<SourceType&&>::value>::type> {
+template<typename MapFunction, typename SourceType> requires std::is_rvalue_reference<SourceType&&>::value
+struct FlatMapper<MapFunction, SourceType> {
     using SourceItemType = typename CollectionInspector<SourceType>::SourceItemType;
-    using ResultItemType = typename std::invoke_result<MapFunction, SourceItemType&&>::type;
-    using DestinationItemType = typename CompactMapTraits<ResultItemType>::ItemType;
+    using DestinationItemType = typename CollectionInspector<typename std::invoke_result<MapFunction, SourceItemType&&>::type>::SourceItemType;
 
-    static Vector<DestinationItemType> compactMap(SourceType&& source, const MapFunction& mapFunction)
+    static Vector<DestinationItemType> flatMap(SourceType&& source, const MapFunction& mapFunction)
     {
         Vector<DestinationItemType> result;
-        for (auto&& item : source) {
-            auto itemResult = mapFunction(WTFMove(item));
-            if (CompactMapTraits<ResultItemType>::hasValue(itemResult))
-                result.append(CompactMapTraits<ResultItemType>::extractValue(WTFMove(itemResult)));
-        }
+        for (auto&& item : source)
+            result.appendVector(mapFunction(WTFMove(item)));
         result.shrinkToFit();
         return result;
     }
 };
 
 template<typename MapFunction, typename SourceType>
-Vector<typename CompactMapper<MapFunction, SourceType>::DestinationItemType> compactMap(SourceType&& source, MapFunction&& mapFunction)
+Vector<typename FlatMapper<MapFunction, SourceType>::DestinationItemType> flatMap(SourceType&& source, MapFunction&& mapFunction)
 {
-    return CompactMapper<MapFunction, SourceType>::compactMap(std::forward<SourceType>(source), std::forward<MapFunction>(mapFunction));
+    return FlatMapper<MapFunction, SourceType>::flatMap(std::forward<SourceType>(source), std::forward<MapFunction>(mapFunction));
 }
 
 template<typename DestinationVector, typename Collection>
@@ -1886,7 +2032,7 @@ inline auto copyToVectorSpecialization(const Collection& collection) -> Destinat
     // FIXME: Use std::size when available on all compilers.
     result.reserveInitialCapacity(collection.size());
     for (auto&& item : collection)
-        result.uncheckedAppend(item);
+        result.unsafeAppendWithoutCapacityCheck(item);
     return result;
 }
 
@@ -1912,8 +2058,8 @@ inline Vector<typename CopyOrMoveToVectorResult<Collection>::Type> copyToVector(
 template<typename DestinationItemType, typename Collection>
 inline auto moveToVectorOf(Collection&& collection) -> Vector<DestinationItemType>
 {
-    return WTF::map(collection, [] (auto&& v) -> DestinationItemType {
-        return WTFMove(v);
+    return WTF::map(collection, [] (auto&& item) -> DestinationItemType {
+        return std::forward<DestinationItemType>(item);
     });
 }
 
@@ -1923,6 +2069,8 @@ inline Vector<typename CopyOrMoveToVectorResult<Collection>::Type> moveToVector(
     return moveToVectorOf<typename CopyOrMoveToVectorResult<Collection>::Type>(collection);
 }
 
+template<typename U> Vector(const U*, size_t) -> Vector<U>;
+
 } // namespace WTF
 
 using WTF::UnsafeVectorOverflow;
@@ -1931,4 +2079,5 @@ using WTF::copyToVector;
 using WTF::copyToVectorOf;
 using WTF::copyToVectorSpecialization;
 using WTF::compactMap;
+using WTF::flatMap;
 using WTF::removeRepeatedElements;
