@@ -47,19 +47,19 @@ ExceptionOr<Ref<BarcodeDetector>> BarcodeDetector::create(ScriptExecutionContext
         const auto& document = downcast<Document>(scriptExecutionContext);
         const auto* page = document.page();
         if (!page)
-            return Exception { AbortError };
+            return Exception { ExceptionCode::AbortError };
         auto backing = page->chrome().createBarcodeDetector(barcodeDetectorOptions.convertToBacking());
         if (!backing)
-            return Exception { AbortError };
+            return Exception { ExceptionCode::AbortError };
         return adoptRef(*new BarcodeDetector(backing.releaseNonNull()));
     }
 
     if (is<WorkerGlobalScope>(scriptExecutionContext)) {
         // FIXME: https://bugs.webkit.org/show_bug.cgi?id=255380 Make the Shape Detection API work in Workers
-        return Exception { AbortError };
+        return Exception { ExceptionCode::AbortError };
     }
 
-    return Exception { AbortError };
+    return Exception { ExceptionCode::AbortError };
 }
 
 
@@ -76,7 +76,7 @@ ExceptionOr<void> BarcodeDetector::getSupportedFormats(ScriptExecutionContext& s
         const auto& document = downcast<Document>(scriptExecutionContext);
         const auto* page = document.page();
         if (!page)
-            return Exception { AbortError };
+            return Exception { ExceptionCode::AbortError };
         page->chrome().getBarcodeDetectorSupportedFormats(([promise = WTFMove(promise)](Vector<ShapeDetection::BarcodeFormat>&& barcodeFormats) mutable {
             promise.resolve(barcodeFormats.map([](auto format) {
                 return convertFromBacking(format);
@@ -87,10 +87,10 @@ ExceptionOr<void> BarcodeDetector::getSupportedFormats(ScriptExecutionContext& s
 
     if (is<WorkerGlobalScope>(scriptExecutionContext)) {
         // FIXME: https://bugs.webkit.org/show_bug.cgi?id=255380 Make the Shape Detection API work in Workers
-        return Exception { AbortError };
+        return Exception { ExceptionCode::AbortError };
     }
 
-    return Exception { AbortError };
+    return Exception { ExceptionCode::AbortError };
 }
 
 void BarcodeDetector::detect(ScriptExecutionContext& scriptExecutionContext, ImageBitmap::Source&& source, DetectPromise&& promise)
