@@ -41,11 +41,13 @@
 
 namespace WebCore {
 
+class GPUDevice;
+
 class GPUBuffer : public RefCounted<GPUBuffer> {
 public:
-    static Ref<GPUBuffer> create(Ref<WebGPU::Buffer>&& backing, size_t bufferSize, GPUBufferUsageFlags usage, bool mappedAtCreation)
+    static Ref<GPUBuffer> create(Ref<WebGPU::Buffer>&& backing, size_t bufferSize, GPUBufferUsageFlags usage, bool mappedAtCreation, GPUDevice& device)
     {
-        return adoptRef(*new GPUBuffer(WTFMove(backing), bufferSize, usage, mappedAtCreation));
+        return adoptRef(*new GPUBuffer(WTFMove(backing), bufferSize, usage, mappedAtCreation, device));
     }
 
     String label() const;
@@ -54,9 +56,9 @@ public:
     using MapAsyncPromise = DOMPromiseDeferred<IDLNull>;
     void mapAsync(GPUMapModeFlags, std::optional<GPUSize64> offset, std::optional<GPUSize64> sizeForMap, MapAsyncPromise&&);
     ExceptionOr<Ref<JSC::ArrayBuffer>> getMappedRange(std::optional<GPUSize64> offset, std::optional<GPUSize64> rangeSize);
-    void unmap();
+    void unmap(ScriptExecutionContext&);
 
-    void destroy();
+    void destroy(ScriptExecutionContext&);
 
     WebGPU::Buffer& backing() { return m_backing; }
     const WebGPU::Buffer& backing() const { return m_backing; }
@@ -68,7 +70,8 @@ public:
 
     ~GPUBuffer();
 private:
-    GPUBuffer(Ref<WebGPU::Buffer>&&, size_t, GPUBufferUsageFlags, bool);
+    GPUBuffer(Ref<WebGPU::Buffer>&&, size_t, GPUBufferUsageFlags, bool, GPUDevice&);
+    void internalUnmap(ScriptExecutionContext&);
 
     Ref<WebGPU::Buffer> m_backing;
     WebGPU::Buffer::MappedRange m_mappedRange;
@@ -77,6 +80,7 @@ private:
     const GPUBufferUsageFlags m_usage { 0 };
     GPUBufferMapState m_mapState { GPUBufferMapState::Unmapped };
     std::optional<MapAsyncPromise> m_pendingMapPromise;
+    GPUDevice& m_device;
 };
 
 }
