@@ -25,6 +25,8 @@
 
 #pragma once
 
+#include "GPUIntegralTypes.h"
+#include "GPUTextureDimension.h"
 #include "GPUTextureFormat.h"
 #include "WebGPUTexture.h"
 #include <optional>
@@ -35,13 +37,15 @@
 namespace WebCore {
 
 class GPUTextureView;
+
+struct GPUTextureDescriptor;
 struct GPUTextureViewDescriptor;
 
 class GPUTexture : public RefCounted<GPUTexture> {
 public:
-    static Ref<GPUTexture> create(Ref<WebGPU::Texture>&& backing, GPUTextureFormat format)
+    static Ref<GPUTexture> create(Ref<WebGPU::Texture>&& backing, const GPUTextureDescriptor& descriptor)
     {
-        return adoptRef(*new GPUTexture(WTFMove(backing), format));
+        return adoptRef(*new GPUTexture(WTFMove(backing), descriptor));
     }
 
     String label() const;
@@ -55,15 +59,26 @@ public:
     const WebGPU::Texture& backing() const { return m_backing; }
     GPUTextureFormat format() const { return m_format; }
 
+    GPUIntegerCoordinateOut width() const;
+    GPUIntegerCoordinateOut height() const;
+    GPUIntegerCoordinateOut depthOrArrayLayers() const;
+    GPUIntegerCoordinateOut mipLevelCount() const;
+    GPUSize32Out sampleCount() const;
+    GPUTextureDimension dimension() const;
+    GPUFlagsConstant usage() const;
+
 private:
-    GPUTexture(Ref<WebGPU::Texture>&& backing, GPUTextureFormat format)
-        : m_backing(WTFMove(backing))
-        , m_format(format)
-    {
-    }
+    GPUTexture(Ref<WebGPU::Texture>&&, const GPUTextureDescriptor&);
 
     Ref<WebGPU::Texture> m_backing;
-    GPUTextureFormat m_format;
+    const GPUTextureFormat m_format;
+    const GPUIntegerCoordinateOut m_width;
+    const GPUIntegerCoordinateOut m_height;
+    const GPUIntegerCoordinateOut m_depthOrArrayLayers;
+    const GPUIntegerCoordinateOut m_mipLevelCount;
+    const GPUSize32Out m_sampleCount;
+    const GPUTextureDimension m_dimension;
+    const GPUFlagsConstant m_usage;
 };
 
 }
