@@ -39,15 +39,15 @@ namespace WebCore {
 static ExceptionOr<ApplePayAMSUIRequest> convertAndValidateApplePayAMSUIRequest(Document& document, JSC::JSValue data)
 {
     if (data.isEmpty())
-        return Exception { TypeError, "Missing payment method data."_s };
+        return Exception { ExceptionCode::TypeError, "Missing payment method data."_s };
 
     auto throwScope = DECLARE_THROW_SCOPE(document.vm());
     auto applePayAMSUIRequest = convertDictionary<ApplePayAMSUIRequest>(*document.globalObject(), data);
     if (throwScope.exception())
-        return Exception { ExistingExceptionError };
+        return Exception { ExceptionCode::ExistingExceptionError };
 
     if (!applePayAMSUIRequest.engagementRequest.startsWith('{'))
-        return Exception { TypeError, "Member ApplePayAMSUIRequest.engagementRequest is required and must be a JSON-serializable object"_s };
+        return Exception { ExceptionCode::TypeError, "Member ApplePayAMSUIRequest.engagementRequest is required and must be a JSON-serializable object"_s };
 
     return WTFMove(applePayAMSUIRequest);
 }
@@ -79,7 +79,7 @@ bool ApplePayAMSUIPaymentHandler::hasActiveSession(Document& document)
 void ApplePayAMSUIPaymentHandler::finishSession(std::optional<bool>&& result)
 {
     if (!result) {
-        m_paymentRequest->reject(Exception { AbortError });
+        m_paymentRequest->reject(Exception { ExceptionCode::AbortError });
         return;
     }
 
@@ -132,8 +132,8 @@ ExceptionOr<void> ApplePayAMSUIPaymentHandler::show(Document& document)
 {
     ASSERT(m_applePayAMSUIRequest);
 
-    if (!page().startApplePayAMSUISession(document, *this, *m_applePayAMSUIRequest))
-        return Exception { AbortError };
+    if (!page().startApplePayAMSUISession(document.topDocument().url(), *this, *m_applePayAMSUIRequest))
+        return Exception { ExceptionCode::AbortError };
 
     return { };
 }
