@@ -53,18 +53,18 @@ void ApplePaySetup::getSetupFeatures(Document& document, SetupFeaturesPromise&&
 {
     auto canCall = PaymentSession::canCreateSession(document);
     if (canCall.hasException()) {
-        promise.settle(canCall.releaseException());
+        promise.reject(canCall.releaseException());
         return;
     }
 
     auto page = document.page();
     if (!page) {
-        promise.settle(Exception { InvalidStateError });
+        promise.reject(Exception { ExceptionCode::InvalidStateError });
         return;
     }
 
     if (m_setupFeaturesPromise) {
-        promise.settle(Exception { InvalidStateError });
+        promise.reject(Exception { ExceptionCode::InvalidStateError });
         return;
     }
 
@@ -78,7 +78,7 @@ void ApplePaySetup::getSetupFeatures(Document& document, SetupFeaturesPromise&&
 
     page->paymentCoordinator().getSetupFeatures(m_configuration, document.url(), [this, pendingActivity = makePendingActivity(*this)](Vector<Ref<ApplePaySetupFeature>>&& setupFeatures) {
         if (m_setupFeaturesPromise)
-            std::exchange(m_setupFeaturesPromise, std::nullopt)->settle(WTFMove(setupFeatures));
+            std::exchange(m_setupFeaturesPromise, std::nullopt)->resolve(WTFMove(setupFeatures));
     });
 }
 
@@ -86,49 +86,55 @@ void ApplePaySetup::begin(Document& document, Vector<RefPtr<ApplePaySetupFeature
 {
     auto canCall = PaymentSession::canCreateSession(document);
     if (canCall.hasException()) {
-        promise.settle(canCall.releaseException());
+        promise.reject(canCall.releaseException());
         return;
     }
 
     if (!UserGestureIndicator::processingUserGesture()) {
-        promise.settle(Exception { InvalidAccessError, "Must call ApplePaySetup.begin from a user gesture handler." });
+        promise.reject(Exception { ExceptionCode::InvalidAccessError, "Must call ApplePaySetup.begin from a user gesture handler."_s });
         return;
     }
 
     auto page = document.page();
     if (!page) {
-        promise.settle(Exception { InvalidStateError });
+        promise.reject(Exception { ExceptionCode::InvalidStateError });
         return;
     }
 
     if (m_beginPromise) {
-        promise.settle(Exception { InvalidStateError });
+        promise.reject(Exception { ExceptionCode::InvalidStateError });
         return;
     }
 
     m_beginPromise = WTFMove(promise);
     m_pendingActivity = makePendingActivity(*this);
 
-    page->paymentCoordinator().beginApplePaySetup(m_configuration, document.url(), WTFMove(features), [this](bool result) {
+    page->paymentCoordinator().beginApplePaySetup(m_configuration, document.topDocument().url(), WTFMove(features), [this](bool result) {
         if (m_beginPromise)
-            std::exchange(m_beginPromise, std::nullopt)->settle(result);
+            std::exchange(m_beginPromise, std::nullopt)->resolve(result);
     });
 }
 
+Ref<ApplePaySetup> ApplePaySetup::create(ScriptExecutionContext& context, ApplePaySetupConfiguration&& configuration)
+{
+    auto setup = adoptRef(*new ApplePaySetup(context, WTFMove(configuration)));
+    setup->suspendIfNeeded();
+    return setup;
+}
+
 ApplePaySetup::ApplePaySetup(ScriptExecutionContext& context, ApplePaySetupConfiguration&& configuration)
     : ActiveDOMObject(&context)
     , m_configuration(WTFMove(configuration))
 {
-    suspendIfNeeded();
 }
 
 void ApplePaySetup::stop()
 {
     if (m_setupFeaturesPromise)
-        std::exchange(m_setupFeaturesPromise, std::nullopt)->settle(Exception { AbortError });
+        std::exchange(m_setupFeaturesPromise, std::nullopt)->reject(Exception { ExceptionCode::AbortError });
 
     if (m_beginPromise)
-        std::exchange(m_beginPromise, std::nullopt)->settle(Exception { AbortError });
+        std::exchange(m_beginPromise, std::nullopt)->reject(Exception { ExceptionCode::AbortError });
 
     if (auto page = downcast<Document>(*scriptExecutionContext()).page())
         page->paymentCoordinator().endApplePaySetup();
