@@ -59,7 +59,7 @@ void BasicCredential::isConditionalMediationAvailable(Document& document, DOMPro
     if (auto* page = document.page())
         page->authenticatorCoordinator().isConditionalMediationAvailable(document, WTFMove(promise));
     else
-        promise.reject(Exception { InvalidStateError });
+        promise.reject(Exception { ExceptionCode::InvalidStateError });
 }
 
 } // namespace WebCore
