@@ -55,11 +55,11 @@ ScopeAndCrossOriginParent CredentialsContainer::scopeAndCrossOriginParent() cons
     auto& origin = m_document->securityOrigin();
     auto& url = m_document->url();
     std::optional<SecurityOriginData> crossOriginParent;
-    for (auto* document = m_document->parentDocument(); document; document = document->parentDocument()) {
+    for (RefPtr document = m_document->parentDocument(); document; document = document->parentDocument()) {
         if (!origin.isSameOriginDomain(document->securityOrigin()) && !areRegistrableDomainsEqual(url, document->url()))
             isSameSite = false;
         if (!crossOriginParent && !origin.isSameOriginAs(document->securityOrigin()))
-            crossOriginParent = origin.data();
+            crossOriginParent = document->securityOrigin().data();
     }
 
     if (!crossOriginParent)
@@ -74,11 +74,11 @@ void CredentialsContainer::get(CredentialRequestOptions&& options, CredentialPro
     // The following implements https://www.w3.org/TR/credential-management-1/#algorithm-request as of 4 August 2017
     // with enhancement from 14 November 2017 Editor's Draft.
     if (!m_document || !m_document->page()) {
-        promise.reject(Exception { NotSupportedError });
+        promise.reject(Exception { ExceptionCode::NotSupportedError });
         return;
     }
     if (options.signal && options.signal->aborted()) {
-        promise.reject(Exception { AbortError, "Aborted by AbortSignal."_s });
+        promise.reject(Exception { ExceptionCode::AbortError, "Aborted by AbortSignal."_s });
         return;
     }
     // Step 1-2.
@@ -88,13 +88,13 @@ void CredentialsContainer::get(CredentialRequestOptions&& options, CredentialPro
     // Step 4-6. Shortcut as we only support PublicKeyCredential which can only
     // be requested from [[discoverFromExternalSource]].
     if (!options.publicKey) {
-        promise.reject(Exception { NotSupportedError, "Only PublicKeyCredential is supported."_s });
+        promise.reject(Exception { ExceptionCode::NotSupportedError, "Only PublicKeyCredential is supported."_s });
         return;
     }
 
     // The request will be aborted in WebAuthenticatorCoordinatorProxy if conditional mediation is not available.
-    if (options.mediation != CredentialRequestOptions::MediationRequirement::Conditional && !m_document->hasFocus()) {
-        promise.reject(Exception { NotAllowedError, "The document is not focused."_s });
+    if (options.mediation != MediationRequirement::Conditional && !m_document->hasFocus()) {
+        promise.reject(Exception { ExceptionCode::NotAllowedError, "The document is not focused."_s });
         return;
     }
 
@@ -103,7 +103,7 @@ void CredentialsContainer::get(CredentialRequestOptions&& options, CredentialPro
 
 void CredentialsContainer::store(const BasicCredential&, CredentialPromise&& promise)
 {
-    promise.reject(Exception { NotSupportedError, "Not implemented."_s });
+    promise.reject(Exception { ExceptionCode::NotSupportedError, "Not implemented."_s });
 }
 
 void CredentialsContainer::isCreate(CredentialCreationOptions&& options, CredentialPromise&& promise)
@@ -111,11 +111,11 @@ void CredentialsContainer::isCreate(CredentialCreationOptions&& options, Credent
     // The following implements https://www.w3.org/TR/credential-management-1/#algorithm-create as of 4 August 2017
     // with enhancement from 14 November 2017 Editor's Draft.
     if (!m_document || !m_document->page()) {
-        promise.reject(Exception { NotSupportedError });
+        promise.reject(Exception { ExceptionCode::NotSupportedError });
         return;
     }
     if (options.signal && options.signal->aborted()) {
-        promise.reject(Exception { AbortError, "Aborted by AbortSignal."_s });
+        promise.reject(Exception { ExceptionCode::AbortError, "Aborted by AbortSignal."_s });
         return;
     }
     // Step 1-2.
@@ -123,22 +123,22 @@ void CredentialsContainer::isCreate(CredentialCreationOptions&& options, Credent
 
     // Step 3-7. Shortcut as we only support one kind of credentials.
     if (!options.publicKey) {
-        promise.reject(Exception { NotSupportedError, "Only PublicKeyCredential is supported."_s });
+        promise.reject(Exception { ExceptionCode::NotSupportedError, "Only PublicKeyCredential is supported."_s });
         return;
     }
 
     // Extra.
     if (!m_document->hasFocus()) {
-        promise.reject(Exception { NotAllowedError, "The document is not focused."_s });
+        promise.reject(Exception { ExceptionCode::NotAllowedError, "The document is not focused."_s });
         return;
     }
 
-    m_document->page()->authenticatorCoordinator().create(*m_document, options.publicKey.value(), scopeAndCrossOriginParent().first, WTFMove(options.signal), WTFMove(promise));
+    m_document->page()->authenticatorCoordinator().create(*m_document, WTFMove(options), scopeAndCrossOriginParent().first, WTFMove(options.signal), WTFMove(promise));
 }
 
 void CredentialsContainer::preventSilentAccess(DOMPromiseDeferred<void>&& promise) const
 {
-    promise.reject(Exception { NotSupportedError, "Not implemented."_s });
+    promise.resolve();
 }
 
 } // namespace WebCore
