@@ -45,9 +45,9 @@ namespace WebCore {
 static std::optional<Exception> setMethod(ResourceRequest& request, const String& initMethod)
 {
     if (!isValidHTTPToken(initMethod))
-        return Exception { TypeError, "Method is not a valid HTTP token."_s };
+        return Exception { ExceptionCode::TypeError, "Method is not a valid HTTP token."_s };
     if (isForbiddenMethod(initMethod))
-        return Exception { TypeError, "Method is forbidden."_s };
+        return Exception { ExceptionCode::TypeError, "Method is forbidden."_s };
     request.setHTTPMethod(normalizeHTTPMethod(initMethod));
     return std::nullopt;
 }
@@ -57,10 +57,9 @@ static ExceptionOr<String> computeReferrer(ScriptExecutionContext& context, cons
     if (referrer.isEmpty())
         return "no-referrer"_str;
 
-    // FIXME: Tighten the URL parsing algorithm according https://url.spec.whatwg.org/#concept-url-parser.
-    URL referrerURL = context.completeURL(referrer);
+    URL referrerURL = context.completeURL(referrer, ScriptExecutionContext::ForceUTF8::Yes);
     if (!referrerURL.isValid())
-        return Exception { TypeError, "Referrer is not a valid URL."_s };
+        return Exception { ExceptionCode::TypeError, "Referrer is not a valid URL."_s };
 
     if (referrerURL.protocolIsAbout() && referrerURL.path() == "client"_s)
         return "client"_str;
@@ -74,7 +73,7 @@ static ExceptionOr<String> computeReferrer(ScriptExecutionContext& context, cons
 static std::optional<Exception> buildOptions(FetchOptions& options, ResourceRequest& request, String& referrer, RequestPriority& fetchPriorityHint, ScriptExecutionContext& context, const FetchRequest::Init& init)
 {
     if (!init.window.isUndefinedOrNull() && !init.window.isEmpty())
-        return Exception { TypeError, "Window can only be null."_s };
+        return Exception { ExceptionCode::TypeError, "Window can only be null."_s };
 
     if (init.hasMembers()) {
         if (options.mode == FetchOptions::Mode::Navigate)
@@ -99,7 +98,7 @@ static std::optional<Exception> buildOptions(FetchOptions& options, ResourceRequ
     if (init.mode) {
         options.mode = init.mode.value();
         if (options.mode == FetchOptions::Mode::Navigate)
-            return Exception { TypeError, "Request constructor does not accept navigate fetch mode."_s };
+            return Exception { ExceptionCode::TypeError, "Request constructor does not accept navigate fetch mode."_s };
     }
 
     if (init.credentials)
@@ -108,7 +107,7 @@ static std::optional<Exception> buildOptions(FetchOptions& options, ResourceRequ
     if (init.cache)
         options.cache = init.cache.value();
     if (options.cache == FetchOptions::Cache::OnlyIfCached && options.mode != FetchOptions::Mode::SameOrigin)
-        return Exception { TypeError, "only-if-cached cache option requires fetch mode to be same-origin."_s  };
+        return Exception { ExceptionCode::TypeError, "only-if-cached cache option requires fetch mode to be same-origin."_s  };
 
     if (init.redirect)
         options.redirect = init.redirect.value();
@@ -154,7 +153,7 @@ ExceptionOr<void> FetchRequest::initializeOptions(const Init& init)
     if (m_options.mode == FetchOptions::Mode::NoCors) {
         const String& method = m_request.httpMethod();
         if (method != "GET"_s && method != "POST"_s && method != "HEAD"_s)
-            return Exception { TypeError, "Method must be GET, POST or HEAD in no-cors mode."_s };
+            return Exception { ExceptionCode::TypeError, "Method must be GET, POST or HEAD in no-cors mode."_s };
         m_headers->setGuard(FetchHeaders::Guard::RequestNoCors);
     }
     
@@ -170,17 +169,16 @@ static inline std::optional<Exception> processInvalidSignal(ScriptExecutionConte
         return { };
 
     RELEASE_LOG_ERROR(ResourceLoading, "FetchRequestInit.signal should be undefined, null or an AbortSignal object.");
-    return Exception { TypeError, message };
+    return Exception { ExceptionCode::TypeError, message };
 }
 
 ExceptionOr<void> FetchRequest::initializeWith(const String& url, Init&& init)
 {
     ASSERT(scriptExecutionContext());
 
-    // FIXME: Tighten the URL parsing algorithm according to https://url.spec.whatwg.org/#concept-url-parser.
     URL requestURL = scriptExecutionContext()->completeURL(url, ScriptExecutionContext::ForceUTF8::Yes);
     if (!requestURL.isValid() || requestURL.hasCredentials())
-        return Exception { TypeError, "URL is not valid or contains user credentials."_s };
+        return Exception { ExceptionCode::TypeError, "URL is not valid or contains user credentials."_s };
 
     m_options.mode = Mode::Cors;
     m_options.credentials = Credentials::SameOrigin;
@@ -261,7 +259,7 @@ ExceptionOr<void> FetchRequest::initializeWith(FetchRequest& input, Init&& init)
 ExceptionOr<void> FetchRequest::setBody(FetchBody::Init&& body)
 {
     if (!methodCanHaveBody(m_request))
-        return Exception { TypeError, makeString("Request has method '", m_request.httpMethod(), "' and cannot have a body") };
+        return Exception { ExceptionCode::TypeError, makeString("Request has method '", m_request.httpMethod(), "' and cannot have a body") };
 
     ASSERT(scriptExecutionContext());
     auto result = extractBody(WTFMove(body));
@@ -269,25 +267,25 @@ ExceptionOr<void> FetchRequest::setBody(FetchBody::Init&& body)
         return result;
 
     if (m_options.keepAlive && hasReadableStreamBody())
-        return Exception { TypeError, "Request cannot have a ReadableStream body and keepalive set to true"_s };
+        return Exception { ExceptionCode::TypeError, "Request cannot have a ReadableStream body and keepalive set to true"_s };
     return { };
 }
 
 ExceptionOr<void> FetchRequest::setBody(FetchRequest& request)
 {
     if (request.isDisturbedOrLocked())
-        return Exception { TypeError, "Request input is disturbed or locked."_s };
+        return Exception { ExceptionCode::TypeError, "Request input is disturbed or locked."_s };
 
     if (!request.isBodyNull()) {
         if (!methodCanHaveBody(m_request))
-            return Exception { TypeError, makeString("Request has method '", m_request.httpMethod(), "' and cannot have a body") };
+            return Exception { ExceptionCode::TypeError, makeString("Request has method '", m_request.httpMethod(), "' and cannot have a body") };
         // FIXME: If body has a readable stream, we should pipe it to this new body stream.
         m_body = WTFMove(*request.m_body);
         request.setDisturbed();
     }
 
     if (m_options.keepAlive && hasReadableStreamBody())
-        return Exception { TypeError, "Request cannot have a ReadableStream body and keepalive set to true"_s };
+        return Exception { ExceptionCode::TypeError, "Request cannot have a ReadableStream body and keepalive set to true"_s };
     return { };
 }
 
@@ -346,7 +344,7 @@ ResourceRequest FetchRequest::resourceRequest() const
 ExceptionOr<Ref<FetchRequest>> FetchRequest::clone()
 {
     if (isDisturbedOrLocked())
-        return Exception { TypeError, "Body is disturbed or locked"_s };
+        return Exception { ExceptionCode::TypeError, "Body is disturbed or locked"_s };
 
     auto clone = adoptRef(*new FetchRequest(*scriptExecutionContext(), std::nullopt, FetchHeaders::create(m_headers.get()), ResourceRequest { m_request }, FetchOptions { m_options }, String { m_referrer }));
     clone->suspendIfNeeded();
