@@ -29,23 +29,26 @@
 #if ENABLE(GAMEPAD)
 
 #include "GamepadButton.h"
+#include "GamepadHapticActuator.h"
 #include "PlatformGamepad.h"
 #include <wtf/text/WTFString.h>
 
 namespace WebCore {
 
-Gamepad::Gamepad(const PlatformGamepad& platformGamepad)
+Gamepad::Gamepad(Document* document, const PlatformGamepad& platformGamepad)
     : m_id(platformGamepad.id())
     , m_index(platformGamepad.index())
     , m_connected(true)
     , m_timestamp(platformGamepad.lastUpdateTime())
     , m_mapping(platformGamepad.mapping())
+    , m_supportedEffectTypes(platformGamepad.supportedEffectTypes())
     , m_axes(platformGamepad.axisValues().size(), 0.0)
+    , m_vibrationActuator(platformGamepad.supportedEffectTypes().contains(GamepadHapticEffectType::DualRumble) ? RefPtr { GamepadHapticActuator::create(document, GamepadHapticActuator::Type::DualRumble, *this) } : nullptr)
 {
     unsigned buttonCount = platformGamepad.buttonValues().size();
-    m_buttons.reserveInitialCapacity(buttonCount);
-    for (unsigned i = 0; i < buttonCount; ++i)
-        m_buttons.uncheckedAppend(GamepadButton::create());
+    m_buttons = Vector<Ref<GamepadButton>>(buttonCount, [](size_t) {
+        return GamepadButton::create();
+    });
 }
 
 Gamepad::~Gamepad() = default;
