@@ -27,11 +27,13 @@
 #include "IDBFactory.h"
 
 #include "Document.h"
+#include "FrameDestructionObserverInlines.h"
 #include "IDBBindingUtilities.h"
 #include "IDBConnectionProxy.h"
 #include "IDBDatabaseIdentifier.h"
 #include "IDBKey.h"
 #include "IDBOpenDBRequest.h"
+#include "JSDOMPromiseDeferred.h"
 #include "JSIDBFactory.h"
 #include "Logging.h"
 #include "Page.h"
@@ -54,10 +56,7 @@ static bool shouldThrowSecurityException(ScriptExecutionContext& context)
             return true;
     }
 
-    if (!context.securityOrigin()->canAccessDatabase(nullptr))
-        return true;
-
-    return false;
+    return context.canAccessResource(ScriptExecutionContext::ResourceType::IndexedDB) == ScriptExecutionContext::HasResourceAccess::No;
 }
 
 Ref<IDBFactory> IDBFactory::create(IDBClient::IDBConnectionProxy& connectionProxy)
@@ -77,7 +76,7 @@ ExceptionOr<Ref<IDBOpenDBRequest>> IDBFactory::open(ScriptExecutionContext& cont
     LOG(IndexedDB, "IDBFactory::open");
     
     if (version && !version.value())
-        return Exception { TypeError, "IDBFactory.open() called with a version of 0"_s };
+        return Exception { ExceptionCode::TypeError, "IDBFactory.open() called with a version of 0"_s };
 
     return openInternal(context, name, version.value_or(0));
 }
@@ -85,16 +84,16 @@ ExceptionOr<Ref<IDBOpenDBRequest>> IDBFactory::open(ScriptExecutionContext& cont
 ExceptionOr<Ref<IDBOpenDBRequest>> IDBFactory::openInternal(ScriptExecutionContext& context, const String& name, uint64_t version)
 {
     if (name.isNull())
-        return Exception { TypeError, "IDBFactory.open() called without a database name"_s };
+        return Exception { ExceptionCode::TypeError, "IDBFactory.open() called without a database name"_s };
 
     if (shouldThrowSecurityException(context))
-        return Exception { SecurityError, "IDBFactory.open() called in an invalid security context"_s };
+        return Exception { ExceptionCode::SecurityError, "IDBFactory.open() called in an invalid security context"_s };
 
     ASSERT(context.securityOrigin());
-    bool isTransient = !context.securityOrigin()->canAccessDatabase(&context.topOrigin());
+    bool isTransient = (context.canAccessResource(ScriptExecutionContext::ResourceType::IndexedDB) == ScriptExecutionContext::HasResourceAccess::DefaultForThirdParty);
     IDBDatabaseIdentifier databaseIdentifier(name, SecurityOriginData { context.securityOrigin()->data() }, SecurityOriginData { context.topOrigin().data() }, isTransient);
     if (!databaseIdentifier.isValid())
-        return Exception { TypeError, "IDBFactory.open() called with an invalid security origin"_s };
+        return Exception { ExceptionCode::TypeError, "IDBFactory.open() called with an invalid security origin"_s };
 
     LOG(IndexedDBOperations, "IDB opening database: %s %" PRIu64, name.utf8().data(), version);
 
@@ -106,16 +105,16 @@ ExceptionOr<Ref<IDBOpenDBRequest>> IDBFactory::deleteDatabase(ScriptExecutionCon
     LOG(IndexedDB, "IDBFactory::deleteDatabase - %s", name.utf8().data());
 
     if (name.isNull())
-        return Exception { TypeError, "IDBFactory.deleteDatabase() called without a database name"_s };
+        return Exception { ExceptionCode::TypeError, "IDBFactory.deleteDatabase() called without a database name"_s };
 
     if (shouldThrowSecurityException(context))
-        return Exception { SecurityError, "IDBFactory.deleteDatabase() called in an invalid security context"_s };
+        return Exception { ExceptionCode::SecurityError, "IDBFactory.deleteDatabase() called in an invalid security context"_s };
 
     ASSERT(context.securityOrigin());
-    bool isTransient = !context.securityOrigin()->canAccessDatabase(&context.topOrigin());
+    bool isTransient = (context.canAccessResource(ScriptExecutionContext::ResourceType::IndexedDB) == ScriptExecutionContext::HasResourceAccess::DefaultForThirdParty);
     IDBDatabaseIdentifier databaseIdentifier(name, SecurityOriginData { context.securityOrigin()->data() }, SecurityOriginData { context.topOrigin().data() }, isTransient);
     if (!databaseIdentifier.isValid())
-        return Exception { TypeError, "IDBFactory.deleteDatabase() called with an invalid security origin"_s };
+        return Exception { ExceptionCode::TypeError, "IDBFactory.deleteDatabase() called with an invalid security origin"_s };
 
     LOG(IndexedDBOperations, "IDB deleting database: %s", name.utf8().data());
 
@@ -126,11 +125,11 @@ ExceptionOr<short> IDBFactory::cmp(JSGlobalObject& execState, JSValue firstValue
 {
     auto first = scriptValueToIDBKey(execState, firstValue);
     if (!first->isValid())
-        return Exception { DataError, "Failed to execute 'cmp' on 'IDBFactory': The parameter is not a valid key."_s };
+        return Exception { ExceptionCode::DataError, "Failed to execute 'cmp' on 'IDBFactory': The parameter is not a valid key."_s };
 
     auto second = scriptValueToIDBKey(execState, secondValue);
     if (!second->isValid())
-        return Exception { DataError, "Failed to execute 'cmp' on 'IDBFactory': The parameter is not a valid key."_s };
+        return Exception { ExceptionCode::DataError, "Failed to execute 'cmp' on 'IDBFactory': The parameter is not a valid key."_s };
 
     return first->compare(second.get());
 }
@@ -140,7 +139,7 @@ void IDBFactory::databases(ScriptExecutionContext& context, IDBDatabasesResponse
     LOG(IndexedDB, "IDBFactory::databases");
 
     if (shouldThrowSecurityException(context)) {
-        promise.reject(SecurityError);
+        promise.reject(ExceptionCode::SecurityError);
         return;
     }
 
@@ -148,7 +147,7 @@ void IDBFactory::databases(ScriptExecutionContext& context, IDBDatabasesResponse
 
     m_connectionProxy->getAllDatabaseNamesAndVersions(context, [promise = WTFMove(promise)](auto&& result) mutable {
         if (!result) {
-            promise.reject(Exception { UnknownError });
+            promise.reject(Exception { ExceptionCode::UnknownError });
             return;
         }
 
