@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2020-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,7 +31,8 @@
 #include "CachedResourceLoader.h"
 #include "DOMPromiseProxy.h"
 #include "Document.h"
-#include "ElementChildIterator.h"
+#include "DocumentInlines.h"
+#include "ElementChildIteratorInlines.h"
 #include "ElementInlines.h"
 #include "EventHandler.h"
 #include "EventNames.h"
@@ -47,12 +48,14 @@
 #include "JSHTMLModelElementCamera.h"
 #include "LayoutRect.h"
 #include "LayoutSize.h"
+#include "MIMETypeRegistry.h"
 #include "Model.h"
 #include "ModelPlayer.h"
 #include "ModelPlayerProvider.h"
 #include "MouseEvent.h"
 #include "Page.h"
 #include "PlatformMouseEvent.h"
+#include "RenderBoxInlines.h"
 #include "RenderLayer.h"
 #include "RenderLayerBacking.h"
 #include "RenderLayerModelObject.h"
@@ -64,14 +67,15 @@
 
 namespace WebCore {
 
+using namespace HTMLNames;
+
 WTF_MAKE_ISO_ALLOCATED_IMPL(HTMLModelElement);
 
 HTMLModelElement::HTMLModelElement(const QualifiedName& tagName, Document& document)
-    : HTMLElement(tagName, document)
+    : HTMLElement(tagName, document, CreateHTMLModelElement)
     , ActiveDOMObject(document)
     , m_readyPromise { makeUniqueRef<ReadyPromise>(*this, &HTMLModelElement::readyPromiseResolve) }
 {
-    setHasCustomStyleResolveCallbacks();
 }
 
 HTMLModelElement::~HTMLModelElement()
@@ -97,23 +101,37 @@ RefPtr<Model> HTMLModelElement::model() const
     return m_model;
 }
 
-void HTMLModelElement::sourcesChanged()
+static bool isSupportedModelType(const AtomString& type)
 {
-    if (!document().hasBrowsingContext()) {
-        setSourceURL(URL { });
-        return;
-    }
+    return type.isEmpty() || MIMETypeRegistry::isSupportedModelMIMEType(type);
+}
+
+URL HTMLModelElement::selectModelSource() const
+{
+    // FIXME: This should probably work more like media element resource
+    // selection, where if a <source> element fails to load, an error event
+    // is dispatched to it, and we continue to try subsequent <source>s.
+
+    if (!document().hasBrowsingContext())
+        return { };
+
+    if (auto src = getNonEmptyURLAttribute(srcAttr); src.isValid())
+        return src;
 
     for (auto& element : childrenOfType<HTMLSourceElement>(*this)) {
-        // FIXME: for now we use the first valid URL without looking at the mime-type.
-        auto url = element.getNonEmptyURLAttribute(HTMLNames::srcAttr);
-        if (url.isValid()) {
-            setSourceURL(url);
-            return;
-        }
+        if (!isSupportedModelType(element.attributeWithoutSynchronization(typeAttr)))
+            continue;
+
+        if (auto src = element.getNonEmptyURLAttribute(srcAttr); src.isValid())
+            return src;
     }
 
-    setSourceURL(URL { });
+    return { };
+}
+
+void HTMLModelElement::sourcesChanged()
+{
+    setSourceURL(selectModelSource());
 }
 
 void HTMLModelElement::setSourceURL(const URL& url)
@@ -135,13 +153,13 @@ void HTMLModelElement::setSourceURL(const URL& url)
         m_modelPlayer = nullptr;
 
     if (!m_readyPromise->isFulfilled())
-        m_readyPromise->reject(Exception { AbortError });
+        m_readyPromise->reject(Exception { ExceptionCode::AbortError });
 
     m_readyPromise = makeUniqueRef<ReadyPromise>(*this, &HTMLModelElement::readyPromiseResolve);
     m_shouldCreateModelPlayerUponRendererAttachment = false;
 
     if (m_sourceURL.isEmpty()) {
-        queueTaskToDispatchEvent(*this, TaskSource::DOMManipulation, Event::create(eventNames().errorEvent, Event::CanBubble::No, Event::IsCancelable::No));
+        ActiveDOMObject::queueTaskToDispatchEvent(*this, TaskSource::DOMManipulation, Event::create(eventNames().errorEvent, Event::CanBubble::No, Event::IsCancelable::No));
         return;
     }
 
@@ -155,8 +173,9 @@ void HTMLModelElement::setSourceURL(const URL& url)
 
     auto resource = document().cachedResourceLoader().requestModelResource(WTFMove(request));
     if (!resource.has_value()) {
-        queueTaskToDispatchEvent(*this, TaskSource::DOMManipulation, Event::create(eventNames().errorEvent, Event::CanBubble::No, Event::IsCancelable::No));
-        m_readyPromise->reject(Exception { NetworkError });
+        ActiveDOMObject::queueTaskToDispatchEvent(*this, TaskSource::DOMManipulation, Event::create(eventNames().errorEvent, Event::CanBubble::No, Event::IsCancelable::No));
+        if (!m_readyPromise->isFulfilled())
+            m_readyPromise->reject(Exception { ExceptionCode::NetworkError });
         return;
     }
 
@@ -216,18 +235,19 @@ void HTMLModelElement::notifyFinished(CachedResource& resource, const NetworkLoa
     if (resource.loadFailedOrCanceled()) {
         m_data.reset();
 
-        queueTaskToDispatchEvent(*this, TaskSource::DOMManipulation, Event::create(eventNames().errorEvent, Event::CanBubble::No, Event::IsCancelable::No));
+        ActiveDOMObject::queueTaskToDispatchEvent(*this, TaskSource::DOMManipulation, Event::create(eventNames().errorEvent, Event::CanBubble::No, Event::IsCancelable::No));
 
         invalidateResourceHandleAndUpdateRenderer();
 
-        m_readyPromise->reject(Exception { NetworkError });
+        if (!m_readyPromise->isFulfilled())
+            m_readyPromise->reject(Exception { ExceptionCode::NetworkError });
         return;
     }
 
     m_dataComplete = true;
     m_model = Model::create(m_data.takeAsContiguous().get(), resource.mimeType(), resource.url());
 
-    queueTaskToDispatchEvent(*this, TaskSource::DOMManipulation, Event::create(eventNames().loadEvent, Event::CanBubble::No, Event::IsCancelable::No));
+    ActiveDOMObject::queueTaskToDispatchEvent(*this, TaskSource::DOMManipulation, Event::create(eventNames().loadEvent, Event::CanBubble::No, Event::IsCancelable::No));
 
     invalidateResourceHandleAndUpdateRenderer();
 
@@ -240,7 +260,8 @@ void HTMLModelElement::modelDidChange()
 {
     auto* page = document().page();
     if (!page) {
-        m_readyPromise->reject(Exception { AbortError });
+        if (!m_readyPromise->isFulfilled())
+            m_readyPromise->reject(Exception { ExceptionCode::AbortError });
         return;
     }
 
@@ -255,16 +276,24 @@ void HTMLModelElement::modelDidChange()
 
 void HTMLModelElement::createModelPlayer()
 {
+    if (!m_model)
+        return;
+
+    auto size = contentSize();
+    if (size.isEmpty())
+        return;
+
     ASSERT(document().page());
     m_modelPlayer = document().page()->modelPlayerProvider().createModelPlayer(*this);
     if (!m_modelPlayer) {
-        m_readyPromise->reject(Exception { AbortError });
+        if (!m_readyPromise->isFulfilled())
+            m_readyPromise->reject(Exception { ExceptionCode::AbortError });
         return;
     }
 
     // FIXME: We need to tell the player if the size changes as well, so passing this
     // in with load probably doesn't make sense.
-    m_modelPlayer->load(*m_model, contentSize());
+    m_modelPlayer->load(*m_model, size);
 }
 
 bool HTMLModelElement::usesPlatformLayer() const
@@ -281,7 +310,10 @@ PlatformLayer* HTMLModelElement::platformLayer() const
 
 void HTMLModelElement::sizeMayHaveChanged()
 {
-    m_modelPlayer->sizeDidChange(contentSize());
+    if (m_modelPlayer)
+        m_modelPlayer->sizeDidChange(contentSize());
+    else
+        createModelPlayer();
 }
 
 void HTMLModelElement::didFinishLoading(ModelPlayer& modelPlayer)
@@ -297,25 +329,26 @@ void HTMLModelElement::didFinishLoading(ModelPlayer& modelPlayer)
 void HTMLModelElement::didFailLoading(ModelPlayer& modelPlayer, const ResourceError&)
 {
     ASSERT_UNUSED(modelPlayer, &modelPlayer == m_modelPlayer);
-    m_readyPromise->reject(Exception { AbortError });
+    if (!m_readyPromise->isFulfilled())
+        m_readyPromise->reject(Exception { ExceptionCode::AbortError });
 }
 
-GraphicsLayer::PlatformLayerID HTMLModelElement::platformLayerID()
+PlatformLayerIdentifier HTMLModelElement::platformLayerID()
 {
     auto* page = document().page();
     if (!page)
-        return 0;
+        return { };
 
     if (!is<RenderLayerModelObject>(this->renderer()))
-        return 0;
+        return { };
 
     auto& renderLayerModelObject = downcast<RenderLayerModelObject>(*this->renderer());
     if (!renderLayerModelObject.isComposited() || !renderLayerModelObject.layer() || !renderLayerModelObject.layer()->backing())
-        return 0;
+        return { };
 
-    auto* graphicsLayer = renderLayerModelObject.layer()->backing()->graphicsLayer();
+    RefPtr graphicsLayer = renderLayerModelObject.layer()->backing()->graphicsLayer();
     if (!graphicsLayer)
-        return 0;
+        return { };
 
     return graphicsLayer->contentsLayerIDForModel();
 }
@@ -348,11 +381,15 @@ bool HTMLModelElement::isInteractive() const
     return hasAttributeWithoutSynchronization(HTMLNames::interactiveAttr);
 }
 
-void HTMLModelElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason reason)
+void HTMLModelElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
-    HTMLElement::attributeChanged(name, oldValue, newValue, reason);
-    if (m_modelPlayer && name == HTMLNames::interactiveAttr)
-        m_modelPlayer->setInteractionEnabled(isInteractive());
+    if (name == srcAttr)
+        sourcesChanged();
+    else if (name == interactiveAttr) {
+        if (m_modelPlayer)
+            m_modelPlayer->setInteractionEnabled(isInteractive());
+    } else
+        HTMLElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
 }
 
 void HTMLModelElement::defaultEventHandler(Event& event)
@@ -369,7 +406,7 @@ void HTMLModelElement::defaultEventHandler(Event& event)
     ASSERT(is<MouseEvent>(event));
     auto& mouseEvent = downcast<MouseEvent>(event);
 
-    if (mouseEvent.button() != LeftButton)
+    if (mouseEvent.button() != MouseButton::Left)
         return;
 
     if (type == eventNames().mousedownEvent && !m_isDragging && !event.defaultPrevented() && isInteractive())
@@ -435,7 +472,7 @@ void HTMLModelElement::dragDidEnd(MouseEvent& event)
 void HTMLModelElement::getCamera(CameraPromise&& promise)
 {
     if (!m_modelPlayer) {
-        promise.reject(Exception { AbortError });
+        promise.reject(Exception { ExceptionCode::AbortError });
         return;
     }
 
@@ -450,7 +487,7 @@ void HTMLModelElement::getCamera(CameraPromise&& promise)
 void HTMLModelElement::setCamera(HTMLModelElementCamera camera, DOMPromiseDeferred<void>&& promise)
 {
     if (!m_modelPlayer) {
-        promise.reject(Exception { AbortError });
+        promise.reject(Exception { ExceptionCode::AbortError });
         return;
     }
 
@@ -653,6 +690,39 @@ LayoutSize HTMLModelElement::contentSize() const
     return downcast<RenderReplaced>(*renderer()).replacedContentRect().size();
 }
 
+#if ENABLE(ARKIT_INLINE_PREVIEW_MAC)
+String HTMLModelElement::inlinePreviewUUIDForTesting() const
+{
+    if (!m_modelPlayer)
+        return emptyString();
+    return m_modelPlayer->inlinePreviewUUIDForTesting();
+}
+#endif
+
+void HTMLModelElement::collectPresentationalHintsForAttribute(const QualifiedName& name, const AtomString& value, MutableStyleProperties& style)
+{
+    if (name == widthAttr) {
+        addHTMLLengthToStyle(style, CSSPropertyWidth, value);
+        applyAspectRatioFromWidthAndHeightAttributesToStyle(value, attributeWithoutSynchronization(heightAttr), style);
+    } else if (name == heightAttr) {
+        addHTMLLengthToStyle(style, CSSPropertyHeight, value);
+        applyAspectRatioFromWidthAndHeightAttributesToStyle(attributeWithoutSynchronization(widthAttr), value, style);
+    } else
+        HTMLElement::collectPresentationalHintsForAttribute(name, value, style);
+}
+
+bool HTMLModelElement::hasPresentationalHintsForAttribute(const QualifiedName& name) const
+{
+    if (name == widthAttr || name == heightAttr)
+        return true;
+    return HTMLElement::hasPresentationalHintsForAttribute(name);
+}
+
+bool HTMLModelElement::isURLAttribute(const Attribute& attribute) const
+{
+    return attribute.name() == srcAttr || HTMLElement::isURLAttribute(attribute);
+}
+
 }
 
 #endif // ENABLE(MODEL_ELEMENT)
