@@ -30,9 +30,12 @@
 
 #include "Document.h"
 #include "PaymentRequest.h"
+#include <wtf/IsoMallocInlines.h>
 
 namespace WebCore {
 
+WTF_MAKE_ISO_ALLOCATED_IMPL(MerchantValidationEvent);
+
 Ref<MerchantValidationEvent> MerchantValidationEvent::create(const AtomString& type, const String& methodName, URL&& validationURL)
 {
     return adoptRef(*new MerchantValidationEvent(type, methodName, WTFMove(validationURL)));
@@ -42,13 +45,13 @@ ExceptionOr<Ref<MerchantValidationEvent>> MerchantValidationEvent::create(Docume
 {
     URL validationURL { document.url(), eventInit.validationURL };
     if (!validationURL.isValid())
-        return Exception { TypeError };
+        return Exception { ExceptionCode::TypeError };
 
     auto methodName = WTFMove(eventInit.methodName);
     if (!methodName.isEmpty()) {
         auto validatedMethodName = convertAndValidatePaymentMethodIdentifier(methodName);
         if (!validatedMethodName)
-            return Exception { RangeError, makeString('"', methodName, "\" is an invalid payment method identifier.") };
+            return Exception { ExceptionCode::RangeError, makeString('"', methodName, "\" is an invalid payment method identifier.") };
     }
 
     return adoptRef(*new MerchantValidationEvent(type, WTFMove(methodName), WTFMove(validationURL), WTFMove(eventInit)));
@@ -80,10 +83,10 @@ EventInterface MerchantValidationEvent::eventInterface() const
 ExceptionOr<void> MerchantValidationEvent::complete(Ref<DOMPromise>&& merchantSessionPromise)
 {
     if (!isTrusted())
-        return Exception { InvalidStateError };
+        return Exception { ExceptionCode::InvalidStateError };
 
     if (m_isCompleted)
-        return Exception { InvalidStateError };
+        return Exception { ExceptionCode::InvalidStateError };
 
     auto exception = downcast<PaymentRequest>(target())->completeMerchantValidation(*this, WTFMove(merchantSessionPromise));
     if (exception.hasException())
