@@ -62,29 +62,29 @@ struct ConnectionInfo {
 static ExceptionOr<ConnectionInfo> connectionInfo(NavigatorBase* navigator)
 {
     if (!navigator)
-        return Exception { InvalidStateError, "Navigator does not exist"_s };
+        return Exception { ExceptionCode::InvalidStateError, "Navigator does not exist"_s };
 
-    auto context = navigator->scriptExecutionContext();
+    RefPtr context = navigator->scriptExecutionContext();
     if (!context)
-        return Exception { InvalidStateError, "Context is invalid"_s };
+        return Exception { ExceptionCode::InvalidStateError, "Context is invalid"_s };
 
     if (context->canAccessResource(ScriptExecutionContext::ResourceType::StorageManager) == ScriptExecutionContext::HasResourceAccess::No)
-        return Exception { TypeError, "Context not access storage"_s };
+        return Exception { ExceptionCode::TypeError, "Context not access storage"_s };
 
-    auto* origin = context->securityOrigin();
+    RefPtr origin = context->securityOrigin();
     ASSERT(origin);
 
     if (is<Document>(context)) {
-        if (auto* connection = downcast<Document>(context)->storageConnection())
+        if (RefPtr connection = downcast<Document>(context)->storageConnection())
             return ConnectionInfo { *connection, { context->topOrigin().data(), origin->data() } };
 
-        return Exception { InvalidStateError, "Connection is invalid"_s };
+        return Exception { ExceptionCode::InvalidStateError, "Connection is invalid"_s };
     }
 
     if (is<WorkerGlobalScope>(context))
         return ConnectionInfo { downcast<WorkerGlobalScope>(context)->storageConnection(), { context->topOrigin().data(), origin->data() } };
 
-    return Exception { NotSupportedError };
+    return Exception { ExceptionCode::NotSupportedError };
 }
 
 void StorageManager::persisted(DOMPromiseDeferred<IDLBoolean>&& promise)
@@ -118,7 +118,7 @@ void StorageManager::estimate(DOMPromiseDeferred<IDLDictionary<StorageEstimate>>
         return promise.reject(connectionInfoOrException.releaseException());
 
     auto connectionInfo = connectionInfoOrException.releaseReturnValue();
-    connectionInfo.connection.getEstimate(WTFMove(connectionInfo.origin), [promise = WTFMove(promise)](auto&& result) mutable {
+    connectionInfo.connection.getEstimate(WTFMove(connectionInfo.origin), [promise = WTFMove(promise)](ExceptionOr<StorageEstimate>&& result) mutable {
         promise.settle(WTFMove(result));
     });
 }
@@ -135,10 +135,10 @@ void StorageManager::fileSystemAccessGetDirectory(DOMPromiseDeferred<IDLInterfac
             return promise.reject(result.releaseException());
 
         auto [identifier, connection] = result.releaseReturnValue();
-        auto* context = weakNavigator ? weakNavigator->scriptExecutionContext() : nullptr;
+        RefPtr context = weakNavigator ? weakNavigator->scriptExecutionContext() : nullptr;
         if (!context) {
             connection->closeHandle(identifier);
-            return promise.reject(Exception { InvalidStateError, "Context has stopped"_s });
+            return promise.reject(Exception { ExceptionCode::InvalidStateError, "Context has stopped"_s });
         }
 
         promise.resolve(FileSystemDirectoryHandle::create(*context, { }, identifier, Ref { *connection }));
