@@ -34,6 +34,7 @@
 #include "AudioNodeOptions.h"
 #include "AudioNodeOutput.h"
 #include "AudioParam.h"
+#include "ContextDestructionObserverInlines.h"
 #include "Logging.h"
 #include <wtf/Atomics.h>
 #include <wtf/IsoMallocInlines.h>
@@ -189,20 +190,20 @@ ExceptionOr<void> AudioNode::connect(AudioNode& destination, unsigned outputInde
     
     // Sanity check input and output indices.
     if (outputIndex >= numberOfOutputs())
-        return Exception { IndexSizeError, "Output index exceeds number of outputs"_s };
+        return Exception { ExceptionCode::IndexSizeError, "Output index exceeds number of outputs"_s };
 
     if (inputIndex >= destination.numberOfInputs())
-        return Exception { IndexSizeError, "Input index exceeds number of inputs"_s };
+        return Exception { ExceptionCode::IndexSizeError, "Input index exceeds number of inputs"_s };
 
     auto& context = this->context();
     if (&context != &destination.context())
-        return Exception { InvalidAccessError, "Source and destination nodes belong to different audio contexts"_s };
+        return Exception { ExceptionCode::InvalidAccessError, "Source and destination nodes belong to different audio contexts"_s };
 
     auto* input = destination.input(inputIndex);
     auto* output = this->output(outputIndex);
 
     if (!output->numberOfChannels())
-        return Exception { InvalidAccessError, "Node has zero output channels"_s };
+        return Exception { ExceptionCode::InvalidAccessError, "Node has zero output channels"_s };
 
     if (is<AudioContext>(context) && &destination == &context.destination() && !downcast<AudioContext>(context).destination().isConnected())
         downcast<AudioContext>(context).defaultDestinationWillBecomeConnected();
@@ -223,10 +224,10 @@ ExceptionOr<void> AudioNode::connect(AudioParam& param, unsigned outputIndex)
     INFO_LOG(LOGIDENTIFIER, param.name(), ", output = ", outputIndex);
 
     if (outputIndex >= numberOfOutputs())
-        return Exception { IndexSizeError, "Output index exceeds number of outputs"_s };
+        return Exception { ExceptionCode::IndexSizeError, "Output index exceeds number of outputs"_s };
 
     if (&context() != param.context())
-        return Exception { InvalidAccessError, "Node and AudioParam belong to different audio contexts"_s };
+        return Exception { ExceptionCode::InvalidAccessError, "Node and AudioParam belong to different audio contexts"_s };
 
     auto* output = this->output(outputIndex);
     param.connect(output);
@@ -254,7 +255,7 @@ ExceptionOr<void> AudioNode::disconnect(unsigned outputIndex)
     Locker locker { context().graphLock() };
 
     if (outputIndex >= numberOfOutputs())
-        return Exception { IndexSizeError, "output index is out of bounds"_s };
+        return Exception { ExceptionCode::IndexSizeError, "output index is out of bounds"_s };
 
     auto* output = this->output(outputIndex);
     INFO_LOG(LOGIDENTIFIER, output->node()->nodeType());
@@ -283,7 +284,7 @@ ExceptionOr<void> AudioNode::disconnect(AudioNode& destinationNode)
     }
 
     if (!didDisconnection)
-        return Exception { InvalidAccessError, "The given destination is not connected"_s };
+        return Exception { ExceptionCode::InvalidAccessError, "The given destination is not connected"_s };
 
     updatePullStatus();
     return { };
@@ -295,7 +296,7 @@ ExceptionOr<void> AudioNode::disconnect(AudioNode& destinationNode, unsigned out
     Locker locker { context().graphLock() };
 
     if (outputIndex >= numberOfOutputs())
-        return Exception { IndexSizeError, "output index is out of bounds"_s };
+        return Exception { ExceptionCode::IndexSizeError, "output index is out of bounds"_s };
 
     bool didDisconnection = false;
     auto* output = this->output(outputIndex);
@@ -308,7 +309,7 @@ ExceptionOr<void> AudioNode::disconnect(AudioNode& destinationNode, unsigned out
     }
 
     if (!didDisconnection)
-        return Exception { InvalidAccessError, "The given destination is not connected"_s };
+        return Exception { ExceptionCode::InvalidAccessError, "The given destination is not connected"_s };
 
     updatePullStatus();
     return { };
@@ -320,15 +321,15 @@ ExceptionOr<void> AudioNode::disconnect(AudioNode& destinationNode, unsigned out
     Locker locker { context().graphLock() };
 
     if (outputIndex >= numberOfOutputs())
-        return Exception { IndexSizeError, "output index is out of bounds"_s };
+        return Exception { ExceptionCode::IndexSizeError, "output index is out of bounds"_s };
 
     if (inputIndex >= destinationNode.numberOfInputs())
-        return Exception { IndexSizeError, "input index is out of bounds"_s };
+        return Exception { ExceptionCode::IndexSizeError, "input index is out of bounds"_s };
 
     auto* output = this->output(outputIndex);
     auto* input = destinationNode.input(inputIndex);
     if (!output->isConnectedTo(*input))
-        return Exception { InvalidAccessError, "The given destination is not connected"_s };
+        return Exception { ExceptionCode::InvalidAccessError, "The given destination is not connected"_s };
 
     input->disconnect(output);
 
@@ -351,7 +352,7 @@ ExceptionOr<void> AudioNode::disconnect(AudioParam& destinationParam)
     }
 
     if (!didDisconnection)
-        return Exception { InvalidAccessError, "The given destination is not connected"_s };
+        return Exception { ExceptionCode::InvalidAccessError, "The given destination is not connected"_s };
 
     updatePullStatus();
     return { };
@@ -363,11 +364,11 @@ ExceptionOr<void> AudioNode::disconnect(AudioParam& destinationParam, unsigned o
     Locker locker { context().graphLock() };
 
     if (outputIndex >= numberOfOutputs())
-        return Exception { IndexSizeError, "output index is out of bounds"_s };
+        return Exception { ExceptionCode::IndexSizeError, "output index is out of bounds"_s };
 
     auto* output = this->output(outputIndex);
     if (!output->isConnectedTo(destinationParam))
-        return Exception { InvalidAccessError, "The given destination is not connected"_s };
+        return Exception { ExceptionCode::InvalidAccessError, "The given destination is not connected"_s };
 
     destinationParam.disconnect(output);
 
@@ -388,10 +389,10 @@ ExceptionOr<void> AudioNode::setChannelCount(unsigned channelCount)
     ALWAYS_LOG(LOGIDENTIFIER, channelCount);
 
     if (!channelCount)
-        return Exception { NotSupportedError, "Channel count cannot be 0"_s };
+        return Exception { ExceptionCode::NotSupportedError, "Channel count cannot be 0"_s };
     
     if (channelCount > AudioContext::maxNumberOfChannels)
-        return Exception { NotSupportedError, "Channel count exceeds maximum limit"_s };
+        return Exception { ExceptionCode::NotSupportedError, "Channel count exceeds maximum limit"_s };
 
     if (m_channelCount == channelCount)
         return { };
