@@ -67,10 +67,20 @@ WebCodecsVideoEncoder::~WebCodecsVideoEncoder()
 {
 }
 
-static bool isValidEncoderConfig(const WebCodecsVideoEncoderConfig& config, const Settings::Values& settings)
+static bool isSupportedEncoderCodec(const String& codec, const Settings::Values& settings)
 {
-    // FIXME: Check codec more accurately.
-    if (!config.codec.startsWith("vp8"_s) && !config.codec.startsWith("vp09."_s) && !config.codec.startsWith("avc1."_s) && !(config.codec.startsWith("hev1."_s) && settings.webCodecsHEVCEnabled) && !(config.codec.startsWith("hvc1."_s) && settings.webCodecsHEVCEnabled) && !(config.codec.startsWith("av01."_s) && settings.webCodecsAV1Enabled))
+    return codec.startsWith("vp8"_s) || codec.startsWith("vp09.00"_s) || codec.startsWith("avc1."_s)
+#if ENABLE(WEB_RTC)
+        || (codec.startsWith("vp09.02"_s) && settings.webRTCVP9Profile2CodecEnabled)
+#endif
+        || (codec.startsWith("hev1."_s) && settings.webCodecsHEVCEnabled)
+        || (codec.startsWith("hvc1."_s) && settings.webCodecsHEVCEnabled)
+        || (codec.startsWith("av01.0"_s) && settings.webCodecsAV1Enabled);
+}
+
+static bool isValidEncoderConfig(const WebCodecsVideoEncoderConfig& config)
+{
+    if (StringView(config.codec).trim(isASCIIWhitespace<UChar>).isEmpty())
         return false;
 
     if (!config.width || !config.height)
@@ -84,7 +94,7 @@ static bool isValidEncoderConfig(const WebCodecsVideoEncoderConfig& config, cons
 static ExceptionOr<VideoEncoder::Config> createVideoEncoderConfig(const WebCodecsVideoEncoderConfig& config)
 {
     if (config.alpha == WebCodecsAlphaOption::Keep)
-        return Exception { NotSupportedError, "Alpha keep is not supported"_s };
+        return Exception { ExceptionCode::NotSupportedError, "Alpha keep is not supported"_s };
 
     VideoEncoder::ScalabilityMode scalabilityMode = VideoEncoder::ScalabilityMode::L1T1;
     if (!config.scalabilityMode.isNull()) {
@@ -93,11 +103,11 @@ static ExceptionOr<VideoEncoder::Config> createVideoEncoderConfig(const WebCodec
         else if (config.scalabilityMode == "L1T2"_s)
             scalabilityMode = VideoEncoder::ScalabilityMode::L1T2;
         else if (config.scalabilityMode != "L1T1"_s)
-            return Exception { TypeError, "Scalabilty mode is not supported"_s };
+            return Exception { ExceptionCode::TypeError, "Scalabilty mode is not supported"_s };
     }
 
     if (config.codec.startsWith("avc1."_s) && (!!(config.width % 2) || !!(config.height % 2)))
-        return Exception { TypeError, "H264 only supports even sized frames"_s };
+        return Exception { ExceptionCode::TypeError, "H264 only supports even sized frames"_s };
 
     bool useAnnexB = config.avc && config.avc->format == AvcBitstreamFormat::Annexb;
     return VideoEncoder::Config { config.width, config.height, useAnnexB, config.bitrate.value_or(0), config.framerate.value_or(0), config.latencyMode == LatencyMode::Realtime, scalabilityMode };
@@ -105,11 +115,11 @@ static ExceptionOr<VideoEncoder::Config> createVideoEncoderConfig(const WebCodec
 
 ExceptionOr<void> WebCodecsVideoEncoder::configure(ScriptExecutionContext& context, WebCodecsVideoEncoderConfig&& config)
 {
-    if (!isValidEncoderConfig(config, context.settingsValues()))
-        return Exception { TypeError, "Config is invalid"_s };
+    if (!isValidEncoderConfig(config))
+        return Exception { ExceptionCode::TypeError, "Config is invalid"_s };
 
     if (m_state == WebCodecsCodecState::Closed || !scriptExecutionContext())
-        return Exception { InvalidStateError, "VideoEncoder is closed"_s };
+        return Exception { ExceptionCode::InvalidStateError, "VideoEncoder is closed"_s };
 
     m_state = WebCodecsCodecState::Configured;
     m_isKeyChunkRequired = true;
@@ -117,62 +127,63 @@ ExceptionOr<void> WebCodecsVideoEncoder::configure(ScriptExecutionContext& conte
     if (m_internalEncoder) {
         queueControlMessageAndProcess([this, config]() mutable {
             m_isMessageQueueBlocked = true;
-            m_internalEncoder->flush([this, weakedThis = WeakPtr { *this }, config = WTFMove(config)]() mutable {
-                if (!weakedThis)
+            m_internalEncoder->flush([weakThis = ThreadSafeWeakPtr { *this }, config = WTFMove(config)]() mutable {
+                RefPtr protectedThis = weakThis.get();
+                if (!protectedThis)
                     return;
 
-                if (m_state == WebCodecsCodecState::Closed || !scriptExecutionContext())
+                if (protectedThis->m_state == WebCodecsCodecState::Closed || !protectedThis->scriptExecutionContext())
                     return;
 
-                m_isMessageQueueBlocked = false;
-                processControlMessageQueue();
+                protectedThis->m_isMessageQueueBlocked = false;
+                protectedThis->processControlMessageQueue();
             });
         });
     }
 
-    queueControlMessageAndProcess([this, config = WTFMove(config), identifier = scriptExecutionContext()->identifier()]() mutable {
+    bool isSupportedCodec = isSupportedEncoderCodec(config.codec, context.settingsValues());
+    queueControlMessageAndProcess([this, config = WTFMove(config), isSupportedCodec, identifier = scriptExecutionContext()->identifier()]() mutable {
         m_isMessageQueueBlocked = true;
-        m_baseConfiguration = config;
+        VideoEncoder::PostTaskCallback postTaskCallback = [weakThis = ThreadSafeWeakPtr { *this }, identifier](auto&& task) {
+            ScriptExecutionContext::postTaskTo(identifier, [weakThis, task = WTFMove(task)](auto&) mutable {
+                RefPtr protectedThis = weakThis.get();
+                if (!protectedThis)
+                    return;
+                protectedThis->queueTaskKeepingObjectAlive(*protectedThis, TaskSource::MediaElement, WTFMove(task));
+            });
+        };
 
-        VideoEncoder::PostTaskCallback postTaskCallback;
-        if (isMainThread()) {
-            postTaskCallback = [](auto&& task) {
-                callOnMainThread(WTFMove(task));
-            };
-        } else {
-            postTaskCallback = [identifier](auto&& task) {
-                ScriptExecutionContext::postTaskTo(identifier, [task = WTFMove(task)](auto&) mutable {
-                    task();
-                });
-            };
+        if (!isSupportedCodec) {
+            postTaskCallback([this] {
+                closeEncoder(Exception { ExceptionCode::NotSupportedError, "Codec is not supported"_s });
+            });
+            return;
         }
 
         auto encoderConfig = createVideoEncoderConfig(config);
         if (encoderConfig.hasException()) {
-            closeEncoder(Exception { NotSupportedError, encoderConfig.releaseException().message() });
+            postTaskCallback([this, message = encoderConfig.releaseException().message()]() mutable {
+                closeEncoder(Exception { ExceptionCode::NotSupportedError, WTFMove(message) });
+            });
             return;
         }
 
-        VideoEncoder::create(config.codec, encoderConfig.releaseReturnValue(), [this, weakThis = WeakPtr { *this }](auto&& result) {
-            if (!weakThis)
-                return;
+        m_baseConfiguration = config;
 
+        VideoEncoder::create(config.codec, encoderConfig.releaseReturnValue(), [this](auto&& result) {
             if (!result.has_value()) {
-                closeEncoder(Exception { NotSupportedError, WTFMove(result.error()) });
+                closeEncoder(Exception { ExceptionCode::NotSupportedError, WTFMove(result.error()) });
                 return;
             }
             setInternalEncoder(WTFMove(result.value()));
-            m_isMessageQueueBlocked = false;
             m_hasNewActiveConfiguration = true;
+            m_isMessageQueueBlocked = false;
             processControlMessageQueue();
-        }, [this, weakThis = WeakPtr { *this }](auto&& configuration) {
-            if (!weakThis)
-                return;
-
+        }, [this](VideoEncoder::ActiveConfiguration&& configuration) {
             m_activeConfiguration = WTFMove(configuration);
             m_hasNewActiveConfiguration = true;
-        }, [this, weakThis = WeakPtr { *this }](auto&& result) {
-            if (!weakThis || m_state != WebCodecsCodecState::Configured)
+        }, [this](auto&& result) {
+            if (m_state != WebCodecsCodecState::Configured)
                 return;
 
             RefPtr<JSC::ArrayBuffer> buffer = JSC::ArrayBuffer::create(result.data.data(), result.data.size());
@@ -229,27 +240,28 @@ ExceptionOr<void> WebCodecsVideoEncoder::encode(Ref<WebCodecsVideoFrame>&& frame
     auto internalFrame = frame->internalFrame();
     if (!internalFrame) {
         ASSERT(frame->isDetached());
-        return Exception { TypeError, "VideoFrame is detached"_s };
+        return Exception { ExceptionCode::TypeError, "VideoFrame is detached"_s };
     }
     ASSERT(!frame->isDetached());
 
     if (m_state != WebCodecsCodecState::Configured)
-        return Exception { InvalidStateError, "VideoEncoder is not configured"_s };
+        return Exception { ExceptionCode::InvalidStateError, "VideoEncoder is not configured"_s };
 
     ++m_encodeQueueSize;
     queueControlMessageAndProcess([this, internalFrame = internalFrame.releaseNonNull(), timestamp = frame->timestamp(), duration = frame->duration(), options = WTFMove(options)]() mutable {
         ++m_beingEncodedQueueSize;
         --m_encodeQueueSize;
         scheduleDequeueEvent();
-        m_internalEncoder->encode({ WTFMove(internalFrame), timestamp, duration }, options.keyFrame, [this, weakedThis = WeakPtr { *this }](auto&& result) {
-            if (!weakedThis)
+        m_internalEncoder->encode({ WTFMove(internalFrame), timestamp, duration }, options.keyFrame, [weakThis = ThreadSafeWeakPtr { *this }](String&& result) {
+            RefPtr protectedThis = weakThis.get();
+            if (!protectedThis)
                 return;
 
-            --m_beingEncodedQueueSize;
+            --(protectedThis->m_beingEncodedQueueSize);
             if (!result.isNull()) {
-                if (auto* context = scriptExecutionContext())
+                if (RefPtr context = protectedThis->scriptExecutionContext())
                     context->addConsoleMessage(MessageSource::JS, MessageLevel::Error, makeString("VideoEncoder encode failed: ", result));
-                closeEncoder(Exception { EncodingError, WTFMove(result) });
+                protectedThis->closeEncoder(Exception { ExceptionCode::EncodingError, WTFMove(result) });
                 return;
             }
         });
@@ -260,37 +272,43 @@ ExceptionOr<void> WebCodecsVideoEncoder::encode(Ref<WebCodecsVideoFrame>&& frame
 void WebCodecsVideoEncoder::flush(Ref<DeferredPromise>&& promise)
 {
     if (m_state != WebCodecsCodecState::Configured) {
-        promise->reject(Exception { InvalidStateError, "VideoEncoder is not configured"_s });
+        promise->reject(Exception { ExceptionCode::InvalidStateError, "VideoEncoder is not configured"_s });
         return;
     }
 
     m_pendingFlushPromises.append(promise.copyRef());
     m_isFlushing = true;
     queueControlMessageAndProcess([this, clearFlushPromiseCount = m_clearFlushPromiseCount]() mutable {
-        m_internalEncoder->flush([this, weakThis = WeakPtr { *this }, clearFlushPromiseCount] {
-            if (!weakThis || clearFlushPromiseCount != m_clearFlushPromiseCount)
+        m_internalEncoder->flush([weakThis = ThreadSafeWeakPtr { *this }, clearFlushPromiseCount] {
+            RefPtr protectedThis = weakThis.get();
+            if (!protectedThis || clearFlushPromiseCount != protectedThis->m_clearFlushPromiseCount)
                 return;
 
-            m_pendingFlushPromises.takeFirst()->resolve();
-            m_isFlushing = !m_pendingFlushPromises.isEmpty();
+            protectedThis->m_pendingFlushPromises.takeFirst()->resolve();
+            protectedThis->m_isFlushing = !protectedThis->m_pendingFlushPromises.isEmpty();
         });
     });
 }
 
 ExceptionOr<void> WebCodecsVideoEncoder::reset()
 {
-    return resetEncoder(Exception { AbortError, "Reset called"_s });
+    return resetEncoder(Exception { ExceptionCode::AbortError, "Reset called"_s });
 }
 
 ExceptionOr<void> WebCodecsVideoEncoder::close()
 {
-    return closeEncoder(Exception { AbortError, "Close called"_s });
+    return closeEncoder(Exception { ExceptionCode::AbortError, "Close called"_s });
 }
 
 void WebCodecsVideoEncoder::isConfigSupported(ScriptExecutionContext& context, WebCodecsVideoEncoderConfig&& config, Ref<DeferredPromise>&& promise)
 {
-    if (!isValidEncoderConfig(config, context.settingsValues())) {
-        promise->reject(Exception { TypeError, "Config is not valid"_s });
+    if (!isValidEncoderConfig(config)) {
+        promise->reject(Exception { ExceptionCode::TypeError, "Config is not valid"_s });
+        return;
+    }
+
+    if (!isSupportedEncoderCodec(config.codec, context.settingsValues())) {
+        promise->template resolve<IDLDictionary<WebCodecsVideoEncoderSupport>>(WebCodecsVideoEncoderSupport { false, WTFMove(config) });
         return;
     }
 
@@ -322,18 +340,16 @@ ExceptionOr<void> WebCodecsVideoEncoder::closeEncoder(Exception&& exception)
         return result;
     m_state = WebCodecsCodecState::Closed;
     m_internalEncoder = nullptr;
-    if (exception.code() != AbortError) {
-        queueTaskKeepingObjectAlive(*this, TaskSource::MediaElement, [this, exception = WTFMove(exception)]() mutable {
-            m_error->handleEvent(DOMException::create(WTFMove(exception)));
-        });
-    }
+    if (exception.code() != ExceptionCode::AbortError)
+        m_error->handleEvent(DOMException::create(WTFMove(exception)));
+
     return { };
 }
 
 ExceptionOr<void> WebCodecsVideoEncoder::resetEncoder(const Exception& exception)
 {
     if (m_state == WebCodecsCodecState::Closed)
-        return Exception { InvalidStateError, "VideoEncoder is closed"_s };
+        return Exception { ExceptionCode::InvalidStateError, "VideoEncoder is closed"_s };
 
     m_state = WebCodecsCodecState::Unconfigured;
     if (m_internalEncoder)
@@ -342,9 +358,6 @@ ExceptionOr<void> WebCodecsVideoEncoder::resetEncoder(const Exception& exception
     if (m_encodeQueueSize) {
         m_encodeQueueSize = 0;
         scheduleDequeueEvent();
-        queueTaskKeepingObjectAlive(*this, TaskSource::MediaElement, [this, exception = Exception { exception }]() mutable {
-            m_error->handleEvent(DOMException::create(WTFMove(exception)));
-        });
     }
     ++m_clearFlushPromiseCount;
     while (!m_pendingFlushPromises.isEmpty())
