@@ -33,6 +33,7 @@
 #include "TextIterator.h"
 
 namespace WebCore {
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(AXTextMarker);
 
 TextMarkerData::TextMarkerData(AXObjectCache& cache, Node* nodeParam, const VisiblePosition& visiblePosition, int charStart, int charOffset, bool ignoredParam)
 {
@@ -147,14 +148,22 @@ AXTextMarker::operator CharacterOffset() const
     if (isIgnored() || isNull())
         return { };
 
-    setNodeIfNeeded();
+    WeakPtr cache = AXTreeStore<AXObjectCache>::axObjectCacheForID(m_data.axTreeID());
+    if (!cache)
+        return { };
+
+    if (m_data.node) {
+        // Make sure that this node is still in cache->m_textMarkerNodes. Since this method can be called as a result of a dispatch from the AX thread, the Node may have gone away in a previous main loop cycle.
+        if (!cache->isNodeInUse(m_data.node))
+            return { };
+    } else
+        setNodeIfNeeded();
+
     CharacterOffset result(m_data.node, m_data.characterStart, m_data.characterOffset);
     // When we are at a line wrap and the VisiblePosition is upstream, it means the text marker is at the end of the previous line.
     // We use the previous CharacterOffset so that it will match the Range.
-    if (m_data.affinity == Affinity::Upstream) {
-        if (WeakPtr cache = AXTreeStore<AXObjectCache>::axObjectCacheForID(m_data.axTreeID()))
-            return cache->previousCharacterOffset(result, false);
-    }
+    if (m_data.affinity == Affinity::Upstream)
+        return cache->previousCharacterOffset(result, false);
     return result;
 }
 
@@ -200,7 +209,6 @@ RefPtr<AXCoreObject> AXTextMarker::object() const
     return tree ? tree->objectForID(objectID()) : nullptr;
 }
 
-#if ENABLE(TREE_DEBUGGING)
 String AXTextMarker::debugDescription() const
 {
     auto separator = ", ";
@@ -209,17 +217,16 @@ String AXTextMarker::debugDescription() const
         "treeID ", treeID().loggingString()
         , separator, "objectID ", objectID().loggingString()
         , separator, "role ", object ? accessibilityRoleToString(object->roleValue()) : String("no object"_s)
-        , separator, isMainThread() ? node()->debugDescription()
+        , isIgnored() ? makeString(separator, "ignored") : ""_s
+        , separator, isMainThread() && node() ? node()->debugDescription()
             : makeString("node 0x", hex(reinterpret_cast<uintptr_t>(m_data.node)))
+        , separator, "anchor ", m_data.anchorType
+        , separator, "affinity ", m_data.affinity
         , separator, "offset ", m_data.offset
-        , separator, "AnchorType ", m_data.anchorType
-        , separator, "Affinity ", m_data.affinity
         , separator, "characterStart ", m_data.characterStart
         , separator, "characterOffset ", m_data.characterOffset
-        , separator, "isIgnored ", isIgnored()
     );
 }
-#endif
 
 AXTextMarkerRange::AXTextMarkerRange(const VisiblePositionRange& range)
     : m_start(range.start)
@@ -292,6 +299,45 @@ std::optional<CharacterRange> AXTextMarkerRange::characterRange() const
     return { { m_start.m_data.characterOffset, m_end.m_data.characterOffset - m_start.m_data.characterOffset } };
 }
 
+std::optional<AXTextMarkerRange> AXTextMarkerRange::intersectionWith(const AXTextMarkerRange& other) const
+{
+    if (UNLIKELY(m_start.m_data.treeID != m_end.m_data.treeID
+        || other.m_start.m_data.treeID != other.m_end.m_data.treeID
+        || m_start.m_data.treeID != other.m_start.m_data.treeID)) {
+        return std::nullopt;
+    }
+
+    // Fast path: both ranges span one object
+    if (m_start.m_data.objectID == m_end.m_data.objectID
+        && other.m_start.m_data.objectID == other.m_end.m_data.objectID) {
+        if (m_start.m_data.objectID != other.m_start.m_data.objectID)
+            return std::nullopt;
+
+        unsigned startOffset = std::max(m_start.m_data.characterOffset, other.m_start.m_data.characterOffset);
+        unsigned endOffset = std::min(m_end.m_data.characterOffset, other.m_end.m_data.characterOffset);
+
+        if (startOffset > endOffset)
+            return std::nullopt;
+
+        auto startMarker = AXTextMarker({ m_start.treeID(), m_start.objectID(), nullptr, startOffset, Position::PositionIsOffsetInAnchor, Affinity::Downstream, 0, startOffset });
+        auto endMarker = AXTextMarker({ m_start.treeID(), m_start.objectID(), nullptr, endOffset, Position::PositionIsOffsetInAnchor, Affinity::Downstream, 0, endOffset });
+        return { { startMarker, endMarker } };
+    }
+
+    return Accessibility::retrieveValueFromMainThread<std::optional<AXTextMarkerRange>>([this, &other] () -> std::optional<AXTextMarkerRange> {
+        auto intersection = WebCore::intersection(*this, other);
+        if (intersection.isNull())
+            return std::nullopt;
+
+        return { AXTextMarkerRange(intersection) };
+    });
+}
+
+String AXTextMarkerRange::debugDescription() const
+{
+    return makeString("start: {", m_start.debugDescription(), "}\nend: {", m_end.debugDescription(), "}");
+}
+
 std::partial_ordering partialOrder(const AXTextMarker& marker1, const AXTextMarker& marker2)
 {
     if (marker1.objectID() == marker2.objectID() && LIKELY(marker1.treeID() == marker2.treeID())) {
