@@ -21,9 +21,9 @@
 #include "AccessibilityObjectAtspi.h"
 
 #if USE(ATSPI)
+#include "AXCoreObject.h"
 #include "AXObjectCache.h"
 #include "AccessibilityAtspiInterfaces.h"
-#include "AccessibilityObjectInterface.h"
 #include "AccessibilityRootAtspi.h"
 #include "AccessibilityTableCell.h"
 #include "ElementInlines.h"
@@ -152,7 +152,6 @@ void AccessibilityObjectAtspi::elementDestroyed()
 static Atspi::Role atspiRole(AccessibilityRole role)
 {
     switch (role) {
-    case AccessibilityRole::Annotation:
     case AccessibilityRole::ApplicationAlert:
         return Atspi::Role::Notification;
     case AccessibilityRole::ApplicationAlertDialog:
@@ -174,7 +173,7 @@ static Atspi::Role atspiRole(AccessibilityRole role)
         return Atspi::Role::ToggleButton;
     case AccessibilityRole::RadioButton:
         return Atspi::Role::RadioButton;
-    case AccessibilityRole::CheckBox:
+    case AccessibilityRole::Checkbox:
         return Atspi::Role::CheckBox;
     case AccessibilityRole::Slider:
         return Atspi::Role::Slider;
@@ -187,7 +186,6 @@ static Atspi::Role atspiRole(AccessibilityRole role)
         return Atspi::Role::Entry;
     case AccessibilityRole::StaticText:
         return Atspi::Role::Static;
-    case AccessibilityRole::Outline:
     case AccessibilityRole::Tree:
         return Atspi::Role::Tree;
     case AccessibilityRole::TreeItem:
@@ -213,16 +211,11 @@ static Atspi::Role atspiRole(AccessibilityRole role)
         return Atspi::Role::ToolBar;
     case AccessibilityRole::Meter:
         return Atspi::Role::LevelBar;
-    case AccessibilityRole::BusyIndicator:
     case AccessibilityRole::ProgressIndicator:
         return Atspi::Role::ProgressBar;
-    case AccessibilityRole::Window:
-        return Atspi::Role::Window;
     case AccessibilityRole::PopUpButton:
     case AccessibilityRole::ComboBox:
         return Atspi::Role::ComboBox;
-    case AccessibilityRole::SplitGroup:
-        return Atspi::Role::SplitPane;
     case AccessibilityRole::Splitter:
         return Atspi::Role::Separator;
     case AccessibilityRole::ColorWell:
@@ -358,16 +351,10 @@ static Atspi::Role atspiRole(AccessibilityRole role)
         return Atspi::Role::DirectoryPane;
     case AccessibilityRole::Mark:
         return Atspi::Role::Mark;
-    case AccessibilityRole::Browser:
     case AccessibilityRole::Details:
-    case AccessibilityRole::DisclosureTriangle:
-    case AccessibilityRole::Drawer:
-    case AccessibilityRole::EditableText:
-    case AccessibilityRole::GrowArea:
-    case AccessibilityRole::HelpTag:
     case AccessibilityRole::Ignored:
     case AccessibilityRole::Incrementor:
-    case AccessibilityRole::Matte:
+    case AccessibilityRole::LineBreak:
     case AccessibilityRole::Model:
     case AccessibilityRole::Presentational:
     case AccessibilityRole::RowGroup:
@@ -376,15 +363,10 @@ static Atspi::Role atspiRole(AccessibilityRole role)
     case AccessibilityRole::RubyInline:
     case AccessibilityRole::RubyRun:
     case AccessibilityRole::RubyText:
-    case AccessibilityRole::Ruler:
-    case AccessibilityRole::RulerMarker:
-    case AccessibilityRole::Sheet:
     case AccessibilityRole::SliderThumb:
     case AccessibilityRole::SpinButtonPart:
     case AccessibilityRole::Summary:
-    case AccessibilityRole::SystemWide:
     case AccessibilityRole::TableHeaderContainer:
-    case AccessibilityRole::ValueIndicator:
     case AccessibilityRole::Suggestion:
         return Atspi::Role::Unknown;
     // Add most new roles above. The release assert is for roles that are handled specially.
@@ -605,13 +587,11 @@ AccessibilityObjectAtspi* AccessibilityObjectAtspi::childAt(unsigned index) cons
 
 Vector<RefPtr<AccessibilityObjectAtspi>> AccessibilityObjectAtspi::wrapperVector(const Vector<RefPtr<AXCoreObject>>& elements) const
 {
-    Vector<RefPtr<AccessibilityObjectAtspi>> wrappers;
-    wrappers.reserveInitialCapacity(elements.size());
-    for (const auto& element : elements) {
-        if (auto* wrapper = element->wrapper())
-            wrappers.uncheckedAppend(wrapper);
-    }
-    return wrappers;
+    return WTF::compactMap(elements, [&](auto& element) -> std::optional<RefPtr<AccessibilityObjectAtspi>> {
+        if (RefPtr wrapper = element->wrapper())
+            return wrapper;
+        return std::nullopt;
+    });
 }
 
 Vector<RefPtr<AccessibilityObjectAtspi>> AccessibilityObjectAtspi::children() const
@@ -790,7 +770,7 @@ OptionSet<Atspi::State> AccessibilityObjectAtspi::states() const
     if (m_coreObject->isRequired())
         states.add(Atspi::State::Required);
 
-    if (m_coreObject->roleValue() == AccessibilityRole::TextArea || m_coreObject->ariaIsMultiline())
+    if (m_coreObject->roleValue() == AccessibilityRole::TextArea || (liveObject && liveObject->ariaIsMultiline()))
         states.add(Atspi::State::MultiLine);
     else if (m_coreObject->roleValue() == AccessibilityRole::TextField || m_coreObject->roleValue() == AccessibilityRole::SearchField)
         states.add(Atspi::State::SingleLine);
@@ -958,7 +938,7 @@ HashMap<String, String> AccessibilityObjectAtspi::attributes() const
         }
     }
 
-    String isReadOnly = m_coreObject->readOnlyValue();
+    String isReadOnly = liveObject ? liveObject->readOnlyValue() : String();
     if (!isReadOnly.isEmpty())
         map.add("readonly"_s, isReadOnly);
 
