@@ -40,6 +40,7 @@
 #include "CSSNamespaceRule.h"
 #include "CSSPageRule.h"
 #include "CSSPropertyRule.h"
+#include "CSSScopeRule.h"
 #include "CSSStyleRule.h"
 #include "CSSSupportsRule.h"
 #include "JSCSSContainerRule.h"
@@ -56,11 +57,12 @@
 #include "JSCSSNamespaceRule.h"
 #include "JSCSSPageRule.h"
 #include "JSCSSPropertyRule.h"
+#include "JSCSSScopeRule.h"
 #include "JSCSSStyleRule.h"
 #include "JSCSSSupportsRule.h"
 #include "JSNode.h"
 #include "JSStyleSheetCustom.h"
-#include "WebCoreOpaqueRoot.h"
+#include "WebCoreOpaqueRootInlines.h"
 
 
 namespace WebCore {
@@ -111,6 +113,8 @@ JSValue toJSNewlyCreated(JSGlobalObject*, JSDOMGlobalObject* globalObject, Ref<C
         return createWrapper<CSSContainerRule>(globalObject, WTFMove(rule));
     case StyleRuleType::Property:
         return createWrapper<CSSPropertyRule>(globalObject, WTFMove(rule));
+    case StyleRuleType::Scope:
+        return createWrapper<CSSScopeRule>(globalObject, WTFMove(rule));
     case StyleRuleType::Unknown:
     case StyleRuleType::Charset:
     case StyleRuleType::Margin:
