@@ -67,7 +67,7 @@ String CryptoAlgorithmRegistry::name(CryptoAlgorithmIdentifier identifier)
 {
     Locker locker { m_lock };
 
-    auto contructor = m_constructors.find(static_cast<unsigned>(identifier));
+    auto contructor = m_constructors.find(enumToUnderlyingType(identifier));
     if (contructor == m_constructors.end())
         return { };
 
@@ -78,7 +78,7 @@ RefPtr<CryptoAlgorithm> CryptoAlgorithmRegistry::create(CryptoAlgorithmIdentifie
 {
     Locker locker { m_lock };
 
-    auto contructor = m_constructors.find(static_cast<unsigned>(identifier));
+    auto contructor = m_constructors.find(enumToUnderlyingType(identifier));
     if (contructor == m_constructors.end())
         return nullptr;
 
@@ -90,10 +90,10 @@ void CryptoAlgorithmRegistry::registerAlgorithm(const String& name, CryptoAlgori
     Locker locker { m_lock };
 
     ASSERT(!m_identifiers.contains(name));
-    ASSERT(!m_constructors.contains(static_cast<unsigned>(identifier)));
+    ASSERT(!m_constructors.contains(enumToUnderlyingType(identifier)));
 
     m_identifiers.add(name, identifier);
-    m_constructors.add(static_cast<unsigned>(identifier), std::make_pair(name, constructor));
+    m_constructors.add(enumToUnderlyingType(identifier), std::make_pair(name, constructor));
 }
 
 
