@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2020 Sony Interactive Entertainment Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -32,14 +33,16 @@
 #include "CryptoAlgorithmAesKeyParams.h"
 #include "CryptoKeyAES.h"
 #include <wtf/CrossThreadCopier.h>
+#include <wtf/FlipBytes.h>
 
 namespace WebCore {
 
 namespace CryptoAlgorithmAES_CTRInternal {
-static const char* const ALG128 = "A128CTR";
-static const char* const ALG192 = "A192CTR";
-static const char* const ALG256 = "A256CTR";
+static constexpr auto ALG128 = "A128CTR"_s;
+static constexpr auto ALG192 = "A192CTR"_s;
+static constexpr auto ALG256 = "A256CTR"_s;
 static const size_t CounterSize = 16;
+static const uint64_t AllBitsSet = ~(uint64_t)0;
 }
 
 static inline bool usagesAreInvalidForCryptoAlgorithmAES_CTR(CryptoKeyUsageBitmap usages)
@@ -71,7 +74,7 @@ void CryptoAlgorithmAES_CTR::encrypt(const CryptoAlgorithmParameters& parameters
 {
     auto& aesParameters = downcast<CryptoAlgorithmAesCtrParams>(parameters);
     if (!parametersAreValid(aesParameters)) {
-        exceptionCallback(OperationError);
+        exceptionCallback(ExceptionCode::OperationError);
         return;
     }
 
@@ -85,7 +88,7 @@ void CryptoAlgorithmAES_CTR::decrypt(const CryptoAlgorithmParameters& parameters
 {
     auto& aesParameters = downcast<CryptoAlgorithmAesCtrParams>(parameters);
     if (!parametersAreValid(aesParameters)) {
-        exceptionCallback(OperationError);
+        exceptionCallback(ExceptionCode::OperationError);
         return;
     }
 
@@ -100,13 +103,13 @@ void CryptoAlgorithmAES_CTR::generateKey(const CryptoAlgorithmParameters& parame
     const auto& aesParameters = downcast<CryptoAlgorithmAesKeyParams>(parameters);
 
     if (usagesAreInvalidForCryptoAlgorithmAES_CTR(usages)) {
-        exceptionCallback(SyntaxError);
+        exceptionCallback(ExceptionCode::SyntaxError);
         return;
     }
 
     auto result = CryptoKeyAES::generate(CryptoAlgorithmIdentifier::AES_CTR, aesParameters.length, extractable, usages);
     if (!result) {
-        exceptionCallback(OperationError);
+        exceptionCallback(ExceptionCode::OperationError);
         return;
     }
 
@@ -118,14 +121,14 @@ void CryptoAlgorithmAES_CTR::importKey(CryptoKeyFormat format, KeyData&& data, c
     using namespace CryptoAlgorithmAES_CTRInternal;
 
     if (usagesAreInvalidForCryptoAlgorithmAES_CTR(usages)) {
-        exceptionCallback(SyntaxError);
+        exceptionCallback(ExceptionCode::SyntaxError);
         return;
     }
 
     RefPtr<CryptoKeyAES> result;
     switch (format) {
     case CryptoKeyFormat::Raw:
-        result = CryptoKeyAES::importRaw(parameters.identifier, WTFMove(WTF::get<Vector<uint8_t>>(data)), extractable, usages);
+        result = CryptoKeyAES::importRaw(parameters.identifier, WTFMove(std::get<Vector<uint8_t>>(data)), extractable, usages);
         break;
     case CryptoKeyFormat::Jwk: {
         auto checkAlgCallback = [](size_t length, const String& alg) -> bool {
@@ -139,15 +142,15 @@ void CryptoAlgorithmAES_CTR::importKey(CryptoKeyFormat format, KeyData&& data, c
             }
             return false;
         };
-        result = CryptoKeyAES::importJwk(parameters.identifier, WTFMove(WTF::get<JsonWebKey>(data)), extractable, usages, WTFMove(checkAlgCallback));
+        result = CryptoKeyAES::importJwk(parameters.identifier, WTFMove(std::get<JsonWebKey>(data)), extractable, usages, WTFMove(checkAlgCallback));
         break;
     }
     default:
-        exceptionCallback(NotSupportedError);
+        exceptionCallback(ExceptionCode::NotSupportedError);
         return;
     }
     if (!result) {
-        exceptionCallback(DataError);
+        exceptionCallback(ExceptionCode::DataError);
         return;
     }
 
@@ -160,7 +163,7 @@ void CryptoAlgorithmAES_CTR::exportKey(CryptoKeyFormat format, Ref<CryptoKey>&&
     const auto& aesKey = downcast<CryptoKeyAES>(key.get());
 
     if (aesKey.key().isEmpty()) {
-        exceptionCallback(OperationError);
+        exceptionCallback(ExceptionCode::OperationError);
         return;
     }
 
@@ -188,7 +191,7 @@ void CryptoAlgorithmAES_CTR::exportKey(CryptoKeyFormat format, Ref<CryptoKey>&&
         break;
     }
     default:
-        exceptionCallback(NotSupportedError);
+        exceptionCallback(ExceptionCode::NotSupportedError);
         return;
     }
 
@@ -200,6 +203,118 @@ ExceptionOr<size_t> CryptoAlgorithmAES_CTR::getKeyLength(const CryptoAlgorithmPa
     return CryptoKeyAES::getKeyLength(parameters);
 }
 
+CryptoAlgorithmAES_CTR::CounterBlockHelper::CounterBlockHelper(const Vector<uint8_t>& counterVector, size_t counterLength)
+    : m_counterLength(counterLength)
+{
+    using namespace CryptoAlgorithmAES_CTRInternal;
+
+    ASSERT(counterVector.size() == CounterSize);
+    ASSERT(counterLength <= CounterSize * 8);
+    bool littleEndian = false; // counterVector is stored in big-endian.
+    memcpy(&m_bits.m_hi, counterVector.data(), 8);
+    m_bits.m_hi = flipBytesIfLittleEndian(m_bits.m_hi, littleEndian);
+    memcpy(&m_bits.m_lo, counterVector.data() + 8, 8);
+    m_bits.m_lo = flipBytesIfLittleEndian(m_bits.m_lo, littleEndian);
+}
+
+size_t CryptoAlgorithmAES_CTR::CounterBlockHelper::countToOverflowSaturating() const
+{
+    CounterBlockBits counterMask;
+    counterMask.set();
+    counterMask <<= m_counterLength;
+    counterMask = ~counterMask;
+
+    auto countMinusOne = ~m_bits & counterMask;
+
+    CounterBlockBits sizeTypeMask;
+    sizeTypeMask.set();
+    sizeTypeMask <<= sizeof(size_t) * 8;
+    if ((sizeTypeMask & countMinusOne).any()) {
+        // Saturating to the size_t max since the count is greater than that.
+        return std::numeric_limits<size_t>::max();
+    }
+
+    countMinusOne &= ~sizeTypeMask;
+    if (countMinusOne.all()) {
+        // As all bits are set, adding one would result in an overflow.
+        // Return size_t max instead.
+        return std::numeric_limits<size_t>::max();
+    }
+
+    static_assert(sizeof(size_t) <= sizeof(uint64_t));
+    return countMinusOne.m_lo + 1;
+}
+
+Vector<uint8_t> CryptoAlgorithmAES_CTR::CounterBlockHelper::counterVectorAfterOverflow() const
+{
+    using namespace CryptoAlgorithmAES_CTRInternal;
+
+    CounterBlockBits nonceMask;
+    nonceMask.set();
+    nonceMask <<= m_counterLength;
+    auto bits = m_bits & nonceMask;
+
+    bool littleEndian = false; // counterVector is stored in big-endian.
+    Vector<uint8_t> counterVector(CounterSize);
+    uint64_t hi = flipBytesIfLittleEndian(bits.m_hi, littleEndian);
+    memcpy(counterVector.data(), &hi, 8);
+    uint64_t lo = flipBytesIfLittleEndian(bits.m_lo, littleEndian);
+    memcpy(counterVector.data() + 8, &lo, 8);
+
+    return counterVector;
+}
+
+void CryptoAlgorithmAES_CTR::CounterBlockHelper::CounterBlockBits::set()
+{
+    using namespace CryptoAlgorithmAES_CTRInternal;
+    m_hi = AllBitsSet;
+    m_lo = AllBitsSet;
+}
+
+bool CryptoAlgorithmAES_CTR::CounterBlockHelper::CounterBlockBits::all() const
+{
+    using namespace CryptoAlgorithmAES_CTRInternal;
+    return m_hi == AllBitsSet && m_lo == AllBitsSet;
+}
+
+bool CryptoAlgorithmAES_CTR::CounterBlockHelper::CounterBlockBits::any() const
+{
+    return m_hi || m_lo;
+}
+
+auto CryptoAlgorithmAES_CTR::CounterBlockHelper::CounterBlockBits::operator&(const CounterBlockBits& rhs) const -> CounterBlockBits
+{
+    return { m_hi & rhs.m_hi, m_lo & rhs.m_lo };
+}
+
+auto CryptoAlgorithmAES_CTR::CounterBlockHelper::CounterBlockBits::operator~() const -> CounterBlockBits
+{
+    return { ~m_hi, ~m_lo };
+}
+
+auto CryptoAlgorithmAES_CTR::CounterBlockHelper::CounterBlockBits::operator <<=(unsigned shift) -> CounterBlockBits&
+{
+    if (shift < 64) {
+        m_hi = (m_hi << shift) | m_lo >> (64 - shift);
+        m_lo <<= shift;
+    } else if (shift < 128) {
+        shift -= 64;
+        m_hi = m_lo << shift;
+        m_lo = 0;
+    } else {
+        m_hi = 0;
+        m_lo = 0;
+    }
+    return *this;
+}
+
+auto CryptoAlgorithmAES_CTR::CounterBlockHelper::CounterBlockBits::operator &=(const CounterBlockBits& rhs) -> CounterBlockBits&
+{
+    m_hi &= rhs.m_hi;
+    m_lo &= rhs.m_lo;
+    return *this;
+}
+
 }
 
 #endif // ENABLE(WEB_CRYPTO)
