@@ -23,6 +23,8 @@
 #if ENABLE(WEB_CRYPTO)
 
 #include "CryptoKeyPair.h"
+#include "GCryptRFC7748.h"
+#include "GCryptRFC8032.h"
 #include "GCryptUtilities.h"
 #include "JsonWebKey.h"
 #include <pal/crypto/gcrypt/Utilities.h>
@@ -33,7 +35,7 @@ namespace WebCore {
 
 bool CryptoKeyOKP::isPlatformSupportedCurve(NamedCurve namedCurve)
 {
-    return namedCurve == NamedCurve::Ed25519;
+    return namedCurve == NamedCurve::Ed25519 || namedCurve == NamedCurve::X25519;
 }
 
 namespace CryptoKeyOKPImpl {
@@ -45,10 +47,15 @@ static bool supportedAlgorithmIdentifier(CryptoAlgorithmIdentifier keyIdentifier
 
     switch (keyIdentifier) {
     case CryptoAlgorithmIdentifier::Ed25519:
-        // Ed25519 only supports id-Ed25519 idenfitied for imported keys.
+        // Ed25519 only supports id-Ed25519 idenfitier for imported keys.
         if (CryptoConstants::matches(data, size, CryptoConstants::s_ed25519Identifier))
             return true;
         return false;
+    case CryptoAlgorithmIdentifier::X25519:
+        // X25519 only supports id-X25519 idenfitier for imported keys.
+        if (CryptoConstants::matches(data, size, CryptoConstants::s_x25519Identifier))
+            return true;
+        return false;
     default:
         ASSERT_NOT_REACHED();
         break;
@@ -59,13 +66,10 @@ static bool supportedAlgorithmIdentifier(CryptoAlgorithmIdentifier keyIdentifier
 
 }
 
-std::optional<CryptoKeyPair> CryptoKeyOKP::platformGeneratePair(CryptoAlgorithmIdentifier identifier, NamedCurve namedCurve, bool extractable, CryptoKeyUsageBitmap usages)
+static std::optional<std::pair<Vector<uint8_t>, Vector<uint8_t>>> gcryptGenerateEd25519Keys()
 {
-    if (namedCurve != NamedCurve::Ed25519)
-        return { };
-
     PAL::GCrypt::Handle<gcry_sexp_t> genkeySexp;
-    gcry_error_t error = gcry_sexp_build(&genkeySexp, nullptr, "(genkey (ecdsa (curve \"Ed25519\") (flags eddsa)))");
+    gcry_error_t error = gcry_sexp_build(&genkeySexp, nullptr, "(genkey (ecdsa (curve Ed25519) (flags eddsa)))");
     if (error != GPG_ERR_NO_ERROR) {
         PAL::GCrypt::logError(error);
         return std::nullopt;
@@ -85,19 +89,83 @@ std::optional<CryptoKeyPair> CryptoKeyOKP::platformGeneratePair(CryptoAlgorithmI
         return std::nullopt;
     }
 
-    auto publicKeyData = mpiData(qMpi);
-    auto privateKeyData = mpiData(dMpi);
-    if (!publicKeyData || publicKeyData->size() != 32 || !privateKeyData || privateKeyData->size() != 32)
+    auto q = mpiData(qMpi);
+    auto d = mpiData(dMpi);
+    if (UNLIKELY(!q || !d))
+        return std::nullopt;
+    return std::make_pair(WTFMove(*q), WTFMove(*d));
+}
+
+static std::optional<std::pair<Vector<uint8_t>, Vector<uint8_t>>> gcryptGenerateX25519Keys()
+{
+    // private key is just 32 random bytes
+    PAL::GCrypt::Handle<gcry_mpi_t> mpi(gcry_mpi_new(256));
+    gcry_mpi_randomize(mpi, 256, GCRY_STRONG_RANDOM);
+    auto q = mpiData(mpi);
+    if (UNLIKELY(!q))
+        return std::nullopt;
+
+    // public key being X25519(a, 9), as defined in [RFC7748], section 6.1.
+    auto d = GCrypt::RFC7748::X25519(*q, GCrypt::RFC7748::c_X25519BasePointU);
+    if (UNLIKELY(!d))
+        return std::nullopt;
+
+    return std::make_pair(WTFMove(*q), WTFMove(*d));
+}
+
+std::optional<CryptoKeyPair> CryptoKeyOKP::platformGeneratePair(CryptoAlgorithmIdentifier identifier, NamedCurve namedCurve, bool extractable, CryptoKeyUsageBitmap usages)
+{
+    if (!isPlatformSupportedCurve(namedCurve))
+        return std::nullopt;
+
+    std::optional<std::pair<Vector<uint8_t>, Vector<uint8_t>>> keyPair;
+    switch (namedCurve) {
+    case NamedCurve::Ed25519:
+        keyPair = gcryptGenerateEd25519Keys();
+        break;
+    case NamedCurve::X25519:
+        keyPair = gcryptGenerateX25519Keys();
+        break;
+    default:
+        ASSERT_NOT_REACHED();
+        return std::nullopt;
+    }
+
+    if (!keyPair)
+        return std::nullopt;
+
+    auto& publicKeyData = keyPair->first;
+    auto& privateKeyData = keyPair->second;
+    if (!(publicKeyData.size() == 32 && privateKeyData.size() == 32))
         return std::nullopt;
 
     bool isPublicKeyExtractable = true;
-    auto publicKey = CryptoKeyOKP::create(identifier, namedCurve, CryptoKeyType::Public, Vector<uint8_t>(*publicKeyData), isPublicKeyExtractable, usages);
+    auto publicKey = CryptoKeyOKP::create(identifier, namedCurve, CryptoKeyType::Public, WTFMove(publicKeyData), isPublicKeyExtractable, usages);
     ASSERT(publicKey);
-    auto privateKey = CryptoKeyOKP::create(identifier, namedCurve, CryptoKeyType::Private, Vector<uint8_t>(*privateKeyData), extractable, usages);
+    auto privateKey = CryptoKeyOKP::create(identifier, namedCurve, CryptoKeyType::Private, WTFMove(privateKeyData), extractable, usages);
     ASSERT(privateKey);
     return CryptoKeyPair { WTFMove(publicKey), WTFMove(privateKey) };
 }
 
+bool CryptoKeyOKP::platformCheckPairedKeys(CryptoAlgorithmIdentifier, NamedCurve namedCurve, const Vector<uint8_t>& privateKey, const Vector<uint8_t>& publicKey)
+{
+    if (!isPlatformSupportedCurve(namedCurve))
+        return false;
+
+    switch (namedCurve) {
+    case NamedCurve::X25519: {
+        // public key being X25519(a, 9), as defined in [RFC7748], section 6.1.
+        auto q = GCrypt::RFC7748::X25519(privateKey, GCrypt::RFC7748::c_X25519BasePointU);
+        return q && q->size() == 32 && *q == publicKey;
+    }
+    case NamedCurve::Ed25519:
+        return GCrypt::RFC8032::validateEd25519KeyPair(privateKey, publicKey);
+    default:
+        ASSERT_NOT_REACHED();
+        return false;
+    }
+}
+
 // Per https://www.ietf.org/rfc/rfc5280.txt
 // SubjectPublicKeyInfo ::= SEQUENCE { algorithm AlgorithmIdentifier, subjectPublicKey BIT STRING }
 // AlgorithmIdentifier  ::= SEQUENCE { algorithm OBJECT IDENTIFIER, parameters ANY DEFINED BY algorithm OPTIONAL }
@@ -109,6 +177,9 @@ std::optional<CryptoKeyPair> CryptoKeyOKP::platformGeneratePair(CryptoAlgorithmI
 // For all of the OIDs, the parameters MUST be absent.
 RefPtr<CryptoKeyOKP> CryptoKeyOKP::importSpki(CryptoAlgorithmIdentifier identifier, NamedCurve curve, Vector<uint8_t>&& keyData, bool extractable, CryptoKeyUsageBitmap usages)
 {
+    if (!isPlatformSupportedCurve(curve))
+        return nullptr;
+
     // Decode the `SubjectPublicKeyInfo` structure using the provided key data.
     PAL::TASN1::Structure spki;
     if (!PAL::TASN1::decodeStructure(&spki, "WebCrypto.SubjectPublicKeyInfo", keyData))
@@ -137,8 +208,18 @@ RefPtr<CryptoKeyOKP> CryptoKeyOKP::importSpki(CryptoAlgorithmIdentifier identifi
             return nullptr;
 
         // Construct the `public-key` expression to be used for generating the MPI structure.
-        gcry_error_t error = gcry_sexp_build(&platformKey, nullptr, "(public-key(ecc(curve Ed25519)(q %b)))",
-            subjectPublicKey->size(), subjectPublicKey->data());
+        gcry_error_t error = GPG_ERR_NO_ERROR;
+        switch (curve) {
+        case CryptoKeyOKP::NamedCurve::Ed25519:
+            error = gcry_sexp_build(&platformKey, nullptr, "(public-key(ecc(curve Ed25519)(q %b)))", subjectPublicKey->size(), subjectPublicKey->data());
+            break;
+        case CryptoKeyOKP::NamedCurve::X25519:
+            error = gcry_sexp_build(&platformKey, nullptr, "(public-key(ecc(curve Curve25519)(q %b)))", subjectPublicKey->size(), subjectPublicKey->data());
+            break;
+        default:
+            ASSERT_NOT_REACHED();
+            return nullptr;
+        }
         if (error != GPG_ERR_NO_ERROR) {
             PAL::GCrypt::logError(error);
             return nullptr;
@@ -160,35 +241,48 @@ RefPtr<CryptoKeyOKP> CryptoKeyOKP::importSpki(CryptoAlgorithmIdentifier identifi
     return create(identifier, curve, CryptoKeyType::Public, Vector<uint8_t>(*rawKey), extractable, usages);
 }
 
+static const std::array<uint8_t, 12> algorithmId(WebCore::CryptoKeyOKP::NamedCurve curve)
+{
+    switch (curve) {
+    case CryptoKeyOKP::NamedCurve::Ed25519:
+        return CryptoConstants::s_ed25519Identifier;
+    case CryptoKeyOKP::NamedCurve::X25519:
+        return CryptoConstants::s_x25519Identifier;
+    default:
+        ASSERT_NOT_REACHED();
+        return { };
+    }
+}
+
 ExceptionOr<Vector<uint8_t>> CryptoKeyOKP::exportSpki() const
 {
     if (type() != CryptoKeyType::Public)
-        return Exception { InvalidAccessError };
+        return Exception { ExceptionCode::InvalidAccessError };
 
     PAL::TASN1::Structure spki;
     {
         // Create the `SubjectPublicKeyInfo` structure.
         if (!PAL::TASN1::createStructure("WebCrypto.SubjectPublicKeyInfo", &spki))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
 
         // Write out the id-edPublicKey identifier under `algorithm.algorithm`.
-        if (!PAL::TASN1::writeElement(spki, "algorithm.algorithm", CryptoConstants::s_ed25519Identifier.data(), 1))
-            return Exception { OperationError };
+        if (!PAL::TASN1::writeElement(spki, "algorithm.algorithm", algorithmId(m_curve).data(), 1))
+            return Exception { ExceptionCode::OperationError };
 
         // The 'paramaters' element should not be present
         if (!PAL::TASN1::writeElement(spki, "algorithm.parameters", nullptr, 0))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
 
         // Write out the public key data under `subjectPublicKey`. Because this is a
         // bit string parameter, the data size has to be multiplied by 8.
         if (!PAL::TASN1::writeElement(spki, "subjectPublicKey", m_data.data(), m_data.size() * 8))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
     }
 
     // Retrieve the encoded `SubjectPublicKeyInfo` data and return it.
     auto result = PAL::TASN1::encodedData(spki, "");
     if (!result)
-        return Exception { OperationError };
+        return Exception { ExceptionCode::OperationError };
 
     return WTFMove(result.value());
 }
@@ -204,6 +298,9 @@ ExceptionOr<Vector<uint8_t>> CryptoKeyOKP::exportSpki() const
 // For all of the OIDs, the parameters MUST be absent.
 RefPtr<CryptoKeyOKP> CryptoKeyOKP::importPkcs8(CryptoAlgorithmIdentifier identifier, NamedCurve curve, Vector<uint8_t>&& keyData, bool extractable, CryptoKeyUsageBitmap usages)
 {
+    if (!isPlatformSupportedCurve(curve))
+        return nullptr;
+
     // Decode the `PrivateKeyInfo` structure using the provided key data.
     PAL::TASN1::Structure pkcs8;
     if (!PAL::TASN1::decodeStructure(&pkcs8, "WebCrypto.PrivateKeyInfo", keyData))
@@ -257,8 +354,18 @@ RefPtr<CryptoKeyOKP> CryptoKeyOKP::importPkcs8(CryptoAlgorithmIdentifier identif
             return nullptr;
 
         // Construct the `private-key` expression that will also be used for the EC context.
-        gcry_error_t error = gcry_sexp_build(&platformKey, nullptr, "(private-key(ecc(curve Ed25519)(flags eddsa)(d %b)))",
-            privateKey->size(), privateKey->data());
+        gcry_error_t error = GPG_ERR_NO_ERROR;
+        switch (curve) {
+        case CryptoKeyOKP::NamedCurve::Ed25519:
+            error = gcry_sexp_build(&platformKey, nullptr, "(private-key(ecc(curve Ed25519)(flags eddsa)(d %b)))", privateKey->size(), privateKey->data());
+            break;
+        case CryptoKeyOKP::NamedCurve::X25519:
+            error = gcry_sexp_build(&platformKey, nullptr, "(private-key(ecc(curve Curve25519)(d %b)))", privateKey->size(), privateKey->data());
+            break;
+        default:
+            ASSERT_NOT_REACHED();
+            return nullptr;
+        }
         if (error != GPG_ERR_NO_ERROR) {
             PAL::GCrypt::logError(error);
             return nullptr;
@@ -301,53 +408,53 @@ RefPtr<CryptoKeyOKP> CryptoKeyOKP::importPkcs8(CryptoAlgorithmIdentifier identif
 ExceptionOr<Vector<uint8_t>> CryptoKeyOKP::exportPkcs8() const
 {
     if (type() != CryptoKeyType::Private)
-        return Exception { InvalidAccessError };
+        return Exception { ExceptionCode::InvalidAccessError };
 
     PAL::TASN1::Structure ecPrivateKey;
     {
         // Create the `ECPrivateKey` structure.
         if (!PAL::TASN1::createStructure("WebCrypto.CurvePrivateKey", &ecPrivateKey))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
 
         // Write out the data under `privateKey`.
         if (!PAL::TASN1::writeElement(ecPrivateKey, "", m_data.data(), m_data.size()))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
     }
 
     PAL::TASN1::Structure pkcs8;
     {
         // Create the `PrivateKeyInfo` structure.
         if (!PAL::TASN1::createStructure("WebCrypto.PrivateKeyInfo", &pkcs8))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
 
         // Write out '0' under `version`.
         if (!PAL::TASN1::writeElement(pkcs8, "version", "0", 0))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
 
         // Write out the id-Ed25519 identifier under `privateKeyAlgorithm.algorithm`.
-        if (!PAL::TASN1::writeElement(pkcs8, "privateKeyAlgorithm.algorithm", CryptoConstants::s_ed25519Identifier.data(), 1))
-            return Exception { OperationError };
+        if (!PAL::TASN1::writeElement(pkcs8, "privateKeyAlgorithm.algorithm", algorithmId(m_curve).data(), 1))
+            return Exception { ExceptionCode::OperationError };
 
         // The 'paramaters' element should not be present
         if (!PAL::TASN1::writeElement(pkcs8, "privateKeyAlgorithm.parameters", nullptr, 0))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
 
         // Write out the `CurvePrivateKey` data under `privateKey`.
         {
             auto data = PAL::TASN1::encodedData(ecPrivateKey, "");
             if (!data || !PAL::TASN1::writeElement(pkcs8, "privateKey", data->data(), data->size()))
-                return Exception { OperationError };
+                return Exception { ExceptionCode::OperationError };
         }
 
         // Eliminate the optional `attributes` element.
         if (!PAL::TASN1::writeElement(pkcs8, "attributes", nullptr, 0))
-            return Exception { OperationError };
+            return Exception { ExceptionCode::OperationError };
     }
 
     // Retrieve the encoded `PrivateKeyInfo` data and return it.
     auto result = PAL::TASN1::encodedData(pkcs8, "");
     if (!result)
-        return Exception { OperationError };
+        return Exception { ExceptionCode::OperationError };
 
     return WTFMove(result.value());
 }
@@ -365,6 +472,15 @@ String CryptoKeyOKP::generateJwkX() const
 
     ASSERT(type() == CryptoKeyType::Private);
 
+    if (m_curve == CryptoKeyOKP::NamedCurve::X25519) {
+        // public key being X25519(a, 9), as defined in [RFC7748], section 6.1.
+        auto q = GCrypt::RFC7748::X25519(m_data, GCrypt::RFC7748::c_X25519BasePointU);
+        if (q && q->size() == 32)
+            return base64URLEncodeToString(*q);
+
+        return { };
+    }
+
     // We get an sexp of the private-key so that we could later extract the public-key associated to it.
     PAL::GCrypt::Handle<gcry_sexp_t> privKey;
     gcry_error_t error = gcry_sexp_build(&privKey, nullptr, "(private-key(ecc(curve Ed25519)(flags eddsa)(d %b)))", m_data.size(), m_data.data());
