@@ -39,8 +39,8 @@ enum class WindRule : bool;
 
 class CSSInsetShapeValue : public CSSValue {
 public:
-    static Ref<CSSInsetShapeValue> create(Ref<CSSValue> top, Ref<CSSValue> right, Ref<CSSValue> bottom, Ref<CSSValue> left,
-        RefPtr<CSSValue> topLeftRadius, RefPtr<CSSValue> topRightRadius, RefPtr<CSSValue> bottomRightRadius, RefPtr<CSSValue> bottomLeftRadius);
+    static Ref<CSSInsetShapeValue> create(Ref<CSSValue>&& top, Ref<CSSValue>&& right, Ref<CSSValue>&& bottom, Ref<CSSValue>&& left,
+        RefPtr<CSSValue>&& topLeftRadius, RefPtr<CSSValue>&& topRightRadius, RefPtr<CSSValue>&& bottomRightRadius, RefPtr<CSSValue>&& bottomLeftRadius);
 
     const CSSValue& top() const { return m_top; }
     const CSSValue& right() const { return m_right; }
@@ -56,7 +56,7 @@ public:
     bool equals(const CSSInsetShapeValue&) const;
 
 private:
-    CSSInsetShapeValue(Ref<CSSValue> top, Ref<CSSValue> right, Ref<CSSValue> bottom, Ref<CSSValue> left, RefPtr<CSSValue> topLeftRadius, RefPtr<CSSValue> topRightRadius, RefPtr<CSSValue> bottomRightRadius, RefPtr<CSSValue> bottomLeftRadius);
+    CSSInsetShapeValue(Ref<CSSValue>&& top, Ref<CSSValue>&& right, Ref<CSSValue>&& bottom, Ref<CSSValue>&& left, RefPtr<CSSValue>&& topLeftRadius, RefPtr<CSSValue>&& topRightRadius, RefPtr<CSSValue>&& bottomRightRadius, RefPtr<CSSValue>&& bottomLeftRadius);
 
     Ref<CSSValue> m_top;
     Ref<CSSValue> m_right;
@@ -71,7 +71,7 @@ private:
 
 class CSSCircleValue : public CSSValue {
 public:
-    static Ref<CSSCircleValue> create(RefPtr<CSSValue> radius, RefPtr<CSSValue> centerX, RefPtr<CSSValue> centerY);
+    static Ref<CSSCircleValue> create(RefPtr<CSSValue>&& radius, RefPtr<CSSValue>&& centerX, RefPtr<CSSValue>&& centerY);
 
     const CSSValue* radius() const { return m_radius.get(); }
     const CSSValue* centerX() const { return m_centerX.get(); }
@@ -81,7 +81,7 @@ public:
     bool equals(const CSSCircleValue&) const;
 
 private:
-    CSSCircleValue(RefPtr<CSSValue> radius, RefPtr<CSSValue> centerX, RefPtr<CSSValue> centerY);
+    CSSCircleValue(RefPtr<CSSValue>&& radius, RefPtr<CSSValue>&& centerX, RefPtr<CSSValue>&& centerY);
 
     RefPtr<CSSValue> m_radius;
     RefPtr<CSSValue> m_centerX;
@@ -90,7 +90,7 @@ private:
 
 class CSSEllipseValue : public CSSValue {
 public:
-    static Ref<CSSEllipseValue> create(RefPtr<CSSValue> radiusX, RefPtr<CSSValue> radiusY, RefPtr<CSSValue> centerX, RefPtr<CSSValue> centerY);
+    static Ref<CSSEllipseValue> create(RefPtr<CSSValue>&& radiusX, RefPtr<CSSValue>&& radiusY, RefPtr<CSSValue>&& centerX, RefPtr<CSSValue>&& centerY);
 
     const CSSValue* radiusX() const { return m_radiusX.get(); }
     const CSSValue* radiusY() const { return m_radiusY.get(); }
@@ -101,7 +101,7 @@ public:
     bool equals(const CSSEllipseValue&) const;
 
 private:
-    CSSEllipseValue(RefPtr<CSSValue> radiusX, RefPtr<CSSValue> radiusY, RefPtr<CSSValue> centerX, RefPtr<CSSValue> centerY);
+    CSSEllipseValue(RefPtr<CSSValue>&& radiusX, RefPtr<CSSValue>&& radiusY, RefPtr<CSSValue>&& centerX, RefPtr<CSSValue>&& centerY);
 
     RefPtr<CSSValue> m_radiusX;
     RefPtr<CSSValue> m_radiusY;
@@ -124,6 +124,68 @@ private:
     WindRule m_windRule { };
 };
 
+class CSSRectShapeValue : public CSSValue {
+public:
+    static Ref<CSSRectShapeValue> create(Ref<CSSValue>&& top, Ref<CSSValue>&& right, Ref<CSSValue>&& bottom, Ref<CSSValue>&& left, RefPtr<CSSValue>&& topLeftRadius, RefPtr<CSSValue>&& topRightRadius, RefPtr<CSSValue>&& bottomRightRadius, RefPtr<CSSValue>&& bottomLeftRadius);
+
+    const CSSValue& top() const { return m_top; }
+    const CSSValue& right() const { return m_right; }
+    const CSSValue& bottom() const { return m_bottom; }
+    const CSSValue& left() const { return m_left; }
+
+    const CSSValue* topLeftRadius() const { return m_topLeftRadius.get(); }
+    const CSSValue* topRightRadius() const { return m_topRightRadius.get(); }
+    const CSSValue* bottomRightRadius() const { return m_bottomRightRadius.get(); }
+    const CSSValue* bottomLeftRadius() const { return m_bottomLeftRadius.get(); }
+
+    String customCSSText() const;
+    bool equals(const CSSRectShapeValue&) const;
+
+private:
+    CSSRectShapeValue(Ref<CSSValue>&& top, Ref<CSSValue>&& right, Ref<CSSValue>&& bottom, Ref<CSSValue>&& left, RefPtr<CSSValue>&& topLeftRadius, RefPtr<CSSValue>&& topRightRadius, RefPtr<CSSValue>&& bottomRightRadius, RefPtr<CSSValue>&& bottomLeftRadius);
+
+    Ref<CSSValue> m_top;
+    Ref<CSSValue> m_right;
+    Ref<CSSValue> m_bottom;
+    Ref<CSSValue> m_left;
+
+    RefPtr<CSSValue> m_topLeftRadius;
+    RefPtr<CSSValue> m_topRightRadius;
+    RefPtr<CSSValue> m_bottomRightRadius;
+    RefPtr<CSSValue> m_bottomLeftRadius;
+};
+
+class CSSXywhValue : public CSSValue {
+public:
+    static Ref<CSSXywhValue> create(Ref<CSSValue>&& insetX, Ref<CSSValue>&& insetY, Ref<CSSValue>&& width, Ref<CSSValue>&& height, RefPtr<CSSValue>&& topLeftRadius, RefPtr<CSSValue>&& topRightRadius, RefPtr<CSSValue>&& bottomRightRadius, RefPtr<CSSValue>&& bottomLeftRadius);
+
+    const CSSValue& insetX() const { return m_insetX; }
+    const CSSValue& insetY() const { return m_insetY; }
+    const CSSValue& width() const { return m_width; }
+    const CSSValue& height() const { return m_height; }
+
+    const CSSValue* topLeftRadius() const { return m_topLeftRadius.get(); }
+    const CSSValue* topRightRadius() const { return m_topRightRadius.get(); }
+    const CSSValue* bottomRightRadius() const { return m_bottomRightRadius.get(); }
+    const CSSValue* bottomLeftRadius() const { return m_bottomLeftRadius.get(); }
+
+    String customCSSText() const;
+    bool equals(const CSSXywhValue&) const;
+
+private:
+    CSSXywhValue(Ref<CSSValue>&& insetX, Ref<CSSValue>&& insetY, Ref<CSSValue>&& width, Ref<CSSValue>&& height, RefPtr<CSSValue>&& topLeftRadius, RefPtr<CSSValue>&& topRightRadius, RefPtr<CSSValue>&& bottomRightRadius, RefPtr<CSSValue>&& bottomLeftRadius);
+
+    Ref<CSSValue> m_insetX;
+    Ref<CSSValue> m_insetY;
+    Ref<CSSValue> m_width;
+    Ref<CSSValue> m_height;
+
+    RefPtr<CSSValue> m_topLeftRadius;
+    RefPtr<CSSValue> m_topRightRadius;
+    RefPtr<CSSValue> m_bottomRightRadius;
+    RefPtr<CSSValue> m_bottomLeftRadius;
+};
+
 class CSSPathValue : public CSSValue {
 public:
     static Ref<CSSPathValue> create(SVGPathByteStream, WindRule);
@@ -148,3 +210,5 @@ SPECIALIZE_TYPE_TRAITS_CSS_VALUE(CSSEllipseValue, isEllipse())
 SPECIALIZE_TYPE_TRAITS_CSS_VALUE(CSSInsetShapeValue, isInsetShape())
 SPECIALIZE_TYPE_TRAITS_CSS_VALUE(CSSPolygonValue, isPolygon())
 SPECIALIZE_TYPE_TRAITS_CSS_VALUE(CSSPathValue, isPath())
+SPECIALIZE_TYPE_TRAITS_CSS_VALUE(CSSRectShapeValue, isRectShape())
+SPECIALIZE_TYPE_TRAITS_CSS_VALUE(CSSXywhValue, isXywhShape())
