@@ -34,12 +34,14 @@
 #include "ColorSerialization.h"
 #include "ComputedStyleExtractor.h"
 #include "RenderStyle.h"
+#include <wtf/NeverDestroyed.h>
 
 namespace WebCore {
 
 Ref<CSSCustomPropertyValue> CSSCustomPropertyValue::createEmpty(const AtomString& name)
 {
-    return adoptRef(*new CSSCustomPropertyValue(name, std::monostate { }));
+    static NeverDestroyed<Ref<CSSVariableData>> empty { CSSVariableData::create({ }) };
+    return createSyntaxAll(name, Ref { empty.get() });
 }
 
 Ref<CSSCustomPropertyValue> CSSCustomPropertyValue::createWithID(const AtomString& name, CSSValueID id)
@@ -52,14 +54,14 @@ bool CSSCustomPropertyValue::equals(const CSSCustomPropertyValue& other) const
 {
     if (m_name != other.m_name || m_value.index() != other.m_value.index())
         return false;
-    return WTF::switchOn(m_value, [&](const std::monostate&) {
-        return true;
-    }, [&](const Ref<CSSVariableReferenceValue>& value) {
-        return value.get() == std::get<Ref<CSSVariableReferenceValue>>(other.m_value).get();
+    return WTF::switchOn(m_value, [&](const Ref<CSSVariableReferenceValue>& value) {
+        auto& otherValue = std::get<Ref<CSSVariableReferenceValue>>(other.m_value);
+        return value.ptr() == otherValue.ptr() || value.get() == otherValue.get();
     }, [&](const CSSValueID& value) {
         return value == std::get<CSSValueID>(other.m_value);
     }, [&](const Ref<CSSVariableData>& value) {
-        return value.get() == std::get<Ref<CSSVariableData>>(other.m_value).get();
+        auto& otherValue = std::get<Ref<CSSVariableData>>(other.m_value);
+        return value.ptr() == otherValue.ptr() || value.get() == otherValue.get();
     }, [&](const SyntaxValue& value) {
         return value == std::get<SyntaxValue>(other.m_value);
     }, [&](const SyntaxValueList& value) {
@@ -71,6 +73,14 @@ String CSSCustomPropertyValue::customCSSText() const
 {
     auto serializeSyntaxValue = [](const SyntaxValue& syntaxValue) -> String {
         return WTF::switchOn(syntaxValue, [&](const Length& value) {
+            if (value.type() == LengthType::Calculated) {
+                auto calcValue = CSSCalcValue::create(value.calculationValue(), RenderStyle::defaultStyle());
+                if (!calcValue) {
+                    ASSERT_NOT_REACHED();
+                    return emptyString();
+                }
+                return calcValue->cssText();
+            }
             return CSSPrimitiveValue::create(value, RenderStyle::defaultStyle())->cssText();
         }, [&](const NumericSyntaxValue& value) {
             return CSSPrimitiveValue::create(value.value, value.unitType)->cssText();
@@ -92,9 +102,7 @@ String CSSCustomPropertyValue::customCSSText() const
     };
 
     auto serialize = [&] {
-        return WTF::switchOn(m_value, [&](const std::monostate&) {
-            return emptyString();
-        }, [&](const Ref<CSSVariableReferenceValue>& value) {
+        return WTF::switchOn(m_value, [&](const Ref<CSSVariableReferenceValue>& value) {
             return value->cssText();
         }, [&](const CSSValueID& value) {
             return nameString(value).string();
@@ -124,10 +132,7 @@ const Vector<CSSParserToken>& CSSCustomPropertyValue::tokens() const
 {
     static NeverDestroyed<Vector<CSSParserToken>> emptyTokens;
 
-    return WTF::switchOn(m_value, [&](const std::monostate&) -> const Vector<CSSParserToken>& {
-        // Do nothing.
-        return emptyTokens;
-    }, [&](const Ref<CSSVariableReferenceValue>&) -> const Vector<CSSParserToken>& {
+    return WTF::switchOn(m_value, [&](const Ref<CSSVariableReferenceValue>&) -> const Vector<CSSParserToken>& {
         ASSERT_NOT_REACHED();
         return emptyTokens;
     }, [&](const CSSValueID&) -> const Vector<CSSParserToken>& {
@@ -162,10 +167,24 @@ Ref<const CSSVariableData> CSSCustomPropertyValue::asVariableData() const
 
 bool CSSCustomPropertyValue::isCurrentColor() const
 {
-    // FIXME: it's surprising that setting a custom property to "currentcolor"
-    // results in m_value being a CSSVariableReferenceValue rather than a
-    // CSSValueID set to CSSValueCurrentcolor or a StyleValue.
-    return std::holds_alternative<Ref<CSSVariableReferenceValue>>(m_value) && std::get<Ref<CSSVariableReferenceValue>>(m_value)->customCSSText() == "currentcolor"_s;
+    // FIXME: Registered properties?
+    auto tokenRange = switchOn(m_value, [&](const Ref<CSSVariableReferenceValue>& variableReferenceValue) {
+        return variableReferenceValue->data().tokenRange();
+    }, [&](const Ref<CSSVariableData>& data) {
+        return data->tokenRange();
+    }, [&](auto&) {
+        return CSSParserTokenRange { };
+    });
+
+    if (tokenRange.atEnd())
+        return false;
+
+    auto token = tokenRange.consumeIncludingWhitespace();
+    if (!tokenRange.atEnd())
+        return false;
+
+    // FIXME: This should probably check all tokens.
+    return token.id() == CSSValueCurrentcolor;
 }
 
 }
