@@ -36,6 +36,7 @@
 #include <wtf/text/WTFString.h>
 
 namespace WebCore {
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(DOMCSSPaintWorklet);
 
 PaintWorklet& DOMCSSPaintWorklet::ensurePaintWorklet(Document& document)
 {
@@ -63,15 +64,16 @@ void PaintWorklet::addModule(const String& moduleURL, WorkletOptions&&, DOMPromi
 {
     auto* document = this->document();
     if (!document) {
-        promise.reject(Exception { InvalidStateError, "This frame is detached"_s });
+        promise.reject(Exception { ExceptionCode::InvalidStateError, "This frame is detached"_s });
         return;
     }
 
     // FIXME: We should download the source from the URL
     // https://bugs.webkit.org/show_bug.cgi?id=191136
-    auto maybeContext = PaintWorkletGlobalScope::tryCreate(*document, ScriptSourceCode(moduleURL));
+    // PaintWorklets don't have access to any sensitive APIs so we don't bother tracking taintedness there.
+    auto maybeContext = PaintWorkletGlobalScope::tryCreate(*document, ScriptSourceCode(moduleURL, JSC::SourceTaintedOrigin::Untainted));
     if (UNLIKELY(!maybeContext)) {
-        promise.reject(Exception { OutOfMemoryError });
+        promise.reject(Exception { ExceptionCode::OutOfMemoryError });
         return;
     }
     auto context = maybeContext.releaseNonNull();
