@@ -250,9 +250,7 @@ const FeatureSchema& color()
     static MainThreadNeverDestroyed<IntegerSchema> schema {
         "color"_s,
         [](auto& context) {
-            if (auto* localFrame = dynamicDowncast<LocalFrame>(context.document.frame()->mainFrame()))
-                return screenDepthPerComponent(localFrame->view()); 
-            return 8;
+            return screenDepthPerComponent(context.document.frame()->mainFrame().virtualView());
         }
     };
     return schema;
@@ -351,9 +349,7 @@ const FeatureSchema& dynamicRange()
                     return true;
                 if (frame.settings().forcedSupportsHighDynamicRangeValue() == ForcedAccessibilityValue::Off)
                     return false;
-                if (auto* localFrame = dynamicDowncast<LocalFrame>(frame.mainFrame()))
-                    return screenSupportsHighDynamicRange(localFrame->view());
-                return false;
+                return screenSupportsHighDynamicRange(frame.mainFrame().virtualView());
             }();
 
             MatchingIdentifiers identifiers { CSSValueStandard };
@@ -515,11 +511,7 @@ const FeatureSchema& prefersContrast()
                 case ForcedAccessibilityValue::Off:
                     return false;
                 case ForcedAccessibilityValue::System:
-#if PLATFORM(MAC) || PLATFORM(IOS_FAMILY) || PLATFORM(GTK)
                     return Theme::singleton().userPrefersContrast();
-#else
-                    return false;
-#endif
                 }
                 return false;
             }();
@@ -559,11 +551,7 @@ const FeatureSchema& prefersReducedMotion()
                 case ForcedAccessibilityValue::Off:
                     return false;
                 case ForcedAccessibilityValue::System:
-#if USE(NEW_THEME) || PLATFORM(IOS_FAMILY)
                     return Theme::singleton().userPrefersReducedMotion();
-#else
-                    return false;
-#endif
                 }
                 return false;
             }();
@@ -607,11 +595,6 @@ const FeatureSchema& scripting()
 
             if (!frame.script().canExecuteScripts(ReasonForCallingCanExecuteScripts::NotAboutToExecuteScript))
                 return MatchingIdentifiers { CSSValueNone };
-
-            auto* frameView = frame.view();
-            if (frameView && frameView->mediaType() == printAtom())
-                return MatchingIdentifiers { CSSValueInitialOnly };
-
             return MatchingIdentifiers { CSSValueEnabled };
         }
     };
