@@ -44,14 +44,14 @@ WTF_MAKE_ISO_ALLOCATED_IMPL(Attr);
 using namespace HTMLNames;
 
 Attr::Attr(Element& element, const QualifiedName& name)
-    : Node(element.document(), CreateOther)
+    : Node(element.document(), CreateAttr)
     , m_element(element)
     , m_name(name)
 {
 }
 
 Attr::Attr(Document& document, const QualifiedName& name, const AtomString& standaloneValue)
-    : Node(document, CreateOther)
+    : Node(document, CreateAttr)
     , m_name(name)
     , m_standaloneValue(standaloneValue)
 {
@@ -72,6 +72,7 @@ Attr::~Attr()
     ASSERT_WITH_SECURITY_IMPLICATION(!isInShadowTree());
     ASSERT_WITH_SECURITY_IMPLICATION(treeScope().rootNode().isDocumentNode());
 
+    // Unable to protect the document here as it may have started destruction.
     willBeDeletedFrom(document());
 }
 
@@ -82,7 +83,7 @@ ExceptionOr<void> Attr::setPrefix(const AtomString& prefix)
         return result.releaseException();
 
     if ((prefix == xmlnsAtom() && namespaceURI() != XMLNSNames::xmlnsNamespaceURI) || qualifiedName() == xmlnsAtom())
-        return Exception { NamespaceError };
+        return Exception { ExceptionCode::NamespaceError };
 
     const AtomString& newPrefix = prefix.isEmpty() ? nullAtom() : prefix;
     if (RefPtr element = m_element.get())
@@ -103,7 +104,7 @@ void Attr::setValue(const AtomString& value)
 
 void Attr::setNodeValue(const String& value)
 {
-    setValue(AtomString { value });
+    setValue(value.isNull() ? emptyAtom() : AtomString(value));
 }
 
 Ref<Node> Attr::cloneNodeInternal(Document& targetDocument, CloningOperation)
@@ -118,9 +119,10 @@ CSSStyleDeclaration* Attr::style()
     RefPtr styledElement = dynamicDowncast<StyledElement>(m_element.get());
     if (!styledElement)
         return nullptr;
-    m_style = MutableStyleProperties::create();
-    styledElement->collectPresentationalHintsForAttribute(qualifiedName(), value(), *m_style);
-    return &m_style->ensureCSSStyleDeclaration();
+    Ref style = MutableStyleProperties::create();
+    m_style = style.copyRef();
+    styledElement->collectPresentationalHintsForAttribute(qualifiedName(), value(), style);
+    return &style->ensureCSSStyleDeclaration();
 }
 
 AtomString Attr::value() const
@@ -136,13 +138,14 @@ void Attr::detachFromElementWithValue(const AtomString& value)
     ASSERT(m_standaloneValue.isNull());
     m_standaloneValue = value;
     m_element = nullptr;
+    // Unable to ref the document here because it may have started destruction.
     setTreeScopeRecursively(document());
 }
 
 void Attr::attachToElement(Element& element)
 {
     ASSERT(!m_element);
-    m_element = element;
+    m_element = &element;
     m_standaloneValue = nullAtom();
     setTreeScopeRecursively(element.treeScope());
 }
