@@ -77,6 +77,7 @@ public:
     void registerChannel();
     void unregisterChannel();
     void postMessage(Ref<SerializedScriptValue>&&);
+    void detach() { m_broadcastChannel = nullptr; }
 
     String name() const { return m_name.isolatedCopy(); }
     BroadcastChannelIdentifier identifier() const { return m_identifier; }
@@ -96,7 +97,7 @@ BroadcastChannel::MainThreadBridge::MainThreadBridge(BroadcastChannel& channel,
     : m_broadcastChannel(channel)
     , m_identifier(BroadcastChannelIdentifier::generate())
     , m_name(name.isolatedCopy())
-    , m_origin(partitionedSecurityOriginFromContext(*channel.scriptExecutionContext()).isolatedCopy())
+    , m_origin(partitionedSecurityOriginFromContext(*channel.protectedScriptExecutionContext()).isolatedCopy())
 {
 }
 
@@ -106,19 +107,23 @@ void BroadcastChannel::MainThreadBridge::ensureOnMainThread(Function<void(Page*)
     if (!m_broadcastChannel)
         return;
 
-    auto* context = m_broadcastChannel->scriptExecutionContext();
+    RefPtr context = m_broadcastChannel->scriptExecutionContext();
     if (!context)
         return;
     ASSERT(context->isContextThread());
 
     Ref protectedThis { *this };
     if (auto* document = dynamicDowncast<Document>(*context)) {
-        task(document->page());
+        task(document->protectedPage().get());
         return;
     }
 
-    downcast<WorkerGlobalScope>(*context).thread().workerLoaderProxy().postTaskToLoader([protectedThis = WTFMove(protectedThis), task = WTFMove(task)](auto& context) {
-        task(downcast<Document>(context).page());
+    auto* workerLoaderProxy = downcast<WorkerGlobalScope>(*context).thread().workerLoaderProxy();
+    if (!workerLoaderProxy)
+        return;
+
+    workerLoaderProxy->postTaskToLoader([protectedThis = WTFMove(protectedThis), task = WTFMove(task)](auto& context) {
+        task(downcast<Document>(context).protectedPage().get());
     });
 }
 
@@ -157,22 +162,29 @@ BroadcastChannel::BroadcastChannel(ScriptExecutionContext& context, const String
     : ActiveDOMObject(&context)
     , m_mainThreadBridge(MainThreadBridge::create(*this, name))
 {
+    Ref mainThreadBridge = m_mainThreadBridge;
     {
         Locker locker { allBroadcastChannelsLock };
-        allBroadcastChannels().add(m_mainThreadBridge->identifier(), this);
+        allBroadcastChannels().add(mainThreadBridge->identifier(), this);
     }
-    m_mainThreadBridge->registerChannel();
+    mainThreadBridge->registerChannel();
 }
 
 BroadcastChannel::~BroadcastChannel()
 {
     close();
+    m_mainThreadBridge->detach();
     {
         Locker locker { allBroadcastChannelsLock };
         allBroadcastChannels().remove(m_mainThreadBridge->identifier());
     }
 }
 
+auto BroadcastChannel::protectedMainThreadBridge() const -> Ref<MainThreadBridge>
+{
+    return m_mainThreadBridge;
+}
+
 BroadcastChannelIdentifier BroadcastChannel::identifier() const
 {
     return m_mainThreadBridge->identifier();
@@ -189,7 +201,7 @@ ExceptionOr<void> BroadcastChannel::postMessage(JSC::JSGlobalObject& globalObjec
         return { };
 
     if (m_isClosed)
-        return Exception { InvalidStateError, "This BroadcastChannel is closed"_s };
+        return Exception { ExceptionCode::InvalidStateError, "This BroadcastChannel is closed"_s };
 
     Vector<RefPtr<MessagePort>> ports;
     auto messageData = SerializedScriptValue::create(globalObject, message, { }, ports, SerializationForStorage::No, SerializationContext::WorkerPostMessage);
@@ -197,7 +209,7 @@ ExceptionOr<void> BroadcastChannel::postMessage(JSC::JSGlobalObject& globalObjec
         return messageData.releaseException();
     ASSERT(ports.isEmpty());
 
-    m_mainThreadBridge->postMessage(messageData.releaseReturnValue());
+    protectedMainThreadBridge()->postMessage(messageData.releaseReturnValue());
     return { };
 }
 
@@ -207,7 +219,7 @@ void BroadcastChannel::close()
         return;
 
     m_isClosed = true;
-    m_mainThreadBridge->unregisterChannel();
+    protectedMainThreadBridge()->unregisterChannel();
 }
 
 void BroadcastChannel::dispatchMessageTo(BroadcastChannelIdentifier channelIdentifier, Ref<SerializedScriptValue>&& message, CompletionHandler<void()>&& completionHandler)
@@ -279,7 +291,7 @@ bool BroadcastChannel::virtualHasPendingActivity() const
 // https://html.spec.whatwg.org/#eligible-for-messaging
 bool BroadcastChannel::isEligibleForMessaging() const
 {
-    auto* context = scriptExecutionContext();
+    RefPtr context = scriptExecutionContext();
     if (!context)
         return false;
 
