@@ -23,20 +23,21 @@
 #include "CDATASection.h"
 
 #include "Document.h"
+#include "DocumentInlines.h"
 #include <wtf/IsoMallocInlines.h>
 
 namespace WebCore {
 
 WTF_MAKE_ISO_ALLOCATED_IMPL(CDATASection);
 
-inline CDATASection::CDATASection(Document& document, const String& data)
-    : Text(document, data, CreateText)
+inline CDATASection::CDATASection(Document& document, String&& data)
+    : Text(document, WTFMove(data), CreateText)
 {
 }
 
-Ref<CDATASection> CDATASection::create(Document& document, const String& data)
+Ref<CDATASection> CDATASection::create(Document& document, String&& data)
 {
-    return adoptRef(*new CDATASection(document, data));
+    return adoptRef(*new CDATASection(document, WTFMove(data)));
 }
 
 String CDATASection::nodeName() const
@@ -51,17 +52,12 @@ Node::NodeType CDATASection::nodeType() const
 
 Ref<Node> CDATASection::cloneNodeInternal(Document& targetDocument, CloningOperation)
 {
-    return create(targetDocument, data());
+    return create(targetDocument, String { data() });
 }
 
-bool CDATASection::childTypeAllowed(NodeType) const
+Ref<Text> CDATASection::virtualCreate(String&& data)
 {
-    return false;
-}
-
-Ref<Text> CDATASection::virtualCreate(const String& data)
-{
-    return create(document(), data);
+    return create(protectedDocument(), WTFMove(data));
 }
 
 } // namespace WebCore
