@@ -29,6 +29,7 @@
 
 #include "Chrome.h"
 #include "ChromeClient.h"
+#include "DocumentInlines.h"
 #include "LocalFrame.h"
 #include "NodeTraversal.h"
 #include "Page.h"
@@ -44,7 +45,7 @@ namespace WebCore {
 constexpr Seconds markerFadeAnimationDuration = 200_ms;
 constexpr double markerFadeAnimationFrameRate = 30;
 
-inline bool DocumentMarkerController::possiblyHasMarkers(OptionSet<DocumentMarker::MarkerType> types)
+inline bool DocumentMarkerController::possiblyHasMarkers(OptionSet<DocumentMarker::Type> types)
 {
     return m_possiblyExistingMarkerTypes.containsAny(types);
 }
@@ -74,13 +75,13 @@ auto DocumentMarkerController::collectTextRanges(const SimpleRange& range) -> Ve
     return ranges;
 }
 
-void DocumentMarkerController::addMarker(const SimpleRange& range, DocumentMarker::MarkerType type, const DocumentMarker::Data& data)
+void DocumentMarkerController::addMarker(const SimpleRange& range, DocumentMarker::Type type, const DocumentMarker::Data& data)
 {
     for (auto& textPiece : collectTextRanges(range))
         addMarker(textPiece.node, { type, textPiece.range, DocumentMarker::Data { data } });
 }
 
-void DocumentMarkerController::addMarker(Text& node, unsigned startOffset, unsigned length, DocumentMarker::MarkerType type, DocumentMarker::Data&& data)
+void DocumentMarkerController::addMarker(Text& node, unsigned startOffset, unsigned length, DocumentMarker::Type type, DocumentMarker::Data&& data)
 {
     addMarker(node, { type, { startOffset, startOffset + length }, WTFMove(data) });
 }
@@ -89,15 +90,15 @@ void DocumentMarkerController::addDraggedContentMarker(const SimpleRange& range)
 {
     // FIXME: Since the marker is already stored in a map keyed by node, we can probably change things around so we don't have to also store the node in the marker.
     for (auto& textPiece : collectTextRanges(range))
-        addMarker(textPiece.node, { DocumentMarker::DraggedContent, textPiece.range, RefPtr<Node> { textPiece.node.ptr() } });
+        addMarker(textPiece.node, { DocumentMarker::Type::DraggedContent, textPiece.range, RefPtr<Node> { textPiece.node.ptr() } });
 }
 
-void DocumentMarkerController::removeMarkers(const SimpleRange& range, OptionSet<DocumentMarker::MarkerType> types, RemovePartiallyOverlappingMarker overlapRule)
+void DocumentMarkerController::removeMarkers(const SimpleRange& range, OptionSet<DocumentMarker::Type> types, RemovePartiallyOverlappingMarker overlapRule)
 {
     filterMarkers(range, nullptr, types, overlapRule);
 }
 
-void DocumentMarkerController::filterMarkers(const SimpleRange& range, const Function<FilterMarkerResult(const DocumentMarker&)>& filter, OptionSet<DocumentMarker::MarkerType> types, RemovePartiallyOverlappingMarker overlapRule)
+void DocumentMarkerController::filterMarkers(const SimpleRange& range, const Function<FilterMarkerResult(const DocumentMarker&)>& filter, OptionSet<DocumentMarker::Type> types, RemovePartiallyOverlappingMarker overlapRule)
 {
     for (auto& textPiece : collectTextRanges(range)) {
         if (!possiblyHasMarkers(types))
@@ -123,7 +124,7 @@ void DocumentMarkerController::invalidateRectsForAllMarkers()
             marker.invalidate();
     }
 
-    if (Page* page = m_document.page())
+    if (RefPtr page = m_document->page())
         page->chrome().client().didInvalidateDocumentMarkerRects();
 }
 
@@ -132,34 +133,36 @@ void DocumentMarkerController::invalidateRectsForMarkersInNode(Node& node)
     if (!hasMarkers())
         return;
 
-    auto markers = m_markers.get(&node);
+    auto markers = m_markers.get(node);
     ASSERT(markers);
 
     for (auto& marker : *markers)
         marker.invalidate();
 
-    if (Page* page = m_document.page())
+    if (RefPtr page = m_document->page())
         page->chrome().client().didInvalidateDocumentMarkerRects();
 }
 
 static void updateMainFrameLayoutIfNeeded(Document& document)
 {
-    auto* frame = document.frame();
+    RefPtr frame = document.frame();
     if (!frame)
         return;
 
-    auto* localFrame = dynamicDowncast<LocalFrame>(frame->mainFrame());
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame->mainFrame());
     if (!localFrame)
         return;
 
-    auto* mainFrameView = localFrame->view();
-    if (!mainFrameView)
-        return;
-
-    mainFrameView->updateLayoutAndStyleIfNeededRecursive();
+    if (RefPtr mainFrameView = localFrame->view())
+        mainFrameView->updateLayoutAndStyleIfNeededRecursive();
 }
 
-void DocumentMarkerController::updateRectsForInvalidatedMarkersOfType(DocumentMarker::MarkerType type)
+Ref<Document> DocumentMarkerController::protectedDocument() const
+{
+    return m_document.get();
+}
+
+void DocumentMarkerController::updateRectsForInvalidatedMarkersOfType(DocumentMarker::Type type)
 {
     if (!possiblyHasMarkers(type))
         return;
@@ -171,15 +174,15 @@ void DocumentMarkerController::updateRectsForInvalidatedMarkersOfType(DocumentMa
             if (marker.type() != type || marker.isValid())
                 continue;
             if (!updatedLayout) {
-                updateMainFrameLayoutIfNeeded(m_document);
+                updateMainFrameLayoutIfNeeded(protectedDocument());
                 updatedLayout = true;
             }
-            updateRenderedRectsForMarker(marker, *nodeMarkers.key);
+            updateRenderedRectsForMarker(marker, nodeMarkers.key);
         }
     }
 }
 
-Vector<FloatRect> DocumentMarkerController::renderedRectsForMarkers(DocumentMarker::MarkerType type)
+Vector<FloatRect> DocumentMarkerController::renderedRectsForMarkers(DocumentMarker::Type type)
 {
     Vector<FloatRect> result;
 
@@ -187,10 +190,10 @@ Vector<FloatRect> DocumentMarkerController::renderedRectsForMarkers(DocumentMark
         return result;
     ASSERT(!m_markers.isEmpty());
 
-    RefPtr frame = m_document.frame();
+    RefPtr frame = m_document->frame();
     if (!frame)
         return result;
-    auto* frameView = frame->view();
+    RefPtr frameView = frame->view();
     if (!frameView)
         return result;
 
@@ -202,7 +205,7 @@ Vector<FloatRect> DocumentMarkerController::renderedRectsForMarkers(DocumentMark
         subframeClipRect = frameView->windowToContents(frameView->windowClipRect());
 
     for (auto& nodeMarkers : m_markers) {
-        auto renderer = nodeMarkers.key->renderer();
+        CheckedPtr renderer = nodeMarkers.key->renderer();
         FloatRect overflowClipRect;
         if (renderer)
             overflowClipRect = renderer->absoluteClippedOverflowRectForRepaint();
@@ -237,16 +240,16 @@ Vector<FloatRect> DocumentMarkerController::renderedRectsForMarkers(DocumentMark
 static bool shouldInsertAsSeparateMarker(const DocumentMarker& marker)
 {
 #if ENABLE(PLATFORM_DRIVEN_TEXT_CHECKING)
-    if (marker.type() == DocumentMarker::PlatformTextChecking)
+    if (marker.type() == DocumentMarker::Type::PlatformTextChecking)
         return true;
 #endif
 
 #if PLATFORM(IOS_FAMILY)
-    if (marker.type() == DocumentMarker::DictationPhraseWithAlternatives || marker.type() == DocumentMarker::DictationResult)
+    if (marker.type() == DocumentMarker::Type::DictationPhraseWithAlternatives || marker.type() == DocumentMarker::Type::DictationResult)
         return true;
 #endif
 
-    if (marker.type() == DocumentMarker::DraggedContent)
+    if (marker.type() == DocumentMarker::Type::DraggedContent)
         return is<RenderReplaced>(std::get<RefPtr<Node>>(marker.data())->renderer());
 
     return false;
@@ -263,7 +266,7 @@ void DocumentMarkerController::addMarker(Node& node, DocumentMarker&& newMarker)
 
     m_possiblyExistingMarkerTypes.add(newMarker.type());
 
-    auto& list = m_markers.add(&node, nullptr).iterator->value;
+    auto& list = m_markers.add(node, nullptr).iterator->value;
 
     if (!list) {
         list = makeUnique<Vector<RenderedDocumentMarker>>();
@@ -318,8 +321,8 @@ void DocumentMarkerController::addMarker(Node& node, DocumentMarker&& newMarker)
         list->insert(i, RenderedDocumentMarker(toInsert));
     }
 
-    if (node.renderer())
-        node.renderer()->repaint();
+    if (CheckedPtr renderer = node.renderer())
+        renderer->repaint();
 
     invalidateRectsForMarkersInNode(node);
 }
@@ -335,7 +338,7 @@ void DocumentMarkerController::copyMarkers(Node& source, OffsetRange range, Node
         return;
     ASSERT(!m_markers.isEmpty());
 
-    auto list = m_markers.get(&source);
+    auto list = m_markers.get(source);
     if (!list)
         return;
 
@@ -361,12 +364,12 @@ void DocumentMarkerController::copyMarkers(Node& source, OffsetRange range, Node
     }
 
     if (needRepaint) {
-        if (auto renderer = destination.renderer())
+        if (CheckedPtr renderer = destination.renderer())
             renderer->repaint();
     }
 }
 
-void DocumentMarkerController::removeMarkers(Node& node, OffsetRange range, OptionSet<DocumentMarker::MarkerType> types, const Function<FilterMarkerResult(const DocumentMarker&)>& filter, RemovePartiallyOverlappingMarker overlapRule)
+void DocumentMarkerController::removeMarkers(Node& node, OffsetRange range, OptionSet<DocumentMarker::Type> types, const Function<FilterMarkerResult(const DocumentMarker&)>& filter, RemovePartiallyOverlappingMarker overlapRule)
 {
     if (range.start >= range.end)
         return;
@@ -375,7 +378,7 @@ void DocumentMarkerController::removeMarkers(Node& node, OffsetRange range, Opti
         return;
     ASSERT(!m_markers.isEmpty());
 
-    auto list = m_markers.get(&node);
+    auto list = m_markers.get(node);
     if (!list)
         return;
 
@@ -421,18 +424,18 @@ void DocumentMarkerController::removeMarkers(Node& node, OffsetRange range, Opti
     }
 
     if (list->isEmpty()) {
-        m_markers.remove(&node);
+        m_markers.remove(node);
         if (m_markers.isEmpty())
             m_possiblyExistingMarkerTypes = { };
     }
 
     if (needRepaint) {
-        if (auto renderer = node.renderer())
+        if (CheckedPtr renderer = node.renderer())
             renderer->repaint();
     }
 }
 
-DocumentMarker* DocumentMarkerController::markerContainingPoint(const LayoutPoint& point, DocumentMarker::MarkerType type)
+WeakPtr<DocumentMarker> DocumentMarkerController::markerContainingPoint(const LayoutPoint& point, DocumentMarker::Type type)
 {
     if (!possiblyHasMarkers(type))
         return nullptr;
@@ -442,31 +445,31 @@ DocumentMarker* DocumentMarkerController::markerContainingPoint(const LayoutPoin
     for (auto& nodeMarkers : m_markers.values()) {
         for (auto& marker : *nodeMarkers) {
             if (marker.type() == type && marker.contains(point))
-                return &marker;
+                return marker;
         }
     }
     return nullptr;
 }
 
-Vector<RenderedDocumentMarker*> DocumentMarkerController::markersFor(Node& node, OptionSet<DocumentMarker::MarkerType> types)
+Vector<WeakPtr<RenderedDocumentMarker>> DocumentMarkerController::markersFor(Node& node, OptionSet<DocumentMarker::Type> types)
 {
     if (!possiblyHasMarkers(types))
         return { };
 
-    Vector<RenderedDocumentMarker*> result;
+    Vector<WeakPtr<RenderedDocumentMarker>> result;
     auto list = m_markers.get(&node);
     if (!list)
         return result;
 
     for (auto& marker : *list) {
         if (types.contains(marker.type()))
-            result.append(&marker);
+            result.append(marker);
     }
 
     return result;
 }
 
-void DocumentMarkerController::forEach(const SimpleRange& range, OptionSet<DocumentMarker::MarkerType> types, Function<bool(Node&, RenderedDocumentMarker&)> function)
+void DocumentMarkerController::forEach(const SimpleRange& range, OptionSet<DocumentMarker::Type> types, Function<bool(Node&, RenderedDocumentMarker&)> function)
 {
     if (!possiblyHasMarkers(types))
         return;
@@ -488,7 +491,7 @@ void DocumentMarkerController::forEach(const SimpleRange& range, OptionSet<Docum
     }
 }
 
-void DocumentMarkerController::forEachOfTypes(OptionSet<DocumentMarker::MarkerType> types, const Function<bool(Node&, RenderedDocumentMarker&)> function)
+void DocumentMarkerController::forEachOfTypes(OptionSet<DocumentMarker::Type> types, const Function<bool(Node&, RenderedDocumentMarker&)> function)
 {
     if (!possiblyHasMarkers(types))
         return;
@@ -499,23 +502,24 @@ void DocumentMarkerController::forEachOfTypes(OptionSet<DocumentMarker::MarkerTy
             if (!types.contains(marker.type()))
                 continue;
 
-            function(*nodeMarkers.key, marker);
+            Ref node = nodeMarkers.key;
+            function(node, marker);
         }
     }
 }
 
-Vector<RenderedDocumentMarker*> DocumentMarkerController::markersInRange(const SimpleRange& range, OptionSet<DocumentMarker::MarkerType> types)
+Vector<WeakPtr<RenderedDocumentMarker>> DocumentMarkerController::markersInRange(const SimpleRange& range, OptionSet<DocumentMarker::Type> types)
 {
     // FIXME: Consider making forEach public and changing callers to use that function instead of this one.
-    Vector<RenderedDocumentMarker*> markers;
+    Vector<WeakPtr<RenderedDocumentMarker>> markers;
     forEach(range, types, [&] (Node&, RenderedDocumentMarker& marker) {
-        markers.append(&marker);
+        markers.append(marker);
         return false;
     });
     return markers;
 }
 
-Vector<SimpleRange> DocumentMarkerController::rangesForMarkersInRange(const SimpleRange& range, OptionSet<DocumentMarker::MarkerType> types)
+Vector<SimpleRange> DocumentMarkerController::rangesForMarkersInRange(const SimpleRange& range, OptionSet<DocumentMarker::Type> types)
 {
     Vector<SimpleRange> ranges;
     forEach(range, types, [&] (Node& node, RenderedDocumentMarker& marker) {
@@ -525,23 +529,23 @@ Vector<SimpleRange> DocumentMarkerController::rangesForMarkersInRange(const Simp
     return ranges;
 }
 
-void DocumentMarkerController::removeMarkers(Node& node, OptionSet<DocumentMarker::MarkerType> types)
+void DocumentMarkerController::removeMarkers(Node& node, OptionSet<DocumentMarker::Type> types)
 {
     if (!possiblyHasMarkers(types))
         return;
     ASSERT(!m_markers.isEmpty());
     
-    auto iterator = m_markers.find(&node);
+    auto iterator = m_markers.find(node);
     if (iterator != m_markers.end())
         removeMarkersFromList(iterator, types);
 }
 
-void DocumentMarkerController::removeMarkers(OptionSet<DocumentMarker::MarkerType> types)
+void DocumentMarkerController::removeMarkers(OptionSet<DocumentMarker::Type> types)
 {
     removeMarkers(types, nullptr);
 }
 
-void DocumentMarkerController::removeMarkers(OptionSet<DocumentMarker::MarkerType> types, const Function<FilterMarkerResult(const RenderedDocumentMarker&)>& filter)
+void DocumentMarkerController::removeMarkers(OptionSet<DocumentMarker::Type> types, const Function<FilterMarkerResult(const RenderedDocumentMarker&)>& filter)
 {
     if (!possiblyHasMarkers(types))
         return;
@@ -554,7 +558,7 @@ void DocumentMarkerController::removeMarkers(OptionSet<DocumentMarker::MarkerTyp
     m_possiblyExistingMarkerTypes.remove(removedMarkerTypes);
 }
 
-OptionSet<DocumentMarker::MarkerType> DocumentMarkerController::removeMarkersFromList(MarkerMap::iterator iterator, OptionSet<DocumentMarker::MarkerType> types, const Function<FilterMarkerResult(const RenderedDocumentMarker&)>& filter)
+OptionSet<DocumentMarker::Type> DocumentMarkerController::removeMarkersFromList(MarkerMap::iterator iterator, OptionSet<DocumentMarker::Type> types, const Function<FilterMarkerResult(const RenderedDocumentMarker&)>& filter)
 {
     bool needsRepainting = false;
     bool listCanBeRemoved;
@@ -592,7 +596,7 @@ OptionSet<DocumentMarker::MarkerType> DocumentMarkerController::removeMarkersFro
     }
 
     if (needsRepainting) {
-        if (auto renderer = iterator->key->renderer())
+        if (CheckedPtr renderer = iterator->key->renderer())
             renderer->repaint();
     }
 
@@ -605,7 +609,7 @@ OptionSet<DocumentMarker::MarkerType> DocumentMarkerController::removeMarkersFro
     return removedMarkerTypes;
 }
 
-void DocumentMarkerController::repaintMarkers(OptionSet<DocumentMarker::MarkerType> types)
+void DocumentMarkerController::repaintMarkers(OptionSet<DocumentMarker::Type> types)
 {
     if (!possiblyHasMarkers(types))
         return;
@@ -614,7 +618,7 @@ void DocumentMarkerController::repaintMarkers(OptionSet<DocumentMarker::MarkerTy
     for (auto& nodeMarkers : m_markers) {
         for (auto& marker : *nodeMarkers.value) {
             if (types.contains(marker.type())) {
-                if (auto renderer = nodeMarkers.key->renderer())
+                if (CheckedPtr renderer = nodeMarkers.key->renderer())
                     renderer->repaint();
                 break;
             }
@@ -628,7 +632,7 @@ void DocumentMarkerController::shiftMarkers(Node& node, unsigned startOffset, in
         return;
     ASSERT(!m_markers.isEmpty());
 
-    auto list = m_markers.get(&node);
+    auto list = m_markers.get(node);
     if (!list)
         return;
 
@@ -668,12 +672,12 @@ void DocumentMarkerController::shiftMarkers(Node& node, unsigned startOffset, in
 
     if (didShiftMarker) {
         invalidateRectsForMarkersInNode(node);
-        if (auto renderer = node.renderer())
+        if (CheckedPtr renderer = node.renderer())
             renderer->repaint();
     }
 }
 
-void DocumentMarkerController::dismissMarkers(OptionSet<DocumentMarker::MarkerType> types)
+void DocumentMarkerController::dismissMarkers(OptionSet<DocumentMarker::Type> types)
 {
     if (!possiblyHasMarkers(types))
         return;
@@ -714,7 +718,7 @@ void DocumentMarkerController::fadeAnimationTimerFired()
             shouldRemoveMarkers = true;
         else {
             cancelTimer = false;
-            if (auto* renderer = node.renderer())
+            if (CheckedPtr renderer = node.renderer())
                 renderer->repaint();
         }
 
@@ -733,7 +737,7 @@ void DocumentMarkerController::fadeAnimationTimerFired()
         m_fadeAnimationTimer.stop();
 }
 
-bool DocumentMarkerController::hasMarkers(const SimpleRange& range, OptionSet<DocumentMarker::MarkerType> types)
+bool DocumentMarkerController::hasMarkers(const SimpleRange& range, OptionSet<DocumentMarker::Type> types)
 {
     bool foundMarker = false;
     forEach(range, types, [&] (Node&, RenderedDocumentMarker&) {
@@ -743,7 +747,7 @@ bool DocumentMarkerController::hasMarkers(const SimpleRange& range, OptionSet<Do
     return foundMarker;
 }
 
-void DocumentMarkerController::clearDescriptionOnMarkersIntersectingRange(const SimpleRange& range, OptionSet<DocumentMarker::MarkerType> types)
+void DocumentMarkerController::clearDescriptionOnMarkersIntersectingRange(const SimpleRange& range, OptionSet<DocumentMarker::Type> types)
 {
     forEach(range, types, [&] (Node&, RenderedDocumentMarker& marker) {
         marker.clearData();
@@ -751,19 +755,19 @@ void DocumentMarkerController::clearDescriptionOnMarkersIntersectingRange(const
     });
 }
 
-void addMarker(const SimpleRange& range, DocumentMarker::MarkerType type, const DocumentMarker::Data& data)
+void addMarker(const SimpleRange& range, DocumentMarker::Type type, const DocumentMarker::Data& data)
 {
-    range.start.document().markers().addMarker(range, type, data);
+    range.start.protectedDocument()->checkedMarkers()->addMarker(range, type, data);
 }
 
-void addMarker(Text& node, unsigned startOffset, unsigned length, DocumentMarker::MarkerType type, DocumentMarker::Data&& data)
+void addMarker(Text& node, unsigned startOffset, unsigned length, DocumentMarker::Type type, DocumentMarker::Data&& data)
 {
-    node.document().markers().addMarker(node, startOffset, length, type, WTFMove(data));
+    node.protectedDocument()->checkedMarkers()->addMarker(node, startOffset, length, type, WTFMove(data));
 }
 
-void removeMarkers(const SimpleRange& range, OptionSet<DocumentMarker::MarkerType> types, RemovePartiallyOverlappingMarker policy)
+void removeMarkers(const SimpleRange& range, OptionSet<DocumentMarker::Type> types, RemovePartiallyOverlappingMarker policy)
 {
-    range.start.document().markers().removeMarkers(range, types, policy);
+    range.start.protectedDocument()->checkedMarkers()->removeMarkers(range, types, policy);
 }
 
 SimpleRange makeSimpleRange(Node& node, const DocumentMarker& marker)
@@ -779,9 +783,9 @@ void DocumentMarkerController::showMarkers() const
 {
     fprintf(stderr, "%d nodes have markers:\n", m_markers.size());
     for (auto& nodeMarkers : m_markers) {
-        fprintf(stderr, "%p", nodeMarkers.key.get());
+        fprintf(stderr, "%p", nodeMarkers.key.ptr());
         for (auto& marker : *nodeMarkers.value)
-            fprintf(stderr, " %d:[%d:%d]", marker.type(), marker.startOffset(), marker.endOffset());
+            fprintf(stderr, " %hu:[%d:%d]", enumToUnderlyingType(marker.type()), marker.startOffset(), marker.endOffset());
         fputc('\n', stderr);
     }
 }
