@@ -58,7 +58,7 @@ namespace WebCore {
 
 WTF_MAKE_ISO_ALLOCATED_IMPL(EventTarget);
 
-struct SameSizeAsEventTarget : public ScriptWrappable, public CanMakeWeakPtr<EventTarget, WeakPtrFactoryInitialization::Lazy, WeakPtrImplWithEventTargetData> {
+struct SameSizeAsEventTarget : ScriptWrappable, CanMakeWeakPtrWithBitField<EventTarget, WeakPtrFactoryInitialization::Lazy, WeakPtrImplWithEventTargetData> {
     virtual ~SameSizeAsEventTarget() = default; // Allocate vtable pointer.
 };
 
@@ -92,10 +92,13 @@ bool EventTarget::addEventListener(const AtomString& eventType, Ref<EventListene
 
     auto passive = options.passive;
 
-    if (!passive.has_value() && Quirks::shouldMakeEventListenerPassive(*this, eventType, listener.get()))
+    if (!passive.has_value() && Quirks::shouldMakeEventListenerPassive(*this, eventType))
         passive = true;
 
-    bool listenerCreatedFromScript = is<JSEventListener>(listener) && !downcast<JSEventListener>(listener.get()).wasCreatedFromMarkup();
+    bool listenerCreatedFromScript = [&] {
+        auto* jsEventListener = dynamicDowncast<JSEventListener>(listener.get());
+        return jsEventListener && !jsEventListener->wasCreatedFromMarkup();
+    }();
 
     if (!ensureEventTargetData().eventListenerMap.add(eventType, listener.copyRef(), { options.capture, passive.value_or(false), options.once }))
         return false;
@@ -103,7 +106,7 @@ bool EventTarget::addEventListener(const AtomString& eventType, Ref<EventListene
     if (options.signal) {
         options.signal->addAlgorithm([weakThis = WeakPtr { *this }, eventType, listener = WeakPtr { listener }, capture = options.capture](JSC::JSValue) {
             if (weakThis && listener)
-                weakThis->removeEventListener(eventType, *listener, capture);
+                Ref { *weakThis }->removeEventListener(eventType, *listener, capture);
         });
     }
 
@@ -166,8 +169,8 @@ bool EventTarget::removeEventListener(const AtomString& eventType, EventListener
 template<typename JSMaybeErrorEventListener>
 void EventTarget::setAttributeEventListener(const AtomString& eventType, JSC::JSValue listener, JSC::JSObject& jsEventTarget)
 {
-    auto& isolatedWorld = worldForDOMObject(jsEventTarget);
-    auto* existingListener = attributeEventListener(eventType, isolatedWorld);
+    Ref isolatedWorld = worldForDOMObject(jsEventTarget);
+    RefPtr existingListener = attributeEventListener(eventType, isolatedWorld);
     if (!listener.isObject()) {
         if (existingListener)
             removeEventListener(eventType, *existingListener, false);
@@ -186,7 +189,7 @@ template void EventTarget::setAttributeEventListener<JSEventListener>(const Atom
 
 bool EventTarget::setAttributeEventListener(const AtomString& eventType, RefPtr<EventListener>&& listener, DOMWrapperWorld& isolatedWorld)
 {
-    auto* existingListener = attributeEventListener(eventType, isolatedWorld);
+    RefPtr existingListener = attributeEventListener(eventType, isolatedWorld);
     if (!listener) {
         if (existingListener)
             removeEventListener(eventType, *existingListener, false);
@@ -199,26 +202,21 @@ bool EventTarget::setAttributeEventListener(const AtomString& eventType, RefPtr<
         listener->checkValidityForEventTarget(*this);
 #endif
 
-        auto listenerPointer = listener.copyRef();
-        eventTargetData()->eventListenerMap.replace(eventType, *existingListener, listener.releaseNonNull(), { });
+        eventTargetData()->eventListenerMap.replace(eventType, *existingListener, *listener, { });
 
-        InspectorInstrumentation::didAddEventListener(*this, eventType, *listenerPointer, false);
+        InspectorInstrumentation::didAddEventListener(*this, eventType, *listener, false);
 
         return true;
     }
     return addEventListener(eventType, listener.releaseNonNull(), { });
 }
 
-JSEventListener* EventTarget::attributeEventListener(const AtomString& eventType, DOMWrapperWorld& isolatedWorld)
+RefPtr<JSEventListener> EventTarget::attributeEventListener(const AtomString& eventType, DOMWrapperWorld& isolatedWorld)
 {
-    for (auto& eventListener : eventListeners(eventType)) {
-        auto& listener = eventListener->callback();
-        if (listener.type() != EventListener::JSEventListenerType)
-            continue;
-
-        auto& jsListener = downcast<JSEventListener>(listener);
-        if (jsListener.isAttribute() && jsListener.isolatedWorld() == &isolatedWorld)
-            return &jsListener;
+    for (RefPtr eventListener : eventListeners(eventType)) {
+        RefPtr jsListener = dynamicDowncast<JSEventListener>(eventListener->callback());
+        if (jsListener && jsListener->isAttribute() && jsListener->isolatedWorld() == &isolatedWorld)
+            return jsListener;
     }
 
     return nullptr;
@@ -233,7 +231,7 @@ bool EventTarget::hasActiveEventListeners(const AtomString& eventType) const
 ExceptionOr<bool> EventTarget::dispatchEventForBindings(Event& event)
 {
     if (!event.isInitialized() || event.isBeingDispatched())
-        return Exception { InvalidStateError };
+        return Exception { ExceptionCode::InvalidStateError };
 
     if (!scriptExecutionContext())
         return false;
@@ -289,7 +287,7 @@ static const AtomString& legacyType(const Event& event)
 void EventTarget::fireEventListeners(Event& event, EventInvokePhase phase)
 {
 #if !ASSERT_WITH_SECURITY_IMPLICATION_DISABLED
-    if (auto* node = dynamicDowncast<Node>(*this))
+    if (RefPtr node = dynamicDowncast<Node>(*this))
         ASSERT_WITH_SECURITY_IMPLICATION(ScriptDisallowedScope::InMainThread::isEventDispatchAllowedInSubtree(*node));
     else
         ASSERT_WITH_SECURITY_IMPLICATION(ScriptDisallowedScope::isScriptAllowedInMainThread());
@@ -325,14 +323,14 @@ void EventTarget::fireEventListeners(Event& event, EventInvokePhase phase)
 // https://dom.spec.whatwg.org/#concept-event-listener-inner-invoke
 void EventTarget::innerInvokeEventListeners(Event& event, EventListenerVector listeners, EventInvokePhase phase)
 {
-    Ref<EventTarget> protectedThis(*this);
+    Ref protectedThis { *this };
     ASSERT(!listeners.isEmpty());
     ASSERT(scriptExecutionContext());
 
     Ref context = *scriptExecutionContext();
-    bool contextIsDocument = is<Document>(context);
-    if (contextIsDocument)
-        InspectorInstrumentation::willDispatchEvent(downcast<Document>(context.get()), event);
+    auto* document = dynamicDowncast<Document>(context.get());
+    if (document)
+        InspectorInstrumentation::willDispatchEvent(*document, event);
 
     for (auto& registeredListener : listeners) {
         if (UNLIKELY(registeredListener->wasRemoved()))
@@ -343,7 +341,8 @@ void EventTarget::innerInvokeEventListeners(Event& event, EventListenerVector li
         if (phase == EventInvokePhase::Bubbling && registeredListener->useCapture())
             continue;
 
-        if (InspectorInstrumentation::isEventListenerDisabled(*this, event.type(), registeredListener->callback(), registeredListener->useCapture()))
+        Ref callback = registeredListener->callback();
+        if (InspectorInstrumentation::isEventListenerDisabled(*this, event.type(), callback, registeredListener->useCapture()))
             continue;
 
         // If stopImmediatePropagation has been called, we just break out immediately, without
@@ -354,30 +353,30 @@ void EventTarget::innerInvokeEventListeners(Event& event, EventListenerVector li
         // Make sure the JS wrapper and function stay alive until the end of this scope. Otherwise,
         // event listeners with 'once' flag may get collected as soon as they get unregistered below,
         // before we call the js function.
-        JSC::EnsureStillAliveScope wrapperProtector(registeredListener->callback().wrapper());
-        JSC::EnsureStillAliveScope jsFunctionProtector(registeredListener->callback().jsFunction());
+        JSC::EnsureStillAliveScope wrapperProtector(callback->wrapper());
+        JSC::EnsureStillAliveScope jsFunctionProtector(callback->jsFunction());
 
         // Do this before invocation to avoid reentrancy issues.
         if (registeredListener->isOnce())
-            removeEventListener(event.type(), registeredListener->callback(), registeredListener->useCapture());
+            removeEventListener(event.type(), callback, registeredListener->useCapture());
 
         if (registeredListener->isPassive())
             event.setInPassiveListener(true);
 
 #if ASSERT_ENABLED
-        registeredListener->callback().checkValidityForEventTarget(*this);
+        callback->checkValidityForEventTarget(*this);
 #endif
 
         InspectorInstrumentation::willHandleEvent(context, event, *registeredListener);
-        registeredListener->callback().handleEvent(context, event);
+        callback->handleEvent(context, event);
         InspectorInstrumentation::didHandleEvent(context, event, *registeredListener);
 
         if (registeredListener->isPassive())
             event.setInPassiveListener(false);
     }
 
-    if (contextIsDocument)
-        InspectorInstrumentation::didDispatchEvent(downcast<Document>(context.get()), event);
+    if (document)
+        InspectorInstrumentation::didDispatchEvent(*document, event);
 }
 
 Vector<AtomString> EventTarget::eventTypes() const
@@ -417,16 +416,17 @@ void EventTarget::removeAllEventListeners()
 
 void EventTarget::invalidateEventListenerRegions()
 {
-    if (is<Element>(*this)) {
-        downcast<Element>(*this).invalidateEventListenerRegions();
+    if (auto* element = dynamicDowncast<Element>(*this)) {
+        element->invalidateEventListenerRegions();
         return;
     }
 
+    // Unable the protect the document as it may have started destruction.
     auto* document = [&]() -> Document* {
-        if (is<Document>(*this))
-            return &downcast<Document>(*this);
-        if (is<LocalDOMWindow>(*this))
-            return downcast<LocalDOMWindow>(*this).document();
+        if (auto* document = dynamicDowncast<Document>(*this))
+            return document;
+        if (auto* window = dynamicDowncast<LocalDOMWindow>(*this))
+            return window->document();
         return nullptr;
     }();
 
