@@ -96,11 +96,10 @@ void InlineStyleSheetOwner::insertedIntoDocument(Element& element)
 
 void InlineStyleSheetOwner::removedFromDocument(Element& element)
 {
-    if (auto* scope = m_styleScope.get()) {
+    if (CheckedPtr scope = m_styleScope.get()) {
         if (scope->hasPendingSheet(element))
             scope->removePendingSheet(element);
         scope->removeStyleSheetCandidateNode(element);
-        scope = nullptr;
     }
     if (m_sheet)
         clearSheet();
@@ -108,13 +107,11 @@ void InlineStyleSheetOwner::removedFromDocument(Element& element)
 
 void InlineStyleSheetOwner::clearDocumentData(Element& element)
 {
-    if (m_sheet)
-        m_sheet->clearOwnerNode();
+    if (RefPtr sheet = m_sheet)
+        sheet->clearOwnerNode();
 
-    if (auto* scope = m_styleScope.get()) {
+    if (CheckedPtr scope = m_styleScope.get())
         scope->removeStyleSheetCandidateNode(element);
-        scope = nullptr;
-    }
 }
 
 void InlineStyleSheetOwner::childrenChanged(Element& element)
@@ -141,53 +138,51 @@ void InlineStyleSheetOwner::createSheetFromTextContents(Element& element)
 void InlineStyleSheetOwner::clearSheet()
 {
     ASSERT(m_sheet);
-    auto sheet = WTFMove(m_sheet);
+    RefPtr sheet = std::exchange(m_sheet, nullptr);
     sheet->clearOwnerNode();
 }
 
-inline bool isValidCSSContentType(Element& element, const AtomString& type)
+inline bool isValidCSSContentType(const AtomString& type)
 {
+    // https://html.spec.whatwg.org/multipage/semantics.html#update-a-style-block
     if (type.isEmpty())
         return true;
-    // FIXME: Should MIME types really be case sensitive in XML documents? Doesn't seem like they should,
-    // even though other things are case sensitive in that context. MIME types should never be case sensitive.
-    // We should verify this and then remove the isHTMLElement check here.
-    return element.isHTMLElement() ? equalLettersIgnoringASCIICase(type, "text/css"_s) : type == cssContentTypeAtom();
+    return equalLettersIgnoringASCIICase(type, "text/css"_s);
 }
 
 void InlineStyleSheetOwner::createSheet(Element& element, const String& text)
 {
     ASSERT(element.isConnected());
-    Document& document = element.document();
-    if (m_sheet) {
-        if (m_sheet->isLoading() && m_styleScope)
-            m_styleScope->removePendingSheet(element);
+    Ref document = element.document();
+    if (RefPtr sheet = m_sheet) {
+        if (sheet->isLoading() && m_styleScope)
+            CheckedRef { *m_styleScope }->removePendingSheet(element);
         clearSheet();
     }
 
-    if (!isValidCSSContentType(element, m_contentType))
+    if (!isValidCSSContentType(m_contentType))
         return;
 
-    ASSERT(document.contentSecurityPolicy());
-    const ContentSecurityPolicy& contentSecurityPolicy = *document.contentSecurityPolicy();
-    if (!contentSecurityPolicy.allowInlineStyle(document.url().string(), m_startTextPosition.m_line, text, CheckUnsafeHashes::No, element, element.nonce(), element.isInUserAgentShadowTree())) {
+    ASSERT(document->contentSecurityPolicy());
+    if (!document->checkedContentSecurityPolicy()->allowInlineStyle(document->url().string(), m_startTextPosition.m_line, text, CheckUnsafeHashes::No, element, element.nonce(), element.isInUserAgentShadowTree())) {
         element.notifyLoadedSheetAndAllCriticalSubresources(true);
         return;
     }
 
     auto mediaQueries = MQ::MediaQueryParser::parse(m_media, MediaQueryParserContext(document));
 
-    if (auto* scope = m_styleScope.get())
+    if (CheckedPtr scope = m_styleScope.get())
         scope->addPendingSheet(element);
 
     auto cacheKey = makeInlineStyleSheetCacheKey(text, element);
     if (cacheKey) {
-        if (auto* cachedSheet = inlineStyleSheetCache().get(*cacheKey)) {
+        if (RefPtr cachedSheet = inlineStyleSheetCache().get(*cacheKey)) {
             ASSERT(cachedSheet->isCacheable());
-            m_sheet = CSSStyleSheet::createInline(*cachedSheet, element, m_startTextPosition);
-            m_sheet->setMediaQueries(WTFMove(mediaQueries));
+            Ref sheet = CSSStyleSheet::createInline(*cachedSheet, element, m_startTextPosition);
+            m_sheet = sheet.copyRef();
+            sheet->setMediaQueries(WTFMove(mediaQueries));
             if (!element.isInShadowTree())
-                m_sheet->setTitle(element.title());
+                sheet->setTitle(element.title());
 
             sheetLoaded(element);
             element.notifyLoadedSheetAndAllCriticalSubresources(false);
@@ -197,12 +192,13 @@ void InlineStyleSheetOwner::createSheet(Element& element, const String& text)
 
     m_loading = true;
 
-    auto contents = StyleSheetContents::create(String(), parserContextForElement(element));
+    Ref contents = StyleSheetContents::create(String(), parserContextForElement(element));
 
-    m_sheet = CSSStyleSheet::createInline(contents.get(), element, m_startTextPosition);
-    m_sheet->setMediaQueries(WTFMove(mediaQueries));
+    Ref sheet = CSSStyleSheet::createInline(contents.get(), element, m_startTextPosition);
+    m_sheet = sheet.copyRef();
+    sheet->setMediaQueries(WTFMove(mediaQueries));
     if (!element.isInShadowTree())
-        m_sheet->setTitle(element.title());
+        sheet->setTitle(element.title());
 
     contents->parseString(text);
 
@@ -211,11 +207,11 @@ void InlineStyleSheetOwner::createSheet(Element& element, const String& text)
     contents->checkLoaded();
 
     if (cacheKey && contents->isCacheable()) {
-        m_sheet->contents().addedToMemoryCache();
-        inlineStyleSheetCache().add(*cacheKey, &m_sheet->contents());
+        sheet->contents().addedToMemoryCache();
+        inlineStyleSheetCache().add(*cacheKey, &sheet->contents());
 
         // Prevent pathological growth.
-        const size_t maximumInlineStyleSheetCacheSize = 50;
+        static constexpr auto maximumInlineStyleSheetCacheSize = 256;
         if (inlineStyleSheetCache().size() > maximumInlineStyleSheetCacheSize) {
             auto toRemove = inlineStyleSheetCache().random();
             toRemove->value->removedFromMemoryCache();
@@ -236,7 +232,7 @@ bool InlineStyleSheetOwner::sheetLoaded(Element& element)
     if (isLoading())
         return false;
 
-    if (auto* scope = m_styleScope.get())
+    if (CheckedPtr scope = m_styleScope.get())
         scope->removePendingSheet(element);
 
     return true;
@@ -244,7 +240,7 @@ bool InlineStyleSheetOwner::sheetLoaded(Element& element)
 
 void InlineStyleSheetOwner::startLoadingDynamicSheet(Element& element)
 {
-    if (auto* scope = m_styleScope.get(); scope && !scope->hasPendingSheet(element))
+    if (CheckedPtr scope = m_styleScope.get(); scope && !scope->hasPendingSheet(element))
         scope->addPendingSheet(element);
 }
 
