@@ -47,9 +47,9 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(MessagePort);
 
 static Lock allMessagePortsLock;
-static HashMap<MessagePortIdentifier, MessagePort*>& allMessagePorts() WTF_REQUIRES_LOCK(allMessagePortsLock)
+static HashMap<MessagePortIdentifier, ThreadSafeWeakPtr<MessagePort>>& allMessagePorts() WTF_REQUIRES_LOCK(allMessagePortsLock)
 {
-    static NeverDestroyed<HashMap<MessagePortIdentifier, MessagePort*>> map;
+    static NeverDestroyed<HashMap<MessagePortIdentifier, ThreadSafeWeakPtr<MessagePort>>> map;
     return map;
 }
 
@@ -59,32 +59,6 @@ static HashMap<MessagePortIdentifier, ScriptExecutionContextIdentifier>& portToC
     return map;
 }
 
-void MessagePort::ref() const
-{
-    ++m_refCount;
-}
-
-void MessagePort::deref() const
-{
-    // This custom deref() function ensures that as long as the lock to allMessagePortsLock is taken, no MessagePort will be destroyed.
-    // This allows notifyMessageAvailable to easily query the map and manipulate MessagePort instances.
-
-    if (!--m_refCount) {
-        Locker locker { allMessagePortsLock };
-
-        if (m_refCount)
-            return;
-
-        auto iterator = allMessagePorts().find(m_identifier);
-        if (iterator != allMessagePorts().end() && iterator->value == this) {
-            allMessagePorts().remove(iterator);
-            portToContextIdentifier().remove(m_identifier);
-        }
-
-        delete this;
-    }
-}
-
 bool MessagePort::isMessagePortAliveForTesting(const MessagePortIdentifier& identifier)
 {
     Locker locker { allMessagePortsLock };
@@ -106,7 +80,7 @@ void MessagePort::notifyMessageAvailable(const MessagePortIdentifier& identifier
         RefPtr<MessagePort> port;
         {
             Locker locker { allMessagePortsLock };
-            port = allMessagePorts().get(identifier);
+            port = allMessagePorts().get(identifier).get();
         }
         if (port)
             port->messageAvailable();
@@ -115,7 +89,7 @@ void MessagePort::notifyMessageAvailable(const MessagePortIdentifier& identifier
 
 Ref<MessagePort> MessagePort::create(ScriptExecutionContext& scriptExecutionContext, const MessagePortIdentifier& local, const MessagePortIdentifier& remote)
 {
-    auto messagePort = adoptRef(*new MessagePort(scriptExecutionContext, local, remote));
+    Ref messagePort = adoptRef(*new MessagePort(scriptExecutionContext, local, remote));
     messagePort->suspendIfNeeded();
     return messagePort;
 }
@@ -128,7 +102,8 @@ MessagePort::MessagePort(ScriptExecutionContext& scriptExecutionContext, const M
     LOG(MessagePorts, "Created MessagePort %s (%p) in process %" PRIu64, m_identifier.logString().utf8().data(), this, Process::identifier().toUInt64());
 
     Locker locker { allMessagePortsLock };
-    allMessagePorts().set(m_identifier, this);
+    // We disable threading assertions since the allMessagePorts() is used from multiple threads in a safe way, using a lock.
+    allMessagePorts().set(m_identifier, ThreadSafeWeakPtr { *this });
     portToContextIdentifier().set(m_identifier, scriptExecutionContext.identifier());
 
     // Make sure the WeakPtrFactory gets initialized eagerly on the thread the MessagePort gets constructed on for thread-safety reasons.
@@ -143,7 +118,16 @@ MessagePort::~MessagePort()
 {
     LOG(MessagePorts, "Destroyed MessagePort %s (%p) in process %" PRIu64, m_identifier.logString().utf8().data(), this, Process::identifier().toUInt64());
 
-    ASSERT(allMessagePortsLock.isLocked());
+    Locker locker { allMessagePortsLock };
+
+    auto iterator = allMessagePorts().find(m_identifier);
+    if (iterator != allMessagePorts().end()) {
+        // ThreadSafeWeakPtr::get() returns null as soon as the object has started destruction.
+        if (RefPtr messagePort = iterator->value.get(); !messagePort) {
+            allMessagePorts().remove(iterator);
+            portToContextIdentifier().remove(m_identifier);
+        }
+    }
 
     if (m_entangled)
         close();
@@ -154,7 +138,7 @@ MessagePort::~MessagePort()
 
 void MessagePort::entangle()
 {
-    MessagePortChannelProvider::fromContext(*scriptExecutionContext()).entangleLocalPortInThisProcessToRemote(m_identifier, m_remoteIdentifier);
+    MessagePortChannelProvider::fromContext(*protectedScriptExecutionContext()).entangleLocalPortInThisProcessToRemote(m_identifier, m_remoteIdentifier);
 }
 
 ExceptionOr<void> MessagePort::postMessage(JSC::JSGlobalObject& state, JSC::JSValue messageValue, StructuredSerializeOptions&& options)
@@ -175,7 +159,7 @@ ExceptionOr<void> MessagePort::postMessage(JSC::JSGlobalObject& state, JSC::JSVa
     if (!ports.isEmpty()) {
         for (auto& port : ports) {
             if (port->identifier() == m_identifier || port->identifier() == m_remoteIdentifier)
-                return Exception { DataCloneError };
+                return Exception { ExceptionCode::DataCloneError };
         }
 
         auto disentangleResult = MessagePort::disentanglePorts(WTFMove(ports));
@@ -188,7 +172,7 @@ ExceptionOr<void> MessagePort::postMessage(JSC::JSGlobalObject& state, JSC::JSVa
 
     LOG(MessagePorts, "Actually posting message to port %s (to be received by port %s)", m_identifier.logString().utf8().data(), m_remoteIdentifier.logString().utf8().data());
 
-    MessagePortChannelProvider::fromContext(*scriptExecutionContext()).postMessageToRemote(WTFMove(message), m_remoteIdentifier);
+    MessagePortChannelProvider::fromContext(*protectedScriptExecutionContext()).postMessageToRemote(WTFMove(message), m_remoteIdentifier);
     return { };
 }
 
@@ -197,13 +181,13 @@ TransferredMessagePort MessagePort::disentangle()
     ASSERT(m_entangled);
     m_entangled = false;
 
-    auto& context = *scriptExecutionContext();
+    Ref context = *scriptExecutionContext();
     MessagePortChannelProvider::fromContext(context).messagePortDisentangled(m_identifier);
 
     // We can't receive any messages or generate any events after this, so remove ourselves from the list of active ports.
-    context.destroyedMessagePort(*this);
-    context.willDestroyActiveDOMObject(*this);
-    context.willDestroyDestructionObserver(*this);
+    context->destroyedMessagePort(*this);
+    context->willDestroyActiveDOMObject(*this);
+    context->willDestroyDestructionObserver(*this);
 
     observeContext(nullptr);
 
@@ -216,7 +200,7 @@ void MessagePort::messageAvailable()
 {
     // This MessagePort object might be disentangled because the port is being transferred,
     // in which case we'll notify it that messages are available once a new end point is created.
-    auto* context = scriptExecutionContext();
+    RefPtr context = scriptExecutionContext();
     if (!context || context->activeDOMObjectsAreSuspended())
         return;
 
@@ -234,7 +218,7 @@ void MessagePort::start()
         return;
 
     m_started = true;
-    scriptExecutionContext()->processMessageWithMessagePortsSoon([pendingActivity = makePendingActivity(*this)] { });
+    protectedScriptExecutionContext()->processMessageWithMessagePortsSoon([pendingActivity = makePendingActivity(*this)] { });
 }
 
 void MessagePort::close()
@@ -264,7 +248,7 @@ void MessagePort::dispatchMessages()
     // The HTML5 spec specifies that any messages sent to a document that is not fully active should be dropped, so this behavior is OK.
     ASSERT(started());
 
-    auto* context = scriptExecutionContext();
+    RefPtr context = scriptExecutionContext();
     if (!context || context->activeDOMObjectsAreSuspended() || !isEntangled())
         return;
 
@@ -273,26 +257,27 @@ void MessagePort::dispatchMessages()
 
         LOG(MessagePorts, "MessagePort %s (%p) dispatching %zu messages", m_identifier.logString().utf8().data(), this, messages.size());
 
+        // Unable to protect the script execution context as it may have started destruction.
         auto* context = scriptExecutionContext();
         if (!context || !context->globalObject())
             return;
 
         ASSERT(context->isContextThread());
         auto* globalObject = context->globalObject();
-        auto& vm = globalObject->vm();
+        Ref vm = globalObject->vm();
         auto scope = DECLARE_CATCH_SCOPE(vm);
 
-        bool contextIsWorker = is<WorkerGlobalScope>(*context);
+        auto* workerGlobalScope = dynamicDowncast<WorkerGlobalScope>(*context);
         for (auto& message : messages) {
             // close() in Worker onmessage handler should prevent next message from dispatching.
-            if (contextIsWorker && downcast<WorkerGlobalScope>(*context).isClosing())
+            if (workerGlobalScope && workerGlobalScope->isClosing())
                 return;
 
             auto ports = MessagePort::entanglePorts(*context, WTFMove(message.transferredPorts));
             auto event = MessageEvent::create(*globalObject, message.message.releaseNonNull(), { }, { }, { }, WTFMove(ports));
             if (UNLIKELY(scope.exception())) {
                 // Currently, we assume that the only way we can get here is if we have a termination.
-                RELEASE_ASSERT(vm.hasPendingTerminationException());
+                RELEASE_ASSERT(vm->hasPendingTerminationException());
                 return;
             }
 
@@ -303,7 +288,7 @@ void MessagePort::dispatchMessages()
         }
     };
 
-    MessagePortChannelProvider::fromContext(*scriptExecutionContext()).takeAllMessagesForPort(m_identifier, WTFMove(messagesTakenHandler));
+    MessagePortChannelProvider::fromContext(*context).takeAllMessagesForPort(m_identifier, WTFMove(messagesTakenHandler));
 }
 
 void MessagePort::dispatchEvent(Event& event)
@@ -311,9 +296,10 @@ void MessagePort::dispatchEvent(Event& event)
     if (m_isDetached)
         return;
 
-    auto* context = scriptExecutionContext();
-    if (is<WorkerGlobalScope>(*context) && downcast<WorkerGlobalScope>(*context).isClosing())
-        return;
+    if (RefPtr globalScope = dynamicDowncast<WorkerGlobalScope>(scriptExecutionContext())) {
+        if (globalScope->isClosing())
+            return;
+    }
 
     EventTarget::dispatchEvent(event);
 }
@@ -346,10 +332,10 @@ ExceptionOr<Vector<TransferredMessagePort>> MessagePort::disentanglePorts(Vector
         return Vector<TransferredMessagePort> { };
 
     // Walk the incoming array - if there are any duplicate ports, or null ports or cloned ports, throw an error (per section 8.3.3 of the HTML5 spec).
-    HashSet<MessagePort*> portSet;
+    HashSet<Ref<MessagePort>> portSet;
     for (auto& port : ports) {
-        if (!port || !port->m_entangled || !portSet.add(port.get()).isNewEntry)
-            return Exception { DataCloneError };
+        if (!port || !port->m_entangled || !portSet.add(*port).isNewEntry)
+            return Exception { ExceptionCode::DataCloneError };
     }
 
     // Passed-in ports passed validity checks, so we can disentangle them.
@@ -372,7 +358,7 @@ Vector<RefPtr<MessagePort>> MessagePort::entanglePorts(ScriptExecutionContext& c
 
 Ref<MessagePort> MessagePort::entangle(ScriptExecutionContext& context, TransferredMessagePort&& transferredPort)
 {
-    auto port = MessagePort::create(context, transferredPort.first, transferredPort.second);
+    Ref port = MessagePort::create(context, transferredPort.first, transferredPort.second);
     port->entangle();
     return port;
 }
@@ -390,7 +376,7 @@ bool MessagePort::addEventListener(const AtomString& eventType, Ref<EventListene
 
 bool MessagePort::removeEventListener(const AtomString& eventType, EventListener& listener, const EventListenerOptions& options)
 {
-    auto result = EventTarget::removeEventListener(eventType, listener, options);
+    bool result = EventTarget::removeEventListener(eventType, listener, options);
 
     if (!hasEventListeners(eventNames().messageEvent))
         m_hasMessageEventListener = false;
