@@ -47,7 +47,7 @@ Ref<NameNodeList> NameNodeList::create(ContainerNode& rootNode, const AtomString
 
 NameNodeList::~NameNodeList()
 {
-    ownerNode().nodeLists()->removeCacheWithAtomName(*this, m_name);
+    protectedOwnerNode()->nodeLists()->removeCacheWithAtomName(*this, m_name);
 }
 
 bool NameNodeList::elementMatches(Element& element) const
