@@ -31,10 +31,12 @@
 #include "CSSTokenizer.h"
 #include "CommonAtomStrings.h"
 #include "ElementAncestorIteratorInlines.h"
+#include "HTMLBaseElement.h"
 #include "HTMLNames.h"
 #include "SelectorChecker.h"
 #include "StaticNodeList.h"
 #include "StyledElement.h"
+#include "TreeScopeInlines.h"
 #include "TypedElementDescendantIteratorInlines.h"
 
 namespace WebCore {
@@ -95,7 +97,7 @@ SelectorDataList::SelectorDataList(const CSSSelectorList& selectorList)
 
     m_selectors.reserveInitialCapacity(selectorCount);
     for (const CSSSelector* selector = selectorList.first(); selector; selector = CSSSelectorList::next(selector))
-        m_selectors.uncheckedAppend({ selector });
+        m_selectors.append({ selector });
 
     if (selectorCount == 1) {
         const CSSSelector& selector = *m_selectors.first().selector;
@@ -151,8 +153,8 @@ inline Element* SelectorDataList::selectorClosest(const SelectorData& selectorDa
 
 bool SelectorDataList::matches(Element& targetElement) const
 {
-    for (auto& selctor : m_selectors) {
-        if (selectorMatches(selctor, targetElement, targetElement))
+    for (auto& selector : m_selectors) {
+        if (selectorMatches(selector, targetElement, targetElement))
             return true;
     }
     return false;
@@ -208,12 +210,12 @@ ALWAYS_INLINE void SelectorDataList::executeFastPathForIdSelector(const Containe
 
     const AtomString& idToMatch = idSelector->value();
     if (UNLIKELY(rootNode.treeScope().containsMultipleElementsWithId(idToMatch))) {
-        const Vector<Element*>* elements = rootNode.treeScope().getAllElementsById(idToMatch);
+        auto* elements = rootNode.treeScope().getAllElementsById(idToMatch);
         ASSERT(elements);
         bool rootNodeIsTreeScopeRoot = rootNode.isTreeScope();
         for (auto& element : *elements) {
-            if ((rootNodeIsTreeScopeRoot || element->isDescendantOf(rootNode)) && selectorMatches(selectorData, *element, rootNode)) {
-                appendOutputForElement(output, *element);
+            if ((rootNodeIsTreeScopeRoot || element->isDescendantOf(rootNode)) && selectorMatches(selectorData, element, rootNode)) {
+                appendOutputForElement(output, element);
                 if constexpr (std::is_same_v<OutputType, Element*>)
                     return;
             }
@@ -221,7 +223,7 @@ ALWAYS_INLINE void SelectorDataList::executeFastPathForIdSelector(const Containe
         return;
     }
 
-    Element* element = rootNode.treeScope().getElementById(idToMatch);
+    RefPtr element = rootNode.treeScope().getElementById(idToMatch);
     if (!element || !(rootNode.isTreeScope() || element->isDescendantOf(rootNode)))
         return;
     if (selectorMatches(selectorData, *element, rootNode))
@@ -249,7 +251,7 @@ static ContainerNode& filterRootById(ContainerNode& rootNode, const CSSSelector&
     for (; selector; selector = selector->tagHistory()) {
         if (canBeUsedForIdFastPath(*selector)) {
             const AtomString& idToMatch = selector->value();
-            if (ContainerNode* searchRoot = rootNode.treeScope().getElementById(idToMatch)) {
+            if (RefPtr<ContainerNode> searchRoot = rootNode.treeScope().getElementById(idToMatch)) {
                 if (LIKELY(!rootNode.treeScope().containsMultipleElementsWithId(idToMatch))) {
                     if (inAdjacentChain)
                         searchRoot = searchRoot->parentNode();
@@ -276,6 +278,16 @@ static ALWAYS_INLINE bool localNameMatches(const Element& element, const AtomStr
 template<typename OutputType>
 static inline void elementsForLocalName(const ContainerNode& rootNode, const AtomString& localName, const AtomString& lowercaseLocalName, OutputType& output)
 {
+    if (auto* rootDocument = dynamicDowncast<Document>(rootNode); rootDocument && lowercaseLocalName == HTMLNames::baseTag->localName()) {
+        RefPtr firstBaseElement = rootDocument->firstBaseElement();
+        if (!firstBaseElement)
+            return;
+        if constexpr (std::is_same_v<OutputType, Element*>) {
+            appendOutputForElement(output, *firstBaseElement);
+            return;
+        }
+    }
+
     if (localName == lowercaseLocalName) {
         for (auto& element : descendantsOfType<Element>(const_cast<ContainerNode&>(rootNode))) {
             if (element.tagQName().localName() == localName) {
@@ -601,7 +613,7 @@ SelectorQuery* SelectorQueryCache::add(const String& selectors, const Document&
 
     return m_entries.ensure(key, [&]() -> std::unique_ptr<SelectorQuery> {
         auto tokenizer = CSSTokenizer { selectors };
-        auto selectorList = parseCSSSelector(tokenizer.tokenRange(), context, nullptr, CSSParserEnum::IsNestedContext::No);
+        auto selectorList = parseCSSSelectorList(tokenizer.tokenRange(), context);
 
         if (!selectorList || selectorList->hasInvalidSelector())
             return nullptr;
