@@ -32,7 +32,6 @@
 #include "ChildListMutationScope.h"
 #include "ElementInlines.h"
 #include "ElementTraversal.h"
-#include "HTMLParserIdioms.h"
 #include "HTMLSlotElement.h"
 #if ENABLE(PICTURE_IN_PICTURE_API)
 #include "NotImplemented.h"
@@ -51,28 +50,31 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(ShadowRoot);
 
 struct SameSizeAsShadowRoot : public DocumentFragment, public TreeScope {
-    bool flags[4];
-    uint8_t mode;
-    void* styleScope;
+    uint8_t flagsAndModes[3];
+    WeakPtr<Element, WeakPtrImplWithEventTargetData> host;
     void* styleSheetList;
-    WeakPtr<Element> host;
+    void* styleScope;
     void* slotAssignment;
     std::optional<HashMap<AtomString, AtomString>> partMappings;
 };
 
 static_assert(sizeof(ShadowRoot) == sizeof(SameSizeAsShadowRoot), "shadowroot should stay small");
 #if !ASSERT_ENABLED
-static_assert(sizeof(WeakPtr<Element>) == sizeof(void*), "WeakPtr should be same size as raw pointer");
+static_assert(sizeof(WeakPtr<Element, WeakPtrImplWithEventTargetData>) == sizeof(void*), "WeakPtr should be same size as raw pointer");
 #endif
 
-ShadowRoot::ShadowRoot(Document& document, ShadowRootMode type, DelegatesFocus delegatesFocus)
+ShadowRoot::ShadowRoot(Document& document, ShadowRootMode mode, SlotAssignmentMode assignmentMode, DelegatesFocus delegatesFocus, Cloneable cloneable, AvailableToElementInternals availableToElementInternals)
     : DocumentFragment(document, CreateShadowRoot)
     , TreeScope(*this, document)
     , m_delegatesFocus(delegatesFocus == DelegatesFocus::Yes)
-    , m_type(type)
+    , m_isCloneable(cloneable == Cloneable::Yes)
+    , m_availableToElementInternals(availableToElementInternals == AvailableToElementInternals::Yes)
+    , m_mode(mode)
+    , m_slotAssignmentMode(assignmentMode)
     , m_styleScope(makeUnique<Style::Scope>(*this))
 {
-    if (type == ShadowRootMode::UserAgent)
+    setNodeFlag(NodeFlag::IsInShadowTree);
+    if (m_mode == ShadowRootMode::UserAgent)
         setNodeFlag(NodeFlag::HasBeenInUserAgentShadowTree);
 }
 
@@ -80,10 +82,11 @@ ShadowRoot::ShadowRoot(Document& document, ShadowRootMode type, DelegatesFocus d
 ShadowRoot::ShadowRoot(Document& document, std::unique_ptr<SlotAssignment>&& slotAssignment)
     : DocumentFragment(document, CreateShadowRoot)
     , TreeScope(*this, document)
-    , m_type(ShadowRootMode::UserAgent)
+    , m_mode(ShadowRootMode::UserAgent)
     , m_styleScope(makeUnique<Style::Scope>(*this))
     , m_slotAssignment(WTFMove(slotAssignment))
 {
+    setNodeFlag(NodeFlag::IsInShadowTree);
     setNodeFlag(NodeFlag::HasBeenInUserAgentShadowTree);
 }
 
@@ -93,13 +96,14 @@ ShadowRoot::~ShadowRoot()
     if (isConnected())
         document().didRemoveInDocumentShadowRoot(*this);
 
-    if (m_styleSheetList)
-        m_styleSheetList->detach();
+    if (RefPtr styleSheetList = m_styleSheetList)
+        styleSheetList->detach();
 
     // We cannot let ContainerNode destructor call willBeDeletedFrom()
     // for this ShadowRoot instance because TreeScope destructor
     // clears Node::m_treeScope thus ContainerNode is no longer able
     // to access it Document reference after that.
+    // We can't ref document() here since it may have started destruction.
     willBeDeletedFrom(document());
 
     ASSERT(!m_hasBegunDeletingDetachedChildren);
@@ -115,22 +119,31 @@ Node::InsertedIntoAncestorResult ShadowRoot::insertedIntoAncestor(InsertionType
 {
     DocumentFragment::insertedIntoAncestor(insertionType, parentOfInsertedTree);
     if (insertionType.connectedToDocument)
-        document().didInsertInDocumentShadowRoot(*this);
+        protectedDocument()->didInsertInDocumentShadowRoot(*this);
+    if (!adoptedStyleSheets().empty() && document().frame())
+        checkedStyleScope()->didChangeActiveStyleSheetCandidates();
     return InsertedIntoAncestorResult::Done;
 }
 
+CheckedRef<Style::Scope> ShadowRoot::checkedStyleScope() const
+{
+    return *m_styleScope;
+}
+
 void ShadowRoot::removedFromAncestor(RemovalType removalType, ContainerNode& oldParentOfRemovedTree)
 {
     DocumentFragment::removedFromAncestor(removalType, oldParentOfRemovedTree);
-    if (removalType.disconnectedFromDocument)
+    if (removalType.disconnectedFromDocument) {
+        // Unable to ref the document as it may have started destruction.
         document().didRemoveInDocumentShadowRoot(*this);
+    }
 }
 
 void ShadowRoot::childrenChanged(const ChildChange& childChange)
 {
     DocumentFragment::childrenChanged(childChange);
 
-    if (!m_host || m_type == ShadowRootMode::UserAgent)
+    if (!m_host || m_mode == ShadowRootMode::UserAgent)
         return; // Don't support first-child, nth-of-type, etc... in UA shadow roots as an optimization.
 
     // FIXME: Avoid always invalidating style just for first-child, etc... as done in Element::childrenChanged.
@@ -152,12 +165,16 @@ void ShadowRoot::childrenChanged(const ChildChange& childChange)
 
 void ShadowRoot::moveShadowRootToNewParentScope(TreeScope& newScope, Document& newDocument)
 {
+    Ref oldDocument = documentScope();
     setParentTreeScope(newScope);
-    moveShadowRootToNewDocument(newDocument);
+    moveShadowRootToNewDocument(oldDocument, newDocument);
 }
 
-void ShadowRoot::moveShadowRootToNewDocument(Document& newDocument)
+void ShadowRoot::moveShadowRootToNewDocument(Document& oldDocument, Document& newDocument)
 {
+    if (oldDocument.templateDocumentHost() != &newDocument && newDocument.templateDocumentHost() != &oldDocument)
+        setAdoptedStyleSheets({ });
+
     setDocumentScope(newDocument);
     RELEASE_ASSERT_WITH_SECURITY_IMPLICATION(!parentTreeScope() || &parentTreeScope()->documentScope() == &newDocument);
 
@@ -166,11 +183,6 @@ void ShadowRoot::moveShadowRootToNewDocument(Document& newDocument)
     RELEASE_ASSERT_WITH_SECURITY_IMPLICATION(&m_styleScope->document() == &newDocument);
 }
 
-Style::Scope& ShadowRoot::styleScope()
-{
-    return *m_styleScope;
-}
-
 StyleSheetList& ShadowRoot::styleSheets()
 {
     if (!m_styleSheetList)
@@ -178,6 +190,27 @@ StyleSheetList& ShadowRoot::styleSheets()
     return *m_styleSheetList;
 }
 
+ExceptionOr<void> ShadowRoot::replaceChildrenWithMarkup(const String& markup, OptionSet<ParserContentPolicy> parserContentPolicy)
+{
+    auto policy = OptionSet<ParserContentPolicy> { ParserContentPolicy::AllowScriptingContent } | parserContentPolicy;
+
+    if (markup.isEmpty()) {
+        ChildListMutationScope mutation(*this);
+        removeChildren();
+        return { };
+    }
+
+    auto fragment = createFragmentForInnerOuterHTML(*protectedHost(), markup, policy);
+    if (fragment.hasException())
+        return fragment.releaseException();
+    return replaceChildrenWithFragment(*this, fragment.releaseReturnValue());
+}
+
+ExceptionOr<void> ShadowRoot::setHTMLUnsafe(const String& html)
+{
+    return replaceChildrenWithMarkup(html, { ParserContentPolicy::AllowDeclarativeShadowRoots, ParserContentPolicy::AlwaysParseAsHTML });
+}
+
 String ShadowRoot::innerHTML() const
 {
     return serializeFragment(*this, SerializedNodes::SubtreesOfChildren, nullptr, ResolveURLs::NoExcludingURLsForPrivacy);
@@ -185,16 +218,7 @@ String ShadowRoot::innerHTML() const
 
 ExceptionOr<void> ShadowRoot::setInnerHTML(const String& markup)
 {
-    if (markup.isEmpty()) {
-        ChildListMutationScope mutation(*this);
-        removeChildren();
-        return { };
-    }
-
-    auto fragment = createFragmentForInnerOuterHTML(*host(), markup, AllowScriptingContent);
-    if (fragment.hasException())
-        return fragment.releaseException();
-    return replaceChildrenWithFragment(*this, fragment.releaseReturnValue());
+    return replaceChildrenWithMarkup(markup, { });
 }
 
 bool ShadowRoot::childTypeAllowed(NodeType type) const
@@ -211,22 +235,26 @@ bool ShadowRoot::childTypeAllowed(NodeType type) const
     }
 }
 
-void ShadowRoot::setResetStyleInheritance(bool value)
+Ref<Node> ShadowRoot::cloneNodeInternal(Document& targetDocument, CloningOperation type)
 {
-    // If this was ever changed after initialization, child styles would need to be invalidated here.
-    m_resetStyleInheritance = value;
-}
+    RELEASE_ASSERT(m_mode != ShadowRootMode::UserAgent);
+    switch (type) {
+    case CloningOperation::SelfWithTemplateContent:
+        return create(targetDocument, m_mode, m_slotAssignmentMode, m_delegatesFocus ? DelegatesFocus::Yes : DelegatesFocus::No,
+            m_isCloneable ? Cloneable::Yes : Cloneable::No, m_availableToElementInternals ? AvailableToElementInternals::Yes : AvailableToElementInternals::No);
+    case CloningOperation::OnlySelf:
+    case CloningOperation::Everything:
+        break;
+    }
 
-Ref<Node> ShadowRoot::cloneNodeInternal(Document&, CloningOperation)
-{
-    RELEASE_ASSERT_NOT_REACHED();
-    return *static_cast<Node*>(nullptr); // ShadowRoots should never be cloned.
+    RELEASE_ASSERT_NOT_REACHED(); // ShadowRoot is never cloned directly on its own.
+    return *this;
 }
 
 void ShadowRoot::removeAllEventListeners()
 {
     DocumentFragment::removeAllEventListeners();
-    for (Node* node = firstChild(); node; node = NodeTraversal::next(*node))
+    for (RefPtr node = firstChild(); node; node = NodeTraversal::next(*node))
         node->removeAllEventListeners();
 }
 
@@ -234,9 +262,7 @@ void ShadowRoot::removeAllEventListeners()
 HTMLSlotElement* ShadowRoot::findAssignedSlot(const Node& node)
 {
     ASSERT(node.parentNode() == host());
-    if (!m_slotAssignment)
-        return nullptr;
-    return m_slotAssignment->findAssignedSlot(node);
+    return m_slotAssignment ? m_slotAssignment->findAssignedSlot(node) : nullptr;
 }
 
 void ShadowRoot::renameSlotElement(HTMLSlotElement& slot, const AtomString& oldName, const AtomString& newName)
@@ -248,8 +274,12 @@ void ShadowRoot::renameSlotElement(HTMLSlotElement& slot, const AtomString& oldN
 void ShadowRoot::addSlotElementByName(const AtomString& name, HTMLSlotElement& slot)
 {
     ASSERT(&slot.rootNode() == this);
-    if (!m_slotAssignment)
-        m_slotAssignment = makeUnique<SlotAssignment>();
+    if (!m_slotAssignment) {
+        if (m_slotAssignmentMode == SlotAssignmentMode::Named)
+            m_slotAssignment = makeUnique<NamedSlotAssignment>();
+        else
+            m_slotAssignment = makeUnique<ManualSlotAssignment>();
+    }
 
     return m_slotAssignment->addSlotElementByName(name, slot, *this);
 }
@@ -260,17 +290,27 @@ void ShadowRoot::removeSlotElementByName(const AtomString& name, HTMLSlotElement
     return m_slotAssignment->removeSlotElementByName(name, slot, &oldParentOfRemovedTree, *this);
 }
 
+void ShadowRoot::slotManualAssignmentDidChange(HTMLSlotElement& slot, Vector<WeakPtr<Node, WeakPtrImplWithEventTargetData>>& previous, Vector<WeakPtr<Node, WeakPtrImplWithEventTargetData>>& current)
+{
+    ASSERT(m_slotAssignment);
+    m_slotAssignment->slotManualAssignmentDidChange(slot, previous, current, *this);
+}
+
+void ShadowRoot::didRemoveManuallyAssignedNode(HTMLSlotElement& slot, const Node& node)
+{
+    ASSERT(m_slotAssignment);
+    m_slotAssignment->didRemoveManuallyAssignedNode(slot, node, *this);
+}
+
 void ShadowRoot::slotFallbackDidChange(HTMLSlotElement& slot)
 {
     ASSERT(&slot.rootNode() == this);
     return m_slotAssignment->slotFallbackDidChange(slot, *this);
 }
 
-const Vector<WeakPtr<Node>>* ShadowRoot::assignedNodesForSlot(const HTMLSlotElement& slot)
+const Vector<WeakPtr<Node, WeakPtrImplWithEventTargetData>>* ShadowRoot::assignedNodesForSlot(const HTMLSlotElement& slot)
 {
-    if (!m_slotAssignment)
-        return nullptr;
-    return m_slotAssignment->assignedNodesForSlot(slot, *this);
+    return m_slotAssignment ? m_slotAssignment->assignedNodesForSlot(slot, *this) : nullptr;
 }
 
 static std::optional<std::pair<AtomString, AtomString>> parsePartMapping(StringView mappingString)
@@ -278,13 +318,13 @@ static std::optional<std::pair<AtomString, AtomString>> parsePartMapping(StringV
     const auto end = mappingString.length();
 
     auto skipWhitespace = [&](auto position) {
-        while (position < end && isHTMLSpace(mappingString[position]))
+        while (position < end && isASCIIWhitespace(mappingString[position]))
             ++position;
         return position;
     };
 
     auto collectValue = [&](auto position) {
-        while (position < end && (!isHTMLSpace(mappingString[position]) && mappingString[position] != ':'))
+        while (position < end && (!isASCIIWhitespace(mappingString[position]) && mappingString[position] != ':'))
             ++position;
         return position;
     };
@@ -360,12 +400,12 @@ void ShadowRoot::invalidatePartMappings()
     m_partMappings = { };
 }
 
-Vector<ShadowRoot*> assignedShadowRootsIfSlotted(const Node& node)
+Vector<Ref<ShadowRoot>> assignedShadowRootsIfSlotted(const Node& node)
 {
-    Vector<ShadowRoot*> result;
+    Vector<Ref<ShadowRoot>> result;
     for (auto* slot = node.assignedSlot(); slot; slot = slot->assignedSlot()) {
         ASSERT(slot->containingShadowRoot());
-        result.append(slot->containingShadowRoot());
+        result.append(*slot->containingShadowRoot());
     }
     return result;
 }
