@@ -49,15 +49,14 @@ private:
     SimulatedMouseEvent(const AtomString& eventType, RefPtr<WindowProxy>&& view, RefPtr<Event>&& underlyingEvent, Element& target, SimulatedClickSource source)
         : MouseEvent(eventType, CanBubble::Yes, IsCancelable::Yes, IsComposed::Yes,
             underlyingEvent ? underlyingEvent->timeStamp() : MonotonicTime::now(), WTFMove(view), /* detail */ 0,
-            { }, { }, 0, 0, modifiersFromUnderlyingEvent(underlyingEvent), 0, 0, nullptr, 0, SyntheticClickType::NoTap, IsSimulated::Yes,
+            { }, { }, 0, 0, modifiersFromUnderlyingEvent(underlyingEvent), MouseButton::Left, 0, nullptr, 0, SyntheticClickType::NoTap, IsSimulated::Yes,
             source == SimulatedClickSource::UserAgent ? IsTrusted::Yes : IsTrusted::No)
     {
         setUnderlyingEvent(underlyingEvent.get());
 
-        if (is<MouseEvent>(this->underlyingEvent())) {
-            MouseEvent& mouseEvent = downcast<MouseEvent>(*this->underlyingEvent());
-            m_screenLocation = mouseEvent.screenLocation();
-            initCoordinates(mouseEvent.clientLocation());
+        if (auto* mouseEvent = dynamicDowncast<MouseEvent>(this->underlyingEvent())) {
+            m_screenLocation = mouseEvent->screenLocation();
+            initCoordinates(mouseEvent->clientLocation());
         } else if (source == SimulatedClickSource::UserAgent) {
             // If there is no underlying event, we only populate the coordinates for events coming
             // from the user agent (e.g. accessibility). For those coming from JavaScript (e.g.
@@ -79,7 +78,7 @@ private:
 
 static void simulateMouseEvent(const AtomString& eventType, Element& element, Event* underlyingEvent, SimulatedClickSource source)
 {
-    element.dispatchEvent(SimulatedMouseEvent::create(eventType, element.document().windowProxy(), underlyingEvent, element, source));
+    element.dispatchEvent(SimulatedMouseEvent::create(eventType, element.document().protectedWindowProxy().get(), underlyingEvent, element, source));
 }
 
 bool simulateClick(Element& element, Event* underlyingEvent, SimulatedClickMouseEventOptions mouseEventOptions, SimulatedClickVisualOptions visualOptions, SimulatedClickSource creationOptions)
@@ -87,8 +86,8 @@ bool simulateClick(Element& element, Event* underlyingEvent, SimulatedClickMouse
     if (element.isDisabledFormControl())
         return false;
 
-    static MainThreadNeverDestroyed<HashSet<Element*>> elementsDispatchingSimulatedClicks;
-    if (!elementsDispatchingSimulatedClicks.get().add(&element).isNewEntry)
+    static MainThreadNeverDestroyed<HashSet<Ref<Element>>> elementsDispatchingSimulatedClicks;
+    if (!elementsDispatchingSimulatedClicks.get().add(element).isNewEntry)
         return false;
 
     auto& eventNames = WebCore::eventNames();
@@ -105,7 +104,7 @@ bool simulateClick(Element& element, Event* underlyingEvent, SimulatedClickMouse
 
     simulateMouseEvent(eventNames.clickEvent, element, underlyingEvent, creationOptions);
 
-    elementsDispatchingSimulatedClicks.get().remove(&element);
+    elementsDispatchingSimulatedClicks.get().remove(element);
     return true;
 }
 
