@@ -28,14 +28,17 @@
 #include "AlternativeTextController.h"
 
 #include "Document.h"
+#include "DocumentFragment.h"
+#include "DocumentInlines.h"
 #include "DocumentMarkerController.h"
 #include "Editing.h"
 #include "Editor.h"
 #include "EditorClient.h"
 #include "Element.h"
+#include "EventLoop.h"
 #include "FloatQuad.h"
-#include "Frame.h"
-#include "FrameView.h"
+#include "LocalFrame.h"
+#include "LocalFrameView.h"
 #include "Page.h"
 #include "Range.h"
 #include "RenderedDocumentMarker.h"
@@ -52,26 +55,26 @@ namespace WebCore {
 
 #if USE(DICTATION_ALTERNATIVES) || USE(AUTOCORRECTION_PANEL)
 
-constexpr OptionSet<DocumentMarker::MarkerType> markerTypesForAppliedDictationAlternative()
+constexpr OptionSet<DocumentMarker::Type> markerTypesForAppliedDictationAlternative()
 {
-    return DocumentMarker::SpellCheckingExemption;
+    return DocumentMarker::Type::SpellCheckingExemption;
 }
 
 #endif
 
 #if USE(AUTOCORRECTION_PANEL)
 
-static inline OptionSet<DocumentMarker::MarkerType> markerTypesForAutocorrection()
+static inline OptionSet<DocumentMarker::Type> markerTypesForAutocorrection()
 {
-    return { DocumentMarker::Autocorrected, DocumentMarker::CorrectionIndicator, DocumentMarker::Replacement, DocumentMarker::SpellCheckingExemption };
+    return { DocumentMarker::Type::Autocorrected, DocumentMarker::Type::CorrectionIndicator, DocumentMarker::Type::Replacement, DocumentMarker::Type::SpellCheckingExemption };
 }
 
-static inline OptionSet<DocumentMarker::MarkerType> markerTypesForReplacement()
+static inline OptionSet<DocumentMarker::Type> markerTypesForReplacement()
 {
-    return { DocumentMarker::Replacement, DocumentMarker::SpellCheckingExemption };
+    return { DocumentMarker::Type::Replacement, DocumentMarker::Type::SpellCheckingExemption };
 }
 
-static bool markersHaveIdenticalDescription(const Vector<RenderedDocumentMarker*>& markers)
+static bool markersHaveIdenticalDescription(const Vector<WeakPtr<RenderedDocumentMarker>>& markers)
 {
     if (markers.isEmpty())
         return true;
@@ -85,44 +88,55 @@ static bool markersHaveIdenticalDescription(const Vector<RenderedDocumentMarker*
 }
 
 AlternativeTextController::AlternativeTextController(Document& document)
-    : m_timer(*this, &AlternativeTextController::timerFired)
-    , m_document(document)
+    : m_document(document)
 {
 }
 
 AlternativeTextController::~AlternativeTextController()
 {
-    dismiss(ReasonForDismissingAlternativeTextIgnored);
+    dismiss(ReasonForDismissingAlternativeText::Ignored);
 }
 
 void AlternativeTextController::startAlternativeTextUITimer(AlternativeTextType type)
 {
     const Seconds correctionPanelTimerInterval { 300_ms };
-    if (!isAutomaticSpellingCorrectionEnabled())
+
+    if (!isAutomaticSpellingCorrectionEnabled()) {
+#if !ENABLE(ALTERNATIVE_TEXT_REQUIRES_AUTOMATIC_SPELLING_CORRECTION)
+        // Exclude correction & reversion bubbles which have accept on dismiss behavior.
+        if (type == AlternativeTextType::Correction || type == AlternativeTextType::Reversion)
+            return;
+#else
         return;
+#endif
+    }
 
     // If type is PanelTypeReversion, then the new range has been set. So we shouldn't clear it.
     if (type == AlternativeTextType::Correction)
         m_rangeWithAlternative = std::nullopt;
     m_type = type;
-    m_timer.startOneShot(correctionPanelTimerInterval);
+    m_timer = protectedDocument()->eventLoop().scheduleTask(correctionPanelTimerInterval, TaskSource::UserInteraction, [weakThis = WeakPtr { *this }] {
+        if (!weakThis)
+            return;
+        weakThis->timerFired();
+    });
 }
 
 void AlternativeTextController::stopAlternativeTextUITimer()
 {
-    m_timer.stop();
+    m_timer = nullptr;
     m_rangeWithAlternative = std::nullopt;
 }
 
 void AlternativeTextController::stopPendingCorrection(const VisibleSelection& oldSelection)
 {
     // Make sure there's no pending autocorrection before we call markMisspellingsAndBadGrammar() below.
-    VisibleSelection currentSelection(m_document.selection().selection());
+    VisibleSelection currentSelection(protectedDocument()->selection().selection());
     if (currentSelection == oldSelection)
         return;
 
     stopAlternativeTextUITimer();
-    dismiss(ReasonForDismissingAlternativeTextIgnored);
+    dismiss(ReasonForDismissingAlternativeText::Ignored);
 }
 
 void AlternativeTextController::applyPendingCorrection(const VisibleSelection& selectionAfterTyping)
@@ -130,7 +144,7 @@ void AlternativeTextController::applyPendingCorrection(const VisibleSelection& s
     // Apply pending autocorrection before next round of spell checking.
     bool doApplyCorrection = true;
     VisiblePosition startOfSelection = selectionAfterTyping.visibleStart();
-    VisibleSelection currentWord = VisibleSelection(startOfWord(startOfSelection, LeftWordIfOnBoundary), endOfWord(startOfSelection, RightWordIfOnBoundary));
+    VisibleSelection currentWord = VisibleSelection(startOfWord(startOfSelection, WordSide::LeftWordIfOnBoundary), endOfWord(startOfSelection, WordSide::RightWordIfOnBoundary));
     if (currentWord.visibleEnd() == startOfSelection) {
         if (auto wordRange = currentWord.firstRange()) {
             String wordText = plainText(*wordRange);
@@ -139,7 +153,7 @@ void AlternativeTextController::applyPendingCorrection(const VisibleSelection& s
         }
     }
     if (doApplyCorrection)
-        handleAlternativeTextUIResult(dismissSoon(ReasonForDismissingAlternativeTextAccepted)); 
+        handleAlternativeTextUIResult(dismissSoon(ReasonForDismissingAlternativeText::Accepted));
     else
         m_rangeWithAlternative = std::nullopt;
 }
@@ -151,7 +165,7 @@ bool AlternativeTextController::hasPendingCorrection() const
 
 bool AlternativeTextController::isSpellingMarkerAllowed(const SimpleRange& misspellingRange) const
 {
-    return !m_document.markers().hasMarkers(misspellingRange, DocumentMarker::SpellCheckingExemption);
+    return !protectedDocument()->markers().hasMarkers(misspellingRange, DocumentMarker::Type::SpellCheckingExemption);
 }
 
 void AlternativeTextController::show(const SimpleRange& rangeToReplace, const String& replacement)
@@ -163,7 +177,7 @@ void AlternativeTextController::show(const SimpleRange& rangeToReplace, const St
     m_rangeWithAlternative = rangeToReplace;
     m_details = replacement;
     m_isActive = true;
-    if (AlternativeTextClient* client = alternativeTextClient())
+    if (CheckedPtr client = alternativeTextClient())
         client->showCorrectionAlternative(m_type, boundingBox, m_originalText, replacement, { });
 }
 
@@ -172,7 +186,7 @@ void AlternativeTextController::handleCancelOperation()
     if (!m_isActive)
         return;
     m_isActive = false;
-    dismiss(ReasonForDismissingAlternativeTextCancelled);
+    dismiss(ReasonForDismissingAlternativeText::Cancelled);
 }
 
 void AlternativeTextController::dismiss(ReasonForDismissingAlternativeText reasonForDismissing)
@@ -181,7 +195,7 @@ void AlternativeTextController::dismiss(ReasonForDismissingAlternativeText reaso
         return;
     m_isActive = false;
     m_isDismissedByEditing = true;
-    if (AlternativeTextClient* client = alternativeTextClient())
+    if (CheckedPtr client = alternativeTextClient())
         client->dismissAlternative(reasonForDismissing);
 }
 
@@ -191,7 +205,7 @@ String AlternativeTextController::dismissSoon(ReasonForDismissingAlternativeText
         return String();
     m_isActive = false;
     m_isDismissedByEditing = true;
-    if (AlternativeTextClient* client = alternativeTextClient())
+    if (CheckedPtr client = alternativeTextClient())
         return client->dismissAlternativeSoon(reasonForDismissing);
     return String();
 }
@@ -204,34 +218,35 @@ bool AlternativeTextController::applyAutocorrectionBeforeTypingIfAppropriate()
     if (m_type != AlternativeTextType::Correction)
         return false;
 
-    Position caretPosition = m_document.selection().selection().start();
+    Position caretPosition = protectedDocument()->selection().selection().start();
 
     if (makeDeprecatedLegacyPosition(m_rangeWithAlternative->end) == caretPosition) {
-        handleAlternativeTextUIResult(dismissSoon(ReasonForDismissingAlternativeTextAccepted));
+        handleAlternativeTextUIResult(dismissSoon(ReasonForDismissingAlternativeText::Accepted));
         return true;
     } 
     
     // Pending correction should always be where caret is. But in case this is not always true, we still want to dismiss the panel without accepting the correction.
     ASSERT(makeDeprecatedLegacyPosition(m_rangeWithAlternative->end) == caretPosition);
-    dismiss(ReasonForDismissingAlternativeTextIgnored);
+    dismiss(ReasonForDismissingAlternativeText::Ignored);
     return false;
 }
 
 void AlternativeTextController::respondToUnappliedSpellCorrection(const VisibleSelection& selectionOfCorrected, const String& corrected, const String& correction)
 {
-    if (auto client = alternativeTextClient())
+    if (CheckedPtr client = alternativeTextClient())
         client->recordAutocorrectionResponse(AutocorrectionResponse::Reverted, corrected, correction);
 
-    RefPtr<Frame> protector(m_document.frame());
-    m_document.updateLayout();
+    auto document = protectedDocument();
+    RefPtr protectedFrame { document->frame() };
+    document->updateLayout();
 
-    m_document.selection().setSelection(selectionOfCorrected, FrameSelection::defaultSetSelectionOptions() | FrameSelection::SpellCorrectionTriggered);
-    auto range = m_document.selection().selection().firstRange();
+    document->selection().setSelection(selectionOfCorrected, FrameSelection::defaultSetSelectionOptions() | FrameSelection::SetSelectionOption::SpellCorrectionTriggered);
+    auto range = document->selection().selection().firstRange();
     if (!range)
         return;
-    removeMarkers(*range, OptionSet<DocumentMarker::MarkerType> { DocumentMarker::Spelling, DocumentMarker::Autocorrected }, RemovePartiallyOverlappingMarker::Yes);
-    addMarker(*range, DocumentMarker::Replacement);
-    addMarker(*range, DocumentMarker::SpellCheckingExemption);
+    removeMarkers(*range, OptionSet<DocumentMarker::Type> { DocumentMarker::Type::Spelling, DocumentMarker::Type::Autocorrected }, RemovePartiallyOverlappingMarker::Yes);
+    addMarker(*range, DocumentMarker::Type::Replacement);
+    addMarker(*range, DocumentMarker::Type::SpellCheckingExemption);
 }
 
 void AlternativeTextController::timerFired()
@@ -239,12 +254,13 @@ void AlternativeTextController::timerFired()
     m_isDismissedByEditing = false;
     switch (m_type) {
     case AlternativeTextType::Correction: {
-        VisibleSelection selection(m_document.selection().selection());
+        auto document = protectedDocument();
+        VisibleSelection selection(document->selection().selection());
         VisiblePosition start(selection.start(), selection.affinity());
-        VisiblePosition p = startOfWord(start, LeftWordIfOnBoundary);
+        VisiblePosition p = startOfWord(start, WordSide::LeftWordIfOnBoundary);
         VisibleSelection adjacentWords = VisibleSelection(p, start);
         auto adjacentWordRange = adjacentWords.toNormalizedRange();
-        m_document.editor().markAllMisspellingsAndBadGrammarInRanges({ TextCheckingType::Spelling, TextCheckingType::Replacement, TextCheckingType::ShowCorrectionPanel }, adjacentWordRange, adjacentWordRange, std::nullopt);
+        document->editor().markAllMisspellingsAndBadGrammarInRanges({ TextCheckingType::Spelling, TextCheckingType::Replacement, TextCheckingType::ShowCorrectionPanel }, adjacentWordRange, adjacentWordRange, std::nullopt);
     }
         break;
     case AlternativeTextType::Reversion: {
@@ -257,17 +273,29 @@ void AlternativeTextController::timerFired()
         m_originalText = plainText(*m_rangeWithAlternative);
         auto boundingBox = rootViewRectForRange(*m_rangeWithAlternative);
         if (!boundingBox.isEmpty()) {
-            if (AlternativeTextClient* client = alternativeTextClient())
+            if (CheckedPtr client = alternativeTextClient()) {
+                removeMarkers(*m_rangeWithAlternative, { DocumentMarker::Type::CorrectionIndicator });
                 client->showCorrectionAlternative(m_type, boundingBox, m_originalText, replacementString, { });
+            }
         }
     }
         break;
-    case AlternativeTextType::SpellingSuggestions: {
+    case AlternativeTextType::SpellingSuggestions:
+    case AlternativeTextType::GrammarSuggestions: {
         if (!m_rangeWithAlternative || plainText(*m_rangeWithAlternative) != m_originalText)
             break;
-        String paragraphText = plainText(TextCheckingParagraph(*m_rangeWithAlternative).paragraphRange());
+
         Vector<String> suggestions;
-        textChecker()->getGuessesForWord(m_originalText, paragraphText, m_document.selection().selection(), suggestions);
+        if (m_type == AlternativeTextType::GrammarSuggestions) {
+            if (CheckedPtr editorClient = this->editorClient()) {
+                TextCheckingHelper checker(*editorClient, *m_rangeWithAlternative);
+                suggestions = checker.guessesForMisspelledWordOrUngrammaticalPhrase(true).guesses;
+            }
+        } else {
+            auto paragraphText = plainText(TextCheckingParagraph(*m_rangeWithAlternative).paragraphRange());
+            textChecker()->getGuessesForWord(m_originalText, paragraphText, protectedDocument()->selection().selection(), suggestions);
+        }
+
         if (suggestions.isEmpty()) {
             m_rangeWithAlternative = std::nullopt;
             break;
@@ -277,7 +305,7 @@ void AlternativeTextController::timerFired()
         m_isActive = true;
         auto boundingBox = rootViewRectForRange(*m_rangeWithAlternative);
         if (!boundingBox.isEmpty()) {
-            if (AlternativeTextClient* client = alternativeTextClient())
+            if (CheckedPtr client = alternativeTextClient())
                 client->showCorrectionAlternative(m_type, boundingBox, m_originalText, topSuggestion, suggestions);
         }
     }
@@ -293,7 +321,7 @@ void AlternativeTextController::timerFired()
         auto boundingBox = rootViewRectForRange(*m_rangeWithAlternative);
         m_isActive = true;
         if (!boundingBox.isEmpty()) {
-            if (auto client = alternativeTextClient())
+            if (CheckedPtr client = alternativeTextClient())
                 client->showDictationAlternativeUI(boundingBox, dictationContext);
         }
 #endif
@@ -304,7 +332,7 @@ void AlternativeTextController::timerFired()
 
 void AlternativeTextController::handleAlternativeTextUIResult(const String& result)
 {
-    if (!m_rangeWithAlternative || &m_document != &m_rangeWithAlternative->start.document())
+    if (!m_rangeWithAlternative || m_document.ptr() != &m_rangeWithAlternative->start.document())
         return;
 
     String currentWord = plainText(*m_rangeWithAlternative);
@@ -319,10 +347,11 @@ void AlternativeTextController::handleAlternativeTextUIResult(const String& resu
         if (result.length())
             applyAlternativeTextToRange(*m_rangeWithAlternative, result, m_type, markerTypesForAutocorrection());
         else if (!m_isDismissedByEditing)
-            addMarker(*m_rangeWithAlternative, DocumentMarker::RejectedCorrection, m_originalText);
+            addMarker(*m_rangeWithAlternative, DocumentMarker::Type::RejectedCorrection, m_originalText);
         break;
     case AlternativeTextType::Reversion:
     case AlternativeTextType::SpellingSuggestions:
+    case AlternativeTextType::GrammarSuggestions:
         if (result.length())
             applyAlternativeTextToRange(*m_rangeWithAlternative, result, m_type, markerTypesForReplacement());
         break;
@@ -338,11 +367,11 @@ void AlternativeTextController::handleAlternativeTextUIResult(const String& resu
 bool AlternativeTextController::canEnableAutomaticSpellingCorrection() const
 {
 #if ENABLE(AUTOCORRECT)
-    auto position = m_document.selection().selection().start();
+    auto position = protectedDocument()->selection().selection().start();
     if (RefPtr control = enclosingTextFormControl(position)) {
         if (!control->shouldAutocorrect())
             return false;
-    } else if (RefPtr editableRoot = position.rootEditableElement(); is<HTMLElement>(editableRoot) && !downcast<HTMLElement>(*editableRoot).shouldAutocorrect())
+    } else if (RefPtr editableRoot = dynamicDowncast<HTMLElement>(position.rootEditableElement()); editableRoot && !editableRoot->shouldAutocorrect())
         return false;
 #endif
 
@@ -359,7 +388,7 @@ bool AlternativeTextController::isAutomaticSpellingCorrectionEnabled()
 
 FloatRect AlternativeTextController::rootViewRectForRange(const SimpleRange& range) const
 {
-    auto* view = m_document.view();
+    RefPtr view = m_document->view();
     if (!view)
         return { };
     return view->contentsToRootView(unitedBoundingBoxes(RenderObject::absoluteTextQuads(range)));
@@ -367,7 +396,7 @@ FloatRect AlternativeTextController::rootViewRectForRange(const SimpleRange& ran
 
 void AlternativeTextController::respondToChangedSelection(const VisibleSelection& oldSelection)
 {
-    VisibleSelection currentSelection(m_document.selection().selection());
+    VisibleSelection currentSelection(protectedDocument()->selection().selection());
     // When user moves caret to the end of autocorrected word and pauses, we show the panel
     // containing the original pre-correction word so that user can quickly revert the
     // undesired autocorrection. Here, we start correction panel timer once we confirm that
@@ -382,7 +411,7 @@ void AlternativeTextController::respondToChangedSelection(const VisibleSelection
     if (selectionPosition.isNull())
         return;
     
-    VisiblePosition endPositionOfWord = endOfWord(selectionPosition, LeftWordIfOnBoundary);
+    VisiblePosition endPositionOfWord = endOfWord(selectionPosition, WordSide::LeftWordIfOnBoundary);
     if (selectionPosition != endPositionOfWord)
         return;
 
@@ -390,26 +419,15 @@ void AlternativeTextController::respondToChangedSelection(const VisibleSelection
     if (position.anchorType() != Position::PositionIsOffsetInAnchor)
         return;
 
-    Node* node = position.containerNode();
+    RefPtr node = position.containerNode();
     ASSERT(node);
-    for (auto* marker : node->document().markers().markersFor(*node)) {
+    for (auto& marker : node->document().markers().markersFor(*node)) {
         ASSERT(marker);
         if (respondToMarkerAtEndOfWord(*marker, position))
             break;
     }
 }
 
-void AlternativeTextController::respondToAppliedEditing(CompositeEditCommand* command)
-{
-    if (command->isTopLevelCommand() && !command->shouldRetainAutocorrectionIndicator())
-        m_document.markers().removeMarkers(DocumentMarker::CorrectionIndicator);
-
-    markPrecedingWhitespaceForDeletedAutocorrectionAfterCommand(command);
-    m_originalStringForLastDeletedAutocorrection = String();
-
-    dismiss(ReasonForDismissingAlternativeTextIgnored);
-}
-
 void AlternativeTextController::respondToUnappliedEditing(EditCommandComposition* command)
 {
     if (!command->wasCreateLinkCommand())
@@ -417,38 +435,38 @@ void AlternativeTextController::respondToUnappliedEditing(EditCommandComposition
     auto range = command->startingSelection().firstRange();
     if (!range)
         return;
-    addMarker(*range, DocumentMarker::Replacement);
-    addMarker(*range, DocumentMarker::SpellCheckingExemption);
+    addMarker(*range, DocumentMarker::Type::Replacement);
+    addMarker(*range, DocumentMarker::Type::SpellCheckingExemption);
 }
 
 EditorClient* AlternativeTextController::editorClient()
 {
-    return m_document.page() ? &m_document.page()->editorClient() : nullptr;
+    return m_document->page() ? &m_document->page()->editorClient() : nullptr;
 }
 
 TextCheckerClient* AlternativeTextController::textChecker()
 {
-    if (EditorClient* owner = editorClient())
+    if (CheckedPtr owner = editorClient())
         return owner->textChecker();
     return nullptr;
 }
 
 void AlternativeTextController::recordAutocorrectionResponse(AutocorrectionResponse response, const String& replacedString, const SimpleRange& replacementRange)
 {
-    if (auto client = alternativeTextClient())
+    if (CheckedPtr client = alternativeTextClient())
         client->recordAutocorrectionResponse(response, replacedString, plainText(replacementRange));
 }
 
 void AlternativeTextController::markReversed(const SimpleRange& changedRange)
 {
-    removeMarkers(changedRange, DocumentMarker::Autocorrected, RemovePartiallyOverlappingMarker::Yes);
-    addMarker(changedRange, DocumentMarker::SpellCheckingExemption);
+    removeMarkers(changedRange, DocumentMarker::Type::Autocorrected, RemovePartiallyOverlappingMarker::Yes);
+    addMarker(changedRange, DocumentMarker::Type::SpellCheckingExemption);
 }
 
 void AlternativeTextController::markCorrection(const SimpleRange& replacedRange, const String& replacedString)
 {
     for (auto markerType : markerTypesForAutocorrection()) {
-        if (markerType == DocumentMarker::Replacement || markerType == DocumentMarker::Autocorrected)
+        if (markerType == DocumentMarker::Type::Replacement || markerType == DocumentMarker::Type::Autocorrected)
             addMarker(replacedRange, markerType, replacedString);
         else
             addMarker(replacedRange, markerType);
@@ -458,11 +476,11 @@ void AlternativeTextController::markCorrection(const SimpleRange& replacedRange,
 void AlternativeTextController::recordSpellcheckerResponseForModifiedCorrection(const SimpleRange& rangeOfCorrection, const String& corrected, const String& correction)
 {
     DocumentMarkerController& markers = rangeOfCorrection.startContainer().document().markers();
-    Vector<RenderedDocumentMarker*> correctedOnceMarkers = markers.markersInRange(rangeOfCorrection, DocumentMarker::Autocorrected);
+    auto correctedOnceMarkers = markers.markersInRange(rangeOfCorrection, DocumentMarker::Type::Autocorrected);
     if (correctedOnceMarkers.isEmpty())
         return;
 
-    if (AlternativeTextClient* client = alternativeTextClient()) {
+    if (CheckedPtr client = alternativeTextClient()) {
         // Spelling corrected text has been edited. We need to determine whether user has reverted it to original text or
         // edited it to something else, and notify spellchecker accordingly.
         if (markersHaveIdenticalDescription(correctedOnceMarkers) && correctedOnceMarkers[0]->description() == corrected)
@@ -471,7 +489,7 @@ void AlternativeTextController::recordSpellcheckerResponseForModifiedCorrection(
             client->recordAutocorrectionResponse(AutocorrectionResponse::Edited, corrected, correction);
     }
 
-    removeMarkers(rangeOfCorrection, DocumentMarker::Autocorrected, RemovePartiallyOverlappingMarker::Yes);
+    removeMarkers(rangeOfCorrection, DocumentMarker::Type::Autocorrected, RemovePartiallyOverlappingMarker::Yes);
 }
 
 void AlternativeTextController::deletedAutocorrectionAtPosition(const Position& position, const String& originalString)
@@ -500,29 +518,30 @@ void AlternativeTextController::markPrecedingWhitespaceForDeletedAutocorrectionA
     // Mark this whitespace to indicate we have deleted an autocorrection following this
     // whitespace. So if the user types the same original word again at this position, we
     // won't autocorrect it again.
-    addMarker(*precedingCharacterRange, DocumentMarker::DeletedAutocorrection, m_originalStringForLastDeletedAutocorrection);
+    addMarker(*precedingCharacterRange, DocumentMarker::Type::DeletedAutocorrection, m_originalStringForLastDeletedAutocorrection);
 }
 
 bool AlternativeTextController::processMarkersOnTextToBeReplacedByResult(const TextCheckingResult& result, const SimpleRange& rangeWithAlternative, const String& stringToBeReplaced)
 {
-    auto& markers = m_document.markers();
-    if (markers.hasMarkers(rangeWithAlternative, DocumentMarker::Replacement)) {
+    auto document = protectedDocument();
+    auto& markers = document->markers();
+    if (markers.hasMarkers(rangeWithAlternative, DocumentMarker::Type::Replacement)) {
         if (result.type == TextCheckingType::Correction)
             recordSpellcheckerResponseForModifiedCorrection(rangeWithAlternative, stringToBeReplaced, result.replacement);
         return false;
     }
 
-    if (markers.hasMarkers(rangeWithAlternative, DocumentMarker::RejectedCorrection))
+    if (markers.hasMarkers(rangeWithAlternative, DocumentMarker::Type::RejectedCorrection))
         return false;
 
-    if (markers.hasMarkers(rangeWithAlternative, DocumentMarker::AcceptedCandidate))
+    if (markers.hasMarkers(rangeWithAlternative, DocumentMarker::Type::AcceptedCandidate))
         return false;
 
     auto precedingCharacterRange = makeSimpleRange(makeDeprecatedLegacyPosition(rangeWithAlternative.start).previous(), rangeWithAlternative.start);
     if (!precedingCharacterRange)
         return false;
 
-    for (auto& marker : markers.markersInRange(*precedingCharacterRange, DocumentMarker::DeletedAutocorrection)) {
+    for (auto& marker : markers.markersInRange(*precedingCharacterRange, DocumentMarker::Type::DeletedAutocorrection)) {
         if (marker->description() == stringToBeReplaced)
             return false;
     }
@@ -532,31 +551,44 @@ bool AlternativeTextController::processMarkersOnTextToBeReplacedByResult(const T
 
 bool AlternativeTextController::shouldStartTimerFor(const WebCore::DocumentMarker &marker, int endOffset) const
 {
-    return (((marker.type() == DocumentMarker::Replacement && !marker.description().isNull()) || marker.type() == DocumentMarker::Spelling || marker.type() == DocumentMarker::DictationAlternatives) && static_cast<int>(marker.endOffset()) == endOffset);
+    if (static_cast<int>(marker.endOffset()) != endOffset)
+        return false;
+
+    switch (marker.type()) {
+    case DocumentMarker::Type::Spelling:
+    case DocumentMarker::Type::Grammar:
+    case DocumentMarker::Type::DictationAlternatives:
+        return true;
+    case DocumentMarker::Type::Replacement:
+        return !marker.description().isNull();
+    default:
+        return false;
+    }
 }
 
 bool AlternativeTextController::respondToMarkerAtEndOfWord(const DocumentMarker& marker, const Position& endOfWordPosition)
 {
     if (!shouldStartTimerFor(marker, endOfWordPosition.offsetInContainerNode()))
         return false;
-    Node* node = endOfWordPosition.containerNode();
+    RefPtr node = endOfWordPosition.containerNode();
     auto wordRange = makeSimpleRange(*node, marker);
     String currentWord = plainText(wordRange);
     if (!currentWord.length())
         return false;
     m_originalText = currentWord;
     switch (marker.type()) {
-    case DocumentMarker::Spelling:
+    case DocumentMarker::Type::Spelling:
+    case DocumentMarker::Type::Grammar:
         m_rangeWithAlternative = WTFMove(wordRange);
         m_details = emptyString();
-        startAlternativeTextUITimer(AlternativeTextType::SpellingSuggestions);
+        startAlternativeTextUITimer((marker.type() == DocumentMarker::Type::Spelling) ? AlternativeTextType::SpellingSuggestions : AlternativeTextType::GrammarSuggestions);
         break;
-    case DocumentMarker::Replacement:
+    case DocumentMarker::Type::Replacement:
         m_rangeWithAlternative = WTFMove(wordRange);
         m_details = marker.description();
         startAlternativeTextUITimer(AlternativeTextType::Reversion);
         break;
-    case DocumentMarker::DictationAlternatives: {
+    case DocumentMarker::Type::DictationAlternatives: {
         auto& markerData = std::get<DocumentMarker::DictationData>(marker.data());
         if (currentWord != markerData.originalText)
             return false;
@@ -578,13 +610,13 @@ bool AlternativeTextController::respondToMarkerAtEndOfWord(const DocumentMarker&
 
 AlternativeTextClient* AlternativeTextController::alternativeTextClient()
 {
-    return m_document.frame() && m_document.page() ? m_document.page()->alternativeTextClient() : nullptr;
+    return m_document->frame() && m_document->page() ? m_document->page()->alternativeTextClient() : nullptr;
 }
 
-String AlternativeTextController::markerDescriptionForAppliedAlternativeText(AlternativeTextType alternativeTextType, DocumentMarker::MarkerType markerType)
+String AlternativeTextController::markerDescriptionForAppliedAlternativeText(AlternativeTextType alternativeTextType, DocumentMarker::Type markerType)
 {
 #if USE(AUTOCORRECTION_PANEL)
-    if (alternativeTextType != AlternativeTextType::Reversion && alternativeTextType != AlternativeTextType::DictationAlternatives && (markerType == DocumentMarker::Replacement || markerType == DocumentMarker::Autocorrected))
+    if (alternativeTextType != AlternativeTextType::Reversion && alternativeTextType != AlternativeTextType::DictationAlternatives && (markerType == DocumentMarker::Type::Replacement || markerType == DocumentMarker::Type::Autocorrected))
         return m_originalText;
 #else
     UNUSED_PARAM(alternativeTextType);
@@ -593,7 +625,7 @@ String AlternativeTextController::markerDescriptionForAppliedAlternativeText(Alt
     return emptyString();
 }
 
-void AlternativeTextController::applyAlternativeTextToRange(const SimpleRange& range, const String& alternative, AlternativeTextType alternativeType, OptionSet<DocumentMarker::MarkerType> markerTypesToAdd)
+void AlternativeTextController::applyAlternativeTextToRange(const SimpleRange& range, const String& alternative, AlternativeTextType alternativeType, OptionSet<DocumentMarker::Type> markerTypesToAdd)
 {
     // After we replace the word at range rangeWithAlternative, we need to add markers to that range.
     // So before we carry out the replacement,store the start position relative to the start position
@@ -612,7 +644,7 @@ void AlternativeTextController::applyAlternativeTextToRange(const SimpleRange& r
 
     // Recalculate pragraphRangeContainingCorrection, since SpellingCorrectionCommand modified the DOM, such that the original paragraphRangeContainingCorrection is no longer valid. Radar: 10305315 Bugzilla: 89526
     auto updatedParagraphStartContainingCorrection = resolveCharacterLocation(makeRangeSelectingNodeContents(treeScopeRoot), paragraphOffsetInTreeScope);
-    auto updatedParagraphEndContainingCorrection = makeBoundaryPoint(m_document.selection().selection().start());
+    auto updatedParagraphEndContainingCorrection = makeBoundaryPoint(protectedDocument()->selection().selection().start());
     if (!updatedParagraphEndContainingCorrection)
         return;
     auto replacementRange = resolveCharacterRange({ updatedParagraphStartContainingCorrection, *updatedParagraphEndContainingCorrection }, CharacterRange(correctionOffsetInParagraph, alternative.length()));
@@ -625,19 +657,48 @@ void AlternativeTextController::applyAlternativeTextToRange(const SimpleRange& r
         addMarker(replacementRange, markerType, markerDescriptionForAppliedAlternativeText(alternativeType, markerType));
 }
 
+#endif // USE(AUTOCORRECTION_PANEL)
+
+void AlternativeTextController::removeCorrectionIndicatorMarkers()
+{
+#if HAVE(AUTOCORRECTION_ENHANCEMENTS)
+    protectedDocument()->markers().dismissMarkers(DocumentMarker::Type::CorrectionIndicator);
+#else
+    protectedDocument()->markers().removeMarkers(DocumentMarker::Type::CorrectionIndicator);
 #endif
+}
+
+void AlternativeTextController::respondToAppliedEditing(CompositeEditCommand* command)
+{
+#if USE(AUTOCORRECTION_PANEL)
+    if (command->isTopLevelCommand() && !command->shouldRetainAutocorrectionIndicator())
+        removeCorrectionIndicatorMarkers();
+
+    markPrecedingWhitespaceForDeletedAutocorrectionAfterCommand(command);
+    m_originalStringForLastDeletedAutocorrection = String();
+
+    dismiss(ReasonForDismissingAlternativeText::Ignored);
+#elif HAVE(AUTOCORRECTION_ENHANCEMENTS) && PLATFORM(IOS_FAMILY)
+    if (!command->shouldRetainAutocorrectionIndicator())
+        removeCorrectionIndicatorMarkers();
+#else
+    UNUSED_PARAM(command);
+#endif
+}
 
 bool AlternativeTextController::insertDictatedText(const String& text, const Vector<DictationAlternative>& dictationAlternatives, Event* triggeringEvent)
 {
-    EventTarget* target;
+    RefPtr<EventTarget> target;
+    auto document = protectedDocument();
     if (triggeringEvent)
         target = triggeringEvent->target();
     else
-        target = eventTargetElementForDocument(&m_document);
+        target = eventTargetElementForDocument(document.ptr());
     if (!target)
         return false;
 
-    auto event = TextEvent::createForDictation(&m_document.frame()->windowProxy(), text, dictationAlternatives);
+    Ref windowProxy = document->frame()->windowProxy();
+    auto event = TextEvent::createForDictation(windowProxy.ptr(), text, dictationAlternatives);
     event->setUnderlyingEvent(triggeringEvent);
 
     target->dispatchEvent(event);
@@ -647,7 +708,7 @@ bool AlternativeTextController::insertDictatedText(const String& text, const Vec
 void AlternativeTextController::removeDictationAlternativesForMarker(const DocumentMarker& marker)
 {
 #if USE(DICTATION_ALTERNATIVES)
-    if (auto* client = alternativeTextClient())
+    if (CheckedPtr client = alternativeTextClient())
         client->removeDictationAlternatives(std::get<DocumentMarker::DictationData>(marker.data()).context);
 #else
     UNUSED_PARAM(marker);
@@ -657,7 +718,7 @@ void AlternativeTextController::removeDictationAlternativesForMarker(const Docum
 Vector<String> AlternativeTextController::dictationAlternativesForMarker(const DocumentMarker& marker)
 {
 #if USE(DICTATION_ALTERNATIVES)
-    if (auto* client = alternativeTextClient())
+    if (CheckedPtr client = alternativeTextClient())
         return client->dictationAlternatives(std::get<DocumentMarker::DictationData>(marker.data()).context);
     return Vector<String>();
 #else
@@ -669,11 +730,12 @@ Vector<String> AlternativeTextController::dictationAlternativesForMarker(const D
 void AlternativeTextController::applyDictationAlternative(const String& alternativeString)
 {
 #if USE(DICTATION_ALTERNATIVES)
-    auto& editor = m_document.editor();
+    auto document = protectedDocument();
+    auto& editor = document->editor();
     auto selection = editor.selectedRange();
     if (!selection || !editor.shouldInsertText(alternativeString, *selection, EditorInsertAction::Pasted))
         return;
-    for (auto* marker : selection->startContainer().document().markers().markersInRange(*selection, DocumentMarker::DictationAlternatives))
+    for (auto& marker : selection->startContainer().document().markers().markersInRange(*selection, DocumentMarker::Type::DictationAlternatives))
         removeDictationAlternativesForMarker(*marker);
     applyAlternativeTextToRange(*selection, alternativeString, AlternativeTextType::DictationAlternatives, markerTypesForAppliedDictationAlternative());
 #else
