@@ -41,8 +41,8 @@ namespace WebCore {
 
 using namespace HTMLNames;
 
-BreakBlockquoteCommand::BreakBlockquoteCommand(Document& document)
-    : CompositeEditCommand(document)
+BreakBlockquoteCommand::BreakBlockquoteCommand(Ref<Document>&& document)
+    : CompositeEditCommand(WTFMove(document))
 {
 }
 
@@ -119,38 +119,38 @@ void BreakBlockquoteCommand::doApply()
     if (lineBreakExistsAtVisiblePosition(visiblePos))
         pos = pos.next();
         
-    // Adjust the position so we don't split at the beginning of a quote.  
-    while (isFirstVisiblePositionInNode(VisiblePosition(pos), enclosingNodeOfType(pos, isMailBlockquote)))
+    // Adjust the position so we don't split at the beginning of a quote.
+    while (isFirstVisiblePositionInNode(VisiblePosition(pos), enclosingNodeOfType(pos, isMailBlockquote).get()))
         pos = pos.previous();
     
     // startNode is the first node that we need to move to the new blockquote.
-    Node* startNode = pos.deprecatedNode();
+    auto startNode = pos.protectedDeprecatedNode();
     ASSERT(startNode);
     // Split at pos if in the middle of a text node.
     if (is<Text>(*startNode)) {
-        Text& textNode = downcast<Text>(*startNode);
-        if ((unsigned)pos.deprecatedEditingOffset() >= textNode.length()) {
-            if (auto* nextNode = NodeTraversal::next(*startNode))
-                startNode = nextNode;
+        Ref textNode = downcast<Text>(*startNode);
+        if (static_cast<unsigned>(pos.deprecatedEditingOffset()) >= textNode->length()) {
+            if (RefPtr nextNode = NodeTraversal::next(*startNode))
+                startNode = WTFMove(nextNode);
         } else if (pos.deprecatedEditingOffset() > 0)
             splitTextNode(textNode, pos.deprecatedEditingOffset());
     } else if (pos.deprecatedEditingOffset() > 0) {
-        if (auto* child = startNode->traverseToChildAt(pos.deprecatedEditingOffset()))
-            startNode = child;
-        else if (auto* next = NodeTraversal::next(*startNode))
-            startNode = next;
+        if (RefPtr child = startNode->traverseToChildAt(pos.deprecatedEditingOffset()))
+            startNode = WTFMove(child);
+        else if (RefPtr next = NodeTraversal::next(*startNode))
+            startNode = WTFMove(next);
     }
     
     // If there's nothing inside topBlockquote to move, we're finished.
     if (!startNode->isDescendantOf(*topBlockquote)) {
-        setEndingSelection(VisibleSelection(VisiblePosition(firstPositionInOrBeforeNode(startNode)), endingSelection().isDirectional()));
+        setEndingSelection(VisibleSelection(VisiblePosition(firstPositionInOrBeforeNode(startNode.get())), endingSelection().isDirectional()));
         return;
     }
     
     // Build up list of ancestors in between the start node and the top blockquote.
     Vector<RefPtr<Element>> ancestors;    
-    for (Element* node = startNode->parentElement(); node && node != topBlockquote; node = node->parentElement())
-        ancestors.append(node);
+    for (RefPtr node = startNode->parentElement(); node && node != topBlockquote; node = node->parentElement())
+        ancestors.append(node.copyRef());
     
     // Insert a clone of the top blockquote after the break.
     auto clonedBlockquote = downcast<Element>(*topBlockquote).cloneElementWithoutChildren(document());
@@ -165,7 +165,7 @@ void BreakBlockquoteCommand::doApply()
         auto clonedChild = ancestors[i - 1]->cloneElementWithoutChildren(document());
         // Preserve list item numbering in cloned lists.
         if (clonedChild->isElementNode() && clonedChild->hasTagName(olTag)) {
-            Node* listChildNode = i > 1 ? ancestors[i - 2].get() : startNode;
+            RefPtr<Node> listChildNode = i > 1 ? ancestors[i - 2].get() : startNode.get();
             // The first child of the cloned list might not be a list item element, 
             // find the first one so that we know where to start numbering.
             while (listChildNode && !listChildNode->hasTagName(liTag))
@@ -178,7 +178,7 @@ void BreakBlockquoteCommand::doApply()
         clonedAncestor = WTFMove(clonedChild);
     }
 
-    moveRemainingSiblingsToNewParent(startNode, 0, *clonedAncestor);
+    moveRemainingSiblingsToNewParent(startNode.get(), nullptr, *clonedAncestor);
 
     if (!ancestors.isEmpty()) {
         // Split the tree up the ancestor chain until the topBlockquote
@@ -196,7 +196,7 @@ void BreakBlockquoteCommand::doApply()
         }
 
         // If the startNode's original parent is now empty, remove it
-        Node* originalParent = ancestors.first().get();
+        RefPtr originalParent = ancestors.first().get();
         if (!originalParent->hasChildNodes())
             removeNode(*originalParent);
     }
