@@ -47,20 +47,20 @@ public:
         RebalanceAllWhitespaces
     };
 
-    static Ref<InsertTextCommand> create(Document& document, const String& text, bool selectInsertedText = false,
+    static Ref<InsertTextCommand> create(Ref<Document>&& document, const String& text, bool selectInsertedText = false,
         RebalanceType rebalanceType = RebalanceLeadingAndTrailingWhitespaces, EditAction editingAction = EditAction::Insert)
     {
-        return adoptRef(*new InsertTextCommand(document, text, selectInsertedText, rebalanceType, editingAction));
+        return adoptRef(*new InsertTextCommand(WTFMove(document), text, selectInsertedText, rebalanceType, editingAction));
     }
 
-    static Ref<InsertTextCommand> createWithMarkerSupplier(Document& document, const String& text, Ref<TextInsertionMarkerSupplier>&& markerSupplier, EditAction editingAction = EditAction::Insert)
+    static Ref<InsertTextCommand> createWithMarkerSupplier(Ref<Document>&& document, const String& text, Ref<TextInsertionMarkerSupplier>&& markerSupplier, EditAction editingAction = EditAction::Insert)
     {
-        return adoptRef(*new InsertTextCommand(document, text, WTFMove(markerSupplier), editingAction));
+        return adoptRef(*new InsertTextCommand(WTFMove(document), text, WTFMove(markerSupplier), editingAction));
     }
 
 protected:
-    InsertTextCommand(Document&, const String& text, Ref<TextInsertionMarkerSupplier>&&, EditAction);
-    InsertTextCommand(Document&, const String& text, bool selectInsertedText, RebalanceType, EditAction);
+    InsertTextCommand(Ref<Document>&&, const String& text, Ref<TextInsertionMarkerSupplier>&&, EditAction);
+    InsertTextCommand(Ref<Document>&&, const String& text, bool selectInsertedText, RebalanceType, EditAction);
 
 private:
 
