@@ -31,8 +31,8 @@
 #pragma once
 
 #include "CaretRectComputation.h"
-#include "LayoutIntegrationLineIterator.h"
-#include "LayoutIntegrationRunIterator.h"
+#include "InlineIteratorBox.h"
+#include "InlineIteratorLineBox.h"
 #include "TextAffinity.h"
 
 namespace WebCore {
@@ -50,7 +50,7 @@ public:
     bool isEquivalent(const RenderedPosition&) const;
 
     bool isNull() const { return !m_renderer; }
-    LayoutIntegration::LineIterator line() const { return m_run ? m_run.line() : LayoutIntegration::LineIterator(); }
+    InlineIterator::LineBoxIterator lineBox() const { return m_box ? m_box->lineBox() : InlineIterator::LineBoxIterator(); }
 
     unsigned char bidiLevelOnLeft() const;
     unsigned char bidiLevelOnRight() const;
@@ -72,21 +72,21 @@ public:
 
 private:
     bool operator==(const RenderedPosition&) const { return false; }
-    explicit RenderedPosition(const RenderObject*, LayoutIntegration::RunIterator, unsigned offset);
+    explicit RenderedPosition(const RenderObject*, InlineIterator::LeafBoxIterator, unsigned offset);
 
-    LayoutIntegration::RunIterator previousLeafOnLine() const;
-    LayoutIntegration::RunIterator nextLeafOnLine() const;
-    bool atLeftmostOffsetInBox() const { return m_run && m_offset == m_run->leftmostCaretOffset(); }
-    bool atRightmostOffsetInBox() const { return m_run && m_offset == m_run->rightmostCaretOffset(); }
+    InlineIterator::LeafBoxIterator previousLeafOnLine() const;
+    InlineIterator::LeafBoxIterator nextLeafOnLine() const;
+    bool atLeftmostOffsetInBox() const { return m_box && m_offset == m_box->leftmostCaretOffset(); }
+    bool atRightmostOffsetInBox() const { return m_box && m_offset == m_box->rightmostCaretOffset(); }
     bool atLeftBoundaryOfBidiRun(ShouldMatchBidiLevel, unsigned char bidiLevelOfRun) const;
     bool atRightBoundaryOfBidiRun(ShouldMatchBidiLevel, unsigned char bidiLevelOfRun) const;
 
-    const RenderObject* m_renderer { nullptr };
-    LayoutIntegration::RunIterator m_run;
+    CheckedPtr<const RenderObject> m_renderer;
+    InlineIterator::LeafBoxIterator m_box;
     unsigned m_offset { 0 };
 
-    mutable std::optional<LayoutIntegration::RunIterator> m_previousLeafOnLine;
-    mutable std::optional<LayoutIntegration::RunIterator> m_nextLeafOnLine;
+    mutable std::optional<InlineIterator::LeafBoxIterator> m_previousLeafOnLine;
+    mutable std::optional<InlineIterator::LeafBoxIterator> m_nextLeafOnLine;
 };
 
 bool renderObjectContainsPosition(const RenderObject*, const Position&);
