@@ -27,6 +27,7 @@
 #include "CachedPage.h"
 
 #include "Document.h"
+#include "DocumentLoader.h"
 #include "Element.h"
 #include "FocusController.h"
 #include "FrameLoader.h"
@@ -58,10 +59,8 @@ DEFINE_DEBUG_ONLY_GLOBAL(WTF::RefCountedLeakCounter, cachedPageCounter, ("Cached
 CachedPage::CachedPage(Page& page)
     : m_page(page)
     , m_expirationTime(MonotonicTime::now() + page.settings().backForwardCacheExpirationInterval())
-    , m_cachedMainFrame(is<LocalFrame>(page.mainFrame()) ? makeUnique<CachedFrame>(downcast<LocalFrame>(page.mainFrame())) : nullptr)
-#if ENABLE(TRACKING_PREVENTION)
+    , m_cachedMainFrame(makeUnique<CachedFrame>(page.mainFrame()))
     , m_loadedSubresourceDomains(is<LocalFrame>(page.mainFrame()) ? downcast<LocalFrame>(page.mainFrame()).loader().client().loadedSubresourceDomains() : Vector<RegistrableDomain>())
-#endif
 {
 #ifndef NDEBUG
     cachedPageCounter.increment();
@@ -81,20 +80,18 @@ CachedPage::~CachedPage()
 static void firePageShowEvent(Page& page)
 {
     // Dispatching JavaScript events can cause frame destruction.
-    auto* localMainFrame = dynamicDowncast<LocalFrame>(page.mainFrame());
-    if (!localMainFrame)
-        return;
+    auto& mainFrame = page.mainFrame();
 
     Vector<Ref<LocalFrame>> childFrames;
-    for (auto* child = localMainFrame->tree().traverseNextInPostOrder(CanWrap::Yes); child; child = child->tree().traverseNextInPostOrder(CanWrap::No)) {
-        auto* localChild = downcast<LocalFrame>(child);
+    for (auto* child = mainFrame.tree().traverseNextInPostOrder(CanWrap::Yes); child; child = child->tree().traverseNextInPostOrder(CanWrap::No)) {
+        auto* localChild = dynamicDowncast<LocalFrame>(child);
         if (!localChild)
             continue;
         childFrames.append(*localChild);
     }
 
     for (auto& child : childFrames) {
-        if (!child->tree().isDescendantOf(localMainFrame))
+        if (!child->tree().isDescendantOf(&mainFrame))
             continue;
         auto* document = child->document();
         if (!document)
@@ -128,15 +125,11 @@ void CachedPage::restore(Page& page)
 {
     ASSERT(m_cachedMainFrame);
     ASSERT(m_cachedMainFrame->view());
-#if ASSERT_ENABLED
-    auto* localFrame = dynamicDowncast<LocalFrame>(m_cachedMainFrame->view()->frame());
-#endif
-    ASSERT(localFrame && localFrame->isMainFrame());
+    ASSERT(m_cachedMainFrame->view()->frame().isMainFrame());
     ASSERT(!page.subframeCount());
 
-    auto* localMainFrame = dynamicDowncast<LocalFrame>(page.mainFrame());
-    if (!localMainFrame)
-        return;
+    auto& mainFrame = page.mainFrame();
+    RefPtr localMainFrame = dynamicDowncast<LocalFrame>(mainFrame);
 
     CachedPageRestorationScope restorationScope(page);
     m_cachedMainFrame->open();
@@ -148,10 +141,11 @@ void CachedPage::restore(Page& page)
 #if PLATFORM(IOS_FAMILY)
         // We don't want focused nodes changing scroll position when restoring from the cache
         // as it can cause ugly jumps before we manage to restore the cached position.
-        localMainFrame->selection().suppressScrolling();
+        if (localMainFrame)
+            localMainFrame->selection().suppressScrolling();
 
         bool hadProhibitsScrolling = false;
-        auto* frameView = localMainFrame->view();
+        auto* frameView = mainFrame.virtualView();
         if (frameView) {
             hadProhibitsScrolling = frameView->prohibitsScrolling();
             frameView->setProhibitsScrolling(true);
@@ -161,12 +155,15 @@ void CachedPage::restore(Page& page)
 #if PLATFORM(IOS_FAMILY)
         if (frameView)
             frameView->setProhibitsScrolling(hadProhibitsScrolling);
-        localMainFrame->selection().restoreScrolling();
+        if (localMainFrame)
+            localMainFrame->selection().restoreScrolling();
 #endif
     }
 
-    if (m_needsDeviceOrPageScaleChanged)
-        localMainFrame->deviceOrPageScaleFactorChanged();
+    if (m_needsDeviceOrPageScaleChanged) {
+        if (auto* localMainFrame = dynamicDowncast<LocalFrame>(mainFrame))
+            localMainFrame->deviceOrPageScaleFactorChanged();
+    }
 
     page.setNeedsRecalcStyleInAllFrames();
 
@@ -176,16 +173,16 @@ void CachedPage::restore(Page& page)
 #endif
 
     if (m_needsUpdateContentsSize) {
-        if (auto* frameView = localMainFrame->view())
+        if (auto* frameView = mainFrame.virtualView())
             frameView->updateContentsSize();
     }
 
     firePageShowEvent(page);
 
-#if ENABLE(TRACKING_PREVENTION)
-    for (auto& domain : m_loadedSubresourceDomains)
-        localMainFrame->loader().client().didLoadFromRegistrableDomain(WTFMove(domain));
-#endif
+    for (auto& domain : m_loadedSubresourceDomains) {
+        if (auto* localMainFrame = dynamicDowncast<LocalFrame>(mainFrame))
+            localMainFrame->loader().client().didLoadFromRegistrableDomain(WTFMove(domain));
+    }
 
     clear();
 }
@@ -200,9 +197,7 @@ void CachedPage::clear()
 #endif
     m_needsDeviceOrPageScaleChanged = false;
     m_needsUpdateContentsSize = false;
-#if ENABLE(TRACKING_PREVENTION)
     m_loadedSubresourceDomains.clear();
-#endif
 }
 
 bool CachedPage::hasExpired() const
@@ -210,4 +205,9 @@ bool CachedPage::hasExpired() const
     return MonotonicTime::now() > m_expirationTime;
 }
 
+RefPtr<DocumentLoader> CachedPage::protectedDocumentLoader() const
+{
+    return documentLoader();
+}
+
 } // namespace WebCore
