@@ -31,6 +31,7 @@
 #include "HTMLIFrameElement.h"
 #include "HTMLNames.h"
 #include "LocalDOMWindow.h"
+#include "Quirks.h"
 #include "SecurityOrigin.h"
 
 namespace WebCore {
@@ -48,6 +49,8 @@ static const char* policyTypeName(FeaturePolicy::Type type)
         return "SpeakerSelection";
     case FeaturePolicy::Type::DisplayCapture:
         return "DisplayCapture";
+    case FeaturePolicy::Type::Gamepad:
+        return "Gamepad";
     case FeaturePolicy::Type::Geolocation:
         return "Geolocation";
     case FeaturePolicy::Type::Payment:
@@ -93,11 +96,11 @@ bool isFeaturePolicyAllowedByDocumentAndAllOwners(FeaturePolicy::Type type, cons
         }
 
         auto* ownerElement = ancestorDocument->ownerElement();
-        if (is<HTMLIFrameElement>(ownerElement)) {
-            const auto& featurePolicy = downcast<HTMLIFrameElement>(ownerElement)->featurePolicy();
+        if (auto* iframe = dynamicDowncast<HTMLIFrameElement>(ownerElement)) {
+            const auto& featurePolicy = iframe->featurePolicy();
             if (!featurePolicy.allows(type, ancestorDocument->securityOrigin().data())) {
                 if (logFailure == LogFeaturePolicyFailure::Yes && document.domWindow()) {
-                    auto& allowValue = downcast<HTMLIFrameElement>(ownerElement)->attributeWithoutSynchronization(HTMLNames::allowAttr);
+                    auto& allowValue = iframe->attributeWithoutSynchronization(HTMLNames::allowAttr);
                     document.domWindow()->printErrorMessage(makeString("Feature policy '", policyTypeName(type), "' check failed for iframe with origin '", document.securityOrigin().toString(), "' and allow attribute '", allowValue, "'."));
                 }
                 return false;
@@ -164,8 +167,12 @@ static inline void processOriginItem(Document& document, const HTMLIFrameElement
 
 static inline void updateList(Document& document, const HTMLIFrameElement& iframe, FeaturePolicy::AllowRule& rule, StringView value)
 {
-    // We keep the empty string value equivalent to '*' for existing websites.
     if (value.isEmpty()) {
+        if (document.quirks().shouldStarBeFeaturePolicyDefaultValue()) {
+            rule.type = FeaturePolicy::AllowRule::Type::All;
+            return;
+        }
+
         // The allowlist for the features named in the attribute may be empty; in that case,
         // the default value for the allowlist is 'src', which represents the origin of the
         // URL in the iframe’s src attribute.
@@ -193,6 +200,7 @@ FeaturePolicy FeaturePolicy::parse(Document& document, const HTMLIFrameElement&
     bool isMicrophoneInitialized = false;
     bool isSpeakerSelectionInitialized = false;
     bool isDisplayCaptureInitialized = false;
+    bool isGamepadInitialized = false;
     bool isGeolocationInitialized = false;
     bool isPaymentInitialized = false;
     bool isScreenWakeLockInitialized = false;
@@ -232,6 +240,11 @@ FeaturePolicy FeaturePolicy::parse(Document& document, const HTMLIFrameElement&
             updateList(document, iframe, policy.m_displayCaptureRule, item.substring(16));
             continue;
         }
+        if (item.startsWith("gamepad"_s)) {
+            isGamepadInitialized = true;
+            updateList(document, iframe, policy.m_gamepadRule, item.substring(8));
+            continue;
+        }
         if (item.startsWith("geolocation"_s)) {
             isGeolocationInitialized = true;
             updateList(document, iframe, policy.m_geolocationRule, item.substring(12));
@@ -304,6 +317,8 @@ FeaturePolicy FeaturePolicy::parse(Document& document, const HTMLIFrameElement&
         policy.m_speakerSelectionRule.allowedList.add(document.securityOrigin().data());
     if (!isDisplayCaptureInitialized)
         policy.m_displayCaptureRule.allowedList.add(document.securityOrigin().data());
+    if (!isGamepadInitialized)
+        policy.m_gamepadRule.type = FeaturePolicy::AllowRule::Type::All;
     if (!isScreenWakeLockInitialized)
         policy.m_screenWakeLockRule.allowedList.add(document.securityOrigin().data());
     if (!isGeolocationInitialized)
@@ -361,6 +376,8 @@ bool FeaturePolicy::allows(Type type, const SecurityOriginData& origin) const
         return isAllowedByFeaturePolicy(m_speakerSelectionRule, origin);
     case Type::DisplayCapture:
         return isAllowedByFeaturePolicy(m_displayCaptureRule, origin);
+    case Type::Gamepad:
+        return isAllowedByFeaturePolicy(m_gamepadRule, origin);
     case Type::Geolocation:
         return isAllowedByFeaturePolicy(m_geolocationRule, origin);
     case Type::Payment:
