@@ -62,7 +62,11 @@ class UploadButtonElement;
 
 SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::UploadButtonElement)
     static bool isType(const WebCore::Element& element) { return element.isUploadButton(); }
-    static bool isType(const WebCore::Node& node) { return is<WebCore::Element>(node) && isType(downcast<WebCore::Element>(node)); }
+    static bool isType(const WebCore::Node& node)
+    {
+        auto* element = dynamicDowncast<WebCore::Element>(node);
+        return element && isType(*element);
+    }
 SPECIALIZE_TYPE_TRAITS_END()
 
 namespace WebCore {
@@ -130,7 +134,7 @@ std::pair<Vector<FileChooserFileInfo>, String> FileInputType::filesFromFormContr
     size_t size = state.size() - 1;
     files.reserveInitialCapacity(size / 2);
     for (size_t i = 0; i < size; i += 2)
-        files.uncheckedAppend({ state[i], { }, state[i + 1] });
+        files.append({ state[i], { }, state[i + 1] });
 
     return { files, state[size] };
 }
@@ -150,10 +154,10 @@ FormControlState FileInputType::saveFormControlState() const
     Vector<AtomString> stateVector;
     stateVector.reserveInitialCapacity(length);
     for (auto& file : m_fileList->files()) {
-        stateVector.uncheckedAppend(AtomString { file->path() });
-        stateVector.uncheckedAppend(AtomString { file->name() });
+        stateVector.append(AtomString { file->path() });
+        stateVector.append(AtomString { file->name() });
     }
-    stateVector.uncheckedAppend(AtomString { m_displayString });
+    stateVector.append(AtomString { m_displayString });
     return FormControlState { WTFMove(stateVector) };
 }
 
@@ -358,6 +362,11 @@ bool FileInputType::allowsDirectories() const
     return element()->hasAttributeWithoutSynchronization(webkitdirectoryAttr);
 }
 
+bool FileInputType::dirAutoUsesValue() const
+{
+    return false;
+}
+
 void FileInputType::setFiles(RefPtr<FileList>&& files, WasSetByJavaScript wasSetByJavaScript)
 {
     setFiles(WTFMove(files), RequestIcon::Yes, wasSetByJavaScript);
@@ -448,11 +457,9 @@ void FileInputType::filesChosen(const Vector<String>& paths, const Vector<String
 
     size_t size = element()->hasAttributeWithoutSynchronization(multipleAttr) ? paths.size() : 1;
 
-    Vector<FileChooserFileInfo> files;
-    files.reserveInitialCapacity(size);
-
-    for (size_t i = 0; i < size; ++i)
-        files.uncheckedAppend({ paths[i], i < replacementPaths.size() ? replacementPaths[i] : nullString(), { } });
+    Vector<FileChooserFileInfo> files(size, [&](size_t i) {
+        return FileChooserFileInfo { paths[i], i < replacementPaths.size() ? replacementPaths[i] : nullString(), { } };
+    });
 
     filesChosen(WTFMove(files));
 }
