@@ -29,9 +29,11 @@
 #include "ElementInlines.h"
 #include "HTMLNames.h"
 #include "HTMLParserIdioms.h"
+#include "NodeName.h"
 #include "RenderLayer.h"
 #include "RenderLayerScrollableArea.h"
 #include "RenderMarquee.h"
+#include "RenderStyleInlines.h"
 #include <wtf/IsoMallocInlines.h>
 
 namespace WebCore {
@@ -65,53 +67,79 @@ int HTMLMarqueeElement::minimumDelay() const
 
 bool HTMLMarqueeElement::hasPresentationalHintsForAttribute(const QualifiedName& name) const
 {
-    if (name == widthAttr || name == heightAttr || name == bgcolorAttr || name == vspaceAttr || name == hspaceAttr || name == scrollamountAttr || name == scrolldelayAttr || name == loopAttr || name == behaviorAttr || name == directionAttr)
+    switch (name.nodeName()) {
+    case AttributeNames::widthAttr:
+    case AttributeNames::heightAttr:
+    case AttributeNames::bgcolorAttr:
+    case AttributeNames::vspaceAttr:
+    case AttributeNames::hspaceAttr:
+    case AttributeNames::scrollamountAttr:
+    case AttributeNames::scrolldelayAttr:
+    case AttributeNames::loopAttr:
+    case AttributeNames::behaviorAttr:
+    case AttributeNames::directionAttr:
         return true;
+    default:
+        break;
+    }
     return HTMLElement::hasPresentationalHintsForAttribute(name);
 }
 
 void HTMLMarqueeElement::collectPresentationalHintsForAttribute(const QualifiedName& name, const AtomString& value, MutableStyleProperties& style)
 {
-    if (name == widthAttr) {
+    switch (name.nodeName()) {
+    case AttributeNames::widthAttr:
         if (!value.isEmpty())
             addHTMLLengthToStyle(style, CSSPropertyWidth, value);
-    } else if (name == heightAttr) {
+        break;
+    case AttributeNames::heightAttr:
         if (!value.isEmpty())
             addHTMLLengthToStyle(style, CSSPropertyHeight, value);
-    } else if (name == bgcolorAttr) {
+        break;
+    case AttributeNames::bgcolorAttr:
         if (!value.isEmpty())
             addHTMLColorToStyle(style, CSSPropertyBackgroundColor, value);
-    } else if (name == vspaceAttr) {
+        break;
+    case AttributeNames::vspaceAttr:
         if (!value.isEmpty()) {
             addHTMLLengthToStyle(style, CSSPropertyMarginTop, value);
             addHTMLLengthToStyle(style, CSSPropertyMarginBottom, value);
         }
-    } else if (name == hspaceAttr) {
+        break;
+    case AttributeNames::hspaceAttr:
         if (!value.isEmpty()) {
             addHTMLLengthToStyle(style, CSSPropertyMarginLeft, value);
             addHTMLLengthToStyle(style, CSSPropertyMarginRight, value);
         }
-    } else if (name == scrollamountAttr) {
+        break;
+    case AttributeNames::scrollamountAttr:
         if (!value.isEmpty())
             addHTMLLengthToStyle(style, CSSPropertyWebkitMarqueeIncrement, value);
-    } else if (name == scrolldelayAttr) {
+        break;
+    case AttributeNames::scrolldelayAttr:
         if (!value.isEmpty())
             addHTMLNumberToStyle(style, CSSPropertyWebkitMarqueeSpeed, value);
-    } else if (name == loopAttr) {
+        break;
+    case AttributeNames::loopAttr:
         if (!value.isEmpty()) {
             if (value == "-1"_s || equalLettersIgnoringASCIICase(value, "infinite"_s))
                 addPropertyToPresentationalHintStyle(style, CSSPropertyWebkitMarqueeRepetition, CSSValueInfinite);
             else
                 addHTMLNumberToStyle(style, CSSPropertyWebkitMarqueeRepetition, value);
         }
-    } else if (name == behaviorAttr) {
+        break;
+    case AttributeNames::behaviorAttr:
         if (!value.isEmpty())
             addPropertyToPresentationalHintStyle(style, CSSPropertyWebkitMarqueeStyle, value);
-    } else if (name == directionAttr) {
+        break;
+    case AttributeNames::directionAttr:
         if (!value.isEmpty())
             addPropertyToPresentationalHintStyle(style, CSSPropertyWebkitMarqueeDirection, value);
-    } else
+        break;
+    default:
         HTMLElement::collectPresentationalHintsForAttribute(name, value, style);
+        break;
+    }
 }
 
 void HTMLMarqueeElement::start()
@@ -155,7 +183,7 @@ int HTMLMarqueeElement::loop() const
 ExceptionOr<void> HTMLMarqueeElement::setLoop(int loop)
 {
     if (loop <= 0 && loop != -1)
-        return Exception { IndexSizeError };
+        return Exception { ExceptionCode::IndexSizeError };
     setIntegralAttribute(loopAttr, loop);
     return { };
 }
