@@ -78,6 +78,7 @@ static const Seconds elementMainContentCheckInterval { 250_ms };
 static bool isElementRectMostlyInMainFrame(const HTMLMediaElement&);
 static bool isElementLargeEnoughForMainContent(const HTMLMediaElement&, MediaSessionMainContentPurpose);
 static bool isElementMainContentForPurposesOfAutoplay(const HTMLMediaElement&, bool shouldHitTestMainFrame);
+static bool isElementLongEnoughForMainContent(const HTMLMediaElement&);
 
 #if !RELEASE_LOG_DISABLED
 
@@ -274,10 +275,10 @@ void MediaElementSession::visibilityChanged()
     if (!isPlayingAudio) {
         if (elementIsHidden) {
             ALWAYS_LOG(LOGIDENTIFIER, "Suspending silent playback after page visibility: hidden");
-            beginInterruption(PlatformMediaSession::EnteringBackground);
+            beginInterruption(PlatformMediaSession::InterruptionType::EnteringBackground);
         } else {
             ALWAYS_LOG(LOGIDENTIFIER, "Resuming silent playback after page visibility: showing");
-            endInterruption(PlatformMediaSession::MayResumePlaying);
+            endInterruption(PlatformMediaSession::EndInterruptionFlags::MayResumePlaying);
         }
         return;
     }
@@ -285,10 +286,10 @@ void MediaElementSession::visibilityChanged()
     if (hasBehaviorRestriction(RequirePageVisibilityToPlayAudio)) {
         if (elementIsHidden) {
             ALWAYS_LOG(LOGIDENTIFIER, "Suspending audible playback after page visibility: hidden");
-            beginInterruption(PlatformMediaSession::EnteringBackground);
+            beginInterruption(PlatformMediaSession::InterruptionType::EnteringBackground);
         } else {
             ALWAYS_LOG(LOGIDENTIFIER, "Resuming audible playback after page visibility: showing");
-            endInterruption(PlatformMediaSession::MayResumePlaying);
+            endInterruption(PlatformMediaSession::EndInterruptionFlags::MayResumePlaying);
         }
     }
 }
@@ -323,7 +324,7 @@ void MediaElementSession::clientDataBufferingTimerFired()
 
     updateClientDataBuffering();
 
-    if (state() != Playing || !m_element.elementIsHidden())
+    if (state() != PlatformMediaSession::State::Playing || !m_element.elementIsHidden())
         return;
 
     PlatformMediaSessionManager::SessionRestrictions restrictions = PlatformMediaSessionManager::sharedManager().restrictions(mediaType());
@@ -499,7 +500,7 @@ MediaPlayer::BufferingPolicy MediaElementSession::preferredBufferingPolicy() con
     if (bufferingSuspended())
         return MediaPlayer::BufferingPolicy::LimitReadAhead;
 
-    if (state() == PlatformMediaSession::Playing)
+    if (state() == PlatformMediaSession::State::Playing)
         return MediaPlayer::BufferingPolicy::Default;
 
     if (shouldOverrideBackgroundLoadingRestriction())
@@ -570,6 +571,13 @@ bool MediaElementSession::canShowControlsManager(PlaybackControlsPurpose purpose
         return true;
     }
 
+    if (client().presentationType() == MediaType::Audio
+        && purpose == PlaybackControlsPurpose::NowPlaying
+        && !isLongEnoughForMainContent()) {
+        INFO_LOG(LOGIDENTIFIER, "returning FALSE: audio too short for NowPlaying");
+        return false;
+    }
+
     if (client().presentationType() == MediaType::Audio && (purpose == PlaybackControlsPurpose::ControlsManager || purpose == PlaybackControlsPurpose::MediaSession)) {
         if (!hasBehaviorRestriction(RequireUserGestureToControlControlsManager) || m_element.document().processingUserGestureForMedia()) {
             INFO_LOG(LOGIDENTIFIER, "returning TRUE: audio element with user gesture");
@@ -656,6 +664,11 @@ bool MediaElementSession::isLargeEnoughForMainContent(MediaSessionMainContentPur
     return isElementLargeEnoughForMainContent(m_element, purpose);
 }
 
+bool MediaElementSession::isLongEnoughForMainContent() const
+{
+    return isElementLongEnoughForMainContent(m_element);
+}
+
 bool MediaElementSession::isMainContentForPurposesOfAutoplayEvents() const
 {
     return isElementMainContentForPurposesOfAutoplay(m_element, false);
@@ -853,7 +866,6 @@ bool MediaElementSession::requiresFullscreenForVideoPlayback() const
         return false;
 
     if (m_element.document().isMediaDocument()) {
-        ASSERT(is<HTMLVideoElement>(m_element));
         const HTMLVideoElement& videoElement = *downcast<const HTMLVideoElement>(&m_element);
         if (m_element.readyState() < HTMLVideoElement::HAVE_METADATA || !videoElement.hasEverHadVideo())
             return false;
@@ -1032,11 +1044,7 @@ static bool isElementRectMostlyInMainFrame(const HTMLMediaElement& element)
     if (!documentFrame)
         return false;
 
-    auto* localFrame = dynamicDowncast<LocalFrame>(documentFrame->mainFrame());
-    if (!localFrame)
-        return false;
-
-    auto mainFrameView = localFrame->view();
+    RefPtr mainFrameView = documentFrame->mainFrame().virtualView();
     if (!mainFrameView)
         return false;
 
@@ -1101,6 +1109,17 @@ static bool isElementLargeEnoughForMainContent(const HTMLMediaElement& element,
     return isElementLargeRelativeToMainFrame(element);
 }
 
+static bool isElementLongEnoughForMainContent(const HTMLMediaElement& element)
+{
+    // Derived from the duration of the "You've got mail!" AOL sound:
+    static constexpr MediaTime YouveGotMailDuration = MediaTime(95, 100);
+
+    if (element.readyState() < HTMLMediaElementEnums::ReadyState::HAVE_METADATA)
+        return false;
+
+    return element.durationMediaTime() > YouveGotMailDuration;
+}
+
 void MediaElementSession::mainContentCheckTimerFired()
 {
     if (!hasBehaviorRestriction(OverrideUserGestureRequirementForMainContent))
@@ -1131,7 +1150,7 @@ bool MediaElementSession::allowsPlaybackControlsForAutoplayingAudio() const
 
 #if ENABLE(MEDIA_SESSION)
 #if ENABLE(MEDIA_STREAM)
-static bool isDocumentPlayingSeveralMediaStreams(Document& document)
+static bool isDocumentPlayingSeveralMediaStreamsAndCapturing(Document& document)
 {
     // We restrict to capturing document for now, until we have a good way to state to the UIProcess application that audio rendering is muted from here.
     auto* page = document.page();
@@ -1144,7 +1163,7 @@ static bool processRemoteControlCommandIfPlayingMediaStreams(Document& document,
     if (!page)
         return false;
 
-    if (!isDocumentPlayingSeveralMediaStreams(document))
+    if (!isDocumentPlayingSeveralMediaStreamsAndCapturing(document))
         return false;
 
     WebCore::MediaProducerMutedStateFlags mutedState;
@@ -1153,14 +1172,14 @@ static bool processRemoteControlCommandIfPlayingMediaStreams(Document& document,
     mutedState.add(WebCore::MediaProducerMutedState::ScreenCaptureIsMuted);
 
     switch (commandType) {
-    case PlatformMediaSession::PlayCommand:
+    case PlatformMediaSession::RemoteControlCommandType::PlayCommand:
         page->setMuted({ });
         return true;
-    case PlatformMediaSession::StopCommand:
-    case PlatformMediaSession::PauseCommand:
+    case PlatformMediaSession::RemoteControlCommandType::StopCommand:
+    case PlatformMediaSession::RemoteControlCommandType::PauseCommand:
         page->setMuted(mutedState);
         return true;
-    case PlatformMediaSession::TogglePlayPauseCommand:
+    case PlatformMediaSession::RemoteControlCommandType::TogglePlayPauseCommand:
         if (page->mutedState().containsAny(mutedState))
             page->setMuted({ });
         else
@@ -1187,27 +1206,27 @@ void MediaElementSession::didReceiveRemoteControlCommand(RemoteControlCommandTyp
 
     MediaSessionActionDetails actionDetails;
     switch (commandType) {
-    case NoCommand:
+    case RemoteControlCommandType::NoCommand:
         return;
-    case PlayCommand:
+    case RemoteControlCommandType::PlayCommand:
         actionDetails.action = MediaSessionAction::Play;
         break;
-    case PauseCommand:
+    case RemoteControlCommandType::PauseCommand:
         actionDetails.action = MediaSessionAction::Pause;
         break;
-    case StopCommand:
+    case RemoteControlCommandType::StopCommand:
         actionDetails.action = MediaSessionAction::Stop;
         break;
-    case TogglePlayPauseCommand:
+    case RemoteControlCommandType::TogglePlayPauseCommand:
         actionDetails.action = m_element.paused() ? MediaSessionAction::Play : MediaSessionAction::Pause;
         break;
-    case BeginScrubbingCommand:
+    case RemoteControlCommandType::BeginScrubbingCommand:
         m_isScrubbing = true;
         return;
-    case EndScrubbingCommand:
+    case RemoteControlCommandType::EndScrubbingCommand:
         m_isScrubbing = false;
         return;
-    case SeekToPlaybackPositionCommand:
+    case RemoteControlCommandType::SeekToPlaybackPositionCommand:
         ASSERT(argument.time);
         if (!argument.time)
             return;
@@ -1215,26 +1234,26 @@ void MediaElementSession::didReceiveRemoteControlCommand(RemoteControlCommandTyp
         actionDetails.seekTime = argument.time.value();
         actionDetails.fastSeek = m_isScrubbing;
         break;
-    case SkipForwardCommand:
+    case RemoteControlCommandType::SkipForwardCommand:
         if (argument.time)
             actionDetails.seekOffset = argument.time.value();
         actionDetails.action = MediaSessionAction::Seekforward;
         break;
-    case SkipBackwardCommand:
+    case RemoteControlCommandType::SkipBackwardCommand:
         if (argument.time)
             actionDetails.seekOffset = argument.time.value();
         actionDetails.action = MediaSessionAction::Seekbackward;
         break;
-    case NextTrackCommand:
+    case RemoteControlCommandType::NextTrackCommand:
         actionDetails.action = MediaSessionAction::Nexttrack;
         break;
-    case PreviousTrackCommand:
+    case RemoteControlCommandType::PreviousTrackCommand:
         actionDetails.action = MediaSessionAction::Previoustrack;
         break;
-    case BeginSeekingBackwardCommand:
-    case EndSeekingBackwardCommand:
-    case BeginSeekingForwardCommand:
-    case EndSeekingForwardCommand:
+    case RemoteControlCommandType::BeginSeekingBackwardCommand:
+    case RemoteControlCommandType::EndSeekingBackwardCommand:
+    case RemoteControlCommandType::BeginSeekingForwardCommand:
+    case RemoteControlCommandType::EndSeekingForwardCommand:
         ASSERT_NOT_REACHED();
         return;
     }
@@ -1246,13 +1265,19 @@ void MediaElementSession::didReceiveRemoteControlCommand(RemoteControlCommandTyp
 std::optional<NowPlayingInfo> MediaElementSession::nowPlayingInfo() const
 {
     auto* page = m_element.document().page();
-    bool allowsNowPlayingControlsVisibility = page && !page->isVisibleAndActive();
-    bool isPlaying = state() == PlatformMediaSession::Playing;
-#if ENABLE(MEDIA_SESSION) && ENABLE(MEDIA_STREAM)
-    if (isPlaying && isDocumentPlayingSeveralMediaStreams(m_element.document()) && page)
-        isPlaying = !page->mutedState().contains(MediaProducerMutedState::AudioIsMuted);
+
+#if ENABLE(MEDIA_SESSION)
+    auto* session = mediaSession();
 #endif
 
+#if ENABLE(MEDIA_SESSION) && ENABLE(MEDIA_STREAM)
+    if (isDocumentPlayingSeveralMediaStreamsAndCapturing(m_element.document()) && (!session || !session->hasActiveActionHandlers()))
+        return { };
+#endif
+
+    bool allowsNowPlayingControlsVisibility = page && !page->isVisibleAndActive();
+    bool isPlaying = state() == PlatformMediaSession::State::Playing;
+
     bool supportsSeeking = m_element.supportsSeeking();
     double rate = 1.0;
     double duration = supportsSeeking ? m_element.duration() : MediaPlayer::invalidTime();
@@ -1269,7 +1294,6 @@ std::optional<NowPlayingInfo> MediaElementSession::nowPlayingInfo() const
 #endif
 
 #if ENABLE(MEDIA_SESSION)
-    auto* session = mediaSession();
     auto positionState = session ? session->positionState() : std::nullopt;
     auto currentPosition = session ? session->currentPosition() : std::nullopt;
     if (positionState) {
@@ -1318,7 +1342,7 @@ void MediaElementSession::updateMediaUsageIfChanged()
     MediaUsageInfo usage = {
         m_element.currentSrc(),
         m_element.hasSource(),
-        state() == PlatformMediaSession::Playing,
+        state() == PlatformMediaSession::State::Playing,
         canShowControlsManager(PlaybackControlsPurpose::ControlsManager),
         !page->isVisibleAndActive(),
         m_element.isSuspended(),
