@@ -35,18 +35,13 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(EXTBlendMinMax);
 
 EXTBlendMinMax::EXTBlendMinMax(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::EXTBlendMinMax)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_EXT_blend_minmax"_s);
 }
 
 EXTBlendMinMax::~EXTBlendMinMax() = default;
 
-WebGLExtension::ExtensionName EXTBlendMinMax::getName() const
-{
-    return EXTBlendMinMaxName;
-}
-
 bool EXTBlendMinMax::supported(GraphicsContextGL& context)
 {
     return context.supportsExtension("GL_EXT_blend_minmax"_s);
