@@ -32,6 +32,7 @@
 #include "EventLoop.h"
 #include "ScriptExecutionContext.h"
 #include "WebGLRenderingContext.h"
+#include "WebGLTimerQueryEXT.h"
 #include <wtf/IsoMallocInlines.h>
 #include <wtf/Lock.h>
 #include <wtf/Locker.h>
@@ -40,19 +41,14 @@ namespace WebCore {
 
 WTF_MAKE_ISO_ALLOCATED_IMPL(EXTDisjointTimerQuery);
 
-EXTDisjointTimerQuery::EXTDisjointTimerQuery(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+EXTDisjointTimerQuery::EXTDisjointTimerQuery(WebGLRenderingContext& context)
+    : WebGLExtension(context, WebGLExtensionName::EXTDisjointTimerQuery)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_EXT_disjoint_timer_query"_s);
 }
 
 EXTDisjointTimerQuery::~EXTDisjointTimerQuery() = default;
 
-WebGLExtension::ExtensionName EXTDisjointTimerQuery::getName() const
-{
-    return EXTDisjointTimerQueryName;
-}
-
 bool EXTDisjointTimerQuery::supported(GraphicsContextGL& context)
 {
     return context.supportsExtension("GL_EXT_disjoint_timer_query"_s);
@@ -60,67 +56,59 @@ bool EXTDisjointTimerQuery::supported(GraphicsContextGL& context)
 
 RefPtr<WebGLTimerQueryEXT> EXTDisjointTimerQuery::createQueryEXT()
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost())
+    if (isContextLost())
         return nullptr;
-
-    auto query = WebGLTimerQueryEXT::create(*context);
-    context->addContextObject(query.get());
-    return query;
+    auto& context = this->context();
+    return WebGLTimerQueryEXT::create(context);
 }
 
 void EXTDisjointTimerQuery::deleteQueryEXT(WebGLTimerQueryEXT* query)
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost())
+    if (isContextLost())
         return;
+    auto& context = this->context();
 
-    Locker locker { context->objectGraphLock() };
+    Locker locker { context.objectGraphLock() };
 
     if (!query)
         return;
 
-    if (!query->validate(context->contextGroup(), *context)) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "delete", "object does not belong to this context");
+    if (!query->validate(context)) {
+        context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "delete", "object does not belong to this context");
         return;
     }
 
     if (query->isDeleted())
         return;
 
-    if (query == context.downcast<WebGLRenderingContext>()->m_activeQuery) {
-        context.downcast<WebGLRenderingContext>()->m_activeQuery = nullptr;
+    if (query == context.m_activeQuery) {
+        context.m_activeQuery = nullptr;
         ASSERT(query->target() == GraphicsContextGL::TIME_ELAPSED_EXT);
-        context->graphicsContextGL()->endQueryEXT(GraphicsContextGL::TIME_ELAPSED_EXT);
+        context.graphicsContextGL()->endQueryEXT(GraphicsContextGL::TIME_ELAPSED_EXT);
     }
 
-    query->deleteObject(locker, context->graphicsContextGL());
+    query->deleteObject(locker, context.graphicsContextGL());
 }
 
 GCGLboolean EXTDisjointTimerQuery::isQueryEXT(WebGLTimerQueryEXT* query)
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost())
+    if (isContextLost())
         return false;
-
-    if (!query || !query->validate(context->contextGroup(), *context))
+    auto& context = this->context();
+    if (!context.validateIsWebGLObject(query))
         return false;
-
-    if (query->isDeleted())
-        return false;
-
-    return context->graphicsContextGL()->isQueryEXT(query->object());
+    return context.graphicsContextGL()->isQueryEXT(query->object());
 }
 
 void EXTDisjointTimerQuery::beginQueryEXT(GCGLenum target, WebGLTimerQueryEXT& query)
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost())
+    if (isContextLost())
         return;
+    auto& context = this->context();
 
-    Locker locker { context->objectGraphLock() };
+    Locker locker { context.objectGraphLock() };
 
-    if (!context->validateWebGLObject("beginQueryEXT", &query))
+    if (!context.validateWebGLObject("beginQueryEXT", query))
         return;
 
     // The WebGL extension requires ending time elapsed queries when they are deleted.
@@ -128,120 +116,123 @@ void EXTDisjointTimerQuery::beginQueryEXT(GCGLenum target, WebGLTimerQueryEXT& q
     // to defer query results until control is returned to the user agent's main loop.
 
     if (target != GraphicsContextGL::TIME_ELAPSED_EXT) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "beginQueryEXT", "invalid target");
+        context.synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "beginQueryEXT", "invalid target");
         return;
     }
 
     if (query.target() && query.target() != target) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "beginQueryEXT", "query type does not match target");
+        context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "beginQueryEXT", "query type does not match target");
         return;
     }
 
-    if (context.downcast<WebGLRenderingContext>()->m_activeQuery) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "beginQueryEXT", "query object of target is already active");
+    if (context.m_activeQuery) {
+        context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "beginQueryEXT", "query object of target is already active");
         return;
     }
 
-    query.setTarget(target);
-    context.downcast<WebGLRenderingContext>()->m_activeQuery = &query;
+    context.m_activeQuery = &query;
 
-    context->graphicsContextGL()->beginQueryEXT(target, query.object());
+    context.graphicsContextGL()->beginQueryEXT(target, query.object());
 }
 
 void EXTDisjointTimerQuery::endQueryEXT(GCGLenum target)
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost() || !context->scriptExecutionContext())
+    if (isContextLost())
+        return;
+    auto& context = this->context();
+    if (!context.scriptExecutionContext())
         return;
 
-    Locker locker { context->objectGraphLock() };
+    Locker locker { context.objectGraphLock() };
 
     if (target != GraphicsContextGL::TIME_ELAPSED_EXT) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "endQueryEXT", "invalid target");
+        context.synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "endQueryEXT", "invalid target");
         return;
     }
 
-    if (!context.downcast<WebGLRenderingContext>()->m_activeQuery) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "endQueryEXT", "query object of target is not active");
+    if (!context.m_activeQuery) {
+        context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "endQueryEXT", "query object of target is not active");
         return;
     }
 
-    context->graphicsContextGL()->endQueryEXT(target);
+    context.graphicsContextGL()->endQueryEXT(target);
 
     // A query's result must not be made available until control has returned to the user agent's main loop.
-    context->scriptExecutionContext()->eventLoop().queueMicrotask([query = WTFMove(context.downcast<WebGLRenderingContext>()->m_activeQuery)] {
+    context.scriptExecutionContext()->eventLoop().queueMicrotask([query = WTFMove(context.m_activeQuery)] {
         query->makeResultAvailable();
     });
 }
 
 void EXTDisjointTimerQuery::queryCounterEXT(WebGLTimerQueryEXT& query, GCGLenum target)
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost() || !context->scriptExecutionContext())
+    if (isContextLost())
+        return;
+    auto& context = this->context();
+    if (!context.scriptExecutionContext())
         return;
 
-    if (!context->validateWebGLObject("queryCounterEXT", &query))
+    if (!context.validateWebGLObject("queryCounterEXT", query))
         return;
 
     if (target != GraphicsContextGL::TIMESTAMP_EXT) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "queryCounterEXT", "invalid target");
+        context.synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "queryCounterEXT", "invalid target");
         return;
     }
 
     if (query.target() && query.target() != target) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "queryCounterEXT", "query type does not match target");
+        context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "queryCounterEXT", "query type does not match target");
         return;
     }
 
     query.setTarget(target);
 
-    context->graphicsContextGL()->queryCounterEXT(query.object(), target);
+    context.graphicsContextGL()->queryCounterEXT(query.object(), target);
 
     // A query's result must not be made available until control has returned to the user agent's main loop.
-    context->scriptExecutionContext()->eventLoop().queueMicrotask([&] {
+    context.scriptExecutionContext()->eventLoop().queueMicrotask([&] {
         query.makeResultAvailable();
     });
 }
 
 WebGLAny EXTDisjointTimerQuery::getQueryEXT(GCGLenum target, GCGLenum pname)
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost())
+    if (isContextLost())
         return nullptr;
+    auto& context = this->context();
 
     if (target != GraphicsContextGL::TIME_ELAPSED_EXT && target != GraphicsContextGL::TIMESTAMP_EXT) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "getQueryEXT", "invalid target");
+        context.synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "getQueryEXT", "invalid target");
         return nullptr;
     }
 
     switch (pname) {
     case GraphicsContextGL::CURRENT_QUERY_EXT:
         if (target == GraphicsContextGL::TIME_ELAPSED_EXT)
-            return context.downcast<WebGLRenderingContext>()->m_activeQuery;
+            return context.m_activeQuery;
         return nullptr;
     case GraphicsContextGL::QUERY_COUNTER_BITS_EXT:
-        return context->graphicsContextGL()->getQueryiEXT(target, pname);
+        return context.graphicsContextGL()->getQueryiEXT(target, pname);
     }
-    context->synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "getQueryEXT", "invalid parameter name");
+    context.synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "getQueryEXT", "invalid parameter name");
     return nullptr;
 }
 
 WebGLAny EXTDisjointTimerQuery::getQueryObjectEXT(WebGLTimerQueryEXT& query, GCGLenum pname)
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost())
+    if (isContextLost())
         return nullptr;
+    auto& context = this->context();
 
-    if (!context->validateWebGLObject("getQueryObjectEXT", &query))
+    if (!context.validateWebGLObject("getQueryObjectEXT", query))
         return nullptr;
 
     if (!query.target()) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "getQueryObjectEXT", "query has not been used");
+        context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "getQueryObjectEXT", "query has not been used");
         return nullptr;
     }
 
-    if (&query == context.downcast<WebGLRenderingContext>()->m_activeQuery) {
-        context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "getQueryObjectEXT", "query is currently active");
+    if (&query == context.m_activeQuery) {
+        context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "getQueryObjectEXT", "query is currently active");
         return nullptr;
     }
 
@@ -249,13 +240,13 @@ WebGLAny EXTDisjointTimerQuery::getQueryObjectEXT(WebGLTimerQueryEXT& query, GCG
     case GraphicsContextGL::QUERY_RESULT_EXT:
         if (!query.isResultAvailable())
             return 0;
-        return static_cast<unsigned long long>(context->graphicsContextGL()->getQueryObjectui64EXT(query.object(), pname));
+        return static_cast<unsigned long long>(context.graphicsContextGL()->getQueryObjectui64EXT(query.object(), pname));
     case GraphicsContextGL::QUERY_RESULT_AVAILABLE_EXT:
         if (!query.isResultAvailable())
             return false;
-        return static_cast<bool>(context->graphicsContextGL()->getQueryObjectiEXT(query.object(), pname));
+        return static_cast<bool>(context.graphicsContextGL()->getQueryObjectiEXT(query.object(), pname));
     }
-    context->synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "getQueryObjectEXT", "invalid parameter name");
+    context.synthesizeGLError(GraphicsContextGL::INVALID_ENUM, "getQueryObjectEXT", "invalid parameter name");
     return nullptr;
 }
 
