@@ -35,18 +35,13 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(EXTShaderTextureLOD);
 
 EXTShaderTextureLOD::EXTShaderTextureLOD(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::EXTShaderTextureLOD)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_EXT_shader_texture_lod"_s);
 }
 
 EXTShaderTextureLOD::~EXTShaderTextureLOD() = default;
 
-WebGLExtension::ExtensionName EXTShaderTextureLOD::getName() const
-{
-    return EXTShaderTextureLODName;
-}
-
 bool EXTShaderTextureLOD::supported(GraphicsContextGL& context)
 {
     return context.supportsExtension("GL_EXT_shader_texture_lod"_s);
