@@ -26,8 +26,8 @@
 #include "config.h"
 
 #if ENABLE(WEBGL)
-
 #include "EXTTextureFilterAnisotropic.h"
+
 #include <wtf/IsoMallocInlines.h>
 
 namespace WebCore {
@@ -35,15 +35,16 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(EXTTextureFilterAnisotropic);
 
 EXTTextureFilterAnisotropic::EXTTextureFilterAnisotropic(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::EXTTextureFilterAnisotropic)
 {
+    context.graphicsContextGL()->ensureExtensionEnabled("GL_EXT_texture_filter_anisotropic"_s);
 }
 
 EXTTextureFilterAnisotropic::~EXTTextureFilterAnisotropic() = default;
 
-WebGLExtension::ExtensionName EXTTextureFilterAnisotropic::getName() const
+bool EXTTextureFilterAnisotropic::supported(GraphicsContextGL& context)
 {
-    return EXTTextureFilterAnisotropicName;
+    return context.supportsExtension("GL_EXT_texture_filter_anisotropic"_s);
 }
 
 } // namespace WebCore
