@@ -35,18 +35,13 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(EXTTextureNorm16);
 
 EXTTextureNorm16::EXTTextureNorm16(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::EXTTextureNorm16)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_EXT_texture_norm16"_s);
 }
 
 EXTTextureNorm16::~EXTTextureNorm16() = default;
 
-WebGLExtension::ExtensionName EXTTextureNorm16::getName() const
-{
-    return EXTTextureNorm16Name;
-}
-
 bool EXTTextureNorm16::supported(GraphicsContextGL& context)
 {
     return context.supportsExtension("GL_EXT_texture_norm16"_s);
