@@ -35,18 +35,13 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(KHRParallelShaderCompile);
 
 KHRParallelShaderCompile::KHRParallelShaderCompile(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::KHRParallelShaderCompile)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_KHR_parallel_shader_compile"_s);
 }
 
 KHRParallelShaderCompile::~KHRParallelShaderCompile() = default;
 
-WebGLExtension::ExtensionName KHRParallelShaderCompile::getName() const
-{
-    return KHRParallelShaderCompileName;
-}
-
 bool KHRParallelShaderCompile::supported(GraphicsContextGL& context)
 {
     return context.supportsExtension("GL_KHR_parallel_shader_compile"_s);
