@@ -35,18 +35,13 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(NVShaderNoperspectiveInterpolation);
 
 NVShaderNoperspectiveInterpolation::NVShaderNoperspectiveInterpolation(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::NVShaderNoperspectiveInterpolation)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_NV_shader_noperspective_interpolation"_s);
 }
 
 NVShaderNoperspectiveInterpolation::~NVShaderNoperspectiveInterpolation() = default;
 
-WebGLExtension::ExtensionName NVShaderNoperspectiveInterpolation::getName() const
-{
-    return NVShaderNoperspectiveInterpolationName;
-}
-
 bool NVShaderNoperspectiveInterpolation::supported(GraphicsContextGL& context)
 {
     return context.supportsExtension("GL_NV_shader_noperspective_interpolation"_s);
