@@ -35,18 +35,13 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(OESStandardDerivatives);
 
 OESStandardDerivatives::OESStandardDerivatives(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::OESStandardDerivatives)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_OES_standard_derivatives"_s);
 }
 
 OESStandardDerivatives::~OESStandardDerivatives() = default;
 
-WebGLExtension::ExtensionName OESStandardDerivatives::getName() const
-{
-    return OESStandardDerivativesName;
-}
-
 bool OESStandardDerivatives::supported(GraphicsContextGL& context)
 {
     return context.supportsExtension("GL_OES_standard_derivatives"_s);
