@@ -35,18 +35,13 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(OESTextureFloatLinear);
 
 OESTextureFloatLinear::OESTextureFloatLinear(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::OESTextureFloatLinear)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_OES_texture_float_linear"_s);
 }
 
 OESTextureFloatLinear::~OESTextureFloatLinear() = default;
 
-WebGLExtension::ExtensionName OESTextureFloatLinear::getName() const
-{
-    return OESTextureFloatLinearName;
-}
-
 bool OESTextureFloatLinear::supported(GraphicsContextGL& context)
 {
     return context.supportsExtension("GL_OES_texture_float_linear"_s);
