@@ -35,18 +35,13 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(WebGLDrawBuffers);
 
 WebGLDrawBuffers::WebGLDrawBuffers(WebGLRenderingContextBase& context)
-    : WebGLExtension(context)
+    : WebGLExtension(context, WebGLExtensionName::WebGLDrawBuffers)
 {
     context.graphicsContextGL()->ensureExtensionEnabled("GL_EXT_draw_buffers"_s);
 }
 
 WebGLDrawBuffers::~WebGLDrawBuffers() = default;
 
-WebGLExtension::ExtensionName WebGLDrawBuffers::getName() const
-{
-    return WebGLExtension::WebGLDrawBuffersName;
-}
-
 bool WebGLDrawBuffers::supported(WebGLRenderingContextBase& context)
 {
     return context.graphicsContextGL()->supportsExtension("GL_EXT_draw_buffers"_s);
@@ -54,36 +49,36 @@ bool WebGLDrawBuffers::supported(WebGLRenderingContextBase& context)
 
 void WebGLDrawBuffers::drawBuffersWEBGL(const Vector<GCGLenum>& buffers)
 {
-    auto context = WebGLExtensionScopedContext(this);
-    if (context.isLost())
+    if (isContextLost())
         return;
+    auto& context = this->context();
     GCGLsizei n = buffers.size();
     const GCGLenum* bufs = buffers.data();
-    if (!context->m_framebufferBinding) {
+    if (!context.m_framebufferBinding) {
         if (n != 1) {
-            context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "drawBuffersWEBGL", "more or fewer than one buffer");
+            context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "drawBuffersWEBGL", "more or fewer than one buffer");
             return;
         }
         if (bufs[0] != GraphicsContextGL::BACK && bufs[0] != GraphicsContextGL::NONE) {
-            context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "drawBuffersWEBGL", "BACK or NONE");
+            context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "drawBuffersWEBGL", "BACK or NONE");
             return;
         }
         // Because the backbuffer is simulated on all current WebKit ports, we need to change BACK to COLOR_ATTACHMENT0.
         GCGLenum value[1] { bufs[0] == GraphicsContextGL::BACK ? GraphicsContextGL::COLOR_ATTACHMENT0 : GraphicsContextGL::NONE };
-        context->graphicsContextGL()->drawBuffersEXT(value);
-        context->setBackDrawBuffer(bufs[0]);
+        context.graphicsContextGL()->drawBuffersEXT(value);
+        context.setBackDrawBuffer(bufs[0]);
     } else {
-        if (n > context->getMaxDrawBuffers()) {
-            context->synthesizeGLError(GraphicsContextGL::INVALID_VALUE, "drawBuffersWEBGL", "more than max draw buffers");
+        if (n > context.maxDrawBuffers()) {
+            context.synthesizeGLError(GraphicsContextGL::INVALID_VALUE, "drawBuffersWEBGL", "more than max draw buffers");
             return;
         }
         for (GCGLsizei i = 0; i < n; ++i) {
             if (bufs[i] != GraphicsContextGL::NONE && bufs[i] != GraphicsContextGL::COLOR_ATTACHMENT0_EXT + i) {
-                context->synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "drawBuffersWEBGL", "COLOR_ATTACHMENTi_EXT or NONE");
+                context.synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, "drawBuffersWEBGL", "COLOR_ATTACHMENTi_EXT or NONE");
                 return;
             }
         }
-        context->m_framebufferBinding->drawBuffers(buffers);
+        context.m_framebufferBinding->drawBuffers(buffers);
     }
 }
 
