@@ -38,12 +38,13 @@
 #include "WebGLAny.h"
 #include "WebGLBuffer.h"
 #include "WebGLContextAttributes.h"
+#include "WebGLExtension.h"
+#include "WebGLExtensionAny.h"
 #include "WebGLFramebuffer.h"
 #include "WebGLProgram.h"
 #include "WebGLQuery.h"
 #include "WebGLRenderbuffer.h"
 #include "WebGLSampler.h"
-#include "WebGLStateTracker.h"
 #include "WebGLTexture.h"
 #include "WebGLTimerQueryEXT.h"
 #include "WebGLTransformFeedback.h"
@@ -52,6 +53,7 @@
 #include <JavaScriptCore/ConsoleTypes.h>
 #include <JavaScriptCore/GenericTypedArrayView.h>
 #include <JavaScriptCore/TypedArrayAdaptors.h>
+#include <array>
 #include <limits>
 #include <memory>
 #include <wtf/CheckedArithmetic.h>
@@ -75,6 +77,7 @@ class AbstractLocker;
 namespace WebCore {
 
 class ANGLEInstancedArrays;
+class EXTBlendFuncExtended;
 class EXTBlendMinMax;
 class EXTClipControl;
 class EXTColorBufferFloat;
@@ -85,15 +88,15 @@ class EXTDisjointTimerQuery;
 class EXTDisjointTimerQueryWebGL2;
 class EXTFloatBlend;
 class EXTFragDepth;
-class EXTShaderTextureLOD;
 class EXTPolygonOffsetClamp;
 class EXTRenderSnorm;
+class EXTShaderTextureLOD;
+class EXTsRGB;
 class EXTTextureCompressionBPTC;
 class EXTTextureCompressionRGTC;
 class EXTTextureFilterAnisotropic;
 class EXTTextureMirrorClampToEdge;
 class EXTTextureNorm16;
-class EXTsRGB;
 class HTMLImageElement;
 class ImageData;
 class IntSize;
@@ -121,24 +124,22 @@ class WebGLCompressedTextureETC1;
 class WebGLCompressedTexturePVRTC;
 class WebGLCompressedTextureS3TC;
 class WebGLCompressedTextureS3TCsRGB;
-class WebGLContextGroup;
-class WebGLContextObject;
 class WebGLDebugRendererInfo;
 class WebGLDebugShaders;
+class WebGLDefaultFramebuffer;
 class WebGLDepthTexture;
 class WebGLDrawBuffers;
 class WebGLDrawInstancedBaseVertexBaseInstance;
-class WebGLExtension;
 class WebGLLoseContext;
 class WebGLMultiDraw;
 class WebGLMultiDrawInstancedBaseVertexBaseInstance;
 class WebGLObject;
 class WebGLPolygonMode;
+class WebGLPolygonMode;
 class WebGLProvokingVertex;
 class WebGLRenderSharedExponent;
 class WebGLShader;
 class WebGLShaderPrecisionFormat;
-class WebGLSharedObject;
 class WebGLStencilTexturing;
 class WebGLUniformLocation;
 
@@ -157,42 +158,14 @@ using WebGLCanvas = std::variant<RefPtr<HTMLCanvasElement>>;
 class VideoFrame;
 #endif
 
-class InspectorScopedShaderProgramHighlight {
-public:
-    InspectorScopedShaderProgramHighlight(WebGLRenderingContextBase&, WebGLProgram*);
-
-    ~InspectorScopedShaderProgramHighlight();
-
-private:
-    void showHighlight();
-    void hideHighlight();
-
-    struct {
-        GCGLfloat color[4];
-        GCGLenum equationRGB;
-        GCGLenum equationAlpha;
-        GCGLenum srcRGB;
-        GCGLenum dstRGB;
-        GCGLenum srcAlpha;
-        GCGLenum dstAlpha;
-        GCGLboolean enabled;
-    } m_savedBlend;
-
-    WebGLRenderingContextBase& m_context;
-    WebGLProgram* m_program { nullptr };
-    bool m_didApply { false };
-};
-
 class WebGLRenderingContextBase : public GraphicsContextGL::Client, public GPUBasedCanvasRenderingContext, private ActivityStateChangeObserver {
     WTF_MAKE_ISO_ALLOCATED(WebGLRenderingContextBase);
 public:
-    using WebGLVersion = GraphicsContextGLWebGLVersion;
-
     using GPUBasedCanvasRenderingContext::weakPtrFactory;
     using GPUBasedCanvasRenderingContext::WeakValueType;
     using GPUBasedCanvasRenderingContext::WeakPtrImplType;
 
-    static std::unique_ptr<WebGLRenderingContextBase> create(CanvasBase&, WebGLContextAttributes&, WebGLVersion);
+    static std::unique_ptr<WebGLRenderingContextBase> create(CanvasBase&, WebGLContextAttributes, WebGLVersion);
     virtual ~WebGLRenderingContextBase();
 
     WebGLCanvas canvas();
@@ -275,8 +248,9 @@ public:
     GCGLint getAttribLocation(WebGLProgram&, const String& name);
     WebGLAny getBufferParameter(GCGLenum target, GCGLenum pname);
     WEBCORE_EXPORT std::optional<WebGLContextAttributes> getContextAttributes();
+    WebGLContextAttributes creationAttributes() const { return m_creationAttributes; }
     GCGLenum getError();
-    virtual WebGLExtension* getExtension(const String& name) = 0;
+    virtual std::optional<WebGLExtensionAny> getExtension(const String& name) = 0;
     virtual WebGLAny getFramebufferAttachmentParameter(GCGLenum target, GCGLenum attachment, GCGLenum pname) = 0;
     virtual WebGLAny getParameter(GCGLenum pname);
     WebGLAny getProgramParameter(WebGLProgram&, GCGLenum pname);
@@ -297,9 +271,6 @@ public:
 
     bool isPreservingDrawingBuffer() const { return m_attributes.preserveDrawingBuffer; }
 
-    bool preventBufferClearForInspector() const { return m_preventBufferClearForInspector; }
-    void setPreventBufferClearForInspector(bool value) { m_preventBufferClearForInspector = value; }
-
     void hint(GCGLenum target, GCGLenum mode);
     GCGLboolean isBuffer(WebGLBuffer*);
     bool isContextLost() const;
@@ -312,12 +283,12 @@ public:
 
     void lineWidth(GCGLfloat);
     void linkProgram(WebGLProgram&);
-    bool linkProgramWithoutInvalidatingAttribLocations(WebGLProgram*);
+    bool linkProgramWithoutInvalidatingAttribLocations(WebGLProgram&);
     virtual void pixelStorei(GCGLenum pname, GCGLint param);
 #if ENABLE(WEBXR)
     using MakeXRCompatiblePromise = DOMPromiseDeferred<void>;
     void makeXRCompatible(MakeXRCompatiblePromise&&);
-    bool isXRCompatible() const { return m_isXRCompatible; }
+    bool isXRCompatible() const { return m_attributes.xrCompatible; }
 #endif
     void polygonOffset(GCGLfloat factor, GCGLfloat units);
     // This must be virtual so more validation can be added in WebGL 2.0.
@@ -434,6 +405,11 @@ public:
 
     void viewport(GCGLint x, GCGLint y, GCGLsizei width, GCGLsizei height);
 
+    virtual GCGLint maxDrawBuffers() = 0;
+    virtual GCGLint maxColorAttachments() = 0;
+    size_t maxVertexAttribs() const { return m_vertexAttribValue.size(); }
+    GCGLint maxSamples() const { return m_maxSamples; }
+
     // WEBKIT_lose_context support
     enum LostContextMode {
         // Lost context occurred at the graphics system level.
@@ -444,27 +420,23 @@ public:
     };
     void forceLostContext(LostContextMode);
     void forceRestoreContext();
-    void loseContextImpl(LostContextMode);
     using SimulatedEventForTesting = GraphicsContextGL::SimulatedEventForTesting;
     WEBCORE_EXPORT void simulateEventForTesting(SimulatedEventForTesting);
 
     GraphicsContextGL* graphicsContextGL() const { return m_context.get(); }
-    WebGLContextGroup* contextGroup() const { return m_contextGroup.get(); }
     RefPtr<GraphicsLayerContentsDisplayDelegate> layerContentsDisplayDelegate() override;
 
     void reshape(int width, int height) override;
 
-    void prepareForDisplayWithPaint() final;
-    void paintRenderingResultsToCanvas() final;
-    RefPtr<PixelBuffer> paintRenderingResultsToPixelBuffer(GraphicsContextGL::FlipY);
+    void drawBufferToCanvas(SurfaceBuffer) final;
+
+    RefPtr<PixelBuffer> drawingBufferToPixelBuffer(GraphicsContextGL::FlipY);
 #if ENABLE(MEDIA_STREAM) || ENABLE(WEB_CODECS)
-    RefPtr<VideoFrame> paintCompositedResultsToVideoFrame();
+    RefPtr<VideoFrame> surfaceBufferToVideoFrame(SurfaceBuffer);
 #endif
 
-    void removeSharedObject(WebGLSharedObject&);
-    void removeContextObject(WebGLContextObject&);
-
-    unsigned getMaxVertexAttribs() const { return m_maxVertexAttribs; }
+    void removeSharedObject(WebGLObject&);
+    void removeContextObject(WebGLObject&);
 
     bool isContextUnrecoverablyLost() const;
 
@@ -475,7 +447,6 @@ public:
 
     // GraphicsContextGL::Client
     void forceContextLost() override;
-    void dispatchContextChangedNotification() override;
 
     void recycleContext();
 
@@ -503,8 +474,9 @@ public:
     const PixelStoreParameters& pixelStorePackParameters() const { return m_packParameters; }
     const PixelStoreParameters& unpackPixelStoreParameters() const { return m_unpackParameters; };
 
+    WeakPtr<WebGLRenderingContextBase> createRefForContextObject();
 protected:
-    WebGLRenderingContextBase(CanvasBase&, Ref<GraphicsContextGL>&&, WebGLContextAttributes);
+    WebGLRenderingContextBase(CanvasBase&, WebGLContextAttributes&&);
 
     friend class EXTDisjointTimerQuery;
     friend class EXTDisjointTimerQueryWebGL2;
@@ -523,6 +495,7 @@ protected:
     friend class WebGLDrawInstancedBaseVertexBaseInstance;
     friend class WebGLMultiDraw;
     friend class WebGLMultiDrawInstancedBaseVertexBaseInstance;
+    friend class WebGLPolygonMode;
 
     friend class WebGLFramebuffer;
     friend class WebGLObject;
@@ -533,10 +506,17 @@ protected:
     friend class WebGLVertexArrayObjectOES;
 
     // Implementation helpers.
-    friend class InspectorScopedShaderProgramHighlight;
+    friend class ScopedDisableRasterizerDiscard;
+    friend class ScopedDisableScissorTest;
+    friend class ScopedEnableBackbuffer;
+    friend class ScopedInspectorShaderProgramHighlight;
+    friend class ScopedWebGLRestoreFramebuffer;
+    friend class ScopedWebGLRestoreRenderbuffer;
+    friend class ScopedWebGLRestoreTexture;
 
-    virtual void initializeNewContext();
-    virtual void initializeVertexArrayObjects() = 0;
+    void initializeNewContext(Ref<GraphicsContextGL>);
+    virtual void initializeContextState();
+    virtual void initializeDefaultObjects();
 
     // ActiveDOMObject
     void stop() override;
@@ -544,11 +524,8 @@ protected:
     void suspend(ReasonForSuspension) override;
     void resume() override;
 
-    void addSharedObject(WebGLSharedObject&);
-    void addContextObject(WebGLContextObject&);
     void detachAndRemoveAllObjects();
 
-    void setGraphicsContextGL(Ref<GraphicsContextGL>&&);
     void destroyGraphicsContextGL();
 
     enum CallerType {
@@ -558,7 +535,6 @@ protected:
         CallerTypeOther,
     };
 
-    void markContextChanged();
     void markContextChangedAndNotifyCanvasObserver(CallerType = CallerTypeDrawOrClear);
 
     void addActivityStateChangeObserverIfNecessary();
@@ -571,15 +547,18 @@ protected:
     // like GL_FLOAT, GL_INT, etc.
     unsigned sizeInBytes(GCGLenum type);
 
-    // Validates the incoming WebGL object, which is assumed to be non-null.
-    // Checks that the object belongs to this context and that it's not marked for
-    // deletion. Performs a context lost check internally.
-    bool validateWebGLObject(const char*, WebGLObject*);
+    // Validates the incoming WebGL object.
+    template<typename T> bool validateWebGLObject(const char*, const T&);
+    // Helper function for APIs which can legally receive null objects, including
+    // the bind* calls (bindBuffer, bindTexture, etc.) and useProgram.
+    // This returns true for null WebGLObject arguments!
+    template<typename T> bool validateNullableWebGLObject(const char*, const T*);
+    template<typename T> GCGLboolean validateIsWebGLObject(const T*) const;
 
     // Validates the incoming WebGL program or shader, which is assumed to be
     // non-null. OpenGL ES's validation rules differ for these types of objects
     // compared to others. Performs a context lost check internally.
-    bool validateWebGLProgramOrShader(const char*, WebGLObject*);
+    bool validateWebGLObject(const char*, WebGLObject*);
 
     bool validateVertexArrayObject(const char* functionName);
 
@@ -589,7 +568,7 @@ protected:
     RefPtr<Image> drawImageIntoBuffer(Image&, int width, int height, int deviceScaleFactor, const char* functionName);
 
 #if ENABLE(VIDEO)
-    RefPtr<Image> videoFrameToImage(HTMLVideoElement&, BackingStoreCopy, const char* functionName);
+    RefPtr<Image> videoFrameToImage(HTMLVideoElement&, const char* functionName);
 #endif
 
     WebGLTexture::TextureExtensionFlag textureExtensionFlags() const;
@@ -615,20 +594,17 @@ protected:
     };
 
     RefPtr<GraphicsContextGL> m_context;
-    RefPtr<WebGLContextGroup> m_contextGroup;
     Lock m_objectGraphLock;
 
     EventLoopTimerHandle m_restoreTimer;
     GCGLErrorCodeSet m_errors;
-    bool m_needsUpdate;
-    bool m_markedCanvasDirty;
-    HashSet<WebGLContextObject*> m_contextObjects;
 
     // List of bound VBO's. Used to maintain info about sizes for ARRAY_BUFFER and stored values for ELEMENT_ARRAY_BUFFER
-    RefPtr<WebGLBuffer> m_boundArrayBuffer;
+    WebGLBindingPoint<WebGLBuffer, GraphicsContextGL::ARRAY_BUFFER> m_boundArrayBuffer;
 
+    std::unique_ptr<WebGLDefaultFramebuffer> m_defaultFramebuffer;
     RefPtr<WebGLVertexArrayObjectBase> m_defaultVertexArrayObject;
-    RefPtr<WebGLVertexArrayObjectBase> m_boundVertexArrayObject;
+    WebGLBindingPoint<WebGLVertexArrayObjectBase> m_boundVertexArrayObject;
 
     void setBoundVertexArrayObject(const AbstractLocker&, WebGLVertexArrayObjectBase*);
 
@@ -656,23 +632,19 @@ protected:
         };
     };
     Vector<VertexAttribValue> m_vertexAttribValue;
-    unsigned m_maxVertexAttribs;
 
     RefPtr<WebGLProgram> m_currentProgram;
-    RefPtr<WebGLFramebuffer> m_framebufferBinding;
-    RefPtr<WebGLRenderbuffer> m_renderbufferBinding;
+    WebGLBindingPoint<WebGLFramebuffer> m_framebufferBinding;
+    WebGLBindingPoint<WebGLRenderbuffer> m_renderbufferBinding;
     struct TextureUnitState {
-        RefPtr<WebGLTexture> texture2DBinding;
-        RefPtr<WebGLTexture> textureCubeMapBinding;
-        RefPtr<WebGLTexture> texture3DBinding;
-        RefPtr<WebGLTexture> texture2DArrayBinding;
+        WebGLBindingPoint<WebGLTexture, GraphicsContextGL::TEXTURE_2D> texture2DBinding;
+        WebGLBindingPoint<WebGLTexture, GraphicsContextGL::TEXTURE_CUBE_MAP> textureCubeMapBinding;
+        WebGLBindingPoint<WebGLTexture, GraphicsContextGL::TEXTURE_3D> texture3DBinding;
+        WebGLBindingPoint<WebGLTexture, GraphicsContextGL::TEXTURE_2D_ARRAY> texture2DArrayBinding;
     };
     Vector<TextureUnitState> m_textureUnits;
     unsigned long m_activeTextureUnit;
 
-    RefPtr<WebGLTexture> m_blackTexture2D;
-    RefPtr<WebGLTexture> m_blackTextureCubeMap;
-
     Vector<GCGLenum> m_compressedTextureFormats;
 
     // Fixed-size cache of reusable image buffers for video texImage2D calls.
@@ -681,7 +653,7 @@ protected:
         LRUImageBufferCache(int capacity);
         // Returns pointer to a cleared image buffer that is owned by the cache. The pointer is valid until next call.
         // Using fillOperator == CompositeOperator::Copy can be used to omit the clear of the buffer.
-        ImageBuffer* imageBuffer(const IntSize&, DestinationColorSpace, CompositeOperator fillOperator = CompositeOperator::SourceOver);
+        RefPtr<ImageBuffer> imageBuffer(const IntSize&, DestinationColorSpace, CompositeOperator fillOperator = CompositeOperator::SourceOver);
     private:
         void bubbleToFront(size_t idx);
         Vector<std::optional<std::pair<DestinationColorSpace, Ref<ImageBuffer>>>> m_buffers;
@@ -691,9 +663,10 @@ protected:
     GCGLint m_maxTextureSize;
     GCGLint m_maxCubeMapTextureSize;
     GCGLint m_maxRenderbufferSize;
-    GCGLint m_maxViewportDims[2] { 0, 0 };
+    std::array<GCGLint, 2> m_maxViewportDims { 0, 0 };
     GCGLint m_maxTextureLevel;
     GCGLint m_maxCubeMapTextureLevel;
+    GCGLint m_maxSamples { 0 };
 
     GCGLint m_maxDrawBuffers;
     GCGLint m_maxColorAttachments;
@@ -708,11 +681,11 @@ protected:
     GCGLenum m_unpackColorspaceConversion;
 
     std::optional<ContextLostState>  m_contextLostState;
-    WebGLContextAttributes m_attributes;
+    WebGLContextAttributes m_attributes; // "actual context parameters" in WebGL 1 spec.
+    WebGLContextAttributes m_creationAttributes; // "context creation parameters" in WebGL 1 spec.
 
     PredefinedColorSpace m_drawingBufferColorSpace { PredefinedColorSpace::SRGB };
 
-    bool m_layerCleared;
     GCGLfloat m_clearColor[4];
     bool m_scissorEnabled;
     GCGLfloat m_clearDepth;
@@ -730,17 +703,15 @@ protected:
     bool m_isGLES2Compliant;
     bool m_isDepthStencilSupported;
 
-    bool m_synthesizedErrorsToConsole { true };
     int m_numGLErrorsToConsoleAllowed;
 
-    bool m_preventBufferClearForInspector { false };
-
     bool m_compositingResultsNeedUpdating { false };
-    bool m_isDisplayingWithPaint { false };
+    std::optional<SurfaceBuffer> m_canvasBufferContents;
 
     // Enabled extension objects.
     // FIXME: Move some of these to WebGLRenderingContext, the ones not needed for WebGL2
     RefPtr<ANGLEInstancedArrays> m_angleInstancedArrays;
+    RefPtr<EXTBlendFuncExtended> m_extBlendFuncExtended;
     RefPtr<EXTBlendMinMax> m_extBlendMinMax;
     RefPtr<EXTClipControl> m_extClipControl;
     RefPtr<EXTColorBufferFloat> m_extColorBufferFloat;
@@ -818,9 +789,6 @@ protected:
     // and the function returns true if that clear is now unnecessary.
     bool clearIfComposited(CallerType, GCGLbitfield clearMask = 0);
 
-    // Helper to restore state that clearing the framebuffer may destroy.
-    void restoreStateAfterClear();
-
     enum class TexImageFunctionType {
         TexImage,
         TexSubImage,
@@ -992,8 +960,6 @@ protected:
     // Get the framebuffer bound to the given target.
     virtual WebGLFramebuffer* getFramebufferBinding(GCGLenum target);
 
-    virtual WebGLFramebuffer* getReadFramebufferBinding();
-
     // Helper function to validate input parameters for framebuffer functions.
     // Generate GL error if parameters are illegal.
     bool validateFramebufferFuncParameters(const char* functionName, GCGLenum target, GCGLenum attachment);
@@ -1001,9 +967,6 @@ protected:
     // Helper function to validate blend equation mode.
     virtual bool validateBlendEquation(const char* functionName, GCGLenum) = 0;
 
-    // Helper function to validate blend func factors.
-    bool validateBlendFuncFactors(const char* functionName, GCGLenum src, GCGLenum dst);
-
     // Helper function to validate a GL capability.
     virtual bool validateCapability(const char* functionName, GCGLenum);
 
@@ -1040,14 +1003,6 @@ protected:
     // Return false if caller should return without further processing.
     bool deleteObject(const AbstractLocker&, WebGLObject*);
 
-    // Helper function for APIs which can legally receive null objects, including
-    // the bind* calls (bindBuffer, bindTexture, etc.) and useProgram. Checks that
-    // the object belongs to this context and that it's not marked for deletion.
-    // Returns false if the caller should return without further processing.
-    // Performs a context lost check internally.
-    // This returns true for null WebGLObject arguments!
-    bool validateNullableWebGLObject(const char* functionName, WebGLObject*);
-
     // Helper function to validate the target for bufferData and
     // getBufferParameter.
     virtual bool validateBufferTarget(const char* functionName, GCGLenum target);
@@ -1062,23 +1017,12 @@ protected:
     void synthesizeGLError(GCGLenum, const char* functionName, const char* description);
     void synthesizeLostContextGLError(GCGLenum, const char* functionName, const char* description);
 
-    String ensureNotNull(const String&) const;
-
-    // Helper for enabling or disabling a capability.
-    void enableOrDisable(GCGLenum capability, bool enable);
-
     // Clamp the width and height to GL_MAX_VIEWPORT_DIMS.
     IntSize clampedCanvasSize();
 
-    virtual GCGLint getMaxDrawBuffers() = 0;
-    virtual GCGLint getMaxColorAttachments() = 0;
-
     void setBackDrawBuffer(GCGLenum);
     void setFramebuffer(const AbstractLocker&, GCGLenum, WebGLFramebuffer*);
 
-    virtual void restoreCurrentFramebuffer();
-    void restoreCurrentTexture2D();
-
     // Check if EXT_draw_buffers extension is supported and if it satisfies the WebGL requirements.
     bool supportsDrawBuffers();
 
@@ -1099,9 +1043,6 @@ private:
     void maybeRestoreContextSoon(Seconds timeout = 0_s);
     void maybeRestoreContext();
 
-    void registerWithWebGLStateTracker();
-    void checkForContextLossHandling();
-
     void activityStateDidChange(OptionSet<ActivityState> oldActivityState, OptionSet<ActivityState> newActivityState) override;
 
     ExceptionOr<void> texImageSource(TexImageFunctionID, GCGLenum target, GCGLint level, GCGLint internalformat, GCGLint border, GCGLenum format, GCGLenum type, GCGLint xoffset, GCGLint yoffset, GCGLint zoffset, const IntRect& inputSourceImageRect, GCGLsizei depth, GCGLint unpackImageHeight, ImageBitmap& source);
@@ -1118,19 +1059,52 @@ private:
     ExceptionOr<void> texImageSource(TexImageFunctionID, GCGLenum target, GCGLint level, GCGLint internalformat, GCGLint border, GCGLenum format, GCGLenum type, GCGLint xoffset, GCGLint yoffset, GCGLint zoffset, const IntRect& inputSourceImageRect, GCGLsizei depth, GCGLint unpackImageHeight, WebCodecsVideoFrame& source);
 #endif
 
-    WebGLStateTracker::Token m_trackerToken;
-    Timer m_checkForContextLossHandlingTimer;
     bool m_isSuspended { false };
 
-#if ENABLE(WEBXR)
-    bool m_isXRCompatible { false };
-#endif
     // The ordinal number of when the context was last active (drew, read pixels).
     uint64_t m_activeOrdinal { 0 };
+    WeakPtrFactory<WebGLRenderingContextBase> m_contextObjectWeakPtrFactory;
 };
 
+template<typename T>
+bool WebGLRenderingContextBase::validateWebGLObject(const char* functionName, const T& object)
+{
+    if (object.context() != this) {
+        synthesizeGLError(GraphicsContextGL::INVALID_OPERATION, functionName, "object does not belong to this context");
+        return false;
+    }
+    if (!object.isUsable()) {
+        constexpr GCGLenum error = (std::is_same_v<T, WebGLProgram> || std::is_same_v<T, WebGLShader>) ? GraphicsContextGL::INVALID_VALUE : GraphicsContextGL::INVALID_OPERATION;
+        synthesizeGLError(error, functionName, "attempt to use a deleted object");
+        return false;
+    }
+    return true;
+}
+
+template<typename T>
+bool WebGLRenderingContextBase::validateNullableWebGLObject(const char* functionName, const T* object)
+{
+    return !object || validateWebGLObject(functionName, *object);
+}
+
+template<typename T>
+GCGLboolean WebGLRenderingContextBase::validateIsWebGLObject(const T* object) const
+{
+    if (!object)
+        return false;
+    if (object->context() != this)
+        return false;
+    if (!object->isUsable())
+        return false;
+    if (!object->isInitialized())
+        return false;
+    return true;
+}
+
 WebCoreOpaqueRoot root(WebGLRenderingContextBase*);
 
+WebCoreOpaqueRoot root(const WebGLExtension<WebGLRenderingContextBase>*);
+
 } // namespace WebCore
 
 SPECIALIZE_TYPE_TRAITS_CANVASRENDERINGCONTEXT(WebCore::WebGLRenderingContextBase, isWebGL())
