@@ -27,9 +27,10 @@
 #pragma once
 
 #include "Document.h"
-#include "FragmentScriptingPermission.h"
 #include "HTMLElementStack.h"
 #include "HTMLFormattingElementList.h"
+#include "ParserContentPolicy.h"
+#include <wtf/CheckedRef.h>
 #include <wtf/FixedVector.h>
 #include <wtf/Noncopyable.h>
 #include <wtf/RefPtr.h>
@@ -73,6 +74,10 @@ struct HTMLConstructionSiteTask {
         return downcast<ContainerNode>(child.get());
     }
 
+    Ref<ContainerNode> protectedNonNullParent() const { return *parent; }
+    Ref<Node> protectedNonNullChild() const { return *child; }
+    Ref<Node> protectedNonNullNextChild() const { return *nextChild; }
+
     Operation operation;
     RefPtr<ContainerNode> parent;
     RefPtr<Node> nextChild;
@@ -159,10 +164,12 @@ public:
     bool isEmpty() const { return !m_openElements.stackDepth(); }
     Element& currentElement() const { return m_openElements.top(); }
     ContainerNode& currentNode() const { return m_openElements.topNode(); }
+    Ref<ContainerNode> protectedCurrentNode() const { return m_openElements.topNode(); }
     ElementName currentElementName() const { return m_openElements.topElementName(); }
     HTMLStackItem& currentStackItem() const { return m_openElements.topStackItem(); }
     HTMLStackItem* oneBelowTop() const { return m_openElements.oneBelowTop(); }
     Document& ownerDocumentForCurrentNode();
+    Ref<Document> protectedOwnerDocumentForCurrentNode() { return ownerDocumentForCurrentNode(); }
     HTMLElementStack& openElements() const { return m_openElements; }
     HTMLFormattingElementList& activeFormattingElements() const { return m_activeFormattingElements; }
     bool currentIsRootNode() { return &m_openElements.topNode() == &m_openElements.rootNode(); }
@@ -177,7 +184,7 @@ public:
     OptionSet<ParserContentPolicy> parserContentPolicy() { return m_parserContentPolicy; }
 
 #if ENABLE(TELEPHONE_NUMBER_DETECTION)
-    bool isTelephoneNumberParsingEnabled() { return m_document.isTelephoneNumberParsingEnabled(); }
+    bool isTelephoneNumberParsingEnabled() { return document().isTelephoneNumberParsingEnabled(); }
 #endif
 
     class RedirectToFosterParentGuard {
@@ -194,6 +201,8 @@ public:
     static bool isFormattingTag(TagName);
 
 private:
+    Document& document() const { return m_document.get(); }
+
     // In the common case, this queue will have only one task because most
     // tokens produce only one DOM mutation.
     typedef Vector<HTMLConstructionSiteTask, 1> TaskQueue;
@@ -201,7 +210,7 @@ private:
     void setCompatibilityMode(DocumentCompatibilityMode);
     void setCompatibilityModeFromDoctype(const AtomString& name, const String& publicId, const String& systemId);
 
-    void attachLater(ContainerNode& parent, Ref<Node>&& child, bool selfClosing = false);
+    void attachLater(Ref<ContainerNode>&& parent, Ref<Node>&& child, bool selfClosing = false);
 
     void findFosterSite(HTMLConstructionSiteTask&);
 
@@ -212,14 +221,19 @@ private:
     void mergeAttributesFromTokenIntoElement(AtomHTMLToken&&, Element&);
     void dispatchDocumentElementAvailableIfNeeded();
 
-    Document& m_document;
+    Ref<Document> protectedDocument() const;
+    Ref<ContainerNode> protectedAttachmentRoot() const;
+
+    // m_head has to be destroyed after destroying CheckedRef of m_document and m_attachmentRoot
+    HTMLStackItem m_head;
+
+    CheckedRef<Document> m_document;
     
     // This is the root ContainerNode to which the parser attaches all newly
     // constructed nodes. It points to a DocumentFragment when parsing fragments
     // and a Document in all other cases.
-    ContainerNode& m_attachmentRoot;
+    WeakRef<ContainerNode, WeakPtrImplWithEventTargetData> m_attachmentRoot;
     
-    HTMLStackItem m_head;
     RefPtr<HTMLFormElement> m_form;
     mutable HTMLElementStack m_openElements;
     mutable HTMLFormattingElementList m_activeFormattingElements;
