@@ -83,16 +83,15 @@ void DateTimeFieldElement::defaultEventHandler(Event& event)
     if (event.type() == eventNames().blurEvent)
         handleBlurEvent(event);
 
-    if (is<KeyboardEvent>(event)) {
-        auto& keyboardEvent = downcast<KeyboardEvent>(event);
+    if (auto* keyboardEvent = dynamicDowncast<KeyboardEvent>(event)) {
         if (!isFieldOwnerDisabled() && !isFieldOwnerReadOnly()) {
-            handleKeyboardEvent(keyboardEvent);
-            if (keyboardEvent.defaultHandled())
+            handleKeyboardEvent(*keyboardEvent);
+            if (keyboardEvent->defaultHandled())
                 return;
         }
 
-        defaultKeyboardEventHandler(keyboardEvent);
-        if (keyboardEvent.defaultHandled())
+        defaultKeyboardEventHandler(*keyboardEvent);
+        if (keyboardEvent->defaultHandled())
             return;
     }
 
@@ -110,12 +109,18 @@ void DateTimeFieldElement::defaultKeyboardEventHandler(KeyboardEvent& keyboardEv
     auto key = keyboardEvent.keyIdentifier();
     auto code = keyboardEvent.code();
 
-    if (key == "Left"_s && m_fieldOwner && m_fieldOwner->focusOnPreviousField(*this)) {
+    bool isHorizontal = isFieldOwnerHorizontal();
+    auto nextKeyIdentifier = isHorizontal ? "Right"_s : "Down"_s;
+    auto previousKeyIdentifier = isHorizontal ? "Left"_s : "Up"_s;
+    auto stepUpKeyIdentifier = isHorizontal ? "Up"_s : "Right"_s;
+    auto stepDownKeyIdentifier = isHorizontal ? "Down"_s : "Left"_s;
+
+    if (key == previousKeyIdentifier && m_fieldOwner && m_fieldOwner->focusOnPreviousField(*this)) {
         keyboardEvent.setDefaultHandled();
         return;
     }
 
-    if ((key == "Right"_s || code == "Comma"_s || code == "Minus"_s || code == "Period"_s || code == "Slash"_s || code == "Semicolon"_s)
+    if ((key == nextKeyIdentifier || code == "Comma"_s || code == "Minus"_s || code == "Period"_s || code == "Slash"_s || code == "Semicolon"_s)
         && m_fieldOwner && m_fieldOwner->focusOnNextField(*this)) {
         keyboardEvent.setDefaultHandled();
         return;
@@ -124,13 +129,13 @@ void DateTimeFieldElement::defaultKeyboardEventHandler(KeyboardEvent& keyboardEv
     if (isFieldOwnerReadOnly())
         return;
 
-    if (key == "Up"_s) {
+    if (key == stepUpKeyIdentifier) {
         stepUp();
         keyboardEvent.setDefaultHandled();
         return;
     }
 
-    if (key == "Down"_s) {
+    if (key == stepDownKeyIdentifier) {
         stepDown();
         keyboardEvent.setDefaultHandled();
         return;
@@ -154,6 +159,13 @@ bool DateTimeFieldElement::isFieldOwnerReadOnly() const
     return m_fieldOwner && m_fieldOwner->isFieldOwnerReadOnly();
 }
 
+bool DateTimeFieldElement::isFieldOwnerHorizontal() const
+{
+    if (m_fieldOwner)
+        return m_fieldOwner->isFieldOwnerHorizontal();
+    return true;
+}
+
 bool DateTimeFieldElement::isFocusable() const
 {
     if (isFieldOwnerDisabled())
