@@ -52,7 +52,7 @@ namespace WebCore {
 // https://dvcs.w3.org/hg/text-tracks/raw-file/default/608toVTT/region.html
 
 // Default region line-height (vh units)
-static const float lineHeight = 5.33;
+static const float lineHeight = 6;
 
 // Default scrolling animation time period (s).
 static const Seconds scrollTime { 433_ms };
@@ -79,23 +79,20 @@ void VTTRegion::setId(const String& id)
 ExceptionOr<void> VTTRegion::setWidth(double value)
 {
     if (!(value >= 0 && value <= 100))
-        return Exception { IndexSizeError };
+        return Exception { WebCore::ExceptionCode::IndexSizeError };
     m_width = value;
     return { };
 }
 
-ExceptionOr<void> VTTRegion::setLines(int value)
+void VTTRegion::setLines(unsigned value)
 {
-    if (value < 0)
-        return Exception { IndexSizeError };
     m_lines = value;
-    return { };
 }
 
 ExceptionOr<void> VTTRegion::setRegionAnchorX(double value)
 {
     if (!(value >= 0 && value <= 100))
-        return Exception { IndexSizeError };
+        return Exception { WebCore::ExceptionCode::IndexSizeError };
     m_regionAnchor.setX(value);
     return { };
 }
@@ -103,7 +100,7 @@ ExceptionOr<void> VTTRegion::setRegionAnchorX(double value)
 ExceptionOr<void> VTTRegion::setRegionAnchorY(double value)
 {
     if (!(value >= 0 && value <= 100))
-        return Exception { IndexSizeError };
+        return Exception { WebCore::ExceptionCode::IndexSizeError };
     m_regionAnchor.setY(value);
     return { };
 }
@@ -111,7 +108,7 @@ ExceptionOr<void> VTTRegion::setRegionAnchorY(double value)
 ExceptionOr<void> VTTRegion::setViewportAnchorX(double value)
 {
     if (!(value >= 0 && value <= 100))
-        return Exception { IndexSizeError };
+        return Exception { WebCore::ExceptionCode::IndexSizeError };
     m_viewportAnchor.setX(value);
     return { };
 }
@@ -119,7 +116,7 @@ ExceptionOr<void> VTTRegion::setViewportAnchorX(double value)
 ExceptionOr<void> VTTRegion::setViewportAnchorY(double value)
 {
     if (!(value >= 0 && value <= 100))
-        return Exception { IndexSizeError };
+        return Exception { WebCore::ExceptionCode::IndexSizeError };
     m_viewportAnchor.setY(value);
     return { };
 }
@@ -130,22 +127,9 @@ static const AtomString& upKeyword()
     return upKeyword;
 }
 
-const AtomString& VTTRegion::scroll() const
+void VTTRegion::setScroll(const ScrollSetting value)
 {
-    return m_scroll ? upKeyword() : emptyAtom();
-}
-
-ExceptionOr<void> VTTRegion::setScroll(const AtomString& value)
-{
-    if (value.isEmpty()) {
-        m_scroll = false;
-        return { };
-    }
-    if (value == upKeyword()) {
-        m_scroll = true;
-        return { };
-    }
-    return Exception { SyntaxError };
+    m_scroll = value;
 }
 
 void VTTRegion::updateParametersFromRegion(const VTTRegion& other)
@@ -159,41 +143,43 @@ void VTTRegion::updateParametersFromRegion(const VTTRegion& other)
 
 void VTTRegion::setRegionSettings(const String& inputString)
 {
+    // https://w3c.github.io/webvtt/#region-settings-parsing
+    // 6.2. WebVTT region settings parsing
     m_settings = inputString;
     VTTScanner input(inputString);
 
     while (!input.isAtEnd()) {
-        input.skipWhile<isTabOrSpace>();
-        if (input.isAtEnd())
-            break;
+        // Step 1 - Split input on spaces.
+        input.skipWhile<isASCIIWhitespace<UChar>>();
+        VTTScanner::Run valueRun = input.collectUntil<isASCIIWhitespace<UChar>>();
+        auto settingValue = input.extractString(valueRun);
+        VTTScanner setting(settingValue);
 
-        // Scan the name part.
-        RegionSetting name = scanSettingName(input);
+        // Step 2.2 - Let name be the leading substring of setting up to and excluding the first U+003A COLON character (:) in that string.
+        RegionSetting name = scanSettingName(setting);
 
-        // Verify that we're looking at a ':'.
-        if (name == None || !input.scan(':')) {
-            input.skipUntil<isASCIIWhitespace<UChar>>();
+        // Step 2.1 - If the first ':' in setting is the last character of setting, then jump to the next setting.
+        if (name == None || setting.isAtEnd())
             continue;
-        }
 
-        // Scan the value part.
-        parseSettingValue(name, input);
+        // Steps 2.3-2.4 - Scan the value part.
+        parseSettingValue(name, setting);
     }
 }
 
 VTTRegion::RegionSetting VTTRegion::scanSettingName(VTTScanner& input)
 {
-    if (input.scan("id"))
+    if (input.scan("id:"))
         return Id;
-    if (input.scan("lines"))
+    if (input.scan("lines:"))
         return Lines;
-    if (input.scan("width"))
+    if (input.scan("width:"))
         return Width;
-    if (input.scan("viewportanchor"))
+    if (input.scan("viewportanchor:"))
         return ViewportAnchor;
-    if (input.scan("regionanchor"))
+    if (input.scan("regionanchor:"))
         return RegionAnchor;
-    if (input.scan("scroll"))
+    if (input.scan("scroll:"))
         return Scroll;
 
     return None;
@@ -224,11 +210,11 @@ void VTTRegion::parseSettingValue(RegionSetting setting, VTTScanner& input)
         break;
     }
     case Lines: {
-        int number;
+        unsigned number;
         if (input.scanDigits(number) && parsedEntireRun(input, valueRun))
             m_lines = number;
         else
-            LOG(Media, "VTTRegion::parseSettingValue, invalid Height");
+            LOG(Media, "VTTRegion::parseSettingValue, invalid Lines");
         break;
     }
     case RegionAnchor: {
@@ -249,7 +235,7 @@ void VTTRegion::parseSettingValue(RegionSetting setting, VTTScanner& input)
     }
     case Scroll:
         if (input.scanRun(valueRun, upKeyword()))
-            m_scroll = true;
+            m_scroll = ScrollSetting::Up;
         else
             LOG(Media, "VTTRegion::parseSettingValue, invalid Scroll");
         break;
@@ -287,7 +273,7 @@ void VTTRegion::displayLastTextTrackCueBox()
         return;
 
     // If it's a scrolling region, add the scrolling class.
-    if (isScrollingRegion())
+    if (scroll() == ScrollSetting::Up)
         m_cueContainer->classList().add(textTrackCueContainerScrollingClass());
 
     float regionBottom = m_regionDisplayTree->boundingClientRect().maxY();
@@ -347,30 +333,6 @@ void VTTRegion::prepareRegionDisplayTree()
     // FIXME: Change the code below to use viewport units when
     // http://crbug/244618 is fixed.
 
-    // Let regionWidth be the text track region width.
-    // Let width be 'regionWidth vw' ('vw' is a CSS unit)
-    m_regionDisplayTree->setInlineStyleProperty(CSSPropertyWidth, m_width, CSSUnitType::CSS_PERCENTAGE);
-
-    // Let lineHeight be '0.0533vh' ('vh' is a CSS unit) and regionHeight be
-    // the text track region height. Let height be 'lineHeight' multiplied
-    // by regionHeight.
-    double height = lineHeight * m_lines;
-    m_regionDisplayTree->setInlineStyleProperty(CSSPropertyHeight, height, CSSUnitType::CSS_VH);
-
-    // Let viewportAnchorX be the x dimension of the text track region viewport
-    // anchor and regionAnchorX be the x dimension of the text track region
-    // anchor. Let leftOffset be regionAnchorX multiplied by width divided by
-    // 100.0. Let left be leftOffset subtracted from 'viewportAnchorX vw'.
-    double leftOffset = m_regionAnchor.x() * m_width / 100;
-    m_regionDisplayTree->setInlineStyleProperty(CSSPropertyLeft, m_viewportAnchor.x() - leftOffset, CSSUnitType::CSS_PERCENTAGE);
-
-    // Let viewportAnchorY be the y dimension of the text track region viewport
-    // anchor and regionAnchorY be the y dimension of the text track region
-    // anchor. Let topOffset be regionAnchorY multiplied by height divided by
-    // 100.0. Let top be topOffset subtracted from 'viewportAnchorY vh'.
-    double topOffset = m_regionAnchor.y() * height / 100;
-    m_regionDisplayTree->setInlineStyleProperty(CSSPropertyTop, m_viewportAnchor.y() - topOffset, CSSUnitType::CSS_PERCENTAGE);
-
     // The cue container is used to wrap the cues and it is the object which is
     // gradually scrolled out as multiple cues are appended to the region.
     if (!m_cueContainer) {
@@ -378,7 +340,39 @@ void VTTRegion::prepareRegionDisplayTree()
         m_cueContainer->setPseudo(ShadowPseudoIds::webkitMediaTextTrackRegionContainer());
         m_regionDisplayTree->appendChild(*m_cueContainer);
     }
-    m_cueContainer->setInlineStyleProperty(CSSPropertyTop, 0.0f, CSSUnitType::CSS_PX);
+
+    // Let regionWidth be the WebVTT region width.
+    // Let width be 'regionWidth vw' ('vw' is a CSS unit)
+    m_regionDisplayTree->setInlineStyleProperty(CSSPropertyWidth, m_width, CSSUnitType::CSS_CQW);
+    m_cueContainer->setInlineStyleProperty(CSSPropertyWidth, m_width, CSSUnitType::CSS_CQW);
+
+    // Let lineHeight be '6vh' ('vh' is a CSS unit) and regionHeight be
+    // the WebVTT region lines. Let lines be 'lineHeight' multiplied
+    // by regionHeight.
+    double lines = lineHeight * m_lines;
+
+    // Although the spec does not say to set the height property to lines, without doing so,
+    // the caption is not visible
+    m_regionDisplayTree->setInlineStyleProperty(CSSPropertyHeight, lines, CSSUnitType::CSS_CQH);
+    m_cueContainer->setInlineStyleProperty(CSSPropertyHeight, lines, CSSUnitType::CSS_CQH);
+
+    // Let viewportAnchorX be the x dimension of the WebVTT viewport
+    // anchor and regionAnchorX be the x dimension of the WebVTT region
+    // anchor. Let leftOffset be regionAnchorX multiplied by width divided by
+    // 100.0. Let left be leftOffset subtracted from 'viewportAnchorX vw'.
+    double leftOffset = m_regionAnchor.x() * m_width / 100;
+    double left = m_viewportAnchor.x() - leftOffset;
+    m_regionDisplayTree->setInlineStyleProperty(CSSPropertyLeft, left, CSSUnitType::CSS_CQW);
+    m_cueContainer->setInlineStyleProperty(CSSPropertyLeft, left, CSSUnitType::CSS_CQW);
+
+    // Let viewportAnchorY be the y dimension of the WebVTT region viewport
+    // anchor and regionAnchorY be the y dimension of the WebVTT region
+    // anchor. Let topOffset be regionAnchorY multiplied by lines divided by
+    // 100.0. Let top be topOffset subtracted from 'viewportAnchorY vh'.
+    double topOffset = m_regionAnchor.y() * lines / 100;
+    double top = m_viewportAnchor.y() - topOffset;
+    m_regionDisplayTree->setInlineStyleProperty(CSSPropertyTop, top, CSSUnitType::CSS_CQH);
+    m_cueContainer->setInlineStyleProperty(CSSPropertyTop, top, CSSUnitType::CSS_CQH);
 
     // 7.5 Every WebVTT region object is initialised with the following CSS
 
@@ -392,7 +386,7 @@ void VTTRegion::startTimer()
     if (m_scrollTimer.isActive())
         return;
 
-    Seconds duration = isScrollingRegion() ? scrollTime : 0_s;
+    Seconds duration = scroll() == ScrollSetting::Up ? scrollTime : 0_s;
     m_scrollTimer.startOneShot(duration);
 }
 
