@@ -301,6 +301,7 @@ public:
 
     static void didChangeCSSCanvasClientNodes(CanvasBase&);
     static void didCreateCanvasRenderingContext(CanvasRenderingContext&);
+    static void didChangeCanvasSize(CanvasRenderingContext&);
     static void didChangeCanvasMemory(CanvasRenderingContext&);
     static void didFinishRecordingCanvasFrame(CanvasRenderingContext&, bool forceDispatch = false);
 #if ENABLE(WEBGL)
@@ -513,6 +514,7 @@ private:
 
     static void didChangeCSSCanvasClientNodesImpl(InstrumentingAgents&, CanvasBase&);
     static void didCreateCanvasRenderingContextImpl(InstrumentingAgents&, CanvasRenderingContext&);
+    static void didChangeCanvasSizeImpl(InstrumentingAgents&, CanvasRenderingContext&);
     static void didChangeCanvasMemoryImpl(InstrumentingAgents&, CanvasRenderingContext&);
     static void didFinishRecordingCanvasFrameImpl(InstrumentingAgents&, CanvasRenderingContext&, bool forceDispatch = false);
 #if ENABLE(WEBGL)
@@ -621,8 +623,7 @@ inline void InspectorInstrumentation::didChangeRendererForDOMNode(Node& node)
 inline void InspectorInstrumentation::didAddOrRemoveScrollbars(LocalFrameView& frameView)
 {
     FAST_RETURN_IF_NO_FRONTENDS(void());
-    auto* localFrame = dynamicDowncast<LocalFrame>(frameView.frame());
-    if (auto* agents = localFrame ? instrumentingAgents(localFrame->document()) : nullptr)
+    if (auto* agents = instrumentingAgents(frameView.frame().document()))
         didAddOrRemoveScrollbarsImpl(*agents, frameView);
 }
 
@@ -1465,6 +1466,13 @@ inline void InspectorInstrumentation::didCreateCanvasRenderingContext(CanvasRend
         didCreateCanvasRenderingContextImpl(*agents, context);
 }
 
+inline void InspectorInstrumentation::didChangeCanvasSize(CanvasRenderingContext& context)
+{
+    FAST_RETURN_IF_NO_FRONTENDS(void());
+    if (auto* agents = instrumentingAgents(context.canvasBase().scriptExecutionContext()))
+        didChangeCanvasSizeImpl(*agents, context);
+}
+
 inline void InspectorInstrumentation::didChangeCanvasMemory(CanvasRenderingContext& context)
 {
     FAST_RETURN_IF_NO_FRONTENDS(void());
