@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2018-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,47 +31,23 @@
 #include "InlineItem.h"
 #include "InlineTextItem.h"
 #include "LayoutElementBox.h"
+#include "RenderStyleInlines.h"
 #include "TextUtil.h"
 
 namespace WebCore {
 namespace Layout {
 
-
-#if ASSERT_ENABLED
-static inline bool hasTrailingTextContent(const InlineContentBreaker::ContinuousContent& continuousContent)
-{
-    for (auto& run : makeReversedRange(continuousContent.runs())) {
-        auto& inlineItem = run.inlineItem;
-        if (inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd())
-            continue;
-        return inlineItem.isText();
-    }
-    return false;
-}
-#endif
-
 static inline bool hasLeadingTextContent(const InlineContentBreaker::ContinuousContent& continuousContent)
 {
     for (auto& run : continuousContent.runs()) {
         auto& inlineItem = run.inlineItem;
-        if (inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd())
+        if (inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd() || inlineItem.isOpaque())
             continue;
         return inlineItem.isText();
     }
     return false;
 }
 
-static inline bool hasTextRun(const InlineContentBreaker::ContinuousContent& continuousContent)
-{
-    // <span>text</span> is considered a text run even with the [inline box start][inline box end] inline items.
-    // Based on standards commit boundary rules it would be enough to check the first inline item.
-    for (auto& run : continuousContent.runs()) {
-        if (run.inlineItem.isText())
-            return true;
-    }
-    return false;
-}
-
 static inline std::optional<size_t> nextTextRunIndex(const InlineContentBreaker::ContinuousContent::RunList& runs, size_t startIndex)
 {
     for (auto index = startIndex + 1; index < runs.size(); ++index) {
@@ -89,7 +65,7 @@ static inline bool isWhitespaceOnlyContent(const InlineContentBreaker::Continuou
     auto hasWhitespace = false;
     for (auto& run : continuousContent.runs()) {
         auto& inlineItem = run.inlineItem;
-        if (inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd())
+        if (inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd() || inlineItem.isOpaque())
             continue;
         auto isWhitespace = inlineItem.isText() && downcast<InlineTextItem>(inlineItem).isWhitespace();
         if (!isWhitespace)
@@ -104,7 +80,7 @@ static inline bool isNonContentRunsOnly(const InlineContentBreaker::ContinuousCo
     // <span></span> <- non content runs.
     for (auto& run : continuousContent.runs()) {
         auto& inlineItem = run.inlineItem;
-        if (inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd())
+        if (inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd() || inlineItem.isOpaque())
             continue;
         return false;
     }
@@ -121,21 +97,15 @@ static inline std::optional<size_t> firstTextRunIndex(const InlineContentBreaker
     return { };
 }
 
-InlineContentBreaker::InlineContentBreaker(std::optional<IntrinsicWidthMode> intrinsicWidthMode)
-    : m_intrinsicWidthMode(intrinsicWidthMode)
-{
-}
-
 InlineContentBreaker::Result InlineContentBreaker::processInlineContent(const ContinuousContent& candidateContent, const LineStatus& lineStatus)
 {
     ASSERT(!std::isnan(lineStatus.availableWidth));
-    auto processCandidateContent = [&] {
-        if (candidateContent.logicalWidth() <= lineStatus.availableWidth)
-            return Result { Result::Action::Keep };
-        return processOverflowingContent(candidateContent, lineStatus);
-    };
+    ASSERT(isMinimumInIntrinsicWidthMode() || candidateContent.logicalWidth() > lineStatus.availableWidth);
 
-    auto result = processCandidateContent();
+    if (auto result = simplifiedMinimumInstrinsicWidthBreak(candidateContent, lineStatus))
+        return *result;
+
+    auto result = processOverflowingContent(candidateContent, lineStatus);
     if (result.action == Result::Action::Wrap && lineStatus.trailingSoftHyphenWidth && hasLeadingTextContent(candidateContent)) {
         // A trailing soft hyphen with a wrapped text content turns into a visible hyphen.
         // Let's check if there's enough space for the hyphen character.
@@ -146,20 +116,20 @@ InlineContentBreaker::Result InlineContentBreaker::processInlineContent(const Co
     return result;
 }
 
-InlineContentBreaker::Result InlineContentBreaker::processOverflowingContent(const ContinuousContent& overflowContent, const LineStatus& lineStatus) const
+InlineContentBreaker::Result InlineContentBreaker::processOverflowingContent(const ContinuousContent& continuousContent, const LineStatus& lineStatus) const
 {
-    auto continuousContent = ContinuousContent { overflowContent };
     ASSERT(!continuousContent.runs().isEmpty());
 
     ASSERT(continuousContent.logicalWidth() > lineStatus.availableWidth);
     auto checkForTrailingContentFit = [&]() -> std::optional<InlineContentBreaker::Result> {
-        if (continuousContent.hasTrimmableContent()) {
+        if (continuousContent.isFullyTrimmable()) {
+            // fully trimmable content stays on the current line (and gets fully trimmed).
+            return InlineContentBreaker::Result { Result::Action::Keep };
+        }
+        if (continuousContent.hasTrimmableSpace()) {
             // Check if the content fits if we trimmed it.
-            if (continuousContent.isFullyTrimmable() || isWhitespaceOnlyContent(continuousContent)) {
-                // If this new content is fully trimmable (including when it is enclosed by an inline box with overflowing decoration)
-                // it should not be wrapped to the next line (as it either fits/or gets fully trimmed).
+            if (isWhitespaceOnlyContent(continuousContent))
                 return InlineContentBreaker::Result { Result::Action::Keep };
-            }
             auto spaceRequired = continuousContent.logicalWidth() - continuousContent.trailingTrimmableWidth();
             if (lineStatus.hasFullyTrimmableTrailingContent)
                 spaceRequired -= continuousContent.leadingTrimmableWidth();
@@ -167,9 +137,9 @@ InlineContentBreaker::Result InlineContentBreaker::processOverflowingContent(con
                 return InlineContentBreaker::Result { Result::Action::Keep };
         }
 
-        if (continuousContent.hasHangingContent()) {
-            if (continuousContent.isHangingContent())
-                return InlineContentBreaker::Result { Result::Action::Keep };
+        if (continuousContent.isHangingContent())
+            return InlineContentBreaker::Result { Result::Action::Keep };
+        if (continuousContent.hasHangingSpace()) {
             auto spaceRequired = continuousContent.logicalWidth() - continuousContent.hangingContentWidth();
             if (spaceRequired <= lineStatus.availableWidth)
                 return InlineContentBreaker::Result { Result::Action::Keep };
@@ -189,7 +159,7 @@ InlineContentBreaker::Result InlineContentBreaker::processOverflowingContent(con
         return *result;
 
     size_t overflowingRunIndex = 0;
-    if (hasTextRun(continuousContent)) {
+    if (continuousContent.hasTextContent()) {
         auto tryBreakingContentWithText = [&]() -> std::optional<Result> {
             // 1. This text content is not breakable.
             // 2. This breakable text content does not fit at all. Not even the first glyph. This is a very special case.
@@ -221,28 +191,29 @@ InlineContentBreaker::Result InlineContentBreaker::processOverflowingContent(con
                         auto& runs = continuousContent.runs();
                         if (leadingTextRunIndex == runs.size() - 1)
                             return { };
-                        if (!runs[leadingTextRunIndex + 1].inlineItem.isInlineBoxEnd())
-                            return leadingTextRunIndex;
                         for (auto runIndex = leadingTextRunIndex + 1; runIndex < runs.size(); ++runIndex) {
-                            if (!runs[runIndex].inlineItem.isInlineBoxEnd())
+                            auto& inlineItem = runs[runIndex].inlineItem;
+                            if (inlineItem.isOpaque())
+                                continue;
+                            if (!inlineItem.isInlineBoxEnd())
                                 return runIndex - 1;
                         }
                         return { };
                     };
                     if (auto runToBreakAfter = trailingRunIndex())
-                        return Result { Result::Action::Break, IsEndOfLine::Yes, Result::PartialTrailingContent { *runToBreakAfter, { } } };
+                        return Result { Result::Action::Break, IsEndOfLine::Yes, Result::PartialTrailingContent { *runToBreakAfter, { }, { } } };
                     return Result { Result::Action::Keep, IsEndOfLine::Yes };
                 }
 
                 auto firstCharacterWidth = TextUtil::width(inlineTextItem, leadingTextRun.style.fontCascade(), inlineTextItem.start(), inlineTextItem.start() + firstCharacterLength, lineStatus.contentLogicalRight);
-                return Result { Result::Action::Break, IsEndOfLine::Yes, Result::PartialTrailingContent { leadingTextRunIndex, PartialRun { firstCharacterLength, firstCharacterWidth } } };
+                return Result { Result::Action::Break, IsEndOfLine::Yes, Result::PartialTrailingContent { leadingTextRunIndex, PartialRun { firstCharacterLength, firstCharacterWidth }, { } } };
             }
             if (trailingContent->overflows && lineStatus.hasContent) {
                 // We managed to break a run with overflow but the line already has content. Let's wrap it to the next line.
                 return Result { Result::Action::Wrap, IsEndOfLine::Yes };
             }
             // Either we managed to break with no overflow or the line is empty.
-            auto trailingPartialContent = Result::PartialTrailingContent { overflowingContent.breakingPosition->runIndex, trailingContent->partialRun };
+            auto trailingPartialContent = Result::PartialTrailingContent { overflowingContent.breakingPosition->runIndex, trailingContent->partialRun, trailingContent->hyphenWidth };
             return Result { Result::Action::Break, IsEndOfLine::Yes, trailingPartialContent };
         };
         if (auto result = tryBreakingContentWithText())
@@ -277,7 +248,36 @@ InlineContentBreaker::Result InlineContentBreaker::processOverflowingContent(con
     return { Result::Action::Keep, IsEndOfLine::No };
 }
 
-static std::optional<size_t> findTrailingRunIndex(const InlineContentBreaker::ContinuousContent::RunList& runs, size_t breakableRunIndex)
+std::optional<InlineContentBreaker::Result> InlineContentBreaker::simplifiedMinimumInstrinsicWidthBreak(const ContinuousContent& candidateContent, const LineStatus& lineStatus) const
+{
+    if (!isMinimumInIntrinsicWidthMode() || !candidateContent.isTextOnlyContent())
+        return { };
+
+    auto& leadingInlineTextItem = downcast<InlineTextItem>(candidateContent.runs().first().inlineItem);
+    auto& style = leadingInlineTextItem.style();
+    if (!TextUtil::isWrappingAllowed(style))
+        return Result { Result::Action::Keep, IsEndOfLine::No };
+
+    if (!lineStatus.hasContent) {
+        if (leadingInlineTextItem.isEmpty())
+            return Result { Result::Action::Keep, IsEndOfLine::No };
+        auto breakBehavior = wordBreakBehavior(style, { });
+        if (breakBehavior.isEmpty())
+            return Result { Result::Action::Keep, IsEndOfLine::No };
+
+        if (breakBehavior.containsAny({ WordBreakRule::AtArbitraryPositionWithinWords, WordBreakRule::AtArbitraryPosition })) {
+            auto firstCharacterLength = TextUtil::firstUserPerceivedCharacterLength(leadingInlineTextItem);
+            if (leadingInlineTextItem.length() <= firstCharacterLength)
+                return Result { Result::Action::Keep, IsEndOfLine::Yes };
+            auto firstCharacterWidth = TextUtil::width(leadingInlineTextItem, style.fontCascade(), leadingInlineTextItem.start(), leadingInlineTextItem.start() + firstCharacterLength, { }, TextUtil::UseTrailingWhitespaceMeasuringOptimization::No);
+            return Result { Result::Action::Break, IsEndOfLine::Yes, Result::PartialTrailingContent { { }, PartialRun { firstCharacterLength, firstCharacterWidth }, { } } };
+        }
+        return { };
+    }
+    return Result { !lineStatus.trailingSoftHyphenWidth ? Result::Action::Wrap : Result::Action::RevertToLastNonOverflowingWrapOpportunity, IsEndOfLine::Yes };
+}
+
+static std::optional<size_t> findTrailingRunIndexBeforeBreakableRun(const InlineContentBreaker::ContinuousContent::RunList& runs, size_t breakableRunIndex)
 {
     // When the breaking position is at the beginning of the run, the trailing run is the previous one.
     if (!breakableRunIndex)
@@ -285,17 +285,23 @@ static std::optional<size_t> findTrailingRunIndex(const InlineContentBreaker::Co
     // Try not break content at inline box boundary
     // e.g. <span>fits</span><span>overflows</span>
     // when the text "overflows" completely overflows, let's break the content right before the '<span>'.
+    auto lastOpaqueItemIndex = std::optional<size_t> { };
     for (auto trailingCandidateIndex = breakableRunIndex; trailingCandidateIndex--;) {
-        auto isAtInlineBox = runs[trailingCandidateIndex].inlineItem.isInlineBoxStart();
+        auto& inlineItem = runs[trailingCandidateIndex].inlineItem;
+        if (inlineItem.isOpaque()) {
+            lastOpaqueItemIndex = trailingCandidateIndex;
+            continue;
+        }
+        auto isAtInlineBox = inlineItem.isInlineBoxStart();
         if (!isAtInlineBox)
-            return trailingCandidateIndex;
+            return lastOpaqueItemIndex.value_or(trailingCandidateIndex);
+        lastOpaqueItemIndex = { };
     }
     return { };
 }
 
-static bool isWrappableRun(const InlineContentBreaker::ContinuousContent::Run& run)
+static bool isBreakableRun(const InlineContentBreaker::ContinuousContent::Run& run)
 {
-    ASSERT(run.inlineItem.isText() || run.inlineItem.isInlineBoxStart() || run.inlineItem.isInlineBoxEnd() || run.inlineItem.layoutBox().isImage() || run.inlineItem.layoutBox().isListMarkerBox());
     if (!run.inlineItem.isText()) {
         // Can't break horizontal spacing -> e.g. <span style="padding-right: 100px;">textcontent</span>, if the [inline box end] is the overflown inline item
         // we need to check if there's another inline item beyond the [inline box end] to split.
@@ -305,7 +311,7 @@ static bool isWrappableRun(const InlineContentBreaker::ContinuousContent::Run& r
     return TextUtil::isWrappingAllowed(run.style);
 }
 
-static inline bool canBreakBefore(UChar32 character, LineBreak lineBreak)
+static inline bool canBreakBefore(char32_t character, LineBreak lineBreak)
 {
     // FIXME: This should include all the cases from https://unicode.org/reports/tr14
     // Use a breaking matrix similar to lineBreakTable in BreakLines.cpp
@@ -331,12 +337,12 @@ static inline std::optional<size_t> lastValidBreakingPosition(const InlineConten
     ASSERT(inlineTextItem.length());
     auto lineBreak = textRun.style.lineBreak();
 
-    auto adjactentTextRunIndex = nextTextRunIndex(runs, textRunIndex);
-    if (!adjactentTextRunIndex)
+    auto adjacentTextRunIndex = nextTextRunIndex(runs, textRunIndex);
+    if (!adjacentTextRunIndex)
         return inlineTextItem.end();
 
-    auto& nextInlineTextItem = downcast<InlineTextItem>(runs[*adjactentTextRunIndex].inlineItem);
-    auto canBreakAtRunBoundary = nextInlineTextItem.isWhitespace() ? nextInlineTextItem.style().whiteSpace() != WhiteSpace::BreakSpaces :
+    auto& nextInlineTextItem = downcast<InlineTextItem>(runs[*adjacentTextRunIndex].inlineItem);
+    auto canBreakAtRunBoundary = nextInlineTextItem.isWhitespace() ? nextInlineTextItem.style().whiteSpaceCollapse() != WhiteSpaceCollapse::BreakSpaces :
         canBreakBefore(nextInlineTextItem.inlineTextBox().content()[nextInlineTextItem.start()], lineBreak);
     if (canBreakAtRunBoundary)
         return inlineTextItem.end();
@@ -381,6 +387,27 @@ static std::optional<TextUtil::WordBreakLeft> midWordBreak(const InlineContentBr
     return TextUtil::WordBreakLeft { right - left, TextUtil::width(inlineTextItem, textRun.style.fontCascade(), left, right, runLogicalLeft) };
 }
 
+static std::optional<size_t> hyphenPosition(StringView content, std::optional<size_t> overflowingPosition, const RenderStyle& style)
+{
+    // Find the hyphen position as follows:
+    // 1. Split the text by taking the hyphen width into account
+    // 2. Find the last hyphen position before the split position
+    auto contentLength = content.length();
+    size_t limitBefore = style.hyphenationLimitBefore() == RenderStyle::initialHyphenationLimitBefore() ? 0 : style.hyphenationLimitBefore();
+    if (overflowingPosition && *overflowingPosition < limitBefore)
+        return { };
+
+    size_t limitAfter = style.hyphenationLimitAfter() == RenderStyle::initialHyphenationLimitAfter() ? 0 : style.hyphenationLimitAfter();
+    if (limitBefore >= contentLength || limitAfter >= contentLength || limitBefore + limitAfter > contentLength)
+        return { };
+    // Adjust before index to accommodate the limit-after value (it's the last potential hyphen location in this run).
+    auto hyphenBefore = std::min(overflowingPosition.value_or(contentLength), contentLength - limitAfter) + 1;
+    size_t hyphenIndex = lastHyphenLocation(content, hyphenBefore, style.computedLocale());
+    if (!hyphenIndex || hyphenIndex < limitBefore)
+        return { };
+    return hyphenIndex;
+}
+
 struct CandidateTextRunForBreaking {
     size_t index { 0 };
     bool isOverflowingRun { true };
@@ -465,39 +492,24 @@ std::optional<InlineContentBreaker::PartialRun> InlineContentBreaker::tryBreakin
 
     if (breakRules.contains(WordBreakRule::AtHyphenationOpportunities)) {
         auto tryBreakingAtHyphenationOpportunity = [&]() -> std::optional<PartialRun> {
-            // Find the hyphen position as follows:
-            // 1. Split the text by taking the hyphen width into account
-            // 2. Find the last hyphen position before the split position
             if (candidateTextRun.isOverflowingRun && !lineHasRoomForContent) {
                 // We won't be able to find hyphen location when there's no available space.
                 return { };
             }
-            auto runLength = inlineTextItem.length();
-            unsigned limitBefore = style.hyphenationLimitBefore() == RenderStyle::initialHyphenationLimitBefore() ? 0 : style.hyphenationLimitBefore();
-            unsigned limitAfter = style.hyphenationLimitAfter() == RenderStyle::initialHyphenationLimitAfter() ? 0 : style.hyphenationLimitAfter();
-            // Check if this run can accommodate the before/after limits at all before start measuring text.
-            if (limitBefore >= runLength || limitAfter >= runLength || limitBefore + limitAfter > runLength)
-                return { };
-
-            unsigned leftSideLength = runLength;
             auto hyphenWidth = InlineLayoutUnit { fontCascade.width(TextRun { StringView { style.hyphenString() } }) };
+            auto leftSideLength = std::optional<size_t> { };
             if (candidateTextRun.isOverflowingRun) {
                 auto availableWidthExcludingHyphen = availableWidth - hyphenWidth;
-                if (availableWidthExcludingHyphen <= 0 || !enoughWidthForHyphenation(availableWidthExcludingHyphen, fontCascade.pixelSize()))
+                if (availableWidthExcludingHyphen <= 0 || !enoughWidthForHyphenation(availableWidthExcludingHyphen, fontCascade.size()))
                     return { };
                 leftSideLength = TextUtil::breakWord(inlineTextItem, fontCascade, candidateRun.logicalWidth, availableWidthExcludingHyphen, candidateTextRun.logicalLeft).length;
             }
-            if (leftSideLength < limitBefore)
-                return { };
-            // Adjust before index to accommodate the limit-after value (it's the last potential hyphen location in this run).
-            auto hyphenBefore = std::min(leftSideLength, runLength - limitAfter) + 1;
-            unsigned hyphenLocation = lastHyphenLocation(StringView(inlineTextItem.inlineTextBox().content()).substring(inlineTextItem.start(), inlineTextItem.length()), hyphenBefore, style.computedLocale());
-            if (!hyphenLocation || hyphenLocation < limitBefore)
-                return { };
-            // hyphenLocation is relative to the start of this InlineItemText.
-            ASSERT(inlineTextItem.start() + hyphenLocation < inlineTextItem.end());
-            auto trailingPartialRunWidthWithHyphen = TextUtil::width(inlineTextItem, fontCascade, inlineTextItem.start(), inlineTextItem.start() + hyphenLocation, candidateTextRun.logicalLeft);
-            return PartialRun { hyphenLocation, trailingPartialRunWidthWithHyphen, hyphenWidth };
+            if (auto hyphenLocation = hyphenPosition(inlineTextItem.inlineTextBox().content().substring(inlineTextItem.start(), inlineTextItem.length()), leftSideLength, style)) {
+                ASSERT(inlineTextItem.start() + *hyphenLocation < inlineTextItem.end());
+                auto trailingPartialRunWidthWithHyphen = TextUtil::width(inlineTextItem, fontCascade, inlineTextItem.start(), inlineTextItem.start() + *hyphenLocation, candidateTextRun.logicalLeft);
+                return PartialRun { *hyphenLocation, trailingPartialRunWidthWithHyphen, hyphenWidth };
+            }
+            return { };
         };
         if (auto partialRun = tryBreakingAtHyphenationOpportunity())
             return partialRun;
@@ -543,7 +555,8 @@ std::optional<InlineContentBreaker::PartialRun> InlineContentBreaker::tryBreakin
 std::optional<InlineContentBreaker::OverflowingTextContent::BreakingPosition> InlineContentBreaker::tryBreakingOverflowingRun(const LineStatus& lineStatus, const ContinuousContent::RunList& runs, size_t overflowingRunIndex, InlineLayoutUnit nonOverflowingContentWidth) const
 {
     auto overflowingRun = runs[overflowingRunIndex];
-    if (!isWrappableRun(overflowingRun))
+    ASSERT(!overflowingRun.inlineItem.isOpaque());
+    if (!isBreakableRun(overflowingRun))
         return { };
 
     auto availableWidth = std::max(0.f, lineStatus.availableWidth - nonOverflowingContentWidth);
@@ -553,7 +566,7 @@ std::optional<InlineContentBreaker::OverflowingTextContent::BreakingPosition> In
     if (partialOverflowingRun->length)
         return OverflowingTextContent::BreakingPosition { overflowingRunIndex, OverflowingTextContent::BreakingPosition::TrailingContent { false, partialOverflowingRun } };
     // When the breaking position is at the beginning of the run, the trailing run is the previous one.
-    if (auto trailingRunIndex = findTrailingRunIndex(runs, overflowingRunIndex))
+    if (auto trailingRunIndex = findTrailingRunIndexBeforeBreakableRun(runs, overflowingRunIndex))
         return OverflowingTextContent::BreakingPosition { *trailingRunIndex, OverflowingTextContent::BreakingPosition::TrailingContent { } };
     // Sometimes we can't accommodate even the very first character.
     // Note that this is different from when there's no breakable run in this set.
@@ -566,7 +579,7 @@ std::optional<InlineContentBreaker::OverflowingTextContent::BreakingPosition> In
     for (auto index = overflowingRunIndex; index--;) {
         auto& run = runs[index];
         previousContentWidth -= run.logicalWidth;
-        if (!isWrappableRun(run))
+        if (!isBreakableRun(run))
             continue;
         ASSERT(run.inlineItem.isText());
         auto availableWidth = std::max(0.f, lineStatus.availableWidth - previousContentWidth);
@@ -592,7 +605,7 @@ std::optional<InlineContentBreaker::OverflowingTextContent::BreakingPosition> In
                     ASSERT(!trailingInlineBoxEndIndex || *trailingInlineBoxEndIndex <= overflowingRunIndex);
                     return trailingInlineBoxEndIndex.value_or(index);
                 };
-                return OverflowingTextContent::BreakingPosition { trailingRunIndex(), OverflowingTextContent::BreakingPosition::TrailingContent { false, std::nullopt } };
+                return OverflowingTextContent::BreakingPosition { trailingRunIndex(), OverflowingTextContent::BreakingPosition::TrailingContent { false } };
             }
             return OverflowingTextContent::BreakingPosition { index, OverflowingTextContent::BreakingPosition::TrailingContent { false, partialRun } };
         }
@@ -605,7 +618,7 @@ std::optional<InlineContentBreaker::OverflowingTextContent::BreakingPosition> In
     auto nextContentWidth = nonOverflowingContentWidth + runs[overflowingRunIndex].logicalWidth;
     for (auto index = overflowingRunIndex + 1; index < runs.size(); ++index) {
         auto& run = runs[index];
-        if (!isWrappableRun(run)) {
+        if (!isBreakableRun(run)) {
             nextContentWidth += run.logicalWidth;
             continue;
         }
@@ -618,9 +631,11 @@ std::optional<InlineContentBreaker::OverflowingTextContent::BreakingPosition> In
                 // We managed to break this text run mid content. It has to be either an arbitrary mid-word or a hyphen break.
                 return OverflowingTextContent::BreakingPosition { index, OverflowingTextContent::BreakingPosition::TrailingContent { true, partialRun } };
             }
-            if (auto trailingRunIndex = findTrailingRunIndex(runs, index)) {
-                // At worst we are back to the overflowing run, like in the example above.
-                ASSERT(*trailingRunIndex >= overflowingRunIndex);
+            if (auto trailingRunIndex = findTrailingRunIndexBeforeBreakableRun(runs, index)) {
+                // We may end up _before_ the overflowing run e.g.
+                // <span></span><span style="border: 10px solid">breakable</span>
+                // with 0px constraint, where while the second (non-empty) [inline box start] overflows, the trailing
+                // run ends up being the first [inline box end] inline item.
                 return OverflowingTextContent::BreakingPosition { *trailingRunIndex, OverflowingTextContent::BreakingPosition::TrailingContent { true } };
             }
             // This happens when the overflowing run is also the first run in this set, no trailing run.
@@ -631,6 +646,80 @@ std::optional<InlineContentBreaker::OverflowingTextContent::BreakingPosition> In
     return { };
 }
 
+std::optional<InlineContentBreaker::OverflowingTextContent::BreakingPosition> InlineContentBreaker::tryHyphenationAcrossOverflowingInlineTextItems(const LineStatus& lineStatus, const ContinuousContent::RunList& runs, size_t overflowingRunIndex) const
+{
+    if (runs.size() == 1)
+        return { };
+
+    auto& style = runs.first().inlineItem.style();
+    if (!wordBreakBehavior(style, lineStatus.hasWrapOpportunityAtPreviousPosition).contains(WordBreakRule::AtHyphenationOpportunities))
+        return { };
+
+    // 1. concatenate adjacent text content
+    // 2. find the last hyphen location before the overflowing position
+    // 3. find the inline text item where the hyphen location is and compute the partial run width
+    auto content = StringBuilder { };
+    size_t overflowingRunStartPosition = 0;
+    for (size_t index = 0; index < runs.size(); ++index) {
+        auto& inlineItem = runs[index].inlineItem;
+        // FIXME: Maybe content across inline boxes should be hyphenated as well.
+        if (inlineItem.isOpaque())
+            continue;
+        if (!inlineItem.isText() || inlineItem.style().fontCascade() != style.fontCascade())
+            return { };
+
+        auto& inlineTextItem = downcast<InlineTextItem>(inlineItem);
+        if (inlineTextItem.isWhitespace())
+            return { };
+        content.append(inlineTextItem.inlineTextBox().content().substring(inlineTextItem.start(), inlineTextItem.length()));
+        overflowingRunStartPosition += index < overflowingRunIndex ? inlineTextItem.length() : 0;
+    }
+    // Only non-whitespace text runs with same style.
+    auto& fontCascade = style.fontCascade();
+    auto hyphenWidth = InlineLayoutUnit { fontCascade.width(TextRun { StringView { style.hyphenString() } }) };
+    auto availableWidthExcludingHyphen = lineStatus.availableWidth - hyphenWidth;
+    if (availableWidthExcludingHyphen <= 0 || !enoughWidthForHyphenation(availableWidthExcludingHyphen, fontCascade.size()))
+        return { };
+
+    auto& overflowingRun = runs[overflowingRunIndex];
+    if (!is<InlineTextItem>(overflowingRun.inlineItem)) {
+        ASSERT_NOT_REACHED();
+        return { };
+    }
+    // Make sure we always hyphenate before the overflow.
+    auto overflowPositionWithHyphen = TextUtil::breakWord(downcast<InlineTextItem>(overflowingRun.inlineItem), fontCascade, overflowingRun.logicalWidth, availableWidthExcludingHyphen, lineStatus.contentLogicalRight).length;
+    auto hyphenLocation = hyphenPosition(content, overflowingRunStartPosition + overflowPositionWithHyphen, style);
+    if (!hyphenLocation)
+        return { };
+
+    // hyphenLocation must be in or before the overflowing run.
+    ASSERT(*hyphenLocation <= overflowingRunStartPosition + overflowPositionWithHyphen);
+    auto hyphenLocationWithinInlineTextItem = *hyphenLocation;
+    size_t hyphenatedRunIndex = 0;
+    for (; hyphenatedRunIndex <= overflowingRunIndex; ++hyphenatedRunIndex) {
+        auto& inlineItem = runs[hyphenatedRunIndex].inlineItem;
+        if (inlineItem.isOpaque())
+            continue;
+        auto& inlineTextItem = downcast<InlineTextItem>(inlineItem);
+        if (inlineTextItem.length() >= hyphenLocationWithinInlineTextItem)
+            break;
+        hyphenLocationWithinInlineTextItem -= inlineTextItem.length();
+    }
+    auto& hyphenatedlineTextItem = downcast<InlineTextItem>(runs[hyphenatedRunIndex].inlineItem);
+    if (hyphenLocationWithinInlineTextItem > hyphenatedlineTextItem.length()) {
+        ASSERT_NOT_REACHED();
+        return { };
+    }
+
+    // Hyphen may be right at the run (end) boundary.
+    auto partialRun = std::optional<InlineContentBreaker::PartialRun> { };
+    if (hyphenLocationWithinInlineTextItem < hyphenatedlineTextItem.length()) {
+        auto trailingPartialRunWidthWithHyphen = TextUtil::width(hyphenatedlineTextItem, fontCascade, hyphenatedlineTextItem.start(), hyphenatedlineTextItem.start() + hyphenLocationWithinInlineTextItem, lineStatus.contentLogicalRight);
+        partialRun = { hyphenLocationWithinInlineTextItem, trailingPartialRunWidthWithHyphen, hyphenWidth };
+    }
+    return OverflowingTextContent::BreakingPosition { hyphenatedRunIndex, OverflowingTextContent::BreakingPosition::TrailingContent { false, partialRun, hyphenWidth } };
+}
+
 InlineContentBreaker::OverflowingTextContent InlineContentBreaker::processOverflowingContentWithText(const ContinuousContent& continuousContent, const LineStatus& lineStatus) const
 {
     auto& runs = continuousContent.runs();
@@ -643,7 +732,10 @@ InlineContentBreaker::OverflowingTextContent InlineContentBreaker::processOverfl
     auto nonOverflowingContentWidth = InlineLayoutUnit { };
     auto overflowingRunIndex = runs.size(); 
     for (size_t index = 0; index < runs.size(); ++index) {
-        auto runLogicalWidth = runs[index].logicalWidth;
+        auto& run = runs[index];
+        if (run.inlineItem.isOpaque())
+            continue;
+        auto runLogicalWidth = run.logicalWidth;
         if (nonOverflowingContentWidth + runLogicalWidth > lineStatus.availableWidth) {
             overflowingRunIndex = index;
             break;
@@ -676,6 +768,9 @@ InlineContentBreaker::OverflowingTextContent InlineContentBreaker::processOverfl
             return { overflowingRunIndex, breakingPosition };
     }
 
+    if (auto breakingPosition = tryHyphenationAcrossOverflowingInlineTextItems(lineStatus, runs, overflowingRunIndex))
+        return { overflowingRunIndex, breakingPosition };
+
     // At this point we know that there's no breakable run all the way to the overflowing run.
     // Now we need to check if any run after the overflowing content can break.
     // e.g. <span>this_content_overflows_but_not_breakable<span><span style="word-break: break-all">but_this_is_breakable</span>
@@ -717,7 +812,7 @@ OptionSet<InlineContentBreaker::WordBreakRule> InlineContentBreaker::wordBreakBe
     // OverflowWrap::BreakWord/Anywhere An otherwise unbreakable sequence of characters may be broken at an arbitrary point if there are no otherwise-acceptable break points in the line.
     // Note that this applies to content where CSS properties (e.g. WordBreak::KeepAll) make it unbreakable. 
     // Soft wrap opportunities introduced by overflow-wrap/word-wrap: break-word are not considered when calculating min-content intrinsic sizes.
-    auto overflowWrapBreakWordIsApplicable = !isInIntrinsicWidthMode();
+    auto overflowWrapBreakWordIsApplicable = !isMinimumInIntrinsicWidthMode();
     if (((overflowWrapBreakWordIsApplicable && style.overflowWrap() == OverflowWrap::BreakWord) || style.overflowWrap() == OverflowWrap::Anywhere) && !hasWrapOpportunityAtPreviousPosition)
         return includeHyphenationIfAllowed(WordBreakRule::AtArbitraryPosition);
     // Breaking is forbidden within “words”.
@@ -737,11 +832,13 @@ void InlineContentBreaker::ContinuousContent::resetTrailingTrimmableContent()
     if (!m_leadingTrimmableWidth)
         m_leadingTrimmableWidth = m_trailingTrimmableWidth;
     m_trailingTrimmableWidth = { };
+    m_isFullyTrimmable = false;
 }
 
 void InlineContentBreaker::ContinuousContent::append(const InlineItem& inlineItem, const RenderStyle& style, InlineLayoutUnit logicalWidth)
 {
-    ASSERT(inlineItem.isBox() || inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd());
+    ASSERT(inlineItem.isBox() || inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd() || inlineItem.isOpaque());
+    m_isTextOnlyContent = false;
     appendToRunList(inlineItem, style, logicalWidth);
     if (inlineItem.isBox()) {
         // Inline boxes (whitespace-> <span></span>) do not prevent the trailing content from getting trimmed/hung
@@ -750,14 +847,28 @@ void InlineContentBreaker::ContinuousContent::append(const InlineItem& inlineIte
     }
 }
 
-void InlineContentBreaker::ContinuousContent::appendTextContent(const InlineTextItem& inlineTextItem, const RenderStyle& style, InlineLayoutUnit logicalWidth, std::optional<InlineLayoutUnit> trimmableWidth)
+void InlineContentBreaker::ContinuousContent::appendTextContent(const InlineTextItem& inlineTextItem, const RenderStyle& style, InlineLayoutUnit logicalWidth)
 {
+    m_hasTextContent = true;
+    // https://www.w3.org/TR/css-text-4/#white-space-phase-2
+    auto isTrailingHangingContent = inlineTextItem.isWhitespace() && TextUtil::shouldTrailingWhitespaceHang(style);
+    if (isTrailingHangingContent)
+        setHangingContentWidth(logicalWidth);
+
+    auto trimmableWidth = [&]() -> std::optional<InlineLayoutUnit> {
+        if (isTrailingHangingContent)
+            return { };
+        if (inlineTextItem.isFullyTrimmable() || inlineTextItem.isQuirkNonBreakingSpace())
+            return logicalWidth;
+        return { };
+    }();
     if (!trimmableWidth) {
         appendToRunList(inlineTextItem, style, logicalWidth);
         resetTrailingTrimmableContent();
         return;
     }
 
+    m_isFullyTrimmable = m_isFullyTrimmable || m_runs.isEmpty();
     ASSERT(*trimmableWidth <= logicalWidth);
     auto isLeadingTrimmable = trimmableWidth && (!this->logicalWidth() || isFullyTrimmable());
     appendToRunList(inlineTextItem, style, logicalWidth);
@@ -775,7 +886,11 @@ void InlineContentBreaker::ContinuousContent::reset()
     m_leadingTrimmableWidth = { };
     m_trailingTrimmableWidth = { };
     m_hangingContentWidth = { };
+    m_minimumRequiredWidth = { };
     m_runs.clear();
+    m_hasTextContent = false;
+    m_isTextOnlyContent = true;
+    m_isFullyTrimmable = false;
 }
 
 }
