@@ -43,7 +43,9 @@ struct WhitespaceContent {
     size_t length { 0 };
     bool isWordSeparator { true };
 };
-static std::optional<WhitespaceContent> moveToNextNonWhitespacePosition(StringView textContent, size_t startPosition, bool preserveNewline, bool preserveTab, bool stopAtWordSeparatorBoundary)
+
+template<typename CharacterType>
+static std::optional<WhitespaceContent> moveToNextNonWhitespacePosition(std::span<const CharacterType> characters, size_t startPosition, bool preserveNewline, bool preserveTab, bool stopAtWordSeparatorBoundary)
 {
     auto hasWordSeparatorCharacter = false;
     auto isWordSeparatorCharacter = false;
@@ -55,7 +57,8 @@ static std::optional<WhitespaceContent> moveToNextNonWhitespacePosition(StringVi
         return isTreatedAsSpaceCharacter || character == tabCharacter;
     };
     auto nextNonWhiteSpacePosition = startPosition;
-    while (nextNonWhiteSpacePosition < textContent.length() && isWhitespaceCharacter(textContent[nextNonWhiteSpacePosition])) {
+    auto* rawCharacters = characters.data(); // Not using characters[nextNonWhiteSpacePosition] to bypass the bounds check.
+    while (nextNonWhiteSpacePosition < characters.size() && isWhitespaceCharacter(rawCharacters[nextNonWhiteSpacePosition])) {
         if (UNLIKELY(stopAtWordSeparatorBoundary && hasWordSeparatorCharacter && !isWordSeparatorCharacter))
             break;
         ++nextNonWhiteSpacePosition;
@@ -77,46 +80,45 @@ static unsigned moveToNextBreakablePosition(unsigned startPosition, CachedLineBr
     return textLength - startPosition;
 }
 
-InlineItemsBuilder::InlineItemsBuilder(const ElementBox& formattingContextRoot, InlineFormattingState& formattingState)
-    : m_root(formattingContextRoot)
-    , m_formattingState(formattingState)
+InlineItemsBuilder::InlineItemsBuilder(InlineContentCache& inlineContentCache, const ElementBox& root)
+    : m_inlineContentCache(inlineContentCache)
+    , m_root(root)
 {
 }
 
 void InlineItemsBuilder::build(InlineItemPosition startPosition)
 {
-    InlineItems inlineItems;
-    FormattingState::OutOfFlowBoxList outOfFlowBoxes;
-
-    collectInlineItems(inlineItems, outOfFlowBoxes, startPosition);
+    InlineItemList inlineItemList;
+    collectInlineItems(inlineItemList, startPosition);
 
     if (!root().style().isLeftToRightDirection() || contentRequiresVisualReordering()) {
         // FIXME: Add support for partial, yet paragraph level bidi content handling.
-        breakAndComputeBidiLevels(inlineItems);
+        breakAndComputeBidiLevels(inlineItemList);
     }
-    computeInlineTextItemWidths(inlineItems);
+    computeInlineTextItemWidths(inlineItemList);
 
-    auto adjustInlineFormattingStateWithNewInlineItems = [&] {
+    auto adjustInlineContentCacheWithNewInlineItems = [&] {
+        auto& inlineItemCache = inlineContentCache().inlineItems();
         if (!startPosition)
-            return m_formattingState.setInlineItems(WTFMove(inlineItems));
+            return inlineItemCache.set(WTFMove(inlineItemList));
         // Let's first remove the dirty inline items if there are any.
-        auto& currentInlineItems = m_formattingState.inlineItems();
+        auto& currentInlineItems = inlineItemCache.content();
         if (startPosition.index >= currentInlineItems.size()) {
             ASSERT_NOT_REACHED();
-            return m_formattingState.setInlineItems(WTFMove(inlineItems));
+            return inlineItemCache.set(WTFMove(inlineItemList));
         }
+        // FIXME: Use replace instead.
         currentInlineItems.remove(startPosition.index, currentInlineItems.size() - startPosition.index);
-        m_formattingState.appendInlineItems(WTFMove(inlineItems));
+        inlineItemCache.append(WTFMove(inlineItemList));
     };
-    adjustInlineFormattingStateWithNewInlineItems();
-    m_formattingState.setOutOfFlowBoxes(WTFMove(outOfFlowBoxes));
+    adjustInlineContentCacheWithNewInlineItems();
 
 #if ASSERT_ENABLED
     // Check if we've got matching inline box start/end pairs and unique inline level items (non-text, non-inline box items).
     size_t inlineBoxStart = 0;
     size_t inlineBoxEnd = 0;
     auto inlineLevelItems = HashSet<const Box*> { };
-    for (auto& inlineItem : m_formattingState.inlineItems()) {
+    for (auto& inlineItem : inlineContentCache().inlineItems().content()) {
         if (inlineItem.isInlineBoxStart())
             ++inlineBoxStart;
         else if (inlineItem.isInlineBoxEnd())
@@ -131,22 +133,39 @@ void InlineItemsBuilder::build(InlineItemPosition startPosition)
 #endif
 }
 
-using LayoutQueue = Vector<CheckedRef<const Box>>;
-
-static bool traverseUntilDamaged(LayoutQueue& layoutQueue, const Box& root, const Box& firstDamagedLayoutBox)
+static inline bool isTextOrLineBreak(const Box& layoutBox)
 {
-    if (&root == &firstDamagedLayoutBox)
+    return layoutBox.isInlineTextBox() || (layoutBox.isLineBreakBox() && !layoutBox.isWordBreakOpportunity());
+}
+
+static bool requiresVisualReordering(const Box& layoutBox)
+{
+    if (is<InlineTextBox>(layoutBox))
+        return TextUtil::containsStrongDirectionalityText(downcast<InlineTextBox>(layoutBox).content());
+    if (layoutBox.isInlineBox() && layoutBox.isInFlow()) {
+        auto& style = layoutBox.style();
+        return !style.isLeftToRightDirection() || (style.rtlOrdering() == Order::Logical && style.unicodeBidi() != UnicodeBidi::Normal);
+    }
+    return false;
+}
+
+bool InlineItemsBuilder::traverseUntilDamaged(LayoutQueue& layoutQueue, const Box& subtreeRoot, const Box& firstDamagedLayoutBox)
+{
+    if (&subtreeRoot == &firstDamagedLayoutBox)
         return true;
 
-    auto shouldSkipSubtree = root.establishesFormattingContext();
-    if (!shouldSkipSubtree && is<ElementBox>(root) && downcast<ElementBox>(root).hasChild()) {
-        auto& firstChild = *downcast<ElementBox>(root).firstChild();
+    m_contentRequiresVisualReordering = m_contentRequiresVisualReordering || requiresVisualReordering(subtreeRoot);
+    m_isNonBidiTextAndForcedLineBreakOnlyContent = m_isNonBidiTextAndForcedLineBreakOnlyContent && !m_contentRequiresVisualReordering && isTextOrLineBreak(subtreeRoot);
+
+    auto shouldSkipSubtree = subtreeRoot.establishesFormattingContext();
+    if (!shouldSkipSubtree && is<ElementBox>(subtreeRoot) && downcast<ElementBox>(subtreeRoot).hasChild()) {
+        auto& firstChild = *downcast<ElementBox>(subtreeRoot).firstChild();
         layoutQueue.append(firstChild);
         if (traverseUntilDamaged(layoutQueue, firstChild, firstDamagedLayoutBox))
             return true;
         layoutQueue.takeLast();
     }
-    if (auto* nextSibling = root.nextSibling()) {
+    if (auto* nextSibling = subtreeRoot.nextSibling()) {
         layoutQueue.takeLast();
         layoutQueue.append(*nextSibling);
         if (traverseUntilDamaged(layoutQueue, *nextSibling, firstDamagedLayoutBox))
@@ -155,16 +174,18 @@ static bool traverseUntilDamaged(LayoutQueue& layoutQueue, const Box& root, cons
     return false;
 }
 
-static LayoutQueue initializeLayoutQueue(const ElementBox& formattingContextRoot, InlineItemPosition startPosition, const InlineItems& currentInlineItems)
+InlineItemsBuilder::LayoutQueue InlineItemsBuilder::initializeLayoutQueue(InlineItemPosition startPosition)
 {
-    if (!formattingContextRoot.firstChild()) {
+    auto& root = this->root();
+    if (!root.firstChild()) {
         // There should always be at least one inflow child in this inline formatting context.
         ASSERT_NOT_REACHED();
         return { };
     }
 
     if (!startPosition)
-        return { *formattingContextRoot.firstChild() };
+        return { *root.firstChild() };
+
     // For partial layout we need to build the layout queue up to the point where the new content is in order
     // to be able to produce non-content type of trailing inline items.
     // e.g <div><span<span>text</span></span> produces
@@ -173,34 +194,36 @@ static LayoutQueue initializeLayoutQueue(const ElementBox& formattingContextRoot
     // <div><span><span>text more_text</span></span> should produce
     // [inline box start][inline box start][text][ ][more_text][inline box end][inline box end]
     // where we start processing the content at the new layout box and continue with whatever we have on the stack (layout queue).
-    if (startPosition.index >= currentInlineItems.size()) {
+    auto& existingInlineItems = inlineContentCache().inlineItems().content();
+    if (startPosition.index >= existingInlineItems.size()) {
         ASSERT_NOT_REACHED();
-        return { *formattingContextRoot.firstChild() };
+        return { *root.firstChild() };
     }
 
-    auto& firstDamagedLayoutBox = currentInlineItems[startPosition.index].layoutBox();
-    auto& firstChild = *formattingContextRoot.firstChild();
+    auto& firstDamagedLayoutBox = existingInlineItems[startPosition.index].layoutBox();
+    auto& firstChild = *root.firstChild();
     LayoutQueue layoutQueue;
     layoutQueue.append(firstChild);
     traverseUntilDamaged(layoutQueue, firstChild, firstDamagedLayoutBox);
 
     if (layoutQueue.isEmpty()) {
         ASSERT_NOT_REACHED();
-        layoutQueue.append(*formattingContextRoot.firstChild());
+        layoutQueue.append(*root.firstChild());
     }
     return layoutQueue;
 }
 
-void InlineItemsBuilder::collectInlineItems(InlineItems& inlineItems, FormattingState::OutOfFlowBoxList& outOfFlowBoxes, InlineItemPosition startPosition)
+void InlineItemsBuilder::collectInlineItems(InlineItemList& inlineItemList, InlineItemPosition startPosition)
 {
     // Traverse the tree and create inline items out of inline boxes and leaf nodes. This essentially turns the tree inline structure into a flat one.
     // <span>text<span></span><img></span> -> [InlineBoxStart][InlineLevelBox][InlineBoxStart][InlineBoxEnd][InlineLevelBox][InlineBoxEnd]
-    auto layoutQueue = initializeLayoutQueue(root(), startPosition, m_formattingState.inlineItems());
+    auto layoutQueue = initializeLayoutQueue(startPosition);
+    auto& inlineItemCache = this->inlineContentCache().inlineItems();
 
     auto partialContentOffset = [&](auto& inlineTextBox) -> std::optional<size_t> {
         if (!startPosition)
             return { };
-        auto& currentInlineItems = m_formattingState.inlineItems();
+        auto& currentInlineItems = inlineItemCache.content();
         if (startPosition.index >= currentInlineItems.size()) {
             ASSERT_NOT_REACHED();
             return { };
@@ -223,7 +246,7 @@ void InlineItemsBuilder::collectInlineItems(InlineItems& inlineItems, Formatting
             if (!isInlineBoxWithInlineContent)
                 break;
             // This is the start of an inline box (e.g. <span>).
-            handleInlineBoxStart(layoutBox, inlineItems);
+            handleInlineBoxStart(layoutBox, inlineItemList);
             auto& inlineBox = downcast<ElementBox>(layoutBox);
             if (!inlineBox.hasChild())
                 break;
@@ -233,19 +256,19 @@ void InlineItemsBuilder::collectInlineItems(InlineItems& inlineItems, Formatting
         while (!layoutQueue.isEmpty()) {
             auto layoutBox = layoutQueue.takeLast();
             if (layoutBox->isOutOfFlowPositioned()) {
-                // Let's not construct InlineItems for out-of-flow content as they don't participate in the inline layout.
-                // However to be able to static positioning them, we need to compute their approximate positions.
-                outOfFlowBoxes.append(layoutBox);
+                m_isNonBidiTextAndForcedLineBreakOnlyContent = false;
+                inlineItemList.append({ layoutBox, InlineItem::Type::Opaque });
             } else if (layoutBox->isInlineTextBox()) {
                 auto& inlineTextBox = downcast<InlineTextBox>(layoutBox);
-                handleTextContent(inlineTextBox, inlineItems, partialContentOffset(inlineTextBox));
+                handleTextContent(inlineTextBox, inlineItemList, partialContentOffset(inlineTextBox));
             } else if (layoutBox->isAtomicInlineLevelBox() || layoutBox->isLineBreakBox())
-                handleInlineLevelBox(layoutBox, inlineItems);
+                handleInlineLevelBox(layoutBox, inlineItemList);
             else if (layoutBox->isInlineBox())
-                handleInlineBoxEnd(layoutBox, inlineItems);
-            else if (layoutBox->isFloatingPositioned())
-                inlineItems.append({ layoutBox, InlineItem::Type::Float });
-            else
+                handleInlineBoxEnd(layoutBox, inlineItemList);
+            else if (layoutBox->isFloatingPositioned()) {
+                inlineItemList.append({ layoutBox, InlineItem::Type::Float });
+                m_isNonBidiTextAndForcedLineBreakOnlyContent = false;
+            } else
                 ASSERT_NOT_REACHED();
 
             if (auto* nextSibling = layoutBox->nextSibling()) {
@@ -254,6 +277,8 @@ void InlineItemsBuilder::collectInlineItems(InlineItems& inlineItems, Formatting
             }
         }
     }
+    inlineItemCache.setIsNonBidiTextAndForcedLineBreakOnlyContent(m_isNonBidiTextAndForcedLineBreakOnlyContent);
+    inlineItemCache.setRequiresVisualReordering(m_contentRequiresVisualReordering);
 }
 
 static void replaceNonPreservedNewLineAndTabCharactersAndAppend(const InlineTextBox& inlineTextBox, StringBuilder& paragraphContentBuilder)
@@ -271,7 +296,7 @@ static void replaceNonPreservedNewLineAndTabCharactersAndAppend(const InlineText
         auto startPosition = position;
         auto isNewLineOrTabCharacter = [&] {
             if (needsUnicodeHandling) {
-                UChar32 character;
+                char32_t character;
                 U16_NEXT(textContent.characters16(), position, contentLength, character);
                 return character == newlineCharacter || character == tabCharacter;
             }
@@ -347,7 +372,7 @@ static inline void handleEnterExitBidiContext(StringBuilder& paragraphContentBui
 }
 
 using InlineItemOffsetList = Vector<std::optional<size_t>>;
-static inline void buildBidiParagraph(const RenderStyle& rootStyle, const InlineItems& inlineItems,  StringBuilder& paragraphContentBuilder, InlineItemOffsetList& inlineItemOffsetList)
+static inline void buildBidiParagraph(const RenderStyle& rootStyle, const InlineItemList& inlineItemList,  StringBuilder& paragraphContentBuilder, InlineItemOffsetList& inlineItemOffsetList)
 {
     auto bidiContextStack = BidiContextStack { };
     handleEnterExitBidiContext(paragraphContentBuilder, rootStyle.unicodeBidi(), rootStyle.isLeftToRightDirection(), EnterExitType::EnteringBlock, bidiContextStack);
@@ -356,8 +381,8 @@ static inline void buildBidiParagraph(const RenderStyle& rootStyle, const Inline
 
     const Box* lastInlineTextBox = nullptr;
     size_t inlineTextBoxOffset = 0;
-    for (size_t index = 0; index < inlineItems.size(); ++index) {
-        auto& inlineItem = inlineItems[index];
+    for (size_t index = 0; index < inlineItemList.size(); ++index) {
+        auto& inlineItem = inlineItemList[index];
         auto& layoutBox = inlineItem.layoutBox();
 
         auto appendTextBasedContent = [&] {
@@ -371,9 +396,9 @@ static inline void buildBidiParagraph(const RenderStyle& rootStyle, const Inline
 
         if (inlineItem.isText()) {
             appendTextBasedContent();
-            inlineItemOffsetList.uncheckedAppend({ inlineTextBoxOffset + downcast<InlineTextItem>(inlineItem).start() });
+            inlineItemOffsetList.append({ inlineTextBoxOffset + downcast<InlineTextItem>(inlineItem).start() });
         } else if (inlineItem.isBox()) {
-            inlineItemOffsetList.uncheckedAppend({ paragraphContentBuilder.length() });
+            inlineItemOffsetList.append({ paragraphContentBuilder.length() });
             paragraphContentBuilder.append(objectReplacementCharacter);
         } else if (inlineItem.isInlineBoxStart() || inlineItem.isInlineBoxEnd()) {
             // https://drafts.csswg.org/css-writing-modes/#unicode-bidi
@@ -381,10 +406,10 @@ static inline void buildBidiParagraph(const RenderStyle& rootStyle, const Inline
             auto initiatesControlCharacter = style.rtlOrdering() == Order::Logical && style.unicodeBidi() != UnicodeBidi::Normal;
             if (!initiatesControlCharacter) {
                 // Opaque items do not have position in the bidi paragraph. They inherit their bidi level from the next inline item.
-                inlineItemOffsetList.uncheckedAppend({ });
+                inlineItemOffsetList.append({ });
                 continue;
             }
-            inlineItemOffsetList.uncheckedAppend({ paragraphContentBuilder.length() });
+            inlineItemOffsetList.append({ paragraphContentBuilder.length() });
             auto isEnteringBidi = inlineItem.isInlineBoxStart();
             handleEnterExitBidiContext(paragraphContentBuilder
                 , style.unicodeBidi()
@@ -395,7 +420,7 @@ static inline void buildBidiParagraph(const RenderStyle& rootStyle, const Inline
         } else if (inlineItem.isSoftLineBreak()) {
             // FIXME: Unwind the bidi stack for soft line break too.
             appendTextBasedContent();
-            inlineItemOffsetList.uncheckedAppend({ inlineTextBoxOffset + downcast<InlineSoftLineBreakItem>(inlineItem).position() });
+            inlineItemOffsetList.append({ inlineTextBoxOffset + downcast<InlineSoftLineBreakItem>(inlineItem).position() });
         } else if (inlineItem.isHardLineBreak()) {
             // Bidi handling requires us to close all the nested bidi contexts at the end of the line triggered by forced line breaks
             // and re-open it for the content on the next line (i.e. paragraph handling).
@@ -429,7 +454,7 @@ static inline void buildBidiParagraph(const RenderStyle& rootStyle, const Inline
             };
             unwindBidiContextStack();
 
-            inlineItemOffsetList.uncheckedAppend({ paragraphContentBuilder.length() });
+            inlineItemOffsetList.append({ paragraphContentBuilder.length() });
             paragraphContentBuilder.append(newlineCharacter);
 
             auto rewindBidiContextStack = [&] {
@@ -454,23 +479,33 @@ static inline void buildBidiParagraph(const RenderStyle& rootStyle, const Inline
             rewindBidiContextStack();
         } else if (inlineItem.isWordBreakOpportunity()) {
             // Soft wrap opportunity markers are opaque to bidi. 
-            inlineItemOffsetList.uncheckedAppend({ });            
+            inlineItemOffsetList.append({ });
         } else if (inlineItem.isFloat()) {
             // Floats are not part of the inline content which make them opaque to bidi.
-            inlineItemOffsetList.uncheckedAppend({ });
+            inlineItemOffsetList.append({ });
+        } else if (inlineItem.isOpaque()) {
+            if (inlineItem.layoutBox().isOutOfFlowPositioned()) {
+                // Out of flow boxes participate in inflow layout as if they were static positioned.
+                inlineItemOffsetList.append({ paragraphContentBuilder.length() });
+                paragraphContentBuilder.append(objectReplacementCharacter);
+            } else {
+                // truly opaque items are also opaque to bidi.
+                inlineItemOffsetList.append({ });
+            }
         } else
             ASSERT_NOT_IMPLEMENTED_YET();
+
     }
 }
 
-void InlineItemsBuilder::breakAndComputeBidiLevels(InlineItems& inlineItems)
+void InlineItemsBuilder::breakAndComputeBidiLevels(InlineItemList& inlineItemList)
 {
-    ASSERT(!inlineItems.isEmpty());
+    ASSERT(!inlineItemList.isEmpty());
 
     StringBuilder paragraphContentBuilder;
     InlineItemOffsetList inlineItemOffsets;
-    inlineItemOffsets.reserveInitialCapacity(inlineItems.size());
-    buildBidiParagraph(root().style(), inlineItems, paragraphContentBuilder, inlineItemOffsets);
+    inlineItemOffsets.reserveInitialCapacity(inlineItemList.size());
+    buildBidiParagraph(root().style(), inlineItemList, paragraphContentBuilder, inlineItemOffsets);
     if (paragraphContentBuilder.isEmpty()) {
         // Style may trigger visual reordering even on a completely empty content.
         // e.g. <div><span style="direction:rtl"></span></div>
@@ -482,7 +517,7 @@ void InlineItemsBuilder::breakAndComputeBidiLevels(InlineItems& inlineItems)
         // UnicodeBidi::Plaintext makes directionality calculated without taking parent direction property into account.
         return;
     }
-    ASSERT(inlineItemOffsets.size() == inlineItems.size());
+    ASSERT(inlineItemOffsets.size() == inlineItemList.size());
     // 1. Setup the bidi boundary loop by calling ubidi_setPara with the paragraph text.
     // 2. Call ubidi_getLogicalRun to advance to the next bidi boundary until we hit the end of the content.
     // 3. Set the computed bidi level on the associated inline items. Split them as needed.
@@ -526,7 +561,7 @@ void InlineItemsBuilder::breakAndComputeBidiLevels(InlineItems& inlineItems)
             // Start of the range is always where we left off (bidi ranges do not have gaps).
             for (; inlineItemIndex < inlineItemOffsets.size(); ++inlineItemIndex) {
                 auto offset = inlineItemOffsets[inlineItemIndex];
-                auto& inlineItem = inlineItems[inlineItemIndex];
+                auto& inlineItem = inlineItemList[inlineItemIndex];
                 if (!offset) {
                     // This is an opaque item. Let's post-process it.
                     hasSeenOpaqueItem = true;
@@ -544,7 +579,7 @@ void InlineItemsBuilder::breakAndComputeBidiLevels(InlineItems& inlineItems)
                 auto& inlineTextItem = downcast<InlineTextItem>(inlineItem);
                 auto endPosition = *offset + inlineTextItem.length();
                 if (endPosition > bidiEnd) {
-                    inlineItems.insert(inlineItemIndex + 1, inlineTextItem.split(bidiEnd - *offset));
+                    inlineItemList.insert(inlineItemIndex + 1, inlineTextItem.split(bidiEnd - *offset));
                     // Right side is going to be processed at the next bidi range.
                     inlineItemOffsets.insert(inlineItemIndex + 1, bidiEnd);
                     ++inlineItemIndex;
@@ -563,18 +598,24 @@ void InlineItemsBuilder::breakAndComputeBidiLevels(InlineItems& inlineItems)
         // Opaque runs are excluded from the visual list (ie. only empty inline boxes should be kept around as bidi content -to figure out their visual order).
         enum class InlineBoxHasContent : bool { No, Yes };
         Vector<InlineBoxHasContent> inlineBoxContentFlagStack;
-        inlineBoxContentFlagStack.reserveInitialCapacity(inlineItems.size());
-        for (auto index = inlineItems.size(); index--;) {
-            auto& inlineItem = inlineItems[index];
+        inlineBoxContentFlagStack.reserveInitialCapacity(inlineItemList.size());
+        for (auto index = inlineItemList.size(); index--;) {
+            auto& inlineItem = inlineItemList[index];
+            auto& style = inlineItem.style();
+            auto initiatesControlCharacter = style.rtlOrdering() == Order::Logical && style.unicodeBidi() != UnicodeBidi::Normal;
+
             if (inlineItem.isInlineBoxStart()) {
                 ASSERT(!inlineBoxContentFlagStack.isEmpty());
-                if (inlineBoxContentFlagStack.takeLast() == InlineBoxHasContent::Yes)
-                    inlineItems[index].setBidiLevel(InlineItem::opaqueBidiLevel);
+                if (inlineBoxContentFlagStack.takeLast() == InlineBoxHasContent::Yes) {
+                    if (!initiatesControlCharacter)
+                        inlineItemList[index].setBidiLevel(InlineItem::opaqueBidiLevel);
+                }
                 continue;
             }
             if (inlineItem.isInlineBoxEnd()) {
                 inlineBoxContentFlagStack.append(InlineBoxHasContent::No);
-                inlineItem.setBidiLevel(InlineItem::opaqueBidiLevel);
+                if (!initiatesControlCharacter)
+                    inlineItem.setBidiLevel(InlineItem::opaqueBidiLevel);
                 continue;
             }
             if (inlineItem.isWordBreakOpportunity()) {
@@ -589,7 +630,7 @@ void InlineItemsBuilder::breakAndComputeBidiLevels(InlineItems& inlineItems)
     setBidiLevelForOpaqueInlineItems();
 }
 
-static inline bool canCacheMeasuredWidthOnInlineTextItem(const InlineTextBox& inlineTextBox, size_t start, size_t length, bool isWhitespace)
+static inline bool canCacheMeasuredWidthOnInlineTextItem(const InlineTextBox& inlineTextBox, bool isWhitespace)
 {
     // Do not cache when:
     // 1. first-line style's unique font properties may produce non-matching width values.
@@ -598,18 +639,12 @@ static inline bool canCacheMeasuredWidthOnInlineTextItem(const InlineTextBox& in
         return false;
     if (!isWhitespace || !TextUtil::shouldPreserveSpacesAndTabs(inlineTextBox))
         return true;
-    // FIXME: Currently we opt out of caching only when we see a preserved \t character (position dependent measured width).
-    auto textContent = inlineTextBox.content();
-    for (auto index = start; index < start + length; ++index) {
-        if (textContent[index] == tabCharacter)
-            return false;
-    }
-    return true;
+    return !inlineTextBox.hasPositionDependentContentWidth();
 }
 
-void InlineItemsBuilder::computeInlineTextItemWidths(InlineItems& inlineItems)
+void InlineItemsBuilder::computeInlineTextItemWidths(InlineItemList& inlineItemList)
 {
-    for (auto& inlineItem : inlineItems) {
+    for (auto& inlineItem : inlineItemList) {
         if (!inlineItem.isText())
             continue;
 
@@ -618,23 +653,24 @@ void InlineItemsBuilder::computeInlineTextItemWidths(InlineItems& inlineItems)
         auto start = inlineTextItem.start();
         auto length = inlineTextItem.length();
         auto needsMeasuring = length && !inlineTextItem.isZeroWidthSpaceSeparator();
-        if (!needsMeasuring || !canCacheMeasuredWidthOnInlineTextItem(inlineTextBox, start, length, inlineTextItem.isWhitespace()))
+        if (!needsMeasuring || !canCacheMeasuredWidthOnInlineTextItem(inlineTextBox, inlineTextItem.isWhitespace()))
             continue;
         inlineTextItem.setWidth(TextUtil::width(inlineTextItem, inlineTextItem.style().fontCascade(), start, start + length, { }));
     }
 }
 
-void InlineItemsBuilder::handleTextContent(const InlineTextBox& inlineTextBox, InlineItems& inlineItems, std::optional<size_t> partialContentOffset)
+void InlineItemsBuilder::handleTextContent(const InlineTextBox& inlineTextBox, InlineItemList& inlineItemList, std::optional<size_t> partialContentOffset)
 {
     auto text = inlineTextBox.content();
     auto contentLength = text.length();
     if (!contentLength)
-        return inlineItems.append(InlineTextItem::createEmptyItem(inlineTextBox));
+        return inlineItemList.append(InlineTextItem::createEmptyItem(inlineTextBox));
 
     if (inlineTextBox.isCombined())
-        return inlineItems.append(InlineTextItem::createNonWhitespaceItem(inlineTextBox, { }, contentLength, UBIDI_DEFAULT_LTR, false, { }));
+        return inlineItemList.append(InlineTextItem::createNonWhitespaceItem(inlineTextBox, { }, contentLength, UBIDI_DEFAULT_LTR, false, { }));
 
-    m_contentRequiresVisualReordering = m_contentRequiresVisualReordering || TextUtil::containsStrongDirectionalityText(text);
+    m_contentRequiresVisualReordering = m_contentRequiresVisualReordering || requiresVisualReordering(inlineTextBox);
+    m_isNonBidiTextAndForcedLineBreakOnlyContent = m_isNonBidiTextAndForcedLineBreakOnlyContent && !m_contentRequiresVisualReordering;
     auto& style = inlineTextBox.style();
     auto shouldPreserveSpacesAndTabs = TextUtil::shouldPreserveSpacesAndTabs(inlineTextBox);
     auto shouldPreserveNewline = TextUtil::shouldPreserveNewline(inlineTextBox);
@@ -648,7 +684,7 @@ void InlineItemsBuilder::handleTextContent(const InlineTextBox& inlineTextBox, I
             auto isSegmentBreakCandidate = text[currentPosition] == newlineCharacter;
             if (!isSegmentBreakCandidate || !shouldPreserveNewline)
                 return false;
-            inlineItems.append(InlineSoftLineBreakItem::createSoftLineBreakItem(inlineTextBox, currentPosition++));
+            inlineItemList.append(InlineSoftLineBreakItem::createSoftLineBreakItem(inlineTextBox, currentPosition++));
             return true;
         };
         if (handleSegmentBreak())
@@ -656,7 +692,9 @@ void InlineItemsBuilder::handleTextContent(const InlineTextBox& inlineTextBox, I
 
         auto handleWhitespace = [&] {
             auto stopAtWordSeparatorBoundary = shouldPreserveSpacesAndTabs && style.fontCascade().wordSpacing();
-            auto whitespaceContent = moveToNextNonWhitespacePosition(text, currentPosition, shouldPreserveNewline, shouldPreserveSpacesAndTabs, stopAtWordSeparatorBoundary);
+            auto whitespaceContent = text.is8Bit() ?
+                moveToNextNonWhitespacePosition(text.span8(), currentPosition, shouldPreserveNewline, shouldPreserveSpacesAndTabs, stopAtWordSeparatorBoundary) :
+                moveToNextNonWhitespacePosition(text.span16(), currentPosition, shouldPreserveNewline, shouldPreserveSpacesAndTabs, stopAtWordSeparatorBoundary);
             if (!whitespaceContent)
                 return false;
 
@@ -666,9 +704,9 @@ void InlineItemsBuilder::handleTextContent(const InlineTextBox& inlineTextBox, I
                 // For break-spaces, a soft wrap opportunity exists after every space and every tab.
                 // FIXME: if this turns out to be a perf hit with too many individual whitespace inline items, we should transition this logic to line breaking.
                 for (size_t i = 0; i < whitespaceContent->length; ++i)
-                    inlineItems.append(InlineTextItem::createWhitespaceItem(inlineTextBox, currentPosition + i, 1, UBIDI_DEFAULT_LTR, whitespaceContent->isWordSeparator, { }));
+                    inlineItemList.append(InlineTextItem::createWhitespaceItem(inlineTextBox, currentPosition + i, 1, UBIDI_DEFAULT_LTR, whitespaceContent->isWordSeparator, { }));
             } else
-                inlineItems.append(InlineTextItem::createWhitespaceItem(inlineTextBox, currentPosition, whitespaceContent->length, UBIDI_DEFAULT_LTR, whitespaceContent->isWordSeparator, { }));
+                inlineItemList.append(InlineTextItem::createWhitespaceItem(inlineTextBox, currentPosition, whitespaceContent->length, UBIDI_DEFAULT_LTR, whitespaceContent->isWordSeparator, { }));
             currentPosition += whitespaceContent->length;
             return true;
         };
@@ -689,7 +727,7 @@ void InlineItemsBuilder::handleTextContent(const InlineTextBox& inlineTextBox, I
             if (startPosition == endPosition)
                 return false;
             for (auto index = startPosition; index < endPosition; ++index)
-                inlineItems.append(InlineTextItem::createNonWhitespaceItem(inlineTextBox, index, 1, UBIDI_DEFAULT_LTR, { }, { }));
+                inlineItemList.append(InlineTextItem::createNonWhitespaceItem(inlineTextBox, index, 1, UBIDI_DEFAULT_LTR, { }, { }));
             currentPosition = endPosition;
             return true;
         };
@@ -712,7 +750,7 @@ void InlineItemsBuilder::handleTextContent(const InlineTextBox& inlineTextBox, I
                 hasTrailingSoftHyphen = text[endPosition - 1] == softHyphen;
             }
             ASSERT_IMPLIES(style.hyphens() == Hyphens::None, !hasTrailingSoftHyphen);
-            inlineItems.append(InlineTextItem::createNonWhitespaceItem(inlineTextBox, startPosition, endPosition - startPosition, UBIDI_DEFAULT_LTR, hasTrailingSoftHyphen, { }));
+            inlineItemList.append(InlineTextItem::createNonWhitespaceItem(inlineTextBox, startPosition, endPosition - startPosition, UBIDI_DEFAULT_LTR, hasTrailingSoftHyphen, { }));
             currentPosition = endPosition;
             return true;
         };
@@ -723,27 +761,36 @@ void InlineItemsBuilder::handleTextContent(const InlineTextBox& inlineTextBox, I
     }
 }
 
-void InlineItemsBuilder::handleInlineBoxStart(const Box& inlineBox, InlineItems& inlineItems)
+void InlineItemsBuilder::handleInlineBoxStart(const Box& inlineBox, InlineItemList& inlineItemList)
 {
-    inlineItems.append({ inlineBox, InlineItem::Type::InlineBoxStart });
-    auto& style = inlineBox.style();
-    m_contentRequiresVisualReordering = m_contentRequiresVisualReordering || !style.isLeftToRightDirection() || (style.rtlOrdering() == Order::Logical && style.unicodeBidi() != UnicodeBidi::Normal);
+    inlineItemList.append({ inlineBox, InlineItem::Type::InlineBoxStart });
+    m_contentRequiresVisualReordering = m_contentRequiresVisualReordering || requiresVisualReordering(inlineBox);
+    m_isNonBidiTextAndForcedLineBreakOnlyContent = false;
 }
 
-void InlineItemsBuilder::handleInlineBoxEnd(const Box& inlineBox, InlineItems& inlineItems)
+void InlineItemsBuilder::handleInlineBoxEnd(const Box& inlineBox, InlineItemList& inlineItemList)
 {
-    inlineItems.append({ inlineBox, InlineItem::Type::InlineBoxEnd });
+    inlineItemList.append({ inlineBox, InlineItem::Type::InlineBoxEnd });
+    ASSERT(!m_isNonBidiTextAndForcedLineBreakOnlyContent);
+    m_isNonBidiTextAndForcedLineBreakOnlyContent = false;
     // Inline box end item itself can not trigger bidi content.
     ASSERT(contentRequiresVisualReordering() || inlineBox.style().isLeftToRightDirection() || inlineBox.style().rtlOrdering() == Order::Visual || inlineBox.style().unicodeBidi() == UnicodeBidi::Normal);
 }
 
-void InlineItemsBuilder::handleInlineLevelBox(const Box& layoutBox, InlineItems& inlineItems)
+void InlineItemsBuilder::handleInlineLevelBox(const Box& layoutBox, InlineItemList& inlineItemList)
 {
-    if (layoutBox.isAtomicInlineLevelBox())
-        return inlineItems.append({ layoutBox, InlineItem::Type::Box });
+    if (layoutBox.isRubyAnnotationBox())
+        return;
 
-    if (layoutBox.isLineBreakBox())
-        return inlineItems.append({ layoutBox, layoutBox.isWordBreakOpportunity() ? InlineItem::Type::WordBreakOpportunity : InlineItem::Type::HardLineBreak });
+    if (layoutBox.isAtomicInlineLevelBox()) {
+        m_isNonBidiTextAndForcedLineBreakOnlyContent = false;
+        return inlineItemList.append({ layoutBox, InlineItem::Type::Box });
+    }
+
+    if (layoutBox.isLineBreakBox()) {
+        m_isNonBidiTextAndForcedLineBreakOnlyContent = m_isNonBidiTextAndForcedLineBreakOnlyContent && isTextOrLineBreak(layoutBox);
+        return inlineItemList.append({ layoutBox, layoutBox.isWordBreakOpportunity() ? InlineItem::Type::WordBreakOpportunity : InlineItem::Type::HardLineBreak });
+    }
 
     ASSERT_NOT_REACHED();
 }
