@@ -25,7 +25,8 @@
 
 #pragma once
 
-#include "InlineFormattingState.h"
+#include "InlineContentCache.h"
+#include "InlineItem.h"
 #include "InlineLineTypes.h"
 #include "LayoutElementBox.h"
 #include <wtf/text/StringBuilder.h>
@@ -36,27 +37,33 @@ class InlineTextBox;
 
 class InlineItemsBuilder {
 public:
-    InlineItemsBuilder(const ElementBox& formattingContextRoot, InlineFormattingState&);
+    InlineItemsBuilder(InlineContentCache&, const ElementBox& root);
     void build(InlineItemPosition startPosition);
 
 private:
-    void collectInlineItems(InlineItems&, FormattingState::OutOfFlowBoxList&, InlineItemPosition startPosition);
-    void breakAndComputeBidiLevels(InlineItems&);
-    void computeInlineTextItemWidths(InlineItems&);
+    void collectInlineItems(InlineItemList&, InlineItemPosition startPosition);
+    using LayoutQueue = Vector<CheckedRef<const Box>>;
+    LayoutQueue initializeLayoutQueue(InlineItemPosition startPosition);
+    bool traverseUntilDamaged(LayoutQueue&, const Box& subtreeRoot, const Box& firstDamagedLayoutBox);
+    void breakAndComputeBidiLevels(InlineItemList&);
+    void computeInlineTextItemWidths(InlineItemList&);
 
-    void handleTextContent(const InlineTextBox&, InlineItems&, std::optional<size_t> partialContentOffset);
-    void handleInlineBoxStart(const Box&, InlineItems&);
-    void handleInlineBoxEnd(const Box&, InlineItems&);
-    void handleInlineLevelBox(const Box&, InlineItems&);
+    void handleTextContent(const InlineTextBox&, InlineItemList&, std::optional<size_t> partialContentOffset);
+    void handleInlineBoxStart(const Box&, InlineItemList&);
+    void handleInlineBoxEnd(const Box&, InlineItemList&);
+    void handleInlineLevelBox(const Box&, InlineItemList&);
     
     bool contentRequiresVisualReordering() const { return m_contentRequiresVisualReordering; }
 
     const ElementBox& root() const { return m_root; }
+    InlineContentCache& inlineContentCache() { return m_inlineContentCache; }
 
+private:
+    InlineContentCache& m_inlineContentCache;
     const ElementBox& m_root;
-    // FIXME: We should not need this here. This is only required by the out of flow boxes.
-    InlineFormattingState& m_formattingState;
+
     bool m_contentRequiresVisualReordering { false };
+    bool m_isNonBidiTextAndForcedLineBreakOnlyContent { true };
 };
 
 }
