@@ -36,12 +36,13 @@ namespace Layout {
 
 static InlineRect flipLogicalLineRectToVisualForWritingMode(const InlineRect& lineLogicalRect, WritingMode writingMode)
 {
-    switch (writingMode) {
-    case WritingMode::TopToBottom:
+    switch (writingModeToBlockFlowDirection(writingMode)) {
+    case BlockFlowDirection::TopToBottom:
+    case BlockFlowDirection::BottomToTop:
         return lineLogicalRect;
-    case WritingMode::LeftToRight:
-    case WritingMode::RightToLeft:
-        // See InlineFormattingGeometry for more info.
+    case BlockFlowDirection::LeftToRight:
+    case BlockFlowDirection::RightToLeft:
+        // See InlineFormattingUtils for more info.
         return { lineLogicalRect.left(), lineLogicalRect.top(), lineLogicalRect.height(), lineLogicalRect.width() };
     default:
         ASSERT_NOT_REACHED();
@@ -50,12 +51,13 @@ static InlineRect flipLogicalLineRectToVisualForWritingMode(const InlineRect& li
     return lineLogicalRect;
 }
 
-InlineDisplayLineBuilder::InlineDisplayLineBuilder(const InlineFormattingContext& inlineFormattingContext)
+InlineDisplayLineBuilder::InlineDisplayLineBuilder(InlineFormattingContext& inlineFormattingContext, const ConstraintsForInlineContent& constraints)
     : m_inlineFormattingContext(inlineFormattingContext)
+    , m_constraints(constraints)
 {
 }
 
-InlineDisplayLineBuilder::EnclosingLineGeometry InlineDisplayLineBuilder::collectEnclosingLineGeometry(const LineBuilder::LayoutResult& lineLayoutResult, const LineBox& lineBox, const InlineRect& lineBoxRect) const
+InlineDisplayLineBuilder::EnclosingLineGeometry InlineDisplayLineBuilder::collectEnclosingLineGeometry(const LineLayoutResult& lineLayoutResult, const LineBox& lineBox, const InlineRect& lineBoxRect) const
 {
     auto& rootInlineBox = lineBox.rootInlineBox();
     auto initialEnclosingTopAndBottom = [&]() -> std::tuple<std::optional<InlineLayoutUnit>, std::optional<InlineLayoutUnit>>  {
@@ -94,11 +96,9 @@ InlineDisplayLineBuilder::EnclosingLineGeometry InlineDisplayLineBuilder::collec
             borderBox.moveBy(lineBoxRect.topLeft());
         } else if (inlineLevelBox.isInlineBox()) {
             auto& boxGeometry = formattingContext().geometryForBox(layoutBox);
-            auto isContentful = [&] {
-                // In standards mode, inline boxes always start with an imaginary strut.
-                return layoutState().inStandardsMode() || inlineLevelBox.hasContent() || boxGeometry.horizontalBorder() || (boxGeometry.horizontalPadding() && boxGeometry.horizontalPadding().value());
-            };
-            if (!isContentful())
+            // In standards mode, inline boxes always start with an imaginary strut.
+            auto isContentful = formattingContext().layoutState().inStandardsMode() || inlineLevelBox.hasContent() || boxGeometry.horizontalBorder() || (boxGeometry.horizontalPadding() && boxGeometry.horizontalPadding().value());
+            if (!isContentful)
                 continue;
             borderBox = lineBox.logicalBorderBoxForInlineBox(layoutBox, boxGeometry);
             borderBox.moveBy(lineBoxRect.topLeft());
@@ -121,8 +121,9 @@ InlineDisplayLineBuilder::EnclosingLineGeometry InlineDisplayLineBuilder::collec
     return { { enclosingTop.value_or(lineBoxRect.top()), enclosingBottom.value_or(lineBoxRect.top()) }, contentOverflowRect };
 }
 
-InlineDisplay::Line InlineDisplayLineBuilder::build(const LineBuilder::LayoutResult& lineLayoutResult, const LineBox& lineBox, const ConstraintsForInlineContent& constraints, bool lineIsFullyTruncatedInBlockDirection) const
+InlineDisplay::Line InlineDisplayLineBuilder::build(const LineLayoutResult& lineLayoutResult, const LineBox& lineBox, bool lineIsFullyTruncatedInBlockDirection) const
 {
+    auto& constraints = this->constraints();
     auto& rootInlineBox = lineBox.rootInlineBox();
     auto isLeftToRightDirection = lineLayoutResult.directionality.inlineBaseDirection == TextDirection::LTR;
     auto lineBoxLogicalRect = lineBox.logicalRect();
@@ -224,10 +225,10 @@ static float truncate(InlineDisplay::Box& displayBox, float contentWidth, float
     return contentWidth;
 }
 
-static float truncateOverflowingDisplayBoxes(const InlineDisplay::Line& displayLine, InlineDisplay::Boxes& boxes, float ellipsisWidth, const RenderStyle& rootStyle, LineEndingEllipsisPolicy lineEndingEllipsisPolicy)
+static float truncateOverflowingDisplayBoxes(InlineDisplay::Boxes& boxes, size_t startIndex, size_t endIndex, float lineBoxVisualLeft, float lineBoxVisualRight, float ellipsisWidth, const RenderStyle& rootStyle, LineEndingEllipsisPolicy lineEndingEllipsisPolicy)
 {
+    ASSERT(endIndex && startIndex <= endIndex);
     // We gotta truncate some runs.
-    ASSERT(displayLine.lineBoxLogicalRect().x() + displayLine.contentLogicalLeft() + displayLine.contentLogicalWidth() + ellipsisWidth > displayLine.lineBoxLogicalRect().maxX());
     auto isHorizontal = rootStyle.isHorizontalWritingMode();
     auto left = [&] (auto& displayBox) {
         return isHorizontal ? displayBox.left() : displayBox.top();
@@ -241,13 +242,14 @@ static float truncateOverflowingDisplayBoxes(const InlineDisplay::Line& displayL
     // The logically first character or atomic inline-level element on a line must be clipped rather than ellipsed.
     auto isFirstContentRun = true;
     if (rootStyle.isLeftToRightDirection()) {
-        auto visualRightForContentEnd = std::max(0.f, right(displayLine) - ellipsisWidth);
+        auto visualRightForContentEnd = std::max(0.f, lineBoxVisualRight - ellipsisWidth);
 #if USE_FLOAT_AS_INLINE_LAYOUT_UNIT
         if (visualRightForContentEnd)
             visualRightForContentEnd += LayoutUnit::epsilon();
 #endif
         auto truncateRight = std::optional<float> { };
-        for (auto& displayBox : boxes) {
+        for (auto index = startIndex; index <= endIndex; ++index) {
+            auto& displayBox = boxes[index];
             if (displayBox.isInlineBox())
                 continue;
             if (right(displayBox) > visualRightForContentEnd) {
@@ -262,12 +264,13 @@ static float truncateOverflowingDisplayBoxes(const InlineDisplay::Line& displayL
     }
 
     auto truncateLeft = std::optional<float> { };
-    auto visualLeftForContentEnd = std::max(0.f, left(displayLine) + ellipsisWidth);
+    auto visualLeftForContentEnd = std::max(0.f, lineBoxVisualLeft + ellipsisWidth);
 #if USE_FLOAT_AS_INLINE_LAYOUT_UNIT
     if (visualLeftForContentEnd)
         visualLeftForContentEnd -= LayoutUnit::epsilon();
 #endif
-    for (auto& displayBox : makeReversedRange(boxes)) {
+    for (size_t index = endIndex + 1; index-- > startIndex;) {
+        auto& displayBox = boxes[index];
         if (displayBox.isInlineBox())
             continue;
         if (left(displayBox) < visualLeftForContentEnd) {
@@ -288,7 +291,7 @@ std::optional<FloatRect> InlineDisplayLineBuilder::trailingEllipsisVisualRectAft
 
     auto contentNeedsEllipsis = [&] {
         switch (lineEndingEllipsisPolicy) {
-        case LineEndingEllipsisPolicy::No:
+        case LineEndingEllipsisPolicy::NoEllipsis:
             return false;
         case LineEndingEllipsisPolicy::WhenContentOverflowsInInlineDirection:
             return displayLine.contentLogicalWidth() > displayLine.lineBoxLogicalRect().width();
@@ -334,14 +337,138 @@ std::optional<FloatRect> InlineDisplayLineBuilder::trailingEllipsisVisualRectAft
             // All we have is the root inline box.
             ellipsisStart = displayBoxes.first().left();
         }
-    } else
-        ellipsisStart = truncateOverflowingDisplayBoxes(displayLine, displayBoxes, ellipsisWidth, rootStyle, lineEndingEllipsisPolicy);
+    } else {
+        auto lineBoxVisualLeft = rootStyle.isHorizontalWritingMode() ? displayLine.left() : displayLine.top();
+        auto lineBoxVisualRight = rootStyle.isHorizontalWritingMode() ? displayLine.right() : displayLine.bottom();
+        ellipsisStart = truncateOverflowingDisplayBoxes(displayBoxes, 0, displayBoxes.size() - 1, lineBoxVisualLeft, lineBoxVisualRight, ellipsisWidth, rootStyle, lineEndingEllipsisPolicy);
+    }
 
     if (rootStyle.isHorizontalWritingMode())
         return FloatRect { ellipsisStart, rootInlineBox.top(), ellipsisWidth, rootInlineBox.height() };
     return FloatRect { rootInlineBox.left(), ellipsisStart, rootInlineBox.width(), ellipsisWidth };
 }
 
+static inline bool isEligibleForLinkBoxLineClamp(auto& displayBoxes)
+{
+    if (displayBoxes.size() < 3) {
+        // We need at least 3 display boxes to generate content with link ([root inline box][inline box][content])
+        return false;
+    }
+    auto& rootStyle = displayBoxes[0].layoutBox().style();
+    if (!rootStyle.isLeftToRightDirection() || !rootStyle.isHorizontalWritingMode())
+        return false;
+    auto& linkCandidateBox = displayBoxes[displayBoxes.size() - 2];
+    if (!linkCandidateBox.isNonRootInlineBox() || !linkCandidateBox.isFirstForLayoutBox()) {
+        // Link spanning multiple lines looks odd with line-clamp.
+        return false;
+    }
+    return linkCandidateBox.layoutBox().style().isLink();
+}
+
+static constexpr int legacyMatchingLinkBoxOffset = 3;
+static inline void makeRoomForLinkBoxOnClampedLineIfNeeded(auto& clampedLine, auto& displayBoxes, auto insertionPosition, auto linkContentWidth)
+{
+    ASSERT(insertionPosition);
+    auto ellipsisBoxRect = *clampedLine.ellipsisVisualRect();
+    if (ellipsisBoxRect.maxX() + linkContentWidth <= clampedLine.right())
+        return;
+    auto& rootStyle = displayBoxes[0].layoutBox().style();
+    auto startIndex = [&]() -> size_t {
+        ASSERT(insertionPosition);
+        for (size_t index = insertionPosition - 1; index--;) {
+            if (displayBoxes[index].isRootInlineBox())
+                return index + 1;
+            return index;
+        }
+        ASSERT_NOT_REACHED();
+        return 0;
+    };
+    auto endIndex = insertionPosition - 1;
+    auto ellispsisPosition = clampedLine.right() - linkContentWidth - legacyMatchingLinkBoxOffset;
+    auto ellipsisStart = truncateOverflowingDisplayBoxes(displayBoxes, startIndex(), endIndex, clampedLine.left(), ellispsisPosition, ellipsisBoxRect.width(), rootStyle, LineEndingEllipsisPolicy::WhenContentOverflowsInBlockDirection);
+    ellipsisBoxRect.setX(ellipsisStart);
+    clampedLine.setEllipsisVisualRect(ellipsisBoxRect);
+}
+
+static inline void moveDisplayBoxToClampedLine(auto& displayLines, auto clampedLineIndex, auto& displayBox, auto horizontalOffset)
+{
+    auto& clampedLine = displayLines[clampedLineIndex];
+    displayBox.setLeft(clampedLine.ellipsisVisualRect()->maxX() + horizontalOffset + legacyMatchingLinkBoxOffset);
+    // Assume baseline alignment here.
+    displayBox.moveVertically((clampedLine.top() + clampedLine.baseline()) - (displayLines.last().top() + displayLines.last().baseline()));
+    displayBox.moveToLine(clampedLineIndex);
+}
+
+void InlineDisplayLineBuilder::addLineClampTrailingLinkBoxIfApplicable(const InlineFormattingContext& inlineFormattingContext, const InlineLayoutState& inlineLayoutState, InlineDisplay::Content& displayContent)
+{
+    // This is link-box type of line clamping (-webkit-line-clamp) where we check if the inline content ends in a link
+    // and move such link content next to the clamped line's trailing ellispsis. It is meant to produce the following rendering
+    //
+    // first line
+    // second line is clamped... more info
+    //
+    // where "more info" is a link and it comes from the end of the inline content (normally invisible due to block direction clamping)
+    // This is to match legacy line clamping behavior (and not a block-ellispsis: <string> implementation) which was introduced
+    // at https://commits.webkit.org/6086@main to support special article rendering with clamped lines.
+    // It supports horizontal, left-to-right content only where the link inline box has (non-split) text content and when
+    // the link content ('more info') fits the clamped line.
+    auto clampedLineIndex = inlineLayoutState.clampedLineIndex();
+    if (!clampedLineIndex)
+        return;
+    auto& displayLines = displayContent.lines;
+    if (*clampedLineIndex >= displayLines.size() || !displayLines[*clampedLineIndex].hasEllipsis()) {
+        ASSERT_NOT_REACHED();
+        return;
+    }
+
+    if (!isEligibleForLinkBoxLineClamp(displayContent.boxes))
+        return;
+
+    auto& displayBoxes = displayContent.boxes;
+    auto insertionPosition = [&]() -> std::optional<size_t> {
+        for (size_t boxIndex = 0; boxIndex < displayBoxes.size(); ++boxIndex) {
+            if (displayBoxes[boxIndex].lineIndex() > *clampedLineIndex)
+                return boxIndex;
+        }
+        return { };
+    }();
+
+    if (!insertionPosition) {
+        ASSERT_NOT_REACHED();
+        return;
+    }
+    auto& clampedLine = displayLines[*clampedLineIndex];
+    auto linkContentWidth = displayBoxes[displayBoxes.size() - 2].visualRectIgnoringBlockDirection().width();
+    if (linkContentWidth >= clampedLine.lineBoxWidth()) {
+        // Not enough space for "more info" content.
+        return;
+    }
+    auto linkContentDisplayBox = displayBoxes.takeLast();
+    auto linkInlineBoxDisplayBox = displayBoxes.takeLast();
+
+    auto handleTrailingLineBreakIfApplicable = [&] {
+        // "more info" gets inserted after the trailing run on the clamped line unless the trailing content is forced line break.
+        auto& trailingDisplayBox = displayBoxes[*insertionPosition - 1];
+        if (!trailingDisplayBox.isLineBreak())
+            return;
+        // Move trailing line break after the link box.
+        trailingDisplayBox.moveHorizontally(linkContentWidth);
+        --*insertionPosition;
+    };
+    handleTrailingLineBreakIfApplicable();
+
+    makeRoomForLinkBoxOnClampedLineIfNeeded(clampedLine, displayBoxes, *insertionPosition, linkContentWidth);
+
+    // link box content
+    moveDisplayBoxToClampedLine(displayLines, *clampedLineIndex, linkContentDisplayBox, inlineFormattingContext.geometryForBox(linkInlineBoxDisplayBox.layoutBox()).marginBorderAndPaddingStart());
+    displayBoxes.insert(*insertionPosition, linkContentDisplayBox);
+    // Link inline box
+    moveDisplayBoxToClampedLine(displayLines, *clampedLineIndex, linkInlineBoxDisplayBox, LayoutUnit { });
+    displayBoxes.insert(*insertionPosition, linkInlineBoxDisplayBox);
+
+    clampedLine.setHasContentAfterEllipsisBox();
+}
+
 }
 }
 
