@@ -27,13 +27,12 @@
 #include "TableFormattingContext.h"
 
 #include "BlockFormattingState.h"
-#include "FloatingState.h"
-#include "InlineFormattingState.h"
 #include "LayoutBox.h"
 #include "LayoutBoxGeometry.h"
 #include "LayoutChildIterator.h"
 #include "LayoutContext.h"
 #include "LayoutInitialContainingBlock.h"
+#include "PlacedFloats.h"
 #include "RenderStyleInlines.h"
 #include "TableFormattingConstraints.h"
 #include "TableFormattingState.h"
@@ -45,8 +44,9 @@ namespace Layout {
 WTF_MAKE_ISO_ALLOCATED_IMPL(TableFormattingContext);
 
 // https://www.w3.org/TR/css-tables-3/#table-layout-algorithm
-TableFormattingContext::TableFormattingContext(const ElementBox& formattingContextRoot, TableFormattingState& formattingState)
-    : FormattingContext(formattingContextRoot, formattingState)
+TableFormattingContext::TableFormattingContext(const ElementBox& formattingContextRoot, TableFormattingState& tableFormattingState)
+    : FormattingContext(formattingContextRoot, tableFormattingState.layoutState())
+    , m_tableFormattingState(tableFormattingState)
     , m_tableFormattingGeometry(*this)
     , m_tableFormattingQuirks(*this)
 {
@@ -98,14 +98,14 @@ void TableFormattingContext::setUsedGeometryForCells(LayoutUnit availableHorizon
 
         cellBoxGeometry.setBorder(formattingGeometry.computedCellBorder(*cell));
         cellBoxGeometry.setPadding(formattingGeometry.computedPadding(cellBox, availableHorizontalSpace));
-        cellBoxGeometry.setLogicalTop(rowList[cell->startRow()].logicalTop() - sectionOffset);
-        cellBoxGeometry.setLogicalLeft(columnList[cell->startColumn()].usedLogicalLeft());
+        cellBoxGeometry.setTop(rowList[cell->startRow()].logicalTop() - sectionOffset);
+        cellBoxGeometry.setLeft(columnList[cell->startColumn()].usedLogicalLeft());
         cellBoxGeometry.setContentBoxWidth(formattingGeometry.horizontalSpaceForCellContent(*cell));
 
         if (cellBox.hasInFlowOrFloatingChild()) {
             // FIXME: This should probably be part of the invalidation state to indicate when we re-layout the cell multiple times as part of the multi-pass table algorithm.
-            auto& floatingStateForCellContent = layoutState().ensureBlockFormattingState(cellBox).floatingState();
-            floatingStateForCellContent.clear();
+            auto& placedFloatsForCellContent = layoutState().ensureBlockFormattingState(cellBox).placedFloats();
+            placedFloatsForCellContent.clear();
             LayoutContext::createFormattingContext(cellBox, layoutState())->layoutInFlowContent(formattingGeometry.constraintsForInFlowContent(cellBox));
         }
         cellBoxGeometry.setContentBoxHeight(formattingGeometry.verticalSpaceForCellContent(*cell, availableVerticalSpace));
@@ -226,8 +226,8 @@ void TableFormattingContext::setUsedGeometryForRows(LayoutUnit availableHorizont
             // This row is in a different section.
             rowLogicalTop = { };
         }
-        rowBoxGeometry.setLogicalTop(rowLogicalTop);
-        rowBoxGeometry.setLogicalLeft({ });
+        rowBoxGeometry.setTop(rowLogicalTop);
+        rowBoxGeometry.setLeft({ });
 
         rowLogicalTop += row.logicalHeight() + grid.verticalSpacing();
         previousRow = &rowBox;
@@ -280,8 +280,8 @@ void TableFormattingContext::setUsedGeometryForSections(const ConstraintsForInFl
         }
         sectionContentHeight += verticalSpacing * (rowCount - 1);
         sectionBoxGeometry.setContentBoxHeight(sectionContentHeight);
-        sectionBoxGeometry.setLogicalLeft(constraints.horizontal().logicalLeft);
-        sectionBoxGeometry.setLogicalTop(logicalTop);
+        sectionBoxGeometry.setLeft(constraints.horizontal().logicalLeft);
+        sectionBoxGeometry.setTop(logicalTop);
 
         logicalTop += sectionBoxGeometry.borderBoxHeight();
     }
