@@ -60,32 +60,36 @@ static inline void addVisitedLink(Page& page, const URL& url)
     page.visitedLinkStore().addVisitedLink(page, computeSharedStringHash(url.string()));
 }
 
-FrameLoader::HistoryController::HistoryController(LocalFrame& frame)
+HistoryController::HistoryController(LocalFrame& frame)
     : m_frame(frame)
     , m_frameLoadComplete(true)
     , m_defersLoading(false)
 {
 }
 
-FrameLoader::HistoryController::~HistoryController() = default;
+HistoryController::~HistoryController() = default;
 
-void FrameLoader::HistoryController::saveScrollPositionAndViewStateToItem(HistoryItem* item)
+void HistoryController::saveScrollPositionAndViewStateToItem(HistoryItem* item)
 {
     auto* frameView = m_frame.view();
     if (!item || !frameView)
         return;
 
-    if (m_frame.document()->backForwardCacheState() != Document::NotInBackForwardCache)
+    if (m_frame.document()->backForwardCacheState() != Document::NotInBackForwardCache) {
         item->setScrollPosition(frameView->cachedScrollPosition());
-    else
-        item->setScrollPosition(frameView->scrollPosition());
-
 #if PLATFORM(IOS_FAMILY)
-    item->setExposedContentRect(frameView->exposedContentRect());
-    item->setUnobscuredContentRect(frameView->unobscuredContentRect());
+        item->setUnobscuredContentRect(frameView->cachedUnobscuredContentRect());
+        item->setExposedContentRect(frameView->cachedExposedContentRect());
 #endif
+    } else {
+        item->setScrollPosition(frameView->scrollPosition());
+#if PLATFORM(IOS_FAMILY)
+        item->setUnobscuredContentRect(frameView->unobscuredContentRect());
+        item->setExposedContentRect(frameView->exposedContentRect());
+#endif
+    }
 
-    Page* page = m_frame.page();
+    auto* page = m_frame.page();
     if (page && m_frame.isMainFrame()) {
         item->setPageScaleFactor(page->pageScaleFactor() / page->viewScaleFactor());
 #if PLATFORM(IOS_FAMILY)
@@ -100,7 +104,7 @@ void FrameLoader::HistoryController::saveScrollPositionAndViewStateToItem(Histor
     item->notifyChanged();
 }
 
-void FrameLoader::HistoryController::clearScrollPositionAndViewState()
+void HistoryController::clearScrollPositionAndViewState()
 {
     if (!m_currentItem)
         return;
@@ -120,19 +124,9 @@ void FrameLoader::HistoryController::clearScrollPositionAndViewState()
  2) If the layout happens after the load completes, the attempt to restore at load completion time silently
  fails.  We then successfully restore it when the layout happens.
 */
-void FrameLoader::HistoryController::restoreScrollPositionAndViewState()
+void HistoryController::restoreScrollPositionAndViewState()
 {
-    if (!m_frame.loader().stateMachine().committedFirstRealDocumentLoad())
-        return;
-
-    ASSERT(m_currentItem);
-    
-    // FIXME: As the ASSERT attests, it seems we should always have a currentItem here.
-    // One counterexample is <rdar://problem/4917290>
-    // For now, to cover this issue in release builds, there is no technical harm to returning
-    // early and from a user standpoint - as in the above radar - the previous page load failed 
-    // so there *is* no scroll or view state to restore!
-    if (!m_currentItem)
+    if (!m_currentItem || !m_frame.loader().stateMachine().committedFirstRealDocumentLoad())
         return;
 
     RefPtr view = m_frame.view();
@@ -178,12 +172,12 @@ void FrameLoader::HistoryController::restoreScrollPositionAndViewState()
 #endif
 }
 
-void FrameLoader::HistoryController::updateBackForwardListForFragmentScroll()
+void HistoryController::updateBackForwardListForFragmentScroll()
 {
     updateBackForwardListClippedAtTarget(false);
 }
 
-void FrameLoader::HistoryController::saveDocumentState()
+void HistoryController::saveDocumentState()
 {
     // FIXME: Reading this bit of FrameLoader state here is unfortunate.  I need to study
     // this more to see if we can remove this dependency.
@@ -218,7 +212,7 @@ void FrameLoader::HistoryController::saveDocumentState()
 
 // Walk the frame tree, telling all frames to save their form state into their current
 // history item.
-void FrameLoader::HistoryController::saveDocumentAndScrollState()
+void HistoryController::saveDocumentAndScrollState()
 {
     for (Frame* frame = &m_frame; frame; frame = frame->tree().traverseNext(&m_frame)) {
         auto* localFrame = dynamicDowncast<LocalFrame>(frame);
@@ -229,7 +223,7 @@ void FrameLoader::HistoryController::saveDocumentAndScrollState()
     }
 }
 
-void FrameLoader::HistoryController::restoreDocumentState()
+void HistoryController::restoreDocumentState()
 {
     switch (m_frame.loader().loadType()) {
     case FrameLoadType::Reload:
@@ -260,7 +254,7 @@ void FrameLoader::HistoryController::restoreDocumentState()
     m_frame.document()->setStateForNewFormElements(m_currentItem->documentState());
 }
 
-void FrameLoader::HistoryController::invalidateCurrentItemCachedPage()
+void HistoryController::invalidateCurrentItemCachedPage()
 {
     if (!currentItem())
         return;
@@ -281,7 +275,7 @@ void FrameLoader::HistoryController::invalidateCurrentItemCachedPage()
     }
 }
 
-bool FrameLoader::HistoryController::shouldStopLoadingForHistoryItem(HistoryItem& targetItem) const
+bool HistoryController::shouldStopLoadingForHistoryItem(HistoryItem& targetItem) const
 {
     if (!m_currentItem)
         return false;
@@ -295,7 +289,7 @@ bool FrameLoader::HistoryController::shouldStopLoadingForHistoryItem(HistoryItem
 
 // Main funnel for navigating to a previous location (back/forward, non-search snap-back)
 // This includes recursion to handle loading into framesets properly
-void FrameLoader::HistoryController::goToItem(HistoryItem& targetItem, FrameLoadType type, ShouldTreatAsContinuingLoad shouldTreatAsContinuingLoad)
+void HistoryController::goToItem(HistoryItem& targetItem, FrameLoadType type, ShouldTreatAsContinuingLoad shouldTreatAsContinuingLoad)
 {
     LOG(History, "HistoryController %p goToItem %p type=%d", this, &targetItem, static_cast<int>(type));
 
@@ -332,7 +326,7 @@ void FrameLoader::HistoryController::goToItem(HistoryItem& targetItem, FrameLoad
     recursiveGoToItem(targetItem, currentItem.get(), type, shouldTreatAsContinuingLoad);
 }
 
-void FrameLoader::HistoryController::setDefersLoading(bool defer)
+void HistoryController::setDefersLoading(bool defer)
 {
     m_defersLoading = defer;
     if (!defer && m_deferredItem) {
@@ -341,7 +335,7 @@ void FrameLoader::HistoryController::setDefersLoading(bool defer)
     }
 }
 
-void FrameLoader::HistoryController::updateForBackForwardNavigation()
+void HistoryController::updateForBackForwardNavigation()
 {
     LOG(History, "HistoryController %p updateForBackForwardNavigation: Updating History for back/forward navigation in frame %p (main frame %d) %s", this, &m_frame, m_frame.isMainFrame(), m_frame.loader().documentLoader() ? m_frame.loader().documentLoader()->url().string().utf8().data() : "");
 
@@ -354,7 +348,7 @@ void FrameLoader::HistoryController::updateForBackForwardNavigation()
     updateCurrentItem();
 }
 
-void FrameLoader::HistoryController::updateForReload()
+void HistoryController::updateForReload()
 {
     LOG(History, "HistoryController %p updateForReload: Updating History for reload in frame %p (main frame %d) %s", this, &m_frame, m_frame.isMainFrame(), m_frame.loader().documentLoader() ? m_frame.loader().documentLoader()->url().string().utf8().data() : "");
 
@@ -379,7 +373,7 @@ void FrameLoader::HistoryController::updateForReload()
 //     2) Global history: Handled by the client.
 //     3) Visited links: Handled by the PageGroup.
 
-void FrameLoader::HistoryController::updateForStandardLoad(HistoryUpdateType updateType)
+void HistoryController::updateForStandardLoad(HistoryUpdateType updateType)
 {
     LOG(History, "HistoryController %p updateForStandardLoad: Updating History for standard load in frame %p (main frame %d) %s", this, &m_frame, m_frame.isMainFrame(), m_frame.loader().documentLoader()->url().string().ascii().data());
 
@@ -413,7 +407,7 @@ void FrameLoader::HistoryController::updateForStandardLoad(HistoryUpdateType upd
     }
 }
 
-void FrameLoader::HistoryController::updateForRedirectWithLockedBackForwardList()
+void HistoryController::updateForRedirectWithLockedBackForwardList()
 {
     LOG(History, "HistoryController %p updateForRedirectWithLockedBackForwardList: Updating History for redirect load in frame %p (main frame %d) %s", this, &m_frame, m_frame.isMainFrame(), m_frame.loader().documentLoader() ? m_frame.loader().documentLoader()->url().string().utf8().data() : "");
     
@@ -435,9 +429,10 @@ void FrameLoader::HistoryController::updateForRedirectWithLockedBackForwardList(
         // The client redirect replaces the current history item.
         updateCurrentItem();
     } else {
+        auto* page = m_frame.page();
         auto* parentFrame = dynamicDowncast<LocalFrame>(m_frame.tree().parent());
-        if (parentFrame && parentFrame->loader().history().currentItem())
-            parentFrame->loader().history().currentItem()->setChildItem(createItem());
+        if (page && parentFrame && parentFrame->loader().history().currentItem())
+            parentFrame->loader().history().currentItem()->setChildItem(createItem(page->historyItemClient()));
     }
 
     if (!historyURL.isEmpty() && !usesEphemeralSession) {
@@ -449,7 +444,7 @@ void FrameLoader::HistoryController::updateForRedirectWithLockedBackForwardList(
     }
 }
 
-void FrameLoader::HistoryController::updateForClientRedirect()
+void HistoryController::updateForClientRedirect()
 {
     LOG(History, "HistoryController %p updateForClientRedirect: Updating History for client redirect in frame %p (main frame %d) %s", this, &m_frame, m_frame.isMainFrame(), m_frame.loader().documentLoader() ? m_frame.loader().documentLoader()->url().string().utf8().data() : "");
 
@@ -469,7 +464,7 @@ void FrameLoader::HistoryController::updateForClientRedirect()
     }
 }
 
-void FrameLoader::HistoryController::updateForCommit()
+void HistoryController::updateForCommit()
 {
     FrameLoader& frameLoader = m_frame.loader();
     LOG(History, "HistoryController %p updateForCommit: Updating History for commit in frame %p (main frame %d) %s", this, &m_frame, m_frame.isMainFrame(), m_frame.loader().documentLoader() ? m_frame.loader().documentLoader()->url().string().utf8().data() : "");
@@ -500,19 +495,19 @@ void FrameLoader::HistoryController::updateForCommit()
     }
 }
 
-bool FrameLoader::HistoryController::isReplaceLoadTypeWithProvisionalItem(FrameLoadType type)
+bool HistoryController::isReplaceLoadTypeWithProvisionalItem(FrameLoadType type)
 {
     // Going back to an error page in a subframe can trigger a FrameLoadType::Replace
     // while m_provisionalItem is set, so we need to commit it.
     return type == FrameLoadType::Replace && m_provisionalItem;
 }
 
-bool FrameLoader::HistoryController::isReloadTypeWithProvisionalItem(FrameLoadType type)
+bool HistoryController::isReloadTypeWithProvisionalItem(FrameLoadType type)
 {
     return (type == FrameLoadType::Reload || type == FrameLoadType::ReloadFromOrigin) && m_provisionalItem;
 }
 
-void FrameLoader::HistoryController::recursiveUpdateForCommit()
+void HistoryController::recursiveUpdateForCommit()
 {
     // The frame that navigated will now have a null provisional item.
     // Ignore it and its children.
@@ -551,7 +546,7 @@ void FrameLoader::HistoryController::recursiveUpdateForCommit()
     }
 }
 
-void FrameLoader::HistoryController::updateForSameDocumentNavigation()
+void HistoryController::updateForSameDocumentNavigation()
 {
     if (m_frame.document()->url().isEmpty())
         return;
@@ -574,7 +569,7 @@ void FrameLoader::HistoryController::updateForSameDocumentNavigation()
     }
 }
 
-void FrameLoader::HistoryController::recursiveUpdateForSameDocumentNavigation()
+void HistoryController::recursiveUpdateForSameDocumentNavigation()
 {
     // The frame that navigated will now have a null provisional item.
     // Ignore it and its children.
@@ -600,7 +595,7 @@ void FrameLoader::HistoryController::recursiveUpdateForSameDocumentNavigation()
     }
 }
 
-void FrameLoader::HistoryController::updateForFrameLoadCompleted()
+void HistoryController::updateForFrameLoadCompleted()
 {
     // Even if already complete, we might have set a previous item on a frame that
     // didn't do any data loading on the past transaction. Make sure to track that
@@ -608,21 +603,21 @@ void FrameLoader::HistoryController::updateForFrameLoadCompleted()
     m_frameLoadComplete = true;
 }
 
-void FrameLoader::HistoryController::setCurrentItem(HistoryItem& item)
+void HistoryController::setCurrentItem(HistoryItem& item)
 {
     m_frameLoadComplete = false;
     m_previousItem = m_currentItem;
     m_currentItem = &item;
 }
 
-void FrameLoader::HistoryController::setCurrentItemTitle(const StringWithDirection& title)
+void HistoryController::setCurrentItemTitle(const StringWithDirection& title)
 {
     // FIXME: This ignores the title's direction.
     if (m_currentItem)
         m_currentItem->setTitle(title.string);
 }
 
-bool FrameLoader::HistoryController::currentItemShouldBeReplaced() const
+bool HistoryController::currentItemShouldBeReplaced() const
 {
     // From the HTML5 spec for location.assign():
     //  "If the browsing context's session history contains only one Document,
@@ -631,7 +626,7 @@ bool FrameLoader::HistoryController::currentItemShouldBeReplaced() const
     return m_currentItem && !m_previousItem && equalIgnoringASCIICase(m_currentItem->urlString(), aboutBlankURL().string());
 }
 
-void FrameLoader::HistoryController::clearPreviousItem()
+void HistoryController::clearPreviousItem()
 {
     m_previousItem = nullptr;
     for (auto* child = m_frame.tree().firstChild(); child; child = child->tree().nextSibling()) {
@@ -642,12 +637,12 @@ void FrameLoader::HistoryController::clearPreviousItem()
     }
 }
 
-void FrameLoader::HistoryController::setProvisionalItem(HistoryItem* item)
+void HistoryController::setProvisionalItem(HistoryItem* item)
 {
     m_provisionalItem = item;
 }
 
-void FrameLoader::HistoryController::initializeItem(HistoryItem& item)
+void HistoryController::initializeItem(HistoryItem& item)
 {
     DocumentLoader* documentLoader = m_frame.loader().documentLoader();
     ASSERT(documentLoader);
@@ -692,9 +687,9 @@ void FrameLoader::HistoryController::initializeItem(HistoryItem& item)
     item.setFormInfoFromRequest(documentLoader->request());
 }
 
-Ref<HistoryItem> FrameLoader::HistoryController::createItem()
+Ref<HistoryItem> HistoryController::createItem(HistoryItemClient& client)
 {
-    Ref<HistoryItem> item = HistoryItem::create();
+    Ref<HistoryItem> item = HistoryItem::create(client);
     initializeItem(item);
     
     // Set the item for which we will save document state
@@ -703,9 +698,9 @@ Ref<HistoryItem> FrameLoader::HistoryController::createItem()
     return item;
 }
 
-Ref<HistoryItem> FrameLoader::HistoryController::createItemTree(LocalFrame& targetFrame, bool clipAtTarget)
+Ref<HistoryItem> HistoryController::createItemTree(HistoryItemClient& client, LocalFrame& targetFrame, bool clipAtTarget)
 {
-    Ref<HistoryItem> bfItem = createItem();
+    Ref<HistoryItem> bfItem = createItem(client);
     if (!m_frameLoadComplete)
         saveScrollPositionAndViewStateToItem(m_previousItem.get());
 
@@ -735,7 +730,7 @@ Ref<HistoryItem> FrameLoader::HistoryController::createItemTree(LocalFrame& targ
             // to be ignored on reload.
             
             if (!(!hasChildLoaded && localChild->ownerElement() && is<HTMLObjectElement>(localChild->ownerElement())))
-                bfItem->addChildItem(childLoader.history().createItemTree(targetFrame, clipAtTarget));
+                bfItem->addChildItem(childLoader.history().createItemTree(client, targetFrame, clipAtTarget));
         }
     }
     // FIXME: Eliminate the isTargetItem flag in favor of itemSequenceNumber.
@@ -748,7 +743,7 @@ Ref<HistoryItem> FrameLoader::HistoryController::createItemTree(LocalFrame& targ
 // tracking whether each frame already has the content the item requests.  If there is
 // a match, we set the provisional item and recurse.  Otherwise we will reload that
 // frame and all its kids in recursiveGoToItem.
-void FrameLoader::HistoryController::recursiveSetProvisionalItem(HistoryItem& item, HistoryItem* fromItem)
+void HistoryController::recursiveSetProvisionalItem(HistoryItem& item, HistoryItem* fromItem)
 {
     if (!itemsAreClones(item, fromItem))
         return;
@@ -770,7 +765,7 @@ void FrameLoader::HistoryController::recursiveSetProvisionalItem(HistoryItem& it
 
 // We now traverse the frame tree and item tree a second time, loading frames that
 // do have the content the item requests.
-void FrameLoader::HistoryController::recursiveGoToItem(HistoryItem& item, HistoryItem* fromItem, FrameLoadType type, ShouldTreatAsContinuingLoad shouldTreatAsContinuingLoad)
+void HistoryController::recursiveGoToItem(HistoryItem& item, HistoryItem* fromItem, FrameLoadType type, ShouldTreatAsContinuingLoad shouldTreatAsContinuingLoad)
 {
     if (!itemsAreClones(item, fromItem)) {
         m_frame.loader().loadItem(item, fromItem, type, shouldTreatAsContinuingLoad);
@@ -791,7 +786,7 @@ void FrameLoader::HistoryController::recursiveGoToItem(HistoryItem& item, Histor
 }
 
 // The following logic must be kept in sync with WebKit::WebBackForwardListItem::itemIsClone().
-bool FrameLoader::HistoryController::itemsAreClones(HistoryItem& item1, HistoryItem* item2) const
+bool HistoryController::itemsAreClones(HistoryItem& item1, HistoryItem* item2) const
 {
     // If the item we're going to is a clone of the item we're at, then we do
     // not need to load it again.  The current frame tree and the frame tree
@@ -806,7 +801,7 @@ bool FrameLoader::HistoryController::itemsAreClones(HistoryItem& item1, HistoryI
 }
 
 // Helper method that determines whether the current frame tree matches given history item's.
-bool FrameLoader::HistoryController::currentFramesMatchItem(HistoryItem& item) const
+bool HistoryController::currentFramesMatchItem(HistoryItem& item) const
 {
     if ((!m_frame.tree().uniqueName().isEmpty() || !item.target().isEmpty()) && m_frame.tree().uniqueName() != item.target())
         return false;
@@ -823,7 +818,7 @@ bool FrameLoader::HistoryController::currentFramesMatchItem(HistoryItem& item) c
     return true;
 }
 
-void FrameLoader::HistoryController::updateBackForwardListClippedAtTarget(bool doClip)
+void HistoryController::updateBackForwardListClippedAtTarget(bool doClip)
 {
     // In the case of saving state about a page with frames, we store a tree of items that mirrors the frame tree.  
     // The item that was the target of the user's navigation is designated as the "targetItem".  
@@ -843,13 +838,13 @@ void FrameLoader::HistoryController::updateBackForwardListClippedAtTarget(bool d
 
     FrameLoader& frameLoader = localFrame->loader();
 
-    Ref<HistoryItem> topItem = frameLoader.history().createItemTree(m_frame, doClip);
+    Ref<HistoryItem> topItem = frameLoader.history().createItemTree(page->historyItemClient(), m_frame, doClip);
     LOG(History, "HistoryController %p updateBackForwardListClippedAtTarget: Adding backforward item %p in frame %p (main frame %d) %s", this, topItem.ptr(), &m_frame, m_frame.isMainFrame(), m_frame.loader().documentLoader()->url().string().utf8().data());
 
     page->backForward().addItem(WTFMove(topItem));
 }
 
-void FrameLoader::HistoryController::updateCurrentItem()
+void HistoryController::updateCurrentItem()
 {
     if (!m_currentItem)
         return;
@@ -873,13 +868,16 @@ void FrameLoader::HistoryController::updateCurrentItem()
     }
 }
 
-void FrameLoader::HistoryController::pushState(RefPtr<SerializedScriptValue>&& stateObject, const String& title, const String& urlString)
+void HistoryController::pushState(RefPtr<SerializedScriptValue>&& stateObject, const String& title, const String& urlString)
 {
     if (!m_currentItem)
         return;
 
-    Page* page = m_frame.page();
-    ASSERT(page);
+    auto* page = m_frame.page();
+    if (!page) {
+        ASSERT_NOT_REACHED();
+        return;
+    }
 
     bool shouldRestoreScrollPosition = m_currentItem->shouldRestoreScrollPosition();
 
@@ -888,12 +886,12 @@ void FrameLoader::HistoryController::pushState(RefPtr<SerializedScriptValue>&& s
     if (!localFrame)
         return;
 
-    Ref<HistoryItem> topItem = localFrame->loader().history().createItemTree(m_frame, false);
+    Ref<HistoryItem> topItem = localFrame->loader().history().createItemTree(page->historyItemClient(), m_frame, false);
 
     auto* document = m_frame.document();
     if (document && !document->hasRecentUserInteractionForNavigationFromJS())
-        m_currentItem->setWasCreatedByJSWithoutUserInteraction(true);
-    
+        topItem->setWasCreatedByJSWithoutUserInteraction(true);
+
     // Override data in the current item (created by createItemTree) to reflect
     // the pushState() arguments.
     m_currentItem->setTitle(title);
@@ -912,7 +910,7 @@ void FrameLoader::HistoryController::pushState(RefPtr<SerializedScriptValue>&& s
     m_frame.loader().client().updateGlobalHistory();
 }
 
-void FrameLoader::HistoryController::replaceState(RefPtr<SerializedScriptValue>&& stateObject, const String& title, const String& urlString)
+void HistoryController::replaceState(RefPtr<SerializedScriptValue>&& stateObject, const String& title, const String& urlString)
 {
     if (!m_currentItem)
         return;
@@ -934,7 +932,7 @@ void FrameLoader::HistoryController::replaceState(RefPtr<SerializedScriptValue>&
     m_frame.loader().client().updateGlobalHistory();
 }
 
-void FrameLoader::HistoryController::replaceCurrentItem(HistoryItem* item)
+void HistoryController::replaceCurrentItem(HistoryItem* item)
 {
     if (!item)
         return;
@@ -946,4 +944,14 @@ void FrameLoader::HistoryController::replaceCurrentItem(HistoryItem* item)
         m_currentItem = item;
 }
 
+RefPtr<HistoryItem> HistoryController::protectedCurrentItem() const
+{
+    return m_currentItem;
+}
+
+RefPtr<HistoryItem> HistoryController::protectedProvisionalItem() const
+{
+    return m_provisionalItem;
+}
+
 } // namespace WebCore
