@@ -35,6 +35,7 @@
 #include "CachedTextTrack.h"
 #include "CrossOriginAccessControl.h"
 #include "Document.h"
+#include "HTMLMediaElement.h"
 #include "HTMLTrackElement.h"
 #include "InspectorInstrumentation.h"
 #include "Logging.h"
@@ -123,6 +124,9 @@ void TextTrackLoader::notifyFinished(CachedResource& resource, const NetworkLoad
     if (m_resource->resourceError().isAccessControl())
         corsPolicyPreventedLoad();
 
+    if (!m_resource->resourceBuffer())
+        m_state = Failed;
+
     if (m_state != Failed) {
         processNewCueData(*m_resource);
         if (m_cueParser)
@@ -198,12 +202,9 @@ Vector<Ref<VTTCue>> TextTrackLoader::getNewCues()
     if (!m_cueParser)
         return { };
 
-    auto cues = m_cueParser->takeCues();
-    Vector<Ref<VTTCue>> result;
-    result.reserveInitialCapacity(cues.size());
-    for (auto& cueData : cues)
-        result.uncheckedAppend(VTTCue::create(m_document, cueData));
-    return result;
+    return m_cueParser->takeCues().map([this](auto& cueData) {
+        return VTTCue::create(m_document, cueData);
+    });
 }
 
 Vector<Ref<VTTRegion>> TextTrackLoader::getNewRegions()
