@@ -68,7 +68,7 @@ RenderPtr<RenderElement> MathMLRowElement::createElementRenderer(RenderStyle&& s
         return createRenderer<RenderMathMLFenced>(*this, WTFMove(style));
 
     ASSERT(hasTagName(merrorTag) || hasTagName(mphantomTag) || hasTagName(mrowTag) || hasTagName(mstyleTag));
-    return createRenderer<RenderMathMLRow>(*this, WTFMove(style));
+    return createRenderer<RenderMathMLRow>(RenderObject::Type::MathMLRow, *this, WTFMove(style));
 }
 
 bool MathMLRowElement::acceptsMathVariantAttribute()
