@@ -37,7 +37,7 @@ public:
     static Ref<MathMLRowElement> create(const QualifiedName& tagName, Document&);
 
 protected:
-    constexpr static auto CreateMathMLRowElement = CreateMathMLPresentationElement;
+    static constexpr auto CreateMathMLRowElement = CreateMathMLPresentationElement;
     MathMLRowElement(const QualifiedName& tagName, Document&, ConstructionType = CreateMathMLRowElement);
     void childrenChanged(const ChildChange&) override;
 
