@@ -46,7 +46,6 @@ namespace WebCore {
 
 static RefPtr<Node> selectionShadowAncestor(LocalFrame& frame)
 {
-    ASSERT(!frame.settings().liveRangeSelectionEnabled());
     auto* node = frame.selection().selection().base().anchorNode();
     if (!node || !node->isInShadowTree())
         return nullptr;
@@ -180,8 +179,14 @@ String DOMSelection::type() const
     if (!frame)
         return "None"_s;
     auto& selection = frame->selection();
-    if (frame->settings().liveRangeSelectionEnabled())
-        return !selection.isInDocumentTree() ? "None"_s : range()->collapsed() ? "Caret"_s : "Range"_s;
+    if (frame->settings().liveRangeSelectionEnabled()) {
+        auto range = frame->selection().selection().range();
+        if (!range)
+            return "None"_s;
+        if (range->collapsed())
+            return "Caret"_s;
+        return "Range"_s;
+    }
     if (selection.isNone())
         return "None"_s;
     if (selection.isCaret())
@@ -205,8 +210,13 @@ unsigned DOMSelection::rangeCount() const
     RefPtr frame = this->frame();
     if (!frame)
         return 0;
-    if (frame->settings().liveRangeSelectionEnabled())
-        return frame->selection().associatedLiveRange() ? 1 : 0;
+    if (frame->settings().liveRangeSelectionEnabled()) {
+        if (frame->selection().associatedLiveRange())
+            return 1;
+        if (selectionShadowAncestor(*frame))
+            return 1;
+        return 0;
+    }
     return frame->selection().isNone() ? 0 : 1;
 }
 
@@ -242,7 +252,7 @@ ExceptionOr<void> DOMSelection::collapseToEnd()
         return { };
     auto& selection = frame->selection();
     if (selection.isNone())
-        return Exception { InvalidStateError };
+        return Exception { ExceptionCode::InvalidStateError };
     if (frame->settings().liveRangeSelectionEnabled()) {
         selection.disassociateLiveRange();
         selection.moveTo(selection.selection().uncanonicalizedEnd(), Affinity::Downstream);
@@ -258,7 +268,7 @@ ExceptionOr<void> DOMSelection::collapseToStart()
         return { };
     auto& selection = frame->selection();
     if (selection.isNone())
-        return Exception { InvalidStateError };
+        return Exception { ExceptionCode::InvalidStateError };
     if (frame->settings().liveRangeSelectionEnabled()) {
         selection.disassociateLiveRange();
         selection.moveTo(selection.selection().uncanonicalizedStart(), Affinity::Downstream);
@@ -280,7 +290,7 @@ ExceptionOr<void> DOMSelection::setBaseAndExtent(Node* baseNode, unsigned baseOf
     if (frame->settings().liveRangeSelectionEnabled()) {
         // FIXME: We should do this by making the arguments non-nullable in the IDL file, once liveRangeSelectionEnabled is always true.
         if (!baseNode || !extentNode)
-            return Exception { TypeError };
+            return Exception { ExceptionCode::TypeError };
         if (auto result = Range::checkNodeOffsetPair(*baseNode, baseOffset); result.hasException())
             return result.releaseException();
         if (auto result = Range::checkNodeOffsetPair(*extentNode, extentOffset); result.hasException())
@@ -363,7 +373,7 @@ ExceptionOr<void> DOMSelection::extend(Node& node, unsigned offset)
         return { };
 
     if (rangeCount() < 1 && !(frame->settings().liveRangeSelectionEnabled() && frame->selection().isCaretOrRange()))
-        return Exception { InvalidStateError, "extend() requires a Range to be added to the Selection"_s };
+        return Exception { ExceptionCode::InvalidStateError, "extend() requires a Range to be added to the Selection"_s };
 
     if (frame->settings().liveRangeSelectionEnabled()) {
         if (!(frame->settings().selectionAPIForShadowDOMEnabled() && node.isConnected() && frame->document() == &node.document())
@@ -378,7 +388,7 @@ ExceptionOr<void> DOMSelection::extend(Node& node, unsigned offset)
         selection.setSelection(newSelection);
     } else {
         if (offset > node.length())
-            return Exception { IndexSizeError };
+            return Exception { ExceptionCode::IndexSizeError };
         if (!isValidForPosition(&node))
             return { };
         frame->selection().setExtent(makeContainerOffsetPosition(&node, offset), Affinity::Downstream);
@@ -386,15 +396,26 @@ ExceptionOr<void> DOMSelection::extend(Node& node, unsigned offset)
     return { };
 }
 
+static RefPtr<Range> createLiveRangeBeforeShadowHostWithSelection(LocalFrame& frame)
+{
+    if (auto shadowAncestor = selectionShadowAncestor(frame))
+        return createLiveRange(makeSimpleRange(*makeBoundaryPointBeforeNode(*shadowAncestor)));
+
+    return nullptr;
+}
+
 ExceptionOr<Ref<Range>> DOMSelection::getRangeAt(unsigned index)
 {
     if (index >= rangeCount())
-        return Exception { IndexSizeError };
+        return Exception { ExceptionCode::IndexSizeError };
     auto frame = this->frame().releaseNonNull();
-    if (frame->settings().liveRangeSelectionEnabled())
-        return frame->selection().associatedLiveRange().releaseNonNull();
-    if (auto shadowAncestor = selectionShadowAncestor(frame))
-        return createLiveRange(makeSimpleRange(*makeBoundaryPointBeforeNode(*shadowAncestor)));
+    if (frame->settings().liveRangeSelectionEnabled()) {
+        if (auto liveRange = frame->selection().associatedLiveRange())
+            return liveRange.releaseNonNull();
+        return createLiveRangeBeforeShadowHostWithSelection(frame.get()).releaseNonNull();
+    }
+    if (auto liveRange = createLiveRangeBeforeShadowHostWithSelection(frame.get()))
+        return liveRange.releaseNonNull();
     return createLiveRange(*frame->selection().selection().firstRange());
 }
 
@@ -433,7 +454,7 @@ ExceptionOr<void> DOMSelection::removeRange(Range& liveRange)
         return { };
     ASSERT(frame->settings().liveRangeSelectionEnabled());
     if (&liveRange != frame->selection().associatedLiveRange())
-        return Exception { NotFoundError };
+        return Exception { ExceptionCode::NotFoundError };
     removeAllRanges();
     return { };
 }
@@ -529,11 +550,6 @@ RefPtr<Node> DOMSelection::shadowAdjustedNode(const Position& position) const
     if (position.isNull())
         return nullptr;
 
-    if (frame()->settings().liveRangeSelectionEnabled()) {
-        auto node = position.containerNode();
-        return !node || node->isInShadowTree() ? nullptr : node;
-    }
-
     auto* containerNode = position.containerNode();
     auto* adjustedNode = frame()->document()->ancestorNodeInThisScope(containerNode);
     if (!adjustedNode)
@@ -550,9 +566,6 @@ unsigned DOMSelection::shadowAdjustedOffset(const Position& position) const
     if (position.isNull())
         return 0;
 
-    if (frame()->settings().liveRangeSelectionEnabled())
-        return shadowAdjustedNode(position) ? position.computeOffsetInContainerNode() : 0;
-
     auto* containerNode = position.containerNode();
     auto* adjustedNode = frame()->document()->ancestorNodeInThisScope(containerNode);
     if (!adjustedNode)
