@@ -29,10 +29,11 @@
 #include "config.h"
 #include "Location.h"
 
-#include "DOMWindow.h"
 #include "Document.h"
-#include "Frame.h"
 #include "FrameLoader.h"
+#include "LocalDOMWindow.h"
+#include "LocalDOMWindowProperty.h"
+#include "LocalFrame.h"
 #include "NavigationScheduler.h"
 #include "SecurityOrigin.h"
 #include <wtf/IsoMallocInlines.h>
@@ -44,16 +45,33 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(Location);
 
 Location::Location(DOMWindow& window)
-    : DOMWindowProperty(&window)
+    : m_window(window)
 {
 }
 
-inline const URL& Location::url() const
+Frame* Location::frame()
 {
-    if (!frame())
-        return aboutBlankURL();
+    if (!m_window)
+        return nullptr;
+    return m_window->frame();
+}
 
-    const URL& url = frame()->document()->url();
+const Frame* Location::frame() const
+{
+    if (!m_window)
+        return nullptr;
+    return m_window->frame();
+}
+
+const URL& Location::url() const
+{
+    RefPtr localWindow = dynamicDowncast<LocalDOMWindow>(*m_window);
+    if (!localWindow) {
+        static NeverDestroyed<URL> nullURL;
+        return nullURL.get();
+    }
+
+    const URL& url = localWindow->document()->urlForBindings();
     if (!url.isValid())
         return aboutBlankURL(); // Use "about:blank" while the page is still loading (before we have a frame).
 
@@ -112,8 +130,10 @@ Ref<DOMStringList> Location::ancestorOrigins() const
     auto* frame = this->frame();
     if (!frame)
         return origins;
-    for (auto* ancestor = frame->tree().parent(); ancestor; ancestor = ancestor->tree().parent())
-        origins->append(ancestor->document()->securityOrigin().toString());
+    for (auto* ancestor = frame->tree().parent(); ancestor; ancestor = ancestor->tree().parent()) {
+        if (auto* localAncestor = dynamicDowncast<LocalFrame>(ancestor))
+            origins->append(localAncestor->document()->securityOrigin().toString());
+    }
     return origins;
 }
 
@@ -122,85 +142,85 @@ String Location::hash() const
     return url().fragmentIdentifier().isEmpty() ? emptyString() : url().fragmentIdentifierWithLeadingNumberSign().toString();
 }
 
-ExceptionOr<void> Location::setHref(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& url)
+ExceptionOr<void> Location::setHref(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& url)
 {
     if (!frame())
         return { };
     return setLocation(incumbentWindow, firstWindow, url);
 }
 
-ExceptionOr<void> Location::setProtocol(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& protocol)
+ExceptionOr<void> Location::setProtocol(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& protocol)
 {
-    auto* frame = this->frame();
-    if (!frame)
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame());
+    if (!localFrame)
         return { };
-    URL url = frame->document()->url();
+    URL url = localFrame->document()->url();
     if (!url.setProtocol(protocol))
-        return Exception { SyntaxError };
+        return Exception { ExceptionCode::SyntaxError };
     return setLocation(incumbentWindow, firstWindow, url.string());
 }
 
-ExceptionOr<void> Location::setHost(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& host)
+ExceptionOr<void> Location::setHost(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& host)
 {
-    auto* frame = this->frame();
-    if (!frame)
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame());
+    if (!localFrame)
         return { };
-    URL url = frame->document()->url();
+    URL url = localFrame->document()->url();
     url.setHostAndPort(host);
     return setLocation(incumbentWindow, firstWindow, url.string());
 }
 
-ExceptionOr<void> Location::setHostname(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& hostname)
+ExceptionOr<void> Location::setHostname(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& hostname)
 {
-    auto* frame = this->frame();
-    if (!frame)
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame());
+    if (!localFrame)
         return { };
-    URL url = frame->document()->url();
+    URL url = localFrame->document()->url();
     url.setHost(hostname);
     return setLocation(incumbentWindow, firstWindow, url.string());
 }
 
-ExceptionOr<void> Location::setPort(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& portString)
+ExceptionOr<void> Location::setPort(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& portString)
 {
-    auto* frame = this->frame();
-    if (!frame)
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame());
+    if (!localFrame)
         return { };
-    URL url = frame->document()->url();
+    URL url = localFrame->document()->url();
     url.setPort(parseInteger<uint16_t>(portString));
     return setLocation(incumbentWindow, firstWindow, url.string());
 }
 
-ExceptionOr<void> Location::setPathname(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& pathname)
+ExceptionOr<void> Location::setPathname(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& pathname)
 {
-    auto* frame = this->frame();
-    if (!frame)
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame());
+    if (!localFrame)
         return { };
-    URL url = frame->document()->url();
+    URL url = localFrame->document()->url();
     url.setPath(pathname);
     return setLocation(incumbentWindow, firstWindow, url.string());
 }
 
-ExceptionOr<void> Location::setSearch(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& search)
+ExceptionOr<void> Location::setSearch(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& search)
 {
-    auto* frame = this->frame();
-    if (!frame)
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame());
+    if (!localFrame)
         return { };
-    URL url = frame->document()->url();
+    URL url = localFrame->document()->url();
     url.setQuery(search);
     return setLocation(incumbentWindow, firstWindow, url.string());
 }
 
-ExceptionOr<void> Location::setHash(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& hash)
+ExceptionOr<void> Location::setHash(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& hash)
 {
-    auto* frame = this->frame();
-    if (!frame)
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame());
+    if (!localFrame)
         return { };
-    ASSERT(frame->document());
-    auto url = frame->document()->url();
+    ASSERT(localFrame->document());
+    auto url = localFrame->document()->url();
     auto oldFragmentIdentifier = url.fragmentIdentifier();
-    auto newFragmentIdentifier = hash;
-    if (hash[0] == '#')
-        newFragmentIdentifier = hash.substring(1);
+    StringView newFragmentIdentifier { hash };
+    if (hash.startsWith('#'))
+        newFragmentIdentifier = newFragmentIdentifier.substring(1);
     url.setFragmentIdentifier(newFragmentIdentifier);
     // Note that by parsing the URL and *then* comparing fragments, we are 
     // comparing fragments post-canonicalization, and so this handles the 
@@ -210,49 +230,48 @@ ExceptionOr<void> Location::setHash(DOMWindow& incumbentWindow, DOMWindow& first
     return setLocation(incumbentWindow, firstWindow, url.string());
 }
 
-ExceptionOr<void> Location::assign(DOMWindow& activeWindow, DOMWindow& firstWindow, const String& url)
+ExceptionOr<void> Location::assign(LocalDOMWindow& activeWindow, LocalDOMWindow& firstWindow, const String& url)
 {
     if (!frame())
         return { };
     return setLocation(activeWindow, firstWindow, url);
 }
 
-ExceptionOr<void> Location::replace(DOMWindow& activeWindow, DOMWindow& firstWindow, const String& urlString)
+ExceptionOr<void> Location::replace(LocalDOMWindow& activeWindow, LocalDOMWindow& firstWindow, const String& urlString)
 {
-    auto* frame = this->frame();
+    RefPtr frame = this->frame();
     if (!frame)
         return { };
-    ASSERT(frame->document());
-    ASSERT(frame->document()->domWindow());
+    ASSERT(frame->window());
 
-    Frame* firstFrame = firstWindow.frame();
+    auto* firstFrame = firstWindow.frame();
     if (!firstFrame || !firstFrame->document())
         return { };
 
     URL completedURL = firstFrame->document()->completeURL(urlString);
     if (!completedURL.isValid())
-        return Exception { SyntaxError };
+        return Exception { ExceptionCode::SyntaxError };
 
-    if (!activeWindow.document()->canNavigate(frame, completedURL))
-        return Exception { SecurityError };
+    if (!activeWindow.document()->canNavigate(dynamicDowncast<LocalFrame>(frame.get()), completedURL))
+        return Exception { ExceptionCode::SecurityError };
 
-    // We call DOMWindow::setLocation directly here because replace() always operates on the current frame.
-    frame->document()->domWindow()->setLocation(activeWindow, completedURL, LockHistoryAndBackForwardList);
+    // We call LocalDOMWindow::setLocation directly here because replace() always operates on the current frame.
+    frame->window()->setLocation(activeWindow, completedURL, SetLocationLocking::LockHistoryAndBackForwardList);
     return { };
 }
 
-void Location::reload(DOMWindow& activeWindow)
+void Location::reload(LocalDOMWindow& activeWindow)
 {
-    auto* frame = this->frame();
-    if (!frame)
+    RefPtr localFrame = dynamicDowncast<LocalFrame>(frame());
+    if (!localFrame)
         return;
 
     ASSERT(activeWindow.document());
-    ASSERT(frame->document());
-    ASSERT(frame->document()->domWindow());
+    ASSERT(localFrame->document());
+    ASSERT(localFrame->document()->domWindow());
 
     auto& activeDocument = *activeWindow.document();
-    auto& targetDocument = *frame->document();
+    auto& targetDocument = *localFrame->document();
 
     // FIXME: It's not clear this cross-origin security check is valuable.
     // We allow one page to change the location of another. Why block attempts to reload?
@@ -266,30 +285,34 @@ void Location::reload(DOMWindow& activeWindow)
     if (targetDocument.url().protocolIsJavaScript())
         return;
 
-    frame->navigationScheduler().scheduleRefresh(activeDocument);
+    localFrame->navigationScheduler().scheduleRefresh(activeDocument);
 }
 
-ExceptionOr<void> Location::setLocation(DOMWindow& incumbentWindow, DOMWindow& firstWindow, const String& urlString)
+ExceptionOr<void> Location::setLocation(LocalDOMWindow& incumbentWindow, LocalDOMWindow& firstWindow, const String& urlString)
 {
-    auto* frame = this->frame();
+    RefPtr frame = this->frame();
     ASSERT(frame);
 
-    Frame* firstFrame = firstWindow.frame();
+    auto* firstFrame = firstWindow.frame();
     if (!firstFrame || !firstFrame->document())
         return { };
 
     URL completedURL = firstFrame->document()->completeURL(urlString);
 
     if (!completedURL.isValid())
-        return Exception { TypeError, "Invalid URL"_s };
+        return Exception { ExceptionCode::SyntaxError, "Invalid URL"_s };
 
-    if (!incumbentWindow.document()->canNavigate(frame, completedURL))
-        return Exception { SecurityError };
+    if (!incumbentWindow.document()->canNavigate(dynamicDowncast<LocalFrame>(frame.get()), completedURL))
+        return Exception { ExceptionCode::SecurityError };
 
-    ASSERT(frame->document());
-    ASSERT(frame->document()->domWindow());
-    frame->document()->domWindow()->setLocation(incumbentWindow, completedURL);
+    ASSERT(frame->window());
+    frame->window()->setLocation(incumbentWindow, completedURL);
     return { };
 }
 
+RefPtr<DOMWindow> Location::protectedWindow()
+{
+    return m_window.get();
+}
+
 } // namespace WebCore
