@@ -40,6 +40,7 @@
 #include "HRTFElevation.h"
 #include "HTMLMediaElement.h"
 #include "HTMLNameCache.h"
+#include "ImmutableStyleProperties.h"
 #include "InlineStyleSheetOwner.h"
 #include "InspectorInstrumentation.h"
 #include "LayoutIntegrationLineLayout.h"
@@ -49,6 +50,8 @@
 #include "Page.h"
 #include "PerformanceLogging.h"
 #include "RenderTheme.h"
+#include "RenderView.h"
+#include "SVGPathElement.h"
 #include "ScrollingThread.h"
 #include "SelectorQuery.h"
 #include "StyleScope.h"
@@ -77,8 +80,8 @@ static void releaseNoncriticalMemory(MaintainMemoryCache maintainMemoryCache)
     GlyphDisplayListCache::singleton().clear();
     SelectorQueryCache::singleton().clear();
 
-    for (auto* document : Document::allDocuments()) {
-        if (auto* renderView = document->renderView())
+    for (auto& document : Document::allDocuments()) {
+        if (CheckedPtr renderView = document->renderView())
             LayoutIntegration::LineLayout::releaseCaches(*renderView);
     }
 
@@ -87,6 +90,8 @@ static void releaseNoncriticalMemory(MaintainMemoryCache maintainMemoryCache)
 
     InlineStyleSheetOwner::clearCache();
     HTMLNameCache::clear();
+    ImmutableStyleProperties::clearDeduplicationMap();
+    SVGPathElement::clearCache();
 }
 
 static void releaseCriticalMemory(Synchronous synchronous, MaintainBackForwardCache maintainBackForwardCache, MaintainMemoryCache maintainMemoryCache)
@@ -111,7 +116,11 @@ static void releaseCriticalMemory(Synchronous synchronous, MaintainBackForwardCa
         page.cookieJar().clearCache();
     });
 
-    for (auto& document : copyToVectorOf<RefPtr<Document>>(Document::allDocuments())) {
+    auto allDocuments = Document::allDocuments();
+    auto protectedDocuments = WTF::map(allDocuments, [](auto& document) -> Ref<Document> {
+        return document.get();
+    });
+    for (auto& document : protectedDocuments) {
         document->styleScope().releaseMemory();
         document->fontSelector().emptyCaches();
         document->cachedResourceLoader().garbageCollectDocumentResources();
