@@ -34,11 +34,13 @@ Animation::Animation()
     , m_iterationCount(initialIterationCount())
     , m_delay(initialDelay())
     , m_duration(initialDuration())
+    , m_timeline(initialTimeline())
     , m_timingFunction(initialTimingFunction())
     , m_direction(static_cast<unsigned>(initialDirection()))
     , m_fillMode(static_cast<unsigned>(initialFillMode()))
     , m_playState(static_cast<unsigned>(initialPlayState()))
     , m_compositeOperation(static_cast<unsigned>(initialCompositeOperation()))
+    , m_allowsDiscreteTransitions(initialAllowsDiscreteTransitions())
     , m_delaySet(false)
     , m_directionSet(false)
     , m_durationSet(false)
@@ -47,8 +49,10 @@ Animation::Animation()
     , m_nameSet(false)
     , m_playStateSet(false)
     , m_propertySet(false)
+    , m_timelineSet(false)
     , m_timingFunctionSet(false)
     , m_compositeOperationSet(false)
+    , m_allowsDiscreteTransitionsSet(false)
     , m_isNone(false)
     , m_delayFilled(false)
     , m_directionFilled(false)
@@ -57,8 +61,10 @@ Animation::Animation()
     , m_iterationCountFilled(false)
     , m_playStateFilled(false)
     , m_propertyFilled(false)
+    , m_timelineFilled(false)
     , m_timingFunctionFilled(false)
     , m_compositeOperationFilled(false)
+    , m_allowsDiscreteTransitionsFilled(false)
 {
 }
 
@@ -69,12 +75,13 @@ Animation::Animation(const Animation& o)
     , m_iterationCount(o.m_iterationCount)
     , m_delay(o.m_delay)
     , m_duration(o.m_duration)
+    , m_timeline(o.m_timeline)
     , m_timingFunction(o.m_timingFunction)
-    , m_nameStyleScopeOrdinal(o.m_nameStyleScopeOrdinal)
     , m_direction(o.m_direction)
     , m_fillMode(o.m_fillMode)
     , m_playState(o.m_playState)
     , m_compositeOperation(o.m_compositeOperation)
+    , m_allowsDiscreteTransitions(o.m_allowsDiscreteTransitions)
     , m_delaySet(o.m_delaySet)
     , m_directionSet(o.m_directionSet)
     , m_durationSet(o.m_durationSet)
@@ -83,8 +90,10 @@ Animation::Animation(const Animation& o)
     , m_nameSet(o.m_nameSet)
     , m_playStateSet(o.m_playStateSet)
     , m_propertySet(o.m_propertySet)
+    , m_timelineSet(o.m_timelineSet)
     , m_timingFunctionSet(o.m_timingFunctionSet)
     , m_compositeOperationSet(o.m_compositeOperationSet)
+    , m_allowsDiscreteTransitionsSet(o.m_allowsDiscreteTransitionsSet)
     , m_isNone(o.m_isNone)
     , m_delayFilled(o.m_delayFilled)
     , m_directionFilled(o.m_directionFilled)
@@ -93,8 +102,10 @@ Animation::Animation(const Animation& o)
     , m_iterationCountFilled(o.m_iterationCountFilled)
     , m_playStateFilled(o.m_playStateFilled)
     , m_propertyFilled(o.m_propertyFilled)
+    , m_timelineFilled(o.m_timelineFilled)
     , m_timingFunctionFilled(o.m_timingFunctionFilled)
     , m_compositeOperationFilled(o.m_compositeOperationFilled)
+    , m_allowsDiscreteTransitionsFilled(o.m_allowsDiscreteTransitionsFilled)
 {
 }
 
@@ -102,15 +113,16 @@ Animation::~Animation() = default;
 
 bool Animation::animationsMatch(const Animation& other, bool matchProperties) const
 {
-    bool result = m_name.string == other.m_name.string
+    bool result = m_name == other.m_name
         && m_playState == other.m_playState
         && m_compositeOperation == other.m_compositeOperation
+        && m_allowsDiscreteTransitions == other.m_allowsDiscreteTransitions
         && m_playStateSet == other.m_playStateSet
         && m_iterationCount == other.m_iterationCount
         && m_delay == other.m_delay
         && m_duration == other.m_duration
+        && m_timeline == other.m_timeline
         && *(m_timingFunction.get()) == *(other.m_timingFunction.get())
-        && m_nameStyleScopeOrdinal == other.m_nameStyleScopeOrdinal
         && m_direction == other.m_direction
         && m_fillMode == other.m_fillMode
         && m_delaySet == other.m_delaySet
@@ -119,8 +131,10 @@ bool Animation::animationsMatch(const Animation& other, bool matchProperties) co
         && m_fillModeSet == other.m_fillModeSet
         && m_iterationCountSet == other.m_iterationCountSet
         && m_nameSet == other.m_nameSet
+        && m_timelineSet == other.m_timelineSet
         && m_timingFunctionSet == other.m_timingFunctionSet
         && m_compositeOperationSet == other.m_compositeOperationSet
+        && m_allowsDiscreteTransitionsSet == other.m_allowsDiscreteTransitionsSet
         && m_isNone == other.m_isNone;
 
     if (!result)
@@ -129,9 +143,9 @@ bool Animation::animationsMatch(const Animation& other, bool matchProperties) co
     return !matchProperties || (m_property.mode == other.m_property.mode && m_property.animatableProperty == other.m_property.animatableProperty && m_propertySet == other.m_propertySet);
 }
 
-auto Animation::initialName() -> const Name&
+auto Animation::initialName() -> const Style::ScopedName&
 {
-    static NeverDestroyed<Name> initialValue { Name { noneAtom(), true } };
+    static NeverDestroyed<Style::ScopedName> initialValue { Style::ScopedName { noneAtom() } };
     return initialValue;
 }
 
@@ -157,13 +171,31 @@ TextStream& operator<<(TextStream& ts, Animation::Direction direction)
     return ts;
 }
 
+TextStream& operator<<(TextStream& ts, const Animation::Timeline& timeline)
+{
+    WTF::switchOn(timeline,
+        [&] (Animation::TimelineKeyword keyword) {
+            switch (keyword) {
+            case Animation::TimelineKeyword::Auto: ts << "auto"; break;
+            case Animation::TimelineKeyword::None: ts << "none"; break;
+            }
+        }, [&] (const AtomString& customIdent) {
+            ts << customIdent;
+        }, [&] (const Ref<ScrollTimeline>& scrollTimeline) {
+            ts << scrollTimeline->toCSSValue()->cssText();
+        }
+    );
+    return ts;
+}
+
 TextStream& operator<<(TextStream& ts, const Animation& animation)
 {
     ts.dumpProperty("property", animation.property());
-    ts.dumpProperty("name", animation.name().string);
+    ts.dumpProperty("name", animation.name().name);
     ts.dumpProperty("iteration count", animation.iterationCount());
     ts.dumpProperty("delay", animation.iterationCount());
     ts.dumpProperty("duration", animation.duration());
+    ts.dumpProperty("timeline", animation.timeline());
     if (animation.timingFunction())
         ts.dumpProperty("timing function", *animation.timingFunction());
     ts.dumpProperty("direction", animation.direction());
