@@ -253,7 +253,7 @@ GstFlowReturn AudioFileReader::handleSample(GstAppSink* sink)
 
 void AudioFileReader::handleMessage(GstMessage* message)
 {
-    ASSERT(&m_runLoop == &RunLoop::current());
+    assertIsCurrent(m_runLoop);
 
     GUniqueOutPtr<GError> error;
     GUniqueOutPtr<gchar> debug;
@@ -389,7 +389,7 @@ void AudioFileReader::plugDeinterleave(GstPad* pad)
 
 void AudioFileReader::decodeAudioForBusCreation()
 {
-    ASSERT(&m_runLoop == &RunLoop::current());
+    assertIsCurrent(m_runLoop);
 
     // Build the pipeline giostreamsrc ! decodebin
     // A deinterleave element is added once a src pad becomes available in decodebin.
@@ -400,7 +400,7 @@ void AudioFileReader::decodeAudioForBusCreation()
     ASSERT(bus);
     gst_bus_set_sync_handler(bus.get(), [](GstBus*, GstMessage* message, gpointer userData) {
         auto& reader = *static_cast<AudioFileReader*>(userData);
-        if (&reader.m_runLoop == &RunLoop::current())
+        if (reader.m_runLoop.isCurrent())
             reader.handleMessage(message);
         else {
             GRefPtr<GstMessage> protectMessage(message);
