@@ -37,7 +37,7 @@
 namespace WebCore {
 
 Path::Path(const Vector<FloatPoint>& points)
-    : m_data(PathStream::create(points))
+    : m_data(DataRef<PathImpl> { PathStream::create(points) })
 {
 }
 
@@ -49,10 +49,10 @@ Path::Path(Vector<PathSegment>&& segments)
     if (segments.size() == 1)
         m_data = WTFMove(segments[0]);
     else
-        m_data = PathStream::create(WTFMove(segments));
+        m_data = DataRef<PathImpl> { PathStream::create(WTFMove(segments)) };
 }
 
-Path::Path(UniqueRef<PathImpl>&& impl)
+Path::Path(Ref<PathImpl>&& impl)
     : m_data(WTFMove(impl))
 {
 }
@@ -67,35 +67,7 @@ Path::Path(PathSegment&& segment)
     m_data = WTFMove(segment);
 }
 
-Path& Path::operator=(const Path& other)
-{
-    if (auto segment = other.asSingle())
-        m_data = *segment;
-    else if (auto impl = other.asImpl())
-        m_data = impl->clone();
-    else
-        m_data = std::monostate { };
-    return *this;
-}
-
-bool Path::operator==(const Path& other) const
-{
-    if (auto segment = asSingle()) {
-        if (auto otherSegment = other.asSingle())
-            return *segment == *otherSegment;
-        return false;
-    }
-
-    if (auto impl = asImpl()) {
-        if (auto otherImpl = other.asImpl())
-            return *impl == *otherImpl;
-        return false;
-    }
-
-    return true;
-}
-
-PathImpl& Path::setImpl(UniqueRef<PathImpl> impl)
+PathImpl& Path::setImpl(Ref<PathImpl>&& impl)
 {
     auto& platformPathImpl = impl.get();
     m_data = WTFMove(impl);
@@ -129,29 +101,24 @@ PathImpl& Path::ensureImpl()
 
 PathImpl* Path::asImpl()
 {
-    if (auto ref = std::get_if<UniqueRef<PathImpl>>(&m_data))
-        return ref->ptr();
+    if (auto ref = std::get_if<DataRef<PathImpl>>(&m_data))
+        return &ref->access();
     return nullptr;
 }
 
 const PathImpl* Path::asImpl() const
 {
-    if (auto ref = std::get_if<UniqueRef<PathImpl>>(&m_data))
+    if (auto ref = std::get_if<DataRef<PathImpl>>(&m_data))
         return ref->ptr();
     return nullptr;
 }
 
-Path Path::polygonPathFromPoints(const Vector<FloatPoint>& points)
-{
-    return Path(points);
-}
-
 void Path::moveTo(const FloatPoint& point)
 {
     if (isEmpty())
         m_data = PathSegment(PathMoveTo { point });
     else
-        ensureImpl().moveTo(point);
+        ensureImpl().add(PathMoveTo { point });
 }
 
 const PathMoveTo* Path::asSingleMoveTo() const
@@ -168,7 +135,7 @@ void Path::addLineTo(const FloatPoint& point)
     else if (auto moveTo = asSingleMoveTo())
         m_data = PathSegment(PathDataLine { moveTo->point, point });
     else
-        ensureImpl().addLineTo(point);
+        ensureImpl().add(PathLineTo { point });
 }
 
 void Path::addQuadCurveTo(const FloatPoint& controlPoint, const FloatPoint& endPoint)
@@ -178,7 +145,7 @@ void Path::addQuadCurveTo(const FloatPoint& controlPoint, const FloatPoint& endP
     else if (auto moveTo = asSingleMoveTo())
         m_data = PathSegment(PathDataQuadCurve { moveTo->point, controlPoint, endPoint });
     else
-        ensureImpl().addQuadCurveTo(controlPoint, endPoint);
+        ensureImpl().add(PathQuadCurveTo { controlPoint, endPoint });
 }
 
 void Path::addBezierCurveTo(const FloatPoint& controlPoint1, const FloatPoint& controlPoint2, const FloatPoint& endPoint)
@@ -188,7 +155,7 @@ void Path::addBezierCurveTo(const FloatPoint& controlPoint1, const FloatPoint& c
     else if (auto moveTo = asSingleMoveTo())
         m_data = PathSegment(PathDataBezierCurve { moveTo->point, controlPoint1, controlPoint2, endPoint });
     else
-        ensureImpl().addBezierCurveTo(controlPoint1, controlPoint2, endPoint);
+        ensureImpl().add(PathBezierCurveTo { controlPoint1, controlPoint2, endPoint });
 }
 
 void Path::addArcTo(const FloatPoint& point1, const FloatPoint& point2, float radius)
@@ -198,7 +165,7 @@ void Path::addArcTo(const FloatPoint& point1, const FloatPoint& point2, float ra
     else if (auto moveTo = asSingleMoveTo())
         m_data = PathSegment(PathDataArc { moveTo->point, point1, point2, radius });
     else
-        ensureImpl().addArcTo(point1, point2, radius);
+        ensureImpl().add(PathArcTo { point1, point2, radius });
 }
 
 void Path::addArc(const FloatPoint& point, float radius, float startAngle, float endAngle, RotationDirection direction)
@@ -212,7 +179,7 @@ void Path::addArc(const FloatPoint& point, float radius, float startAngle, float
     if (isEmpty())
         m_data = PathSegment(PathArc { point, radius, startAngle, endAngle, direction });
     else
-        ensureImpl().addArc(point, radius, startAngle, endAngle, direction);
+        ensureImpl().add(PathArc { point, radius, startAngle, endAngle, direction });
 }
 
 void Path::addEllipse(const FloatPoint& point, float radiusX, float radiusY, float rotation, float startAngle, float endAngle, RotationDirection direction)
@@ -220,7 +187,7 @@ void Path::addEllipse(const FloatPoint& point, float radiusX, float radiusY, flo
     if (isEmpty())
         m_data = PathSegment(PathEllipse { point, radiusX, radiusY, rotation, startAngle, endAngle, direction });
     else
-        ensureImpl().addEllipse(point, radiusX, radiusY, rotation, startAngle, endAngle, direction);
+        ensureImpl().add(PathEllipse { point, radiusX, radiusY, rotation, startAngle, endAngle, direction });
 }
 
 void Path::addEllipseInRect(const FloatRect& rect)
@@ -228,7 +195,7 @@ void Path::addEllipseInRect(const FloatRect& rect)
     if (isEmpty())
         m_data = PathSegment(PathEllipseInRect { rect });
     else
-        ensureImpl().addEllipseInRect(rect);
+        ensureImpl().add(PathEllipseInRect { rect });
 }
 
 void Path::addRect(const FloatRect& rect)
@@ -236,7 +203,7 @@ void Path::addRect(const FloatRect& rect)
     if (isEmpty())
         m_data = PathSegment(PathRect { rect });
     else
-        ensureImpl().addRect(rect);
+        ensureImpl().add(PathRect { rect });
 }
 
 static FloatRoundedRect calculateEvenRoundedRect(const FloatRect& rect, const FloatSize& roundingRadii)
@@ -278,7 +245,7 @@ void Path::addRoundedRect(const FloatRoundedRect& roundedRect, PathRoundedRect::
     if (isEmpty())
         m_data = PathSegment(PathRoundedRect { roundedRect, strategy });
     else
-        ensureImpl().addRoundedRect(roundedRect, strategy);
+        ensureImpl().add(PathRoundedRect { roundedRect, strategy });
 }
 
 void Path::addRoundedRect(const FloatRect& rect, const FloatSize& roundingRadii, PathRoundedRect::Strategy strategy)
@@ -289,7 +256,7 @@ void Path::addRoundedRect(const FloatRect& rect, const FloatSize& roundingRadii,
     if (isEmpty())
         m_data = PathSegment(PathRoundedRect { calculateEvenRoundedRect(rect, roundingRadii), strategy });
     else
-        ensureImpl().addRoundedRect(calculateEvenRoundedRect(rect, roundingRadii), strategy);
+        ensureImpl().add(PathRoundedRect { calculateEvenRoundedRect(rect, roundingRadii), strategy });
 }
 
 void Path::addRoundedRect(const RoundedRect& rect)
@@ -302,7 +269,7 @@ void Path::closeSubpath()
     if (isEmpty() || isClosed())
         return;
 
-    ensureImpl().closeSubpath();
+    ensureImpl().add(PathCloseSubpath { });
 }
 
 void Path::addPath(const Path& path, const AffineTransform& transform)
@@ -437,6 +404,11 @@ bool Path::isEmpty() const
     return false;
 }
 
+bool Path::definitelySingleLine() const
+{
+    return !!singleDataLine();
+}
+
 PlatformPathPtr Path::platformPath() const
 {
     return const_cast<Path&>(*this).ensurePlatformPathImpl().platformPath();
@@ -531,6 +503,17 @@ bool Path::strokeContains(const FloatPoint& point, const Function<void(GraphicsC
     return const_cast<Path&>(*this).ensurePlatformPathImpl().strokeContains(point, strokeStyleApplier);
 }
 
+bool Path::hasSubpaths() const
+{
+    if (auto segment = asSingle())
+        return PathStream::computeHasSubpaths({ segment, 1 });
+
+    if (auto impl = asImpl())
+        return impl->hasSubpaths();
+
+    return false;
+}
+
 FloatRect Path::fastBoundingRect() const
 {
     if (auto segment = asSingle())
