@@ -27,7 +27,6 @@
 #include "PathSegmentData.h"
 
 #include "AffineTransform.h"
-#include "PathImpl.h"
 #include <wtf/text/TextStream.h>
 
 namespace WebCore {
@@ -38,6 +37,12 @@ FloatPoint PathMoveTo::calculateEndPoint(const FloatPoint&, FloatPoint& lastMove
     return lastMoveToPoint;
 }
 
+std::optional<FloatPoint> PathMoveTo::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathMoveTo::extendFastBoundingRect(const FloatPoint&, const FloatPoint&, FloatRect&) const
 {
 }
@@ -46,11 +51,6 @@ void PathMoveTo::extendBoundingRect(const FloatPoint&, const FloatPoint&, FloatR
 {
 }
 
-void PathMoveTo::addToImpl(PathImpl& impl) const
-{
-    impl.moveTo(point);
-}
-
 void PathMoveTo::applyElements(const PathElementApplier& applier) const
 {
     applier({ PathElement::Type::MoveToPoint, { point } });
@@ -72,6 +72,12 @@ FloatPoint PathLineTo::calculateEndPoint(const FloatPoint&, FloatPoint&) const
     return point;
 }
 
+std::optional<FloatPoint> PathLineTo::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathLineTo::extendFastBoundingRect(const FloatPoint& currentPoint, const FloatPoint& lastMoveToPoint, FloatRect& boundingRect) const
 {
     extendBoundingRect(currentPoint, lastMoveToPoint, boundingRect);
@@ -83,11 +89,6 @@ void PathLineTo::extendBoundingRect(const FloatPoint& currentPoint, const FloatP
     boundingRect.extend(point);
 }
 
-void PathLineTo::addToImpl(PathImpl& impl) const
-{
-    impl.addLineTo(point);
-}
-
 void PathLineTo::applyElements(const PathElementApplier& applier) const
 {
     applier({ PathElement::Type::AddLineToPoint, { point } });
@@ -109,6 +110,12 @@ FloatPoint PathQuadCurveTo::calculateEndPoint(const FloatPoint&, FloatPoint&) co
     return endPoint;
 }
 
+std::optional<FloatPoint> PathQuadCurveTo::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathQuadCurveTo::extendFastBoundingRect(const FloatPoint& currentPoint, const FloatPoint&, FloatRect& boundingRect) const
 {
     boundingRect.extend(currentPoint);
@@ -158,11 +165,6 @@ void PathQuadCurveTo::extendBoundingRect(const FloatPoint& currentPoint, const F
     boundingRect.extend(endPoint);
 }
 
-void PathQuadCurveTo::addToImpl(PathImpl& impl) const
-{
-    impl.addQuadCurveTo(controlPoint, endPoint);
-}
-
 void PathQuadCurveTo::applyElements(const PathElementApplier& applier) const
 {
     applier({ PathElement::Type::AddQuadCurveToPoint, { controlPoint, endPoint } });
@@ -185,6 +187,12 @@ FloatPoint PathBezierCurveTo::calculateEndPoint(const FloatPoint&, FloatPoint&)
     return endPoint;
 }
 
+std::optional<FloatPoint> PathBezierCurveTo::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathBezierCurveTo::extendFastBoundingRect(const FloatPoint& currentPoint, const FloatPoint&, FloatRect& boundingRect) const
 {
     boundingRect.extend(currentPoint);
@@ -271,11 +279,6 @@ void PathBezierCurveTo::extendBoundingRect(const FloatPoint& currentPoint, const
     boundingRect.extend(endPoint);
 }
 
-void PathBezierCurveTo::addToImpl(PathImpl& impl) const
-{
-    impl.addBezierCurveTo(controlPoint1, controlPoint2, endPoint);
-}
-
 void PathBezierCurveTo::applyElements(const PathElementApplier& applier) const
 {
     applier({ PathElement::Type::AddCurveToPoint, { controlPoint1, controlPoint2, endPoint } });
@@ -322,6 +325,11 @@ FloatPoint PathArcTo::calculateEndPoint(const FloatPoint& currentPoint, FloatPoi
     return calculateArcToEndPoint(currentPoint, controlPoint1, controlPoint2, radius);
 }
 
+std::optional<FloatPoint> PathArcTo::tryGetEndPointWithoutContext() const
+{
+    return std::nullopt;
+}
+
 void PathArcTo::extendFastBoundingRect(const FloatPoint& currentPoint, const FloatPoint&, FloatRect& boundingRect) const
 {
     boundingRect.extend(currentPoint);
@@ -336,11 +344,6 @@ void PathArcTo::extendBoundingRect(const FloatPoint& currentPoint, const FloatPo
     boundingRect.extend(calculateArcToEndPoint(currentPoint, controlPoint1, controlPoint2, radius));
 }
 
-void PathArcTo::addToImpl(PathImpl& impl) const
-{
-    impl.addArcTo(controlPoint1, controlPoint2, radius);
-}
-
 WTF::TextStream& operator<<(WTF::TextStream& ts, const PathArcTo& data)
 {
     ts << "add arc to " << data.controlPoint1 << " " << data.controlPoint2 << " " << data.radius;
@@ -353,6 +356,12 @@ FloatPoint PathArc::calculateEndPoint(const FloatPoint&, FloatPoint& lastMoveToP
     return center + FloatSize { radius * cos(endAngle), - radius * sin(endAngle) };
 }
 
+std::optional<FloatPoint> PathArc::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathArc::extendFastBoundingRect(const FloatPoint&, const FloatPoint&, FloatRect& boundingRect) const
 {
     auto minXMinYCorner = center - FloatSize { radius, radius };
@@ -412,11 +421,6 @@ void PathArc::extendBoundingRect(const FloatPoint&, const FloatPoint&, FloatRect
     boundingRect.extend({ x2, y2 });
 }
 
-void PathArc::addToImpl(PathImpl& impl) const
-{
-    impl.addArc(center, radius, startAngle, endAngle, direction);
-}
-
 WTF::TextStream& operator<<(WTF::TextStream& ts, const PathArc& data)
 {
     ts << "add arc " << data.center << " " << data.radius  << " " << data.startAngle  << " " << data.endAngle  << " " << data.direction;
@@ -435,6 +439,12 @@ FloatPoint PathEllipse::calculateEndPoint(const FloatPoint&, FloatPoint& lastMov
     return rotation.mapPoint(endPoint);
 }
 
+std::optional<FloatPoint> PathEllipse::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathEllipse::extendFastBoundingRect(const FloatPoint&, const FloatPoint&, FloatRect& boundingRect) const
 {
     auto minXMinYCorner = center - FloatSize { radiusX, radiusY };
@@ -460,11 +470,6 @@ void PathEllipse::extendBoundingRect(const FloatPoint& currentPoint, const Float
     boundingRect.extend(currentPoint);
 }
 
-void PathEllipse::addToImpl(PathImpl& impl) const
-{
-    impl.addEllipse(center, radiusX, radiusY, rotation, startAngle, endAngle, direction);
-}
-
 WTF::TextStream& operator<<(WTF::TextStream& ts, const PathEllipse& data)
 {
     ts << "add ellipse " << data.center << " " << data.radiusX << " " << data.radiusY  << " " << data.rotation << " " << data.startAngle  << " " << data.endAngle  << " " << data.direction;
@@ -477,6 +482,12 @@ FloatPoint PathEllipseInRect::calculateEndPoint(const FloatPoint&, FloatPoint& l
     return lastMoveToPoint;
 }
 
+std::optional<FloatPoint> PathEllipseInRect::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathEllipseInRect::extendFastBoundingRect(const FloatPoint& currentPoint, const FloatPoint& lastMoveToPoint, FloatRect& boundingRect) const
 {
     extendBoundingRect(currentPoint, lastMoveToPoint, boundingRect);
@@ -488,11 +499,6 @@ void PathEllipseInRect::extendBoundingRect(const FloatPoint&, const FloatPoint&,
     boundingRect.extend(rect.maxXMaxYCorner());
 }
 
-void PathEllipseInRect::addToImpl(PathImpl& impl) const
-{
-    impl.addEllipseInRect(rect);
-}
-
 WTF::TextStream& operator<<(WTF::TextStream& ts, const PathEllipseInRect& data)
 {
     ts << "add ellipse in rect " << data.rect;
@@ -505,6 +511,12 @@ FloatPoint PathRect::calculateEndPoint(const FloatPoint&, FloatPoint& lastMoveTo
     return lastMoveToPoint;
 }
 
+std::optional<FloatPoint> PathRect::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathRect::extendFastBoundingRect(const FloatPoint& currentPoint, const FloatPoint& lastMoveToPoint, FloatRect& boundingRect) const
 {
     extendBoundingRect(currentPoint, lastMoveToPoint, boundingRect);
@@ -516,11 +528,6 @@ void PathRect::extendBoundingRect(const FloatPoint&, const FloatPoint&, FloatRec
     boundingRect.extend(rect.maxXMaxYCorner());
 }
 
-void PathRect::addToImpl(PathImpl& impl) const
-{
-    impl.addRect(rect);
-}
-
 WTF::TextStream& operator<<(WTF::TextStream& ts, const PathRect& data)
 {
     ts << "add rect " << data.rect;
@@ -533,6 +540,12 @@ FloatPoint PathRoundedRect::calculateEndPoint(const FloatPoint&, FloatPoint& las
     return lastMoveToPoint;
 }
 
+std::optional<FloatPoint> PathRoundedRect::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathRoundedRect::extendFastBoundingRect(const FloatPoint& currentPoint, const FloatPoint& lastMoveToPoint, FloatRect& boundingRect) const
 {
     extendBoundingRect(currentPoint, lastMoveToPoint, boundingRect);
@@ -544,11 +557,6 @@ void PathRoundedRect::extendBoundingRect(const FloatPoint&, const FloatPoint&, F
     boundingRect.extend(roundedRect.rect().maxXMaxYCorner());
 }
 
-void PathRoundedRect::addToImpl(PathImpl& impl) const
-{
-    impl.addRoundedRect(roundedRect, strategy);
-}
-
 WTF::TextStream& operator<<(WTF::TextStream& ts, const PathRoundedRect& data)
 {
     ts << "add rounded rect " << data.roundedRect;
@@ -561,6 +569,12 @@ FloatPoint PathDataLine::calculateEndPoint(const FloatPoint&, FloatPoint& lastMo
     return end;
 }
 
+std::optional<FloatPoint> PathDataLine::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathDataLine::extendFastBoundingRect(const FloatPoint& currentPoint, const FloatPoint& lastMoveToPoint, FloatRect& boundingRect) const
 {
     extendBoundingRect(currentPoint, lastMoveToPoint, boundingRect);
@@ -572,12 +586,6 @@ void PathDataLine::extendBoundingRect(const FloatPoint&, const FloatPoint&, Floa
     boundingRect.extend(end);
 }
 
-void PathDataLine::addToImpl(PathImpl& impl) const
-{
-    impl.moveTo(start);
-    impl.addLineTo(end);
-}
-
 void PathDataLine::applyElements(const PathElementApplier& applier) const
 {
     applier({ PathElement::Type::MoveToPoint, { start } });
@@ -604,6 +612,12 @@ FloatPoint PathDataQuadCurve::calculateEndPoint(const FloatPoint&, FloatPoint& l
     return endPoint;
 }
 
+std::optional<FloatPoint> PathDataQuadCurve::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathDataQuadCurve::extendFastBoundingRect(const FloatPoint&, const FloatPoint&, FloatRect& boundingRect) const
 {
     boundingRect.extend(start);
@@ -619,12 +633,6 @@ void PathDataQuadCurve::extendBoundingRect(const FloatPoint&, const FloatPoint&,
     boundingRect.extend(endPoint);
 }
 
-void PathDataQuadCurve::addToImpl(PathImpl& impl) const
-{
-    impl.moveTo(start);
-    impl.addQuadCurveTo(controlPoint, endPoint);
-}
-
 void PathDataQuadCurve::applyElements(const PathElementApplier& applier) const
 {
     applier({ PathElement::Type::MoveToPoint, { start } });
@@ -652,6 +660,12 @@ FloatPoint PathDataBezierCurve::calculateEndPoint(const FloatPoint&, FloatPoint&
     return endPoint;
 }
 
+std::optional<FloatPoint> PathDataBezierCurve::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathDataBezierCurve::extendFastBoundingRect(const FloatPoint&, const FloatPoint&, FloatRect& boundingRect) const
 {
     boundingRect.extend(start);
@@ -669,12 +683,6 @@ void PathDataBezierCurve::extendBoundingRect(const FloatPoint&, const FloatPoint
     boundingRect.extend(endPoint);
 }
 
-void PathDataBezierCurve::addToImpl(PathImpl& impl) const
-{
-    impl.moveTo(start);
-    impl.addBezierCurveTo(controlPoint1, controlPoint2, endPoint);
-}
-
 void PathDataBezierCurve::applyElements(const PathElementApplier& applier) const
 {
     applier({ PathElement::Type::MoveToPoint, { start } });
@@ -703,6 +711,12 @@ FloatPoint PathDataArc::calculateEndPoint(const FloatPoint&, FloatPoint& lastMov
     return calculateArcToEndPoint(start, controlPoint1, controlPoint2, radius);
 }
 
+std::optional<FloatPoint> PathDataArc::tryGetEndPointWithoutContext() const
+{
+    FloatPoint lastMoveToPoint;
+    return calculateEndPoint({ }, lastMoveToPoint);
+}
+
 void PathDataArc::extendFastBoundingRect(const FloatPoint&, const FloatPoint&, FloatRect& boundingRect) const
 {
     boundingRect.extend(start);
@@ -717,12 +731,6 @@ void PathDataArc::extendBoundingRect(const FloatPoint&, const FloatPoint&, Float
     boundingRect.extend(calculateArcToEndPoint(start, controlPoint1, controlPoint2, radius));
 }
 
-void PathDataArc::addToImpl(PathImpl& impl) const
-{
-    impl.moveTo(start);
-    impl.addArcTo(controlPoint1, controlPoint2, radius);
-}
-
 WTF::TextStream& operator<<(WTF::TextStream& ts, const PathDataArc& data)
 {
     ts << "move to " << data.start;
@@ -736,6 +744,11 @@ FloatPoint PathCloseSubpath::calculateEndPoint(const FloatPoint&, FloatPoint& la
     return lastMoveToPoint;
 }
 
+std::optional<FloatPoint> PathCloseSubpath::tryGetEndPointWithoutContext() const
+{
+    return std::nullopt;
+}
+
 void PathCloseSubpath::extendFastBoundingRect(const FloatPoint&, const FloatPoint& lastMoveToPoint, FloatRect& boundingRect) const
 {
     boundingRect.extend(lastMoveToPoint);
@@ -746,11 +759,6 @@ void PathCloseSubpath::extendBoundingRect(const FloatPoint&, const FloatPoint& l
     boundingRect.extend(lastMoveToPoint);
 }
 
-void PathCloseSubpath::addToImpl(PathImpl& impl) const
-{
-    impl.closeSubpath();
-}
-
 void PathCloseSubpath::applyElements(const PathElementApplier& applier) const
 {
     applier({ PathElement::Type::CloseSubpath, { } });
