@@ -35,9 +35,9 @@
 
 namespace WebCore {
 
-Ref<FEComponentTransfer> FEComponentTransfer::create(const ComponentTransferFunction& redFunction, const ComponentTransferFunction& greenFunction, const ComponentTransferFunction& blueFunction, const ComponentTransferFunction& alphaFunction)
+Ref<FEComponentTransfer> FEComponentTransfer::create(const ComponentTransferFunction& redFunction, const ComponentTransferFunction& greenFunction, const ComponentTransferFunction& blueFunction, const ComponentTransferFunction& alphaFunction, DestinationColorSpace colorSpace)
 {
-    return adoptRef(*new FEComponentTransfer(redFunction, greenFunction, blueFunction, alphaFunction));
+    return adoptRef(*new FEComponentTransfer(redFunction, greenFunction, blueFunction, alphaFunction, colorSpace));
 }
 
 Ref<FEComponentTransfer> FEComponentTransfer::create(ComponentTransferFunctions&& functions)
@@ -45,8 +45,8 @@ Ref<FEComponentTransfer> FEComponentTransfer::create(ComponentTransferFunctions&
     return adoptRef(*new FEComponentTransfer(WTFMove(functions)));
 }
 
-FEComponentTransfer::FEComponentTransfer(const ComponentTransferFunction& redFunction, const ComponentTransferFunction& greenFunction, const ComponentTransferFunction& blueFunction, const ComponentTransferFunction& alphaFunction)
-    : FilterEffect(FilterEffect::Type::FEComponentTransfer)
+FEComponentTransfer::FEComponentTransfer(const ComponentTransferFunction& redFunction, const ComponentTransferFunction& greenFunction, const ComponentTransferFunction& blueFunction, const ComponentTransferFunction& alphaFunction, DestinationColorSpace colorSpace)
+    : FilterEffect(FilterEffect::Type::FEComponentTransfer, colorSpace)
     , m_functions(std::array { redFunction, greenFunction, blueFunction, alphaFunction })
 {
 }
