@@ -264,6 +264,8 @@ Vector<String> extractGStreamerOptionsFromCommandLine()
 
 bool ensureGStreamerInitialized()
 {
+    // WARNING: Please note this function can be called from any thread, for instance when creating
+    // a WebCodec element from a JS Worker.
     RELEASE_ASSERT(isInWebProcess());
     static std::once_flag onceFlag;
     static bool isGStreamerInitialized;
@@ -796,7 +798,7 @@ PlatformVideoColorSpace videoColorSpaceFromInfo(const GstVideoInfo& info)
         colorSpace.matrix = PlatformVideoMatrixCoefficients::Fcc;
         break;
     case GST_VIDEO_COLOR_MATRIX_BT2020:
-        colorSpace.matrix = PlatformVideoMatrixCoefficients::Bt2020ConstantLuminance;
+        colorSpace.matrix = PlatformVideoMatrixCoefficients::Bt2020Ncl;
         break;
     case GST_VIDEO_COLOR_MATRIX_UNKNOWN:
         colorSpace.matrix = PlatformVideoMatrixCoefficients::Unspecified;
@@ -925,7 +927,7 @@ void fillVideoInfoColorimetryFromColorSpace(GstVideoInfo* info, const PlatformVi
         case PlatformVideoMatrixCoefficients::Fcc:
             GST_VIDEO_INFO_COLORIMETRY(info).matrix = GST_VIDEO_COLOR_MATRIX_FCC;
             break;
-        case PlatformVideoMatrixCoefficients::Bt2020ConstantLuminance:
+        case PlatformVideoMatrixCoefficients::Bt2020NonconstantLuminance:
             GST_VIDEO_INFO_COLORIMETRY(info).matrix = GST_VIDEO_COLOR_MATRIX_BT2020;
             break;
         case PlatformVideoMatrixCoefficients::Unspecified:
@@ -1034,6 +1036,12 @@ void fillVideoInfoColorimetryFromColorSpace(GstVideoInfo* info, const PlatformVi
         GST_VIDEO_INFO_COLORIMETRY(info).range = GST_VIDEO_COLOR_RANGE_UNKNOWN;
 }
 
+void configureAudioDecoderForHarnessing(const GRefPtr<GstElement>& element)
+{
+    if (gstObjectHasProperty(element.get(), "max-errors"))
+        g_object_set(element.get(), "max-errors", 0, nullptr);
+}
+
 void configureVideoDecoderForHarnessing(const GRefPtr<GstElement>& element)
 {
     if (gstObjectHasProperty(element.get(), "max-threads"))
