@@ -33,15 +33,15 @@
 #include "TrackPrivateBase.h"
 #include <gst/tag/tag.h>
 #include <wtf/text/CString.h>
+#include <wtf/text/StringToIntegerConversion.h>
 
 GST_DEBUG_CATEGORY_EXTERN(webkit_media_player_debug);
 #define GST_CAT_DEFAULT webkit_media_player_debug
 
 namespace WebCore {
 
-AtomString TrackPrivateBaseGStreamer::generateUniquePlaybin2StreamID(TrackType trackType, unsigned index)
+char TrackPrivateBaseGStreamer::prefixForType(TrackType trackType)
 {
-    auto prefix = [trackType]() -> char {
         switch (trackType) {
         case TrackPrivateBaseGStreamer::TrackType::Audio:
             return 'A';
@@ -53,13 +53,18 @@ AtomString TrackPrivateBaseGStreamer::generateUniquePlaybin2StreamID(TrackType t
             ASSERT_NOT_REACHED();
             return 'U';
         }
-    }();
+}
 
-    return makeAtomString(prefix, index);
+AtomString TrackPrivateBaseGStreamer::generateUniquePlaybin2StreamID(TrackType trackType, unsigned index)
+{
+    return makeAtomString(prefixForType(trackType), index);
 }
 
 static GRefPtr<GstPad> findBestUpstreamPad(GRefPtr<GstPad> pad)
 {
+    if (!pad)
+        return nullptr;
+
     GRefPtr<GstPad> sinkPad = pad;
     GRefPtr<GstPad> peerSrcPad;
 
@@ -88,12 +93,13 @@ TrackPrivateBaseGStreamer::TrackPrivateBaseGStreamer(TrackType type, TrackPrivat
 TrackPrivateBaseGStreamer::TrackPrivateBaseGStreamer(TrackType type, TrackPrivateBase* owner, unsigned index, GstStream* stream)
     : m_notifier(MainThreadNotifier<MainThreadNotification>::create())
     , m_index(index)
+    , m_stringId(AtomString::fromLatin1(gst_stream_get_stream_id(stream)))
+    , m_id(trackIdFromStringIdOrIndex(type, m_stringId, index))
     , m_stream(stream)
     , m_type(type)
     , m_owner(owner)
 {
     ASSERT(m_stream);
-    m_id = AtomString::fromLatin1(gst_stream_get_stream_id(m_stream.get()));
 
     g_signal_connect_swapped(m_stream.get(), "notify::tags", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
         track->tagsChanged();
@@ -110,9 +116,13 @@ void TrackPrivateBaseGStreamer::setPad(GRefPtr<GstPad>&& pad)
 
     m_pad = WTFMove(pad);
     m_bestUpstreamPad = findBestUpstreamPad(m_pad);
-    m_id = generateUniquePlaybin2StreamID(m_type, m_index);
+    m_stringId = AtomString(trackIdFromPadStreamStartOrUniqueID(m_type, m_index, m_pad));
+    m_id = trackIdFromStringIdOrIndex(m_type, m_stringId, m_index);
 
-    m_eventProbe = gst_pad_add_probe(m_bestUpstreamPad.get(), GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM, reinterpret_cast<GstPadProbeCallback>(+[](GstPad* pad, GstPadProbeInfo* info, TrackPrivateBaseGStreamer* track) -> GstPadProbeReturn {
+    if (!m_bestUpstreamPad)
+        return;
+
+    m_eventProbe = gst_pad_add_probe(m_bestUpstreamPad.get(), GST_PAD_PROBE_TYPE_EVENT_DOWNSTREAM, reinterpret_cast<GstPadProbeCallback>(+[](GstPad*, GstPadProbeInfo* info, TrackPrivateBaseGStreamer* track) -> GstPadProbeReturn {
         auto* event = gst_pad_probe_info_get_event(info);
         switch (GST_EVENT_TYPE(event)) {
         case GST_EVENT_TAG:
@@ -123,12 +133,11 @@ void TrackPrivateBaseGStreamer::setPad(GRefPtr<GstPad>&& pad)
                 track->streamChanged();
             break;
         case GST_EVENT_CAPS: {
-            GUniquePtr<char> streamId(gst_pad_get_stream_id(pad));
+            String streamId = track->m_stringId;
             if (!streamId)
                 break;
 
-            auto streamIdString = String::fromLatin1(streamId.get());
-            track->m_taskQueue.enqueueTask([track, streamId = WTFMove(streamIdString), event = GRefPtr<GstEvent>(event)]() {
+            track->m_taskQueue.enqueueTask([track, streamId = WTFMove(streamId), event = GRefPtr<GstEvent>(event)]() {
                 GstCaps* caps;
                 gst_event_parse_caps(event.get(), &caps);
                 if (!caps)
@@ -161,10 +170,8 @@ GstObject* TrackPrivateBaseGStreamer::objectForLogging() const
 
 void TrackPrivateBaseGStreamer::disconnect()
 {
-    if (m_stream) {
+    if (m_stream)
         g_signal_handlers_disconnect_matched(m_stream.get(), G_SIGNAL_MATCH_DATA, 0, 0, nullptr, nullptr, this);
-        ASSERT(!g_signal_handler_find(m_stream.get(), G_SIGNAL_MATCH_UNBLOCKED, 0, 0, nullptr, nullptr, nullptr));
-    }
 
     m_tags.clear();
 
@@ -176,11 +183,8 @@ void TrackPrivateBaseGStreamer::disconnect()
         m_bestUpstreamPad.clear();
     }
 
-    if (m_pad) {
-        g_signal_handlers_disconnect_matched(m_pad.get(), G_SIGNAL_MATCH_DATA, 0, 0, nullptr, nullptr, this);
-        ASSERT(!g_signal_handler_find(m_pad.get(), G_SIGNAL_MATCH_UNBLOCKED, 0, 0, nullptr, nullptr, nullptr));
+    if (m_pad)
         m_pad.clear();
-    }
 }
 
 void TrackPrivateBaseGStreamer::tagsChanged()
@@ -193,7 +197,7 @@ void TrackPrivateBaseGStreamer::tagsChanged()
         do {
             tagEvent = adoptGRef(gst_pad_get_sticky_event(m_bestUpstreamPad.get(), GST_EVENT_TAG, i));
             if (tagEvent) {
-                GstTagList* tagsFromEvent;
+                GstTagList* tagsFromEvent = nullptr;
                 gst_event_parse_tag(tagEvent.get(), &tagsFromEvent);
                 tags = adoptGRef(gst_tag_list_copy(tagsFromEvent));
                 String language;
@@ -258,7 +262,7 @@ void TrackPrivateBaseGStreamer::notifyTrackOfTagsChanged()
     if (!tags)
         return;
 
-    tagsChanged(tags);
+    tagsChanged(GRefPtr<GstTagList>(tags));
 
     if (getTag(tags.get(), GST_TAG_TITLE, m_label) && client)
         client->labelChanged(m_label);
@@ -277,12 +281,15 @@ void TrackPrivateBaseGStreamer::notifyTrackOfTagsChanged()
 
 void TrackPrivateBaseGStreamer::notifyTrackOfStreamChanged()
 {
+    if (!m_pad)
+        return;
+
     GUniquePtr<char> streamId(gst_pad_get_stream_id(m_pad.get()));
     if (!streamId)
         return;
 
     GST_INFO("Track %d got stream start for stream %s.", m_index, streamId.get());
-    m_id = AtomString::fromLatin1(streamId.get());
+    m_stringId = AtomString::fromLatin1(streamId.get());
 }
 
 void TrackPrivateBaseGStreamer::streamChanged()
@@ -292,6 +299,91 @@ void TrackPrivateBaseGStreamer::streamChanged()
     });
 }
 
+void TrackPrivateBaseGStreamer::installUpdateConfigurationHandlers()
+{
+    if (m_pad) {
+        g_signal_connect_swapped(m_pad.get(), "notify::caps", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
+            if (!track->m_pad)
+                return;
+            auto caps = adoptGRef(gst_pad_get_current_caps(track->m_pad.get()));
+            // We will receive a synchronous notification for caps being unset during pipeline teardown.
+            if (!caps)
+                return;
+
+            track->m_taskQueue.enqueueTask([track, caps = WTFMove(caps)]() mutable {
+                track->capsChanged(String::fromLatin1(GUniquePtr<char>(gst_pad_get_stream_id(track->m_pad.get())).get()), WTFMove(caps));
+            });
+        }), this);
+        g_signal_connect_swapped(m_pad.get(), "notify::tags", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
+            track->m_taskQueue.enqueueTask([track]() {
+                if (!track->m_pad)
+                    return;
+                track->updateConfigurationFromTags(getAllTags(track->m_pad));
+            });
+        }), this);
+    } else if (m_stream) {
+        g_signal_connect_swapped(m_stream.get(), "notify::caps", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
+            track->m_taskQueue.enqueueTask([track]() {
+                auto caps = adoptGRef(gst_stream_get_caps(track->m_stream.get()));
+                track->capsChanged(String::fromLatin1(gst_stream_get_stream_id(track->m_stream.get())), WTFMove(caps));
+            });
+        }), this);
+
+        g_signal_connect_swapped(m_stream.get(), "notify::tags", G_CALLBACK(+[](TrackPrivateBaseGStreamer* track) {
+            track->m_taskQueue.enqueueTask([track]() {
+                auto tags = adoptGRef(gst_stream_get_tags(track->m_stream.get()));
+                track->updateConfigurationFromTags(WTFMove(tags));
+            });
+        }), this);
+    }
+}
+
+String TrackPrivateBaseGStreamer::trackIdFromPadStreamStartOrUniqueID(TrackType type, unsigned index, const GRefPtr<GstPad>& pad)
+{
+    String streamId = nullString();
+    if (!pad)
+        return generateUniquePlaybin2StreamID(type, index);
+
+    auto streamStart = adoptGRef(gst_pad_get_sticky_event(pad.get(), GST_EVENT_STREAM_START, 0));
+    if (!streamStart)
+        return generateUniquePlaybin2StreamID(type, index);
+
+    const gchar* streamIdAsCharacters;
+    gst_event_parse_stream_start(streamStart.get(), &streamIdAsCharacters);
+
+    if (!streamIdAsCharacters)
+        return generateUniquePlaybin2StreamID(type, index);
+
+    StringView streamIdView = StringView::fromLatin1(streamIdAsCharacters);
+    size_t position = streamIdView.find('/');
+    if (position == notFound || position + 1 == streamIdView.length())
+        return generateUniquePlaybin2StreamID(type, index);
+
+    return streamIdView.substring(position + 1).toString();
+}
+
+TrackID TrackPrivateBaseGStreamer::trackIdFromStringIdOrIndex(TrackType type, const AtomString& stringId, unsigned index)
+{
+    if (!stringId.startsWith(prefixForType(type)))
+        return index;
+    auto stringView = StringView { stringId }.substring(1);
+    return parseIntegerAllowingTrailingJunk<TrackID>(stringView).value_or(index);
+}
+
+GRefPtr<GstTagList> TrackPrivateBaseGStreamer::getAllTags(const GRefPtr<GstPad>& pad)
+{
+    auto allTags = adoptGRef(gst_tag_list_new_empty());
+    GstTagList* taglist = nullptr;
+    for (guint i = 0;; i++) {
+        GRefPtr<GstEvent> tagsEvent = adoptGRef(gst_pad_get_sticky_event(pad.get(), GST_EVENT_TAG, i));
+        if (!tagsEvent)
+            break;
+        gst_event_parse_tag(tagsEvent.get(), &taglist);
+        allTags = adoptGRef(gst_tag_list_merge(allTags.get(), taglist, GST_TAG_MERGE_APPEND));
+    }
+    return allTags;
+}
+
 #undef GST_CAT_DEFAULT
 
 } // namespace WebCore
