@@ -47,12 +47,15 @@ static WorkQueue& gstEncoderWorkQueue()
     return queue.get();
 }
 
-class GStreamerInternalVideoEncoder : public ThreadSafeRefCounted<GStreamerInternalVideoEncoder> {
-public:
-    static Ref<GStreamerInternalVideoEncoder> create(const String& codecName, VideoEncoder::OutputCallback&& outputCallback, VideoEncoder::PostTaskCallback&& postTaskCallback) { return adoptRef(*new GStreamerInternalVideoEncoder(codecName, WTFMove(outputCallback), WTFMove(postTaskCallback))); }
-    ~GStreamerInternalVideoEncoder() = default;
+class GStreamerInternalVideoEncoder : public ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<GStreamerInternalVideoEncoder, WTF::DestructionThread::Main> {
+    WTF_MAKE_NONCOPYABLE(GStreamerInternalVideoEncoder);
+    WTF_MAKE_FAST_ALLOCATED;
 
-    String initialize(const VideoEncoder::Config&);
+public:
+    static Ref<GStreamerInternalVideoEncoder> create(VideoEncoder::DescriptionCallback&& descriptionCallback, VideoEncoder::OutputCallback&& outputCallback, VideoEncoder::PostTaskCallback&& postTaskCallback) { return adoptRef(*new GStreamerInternalVideoEncoder(WTFMove(descriptionCallback), WTFMove(outputCallback), WTFMove(postTaskCallback))); }
+    ~GStreamerInternalVideoEncoder();
+
+    String initialize(const String& codecName, const VideoEncoder::Config&);
     void postTask(Function<void()>&& task) { m_postTaskCallback(WTFMove(task)); }
     bool encode(VideoEncoder::RawFrame&&, bool shouldGenerateKeyFrame);
     void flush(Function<void()>&&);
@@ -62,9 +65,9 @@ public:
     bool isClosed() const { return m_isClosed; }
 
 private:
-    GStreamerInternalVideoEncoder(const String& codecName, VideoEncoder::OutputCallback&&, VideoEncoder::PostTaskCallback&&);
+    GStreamerInternalVideoEncoder(VideoEncoder::DescriptionCallback&&, VideoEncoder::OutputCallback&&, VideoEncoder::PostTaskCallback&&);
 
-    const String m_codecName;
+    VideoEncoder::DescriptionCallback m_descriptionCallback;
     VideoEncoder::OutputCallback m_outputCallback;
     VideoEncoder::PostTaskCallback m_postTaskCallback;
     int64_t m_timestamp { 0 };
@@ -86,36 +89,34 @@ void GStreamerVideoEncoder::create(const String& codecName, const VideoEncoder::
     registerWebKitGStreamerVideoEncoder();
     auto& scanner = GStreamerRegistryScanner::singleton();
     if (!scanner.isCodecSupported(GStreamerRegistryScanner::Configuration::Encoding, codecName)) {
-        gstEncoderWorkQueue().dispatch([callback = WTFMove(callback), codecName]() mutable {
-            callback(makeUnexpected(makeString("No GStreamer encoder found for codec ", codecName)));
+        auto errorMessage = makeString("No GStreamer encoder found for codec "_s, codecName);
+        postTaskCallback([callback = WTFMove(callback), errorMessage = WTFMove(errorMessage)]() mutable {
+            callback(makeUnexpected(WTFMove(errorMessage)));
         });
         return;
     }
 
-    auto encoder = makeUniqueRef<GStreamerVideoEncoder>(codecName, WTFMove(outputCallback), WTFMove(postTaskCallback));
-    auto error = encoder->initialize(config);
-    gstEncoderWorkQueue().dispatch([callback = WTFMove(callback), descriptionCallback = WTFMove(descriptionCallback), encoder = WTFMove(encoder), error]() mutable {
+    auto encoder = makeUniqueRef<GStreamerVideoEncoder>(WTFMove(descriptionCallback), WTFMove(outputCallback), WTFMove(postTaskCallback));
+    auto internalEncoder = encoder->m_internalEncoder;
+    auto error = internalEncoder->initialize(codecName, config);
+    if (!error.isEmpty()) {
+        encoder->m_internalEncoder->postTask([callback = WTFMove(callback), error = WTFMove(error)]() mutable {
+            GST_WARNING("Error creating encoder: %s", error.ascii().data());
+            callback(makeUnexpected(makeString("GStreamer encoding initialization failed with error: "_s, error)));
+        });
+        return;
+    }
+    gstEncoderWorkQueue().dispatch([callback = WTFMove(callback), encoder = WTFMove(encoder)]() mutable {
         auto internalEncoder = encoder->m_internalEncoder;
-        internalEncoder->postTask([callback = WTFMove(callback), descriptionCallback = WTFMove(descriptionCallback), encoder = WTFMove(encoder), error]() mutable {
-            if (!error.isEmpty()) {
-                GST_WARNING("Error creating encoder: %s", error.ascii().data());
-                callback(makeUnexpected(makeString("GStreamer encoding initialization failed with error: ", error)));
-                return;
-            }
-
+        internalEncoder->postTask([callback = WTFMove(callback), encoder = WTFMove(encoder)]() mutable {
             GST_DEBUG("Encoder created");
             callback(UniqueRef<VideoEncoder> { WTFMove(encoder) });
-
-            VideoEncoder::ActiveConfiguration configuration;
-            // FIXME: How to properly fill configuration.colorspace?
-            configuration.colorSpace = PlatformVideoColorSpace { PlatformVideoColorPrimaries::Smpte170m, PlatformVideoTransferCharacteristics::Smpte170m, PlatformVideoMatrixCoefficients::Smpte170m, false };
-            descriptionCallback(WTFMove(configuration));
         });
     });
 }
 
-GStreamerVideoEncoder::GStreamerVideoEncoder(const String& codecName, OutputCallback&& outputCallback, PostTaskCallback&& postTaskCallback)
-    : m_internalEncoder(GStreamerInternalVideoEncoder::create(codecName, WTFMove(outputCallback), WTFMove(postTaskCallback)))
+GStreamerVideoEncoder::GStreamerVideoEncoder(DescriptionCallback&& descriptionCallback, OutputCallback&& outputCallback, PostTaskCallback&& postTaskCallback)
+    : m_internalEncoder(GStreamerInternalVideoEncoder::create(WTFMove(descriptionCallback), WTFMove(outputCallback), WTFMove(postTaskCallback)))
 {
 }
 
@@ -125,11 +126,6 @@ GStreamerVideoEncoder::~GStreamerVideoEncoder()
     close();
 }
 
-String GStreamerVideoEncoder::initialize(const VideoEncoder::Config& config)
-{
-    return m_internalEncoder->initialize(config);
-}
-
 void GStreamerVideoEncoder::encode(RawFrame&& frame, bool shouldGenerateKeyFrame, EncodeCallback&& callback)
 {
     gstEncoderWorkQueue().dispatch([frame = WTFMove(frame), shouldGenerateKeyFrame, encoder = m_internalEncoder, callback = WTFMove(callback)]() mutable {
@@ -142,7 +138,14 @@ void GStreamerVideoEncoder::encode(RawFrame&& frame, bool shouldGenerateKeyFrame
             encoder->harness()->processOutputBuffers();
         else
             resultString = "Encoding failed"_s;
-        callback(WTFMove(resultString));
+
+        encoder->postTask([weakEncoder = ThreadSafeWeakPtr { encoder.get() }, result = WTFMove(resultString), callback = WTFMove(callback)]() mutable {
+            auto encoder = weakEncoder.get();
+            if (!encoder || encoder->isClosed())
+                return;
+
+            callback(WTFMove(result));
+        });
     });
 }
 
@@ -165,16 +168,66 @@ void GStreamerVideoEncoder::close()
     m_internalEncoder->close();
 }
 
-GStreamerInternalVideoEncoder::GStreamerInternalVideoEncoder(const String& codecName, VideoEncoder::OutputCallback&& outputCallback, VideoEncoder::PostTaskCallback&& postTaskCallback)
-    : m_codecName(codecName)
+GStreamerInternalVideoEncoder::GStreamerInternalVideoEncoder(VideoEncoder::DescriptionCallback&& descriptionCallback, VideoEncoder::OutputCallback&& outputCallback, VideoEncoder::PostTaskCallback&& postTaskCallback)
+    : m_descriptionCallback(WTFMove(descriptionCallback))
     , m_outputCallback(WTFMove(outputCallback))
     , m_postTaskCallback(WTFMove(postTaskCallback))
 {
     GRefPtr<GstElement> element = gst_element_factory_make("webkitvideoencoder", nullptr);
-    m_harness = GStreamerElementHarness::create(WTFMove(element), [protectedThis = Ref { *this }, this](auto&, const GRefPtr<GstBuffer>& outputBuffer) {
-        if (protectedThis->m_isClosed)
+
+    auto pad = adoptGRef(gst_element_get_static_pad(element.get(), "src"));
+    g_signal_connect_data(pad.get(), "notify::caps", G_CALLBACK(+[](GObject* pad, GParamSpec*, gpointer userData) {
+        auto weakEncoder = static_cast<ThreadSafeWeakPtr<GStreamerInternalVideoEncoder>*>(userData);
+        auto encoder = weakEncoder->get();
+        if (!encoder)
             return;
 
+        GRefPtr<GstCaps> caps;
+        g_object_get(pad, "caps", &caps.outPtr(), nullptr);
+        if (!caps)
+            return;
+
+        encoder->postTask([weakEncoder = WTFMove(weakEncoder), caps = WTFMove(caps)] {
+            auto encoder = weakEncoder->get();
+            if (!encoder)
+                return;
+
+            VideoEncoder::ActiveConfiguration configuration;
+            configuration.colorSpace = videoColorSpaceFromCaps(caps.get());
+
+            auto structure = gst_caps_get_structure(caps.get(), 0);
+            GstBuffer* header = nullptr;
+            if (auto streamHeader = gst_structure_get_value(structure, "streamheader")) {
+                RELEASE_ASSERT(GST_VALUE_HOLDS_ARRAY(streamHeader));
+                auto firstValue = gst_value_array_get_value(streamHeader, 0);
+                RELEASE_ASSERT(GST_VALUE_HOLDS_BUFFER(firstValue));
+                header = gst_value_get_buffer(firstValue);
+            } else if (auto codecData = gst_structure_get_value(structure, "codec_data")) {
+                RELEASE_ASSERT(GST_VALUE_HOLDS_BUFFER(codecData));
+                header = gst_value_get_buffer(codecData);
+            }
+
+            if (header) {
+                GstMappedBuffer buffer(header, GST_MAP_READ);
+                configuration.description = { { buffer.data(), buffer.size() } };
+            }
+            encoder->m_descriptionCallback(WTFMove(configuration));
+        });
+    }), new ThreadSafeWeakPtr { *this }, [](void* data, GClosure*) {
+        delete static_cast<ThreadSafeWeakPtr<GStreamerInternalVideoEncoder>*>(data);
+    }, static_cast<GConnectFlags>(0));
+
+    m_harness = GStreamerElementHarness::create(WTFMove(element), [weakThis = ThreadSafeWeakPtr { *this }, this](auto&, const GRefPtr<GstBuffer>& outputBuffer) {
+        if (!weakThis.get())
+            return;
+        if (m_isClosed)
+            return;
+
+        static std::once_flag onceFlag;
+        std::call_once(onceFlag, [this] {
+            m_harness->dumpGraph("video-encoder");
+        });
+
         bool isKeyFrame = !GST_BUFFER_FLAG_IS_SET(outputBuffer.get(), GST_BUFFER_FLAG_DELTA_UNIT);
         GST_TRACE_OBJECT(m_harness->element(), "Notifying encoded%s frame", isKeyFrame ? " key" : "");
         GstMappedBuffer encodedImage(outputBuffer.get(), GST_MAP_READ);
@@ -191,34 +244,43 @@ GStreamerInternalVideoEncoder::GStreamerInternalVideoEncoder(const String& codec
     });
 }
 
-String GStreamerInternalVideoEncoder::initialize(const VideoEncoder::Config& config)
+GStreamerInternalVideoEncoder::~GStreamerInternalVideoEncoder()
 {
-    GST_DEBUG_OBJECT(m_harness->element(), "Initializing encoder for codec %s", m_codecName.ascii().data());
+    if (!m_harness)
+        return;
+
+    auto pad = adoptGRef(gst_element_get_static_pad(m_harness->element(), "src"));
+    g_signal_handlers_disconnect_by_data(pad.get(), this);
+}
+
+String GStreamerInternalVideoEncoder::initialize(const String& codecName, const VideoEncoder::Config& config)
+{
+    GST_DEBUG_OBJECT(m_harness->element(), "Initializing encoder for codec %s", codecName.ascii().data());
     GRefPtr<GstCaps> encoderCaps;
-    if (m_codecName == "vp8"_s)
+    if (codecName == "vp8"_s)
         encoderCaps = adoptGRef(gst_caps_new_empty_simple("video/x-vp8"));
-    else if (m_codecName.startsWith("vp09"_s)) {
+    else if (codecName.startsWith("vp09"_s)) {
         encoderCaps = adoptGRef(gst_caps_new_empty_simple("video/x-vp9"));
-        if (auto profileId = GStreamerCodecUtilities::parseVP9Profile(m_codecName)) {
+        if (auto profileId = GStreamerCodecUtilities::parseVP9Profile(codecName)) {
             auto profile = makeString(profileId);
             gst_caps_set_simple(encoderCaps.get(), "profile", G_TYPE_STRING, profile.ascii().data(), nullptr);
         }
-    } else if (m_codecName.startsWith("avc1"_s)) {
+    } else if (codecName.startsWith("avc1"_s)) {
         encoderCaps = adoptGRef(gst_caps_new_empty_simple("video/x-h264"));
-        auto [profile, level] = GStreamerCodecUtilities::parseH264ProfileAndLevel(m_codecName);
+        auto [profile, level] = GStreamerCodecUtilities::parseH264ProfileAndLevel(codecName);
         if (profile)
             gst_caps_set_simple(encoderCaps.get(), "profile", G_TYPE_STRING, profile, nullptr);
         // FIXME: Set level on caps too?
         UNUSED_VARIABLE(level);
-    } else if (m_codecName.startsWith("av01"_s)) {
+    } else if (codecName.startsWith("av01"_s)) {
         // FIXME: parse codec parameters.
         encoderCaps = adoptGRef(gst_caps_new_empty_simple("video/x-av1"));
-    } else if (m_codecName.startsWith("hvc1"_s) || m_codecName.startsWith("hev1"_s)) {
+    } else if (codecName.startsWith("hvc1"_s) || codecName.startsWith("hev1"_s)) {
         encoderCaps = adoptGRef(gst_caps_new_empty_simple("video/x-h265"));
-        if (const char* profile = GStreamerCodecUtilities::parseHEVCProfile(m_codecName))
+        if (const char* profile = GStreamerCodecUtilities::parseHEVCProfile(codecName))
             gst_caps_set_simple(encoderCaps.get(), "profile", G_TYPE_STRING, profile, nullptr);
     } else
-        return makeString("Unsupported outgoing video encoding: "_s, m_codecName);
+        return makeString("Unsupported outgoing video encoding: "_s, codecName);
 
     if (config.width)
         gst_caps_set_simple(encoderCaps.get(), "width", G_TYPE_INT, static_cast<int>(config.width), nullptr);
@@ -230,8 +292,8 @@ String GStreamerInternalVideoEncoder::initialize(const VideoEncoder::Config& con
     if (!videoEncoderSetFormat(WEBKIT_VIDEO_ENCODER(m_harness->element()), WTFMove(encoderCaps)))
         return "Unable to set encoder format"_s;
 
-    if (config.bitRate)
-        g_object_set(m_harness->element(), "bitrate", static_cast<uint32_t>(config.bitRate / 1024), nullptr);
+    if (config.bitRate > 1000)
+        g_object_set(m_harness->element(), "bitrate", static_cast<uint32_t>(config.bitRate / 1000), nullptr);
     m_isInitialized = true;
     return emptyString();
 }
@@ -253,9 +315,6 @@ bool GStreamerInternalVideoEncoder::encode(VideoEncoder::RawFrame&& rawFrame, bo
 
     auto& gstVideoFrame = downcast<VideoFrameGStreamer>(rawFrame.frame.get());
 
-    auto* buffer = gst_sample_get_buffer(gstVideoFrame.sample());
-    GST_BUFFER_PTS(buffer) = m_timestamp;
-
     // FIXME: The WebRTC encoder doesn't support GL memories ingesting yet, so until then we do a conversion here.
     return m_harness->pushSample(gstVideoFrame.downloadSample());
 }
