@@ -30,7 +30,7 @@ namespace WebCore {
 class GStreamerRegistryScannerMSE : public GStreamerRegistryScanner {
 public:
     static GStreamerRegistryScannerMSE& singleton();
-    static void getSupportedDecodingTypes(HashSet<String, ASCIICaseInsensitiveHash>&);
+    static void getSupportedDecodingTypes(HashSet<String>&);
     GStreamerRegistryScannerMSE();
 };
 
