@@ -36,13 +36,16 @@ using namespace WebCore;
 GST_DEBUG_CATEGORY(video_encoder_debug);
 #define GST_CAT_DEFAULT video_encoder_debug
 
-#define KBIT_TO_BIT 1024
+#define KBIT_TO_BIT 1000
 
 // FIXME: Make this configurable at runtime?
 #define NUMBER_OF_THREADS 4
 
+#define MAX_WIDTH 4096
+#define MAX_HEIGHT 4096
+
 static GstStaticPadTemplate sinkTemplate = GST_STATIC_PAD_TEMPLATE("sink", GST_PAD_SINK, GST_PAD_ALWAYS, GST_STATIC_CAPS("video/x-raw(ANY)"));
-static GstStaticPadTemplate srcTemplate = GST_STATIC_PAD_TEMPLATE("src", GST_PAD_SRC, GST_PAD_ALWAYS, GST_STATIC_CAPS("video/x-h264;video/x-vp8;video/x-vp9;video/x-h265"));
+static GstStaticPadTemplate srcTemplate = GST_STATIC_PAD_TEMPLATE("src", GST_PAD_SRC, GST_PAD_ALWAYS, GST_STATIC_CAPS("video/x-h264;video/x-vp8;video/x-vp9;video/x-h265;video/x-av1"));
 
 // https://www.w3.org/TR/mediastream-recording/#bitratemode
 typedef enum {
@@ -108,12 +111,14 @@ struct EncoderDefinition {
 enum EncoderId {
     None,
     X264,
+    X265,
     OpenH264,
     OmxH264,
     VaapiH264,
     VaapiH265,
     Vp8,
-    Vp9
+    Vp9,
+    Av1
 };
 
 class Encoders {
@@ -124,7 +129,7 @@ public:
         return encoders;
     }
 
-    static void registerEncoder(EncoderId id, const char* name, const char* parserName, const char* caps, const char* encodedFormat,
+    static void registerEncoder(EncoderId id, const char* name, const char* parserName, const char* capsString, const char* encodedFormatString,
         SetupFunc&& setupEncoder, const char* bitratePropertyName, SetBitrateFunc&& setBitrate, const char* keyframeIntervalPropertyName, SetBitrateModeFunc&& setBitrateMode, SetLatencyModeFunc&& setLatency)
     {
         auto encoderFactory = adoptGRef(gst_element_factory_find(name));
@@ -146,12 +151,21 @@ public:
             }
         }
 
+        auto caps = adoptGRef(gst_caps_from_string(capsString));
+        GST_MINI_OBJECT_FLAG_SET(caps.get(), GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+
+        GRefPtr<GstCaps> encodedFormat;
+        if (encodedFormatString) {
+            encodedFormat = adoptGRef(gst_caps_from_string(encodedFormatString));
+            GST_MINI_OBJECT_FLAG_SET(encodedFormat.get(), GST_MINI_OBJECT_FLAG_MAY_BE_LEAKED);
+        }
+
         singleton().emplace(std::make_pair(id, EncoderDefinition {
-            .caps = adoptGRef(gst_caps_from_string(caps)),
+            .caps = WTFMove(caps),
             .name = name,
             .parserName = parserName,
             .factory = WTFMove(encoderFactory),
-            .encodedFormat = encodedFormat ? adoptGRef(gst_caps_from_string(encodedFormat)) : nullptr,
+            .encodedFormat = WTFMove(encodedFormat),
             .setBitrate = WTFMove(setBitrate),
             .setupEncoder = WTFMove(setupEncoder),
             .setBitrateMode = WTFMove(setBitrateMode),
@@ -189,7 +203,7 @@ struct _WebKitVideoEncoderPrivate {
 
 #define webkit_video_encoder_parent_class parent_class
 WEBKIT_DEFINE_TYPE_WITH_CODE(WebKitVideoEncoder, webkit_video_encoder, GST_TYPE_BIN,
-    GST_DEBUG_CATEGORY_INIT(video_encoder_debug, "webkitvideoencoder", 0, "WebKit Video encoder"))
+    GST_DEBUG_CATEGORY_INIT(video_encoder_debug, "webkitvideoencoderprivate", 0, "WebKit Video Encoder Private"))
 
 enum {
     PROP_FORMAT = 1,
@@ -249,10 +263,25 @@ static void videoEncoderSetBitrate(WebKitVideoEncoder* self, guint bitrate)
     }
 }
 
-static void videoEncoderSetEncoder(WebKitVideoEncoder* self, EncoderId encoderId, GRefPtr<GstCaps>&& encodedCaps)
+static bool videoEncoderSetEncoder(WebKitVideoEncoder* self, EncoderId encoderId, GRefPtr<GstCaps>&& encodedCaps)
 {
     ASSERT(encoderId != EncoderId::None);
 
+    auto* structure = gst_caps_get_structure(encodedCaps.get(), 0);
+    if (structure) {
+        int width;
+        if (gst_structure_get_int(structure, "width", &width) && width > MAX_WIDTH) {
+            GST_WARNING_OBJECT(self, "Encoded width (%d) is too high. Maximum allowed: %d.", width, MAX_WIDTH);
+            return false;
+        }
+
+        int height;
+        if (gst_structure_get_int(structure, "height", &height) && height > MAX_HEIGHT) {
+            GST_WARNING_OBJECT(self, "Encoded height (%d) is too high. Maximum allowed: %d.", height, MAX_HEIGHT);
+            return false;
+        }
+    }
+
     auto* priv = self->priv;
     auto srcPad = adoptGRef(gst_element_get_static_pad(GST_ELEMENT_CAST(self), "src"));
 
@@ -323,7 +352,7 @@ static void videoEncoderSetEncoder(WebKitVideoEncoder* self, EncoderId encoderId
         auto sinkPad = adoptGRef(gst_element_get_static_pad(GST_ELEMENT_CAST(self), "sink"));
         gst_ghost_pad_set_target(GST_GHOST_PAD(sinkPad.get()), sinkPadTarget.get());
     } else {
-        gst_element_unlink(priv->videoConvert.get(), priv->inputCapsFilter.get());
+        gst_element_unlink_many(priv->videoConvert.get(), priv->videoScale.get(), priv->inputCapsFilter.get(), nullptr);
         auto caps = adoptGRef(gst_caps_new_any());
         g_object_set(priv->inputCapsFilter.get(), "caps", caps.get(), nullptr);
     }
@@ -342,15 +371,26 @@ static void videoEncoderSetEncoder(WebKitVideoEncoder* self, EncoderId encoderId
     encoderDefinition->setBitrateMode(priv->encoder.get(), priv->bitrateMode);
     encoderDefinition->setLatencyMode(priv->encoder.get(), priv->latencyMode);
 
-    gst_element_link_many(priv->videoConvert.get(), priv->videoScale.get(), priv->inputCapsFilter.get(), nullptr);
-    if (shouldLinkEncoder)
-        gst_element_link(priv->inputCapsFilter.get(), priv->encoder.get());
+    if (!gst_element_link_many(priv->videoConvert.get(), priv->videoScale.get(), priv->inputCapsFilter.get(), nullptr)) {
+        GST_WARNING_OBJECT(self, "Failed to link videoconvert, videoscale and input capsfilter");
+        return false;
+    }
+
+    if (shouldLinkEncoder && !gst_element_link(priv->inputCapsFilter.get(), priv->encoder.get())) {
+        GST_WARNING_OBJECT(self, "Failed to link input capsfilter to encoder");
+        return false;
+    }
 
     if (priv->parser) {
         gst_bin_add(GST_BIN_CAST(self), priv->parser.get());
-        if (shouldLinkEncoder)
-            gst_element_link(priv->encoder.get(), priv->outputCapsFilter.get());
-        gst_element_link(priv->outputCapsFilter.get(), priv->parser.get());
+        if (shouldLinkEncoder && !gst_element_link(priv->encoder.get(), priv->outputCapsFilter.get())) {
+            GST_WARNING_OBJECT(self, "Failed to link encoder to output capsfilter");
+            return false;
+        }
+        if (!gst_element_link(priv->outputCapsFilter.get(), priv->parser.get())) {
+            GST_WARNING_OBJECT(self, "Failed to link output capsfilter to parser");
+            return false;
+        }
     }
 
     priv->capsFilter = gst_element_factory_make("capsfilter", nullptr);
@@ -364,12 +404,17 @@ static void videoEncoderSetEncoder(WebKitVideoEncoder* self, EncoderId encoderId
     auto srcPadTarget = adoptGRef(gst_element_get_static_pad(priv->capsFilter.get(), "src"));
     gst_ghost_pad_set_target(GST_GHOST_PAD(srcPad.get()), srcPadTarget.get());
 
-    gst_element_link(priv->parser ? priv->parser.get() : priv->encoder.get(), priv->capsFilter.get());
+    if (!gst_element_link(priv->parser ? priv->parser.get() : priv->encoder.get(), priv->capsFilter.get())) {
+        GST_WARNING_OBJECT(self, "Failed to link to final capsfilter");
+        return false;
+    }
 
     gst_bin_sync_children_states(GST_BIN_CAST(self));
     gst_element_set_locked_state(GST_ELEMENT_CAST(self), FALSE);
+    GST_DEBUG_BIN_TO_DOT_FILE_WITH_TS(GST_BIN_CAST(self), GST_DEBUG_GRAPH_SHOW_ALL, "configured-encoder");
 
     videoEncoderSetBitrate(self, priv->bitrate);
+    return true;
 }
 
 EncoderId videoEncoderFindForFormat(WebKitVideoEncoder* self, const GRefPtr<GstCaps>& caps)
@@ -412,8 +457,7 @@ bool videoEncoderSetFormat(WebKitVideoEncoder* self, GRefPtr<GstCaps>&& caps)
         return false;
     }
 
-    videoEncoderSetEncoder(self, encoderId, WTFMove(caps));
-    return true;
+    return videoEncoderSetEncoder(self, encoderId, WTFMove(caps));
 }
 
 static void videoEncoderSetProperty(GObject* object, guint propertyId, const GValue* value, GParamSpec* pspec)
@@ -542,15 +586,57 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
     Encoders::registerEncoder(X264, "x264enc", "h264parse", "video/x-h264",
         "video/x-h264,alignment=au,stream-format=byte-stream",
         [](WebKitVideoEncoder* self) {
-            g_object_set(self->priv->encoder.get(), "key-int-max", 15, "threads", NUMBER_OF_THREADS, nullptr);
+            g_object_set(self->priv->encoder.get(), "key-int-max", 15, "threads", NUMBER_OF_THREADS, "b-adapt", FALSE, "vbv-buf-capacity", 120, nullptr);
             g_object_set(self->priv->parser.get(), "config-interval", 1, nullptr);
 
             const auto* structure = gst_caps_get_structure(self->priv->encodedCaps.get(), 0);
             auto inputCaps = adoptGRef(gst_caps_new_any());
             if (const char* profileString = gst_structure_get_string(structure, "profile")) {
-                auto profile = StringView::fromLatin1(profileString);
-                if (profile.find("high"_s) != notFound)
-                    inputCaps = adoptGRef(gst_caps_new_simple("video/x-raw", "format", G_TYPE_STRING, "Y444", nullptr));
+                const char* pixelFormat = nullptr;
+                auto pad = adoptGRef(gst_element_get_static_pad(self->priv->encoder.get(), "sink"));
+                auto allowedCaps = adoptGRef(gst_pad_query_caps(pad.get(), nullptr));
+                const auto* structure = gst_caps_get_structure(allowedCaps.get(), 0);
+                const auto* formatValue = gst_structure_get_value(structure, "format");
+                unsigned size = gst_value_list_get_size(formatValue);
+                bool supports10BitsLittleEndian = false;
+                bool supports10BitsBigEndian = false;
+
+                for (unsigned i = 0; i < size; i++) {
+                    auto* value = gst_value_list_get_value(formatValue, i);
+                    const char* format = g_value_get_string(value);
+                    if (g_str_has_suffix(format, "_10LE")) {
+                        supports10BitsLittleEndian = true;
+                        break;
+                    }
+                    if (g_str_has_suffix(format, "_10BE")) {
+                        supports10BitsBigEndian = true;
+                        break;
+                    }
+                }
+
+                if (g_str_has_prefix(profileString, "high-4:4:4")) {
+                    if (supports10BitsLittleEndian)
+                        pixelFormat = "Y444_10LE";
+                    else if (supports10BitsBigEndian)
+                        pixelFormat = "Y444_10BE";
+                    else
+                        pixelFormat = "Y444";
+                } else if (g_str_has_prefix(profileString, "high-4:2:2")) {
+                    if (supports10BitsLittleEndian)
+                        pixelFormat = "Y422_10LE";
+                    else if (supports10BitsBigEndian)
+                        pixelFormat = "Y422_10BE";
+                    else
+                        pixelFormat = "Y42B";
+                } else if (g_str_has_prefix(profileString, "high-10")) {
+                    if (supports10BitsLittleEndian)
+                        pixelFormat = "Y420_10LE";
+                    else if (supports10BitsBigEndian)
+                        pixelFormat = "Y420_10BE";
+                } else
+                    pixelFormat = "I420";
+                if (pixelFormat)
+                    inputCaps = adoptGRef(gst_caps_new_simple("video/x-raw", "format", G_TYPE_STRING, pixelFormat, nullptr));
             }
             g_object_set(self->priv->inputCapsFilter.get(), "caps", inputCaps.get(), nullptr);
             g_object_set(self->priv->outputCapsFilter.get(), "caps", self->priv->encodedCaps.get(), nullptr);
@@ -586,9 +672,34 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
         }, [](GstElement*, LatencyMode) {
             notImplemented();
         });
+
+    auto setVpxEncoderInputFormat = [](auto* self) {
+        g_object_set(self->priv->encoder.get(), "buffer-initial-size", 100, "buffer-optimal-size", 120, "buffer-size" , 150, "max-intra-bitrate", 250, nullptr);
+        gst_util_set_object_arg(G_OBJECT(self->priv->encoder.get()), "error-resilient", "default");
+        auto inputCaps = adoptGRef(gst_caps_new_any());
+        const auto* structure = gst_caps_get_structure(self->priv->encodedCaps.get(), 0);
+        if (const char* profileString = gst_structure_get_string(structure, "profile")) {
+            auto profile = StringView::fromLatin1(profileString);
+            auto profileId = parseInteger<int>(profile, 10);
+            if (profileId) {
+                const char* format = "I420";
+                if (*profileId == 3)
+                    format = "I422_10LE";
+                else if (*profileId == 2)
+                    format = "I420_10LE";
+                else if (*profileId == 1)
+                    format = "Y444";
+                inputCaps = adoptGRef(gst_caps_new_simple("video/x-raw", "format", G_TYPE_STRING, format, nullptr));
+                GST_DEBUG_OBJECT(self, "Encoder expecting input with caps %" GST_PTR_FORMAT, inputCaps.get());
+            }
+        }
+        g_object_set(self->priv->inputCapsFilter.get(), "caps", inputCaps.get(), nullptr);
+    };
+
     Encoders::registerEncoder(Vp8, "vp8enc", nullptr, "video/x-vp8", nullptr,
-        [](WebKitVideoEncoder* self) {
+        [&](WebKitVideoEncoder* self) {
             gst_util_set_object_arg(G_OBJECT(self->priv->encoder.get()), "keyframe-mode", "disabled");
+            setVpxEncoderInputFormat(self);
         }, "target-bitrate", setBitrateBitPerSec, "keyframe-max-dist", [](GstElement* encoder, BitrateMode mode) {
             switch (mode) {
             case CONSTANT_BITRATE_MODE:
@@ -611,16 +722,8 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
         });
 
     Encoders::registerEncoder(Vp9, "vp9enc", nullptr, "video/x-vp9", nullptr,
-        [](WebKitVideoEncoder* self) {
-            auto inputCaps = adoptGRef(gst_caps_new_any());
-            const auto* structure = gst_caps_get_structure(self->priv->encodedCaps.get(), 0);
-            if (const char* profileString = gst_structure_get_string(structure, "profile")) {
-                auto profile = StringView::fromLatin1(profileString);
-                auto profileId = parseInteger<int>(profile, 10);
-                if (profileId && *profileId >= 2)
-                    inputCaps = adoptGRef(gst_caps_new_simple("video/x-raw", "format", G_TYPE_STRING, "I420_10LE", nullptr));
-            }
-            g_object_set(self->priv->inputCapsFilter.get(), "caps", inputCaps.get(), nullptr);
+        [&](WebKitVideoEncoder* self) {
+            setVpxEncoderInputFormat(self);
         }, "target-bitrate", setBitrateBitPerSec, "keyframe-max-dist", [](GstElement* encoder, BitrateMode mode) {
             switch (mode) {
             case CONSTANT_BITRATE_MODE:
@@ -632,9 +735,11 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
             };
         }, [](GstElement* encoder, LatencyMode mode) {
             switch (mode) {
-            case REALTIME_LATENCY_MODE:
-                g_object_set(encoder, "threads", NUMBER_OF_THREADS, "cpu-used", NUMBER_OF_THREADS, "deadline", 1, "lag-in-frames", 0, nullptr);
+            case REALTIME_LATENCY_MODE: {
+                int64_t deadline = 1000000;
+                g_object_set(encoder, "threads", NUMBER_OF_THREADS, "cpu-used", NUMBER_OF_THREADS, "deadline", deadline, "lag-in-frames", 0, nullptr);
                 break;
+            }
             case QUALITY_LATENCY_MODE:
                 g_object_set(encoder, "threads", NUMBER_OF_THREADS, "cpu-used", NUMBER_OF_THREADS, "deadline", 0, "lag-in-frames", 25, nullptr);
                 gst_util_set_object_arg(G_OBJECT(encoder), "end-usage", "cq");
@@ -707,6 +812,151 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
             };
         });
 
+    if (webkitGstCheckVersion(1, 22, 0)) {
+        Encoders::registerEncoder(Av1, "av1enc", "av1parse", "video/x-av1", nullptr,
+            [](WebKitVideoEncoder* self) {
+                g_object_set(self->priv->encoder.get(), "threads", NUMBER_OF_THREADS, nullptr);
+                gst_util_set_object_arg(G_OBJECT(self->priv->encoder.get()), "keyframe-mode", "disabled");
+            }, "target-bitrate", setBitrateKbitPerSec, "keyframe-max-dist", [](GstElement* encoder, BitrateMode mode) {
+                switch (mode) {
+                case CONSTANT_BITRATE_MODE:
+                    gst_util_set_object_arg(G_OBJECT(encoder), "end-usage", "cbr");
+                    break;
+                case VARIABLE_BITRATE_MODE:
+                    gst_util_set_object_arg(G_OBJECT(encoder), "end-usage", "vbr");
+                    break;
+                }
+            }, [](GstElement* encoder, LatencyMode mode) {
+                if (!gstObjectHasProperty(encoder, "usage-profile"))
+                    return;
+                switch (mode) {
+                case REALTIME_LATENCY_MODE:
+                    gst_util_set_object_arg(G_OBJECT(encoder), "usage-profile", "realtime");
+                    break;
+                case QUALITY_LATENCY_MODE:
+                    gst_util_set_object_arg(G_OBJECT(encoder), "usage-profile", "good-quality");
+                    gst_util_set_object_arg(G_OBJECT(encoder), "end-usage", "q");
+                    break;
+                }
+            });
+    }
+
+    static GQuark x265BitrateQuark = g_quark_from_static_string("x265-bitrate-mode");
+    Encoders::registerEncoder(X265, "x265enc", "h265parse", "video/x-h265",
+        "video/x-h265,alignment=au,stream-format=byte-stream",
+        [](WebKitVideoEncoder* self) {
+            g_object_set(self->priv->encoder.get(), "key-int-max", 15, nullptr);
+
+            const auto* structure = gst_caps_get_structure(self->priv->encodedCaps.get(), 0);
+            auto inputCaps = adoptGRef(gst_caps_new_any());
+            if (const char* profileString = gst_structure_get_string(structure, "profile")) {
+                const char* pixelFormat = nullptr;
+                auto pad = adoptGRef(gst_element_get_static_pad(self->priv->encoder.get(), "sink"));
+                auto allowedCaps = adoptGRef(gst_pad_query_caps(pad.get(), nullptr));
+                const auto* structure = gst_caps_get_structure(allowedCaps.get(), 0);
+                const auto* formatValue = gst_structure_get_value(structure, "format");
+                unsigned size = gst_value_list_get_size(formatValue);
+                bool supports10BitsLittleEndian = false;
+                bool supports10BitsBigEndian = false;
+                bool supports12BitsLittleEndian = false;
+                bool supports12BitsBigEndian = false;
+
+                for (unsigned i = 0; i < size; i++) {
+                    auto* value = gst_value_list_get_value(formatValue, i);
+                    const char* format = g_value_get_string(value);
+                    if (g_str_has_suffix(format, "_10LE"))
+                        supports10BitsLittleEndian = true;
+                    if (g_str_has_suffix(format, "_10BE"))
+                        supports10BitsBigEndian = true;
+                    if (g_str_has_suffix(format, "_12LE"))
+                        supports12BitsLittleEndian = true;
+                    if (g_str_has_suffix(format, "_12BE"))
+                        supports12BitsBigEndian = true;
+                }
+
+                StringView profile { profileString, static_cast<unsigned>(strlen(profileString)) };
+                auto is12Bits = profile.findIgnoringASCIICase("-12"_s) != notFound;
+                auto is10Bits = profile.findIgnoringASCIICase("-10"_s) != notFound;
+                auto isY444 = profile.findIgnoringASCIICase("-444"_s) != notFound;
+                auto isY422 = profile.findIgnoringASCIICase("-422"_s) != notFound;
+
+                if (is12Bits) {
+                    if (isY444) {
+                        if (supports12BitsLittleEndian)
+                            pixelFormat = "Y444_12LE";
+                        else if (supports12BitsBigEndian)
+                            pixelFormat = "Y444_12BE";
+                        else
+                            pixelFormat = "Y444";
+                    } else if (isY422) {
+                        if (supports12BitsLittleEndian)
+                            pixelFormat = "Y422_12LE";
+                        else if (supports12BitsBigEndian)
+                            pixelFormat = "Y422_12BE";
+                        else
+                            pixelFormat = "Y42B";
+                    }
+                } else if (is10Bits) {
+                    if (isY444) {
+                        if (supports10BitsLittleEndian)
+                            pixelFormat = "Y444_10LE";
+                        else if (supports10BitsBigEndian)
+                            pixelFormat = "Y444_10BE";
+                        else
+                            pixelFormat = "Y444";
+                    } else if (isY422) {
+                        if (supports10BitsLittleEndian)
+                            pixelFormat = "Y422_10LE";
+                        else if (supports10BitsBigEndian)
+                            pixelFormat = "Y422_10BE";
+                        else
+                            pixelFormat = "Y42B";
+                    } else if (profile == "high-10"_s) {
+                        if (supports10BitsLittleEndian)
+                            pixelFormat = "Y420_10LE";
+                        else if (supports10BitsBigEndian)
+                            pixelFormat = "Y420_10BE";
+                    }
+                } else
+                    pixelFormat = "I420";
+
+                GST_DEBUG("Setting pixel format %s for profile %s", pixelFormat, profileString);
+                if (pixelFormat)
+                    inputCaps = adoptGRef(gst_caps_new_simple("video/x-raw", "format", G_TYPE_STRING, pixelFormat, nullptr));
+            }
+            g_object_set(self->priv->inputCapsFilter.get(), "caps", inputCaps.get(), nullptr);
+            g_object_set(self->priv->outputCapsFilter.get(), "caps", self->priv->encodedCaps.get(), nullptr);
+        }, "bitrate", [](GObject* object, const char* propertyName, int bitrate) {
+            if (UNLIKELY(!bitrate))
+                return;
+            setBitrateKbitPerSec(object, propertyName, bitrate);
+            auto bitrateMode = GPOINTER_TO_INT(g_object_get_qdata(object, x265BitrateQuark));
+            StringBuilder builder;
+            switch (bitrateMode) {
+            case CONSTANT_BITRATE_MODE:
+                builder.append("vbv-maxrate="_s, bitrate, ":vbv-bufsize="_s, bitrate / 2);
+                break;
+            case VARIABLE_BITRATE_MODE:
+                builder.append("vbv-maxrate=0:vbvbufsize=0"_s);
+                break;
+            };
+            auto options = builder.toString();
+            g_object_set(object, "option-string", options.ascii().data(), nullptr);
+        }, "key-int-max", [](GstElement* encoder, BitrateMode mode) {
+            g_object_set_qdata(G_OBJECT(encoder), x265BitrateQuark, GINT_TO_POINTER(mode));
+        }, [](GstElement* encoder, LatencyMode mode) {
+            switch (mode) {
+            case REALTIME_LATENCY_MODE:
+                gst_util_set_object_arg(G_OBJECT(encoder), "tune", "zerolatency");
+                gst_util_set_object_arg(G_OBJECT(encoder), "speed-preset", "ultrafast");
+                break;
+            case QUALITY_LATENCY_MODE:
+                g_object_set(encoder, "tune", 0, nullptr);
+                gst_util_set_object_arg(G_OBJECT(encoder), "speed-preset", "No preset");
+                break;
+            };
+        });
+
     auto srcPadTemplateCaps = createSrcPadTemplateCaps();
     gst_element_class_add_pad_template(elementClass, gst_pad_template_new("src", GST_PAD_SRC, GST_PAD_ALWAYS, srcPadTemplateCaps.get()));
 
@@ -726,5 +976,6 @@ static void webkit_video_encoder_class_init(WebKitVideoEncoderClass* klass)
 }
 
 #undef NUMBER_OF_THREADS
+#undef GST_CAT_DEFAULT
 
 #endif // ENABLE(VIDEO) && USE(GSTREAMER)
