@@ -45,6 +45,8 @@ private:
 
     GRefPtr<GstElement> m_audioconvert;
     GRefPtr<GstElement> m_audioresample;
+    GRefPtr<GstElement> m_inputCapsFilter;
+    GRefPtr<GstCaps> m_inputCaps;
 };
 
 } // namespace WebCore
