@@ -76,9 +76,10 @@ static Result parseMediaType(const String& mediaType)
 struct DecodeTask {
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    DecodeTask(const URL& url, const ScheduleContext& scheduleContext, DecodeCompletionHandler&& completionHandler)
+    DecodeTask(const URL& url, const ScheduleContext& scheduleContext, ShouldValidatePadding shouldValidatePadding, DecodeCompletionHandler&& completionHandler)
         : url(url.isolatedCopy())
         , scheduleContext(scheduleContext)
+        , shouldValidatePadding(shouldValidatePadding)
         , completionHandler(WTFMove(completionHandler))
     {
     }
@@ -133,16 +134,18 @@ public:
     StringView encodedData;
     bool isBase64 { false };
     const ScheduleContext scheduleContext;
+    ShouldValidatePadding shouldValidatePadding;
     DecodeCompletionHandler completionHandler;
 
     Result result;
 };
 
-static std::unique_ptr<DecodeTask> createDecodeTask(const URL& url, const ScheduleContext& scheduleContext, DecodeCompletionHandler&& completionHandler)
+static std::unique_ptr<DecodeTask> createDecodeTask(const URL& url, const ScheduleContext& scheduleContext, ShouldValidatePadding shouldValidatePadding, DecodeCompletionHandler&& completionHandler)
 {
     return makeUnique<DecodeTask>(
         url,
         scheduleContext,
+        shouldValidatePadding,
         WTFMove(completionHandler)
     );
 }
@@ -160,7 +163,8 @@ static std::optional<Result> decodeSynchronously(DecodeTask& task)
         return std::nullopt;
 
     if (task.isBase64) {
-        auto decodedData = base64Decode(PAL::decodeURLEscapeSequences(task.encodedData), Base64DecodeMode::DefaultValidatePaddingAndIgnoreWhitespace);
+        auto mode = task.shouldValidatePadding == ShouldValidatePadding::Yes ? Base64DecodeMode::DefaultValidatePaddingAndIgnoreWhitespace : Base64DecodeMode::DefaultIgnoreWhitespaceForQuirk;
+        auto decodedData = base64Decode(PAL::decodeURLEscapeSequences(task.encodedData), mode);
         if (!decodedData)
             return std::nullopt;
         task.result.data = WTFMove(*decodedData);
@@ -171,11 +175,11 @@ static std::optional<Result> decodeSynchronously(DecodeTask& task)
     return WTFMove(task.result);
 }
 
-void decode(const URL& url, const ScheduleContext& scheduleContext, DecodeCompletionHandler&& completionHandler)
+void decode(const URL& url, const ScheduleContext& scheduleContext, ShouldValidatePadding shouldValidatePadding, DecodeCompletionHandler&& completionHandler)
 {
     ASSERT(url.protocolIsData());
 
-    decodeQueue().dispatch([decodeTask = createDecodeTask(url, scheduleContext, WTFMove(completionHandler))]() mutable {
+    decodeQueue().dispatch([decodeTask = createDecodeTask(url, scheduleContext, shouldValidatePadding, WTFMove(completionHandler))]() mutable {
         auto result = decodeSynchronously(*decodeTask);
 
 #if USE(COCOA_EVENT_LOOP)
@@ -194,11 +198,11 @@ void decode(const URL& url, const ScheduleContext& scheduleContext, DecodeComple
     });
 }
 
-std::optional<Result> decode(const URL& url)
+std::optional<Result> decode(const URL& url, ShouldValidatePadding shouldValidatePadding)
 {
     ASSERT(url.protocolIsData());
 
-    auto task = createDecodeTask(url, { }, nullptr);
+    auto task = createDecodeTask(url, { }, shouldValidatePadding, nullptr);
     return decodeSynchronously(*task);
 }
 
