@@ -99,6 +99,7 @@ void ResourceRequestBase::setAsIsolatedCopy(const ResourceRequest& other)
     setIsAppInitiated(other.isAppInitiated());
     setPrivacyProxyFailClosedForUnreachableNonMainHosts(other.privacyProxyFailClosedForUnreachableNonMainHosts());
     setUseAdvancedPrivacyProtections(other.useAdvancedPrivacyProtections());
+    setDidFilterLinkDecoration(other.didFilterLinkDecoration());
 }
 
 bool ResourceRequestBase::isEmpty() const
@@ -122,11 +123,12 @@ const URL& ResourceRequestBase::url() const
     return m_requestData.m_url;
 }
 
-void ResourceRequestBase::setURL(const URL& url)
+void ResourceRequestBase::setURL(const URL& url, bool didFilterLinkDecoration)
 { 
     updateResourceRequest(); 
 
     m_requestData.m_url = url;
+    m_requestData.m_didFilterLinkDecoration = didFilterLinkDecoration;
     
     m_platformRequestUpdated = false;
 }
@@ -490,11 +492,11 @@ void ResourceRequestBase::setResponseContentDispositionEncodingFallbackArray(con
     m_requestData.m_responseContentDispositionEncodingFallbackArray.clear();
     m_requestData.m_responseContentDispositionEncodingFallbackArray.reserveInitialCapacity(!encoding1.isNull() + !encoding2.isNull() + !encoding3.isNull());
     if (!encoding1.isNull())
-        m_requestData.m_responseContentDispositionEncodingFallbackArray.uncheckedAppend(encoding1);
+        m_requestData.m_responseContentDispositionEncodingFallbackArray.append(encoding1);
     if (!encoding2.isNull())
-        m_requestData.m_responseContentDispositionEncodingFallbackArray.uncheckedAppend(encoding2);
+        m_requestData.m_responseContentDispositionEncodingFallbackArray.append(encoding2);
     if (!encoding3.isNull())
-        m_requestData.m_responseContentDispositionEncodingFallbackArray.uncheckedAppend(encoding3);
+        m_requestData.m_responseContentDispositionEncodingFallbackArray.append(encoding3);
     
     m_platformRequestUpdated = false;
 }
@@ -663,6 +665,13 @@ void ResourceRequestBase::setUseAdvancedPrivacyProtections(bool useAdvancedPriva
     m_platformRequestUpdated = false;
 }
 
+void ResourceRequestBase::setDidFilterLinkDecoration(bool didFilterLinkDecoration)
+{
+    if (m_requestData.m_didFilterLinkDecoration == didFilterLinkDecoration)
+        return;
+    m_requestData.m_didFilterLinkDecoration = didFilterLinkDecoration;
+}
+
 bool equalIgnoringHeaderFields(const ResourceRequestBase& a, const ResourceRequestBase& b)
 {
     if (a.url() != b.url())
