@@ -47,6 +47,7 @@ struct Result;
 }
 
 class ResourceResponse;
+struct ResourceResponseData;
 
 bool isScriptAllowedByNosniff(const ResourceResponse&);
 
@@ -58,89 +59,24 @@ enum class WasPrivateRelayed : bool { No, Yes };
 static constexpr unsigned bitWidthOfWasPrivateRelayed = 1;
 static_assert(static_cast<unsigned>(WasPrivateRelayed::Yes) <= ((1U << bitWidthOfWasPrivateRelayed) - 1));
 
+enum class ResourceResponseBaseType : uint8_t { Basic, Cors, Default, Error, Opaque, Opaqueredirect };
+enum class ResourceResponseBaseTainting : uint8_t { Basic, Cors, Opaque, Opaqueredirect };
+enum class ResourceResponseBaseSource : uint8_t { Unknown, Network, DiskCache, DiskCacheAfterValidation, MemoryCache, MemoryCacheAfterValidation, ServiceWorker, ApplicationCache, DOMCache, InspectorOverride };
+
 // Do not use this class directly, use the class ResourceResponse instead
 class ResourceResponseBase {
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    enum class Type : uint8_t { Basic, Cors, Default, Error, Opaque, Opaqueredirect };
+    using Type = ResourceResponseBaseType;
     static constexpr unsigned bitWidthOfType = 3;
-    enum class Tainting : uint8_t { Basic, Cors, Opaque, Opaqueredirect };
+    using Tainting = ResourceResponseBaseTainting;
     static constexpr unsigned bitWidthOfTainting = 2;
-    enum class Source : uint8_t { Unknown, Network, DiskCache, DiskCacheAfterValidation, MemoryCache, MemoryCacheAfterValidation, ServiceWorker, ApplicationCache, DOMCache, InspectorOverride };
+    using Source = ResourceResponseBaseSource;
     static constexpr unsigned bitWidthOfSource = 4;
 
     static bool isRedirectionStatusCode(int code) { return code == 301 || code == 302 || code == 303 || code == 307 || code == 308; }
 
-    struct CrossThreadData {
-        CrossThreadData(const CrossThreadData&) = delete;
-        CrossThreadData& operator=(const CrossThreadData&) = delete;
-        CrossThreadData() = default;
-        CrossThreadData(CrossThreadData&&) = default;
-        CrossThreadData& operator=(CrossThreadData&&) = default;
-        CrossThreadData(URL&& url, String&& mimeType, long long expectedContentLength, String&& textEncodingName, int httpStatusCode, String&& httpStatusText, String&& httpVersion, HTTPHeaderMap&& httpHeaderFields, std::optional<NetworkLoadMetrics>&& networkLoadMetrics, Source source, Type type, Tainting tainting, bool isRedirected, UsedLegacyTLS usedLegacyTLS, WasPrivateRelayed wasPrivateRelayed, bool isRangeRequested, std::optional<CertificateInfo> certificateInfo)
-            : url(WTFMove(url))
-            , mimeType(WTFMove(mimeType))
-            , expectedContentLength(expectedContentLength)
-            , textEncodingName(WTFMove(textEncodingName))
-            , httpStatusCode(httpStatusCode)
-            , httpStatusText(WTFMove(httpStatusText))
-            , httpVersion(WTFMove(httpVersion))
-            , httpHeaderFields(WTFMove(httpHeaderFields))
-            , networkLoadMetrics(WTFMove(networkLoadMetrics))
-            , source(source)
-            , type(type)
-            , tainting(tainting)
-            , isRedirected(isRedirected)
-            , usedLegacyTLS(usedLegacyTLS)
-            , wasPrivateRelayed(wasPrivateRelayed)
-            , isRangeRequested(isRangeRequested)
-            , certificateInfo(certificateInfo)
-        {
-        }
-
-        WEBCORE_EXPORT CrossThreadData isolatedCopy() const;
-
-        URL url;
-        String mimeType;
-        long long expectedContentLength;
-        String textEncodingName;
-        short httpStatusCode;
-        String httpStatusText;
-        String httpVersion;
-        HTTPHeaderMap httpHeaderFields;
-        std::optional<NetworkLoadMetrics> networkLoadMetrics;
-        Source source;
-        Type type;
-        Tainting tainting;
-        bool isRedirected;
-        UsedLegacyTLS usedLegacyTLS;
-        WasPrivateRelayed wasPrivateRelayed;
-        bool isRangeRequested;
-        std::optional<CertificateInfo> certificateInfo;
-    };
-    
-    struct ResponseData {
-        URL m_url;
-        AtomString m_mimeType;
-        long long m_expectedContentLength;
-        AtomString m_textEncodingName;
-        AtomString m_httpStatusText;
-        AtomString m_httpVersion;
-        HTTPHeaderMap m_httpHeaderFields;
-        Box<WebCore::NetworkLoadMetrics> m_networkLoadMetrics;
-
-        short m_httpStatusCode;
-        std::optional<CertificateInfo> m_certificateInfo;
-        
-        ResourceResponseBase::Source m_source;
-        ResourceResponseBase::Type m_type;
-        ResourceResponseBase::Tainting m_tainting;
-
-        bool m_isRedirected;
-        UsedLegacyTLS m_usedLegacyTLS;
-        WasPrivateRelayed m_wasPrivateRelayed;
-        bool m_isRangeRequested;
-    };
+    using CrossThreadData = ResourceResponseData;
 
     WEBCORE_EXPORT CrossThreadData crossThreadData() const;
     WEBCORE_EXPORT static ResourceResponse fromCrossThreadData(CrossThreadData&&);
@@ -272,11 +208,6 @@ public:
 
     static bool equalForWebKitLegacyChallengeComparison(const ResourceResponse&, const ResourceResponse&);
 
-    template<class Encoder, typename = std::enable_if_t<!std::is_same_v<Encoder, IPC::Encoder>>>
-    void encode(Encoder&) const;
-    template<class Decoder, typename = std::enable_if_t<!std::is_same_v<Decoder, IPC::Decoder>>>
-    static WARN_UNUSED_RETURN bool decode(Decoder&, ResourceResponseBase&);
-
     bool isRangeRequested() const { return m_isRangeRequested; }
     void setAsRangeRequested() { m_isRangeRequested = true; }
 
@@ -284,9 +215,9 @@ public:
 
     WEBCORE_EXPORT static ResourceResponse dataURLResponse(const URL&, const DataURLDecoder::Result&);
     
-    WEBCORE_EXPORT ResourceResponseBase(std::optional<ResponseData>);
+    WEBCORE_EXPORT ResourceResponseBase(std::optional<ResourceResponseData>);
     
-    WEBCORE_EXPORT std::optional<ResponseData> getResponseData() const;
+    WEBCORE_EXPORT std::optional<ResourceResponseData> getResponseData() const;
 
 protected:
     enum InitLevel {
@@ -332,6 +263,7 @@ protected:
     mutable WasPrivateRelayed m_wasPrivateRelayed : bitWidthOfWasPrivateRelayed { WasPrivateRelayed::No };
 
 private:
+    friend struct WTF::Persistence::Coder<ResourceResponse>;
     mutable Markable<Seconds, Seconds::MarkableTraits> m_age;
     mutable Markable<WallTime> m_date;
     mutable Markable<WallTime> m_expires;
@@ -353,146 +285,53 @@ private:
     Type m_type : bitWidthOfType { Type::Default };
 };
 
-template<class Encoder, typename>
-void ResourceResponseBase::encode(Encoder& encoder) const
-{
-    encoder << m_isNull;
-    if (m_isNull)
-        return;
-    lazyInit(AllFields);
+struct ResourceResponseData {
+    ResourceResponseData(const ResourceResponseData&) = delete;
+    ResourceResponseData& operator=(const ResourceResponseData&) = delete;
+    ResourceResponseData() = default;
+    ResourceResponseData(ResourceResponseData&&) = default;
+    ResourceResponseData& operator=(ResourceResponseData&&) = default;
+    ResourceResponseData(URL&& url, String&& mimeType, long long expectedContentLength, String&& textEncodingName, int httpStatusCode, String&& httpStatusText, String&& httpVersion, HTTPHeaderMap&& httpHeaderFields, std::optional<NetworkLoadMetrics>&& networkLoadMetrics, ResourceResponseBaseSource source, ResourceResponseBaseType type, ResourceResponseBaseTainting tainting, bool isRedirected, UsedLegacyTLS usedLegacyTLS, WasPrivateRelayed wasPrivateRelayed, bool isRangeRequested, std::optional<CertificateInfo> certificateInfo)
+        : url(WTFMove(url))
+        , mimeType(WTFMove(mimeType))
+        , expectedContentLength(expectedContentLength)
+        , textEncodingName(WTFMove(textEncodingName))
+        , httpStatusCode(httpStatusCode)
+        , httpStatusText(WTFMove(httpStatusText))
+        , httpVersion(WTFMove(httpVersion))
+        , httpHeaderFields(WTFMove(httpHeaderFields))
+        , networkLoadMetrics(WTFMove(networkLoadMetrics))
+        , source(source)
+        , type(type)
+        , tainting(tainting)
+        , isRedirected(isRedirected)
+        , usedLegacyTLS(usedLegacyTLS)
+        , wasPrivateRelayed(wasPrivateRelayed)
+        , isRangeRequested(isRangeRequested)
+        , certificateInfo(certificateInfo)
+    {
+    }
 
-    encoder << m_url;
-    encoder << m_mimeType;
-    encoder << static_cast<int64_t>(m_expectedContentLength);
-    encoder << m_textEncodingName;
-    encoder << m_httpStatusText;
-    encoder << m_httpVersion;
-    encoder << m_httpHeaderFields;
+    WEBCORE_EXPORT ResourceResponseData isolatedCopy() const;
 
-    encoder << m_httpStatusCode;
-    encoder << m_certificateInfo;
-    encoder << m_source;
-    encoder << m_type;
-    encoder << m_tainting;
-    encoder << m_isRedirected;
-    UsedLegacyTLS usedLegacyTLS = m_usedLegacyTLS;
-    encoder << usedLegacyTLS;
-    WasPrivateRelayed wasPrivateRelayed = m_wasPrivateRelayed;
-    encoder << wasPrivateRelayed;
-    encoder << m_isRangeRequested;
-}
-
-template<class Decoder, typename>
-bool ResourceResponseBase::decode(Decoder& decoder, ResourceResponseBase& response)
-{
-    ASSERT(response.m_isNull);
-    std::optional<bool> responseIsNull;
-    decoder >> responseIsNull;
-    if (!responseIsNull)
-        return false;
-    if (*responseIsNull)
-        return true;
-
-    response.m_isNull = false;
-
-    std::optional<URL> url;
-    decoder >> url;
-    if (!url)
-        return false;
-    response.m_url = WTFMove(*url);
-
-    std::optional<AtomString> mimeType;
-    decoder >> mimeType;
-    if (!mimeType)
-        return false;
-    response.m_mimeType = WTFMove(*mimeType);
-
-    std::optional<int64_t> expectedContentLength;
-    decoder >> expectedContentLength;
-    if (!expectedContentLength)
-        return false;
-    response.m_expectedContentLength = *expectedContentLength;
-
-    std::optional<AtomString> textEncodingName;
-    decoder >> textEncodingName;
-    if (!textEncodingName)
-        return false;
-    response.m_textEncodingName = WTFMove(*textEncodingName);
-
-    std::optional<AtomString> httpStatusText;
-    decoder >> httpStatusText;
-    if (!httpStatusText)
-        return false;
-    response.m_httpStatusText = WTFMove(*httpStatusText);
-
-    std::optional<AtomString> httpVersion;
-    decoder >> httpVersion;
-    if (!httpVersion)
-        return false;
-    response.m_httpVersion = WTFMove(*httpVersion);
-
-    std::optional<HTTPHeaderMap> httpHeaderFields;
-    decoder >> httpHeaderFields;
-    if (!httpHeaderFields)
-        return false;
-    response.m_httpHeaderFields = WTFMove(*httpHeaderFields);
-
-    std::optional<short> httpStatusCode;
-    decoder >> httpStatusCode;
-    if (!httpStatusCode)
-        return false;
-    response.m_httpStatusCode = WTFMove(*httpStatusCode);
-
-    std::optional<std::optional<CertificateInfo>> certificateInfo;
-    decoder >> certificateInfo;
-    if (!certificateInfo)
-        return false;
-    response.m_certificateInfo = WTFMove(*certificateInfo);
-
-    std::optional<Source> source;
-    decoder >> source;
-    if (!source)
-        return false;
-    response.m_source = WTFMove(*source);
-
-    std::optional<Type> type;
-    decoder >> type;
-    if (!type)
-        return false;
-    response.m_type = WTFMove(*type);
-
-    std::optional<Tainting> tainting;
-    decoder >> tainting;
-    if (!tainting)
-        return false;
-    response.m_tainting = WTFMove(*tainting);
-
-    std::optional<bool> isRedirected;
-    decoder >> isRedirected;
-    if (!isRedirected)
-        return false;
-    response.m_isRedirected = WTFMove(*isRedirected);
-
-    std::optional<UsedLegacyTLS> usedLegacyTLS;
-    decoder >> usedLegacyTLS;
-    if (!usedLegacyTLS)
-        return false;
-    response.m_usedLegacyTLS = WTFMove(*usedLegacyTLS);
-
-    std::optional<WasPrivateRelayed> wasPrivateRelayed;
-    decoder >> wasPrivateRelayed;
-    if (!wasPrivateRelayed)
-        return false;
-    response.m_wasPrivateRelayed = WTFMove(*wasPrivateRelayed);
-
-    std::optional<bool> isRangeRequested;
-    decoder >> isRangeRequested;
-    if (!isRangeRequested)
-        return false;
-    response.m_isRangeRequested = WTFMove(*isRangeRequested);
-
-    return true;
-}
+    URL url;
+    String mimeType;
+    long long expectedContentLength;
+    String textEncodingName;
+    short httpStatusCode;
+    String httpStatusText;
+    String httpVersion;
+    HTTPHeaderMap httpHeaderFields;
+    std::optional<NetworkLoadMetrics> networkLoadMetrics;
+    ResourceResponseBase::Source source;
+    ResourceResponseBase::Type type;
+    ResourceResponseBase::Tainting tainting;
+    bool isRedirected;
+    UsedLegacyTLS usedLegacyTLS;
+    WasPrivateRelayed wasPrivateRelayed;
+    bool isRangeRequested;
+    std::optional<CertificateInfo> certificateInfo;
+};
 
 } // namespace WebCore
 
@@ -541,9 +380,9 @@ namespace Persistence {
 class Decoder;
 class Encoder;
 
-template<> struct Coder<WebCore::ResourceResponseBase::CrossThreadData> {
-    WEBCORE_EXPORT static void encode(Encoder&, const WebCore::ResourceResponseBase::CrossThreadData&);
-    WEBCORE_EXPORT static std::optional<WebCore::ResourceResponseBase::CrossThreadData> decode(Decoder&);
+template<> struct Coder<WebCore::ResourceResponseData> {
+    WEBCORE_EXPORT static void encodeForPersistence(Encoder&, const WebCore::ResourceResponseData&);
+    WEBCORE_EXPORT static std::optional<WebCore::ResourceResponseData> decodeForPersistence(Decoder&);
 };
 
 } // namespace Persistence
