@@ -31,6 +31,7 @@
 #include "config.h"
 #include "SQLiteFileSystem.h"
 
+#include "Logging.h"
 #include "SQLiteDatabase.h"
 #include "SQLiteStatement.h"
 #include <pal/crypto/CryptoDigest.h>
@@ -41,6 +42,10 @@
 #include <pal/spi/ios/SQLite3SPI.h>
 #endif
 
+#if PLATFORM(COCOA)
+#include <sys/xattr.h>
+#endif
+
 namespace WebCore {
 
 static constexpr std::array<const char *, 3> databaseFileSuffixes { "", "-shm", "-wal" };
@@ -49,7 +54,7 @@ SQLiteFileSystem::SQLiteFileSystem()
 {
 }
 
-String SQLiteFileSystem::appendDatabaseFileNameToPath(const String& path, const String& fileName)
+String SQLiteFileSystem::appendDatabaseFileNameToPath(StringView path, StringView fileName)
 {
     return FileSystem::pathByAppendingComponent(path, fileName);
 }
@@ -91,10 +96,26 @@ bool SQLiteFileSystem::deleteDatabaseFile(const String& filePath)
     return !fileExists;
 }
 
-void SQLiteFileSystem::moveDatabaseFile(const String& oldFilePath, const String& newFilePath)
+#if PLATFORM(COCOA)
+void SQLiteFileSystem::setCanSuspendLockedFileAttribute(const String& filePath)
 {
+    for (const auto* suffix : databaseFileSuffixes) {
+        String path = filePath + suffix;
+        char excluded = 0xff;
+        auto result = setxattr(FileSystem::fileSystemRepresentation(path).data(), "com.apple.runningboard.can-suspend-locked", &excluded, sizeof(excluded), 0, 0);
+        if (result < 0 && !strcmp(suffix, ""))
+            RELEASE_LOG_ERROR(SQLDatabase, "SQLiteFileSystem::setCanSuspendLockedFileAttribute: setxattr failed: %" PUBLIC_LOG_STRING, strerror(errno));
+    }
+}
+#endif
+
+bool SQLiteFileSystem::moveDatabaseFile(const String& oldFilePath, const String& newFilePath)
+{
+    bool allMoved = true;
     for (const auto* suffix : databaseFileSuffixes)
-        FileSystem::moveFile(makeString(oldFilePath, suffix), makeString(newFilePath, suffix));
+        allMoved &= FileSystem::moveFile(makeString(oldFilePath, suffix), makeString(newFilePath, suffix));
+
+    return allMoved;
 }
 
 #if PLATFORM(IOS_FAMILY)
@@ -125,7 +146,7 @@ std::optional<WallTime> SQLiteFileSystem::databaseModificationTime(const String&
     return FileSystem::fileModificationTime(fileName);
 }
     
-String SQLiteFileSystem::computeHashForFileName(const String& fileName)
+String SQLiteFileSystem::computeHashForFileName(StringView fileName)
 {
     auto cryptoDigest = PAL::CryptoDigest::create(PAL::CryptoDigest::Algorithm::SHA_256);
     auto utf8FileName = fileName.utf8();
