@@ -33,7 +33,6 @@
 #include "EventRegion.h"
 #include "FloatRoundedRect.h"
 #include "GraphicsContext.h"
-#include "HighlightData.h"
 #include "HitTestResult.h"
 #include "ImageBuffer.h"
 #include "InlineIteratorBoxInlines.h"
@@ -47,6 +46,7 @@
 #include "RenderBlock.h"
 #include "RenderCombineText.h"
 #include "RenderElementInlines.h"
+#include "RenderHighlight.h"
 #include "RenderLineBreak.h"
 #include "RenderRubyRun.h"
 #include "RenderRubyText.h"
