@@ -49,7 +49,7 @@ using namespace HTMLNames;
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderCounter);
 
 using CounterMap = HashMap<AtomString, Ref<CounterNode>>;
-using CounterMaps = WeakHashMap<RenderElement, std::unique_ptr<CounterMap>>;
+using CounterMaps = SingleThreadWeakHashMap<RenderElement, std::unique_ptr<CounterMap>>;
 
 static CounterNode* makeCounterNode(RenderElement&, const AtomString& identifier, bool alwaysCreateCounter);
 
@@ -165,23 +165,33 @@ static CounterDirectives listItemCounterDirectives(RenderElement& renderer)
 {
     if (is<RenderListItem>(renderer)) {
         auto& item = downcast<RenderListItem>(renderer);
-        if (auto explicitValue = item.explicitValue())
-            return { *explicitValue, std::nullopt };
-        return { std::nullopt, item.isInReversedOrderedList() ? -1 : 1 };
+        return {
+            .resetValue = std::nullopt,
+            .incrementValue = item.isInReversedOrderedList() ? -1 : 1,
+            .setValue = std::nullopt
+        };
     }
     if (auto element = renderer.element()) {
         if (is<HTMLOListElement>(*element)) {
             auto& list = downcast<HTMLOListElement>(*element);
-            return { list.start(), list.isReversed() ? 1 : -1 };
+            return {
+                .resetValue = list.start(),
+                .incrementValue = list.isReversed() ? 1 : -1,
+                .setValue = std::nullopt
+            };
         }
         if (isHTMLListElement(*element))
-            return { 0, std::nullopt };
+            return {
+                .resetValue = 0,
+                .incrementValue = std::nullopt,
+                .setValue = std::nullopt
+            };
     }
     return { };
 }
 
 struct CounterPlan {
-    bool isReset;
+    OptionSet<CounterNode::Type> type;
     int value;
 };
 
@@ -216,12 +226,25 @@ static std::optional<CounterPlan> planCounter(RenderElement& renderer, const Ato
             directives.resetValue = itemDirectives.resetValue;
         if (!directives.incrementValue)
             directives.incrementValue = itemDirectives.incrementValue;
+        if (!directives.setValue)
+            directives.setValue = itemDirectives.setValue;
     }
 
+    OptionSet<CounterNode::Type> type;
+
+    if (directives.setValue)
+        type.add(CounterNode::Type::Set);
     if (directives.resetValue)
-        return CounterPlan { true, saturatedSum<int>(*directives.resetValue, directives.incrementValue.value_or(0)) };
+        type.add(CounterNode::Type::Reset);
     if (directives.incrementValue)
-        return CounterPlan { false, *directives.incrementValue };
+        type.add(CounterNode::Type::Increment);
+
+    if (directives.setValue)
+        return CounterPlan { type, *directives.setValue };
+    if (directives.resetValue)
+        return CounterPlan { type, saturatedSum<int>(*directives.resetValue, directives.incrementValue.value_or(0)) };
+    if (directives.incrementValue)
+        return CounterPlan { type, *directives.incrementValue };
     return std::nullopt;
 }
 
@@ -246,7 +269,7 @@ struct CounterInsertionPoint {
     RefPtr<CounterNode> previousSibling;
 };
 
-static CounterInsertionPoint findPlaceForCounter(RenderElement& counterOwner, const AtomString& identifier, bool isReset)
+static CounterInsertionPoint findPlaceForCounter(RenderElement& counterOwner, const AtomString& identifier, OptionSet<CounterNode::Type> type)
 {
     // We cannot stop searching for counters with the same identifier before we also
     // check this renderer, because it may affect the positioning in the tree of our counter.
@@ -271,6 +294,7 @@ static CounterInsertionPoint findPlaceForCounter(RenderElement& counterOwner, co
             makeCounterNode(*previousRenderers.takeLast(), identifier, false);
     }
 
+    bool isReset = type.contains(CounterNode::Type::Reset);
     while (currentRenderer) {
         auto currentCounter = makeCounterNode(*currentRenderer, identifier, false);
         if (searchEndRenderer == currentRenderer) {
@@ -373,9 +397,9 @@ static CounterNode* makeCounterNode(RenderElement& renderer, const AtomString& i
 
     auto& maps = counterMaps();
 
-    auto newNode = CounterNode::create(renderer, plan && plan->isReset, plan ? plan->value : 0);
+    auto newNode = CounterNode::create(renderer, plan ? plan->type : OptionSet<CounterNode::Type> { }, plan ? plan->value : 0);
 
-    auto place = findPlaceForCounter(renderer, identifier, plan && plan->isReset);
+    auto place = findPlaceForCounter(renderer, identifier, plan ? plan->type : OptionSet<CounterNode::Type> { });
     if (place.parent)
         place.parent->insertAfter(newNode, place.previousSibling.get(), identifier);
 
@@ -393,7 +417,7 @@ static CounterNode* makeCounterNode(RenderElement& renderer, const AtomString& i
         skipDescendants = currentRenderer->shouldApplyStyleContainment();
         if (!currentRenderer->hasCounterNodeMap())
             continue;
-        CheckedPtr currentCounter = maps.find(*currentRenderer)->value->get(identifier);
+        RefPtr currentCounter = maps.find(*currentRenderer)->value->get(identifier);
         if (!currentCounter)
             continue;
         skipDescendants = true;
@@ -408,9 +432,10 @@ static CounterNode* makeCounterNode(RenderElement& renderer, const AtomString& i
 }
 
 RenderCounter::RenderCounter(Document& document, const CounterContent& counter)
-    : RenderText(document, emptyString())
+    : RenderText(Type::Counter, document, emptyString())
     , m_counter(counter)
 {
+    ASSERT(isRenderCounter());
     view().addCounterNeedingUpdate(*this);
 }
 
@@ -434,17 +459,12 @@ ASCIILiteral RenderCounter::renderName() const
     return "RenderCounter"_s;
 }
 
-bool RenderCounter::isCounter() const
-{
-    return true;
-}
-
 String RenderCounter::originalText() const
 {
     if (!m_counterNode)
         return emptyString();
 
-    CheckedPtr child = m_counterNode;
+    RefPtr child = m_counterNode.get();
     int value = child->actsAsReset() ? child->value() : child->countInParent();
 
     auto counterText = [](const ListStyleType& styleType, int value, CSSCounterStyle* counterStyle) {
@@ -566,6 +586,9 @@ void RenderCounter::rendererStyleChangedSlowCase(RenderElement& renderer, const
                 RenderCounter::destroyCounterNodes(renderer);
         }
     } else {
+        if (renderer.hasCounterNodeMap())
+            RenderCounter::destroyCounterNodes(renderer);
+
         for (auto& key : newStyle.counterDirectives().map.keys()) {
             // We must create this node here, because the added node may be a node with no display such as
             // as those created by the increment or reset directives and the re-layout that will happen will
