@@ -64,16 +64,18 @@ namespace WebCore {
 
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderInline);
 
-RenderInline::RenderInline(Element& element, RenderStyle&& style)
-    : RenderBoxModelObject(element, WTFMove(style), RenderInlineFlag)
+RenderInline::RenderInline(Type type, Element& element, RenderStyle&& style)
+    : RenderBoxModelObject(type, element, WTFMove(style), TypeFlag::IsRenderInline, { })
 {
     setChildrenInline(true);
+    ASSERT(isRenderInline());
 }
 
-RenderInline::RenderInline(Document& document, RenderStyle&& style)
-    : RenderBoxModelObject(document, WTFMove(style), RenderInlineFlag)
+RenderInline::RenderInline(Type type, Document& document, RenderStyle&& style)
+    : RenderBoxModelObject(type, document, WTFMove(style), TypeFlag::IsRenderInline, { })
 {
     setChildrenInline(true);
+    ASSERT(isRenderInline());
 }
 
 void RenderInline::willBeDestroyed()
@@ -196,13 +198,14 @@ void RenderInline::styleDidChange(StyleDifference diff, const RenderStyle* oldSt
 
     if (diff >= StyleDifference::Repaint) {
         if (auto* lineLayout = LayoutIntegration::LineLayout::containing(*this)) {
-            auto shouldInvalidateLineLayoutPath = selfNeedsLayout() || !LayoutIntegration::LineLayout::canUseForAfterInlineBoxStyleChange(*this, diff);
-            if (shouldInvalidateLineLayoutPath)
+            if (selfNeedsLayout())
                 lineLayout->flow().invalidateLineLayoutPath();
             else
                 lineLayout->updateStyle(*this, *oldStyle);
         }
     }
+
+    propagateStyleToAnonymousChildren(PropagateToAllChildren);
 }
 
 bool RenderInline::mayAffectLayout() const
@@ -343,6 +346,102 @@ LayoutUnit RenderInline::offsetTop() const
     return adjustedPositionRelativeToOffsetParent(firstInlineBoxTopLeft()).y();
 }
 
+LayoutUnit RenderInline::offsetWidth() const
+{
+    auto offsetWidthForContinuation = [&]() -> std::optional<LayoutUnit> {
+        if (!continuation())
+            return { };
+        // In case of continuation we simply use the enclosing block's width for all the inline boxes participating in this continuation matching other engines.
+        CheckedPtr preBlock = this->containingBlock();
+        if (!preBlock || !preBlock->isAnonymous()) {
+            ASSERT_NOT_REACHED();
+            return { };
+        }
+        return { preBlock->offsetWidth() };
+    };
+    if (auto offsetWidth = offsetWidthForContinuation())
+        return *offsetWidth;
+    return linesBoundingBox().width();
+}
+
+static bool hasValidContinuation(const RenderInline& renderer)
+{
+    CheckedPtr preBlock = renderer.containingBlock();
+    if (!preBlock || !preBlock->isAnonymous())
+        return false;
+
+    CheckedPtr middleBlock = dynamicDowncast<RenderBlock>(preBlock->nextInFlowSibling());
+    if (!middleBlock || !middleBlock->isAnonymous() || !middleBlock->isContinuation())
+        return false;
+
+    CheckedPtr postBlock = dynamicDowncast<RenderBlock>(middleBlock->nextInFlowSibling());
+    if (!postBlock || !postBlock->isAnonymous())
+        return false;
+
+    return true;
+}
+
+static inline std::optional<LayoutUnit> offsetHeightForMiddleAndPostBlocks(const RenderBlock& middleBlock)
+{
+    // At this point we've got only middle and post blocks potentially followed by another nested continuation.
+    if (!middleBlock.isContinuation()) {
+        ASSERT_NOT_REACHED();
+        return { };
+    }
+    // post-block is supposed to include all inline content after the intrusive block.
+    // e.g. <span><span><block></block>this is after content</span>this too</span.
+    CheckedPtr postBlock = dynamicDowncast<RenderBlock>(middleBlock.nextInFlowSibling());
+    if (!postBlock || !postBlock->isAnonymous()) {
+        ASSERT_NOT_REACHED();
+        return { };
+    }
+    // With multiple intrusive blocks we end up with a chain of continuations after the post-block starting with
+    // another middle block for each nesting.
+    // e.g. <span><span><div></div><span><div>
+    CheckedPtr nestedContinuationCandidate = dynamicDowncast<RenderBlock>(postBlock->nextInFlowSibling());
+    if (!nestedContinuationCandidate || !nestedContinuationCandidate->isContinuation())
+        return { middleBlock.offsetHeight() + postBlock->offsetHeight() };
+
+    auto nestedOffsetHeight = offsetHeightForMiddleAndPostBlocks(*nestedContinuationCandidate);
+    if (!nestedOffsetHeight) {
+        ASSERT_NOT_REACHED();
+        return { };
+    }
+    return { middleBlock.offsetHeight() + postBlock->offsetHeight() + *nestedOffsetHeight };
+};
+
+static inline std::optional<LayoutUnit> offsetHeightForContinuation(const RenderInline& renderer)
+{
+    if (!renderer.continuation())
+        return { };
+
+    if (!hasValidContinuation(renderer)) {
+        ASSERT_NOT_REACHED();
+        return { };
+    }
+    // Whether this is the continuation initiating inline (<span id=initiating><div>) or just participating (<span id=participating><span id=initiating><div>)
+    // we have to find the closest anonymous block container constructed for continuation (pre-block) and
+    // use that as the anchor renderer to walk and compute the enclosing height.
+
+    // pre-block is supposed to enclose all inline content in front of the intrusive block.
+    // e.g. <span>this is pre content<span>this too<div>
+    // while middle-block is the intrusive block container.
+    CheckedPtr preBlock = renderer.containingBlock();
+    CheckedPtr middleBlock = dynamicDowncast<RenderBlock>(preBlock->nextInFlowSibling());
+    if (auto middleAndPostBlocksOffsetHeight = offsetHeightForMiddleAndPostBlocks(*middleBlock))
+        return { preBlock->offsetHeight() + *middleAndPostBlocksOffsetHeight };
+
+    ASSERT_NOT_REACHED();
+    return { };
+}
+
+LayoutUnit RenderInline::offsetHeight() const
+{
+    if (auto offsetHeight = offsetHeightForContinuation(*this))
+        return *offsetHeight;
+    return linesBoundingBox().height();
+}
+
 LayoutPoint RenderInline::firstInlineBoxTopLeft() const
 {
     if (auto* lineLayout = LayoutIntegration::LineLayout::containing(*this))
@@ -464,10 +563,17 @@ LayoutUnit RenderInline::innerPaddingBoxWidth() const
 
     if (LayoutIntegration::LineLayout::containing(*this)) {
         if (auto inlineBox = InlineIterator::firstInlineBoxFor(*this)) {
-            firstInlineBoxPaddingBoxLeft = inlineBox->logicalLeftIgnoringInlineDirection() + borderStart();
-            for (; inlineBox->nextInlineBox(); inlineBox.traverseNextInlineBox()) { }
-            ASSERT(inlineBox);
-            lastInlineBoxPaddingBoxRight = inlineBox->logicalRightIgnoringInlineDirection() - borderEnd();
+            if (style().isLeftToRightDirection()) {
+                firstInlineBoxPaddingBoxLeft = inlineBox->logicalLeftIgnoringInlineDirection() + borderStart();
+                for (; inlineBox->nextInlineBox(); inlineBox.traverseNextInlineBox()) { }
+                ASSERT(inlineBox);
+                lastInlineBoxPaddingBoxRight = inlineBox->logicalRightIgnoringInlineDirection() - borderEnd();
+            } else {
+                lastInlineBoxPaddingBoxRight = inlineBox->logicalRightIgnoringInlineDirection() - borderStart();
+                for (; inlineBox->nextInlineBox(); inlineBox.traverseNextInlineBox()) { }
+                ASSERT(inlineBox);
+                firstInlineBoxPaddingBoxLeft = inlineBox->logicalLeftIgnoringInlineDirection() + borderEnd();
+            }
             return std::max(0_lu, lastInlineBoxPaddingBoxRight - firstInlineBoxPaddingBoxLeft);
         }
         return { };
@@ -637,7 +743,7 @@ LayoutRect RenderInline::clippedOverflowRect(const RenderLayerModelObject* repai
     if (knownEmpty())
         return LayoutRect();
 
-    LayoutRect repaintRect(linesVisualOverflowBoundingBox());
+    auto repaintRect = linesVisualOverflowBoundingBox();
     bool hitRepaintContainer = false;
 
     // We need to add in the in-flow position offsets of any inlines (including us) up to our
@@ -659,10 +765,13 @@ LayoutRect RenderInline::clippedOverflowRect(const RenderLayerModelObject* repai
     if (hitRepaintContainer || !containingBlock)
         return repaintRect;
 
-    if (containingBlock->hasNonVisibleOverflow())
-        containingBlock->applyCachedClipAndScrollPosition(repaintRect, repaintContainer, context);
+    auto rects = RepaintRects { repaintRect };
 
-    repaintRect = containingBlock->computeRect(repaintRect, repaintContainer, context);
+    if (containingBlock->hasNonVisibleOverflow())
+        containingBlock->applyCachedClipAndScrollPosition(rects, repaintContainer, context);
+
+    rects = containingBlock->computeRects(rects, repaintContainer, context);
+    repaintRect = rects.clippedOverflowRect;
 
     if (outlineSize) {
         for (auto& child : childrenOfType<RenderElement>(*this))
@@ -677,6 +786,12 @@ LayoutRect RenderInline::clippedOverflowRect(const RenderLayerModelObject* repai
     return repaintRect;
 }
 
+auto RenderInline::rectsForRepaintingAfterLayout(const RenderLayerModelObject* repaintContainer, RepaintOutlineBounds) const -> RepaintRects
+{
+    // RepaintOutlineBounds is unused for inlines.
+    return { clippedOverflowRect(repaintContainer, visibleRectContextForRepaint()) };
+}
+
 LayoutRect RenderInline::rectWithOutlineForRepaint(const RenderLayerModelObject* repaintContainer, LayoutUnit outlineWidth) const
 {
     LayoutRect r(RenderBoxModelObject::rectWithOutlineForRepaint(repaintContainer, outlineWidth));
@@ -685,64 +800,61 @@ LayoutRect RenderInline::rectWithOutlineForRepaint(const RenderLayerModelObject*
     return r;
 }
 
-LayoutRect RenderInline::computeVisibleRectUsingPaintOffset(const LayoutRect& rect) const
+auto RenderInline::computeVisibleRectsUsingPaintOffset(const RepaintRects& rects) const -> RepaintRects
 {
-    LayoutRect adjustedRect = rect;
+    auto adjustedRects = rects;
     auto* layoutState = view().frameView().layoutContext().layoutState();
     if (style().hasInFlowPosition() && layer())
-        adjustedRect.move(layer()->offsetForInFlowPosition());
-    adjustedRect.move(layoutState->paintOffset());
+        adjustedRects.move(layer()->offsetForInFlowPosition());
+    adjustedRects.move(layoutState->paintOffset());
     if (layoutState->isClipped())
-        adjustedRect.intersect(layoutState->clipRect());
-    return adjustedRect;
+        adjustedRects.clippedOverflowRect.intersect(layoutState->clipRect());
+    return adjustedRects;
 }
 
-std::optional<LayoutRect> RenderInline::computeVisibleRectInContainer(const LayoutRect& rect, const RenderLayerModelObject* container, VisibleRectContext context) const
+auto RenderInline::computeVisibleRectsInContainer(const RepaintRects& rects, const RenderLayerModelObject* container, VisibleRectContext context) const -> std::optional<RepaintRects>
 {
     // Repaint offset cache is only valid for root-relative repainting
     if (view().frameView().layoutContext().isPaintOffsetCacheEnabled() && !container && !context.options.contains(VisibleRectContextOption::UseEdgeInclusiveIntersection))
-        return computeVisibleRectUsingPaintOffset(rect);
+        return computeVisibleRectsUsingPaintOffset(rects);
 
     if (container == this)
-        return rect;
+        return rects;
 
     bool containerSkipped;
     RenderElement* localContainer = this->container(container, containerSkipped);
     if (!localContainer)
-        return rect;
-
-    LayoutRect adjustedRect = rect;
-    LayoutPoint topLeft = adjustedRect.location();
+        return rects;
 
+    auto adjustedRects = rects;
     if (style().hasInFlowPosition() && layer()) {
         // Apply the in-flow position offset when invalidating a rectangle. The layer
         // is translated, but the render box isn't, so we need to do this to get the
         // right dirty rect. Since this is called from RenderObject::setStyle, the relative or sticky position
         // flag on the RenderObject has been cleared, so use the one on the style().
-        topLeft += layer()->offsetForInFlowPosition();
+        auto offsetForInFlowPosition = layer()->offsetForInFlowPosition();
+        adjustedRects.move(offsetForInFlowPosition);
     }
-    
-    // FIXME: We ignore the lightweight clipping rect that controls use, since if |o| is in mid-layout,
-    // its controlClipRect will be wrong. For overflow clip we use the values cached by the layer.
-    adjustedRect.setLocation(topLeft);
+
     if (localContainer->hasNonVisibleOverflow()) {
         // FIXME: Respect the value of context.options.
         SetForScope change(context.options, context.options | VisibleRectContextOption::ApplyCompositedContainerScrolls);
-        bool isEmpty = !downcast<RenderLayerModelObject>(*localContainer).applyCachedClipAndScrollPosition(adjustedRect, container, context);
+        bool isEmpty = !downcast<RenderLayerModelObject>(*localContainer).applyCachedClipAndScrollPosition(adjustedRects, container, context);
         if (isEmpty) {
             if (context.options.contains(VisibleRectContextOption::UseEdgeInclusiveIntersection))
                 return std::nullopt;
-            return adjustedRect;
+            return adjustedRects;
         }
     }
 
     if (containerSkipped) {
         // If the repaintContainer is below o, then we need to map the rect into repaintContainer's coordinates.
-        LayoutSize containerOffset = container->offsetFromAncestorContainer(*localContainer);
-        adjustedRect.move(-containerOffset);
-        return adjustedRect;
+        auto containerOffset = container->offsetFromAncestorContainer(*localContainer);
+        adjustedRects.move(-containerOffset);
+        return adjustedRects;
     }
-    return localContainer->computeVisibleRectInContainer(adjustedRect, container, context);
+
+    return localContainer->computeVisibleRectsInContainer(adjustedRects, container, context);
 }
 
 LayoutSize RenderInline::offsetFromContainer(RenderElement& container, const LayoutPoint&, bool* offsetDependsOnPoint) const
