@@ -1,5 +1,6 @@
 /**
  * Copyright (C) 2003-2023 Apple Inc. All rights reserved.
+ * Copyright (C) 2023 Igalia S.L.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,15 +19,16 @@
 
 #pragma once
 
+#include "RenderElementInlines.h"
 #include "RenderLayer.h"
 #include "RenderObjectInlines.h"
+#include "RenderSVGResourceClipper.h"
 
 namespace WebCore {
 
 inline bool RenderLayer::canPaintTransparencyWithSetOpacity() const { return isBitmapOnly() && !hasNonOpacityTransparency(); }
 inline bool RenderLayer::hasBackdropFilter() const { return renderer().hasBackdropFilter(); }
 inline bool RenderLayer::hasFilter() const { return renderer().hasFilter(); }
-inline bool RenderLayer::hasNonOpacityTransparency() const { return renderer().hasMask() || hasBlendMode() || (isolatesBlending() && !renderer().isDocumentElementRenderer()); }
 inline bool RenderLayer::hasPerspective() const { return renderer().style().hasPerspective(); }
 inline bool RenderLayer::isTransformed() const { return renderer().isTransformed(); }
 inline bool RenderLayer::isTransparent() const { return renderer().isTransparent() || renderer().hasMask(); }
@@ -41,7 +43,7 @@ inline bool RenderLayer::hasBlendMode() const { return renderer().hasBlendMode()
 inline bool RenderLayer::canUseOffsetFromAncestor() const
 {
     // FIXME: This really needs to know if there are transforms on this layer and any of the layers between it and the ancestor in question.
-    return !isTransformed() && !renderer().isSVGRootOrLegacySVGRoot();
+    return !isTransformed() && !renderer().isRenderOrLegacyRenderSVGRoot();
 }
 
 inline bool RenderLayer::paintsWithTransparency(OptionSet<PaintBehavior> paintBehavior) const
@@ -51,4 +53,31 @@ inline bool RenderLayer::paintsWithTransparency(OptionSet<PaintBehavior> paintBe
     return (paintBehavior & PaintBehavior::FlattenCompositingLayers) || !isComposited();
 }
 
+inline bool RenderLayer::hasNonOpacityTransparency() const
+{
+    if (renderer().hasMask())
+        return true;
+
+    if (hasBlendMode() || isolatesBlending())
+        return true;
+
+#if ENABLE(LAYER_BASED_SVG_ENGINE)
+    if (!renderer().document().settings().layerBasedSVGEngineEnabled())
+        return false;
+
+    // SVG clip-paths may use clipping masks, if so, flag this layer as transparent.
+    if (auto* svgClipper = renderer().svgClipperResourceFromStyle(); svgClipper && !svgClipper->shouldApplyPathClipping())
+        return true;
+#endif
+
+    return false;
+}
+
+#if ENABLE(LAYER_BASED_SVG_ENGINE)
+inline RenderSVGHiddenContainer* RenderLayer::enclosingSVGHiddenOrResourceContainer() const
+{
+    return m_enclosingSVGHiddenOrResourceContainer.get();
+}
+#endif
+
 } // namespace WebCore
