@@ -50,8 +50,9 @@ using namespace HTMLNames;
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderListItem);
 
 RenderListItem::RenderListItem(Element& element, RenderStyle&& style)
-    : RenderBlockFlow(element, WTFMove(style))
+    : RenderBlockFlow(Type::ListItem, element, WTFMove(style))
 {
+    ASSERT(isRenderListItem());
     setInline(false);
 }
 
@@ -82,7 +83,7 @@ RenderStyle RenderListItem::computeMarkerStyle() const
     markerStyle.fontCascade().update(&document().fontSelector());
     markerStyle.setUnicodeBidi(UnicodeBidi::Isolate);
     markerStyle.setWhiteSpaceCollapse(WhiteSpaceCollapse::Preserve);
-    markerStyle.setTextWrap(TextWrap::NoWrap);
+    markerStyle.setTextWrapMode(TextWrapMode::NoWrap);
     markerStyle.setTextTransform({ });
     return markerStyle;
 }
@@ -221,6 +222,7 @@ void RenderListItem::updateValueNow() const
 
     // The start item is either the closest item before this one in the list that already has a value,
     // or the first item in the list if none have before this have values yet.
+    // FIXME: This should skip over items with counter-reset.
     auto* startItem = this;
     if (list) {
         auto* item = this;
@@ -231,25 +233,58 @@ void RenderListItem::updateValueNow() const
         }
     }
 
+    int defaultIncrement = orderedList && orderedList->isReversed() ? -1 : 1;
+    auto valueForItem = [&](int previousValue, CounterDirectives& directives) {
+        if (directives.setValue)
+            return *directives.setValue;
+        int increment = directives.incrementValue.value_or(defaultIncrement);
+        if (directives.resetValue)
+            return *directives.resetValue + increment;
+        return previousValue + increment;
+    };
+
     auto& startValue = startItem->m_value;
-    if (!startValue)
-        startValue = orderedList ? orderedList->start() : 1;
+    if (!startValue) {
+        // Take in account enclosing list counter-reset.
+        // FIXME: This can be a lot more simple when lists use presentational hints.
+        if (list && list->renderer()) {
+            auto listDirectives = list->renderer()->style().counterDirectives().map.get("list-item"_s);
+            if (listDirectives.resetValue)
+                startValue = *listDirectives.resetValue;
+            else
+                startValue = orderedList ? orderedList->start() - defaultIncrement : 0;
+        }
+        auto directives = startItem->style().counterDirectives().map.get("list-item"_s);
+        startValue = valueForItem(startValue.value_or(0), directives);
+    }
+
     int value = *startValue;
-    int increment = (orderedList && orderedList->isReversed()) ? -1 : 1;
 
     for (auto* item = startItem; item != this; ) {
         item = nextListItem(*list, *item);
-        item->m_value = (value += increment);
+        auto directives = item->style().counterDirectives().map.get("list-item"_s);
+        item->m_value = valueForItem(value, directives);
+        // counter-reset creates a new nested counter, so it should not be counted towards the current counter.
+        if (!directives.resetValue)
+            value = *item->m_value;
     }
 }
 
 void RenderListItem::updateValue()
 {
-    if (!m_valueWasSetExplicitly) {
-        m_value = std::nullopt;
-        if (m_marker)
-            m_marker->setNeedsLayoutAndPrefWidthsRecalc();
-    }
+    m_value = std::nullopt;
+    if (m_marker)
+        m_marker->setNeedsLayoutAndPrefWidthsRecalc();
+}
+
+void RenderListItem::styleDidChange(StyleDifference diff, const RenderStyle* oldStyle)
+{
+    RenderBlockFlow::styleDidChange(diff, oldStyle);
+
+    if (!oldStyle || oldStyle->counterDirectives().map.get("list-item"_s) == style().counterDirectives().map.get("list-item"_s))
+        return;
+
+    counterDirectivesChanged();
 }
 
 void RenderListItem::layout()
@@ -298,7 +333,7 @@ StringView RenderListItem::markerTextWithSuffix() const
     return m_marker->textWithSuffix();
 }
 
-void RenderListItem::explicitValueChanged()
+void RenderListItem::counterDirectivesChanged()
 {
     if (m_marker)
         m_marker->setNeedsLayoutAndPrefWidthsRecalc();
@@ -312,20 +347,6 @@ void RenderListItem::explicitValueChanged()
         item->updateValue();
 }
 
-void RenderListItem::setExplicitValue(std::optional<int> value)
-{
-    if (!value) {
-        if (!m_valueWasSetExplicitly)
-            return;
-    } else {
-        if (m_valueWasSetExplicitly && m_value == value)
-            return;
-    }
-    m_valueWasSetExplicitly = value.has_value();
-    m_value = value;
-    explicitValueChanged();
-}
-
 void RenderListItem::updateListMarkerNumbers()
 {
     auto* list = enclosingList(*this);
