@@ -56,11 +56,12 @@ using namespace HTMLNames;
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderSearchField);
 
 RenderSearchField::RenderSearchField(HTMLInputElement& element, RenderStyle&& style)
-    : RenderTextControlSingleLine(element, WTFMove(style))
+    : RenderTextControlSingleLine(Type::SearchField, element, WTFMove(style))
     , m_searchPopupIsVisible(false)
     , m_searchPopup(nullptr)
 {
     ASSERT(element.isSearchField());
+    ASSERT(isRenderSearchField());
 }
 
 RenderSearchField::~RenderSearchField()
