@@ -38,8 +38,9 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderTextControlMultiLine);
 
 RenderTextControlMultiLine::RenderTextControlMultiLine(HTMLTextAreaElement& element, RenderStyle&& style)
-    : RenderTextControl(element, WTFMove(style))
+    : RenderTextControl(Type::TextControlMultiLine, element, WTFMove(style))
 {
+    ASSERT(isRenderTextControlMultiLine());
 }
 
 RenderTextControlMultiLine::~RenderTextControlMultiLine()
