@@ -59,9 +59,10 @@ using namespace HTMLNames;
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderTextControlSingleLine);
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderTextControlInnerBlock);
 
-RenderTextControlSingleLine::RenderTextControlSingleLine(HTMLInputElement& element, RenderStyle&& style)
-    : RenderTextControl(element, WTFMove(style))
+RenderTextControlSingleLine::RenderTextControlSingleLine(Type type, HTMLInputElement& element, RenderStyle&& style)
+    : RenderTextControl(type, element, WTFMove(style))
 {
+    ASSERT(isRenderTextControlSingleLine());
 }
 
 RenderTextControlSingleLine::~RenderTextControlSingleLine() = default;
@@ -153,11 +154,11 @@ void RenderTextControlSingleLine::layout()
         oldContainerLogicalTop = containerRenderer->logicalTop();
         LayoutUnit containerLogicalHeight = containerRenderer->logicalHeight();
 
-        auto* autoFillStrongPasswordButtonRenderer = [&]() -> RenderBox* {
+        CheckedPtr autoFillStrongPasswordButtonRenderer = [&]() -> RenderBox* {
             if (!inputElement().hasAutoFillStrongPasswordButton())
                 return nullptr;
 
-            auto* autoFillButtonElement = inputElement().autoFillButtonElement();
+            CheckedPtr autoFillButtonElement = inputElement().autoFillButtonElement();
             if (!autoFillButtonElement)
                 return nullptr;
 
@@ -487,8 +488,9 @@ HTMLInputElement& RenderTextControlSingleLine::inputElement() const
 }
 
 RenderTextControlInnerBlock::RenderTextControlInnerBlock(Element& element, RenderStyle&& style)
-    : RenderBlockFlow(element, WTFMove(style))
+    : RenderBlockFlow(Type::TextControlInnerBlock, element, WTFMove(style))
 {
+    ASSERT(isRenderTextControlInnerBlock());
 }
 
 }
