@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2021-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,22 +26,30 @@
 #include "TextBoxPainter.h"
 
 #include "CompositionHighlight.h"
+#include "DocumentInlines.h"
 #include "DocumentMarkerController.h"
 #include "Editor.h"
 #include "EventRegion.h"
 #include "GraphicsContext.h"
 #include "HTMLAnchorElement.h"
+#include "InlineIteratorBoxInlines.h"
 #include "InlineIteratorLineBox.h"
+#include "InlineIteratorTextBoxInlines.h"
 #include "InlineTextBoxStyle.h"
 #include "LegacyInlineTextBox.h"
 #include "LineSelection.h"
 #include "PaintInfo.h"
 #include "RenderBlock.h"
+#include "RenderBoxModelObjectInlines.h"
 #include "RenderCombineText.h"
+#include "RenderElementInlines.h"
 #include "RenderText.h"
+#include "RenderTheme.h"
 #include "RenderView.h"
+#include "RenderedDocumentMarker.h"
 #include "ShadowData.h"
 #include "StyledMarkedText.h"
+#include "TextDecorationThickness.h"
 #include "TextPaintStyle.h"
 #include "TextPainter.h"
 
@@ -79,7 +87,7 @@ TextBoxPainter<TextBoxPath>::TextBoxPainter(TextBoxPath&& textBox, PaintInfo& pa
     , m_useCustomUnderlines(m_containsComposition && m_renderer.frame().editor().compositionUsesCustomUnderlines())
     , m_emphasisMarkExistsAndIsAbove(RenderText::emphasisMarkExistsAndIsAbove(m_renderer, m_style))
 {
-    ASSERT(paintInfo.phase == PaintPhase::Foreground || paintInfo.phase == PaintPhase::Selection || paintInfo.phase == PaintPhase::TextClip || paintInfo.phase == PaintPhase::EventRegion);
+    ASSERT(paintInfo.phase == PaintPhase::Foreground || paintInfo.phase == PaintPhase::Selection || paintInfo.phase == PaintPhase::TextClip || paintInfo.phase == PaintPhase::EventRegion || paintInfo.phase == PaintPhase::Accessibility);
 }
 
 template<typename TextBoxPath>
@@ -103,13 +111,18 @@ void TextBoxPainter<TextBoxPath>::paint()
     if (m_paintInfo.phase == PaintPhase::EventRegion) {
         constexpr OptionSet<HitTestRequest::Type> hitType { HitTestRequest::Type::IgnoreCSSPointerEventsProperty };
         if (m_renderer.parent()->visibleToHitTesting(hitType))
-            m_paintInfo.eventRegionContext->unite(enclosingIntRect(m_paintRect), const_cast<RenderText&>(m_renderer), m_style);
+            m_paintInfo.eventRegionContext()->unite(FloatRoundedRect(m_paintRect), const_cast<RenderText&>(m_renderer), m_style);
+        return;
+    }
+
+    if (m_paintInfo.phase == PaintPhase::Accessibility) {
+        m_paintInfo.accessibilityRegionContext()->takeBounds(m_renderer, m_paintRect);
         return;
     }
 
     bool shouldRotate = !textBox().isHorizontal() && !m_isCombinedText;
     if (shouldRotate)
-        m_paintInfo.context().concatCTM(rotation(m_paintRect, Clockwise));
+        m_paintInfo.context().concatCTM(rotation(m_paintRect, RotationDirection::Clockwise));
 
     if (m_paintInfo.phase == PaintPhase::Foreground) {
         if (!m_isPrinting)
@@ -124,11 +137,11 @@ void TextBoxPainter<TextBoxPath>::paint()
         if (m_useCustomUnderlines)
             paintCompositionUnderlines();
 
-        m_renderer.page().addRelevantRepaintedObject(const_cast<RenderText*>(&m_renderer), enclosingLayoutRect(m_paintRect));
+        m_renderer.page().addRelevantRepaintedObject(m_renderer, enclosingLayoutRect(m_paintRect));
     }
 
     if (shouldRotate)
-        m_paintInfo.context().concatCTM(rotation(m_paintRect, Counterclockwise));
+        m_paintInfo.context().concatCTM(rotation(m_paintRect, RotationDirection::Counterclockwise));
 }
 
 template<typename TextBoxPath>
@@ -136,7 +149,7 @@ MarkedText TextBoxPainter<TextBoxPath>::createMarkedTextFromSelectionInBox()
 {
     auto [selectionStart, selectionEnd] = m_renderer.view().selection().rangeForTextBox(m_renderer, m_selectableRange);
     if (selectionStart < selectionEnd)
-        return { selectionStart, selectionEnd, MarkedText::Selection };
+        return { selectionStart, selectionEnd, MarkedText::Type::Selection };
     return { };
 }
 
@@ -187,13 +200,91 @@ void TextBoxPainter<TextBoxPath>::paintBackground()
         paintBackground(markedText);
 }
 
+template<typename TextBoxPath>
+void TextBoxPainter<TextBoxPath>::paintCompositionForeground(const StyledMarkedText& markedText)
+{
+    auto& editor = m_renderer.frame().editor();
+
+    auto highlights = editor.customCompositionHighlights();
+    Vector<CompositionHighlight> highlightsWithForeground;
+
+    if (!(editor.compositionUsesCustomHighlights() && m_containsComposition)) {
+        paintForeground(markedText);
+        return;
+    }
+
+    // The highlight ranges must be "packed" so that there is no non-empty interval between
+    // any two adjacent highlight ranges. This is needed since otherwise, `paintForeground`
+    // will not be called in those would-be non-empty intervals.
+    highlightsWithForeground.append({ textBox().start(), highlights[0].startOffset, { }, { } });
+
+    for (size_t i = 0; i < highlights.size(); ++i) {
+        highlightsWithForeground.append(highlights[i]);
+        if (i != highlights.size() - 1)
+            highlightsWithForeground.append({ highlights[i].endOffset, highlights[i + 1].startOffset, { }, { } });
+    }
+
+    highlightsWithForeground.append({ highlights.last().endOffset, textBox().end(), { }, { } });
+
+    auto& lineStyle = m_isFirstLine ? m_renderer.firstLineStyle() : m_renderer.style();
+
+    for (auto& highlight : highlightsWithForeground) {
+        auto style = StyledMarkedText::computeStyleForUnmarkedMarkedText(m_renderer, lineStyle, m_isFirstLine, m_paintInfo);
+
+        if (highlight.endOffset <= textBox().start())
+            continue;
+
+        if (highlight.startOffset >= textBox().end())
+            break;
+
+        auto [clampedStart, clampedEnd] = m_selectableRange.clamp(highlight.startOffset, highlight.endOffset);
+
+        if (highlight.foregroundColor)
+            style.textStyles.fillColor = *highlight.foregroundColor;
+
+        paintForeground({ MarkedText { clampedStart, clampedEnd, MarkedText::Type::Unmarked }, style });
+
+        if (highlight.endOffset > textBox().end())
+            break;
+    }
+}
+
 template<typename TextBoxPath>
 void TextBoxPainter<TextBoxPath>::paintForegroundAndDecorations()
 {
     auto shouldPaintSelectionForeground = m_haveSelection && !m_useCustomUnderlines;
     auto hasTextDecoration = !m_style.textDecorationsInEffect().isEmpty();
     auto hasHighlightDecoration = m_document.hasHighlight() && !MarkedText::collectForHighlights(m_renderer, m_selectableRange, MarkedText::PaintPhase::Decoration).isEmpty();
-    auto hasDecoration = hasTextDecoration || hasHighlightDecoration;
+    auto hasMismatchingContentDirection = m_renderer.containingBlock()->style().direction() != textBox().direction();
+    auto hasBackwardTrunctation = m_selectableRange.truncation && hasMismatchingContentDirection;
+
+    auto hasSpellingOrGrammarDecoration = [&] {
+        auto markedTexts = MarkedText::collectForDocumentMarkers(m_renderer, m_selectableRange, MarkedText::PaintPhase::Decoration);
+
+        auto hasSpellingError = markedTexts.containsIf([](auto&& markedText) {
+            return markedText.type == MarkedText::Type::SpellingError;
+        });
+
+        if (hasSpellingError) {
+            auto spellingErrorStyle = m_renderer.spellingErrorPseudoStyle();
+            if (spellingErrorStyle)
+                return !spellingErrorStyle->textDecorationsInEffect().isEmpty();
+        }
+
+        auto hasGrammarError = markedTexts.containsIf([](auto&& markedText) {
+            return markedText.type == MarkedText::Type::GrammarError;
+        });
+
+        if (hasGrammarError) {
+            auto grammarErrorStyle = m_renderer.grammarErrorPseudoStyle();
+            if (grammarErrorStyle)
+                return !grammarErrorStyle->textDecorationsInEffect().isEmpty();
+        }
+
+        return false;
+    };
+
+    auto hasDecoration = hasTextDecoration || hasHighlightDecoration || hasSpellingOrGrammarDecoration();
 
     auto contentMayNeedStyledMarkedText = [&] {
         if (hasDecoration)
@@ -206,17 +297,24 @@ void TextBoxPainter<TextBoxPath>::paintForegroundAndDecorations()
             return true;
         return false;
     };
+    auto startPosition = [&] {
+        return !hasBackwardTrunctation ? m_selectableRange.clamp(textBox().start()) : textBox().length() - *m_selectableRange.truncation;
+    };
+    auto endPosition = [&] {
+        return !hasBackwardTrunctation ? m_selectableRange.clamp(textBox().end()) : textBox().length();
+    };
     if (!contentMayNeedStyledMarkedText()) {
         auto& lineStyle = m_isFirstLine ? m_renderer.firstLineStyle() : m_renderer.style();
-        paintForeground({ MarkedText { m_selectableRange.clamp(textBox().start()), m_selectableRange.clamp(textBox().end()), MarkedText::Unmarked },
-            StyledMarkedText::computeStyleForUnmarkedMarkedText(m_renderer, lineStyle, m_isFirstLine, m_paintInfo) });
+        auto markedText = MarkedText { startPosition(), endPosition(), MarkedText::Type::Unmarked };
+        auto styledMarkedText = StyledMarkedText { markedText, StyledMarkedText::computeStyleForUnmarkedMarkedText(m_renderer, lineStyle, m_isFirstLine, m_paintInfo) };
+        paintCompositionForeground(styledMarkedText);
         return;
     }
 
     Vector<MarkedText> markedTexts;
     if (m_paintInfo.phase != PaintPhase::Selection) {
         // The marked texts for the gaps between document markers and selection are implicitly created by subdividing the entire line.
-        markedTexts.append({ m_selectableRange.clamp(textBox().start()), m_selectableRange.clamp(textBox().end()), MarkedText::Unmarked });
+        markedTexts.append({ startPosition(), endPosition(), MarkedText::Type::Unmarked });
 
         if (!m_isPrinting) {
             markedTexts.appendVector(MarkedText::collectForDocumentMarkers(m_renderer, m_selectableRange, MarkedText::PaintPhase::Foreground));
@@ -245,7 +343,7 @@ void TextBoxPainter<TextBoxPath>::paintForegroundAndDecorations()
     // ... now remove the selection marked text if we are excluding selection.
     if (!m_isPrinting && m_paintInfo.paintBehavior.contains(PaintBehavior::ExcludeSelection)) {
         styledMarkedTexts.removeAllMatching([] (const StyledMarkedText& markedText) {
-            return markedText.type == MarkedText::Selection;
+            return markedText.type == MarkedText::Type::Selection;
         });
     }
 
@@ -291,7 +389,7 @@ void TextBoxPainter<TextBoxPath>::paintForegroundAndDecorations()
                 }
                 auto decorationPainter = createDecorationPainter(markedText, textDecorationSelectionClipOutRect);
                 paintBackgroundDecorations(decorationPainter, markedText, snappedPaintRect);
-                paintForeground(markedText);
+                paintCompositionForeground(markedText);
                 paintForegroundDecorations(decorationPainter, markedText, snappedPaintRect);
             }
         }
@@ -299,8 +397,11 @@ void TextBoxPainter<TextBoxPath>::paintForegroundAndDecorations()
         // Coalesce styles of adjacent marked texts to minimize the number of drawing commands.
         auto coalescedStyledMarkedTexts = StyledMarkedText::coalesceAdjacentWithEqualForeground(styledMarkedTexts);
 
+        if (coalescedStyledMarkedTexts.isEmpty())
+            return;
+
         for (auto& markedText : coalescedStyledMarkedTexts)
-            paintForeground(markedText);
+            paintCompositionForeground(markedText);
     }
 }
 
@@ -317,6 +418,9 @@ void TextBoxPainter<TextBoxPath>::paintCompositionBackground()
     }
 
     for (auto& highlight : editor.customCompositionHighlights()) {
+        if (!highlight.backgroundColor)
+            continue;
+
         if (highlight.endOffset <= textBox().start())
             continue;
 
@@ -325,7 +429,7 @@ void TextBoxPainter<TextBoxPath>::paintCompositionBackground()
 
         auto [clampedStart, clampedEnd] = m_selectableRange.clamp(highlight.startOffset, highlight.endOffset);
 
-        paintBackground(clampedStart, clampedEnd, highlight.color, BackgroundStyle::Rounded);
+        paintBackground(clampedStart, clampedEnd, *highlight.backgroundColor, BackgroundStyle::Rounded);
 
         if (highlight.endOffset > textBox().end())
             break;
@@ -392,7 +496,7 @@ void TextBoxPainter<TextBoxPath>::paintForeground(const StyledMarkedText& marked
     if (!emphasisMark.isEmpty())
         emphasisMarkOffset = *m_emphasisMarkExistsAndIsAbove ? -font.metricsOfPrimaryFont().ascent() - font.emphasisMarkDescent(emphasisMark) : font.metricsOfPrimaryFont().descent() + font.emphasisMarkAscent(emphasisMark);
 
-    TextPainter textPainter { context, font };
+    TextPainter textPainter { context, font, m_style };
     textPainter.setStyle(markedText.style.textStyles);
     textPainter.setIsHorizontal(textBox().isHorizontal());
     if (markedText.style.textShadow) {
@@ -404,8 +508,8 @@ void TextBoxPainter<TextBoxPath>::paintForeground(const StyledMarkedText& marked
     if (auto* debugShadow = debugTextShadow())
         textPainter.setShadow(debugShadow);
 
-    GraphicsContextStateSaver stateSaver(context, markedText.style.textStyles.strokeWidth > 0 || markedText.type == MarkedText::DraggedContent);
-    if (markedText.type == MarkedText::DraggedContent)
+    GraphicsContextStateSaver stateSaver(context, markedText.style.textStyles.strokeWidth > 0 || markedText.type == MarkedText::Type::DraggedContent);
+    if (markedText.type == MarkedText::Type::DraggedContent)
         context.setAlpha(markedText.style.alpha);
     updateGraphicsContext(context, markedText.style.textStyles);
 
@@ -428,7 +532,7 @@ TextDecorationPainter TextBoxPainter<TextBoxPath>::createDecorationPainter(const
     // Note that if the text is truncated, we let the thing being painted in the truncation
     // draw its own decoration.
     GraphicsContextStateSaver stateSaver { context, false };
-    bool isDraggedContent = markedText.type == MarkedText::DraggedContent;
+    bool isDraggedContent = markedText.type == MarkedText::Type::DraggedContent;
     if (isDraggedContent || !clipOutRect.isEmpty()) {
         stateSaver.save();
         if (isDraggedContent)
@@ -480,6 +584,11 @@ template<typename TextBoxPath>
 void TextBoxPainter<TextBoxPath>::collectDecoratingBoxesForTextBox(DecoratingBoxList& decoratingBoxList, const InlineIterator::TextBoxIterator& textBox, FloatPoint textBoxLocation, const TextDecorationPainter::Styles& overrideDecorationStyle)
 {
     auto ancestorInlineBox = textBox->parentInlineBox();
+    if (!ancestorInlineBox) {
+        ASSERT_NOT_REACHED();
+        return;
+    }
+
     // FIXME: Vertical writing mode needs some coordinate space transformation for parent inline boxes as we rotate the content with m_paintRect (see ::paint)
     if (ancestorInlineBox->isRootInlineBox() || !textBox->isHorizontal()) {
         decoratingBoxList.append({
@@ -491,7 +600,7 @@ void TextBoxPainter<TextBoxPath>::collectDecoratingBoxesForTextBox(DecoratingBox
         return;
     }
 
-    enum UseOverriderDecorationStyle : bool { Yes, No };
+    enum UseOverriderDecorationStyle : bool { No, Yes };
     auto appendIfIsDecoratingBoxForBackground = [&] (auto& inlineBox, auto useOverriderDecorationStyle) {
         auto& style = m_isFirstLine ? inlineBox->renderer().firstLineStyle() : inlineBox->renderer().style();
 
@@ -517,6 +626,10 @@ void TextBoxPainter<TextBoxPath>::collectDecoratingBoxesForTextBox(DecoratingBox
     appendIfIsDecoratingBoxForBackground(ancestorInlineBox, UseOverriderDecorationStyle::Yes);
     while (!ancestorInlineBox->isRootInlineBox()) {
         ancestorInlineBox = ancestorInlineBox->parentInlineBox();
+        if (!ancestorInlineBox) {
+            ASSERT_NOT_REACHED();
+            break;
+        }
         appendIfIsDecoratingBoxForBackground(ancestorInlineBox, UseOverriderDecorationStyle::No);
     }
 }
@@ -525,7 +638,7 @@ template<typename TextBoxPath>
 void TextBoxPainter<TextBoxPath>::paintBackgroundDecorations(TextDecorationPainter& decorationPainter, const StyledMarkedText& markedText, const FloatRect& textBoxPaintRect)
 {
     if (m_isCombinedText)
-        m_paintInfo.context().concatCTM(rotation(m_paintRect, Clockwise));
+        m_paintInfo.context().concatCTM(rotation(m_paintRect, RotationDirection::Clockwise));
 
     auto textRun = m_paintTextRun.subRun(markedText.startOffset, markedText.endOffset - markedText.startOffset);
 
@@ -560,15 +673,15 @@ void TextBoxPainter<TextBoxPath>::paintBackgroundDecorations(TextDecorationPaint
                 overlineOffset(),
                 computedLinethroughCenter(decoratingBox.style, textDecorationThickness, autoTextDecorationThickness),
                 decoratingBox.style.metricsOfPrimaryFont().ascent() + 2.f,
-                wavyStrokeParameters(decoratingBox.style.computedFontPixelSize())
+                wavyStrokeParameters(decoratingBox.style.computedFontSize())
             };
         };
 
-        decorationPainter.paintBackgroundDecorations(textRun, computedBackgroundDecorationGeometry(), computedTextDecorationType, decoratingBox.textDecorationStyles);
+        decorationPainter.paintBackgroundDecorations(m_style, textRun, computedBackgroundDecorationGeometry(), computedTextDecorationType, decoratingBox.textDecorationStyles);
     }
 
     if (m_isCombinedText)
-        m_paintInfo.context().concatCTM(rotation(m_paintRect, Counterclockwise));
+        m_paintInfo.context().concatCTM(rotation(m_paintRect, RotationDirection::Counterclockwise));
 }
 
 template<typename TextBoxPath>
@@ -585,7 +698,7 @@ void TextBoxPainter<TextBoxPath>::paintForegroundDecorations(TextDecorationPaint
         return;
 
     if (m_isCombinedText)
-        m_paintInfo.context().concatCTM(rotation(m_paintRect, Clockwise));
+        m_paintInfo.context().concatCTM(rotation(m_paintRect, RotationDirection::Clockwise));
 
     auto deviceScaleFactor = m_document.deviceScaleFactor();
     auto textDecorationThickness = computedTextDecorationThickness(styleToUse, deviceScaleFactor);
@@ -594,45 +707,203 @@ void TextBoxPainter<TextBoxPath>::paintForegroundDecorations(TextDecorationPaint
         , textBoxPaintRect.width()
         , textDecorationThickness
         , linethroughCenter
-        , wavyStrokeParameters(styleToUse.computedFontPixelSize()) }, markedText.style.textDecorationStyles);
+        , wavyStrokeParameters(styleToUse.computedFontSize()) }, markedText.style.textDecorationStyles);
 
     if (m_isCombinedText)
-        m_paintInfo.context().concatCTM(rotation(m_paintRect, Counterclockwise));
+        m_paintInfo.context().concatCTM(rotation(m_paintRect, RotationDirection::Counterclockwise));
 }
 
-#if USE(APPLE_INTERNAL_SDK)
-#include <WebKitAdditions/TextBoxPainterAdditions.cpp>
-#else
-static FloatRoundedRect::Radii radiiForUnderline(const CompositionUnderline&, unsigned, unsigned)
+static FloatRoundedRect::Radii radiiForUnderline(const CompositionUnderline& underline, unsigned markedTextStartOffset, unsigned markedTextEndOffset)
 {
-    return FloatRoundedRect::Radii { 0 };
+    auto radii = FloatRoundedRect::Radii { 0 };
+
+#if HAVE(REDESIGNED_TEXT_CURSOR)
+    if (!redesignedTextCursorEnabled())
+        return radii;
+
+    if (underline.startOffset >= markedTextStartOffset) {
+        radii.setTopLeft({ 1, 1 });
+        radii.setBottomLeft({ 1, 1 });
+    }
+
+    if (underline.endOffset <= markedTextEndOffset) {
+        radii.setTopRight({ 1, 1 });
+        radii.setBottomRight({ 1, 1 });
+    }
+#else
+    UNUSED_PARAM(underline);
+    UNUSED_PARAM(markedTextStartOffset);
+    UNUSED_PARAM(markedTextEndOffset);
+#endif
+
+    return radii;
 }
 
+#if HAVE(REDESIGNED_TEXT_CURSOR)
+enum class TrimSide : bool {
+    Left,
+    Right,
+};
+
+static FloatRoundedRect::Radii trimRadii(const FloatRoundedRect::Radii& radii, TrimSide trimSide)
+{
+    switch (trimSide) {
+    case TrimSide::Left:
+        return { { }, radii.topRight(), { }, radii.bottomRight() };
+    case TrimSide::Right:
+        return { radii.topLeft(), { }, radii.bottomLeft(), { } };
+    }
+}
+
+enum class SnapDirection : uint8_t {
+    Left,
+    Right,
+    Both,
+};
+
+static FloatRect snapRectToDevicePixelsInDirection(const FloatRect& rect, float deviceScaleFactor, SnapDirection snapDirection)
+{
+    const auto layoutRect = LayoutRect { rect };
+    switch (snapDirection) {
+    case SnapDirection::Left:
+        return snapRectToDevicePixelsWithWritingDirection(layoutRect, deviceScaleFactor, true);
+    case SnapDirection::Right:
+        return snapRectToDevicePixelsWithWritingDirection(layoutRect, deviceScaleFactor, false);
+    case SnapDirection::Both:
+        auto snappedRectLeft = snapRectToDevicePixelsWithWritingDirection(layoutRect, deviceScaleFactor, true);
+        return snapRectToDevicePixelsWithWritingDirection(LayoutRect { snappedRectLeft }, deviceScaleFactor, false);
+    }
+}
+
+enum class LayoutBoxLocation : uint8_t {
+    OnlyBox,
+    StartOfSequence,
+    EndOfSequence,
+    MiddleOfSequence,
+    Unknown,
+};
+
+static LayoutBoxLocation layoutBoxSequenceLocation(const InlineIterator::BoxModernPath& textBox)
+{
+    auto isFirstForLayoutBox = textBox.box().isFirstForLayoutBox();
+    auto isLastForLayoutBox = textBox.box().isLastForLayoutBox();
+    if (isFirstForLayoutBox && isLastForLayoutBox)
+        return LayoutBoxLocation::OnlyBox;
+    if (isFirstForLayoutBox)
+        return LayoutBoxLocation::StartOfSequence;
+    if (isLastForLayoutBox)
+        return LayoutBoxLocation::EndOfSequence;
+    return LayoutBoxLocation::MiddleOfSequence;
+}
+
+static LayoutBoxLocation layoutBoxSequenceLocation(const InlineIterator::BoxLegacyPath&)
+{
+    return LayoutBoxLocation::Unknown;
+}
+#endif
+
 template<typename TextBoxPath>
-void TextBoxPainter<TextBoxPath>::fillCompositionUnderline(float start, float width, const CompositionUnderline& underline, const FloatRoundedRect::Radii&, bool) const
+void TextBoxPainter<TextBoxPath>::fillCompositionUnderline(float start, float width, const CompositionUnderline& underline, const FloatRoundedRect::Radii& radii, bool hasLiveConversion) const
 {
+#if HAVE(REDESIGNED_TEXT_CURSOR)
+    if (!redesignedTextCursorEnabled())
+#endif
+    {
+        // Thick marked text underlines are 2px thick as long as there is room for the 2px line under the baseline.
+        // All other marked text underlines are 1px thick.
+        // If there's not enough space the underline will touch or overlap characters.
+        int lineThickness = 1;
+        int baseline = m_style.metricsOfPrimaryFont().ascent();
+        if (underline.thick && m_logicalRect.height() - baseline >= 2)
+            lineThickness = 2;
+
+        // We need to have some space between underlines of subsequent clauses, because some input methods do not use different underline styles for those.
+        // We make each line shorter, which has a harmless side effect of shortening the first and last clauses, too.
+        start += 1;
+        width -= 2;
+
+        auto& style = m_renderer.style();
+        auto underlineColor = underline.compositionUnderlineColor == CompositionUnderlineColor::TextColor ? style.visitedDependentColorWithColorFilter(CSSPropertyWebkitTextFillColor) : style.colorByApplyingColorFilter(underline.color);
+
+        auto& context = m_paintInfo.context();
+        context.setStrokeColor(underlineColor);
+        context.setStrokeThickness(lineThickness);
+        context.drawLineForText(FloatRect(m_paintRect.x() + start, m_paintRect.y() + m_logicalRect.height() - lineThickness, width, lineThickness), m_isPrinting);
+        return;
+    }
+
+#if HAVE(REDESIGNED_TEXT_CURSOR)
+    if (!underline.color.isVisible())
+        return;
+
     // Thick marked text underlines are 2px thick as long as there is room for the 2px line under the baseline.
     // All other marked text underlines are 1px thick.
     // If there's not enough space the underline will touch or overlap characters.
     int lineThickness = 1;
     int baseline = m_style.metricsOfPrimaryFont().ascent();
-    if (underline.thick && m_logicalRect.height() - baseline >= 2)
+    if (m_logicalRect.height() - baseline >= 2)
         lineThickness = 2;
 
-    // We need to have some space between underlines of subsequent clauses, because some input methods do not use different underline styles for those.
-    // We make each line shorter, which has a harmless side effect of shortening the first and last clauses, too.
-    start += 1;
-    width -= 2;
+    auto underlineColor = [this] {
+#if PLATFORM(MAC)
+        auto cssColorValue = CSSValueAppleSystemControlAccent;
+#else
+        auto cssColorValue = CSSValueAppleSystemBlue;
+#endif
+        auto styleColorOptions = m_renderer.styleColorOptions();
+        return RenderTheme::singleton().systemColor(cssColorValue, styleColorOptions | StyleColorOptions::UseSystemAppearance);
+    }();
 
-    auto& style = m_renderer.style();
-    auto underlineColor = underline.compositionUnderlineColor == CompositionUnderlineColor::TextColor ? style.visitedDependentColorWithColorFilter(CSSPropertyWebkitTextFillColor) : style.colorByApplyingColorFilter(underline.color);
+    if (!underline.thick && hasLiveConversion)
+        underlineColor = underlineColor.colorWithAlpha(0.35);
 
     auto& context = m_paintInfo.context();
     context.setStrokeColor(underlineColor);
     context.setStrokeThickness(lineThickness);
-    context.drawLineForText(FloatRect(m_paintRect.x() + start, m_paintRect.y() + m_logicalRect.height() - lineThickness, width, lineThickness), m_isPrinting);
-}
+
+    auto rect = FloatRect(m_paintRect.x() + start, m_paintRect.y() + m_logicalRect.height() - lineThickness, width, lineThickness);
+
+    if (radii.isZero()) {
+        context.drawLineForText(rect, m_isPrinting);
+        return;
+    }
+
+    // We cannot directly draw rounded edges for every rect, since a single textbox path may be split up over multiple rects.
+    // Drawing rounded edges unconditionally could then produce broken underlines between continuous rects.
+    // As a mitigation, we consult the textbox path to understand the current rect's position in the textbox path.
+    // If we're the only box in the path, then we fallback to unconditionally drawing rounded edges.
+    // If not, we flatten out the right, left, or both edges depending on whether we're at the start, end, or middle of a path, respectively.
+
+    auto deviceScaleFactor = m_document.deviceScaleFactor();
+
+    switch (layoutBoxSequenceLocation(m_textBox)) {
+    case LayoutBoxLocation::Unknown:
+    case LayoutBoxLocation::OnlyBox: {
+        context.fillRoundedRect(FloatRoundedRect { rect, radii }, underlineColor);
+        return;
+    }
+    case LayoutBoxLocation::StartOfSequence: {
+        auto snappedRectRight = snapRectToDevicePixelsInDirection(rect, deviceScaleFactor, SnapDirection::Right);
+        context.fillRoundedRect(FloatRoundedRect { snappedRectRight, trimRadii(radii, TrimSide::Right) }, underlineColor);
+        return;
+    }
+    case LayoutBoxLocation::EndOfSequence: {
+        auto snappedRectLeft = snapRectToDevicePixelsInDirection(rect, deviceScaleFactor, SnapDirection::Left);
+        context.fillRoundedRect(FloatRoundedRect { snappedRectLeft, trimRadii(radii, TrimSide::Left) }, underlineColor);
+        return;
+    }
+    case LayoutBoxLocation::MiddleOfSequence: {
+        auto snappedRectBoth = snapRectToDevicePixelsInDirection(rect, deviceScaleFactor, SnapDirection::Both);
+        context.fillRect(snappedRectBoth, underlineColor);
+        return;
+    }
+    }
+    ASSERT_NOT_REACHED("Unexpected LayoutBoxLocation value, underline not drawn");
+#else
+    UNUSED_PARAM(radii);
+    UNUSED_PARAM(hasLiveConversion);
 #endif
+}
 
 template<typename TextBoxPath>
 void TextBoxPainter<TextBoxPath>::paintCompositionUnderlines()
@@ -731,6 +1002,21 @@ template<typename TextBoxPath>
 void TextBoxPainter<TextBoxPath>::paintPlatformDocumentMarkers()
 {
     auto markedTexts = MarkedText::collectForDocumentMarkers(m_renderer, m_selectableRange, MarkedText::PaintPhase::Decoration);
+
+    auto spellingErrorStyle = m_renderer.spellingErrorPseudoStyle();
+    if (spellingErrorStyle && !spellingErrorStyle->textDecorationsInEffect().isEmpty()) {
+        markedTexts.removeAllMatching([] (auto&& markedText) {
+            return markedText.type == MarkedText::Type::SpellingError;
+        });
+    }
+
+    auto grammarErrorStyle = m_renderer.grammarErrorPseudoStyle();
+    if (grammarErrorStyle && !grammarErrorStyle->textDecorationsInEffect().isEmpty()) {
+        markedTexts.removeAllMatching([] (auto&& markedText) {
+            return markedText.type == MarkedText::Type::GrammarError;
+        });
+    }
+
     for (auto& markedText : MarkedText::subdivide(markedTexts, MarkedText::OverlapStrategy::Frontmost))
         paintPlatformDocumentMarker(markedText);
 }
@@ -754,30 +1040,33 @@ void TextBoxPainter<TextBoxPath>::paintPlatformDocumentMarker(const MarkedText&
 
     auto bounds = calculateDocumentMarkerBounds(makeIterator(), markedText);
 
-    auto lineStyleForMarkedTextType = [&]() -> DocumentMarkerLineStyle {
-        bool shouldUseDarkAppearance = m_renderer.useDarkAppearance();
+    auto lineStyleMode = [&] {
         switch (markedText.type) {
-        case MarkedText::SpellingError:
-            return { DocumentMarkerLineStyleMode::Spelling, shouldUseDarkAppearance };
-        case MarkedText::GrammarError:
-            return { DocumentMarkerLineStyleMode::Grammar, shouldUseDarkAppearance };
-        case MarkedText::Correction:
-            return { DocumentMarkerLineStyleMode::AutocorrectionReplacement, shouldUseDarkAppearance };
-        case MarkedText::DictationAlternatives:
-            return { DocumentMarkerLineStyleMode::DictationAlternatives, shouldUseDarkAppearance };
+        case MarkedText::Type::SpellingError:
+            return DocumentMarkerLineStyleMode::Spelling;
+        case MarkedText::Type::GrammarError:
+            return DocumentMarkerLineStyleMode::Grammar;
+        case MarkedText::Type::Correction:
+            return DocumentMarkerLineStyleMode::AutocorrectionReplacement;
+        case MarkedText::Type::DictationAlternatives:
+            return DocumentMarkerLineStyleMode::DictationAlternatives;
 #if PLATFORM(IOS_FAMILY)
-        case MarkedText::DictationPhraseWithAlternatives:
+        case MarkedText::Type::DictationPhraseWithAlternatives:
             // FIXME: Rename DocumentMarkerLineStyle::TextCheckingDictationPhraseWithAlternatives and remove the PLATFORM(IOS_FAMILY)-guard.
-            return { DocumentMarkerLineStyleMode::TextCheckingDictationPhraseWithAlternatives, shouldUseDarkAppearance };
+            return DocumentMarkerLineStyleMode::TextCheckingDictationPhraseWithAlternatives;
 #endif
         default:
             ASSERT_NOT_REACHED();
-            return { DocumentMarkerLineStyleMode::Spelling, shouldUseDarkAppearance };
+            return DocumentMarkerLineStyleMode::Spelling;
         }
-    };
+    }();
+
+    auto lineStyleColor = RenderTheme::singleton().documentMarkerLineColor(m_renderer, lineStyleMode);
+    if (auto* marker = markedText.marker)
+        lineStyleColor = lineStyleColor.colorWithAlphaMultipliedBy(marker->opacity());
 
     bounds.moveBy(m_paintRect.location());
-    m_paintInfo.context().drawDotsForDocumentMarker(bounds, lineStyleForMarkedTextType());
+    m_paintInfo.context().drawDotsForDocumentMarker(bounds, { lineStyleMode, lineStyleColor });
 }
 
 template<typename TextBoxPath>
@@ -785,25 +1074,7 @@ FloatRect TextBoxPainter<TextBoxPath>::computePaintRect(const LayoutPoint& paint
 {
     FloatPoint localPaintOffset(paintOffset);
 
-    if (m_selectableRange.truncation) {
-        if (m_renderer.containingBlock()->style().direction() != textBox().direction()) {
-            // Make the visible fragment of text hug the edge closest to the rest of the run by moving the origin
-            // at which we start drawing text.
-            // e.g. In the case of LTR text truncated in an RTL Context, the correct behavior is:
-            // |Hello|CBA| -> |...He|CBA|
-            // In order to draw the fragment "He" aligned to the right edge of it's box, we need to start drawing
-            // farther to the right.
-            // NOTE: WebKit's behavior differs from that of IE which appears to just overlay the ellipsis on top of the
-            // truncated string i.e.  |Hello|CBA| -> |...lo|CBA|
-            LayoutUnit widthOfVisibleText { m_renderer.width(textBox().start(), *m_selectableRange.truncation, textPosition(), m_isFirstLine) };
-            LayoutUnit widthOfHiddenText { m_logicalRect.width() - widthOfVisibleText };
-            LayoutSize truncationOffset(textBox().direction() == TextDirection::LTR ? widthOfHiddenText : -widthOfHiddenText, 0_lu);
-            localPaintOffset.move(textBox().isHorizontal() ? truncationOffset : truncationOffset.transposedSize());
-        }
-    }
-
     localPaintOffset.move(0, m_style.isHorizontalWritingMode() ? 0 : -m_logicalRect.height());
-
     auto visualRect = textBox().visualRectIgnoringBlockDirection();
     textBox().formattingContextRoot().flipForWritingMode(visualRect);
 
