@@ -63,10 +63,7 @@ void BasicShapeCenterCoordinate::updateComputedLength()
 }
 
 struct SVGPathTransformedByteStream {
-    bool operator==(const SVGPathTransformedByteStream& other) const 
-    { 
-        return other.offset == offset && other.zoom == zoom && other.rawStream == rawStream;
-    }
+    friend bool operator==(const SVGPathTransformedByteStream&, const SVGPathTransformedByteStream&) = default;
     bool isEmpty() const { return rawStream.isEmpty(); }
 
     Path path() const
@@ -102,7 +99,7 @@ public:
     static Path createValueForKey(const FloatRoundedRect& rect)
     {
         Path path;
-        path.addRoundedRect(rect);
+        path.addRoundedRect(rect, PathRoundedRect::Strategy::PreferBezier);
         return path;
     }
 };
@@ -172,33 +169,33 @@ bool BasicShapeCircle::operator==(const BasicShape& other) const
     auto& otherCircle = downcast<BasicShapeCircle>(other);
     return m_centerX == otherCircle.m_centerX
         && m_centerY == otherCircle.m_centerY
-        && m_radius == otherCircle.m_radius;
+        && m_radius == otherCircle.m_radius
+        && positionWasOmitted() == otherCircle.positionWasOmitted();
 }
 
-float BasicShapeCircle::floatValueForRadiusInBox(float boxWidth, float boxHeight) const
+float BasicShapeCircle::floatValueForRadiusInBox(float boxWidth, float boxHeight, FloatPoint center) const
 {
     if (m_radius.type() == BasicShapeRadius::Type::Value)
         return floatValueForLength(m_radius.value(), std::hypot(boxWidth, boxHeight) / sqrtOfTwoFloat);
 
-    float centerX = floatValueForCenterCoordinate(m_centerX, boxWidth);
-    float centerY = floatValueForCenterCoordinate(m_centerY, boxHeight);
-
-    float widthDelta = std::abs(boxWidth - centerX);
-    float heightDelta = std::abs(boxHeight - centerY);
+    float widthDelta = std::abs(boxWidth - center.x());
+    float heightDelta = std::abs(boxHeight - center.y());
     if (m_radius.type() == BasicShapeRadius::Type::ClosestSide)
-        return std::min(std::min(std::abs(centerX), widthDelta), std::min(std::abs(centerY), heightDelta));
+        return std::min(std::min(std::abs(center.x()), widthDelta), std::min(std::abs(center.y()), heightDelta));
 
     // If radius.type() == BasicShapeRadius::Type::FarthestSide.
-    return std::max(std::max(std::abs(centerX), widthDelta), std::max(std::abs(centerY), heightDelta));
+    return std::max(std::max(std::abs(center.x()), widthDelta), std::max(std::abs(center.y()), heightDelta));
+}
+
+const Path& BasicShapeCircle::pathForCenterCoordinate(const FloatRect& boundingBox, FloatPoint center) const
+{
+    float radius = floatValueForRadiusInBox(boundingBox.width(), boundingBox.height(),  center);
+    return cachedEllipsePath(FloatRect(center.x() - radius + boundingBox.x(), center.y() - radius + boundingBox.y(), radius * 2, radius * 2));
 }
 
 const Path& BasicShapeCircle::path(const FloatRect& boundingBox)
 {
-    float centerX = floatValueForCenterCoordinate(m_centerX, boundingBox.width());
-    float centerY = floatValueForCenterCoordinate(m_centerY, boundingBox.height());
-    float radius = floatValueForRadiusInBox(boundingBox.width(), boundingBox.height());
-
-    return cachedEllipsePath(FloatRect(centerX - radius + boundingBox.x(), centerY - radius + boundingBox.y(), radius * 2, radius * 2));
+    return pathForCenterCoordinate(boundingBox, { floatValueForCenterCoordinate(m_centerX, boundingBox.width()), floatValueForCenterCoordinate(m_centerY, boundingBox.height()) });
 }
 
 bool BasicShapeCircle::canBlend(const BasicShape& other) const
@@ -218,6 +215,7 @@ Ref<BasicShape> BasicShapeCircle::blend(const BasicShape& other, const BlendingC
     result->setCenterX(m_centerX.blend(otherCircle.centerX(), context));
     result->setCenterY(m_centerY.blend(otherCircle.centerY(), context));
     result->setRadius(m_radius.blend(otherCircle.radius(), context));
+    result->setPositionWasOmitted(positionWasOmitted() && otherCircle.positionWasOmitted());
     return result;
 }
 
@@ -259,7 +257,8 @@ bool BasicShapeEllipse::operator==(const BasicShape& other) const
     return m_centerX == otherEllipse.m_centerX
         && m_centerY == otherEllipse.m_centerY
         && m_radiusX == otherEllipse.m_radiusX
-        && m_radiusY == otherEllipse.m_radiusY;
+        && m_radiusY == otherEllipse.m_radiusY
+        && positionWasOmitted() == otherEllipse.positionWasOmitted();
 }
 
 float BasicShapeEllipse::floatValueForRadiusInBox(const BasicShapeRadius& radius, float center, float boxWidthOrHeight) const
@@ -275,14 +274,17 @@ float BasicShapeEllipse::floatValueForRadiusInBox(const BasicShapeRadius& radius
     return std::max(std::abs(center), widthOrHeightDelta);
 }
 
+const Path& BasicShapeEllipse::pathForCenterCoordinate(const FloatRect& boundingBox, FloatPoint center) const
+{
+    float radiusX = floatValueForRadiusInBox(m_radiusX, center.x(), boundingBox.width());
+    float radiusY = floatValueForRadiusInBox(m_radiusY, center.y(), boundingBox.height());
+
+    return cachedEllipsePath(FloatRect(center.x() - radiusX + boundingBox.x(), center.y() - radiusY + boundingBox.y(), radiusX * 2, radiusY * 2));
+}
+
 const Path& BasicShapeEllipse::path(const FloatRect& boundingBox)
 {
-    float centerX = floatValueForCenterCoordinate(m_centerX, boundingBox.width());
-    float centerY = floatValueForCenterCoordinate(m_centerY, boundingBox.height());
-    float radiusX = floatValueForRadiusInBox(m_radiusX, centerX, boundingBox.width());
-    float radiusY = floatValueForRadiusInBox(m_radiusY, centerY, boundingBox.height());
-
-    return cachedEllipsePath(FloatRect(centerX - radiusX + boundingBox.x(), centerY - radiusY + boundingBox.y(), radiusX * 2, radiusY * 2));
+    return pathForCenterCoordinate(boundingBox, { floatValueForCenterCoordinate(m_centerX, boundingBox.width()), floatValueForCenterCoordinate(m_centerY, boundingBox.height()) });
 }
 
 bool BasicShapeEllipse::canBlend(const BasicShape& other) const
@@ -313,6 +315,7 @@ Ref<BasicShape> BasicShapeEllipse::blend(const BasicShape& other, const Blending
     result->setCenterY(m_centerY.blend(otherEllipse.centerY(), context));
     result->setRadiusX(m_radiusX.blend(otherEllipse.radiusX(), context));
     result->setRadiusY(m_radiusY.blend(otherEllipse.radiusY(), context));
+    result->setPositionWasOmitted(positionWasOmitted() && otherEllipse.positionWasOmitted());
     return result;
 }
 
@@ -324,6 +327,218 @@ void BasicShapeEllipse::dump(TextStream& ts) const
     ts.dumpProperty("radius-y", radiusY());
 }
 
+Ref<BasicShapeRect> BasicShapeRect::create(Length&& top, Length&& right, Length&& bottom, Length&& left, LengthSize&& topLeftRadius, LengthSize&& topRightRadius, LengthSize&& bottomRightRadius, LengthSize&& bottomLeftRadius)
+{
+    return adoptRef(*new BasicShapeRect(WTFMove(top), WTFMove(right), WTFMove(bottom), WTFMove(left), WTFMove(topLeftRadius), WTFMove(topRightRadius), WTFMove(bottomRightRadius), WTFMove(bottomLeftRadius)));
+}
+
+BasicShapeRect::BasicShapeRect(Length&& top, Length&& right, Length&& bottom, Length&& left, LengthSize&& topLeftRadius, LengthSize&& topRightRadius, LengthSize&& bottomRightRadius, LengthSize&& bottomLeftRadius)
+    : m_edges(RectEdges<Length> { WTFMove(top), WTFMove(right), WTFMove(bottom), WTFMove(left) })
+    , m_topLeftRadius(WTFMove(topLeftRadius))
+    , m_topRightRadius(WTFMove(topRightRadius))
+    , m_bottomRightRadius(WTFMove(bottomRightRadius))
+    , m_bottomLeftRadius(WTFMove(bottomLeftRadius))
+{
+}
+
+BasicShapeRect::BasicShapeRect(RectEdges<Length>&& edges, LengthSize&& topLeftRadius, LengthSize&& topRightRadius, LengthSize&& bottomRightRadius, LengthSize&& bottomLeftRadius)
+    : m_edges(WTFMove(edges))
+    , m_topLeftRadius(WTFMove(topLeftRadius))
+    , m_topRightRadius(WTFMove(topRightRadius))
+    , m_bottomRightRadius(WTFMove(bottomRightRadius))
+    , m_bottomLeftRadius(WTFMove(bottomLeftRadius))
+{
+}
+
+Ref<BasicShape> BasicShapeRect::clone() const
+{
+    auto edges = m_edges;
+
+    auto topLeftRadius = m_topLeftRadius;
+    auto topRightRadius = m_topRightRadius;
+    auto bottomRightRadius = m_bottomRightRadius;
+    auto bottomLeftRadius = m_bottomLeftRadius;
+
+    return adoptRef(*new BasicShapeRect(WTFMove(edges), WTFMove(topLeftRadius), WTFMove(topRightRadius), WTFMove(bottomRightRadius), WTFMove(bottomLeftRadius)));
+}
+
+bool BasicShapeRect::operator==(const BasicShape& other) const
+{
+    if (type() != other.type())
+        return false;
+
+    auto& otherRect = downcast<BasicShapeRect>(other);
+    return m_edges == otherRect.m_edges
+        && m_topLeftRadius == otherRect.m_topLeftRadius
+        && m_topRightRadius == otherRect.m_topRightRadius
+        && m_bottomRightRadius == otherRect.m_bottomRightRadius
+        && m_bottomLeftRadius == otherRect.m_bottomLeftRadius;
+}
+
+const Path& BasicShapeRect::path(const FloatRect& boundingBox)
+{
+    auto top = m_edges.top().isAuto() ? 0 : floatValueForLength(m_edges.top(), boundingBox.height());
+    auto right = m_edges.right().isAuto() ? boundingBox.width() : floatValueForLength(m_edges.right(), boundingBox.width());
+    auto bottom = m_edges.bottom().isAuto() ? boundingBox.height() : floatValueForLength(m_edges.bottom(), boundingBox.height());
+    auto left = m_edges.left().isAuto() ? 0 : floatValueForLength(m_edges.left(), boundingBox.width());
+
+    right = std::max(right, left);
+    bottom = std::max(top, bottom);
+
+    auto rect = FloatRect(left + boundingBox.x(), top + boundingBox.y(), right - left, bottom - top);
+    auto radii = FloatRoundedRect::Radii(floatSizeForLengthSize(m_topLeftRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_topRightRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_bottomLeftRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_bottomRightRadius, boundingBox.size()));
+
+    radii.scale(calcBorderRadiiConstraintScaleFor(rect, radii));
+
+    return cachedRoundedRectPath(FloatRoundedRect(rect, radii));
+}
+
+bool BasicShapeRect::canBlend(const BasicShape& other) const
+{
+    // FIXME: Allow interpolation with inset() shape.
+    return type() == other.type();
+}
+
+Ref<BasicShape> BasicShapeRect::blend(const BasicShape& other, const BlendingContext& context) const
+{
+    ASSERT(type() == other.type());
+    auto& otherRect = downcast<BasicShapeRect>(other);
+    auto result = BasicShapeRect::create();
+
+    result->setTop(WebCore::blend(otherRect.top(), top(), context));
+    result->setRight(WebCore::blend(otherRect.right(), right(), context));
+    result->setBottom(WebCore::blend(otherRect.bottom(), bottom(), context));
+    result->setLeft(WebCore::blend(otherRect.left(), left(), context));
+
+    result->setTopLeftRadius(WebCore::blend(otherRect.topLeftRadius(), topLeftRadius(), context, ValueRange::NonNegative));
+    result->setTopRightRadius(WebCore::blend(otherRect.topRightRadius(), topRightRadius(), context, ValueRange::NonNegative));
+    result->setBottomRightRadius(WebCore::blend(otherRect.bottomRightRadius(), bottomRightRadius(), context, ValueRange::NonNegative));
+    result->setBottomLeftRadius(WebCore::blend(otherRect.bottomLeftRadius(), bottomLeftRadius(), context, ValueRange::NonNegative));
+
+    return result;
+}
+
+void BasicShapeRect::dump(TextStream& ts) const
+{
+    ts.dumpProperty("top", top());
+    ts.dumpProperty("right", right());
+    ts.dumpProperty("bottom", bottom());
+    ts.dumpProperty("left", left());
+
+    ts.dumpProperty("top-left-radius", topLeftRadius());
+    ts.dumpProperty("top-right-radius", topRightRadius());
+    ts.dumpProperty("bottom-right-radius", bottomRightRadius());
+    ts.dumpProperty("bottom-left-radius", bottomLeftRadius());
+}
+
+Ref<BasicShapeXywh> BasicShapeXywh::create(Length&& insetX, Length&& insetY, Length&& width, Length&& height, LengthSize&& topLeftRadius, LengthSize&& topRightRadius, LengthSize&& bottomRightRadius, LengthSize&& bottomLeftRadius)
+{
+    return adoptRef(*new BasicShapeXywh(WTFMove(insetX), WTFMove(insetY), WTFMove(width), WTFMove(height), WTFMove(topLeftRadius), WTFMove(topRightRadius), WTFMove(bottomRightRadius), WTFMove(bottomLeftRadius)));
+}
+
+BasicShapeXywh::BasicShapeXywh(Length&& insetX, Length&& insetY, Length&& width, Length&& height, LengthSize&& topLeftRadius, LengthSize&& topRightRadius, LengthSize&& bottomRightRadius, LengthSize&& bottomLeftRadius)
+    : m_insetX(WTFMove(insetX))
+    , m_insetY(WTFMove(insetY))
+    , m_width(WTFMove(width))
+    , m_height(WTFMove(height))
+    , m_topLeftRadius(WTFMove(topLeftRadius))
+    , m_topRightRadius(WTFMove(topRightRadius))
+    , m_bottomRightRadius(WTFMove(bottomRightRadius))
+    , m_bottomLeftRadius(WTFMove(bottomLeftRadius))
+{
+}
+
+Ref<BasicShape> BasicShapeXywh::clone() const
+{
+    auto insetX = m_insetX;
+    auto insetY = m_insetY;
+    auto width = m_width;
+    auto height = m_height;
+
+    auto topLeftRadius = m_topLeftRadius;
+    auto topRightRadius = m_topRightRadius;
+    auto bottomRightRadius = m_bottomRightRadius;
+    auto bottomLeftRadius = m_bottomLeftRadius;
+
+    return adoptRef(*new BasicShapeXywh(WTFMove(insetX), WTFMove(insetY), WTFMove(width), WTFMove(height), WTFMove(topLeftRadius), WTFMove(topRightRadius), WTFMove(bottomRightRadius), WTFMove(bottomLeftRadius)));
+}
+
+bool BasicShapeXywh::operator==(const BasicShape& other) const
+{
+    if (type() != other.type())
+        return false;
+
+    auto& otherXywh = downcast<BasicShapeXywh>(other);
+    return m_insetX == otherXywh.m_insetX
+        && m_insetY == otherXywh.m_insetY
+        && m_width == otherXywh.m_width
+        && m_height == otherXywh.m_height
+        && m_topLeftRadius == otherXywh.m_topLeftRadius
+        && m_topRightRadius == otherXywh.m_topRightRadius
+        && m_bottomRightRadius == otherXywh.m_bottomRightRadius
+        && m_bottomLeftRadius == otherXywh.m_bottomLeftRadius;
+}
+
+const Path& BasicShapeXywh::path(const FloatRect& boundingBox)
+{
+    auto insetX = floatValueForLength(m_insetX, boundingBox.width());
+    auto insetY = floatValueForLength(m_insetY, boundingBox.height());
+    auto width = floatValueForLength(m_width, boundingBox.width());
+    auto height = floatValueForLength(m_height, boundingBox.height());
+
+    auto rect = FloatRect(insetX, insetY, width, height);
+    auto radii = FloatRoundedRect::Radii(floatSizeForLengthSize(m_topLeftRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_topRightRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_bottomLeftRadius, boundingBox.size()),
+        floatSizeForLengthSize(m_bottomRightRadius, boundingBox.size()));
+
+    radii.scale(calcBorderRadiiConstraintScaleFor(rect, radii));
+
+    return cachedRoundedRectPath(FloatRoundedRect(rect, radii));
+}
+
+bool BasicShapeXywh::canBlend(const BasicShape& other) const
+{
+    // FIXME: Allow interpolation with inset() shape.
+    return type() == other.type();
+}
+
+Ref<BasicShape> BasicShapeXywh::blend(const BasicShape& other, const BlendingContext& context) const
+{
+    ASSERT(type() == other.type());
+    auto& otherXywh = downcast<BasicShapeXywh>(other);
+    auto result = BasicShapeXywh::create();
+
+    result->setInsetX(WebCore::blend(otherXywh.insetX(), m_insetX, context));
+    result->setInsetY(WebCore::blend(otherXywh.insetY(), m_insetY, context));
+    result->setWidth(WebCore::blend(otherXywh.width(), m_width, context));
+    result->setHeight(WebCore::blend(otherXywh.height(), m_height, context));
+
+    result->setTopLeftRadius(WebCore::blend(otherXywh.topLeftRadius(), topLeftRadius(), context, ValueRange::NonNegative));
+    result->setTopRightRadius(WebCore::blend(otherXywh.topRightRadius(), topRightRadius(), context, ValueRange::NonNegative));
+    result->setBottomRightRadius(WebCore::blend(otherXywh.bottomRightRadius(), bottomRightRadius(), context, ValueRange::NonNegative));
+    result->setBottomLeftRadius(WebCore::blend(otherXywh.bottomLeftRadius(), bottomLeftRadius(), context, ValueRange::NonNegative));
+
+    return result;
+}
+
+void BasicShapeXywh::dump(TextStream& ts) const
+{
+    ts.dumpProperty("x", insetX());
+    ts.dumpProperty("y", insetY());
+    ts.dumpProperty("width", width());
+    ts.dumpProperty("height", height());
+
+    ts.dumpProperty("top-left-radius", topLeftRadius());
+    ts.dumpProperty("top-right-radius", topRightRadius());
+    ts.dumpProperty("bottom-right-radius", bottomRightRadius());
+    ts.dumpProperty("bottom-left-radius", bottomLeftRadius());
+
+}
+
 Ref<BasicShapePolygon> BasicShapePolygon::create(WindRule windRule, Vector<Length>&& values)
 {
     return adoptRef(*new BasicShapePolygon(windRule, WTFMove(values)));
