@@ -62,9 +62,6 @@ struct GreaterThanOrSameSizeAsStyleRareInheritedData : public RefCounted<Greater
 #if ENABLE(DARK_MODE_CSS)
     StyleColorScheme colorScheme;
 #endif
-    TextSpacingTrim textSpacingTrim;
-    TextAutospace textAutospace;
-
     ListStyleType listStyleType;
 
     Markable<ScrollbarColor> scrollbarColor;
@@ -140,7 +137,7 @@ StyleRareInheritedData::StyleRareInheritedData()
     , hasAutoAccentColor(true)
     , effectiveInert(false)
     , isInSubtreeWithBlendMode(false)
-    , effectiveSkippedContent(false)
+    , effectiveSkippedContent(static_cast<unsigned>(ContentVisibility::Visible))
     , effectiveTouchActions(RenderStyle::initialTouchActions())
     , strokeWidth(RenderStyle::initialStrokeWidth())
     , strokeColor(RenderStyle::initialStrokeColor())
@@ -156,8 +153,6 @@ StyleRareInheritedData::StyleRareInheritedData()
 #if ENABLE(TOUCH_EVENTS)
     , tapHighlightColor(RenderStyle::initialTapHighlightColor())
 #endif
-    , textSpacingTrim(RenderStyle::initialTextSpacingTrim())
-    , textAutospace(RenderStyle::initialTextAutospace())
     , listStyleType(RenderStyle::initialListStyleType())
     , scrollbarColor(RenderStyle::initialScrollbarColor())
 {
@@ -256,8 +251,6 @@ inline StyleRareInheritedData::StyleRareInheritedData(const StyleRareInheritedDa
 #if ENABLE(TOUCH_EVENTS)
     , tapHighlightColor(o.tapHighlightColor)
 #endif
-    , textSpacingTrim(o.textSpacingTrim)
-    , textAutospace(o.textAutospace)
     , listStyleType(o.listStyleType)
     , scrollbarColor(o.scrollbarColor)
 {
@@ -366,8 +359,6 @@ bool StyleRareInheritedData::operator==(const StyleRareInheritedData& o) const
         && customProperties == o.customProperties
         && arePointingToEqualData(listStyleImage, o.listStyleImage)
         && listStyleType == o.listStyleType
-        && textSpacingTrim == o.textSpacingTrim
-        && textAutospace == o.textAutospace
         && scrollbarColor == o.scrollbarColor;
 }
 
