@@ -60,13 +60,13 @@ bool WillChangeData::containsProperty(CSSPropertyID property) const
     return false;
 }
 
-bool WillChangeData::createsContainingBlockForAbsolutelyPositioned() const
+bool WillChangeData::createsContainingBlockForAbsolutelyPositioned(bool isRootElement) const
 {
-    return createsContainingBlockForOutOfFlowPositioned()
+    return createsContainingBlockForOutOfFlowPositioned(isRootElement)
         || containsProperty(CSSPropertyPosition);
 }
 
-bool WillChangeData::createsContainingBlockForOutOfFlowPositioned() const
+bool WillChangeData::createsContainingBlockForOutOfFlowPositioned(bool isRootElement) const
 {
     return containsProperty(CSSPropertyPerspective)
         // CSS transforms
@@ -78,12 +78,24 @@ bool WillChangeData::createsContainingBlockForOutOfFlowPositioned() const
         || containsProperty(CSSPropertyContain)
         // CSS filter & backdrop-filter
         // FIXME: exclude root element for those properties (bug 225034)
-#if ENABLE(FILTERS_LEVEL_2)
-        || containsProperty(CSSPropertyWebkitBackdropFilter)
-#endif
+        || (containsProperty(CSSPropertyBackdropFilter) && !isRootElement)
+        || (containsProperty(CSSPropertyWebkitBackdropFilter) && !isRootElement)
         || containsProperty(CSSPropertyFilter);
 }
 
+bool WillChangeData::canBeBackdropRoot() const
+{
+    return containsProperty(CSSPropertyOpacity)
+        || containsProperty(CSSPropertyBackdropFilter)
+        || containsProperty(CSSPropertyWebkitBackdropFilter)
+        || containsProperty(CSSPropertyClipPath)
+        || containsProperty(CSSPropertyFilter)
+#if ENABLE(CSS_COMPOSITING)
+        || containsProperty(CSSPropertyMixBlendMode)
+#endif
+        || containsProperty(CSSPropertyMask);
+}
+
 // "If any non-initial value of a property would create a stacking context on the element,
 // specifying that property in will-change must create a stacking context on the element."
 bool WillChangeData::propertyCreatesStackingContext(CSSPropertyID property)
@@ -108,10 +120,10 @@ bool WillChangeData::propertyCreatesStackingContext(CSSPropertyID property)
     case CSSPropertyIsolation:
 #endif
     case CSSPropertyFilter:
-#if ENABLE(FILTERS_LEVEL_2)
+    case CSSPropertyBackdropFilter:
     case CSSPropertyWebkitBackdropFilter:
-#endif
     case CSSPropertyMaskImage:
+    case CSSPropertyMaskBorder:
     case CSSPropertyWebkitMaskBoxImage:
 #if ENABLE(OVERFLOW_SCROLLING_TOUCH)
     case CSSPropertyWebkitOverflowScrolling:
@@ -128,9 +140,8 @@ static bool propertyTriggersCompositing(CSSPropertyID property)
     switch (property) {
     case CSSPropertyOpacity:
     case CSSPropertyFilter:
-#if ENABLE(FILTERS_LEVEL_2)
+    case CSSPropertyBackdropFilter:
     case CSSPropertyWebkitBackdropFilter:
-#endif
         return true;
     default:
         return false;
