@@ -20,7 +20,10 @@
 #include "config.h"
 #include "RenderSVGGradientStop.h"
 
-#include "RenderSVGResourceContainer.h"
+#include "ElementInlines.h"
+#include "LegacyRenderSVGResourceContainer.h"
+#include "RenderSVGGradientStopInlines.h"
+#include "SVGElementTypeHelpers.h"
 #include "SVGGradientElement.h"
 #include "SVGNames.h"
 #include "SVGResourcesCache.h"
@@ -35,8 +38,9 @@ using namespace SVGNames;
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderSVGGradientStop);
 
 RenderSVGGradientStop::RenderSVGGradientStop(SVGStopElement& element, RenderStyle&& style)
-    : RenderElement(element, WTFMove(style), 0)
+    : RenderElement(Type::SVGGradientStop, element, WTFMove(style), { }, { })
 {
+    ASSERT(isRenderSVGGradientStop());
 }
 
 RenderSVGGradientStop::~RenderSVGGradientStop() = default;
@@ -57,7 +61,7 @@ void RenderSVGGradientStop::styleDidChange(StyleDifference diff, const RenderSty
     if (!renderer)
         return;
 
-    downcast<RenderSVGResourceContainer>(*renderer).removeAllClientsFromCache();
+    downcast<LegacyRenderSVGResourceContainer>(*renderer).removeAllClientsFromCache();
 }
 
 void RenderSVGGradientStop::layout()
