@@ -38,14 +38,16 @@ namespace WebCore {
 WTF_MAKE_ISO_ALLOCATED_IMPL(RenderSVGViewportContainer);
 
 RenderSVGViewportContainer::RenderSVGViewportContainer(RenderSVGRoot& parent, RenderStyle&& style)
-    : RenderSVGContainer(parent.document(), WTFMove(style))
+    : RenderSVGContainer(Type::SVGViewportContainer, parent.document(), WTFMove(style))
     , m_owningSVGRoot(parent)
 {
+    ASSERT(isRenderSVGViewportContainer());
 }
 
 RenderSVGViewportContainer::RenderSVGViewportContainer(SVGSVGElement& element, RenderStyle&& style)
-    : RenderSVGContainer(element, WTFMove(style))
+    : RenderSVGContainer(Type::SVGViewportContainer, element, WTFMove(style))
 {
+    ASSERT(isRenderSVGViewportContainer());
 }
 
 SVGSVGElement& RenderSVGViewportContainer::svgSVGElement() const
