@@ -34,6 +34,7 @@
 #include "HTMLSlotElement.h"
 #include "LayoutState.h"
 #include "LayoutTreeBuilder.h"
+#include "LegacyRenderSVGResource.h"
 #include "LocalFrameView.h"
 #include "LocalFrameViewLayoutContext.h"
 #include "NodeRenderStyle.h"
@@ -43,7 +44,6 @@
 #include "RenderListItem.h"
 #include "RenderMultiColumnFlow.h"
 #include "RenderMultiColumnSet.h"
-#include "RenderSVGResource.h"
 #include "RenderStyleConstants.h"
 #include "RenderStyleInlines.h"
 #include "RenderTreeUpdaterGeneratedContent.h"
@@ -83,10 +83,8 @@ RenderTreeUpdater::RenderTreeUpdater(Document& document, Style::PostResolutionCa
 
 RenderTreeUpdater::~RenderTreeUpdater() = default;
 
-static ContainerNode* findRenderingRoot(ContainerNode& node)
+static Element* findRenderingAncestor(Node& node)
 {
-    if (node.renderer())
-        return &node;
     for (auto& ancestor : composedTreeAncestors(node)) {
         if (ancestor.renderer())
             return &ancestor;
@@ -96,7 +94,14 @@ static ContainerNode* findRenderingRoot(ContainerNode& node)
     return nullptr;
 }
 
-void RenderTreeUpdater::commit(std::unique_ptr<const Style::Update> styleUpdate)
+static ContainerNode* findRenderingRoot(ContainerNode& node)
+{
+    if (node.renderer())
+        return &node;
+    return findRenderingAncestor(node);
+}
+
+void RenderTreeUpdater::commit(std::unique_ptr<Style::Update> styleUpdate)
 {
     ASSERT(&m_document == &styleUpdate->document());
 
@@ -107,6 +112,8 @@ void RenderTreeUpdater::commit(std::unique_ptr<const Style::Update> styleUpdate)
 
     m_styleUpdate = WTFMove(styleUpdate);
 
+    updateRebuildRoots();
+
     updateRenderViewStyle();
 
     for (auto& root : m_styleUpdate->roots()) {
@@ -126,6 +133,68 @@ void RenderTreeUpdater::commit(std::unique_ptr<const Style::Update> styleUpdate)
     m_styleUpdate = nullptr;
 }
 
+void RenderTreeUpdater::updateRebuildRoots()
+{
+    auto findNewRebuildRoot = [&](auto& root) -> Element* {
+        auto* renderingAncestor = findRenderingAncestor(root);
+        if (!renderingAncestor)
+            return nullptr;
+        if (!RenderTreeBuilder::isRebuildRootForChildren(*renderingAncestor->renderer()))
+            return nullptr;
+        return renderingAncestor;
+    };
+
+    auto addForRebuild = [&](auto& element) {
+        auto* existingUpdate = m_styleUpdate->elementUpdate(element);
+        if (existingUpdate) {
+            if (existingUpdate->change == Style::Change::Renderer)
+                return false;
+            existingUpdate->change = Style::Change::Renderer;
+            return true;
+        }
+
+        if (!element.renderer())
+            return element.hasDisplayContents();
+
+        auto* parent = composedTreeAncestors(element).first();
+        m_styleUpdate->addElement(element, parent, Style::ElementUpdate {
+            RenderStyle::clonePtr(element.renderer()->style()),
+            Style::Change::Renderer
+        });
+        return true;
+    };
+
+    auto addSubtreeForRebuild = [&](auto& root) {
+        if (!addForRebuild(root))
+            return;
+        auto descendants = composedTreeDescendants(root);
+        auto it = descendants.begin();
+        auto end = descendants.end();
+        while (it != end) {
+            auto* descendant = dynamicDowncast<Element>(*it);
+            if (!descendant) {
+                it.traverseNext();
+                continue;
+            }
+            if (!addForRebuild(*descendant)) {
+                it.traverseNextSkippingChildren();
+                continue;
+            }
+            it.traverseNext();
+        }
+    };
+
+    while (true) {
+        auto rebuildRoots = m_styleUpdate->takeRebuildRoots();
+        if (rebuildRoots.isEmpty())
+            break;
+        for (auto& rebuildRoot : rebuildRoots) {
+            if (auto* newRebuildRoot = findNewRebuildRoot(*rebuildRoot))
+                addSubtreeForRebuild(*newRebuildRoot);
+        }
+    }
+}
+
 static bool shouldCreateRenderer(const Element& element, const RenderElement& parentRenderer)
 {
     if (!parentRenderer.canHaveChildren() && !(element.isPseudoElement() && parentRenderer.canHaveGeneratedChildren()))
@@ -156,18 +225,17 @@ void RenderTreeUpdater::updateRenderTree(ContainerNode& root)
 
         if (auto* renderer = node.renderer())
             renderTreePosition().invalidateNextSibling(*renderer);
-        else if (is<Element>(node) && downcast<Element>(node).hasDisplayContents())
+        else if (auto* element = dynamicDowncast<Element>(node); element && element->hasDisplayContents())
             renderTreePosition().invalidateNextSibling();
 
-        if (is<Text>(node)) {
-            auto& text = downcast<Text>(node);
-            auto* textUpdate = m_styleUpdate->textUpdate(text);
+        if (auto* text = dynamicDowncast<Text>(node)) {
+            auto* textUpdate = m_styleUpdate->textUpdate(*text);
             bool didCreateParent = parent().update && parent().update->change == Style::Change::Renderer;
-            bool mayNeedUpdateWhitespaceOnlyRenderer = renderingParent().didCreateOrDestroyChildRenderer && text.data().containsOnly<isASCIIWhitespace>();
+            bool mayNeedUpdateWhitespaceOnlyRenderer = renderingParent().didCreateOrDestroyChildRenderer && text->data().containsOnly<isASCIIWhitespace>();
             if (didCreateParent || textUpdate || mayNeedUpdateWhitespaceOnlyRenderer)
-                updateTextRenderer(text, textUpdate);
+                updateTextRenderer(*text, textUpdate);
 
-            storePreviousRenderer(text);
+            storePreviousRenderer(*text);
             it.traverseNextSkippingChildren();
             continue;
         }
@@ -195,7 +263,7 @@ void RenderTreeUpdater::updateRenderTree(ContainerNode& root)
 
         auto mayHaveRenderedDescendants = [&]() {
             if (element.renderer())
-                return !(element.isInTopLayer() && element.renderer()->style().effectiveSkippedContent());
+                return !(element.isInTopLayer() && element.renderer()->isSkippedContent());
             return element.hasDisplayContents() && shouldCreateRenderer(element, renderTreePosition().parent());
         }();
 
@@ -308,8 +376,19 @@ void RenderTreeUpdater::updateSVGRenderer(Element& element)
 {
     ASSERT(element.needsSVGRendererUpdate());
     element.setNeedsSVGRendererUpdate(false);
-    if (element.renderer())
-        RenderSVGResource::markForLayoutAndParentResourceInvalidation(*element.renderer());
+
+    auto* renderer = element.renderer();
+    if (!renderer)
+        return;
+
+#if ENABLE(LAYER_BASED_SVG_ENGINE)
+    if (element.document().settings().layerBasedSVGEngineEnabled()) {
+        renderer->setNeedsLayout();
+        return;
+    }
+#endif
+
+    LegacyRenderSVGResource::markForLayoutAndParentResourceInvalidation(*renderer);
 }
 
 void RenderTreeUpdater::updateElementRenderer(Element& element, const Style::ElementUpdate& elementUpdate)
@@ -324,7 +403,7 @@ void RenderTreeUpdater::updateElementRenderer(Element& element, const Style::Ele
     auto elementUpdateStyle = RenderStyle::cloneIncludingPseudoElements(*elementUpdate.style);
 
     bool shouldTearDownRenderers = [&]() {
-        if (element.isInTopLayer() && elementUpdate.change == Style::Change::Inherited && elementUpdate.style->effectiveSkippedContent())
+        if (element.isInTopLayer() && elementUpdate.change == Style::Change::Inherited && elementUpdate.style->skippedContentReason().has_value())
             return true;
         return elementUpdate.change == Style::Change::Renderer && (element.renderer() || element.hasDisplayContents());
     }();
@@ -364,7 +443,7 @@ void RenderTreeUpdater::updateElementRenderer(Element& element, const Style::Ele
         }
     });
 
-    bool shouldCreateNewRenderer = !element.renderer() && !hasDisplayContents && !(element.isInTopLayer() && renderTreePosition().parent().style().effectiveSkippedContent());
+    bool shouldCreateNewRenderer = !element.renderer() && !hasDisplayContents && !(element.isInTopLayer() && renderTreePosition().parent().style().skippedContentReason().has_value());
     if (shouldCreateNewRenderer) {
         if (element.hasCustomStyleResolveCallbacks())
             element.willAttachRenderers();
@@ -446,7 +525,7 @@ bool RenderTreeUpdater::textRendererIsNeeded(const Text& textNode)
     if (is<RenderText>(renderingParent.previousChildRenderer))
         return true;
     // This text node has nothing but white space. We may still need a renderer in some cases.
-    if (parentRenderer.isTable() || parentRenderer.isTableRow() || parentRenderer.isTableSection() || parentRenderer.isRenderTableCol() || parentRenderer.isFrameSet() || parentRenderer.isRenderGrid() || (parentRenderer.isFlexibleBox() && !parentRenderer.isRenderButton()))
+    if (parentRenderer.isRenderTable() || parentRenderer.isRenderTableRow() || parentRenderer.isRenderTableSection() || parentRenderer.isRenderTableCol() || parentRenderer.isRenderFrameSet() || parentRenderer.isRenderGrid() || (parentRenderer.isRenderFlexibleBox() && !parentRenderer.isRenderButton()))
         return false;
     if (parentRenderer.style().preserveNewline()) // pre/pre-wrap/pre-line always make renderers.
         return true;
@@ -492,7 +571,7 @@ void RenderTreeUpdater::createTextRenderer(Text& textNode, const Style::TextUpda
     if (textUpdate && textUpdate->inheritedDisplayContentsStyle && *textUpdate->inheritedDisplayContentsStyle) {
         // Wrap text renderer into anonymous inline so we can give it a style.
         // This is to support "<div style='display:contents;color:green'>text</div>" type cases
-        auto newDisplayContentsAnonymousWrapper = WebCore::createRenderer<RenderInline>(textNode.document(), RenderStyle::clone(**textUpdate->inheritedDisplayContentsStyle));
+        auto newDisplayContentsAnonymousWrapper = WebCore::createRenderer<RenderInline>(RenderObject::Type::Inline, textNode.document(), RenderStyle::clone(**textUpdate->inheritedDisplayContentsStyle));
         newDisplayContentsAnonymousWrapper->initializeStyle();
         auto& displayContentsAnonymousWrapper = *newDisplayContentsAnonymousWrapper;
         m_builder.attach(renderTreePosition.parent(), WTFMove(newDisplayContentsAnonymousWrapper), renderTreePosition.nextSibling());
@@ -553,13 +632,27 @@ void RenderTreeUpdater::updateRenderViewStyle()
         m_document.renderView()->setStyle(RenderStyle::clone(*m_styleUpdate->initialContainingBlockUpdate()));
 }
 
+static void invalidateRebuildRootIfNeeded(Node& node)
+{
+    auto* ancestor = findRenderingAncestor(node);
+    if (!ancestor)
+        return;
+    if (!RenderTreeBuilder::isRebuildRootForChildren(*ancestor->renderer()))
+        return;
+    ancestor->invalidateRenderer();
+}
+
 void RenderTreeUpdater::tearDownRenderers(Element& root)
 {
+    if (!root.renderer() && !root.hasDisplayContents())
+        return;
     auto* view = root.document().renderView();
     if (!view)
         return;
+
     RenderTreeBuilder builder(*view);
     tearDownRenderers(root, TeardownType::Full, builder);
+    invalidateRebuildRootIfNeeded(root);
 }
 
 void RenderTreeUpdater::tearDownRenderersAfterSlotChange(Element& host)
@@ -570,8 +663,10 @@ void RenderTreeUpdater::tearDownRenderersAfterSlotChange(Element& host)
     auto* view = host.document().renderView();
     if (!view)
         return;
+
     RenderTreeBuilder builder(*view);
     tearDownRenderers(host, TeardownType::FullAfterSlotChange, builder);
+    invalidateRebuildRootIfNeeded(host);
 }
 
 void RenderTreeUpdater::tearDownRenderer(Text& text)
@@ -579,8 +674,10 @@ void RenderTreeUpdater::tearDownRenderer(Text& text)
     auto* view = text.document().renderView();
     if (!view)
         return;
+
     RenderTreeBuilder builder(*view);
     tearDownTextRenderer(text, builder);
+    invalidateRebuildRootIfNeeded(text);
 }
 
 void RenderTreeUpdater::tearDownRenderers(Element& root, TeardownType teardownType, RenderTreeBuilder& builder)
@@ -653,8 +750,8 @@ void RenderTreeUpdater::tearDownRenderers(Element& root, TeardownType teardownTy
     for (auto it = descendants.begin(), end = descendants.end(); it != end; ++it) {
         pop(it.depth());
 
-        if (is<Text>(*it)) {
-            tearDownTextRenderer(downcast<Text>(*it), builder);
+        if (auto* text = dynamicDowncast<Text>(*it)) {
+            tearDownTextRenderer(*text, builder);
             continue;
         }
 
@@ -692,12 +789,12 @@ void RenderTreeUpdater::tearDownLeftoverChildrenOfComposedTree(Element& element,
     for (auto* child = element.firstChild(); child; child = child->nextSibling()) {
         if (!child->renderer())
             continue;
-        if (is<Text>(*child)) {
-            tearDownTextRenderer(downcast<Text>(*child), builder);
+        if (auto* text = dynamicDowncast<Text>(*child)) {
+            tearDownTextRenderer(*text, builder);
             continue;
         }
-        if (is<Element>(*child))
-            tearDownRenderers(downcast<Element>(*child), TeardownType::Full, builder);
+        if (auto* element = dynamicDowncast<Element>(*child))
+            tearDownRenderers(*element, TeardownType::Full, builder);
     }
 }
 
