@@ -46,22 +46,20 @@ struct MatchedProperties {
 
 struct MatchResult {
     WTF_MAKE_STRUCT_FAST_ALLOCATED;
+    MatchResult(bool isForLink = false)
+        : isForLink(isForLink)
+    { }
 
+    bool isForLink { false };
     bool isCacheable { true };
     Vector<MatchedProperties> userAgentDeclarations;
     Vector<MatchedProperties> userDeclarations;
     Vector<MatchedProperties> authorDeclarations;
 
     bool isEmpty() const { return userAgentDeclarations.isEmpty() && userDeclarations.isEmpty() && authorDeclarations.isEmpty(); }
-};
 
-inline bool operator==(const MatchResult& a, const MatchResult& b)
-{
-    return a.isCacheable == b.isCacheable
-        && a.userAgentDeclarations == b.userAgentDeclarations
-        && a.userDeclarations == b.userDeclarations
-        && a.authorDeclarations == b.authorDeclarations;
-}
+    friend bool operator==(const MatchResult&, const MatchResult&) = default;
+};
 
 inline bool operator==(const MatchedProperties& a, const MatchedProperties& b)
 {
@@ -87,7 +85,7 @@ inline void add(Hasher& hasher, const MatchedProperties& matchedProperties)
 
 inline void add(Hasher& hasher, const MatchResult& matchResult)
 {
-    add(hasher, matchResult.userAgentDeclarations, matchResult.userDeclarations, matchResult.authorDeclarations);
+    add(hasher, matchResult.isForLink, matchResult.userAgentDeclarations, matchResult.userDeclarations, matchResult.authorDeclarations);
 }
 
 }
