@@ -32,6 +32,7 @@
 #include "CSSSelector.h"
 #include "CSSSelectorList.h"
 #include "RuleSet.h"
+#include "StyleRule.h"
 
 namespace WebCore {
 namespace Style {
@@ -44,16 +45,21 @@ static bool isSiblingOrSubject(MatchElement matchElement)
     case MatchElement::DirectSibling:
     case MatchElement::AnySibling:
     case MatchElement::HasSibling:
+    case MatchElement::HasAnySibling:
     case MatchElement::Host:
+    case MatchElement::HostChild:
         return true;
     case MatchElement::Parent:
     case MatchElement::Ancestor:
     case MatchElement::ParentSibling:
     case MatchElement::AncestorSibling:
+    case MatchElement::ParentAnySibling:
+    case MatchElement::AncestorAnySibling:
     case MatchElement::HasChild:
     case MatchElement::HasDescendant:
     case MatchElement::HasSiblingDescendant:
-    case MatchElement::HasNonSubjectOrScopeBreaking:
+    case MatchElement::HasNonSubject:
+    case MatchElement::HasScopeBreaking:
         return false;
     }
     ASSERT_NOT_REACHED();
@@ -67,13 +73,44 @@ bool isHasPseudoClassMatchElement(MatchElement matchElement)
     case MatchElement::HasDescendant:
     case MatchElement::HasSibling:
     case MatchElement::HasSiblingDescendant:
-    case MatchElement::HasNonSubjectOrScopeBreaking:
+    case MatchElement::HasAnySibling:
+    case MatchElement::HasNonSubject:
+    case MatchElement::HasScopeBreaking:
         return true;
     default:
         return false;
     }
 }
 
+static bool isScopeBreaking(MatchElement matchElement)
+{
+    switch (matchElement) {
+    case MatchElement::HasAnySibling:
+    case MatchElement::HasScopeBreaking:
+        return true;
+    case MatchElement::Subject:
+    case MatchElement::IndirectSibling:
+    case MatchElement::DirectSibling:
+    case MatchElement::AnySibling:
+    case MatchElement::HasSibling:
+    case MatchElement::Host:
+    case MatchElement::HostChild:
+    case MatchElement::Parent:
+    case MatchElement::Ancestor:
+    case MatchElement::ParentSibling:
+    case MatchElement::AncestorSibling:
+    case MatchElement::ParentAnySibling:
+    case MatchElement::AncestorAnySibling:
+    case MatchElement::HasChild:
+    case MatchElement::HasDescendant:
+    case MatchElement::HasSiblingDescendant:
+    case MatchElement::HasNonSubject:
+        return false;
+    }
+    ASSERT_NOT_REACHED();
+    return false;
+}
+
 RuleAndSelector::RuleAndSelector(const RuleData& ruleData)
     : styleRule(&ruleData.styleRule())
     , selectorIndex(ruleData.selectorIndex())
@@ -121,8 +158,7 @@ static MatchElement computeNextMatchElement(MatchElement matchElement, CSSSelect
         case CSSSelector::RelationType::ShadowPartDescendant:
             return MatchElement::Host;
         case CSSSelector::RelationType::ShadowSlotted:
-            // FIXME: Implement accurate invalidation.
-            return matchElement;
+            return MatchElement::HostChild;
         };
     }
     switch (relation) {
@@ -138,8 +174,7 @@ static MatchElement computeNextMatchElement(MatchElement matchElement, CSSSelect
     case CSSSelector::RelationType::ShadowPartDescendant:
         return MatchElement::Host;
     case CSSSelector::RelationType::ShadowSlotted:
-        // FIXME: Implement accurate invalidation.
-        return matchElement;
+        return MatchElement::HostChild;
     };
     ASSERT_NOT_REACHED();
     return matchElement;
@@ -149,11 +184,23 @@ static MatchElement computeNextHasPseudoClassMatchElement(MatchElement matchElem
 {
     ASSERT(isHasPseudoClassMatchElement(matchElement));
 
-    // :has(:is(foo bar)) can be affected by changes outside the :has scope.
-    if (canBreakScope == CanBreakScope::Yes) {
-        if (relation == CSSSelector::RelationType::DescendantSpace || relation == CSSSelector::RelationType::Child)
-            return MatchElement::HasNonSubjectOrScopeBreaking;
+    if (canBreakScope == CanBreakScope::No)
+        return matchElement;
+
+    // `:has(:is(foo bar))` can be affected by changes outside the :has scope.
+    if (relation == CSSSelector::RelationType::DescendantSpace || relation == CSSSelector::RelationType::Child)
+        return MatchElement::HasScopeBreaking;
+
+    if (relation == CSSSelector::RelationType::IndirectAdjacent || relation == CSSSelector::RelationType::DirectAdjacent) {
+        // `:has(~ :is(.x ~ .y))` must look at previous siblings of the :scope scope too.
+        if (matchElement == MatchElement::HasSibling)
+            return MatchElement::HasAnySibling;
+
+        // `:has(~ :is(.x ~ .y)) .z` must be treated as scope breaking, rather than HasAnySibling like the previous case.
+        if (matchElement == MatchElement::HasNonSubject)
+            return MatchElement::HasScopeBreaking;
     }
+
     return matchElement;
 }
 
@@ -175,13 +222,18 @@ MatchElement computeHasPseudoClassMatchElement(const CSSSelector& hasSelector)
         return MatchElement::HasSibling;
     case MatchElement::ParentSibling:
     case MatchElement::AncestorSibling:
+    case MatchElement::ParentAnySibling:
+    case MatchElement::AncestorAnySibling:
         return MatchElement::HasSiblingDescendant;
     case MatchElement::HasChild:
     case MatchElement::HasDescendant:
     case MatchElement::HasSibling:
     case MatchElement::HasSiblingDescendant:
-    case MatchElement::HasNonSubjectOrScopeBreaking:
+    case MatchElement::HasAnySibling:
+    case MatchElement::HasNonSubject:
+    case MatchElement::HasScopeBreaking:
     case MatchElement::Host:
+    case MatchElement::HostChild:
         ASSERT_NOT_REACHED();
         break;
     }
@@ -193,8 +245,13 @@ static MatchElement computeSubSelectorMatchElement(MatchElement matchElement, co
     if (selector.match() == CSSSelector::Match::PseudoClass) {
         auto type = selector.pseudoClassType();
         // For :nth-child(n of .some-subselector) where an element change may affect other elements similar to sibling combinators.
-        if (type == CSSSelector::PseudoClassType::NthChild || type == CSSSelector::PseudoClassType::NthLastChild)
+        if (type == CSSSelector::PseudoClassType::NthChild || type == CSSSelector::PseudoClassType::NthLastChild) {
+            if (matchElement == MatchElement::Parent)
+                return MatchElement::ParentAnySibling;
+            if (matchElement == MatchElement::Ancestor)
+                return MatchElement::AncestorAnySibling;
             return MatchElement::AnySibling;
+        }
 
         // Similarly for :host().
         if (type == CSSSelector::PseudoClassType::Host)
@@ -202,7 +259,7 @@ static MatchElement computeSubSelectorMatchElement(MatchElement matchElement, co
 
         if (type == CSSSelector::PseudoClassType::Has) {
             if (matchElement != MatchElement::Subject)
-                return MatchElement::HasNonSubjectOrScopeBreaking;
+                return MatchElement::HasNonSubject;
             return computeHasPseudoClassMatchElement(childSelector);
         }
 
@@ -214,9 +271,9 @@ static MatchElement computeSubSelectorMatchElement(MatchElement matchElement, co
     }
 
     return matchElement;
-};
+}
 
-void RuleFeatureSet::recursivelyCollectFeaturesFromSelector(SelectorFeatures& selectorFeatures, const CSSSelector& firstSelector, MatchElement matchElement, IsNegation isNegation, CanBreakScope canBreakScope)
+DoesBreakScope RuleFeatureSet::recursivelyCollectFeaturesFromSelector(SelectorFeatures& selectorFeatures, const CSSSelector& firstSelector, MatchElement matchElement, IsNegation isNegation, CanBreakScope canBreakScope)
 {
     const CSSSelector* selector = &firstSelector;
     do {
@@ -224,7 +281,7 @@ void RuleFeatureSet::recursivelyCollectFeaturesFromSelector(SelectorFeatures& se
             idsInRules.add(selector->value());
             if (matchElement == MatchElement::Parent || matchElement == MatchElement::Ancestor)
                 idsMatchingAncestorsInRules.add(selector->value());
-            else if (isHasPseudoClassMatchElement(matchElement) || matchElement == MatchElement::AnySibling)
+            else if (isHasPseudoClassMatchElement(matchElement) || matchElement == MatchElement::AnySibling || matchElement == MatchElement::Host || matchElement == MatchElement::HostChild)
                 selectorFeatures.ids.append({ selector, matchElement, isNegation });
         } else if (selector->match() == CSSSelector::Match::Class)
             selectorFeatures.classes.append({ selector, matchElement, isNegation });
@@ -260,12 +317,10 @@ void RuleFeatureSet::recursivelyCollectFeaturesFromSelector(SelectorFeatures& se
 
             for (const CSSSelector* subSelector = selectorList->first(); subSelector; subSelector = CSSSelectorList::next(subSelector)) {
                 auto subSelectorMatchElement = computeSubSelectorMatchElement(matchElement, *selector, *subSelector);
-                if (!selectorFeatures.hasSiblingSelector && selector->isSiblingSelector())
-                    selectorFeatures.hasSiblingSelector = true;
-                recursivelyCollectFeaturesFromSelector(selectorFeatures, *subSelector, subSelectorMatchElement, subSelectorIsNegation, canBreakScope);
+                auto doesBreakScope = recursivelyCollectFeaturesFromSelector(selectorFeatures, *subSelector, subSelectorMatchElement, subSelectorIsNegation, canBreakScope);
 
                 if (selector->match() == CSSSelector::Match::PseudoClass && selector->pseudoClassType() == CSSSelector::PseudoClassType::Has)
-                    selectorFeatures.hasPseudoClasses.append({ subSelector, subSelectorMatchElement, isNegation });
+                    selectorFeatures.hasPseudoClasses.append({ subSelector, subSelectorMatchElement, isNegation, doesBreakScope });
             }
         }
 
@@ -277,6 +332,8 @@ void RuleFeatureSet::recursivelyCollectFeaturesFromSelector(SelectorFeatures& se
 
         selector = selector->tagHistory();
     } while (selector);
+
+    return isScopeBreaking(matchElement) ? DoesBreakScope::Yes : DoesBreakScope::No;
 }
 
 PseudoClassInvalidationKey makePseudoClassInvalidationKey(CSSSelector::PseudoClassType pseudoClass, InvalidationKeyType keyType, const AtomString& keyString)
@@ -315,10 +372,24 @@ static PseudoClassInvalidationKey makePseudoClassInvalidationKey(CSSSelector::Ps
     return makePseudoClassInvalidationKey(pseudoClassType, InvalidationKeyType::Universal);
 };
 
-void RuleFeatureSet::collectFeatures(const RuleData& ruleData)
+void RuleFeatureSet::collectFeatures(const RuleData& ruleData, const Vector<Ref<const StyleRuleScope>>& scopeRules)
 {
     SelectorFeatures selectorFeatures;
     recursivelyCollectFeaturesFromSelector(selectorFeatures, *ruleData.selector());
+
+    for (auto& scopeRule : scopeRules) {
+        auto collectSelectorList = [&] (const auto& selectorList) {
+            if (!selectorList.isEmpty()) {
+                for (const auto* subSelector = selectorList.first() ; subSelector; subSelector = CSSSelectorList::next(subSelector)) {
+                    recursivelyCollectFeaturesFromSelector(selectorFeatures, *subSelector, MatchElement::Ancestor);
+                    recursivelyCollectFeaturesFromSelector(selectorFeatures, *subSelector, MatchElement::Subject);
+                }
+            }
+        };
+        collectSelectorList(scopeRule->scopeStart());
+        collectSelectorList(scopeRule->scopeEnd());
+    }
+
     if (selectorFeatures.hasSiblingSelector)
         siblingRules.append({ ruleData });
     if (ruleData.containsUncommonAttributeSelector())
@@ -369,12 +440,15 @@ void RuleFeatureSet::collectFeatures(const RuleData& ruleData)
     }
 
     for (auto& entry : selectorFeatures.hasPseudoClasses) {
-        auto& [selector, matchElement, isNegation] = entry;
+        auto& [selector, matchElement, isNegation, doesBreakScope] = entry;
         // The selector argument points to a selector inside :has() selector list instead of :has() itself.
         hasPseudoClassRules.ensure(makePseudoClassInvalidationKey(CSSSelector::PseudoClassType::Has, *selector), [] {
             return makeUnique<Vector<RuleFeatureWithInvalidationSelector>>();
         }).iterator->value->append({ ruleData, matchElement, isNegation, selector });
 
+        if (doesBreakScope == DoesBreakScope::Yes)
+            scopeBreakingHasPseudoClassRules.append({ ruleData });
+
         setUsesMatchElement(matchElement);
     }
 }
@@ -410,6 +484,7 @@ void RuleFeatureSet::add(const RuleFeatureSet& other)
     pseudoClassTypes.add(other.pseudoClassTypes.begin(), other.pseudoClassTypes.end());
 
     addMap(hasPseudoClassRules, other.hasPseudoClassRules);
+    scopeBreakingHasPseudoClassRules.appendVector(other.scopeBreakingHasPseudoClassRules);
 
     for (size_t i = 0; i < usedMatchElements.size(); ++i)
         usedMatchElements[i] = usedMatchElements[i] || other.usedMatchElements[i];
@@ -437,6 +512,7 @@ void RuleFeatureSet::clear()
     idRules.clear();
     classRules.clear();
     hasPseudoClassRules.clear();
+    scopeBreakingHasPseudoClassRules.clear();
     classesAffectingHost.clear();
     attributeRules.clear();
     attributesAffectingHost.clear();
@@ -451,6 +527,7 @@ void RuleFeatureSet::shrinkToFit()
 {
     siblingRules.shrinkToFit();
     uncommonAttributeRules.shrinkToFit();
+    scopeBreakingHasPseudoClassRules.shrinkToFit();
     for (auto& rules : idRules.values())
         rules->shrinkToFit();
     for (auto& rules : classRules.values())
