@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
- * Copyright (C) 2003-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2003-2023 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -21,6 +21,7 @@
 
 #pragma once
 
+#include "CSSSelectorList.h"
 #include "MediaQuery.h"
 #include "RuleData.h"
 #include "RuleFeature.h"
@@ -118,6 +119,9 @@ public:
     bool hasContainerQueries() const { return !m_containerQueries.isEmpty(); }
     Vector<const CQ::ContainerQuery*> containerQueriesFor(const RuleData&) const;
 
+    bool hasScopeRules() const { return !m_scopeRules.isEmpty(); }
+    Vector<Ref<const StyleRuleScope>> scopeRulesFor(const RuleData&) const;
+
 private:
     friend class RuleSetBuilder;
 
@@ -125,8 +129,9 @@ private:
 
     using CascadeLayerIdentifier = unsigned;
     using ContainerQueryIdentifier = unsigned;
+    using ScopeRuleIdentifier = unsigned;
 
-    void addRule(RuleData&&, CascadeLayerIdentifier, ContainerQueryIdentifier);
+    void addRule(RuleData&&, CascadeLayerIdentifier, ContainerQueryIdentifier, ScopeRuleIdentifier);
 
     struct ResolverMutatingRule {
         Ref<StyleRuleBase> rule;
@@ -151,6 +156,11 @@ private:
     const CascadeLayer& cascadeLayerForIdentifier(CascadeLayerIdentifier identifier) const { return m_cascadeLayers[identifier - 1]; }
     CascadeLayerPriority cascadeLayerPriorityForIdentifier(CascadeLayerIdentifier) const;
 
+    struct ScopeAndParent {
+        Ref<const StyleRuleScope> scopeRule;
+        ScopeRuleIdentifier parent;
+    };
+
     struct ContainerQueryAndParent {
         Ref<StyleRuleContainer> containerRule;
         ContainerQueryIdentifier parent;
@@ -202,6 +212,10 @@ private:
     Vector<ContainerQueryAndParent> m_containerQueries;
     Vector<ContainerQueryIdentifier> m_containerQueryIdentifierForRulePosition;
 
+    // @scope
+    Vector<ScopeAndParent> m_scopeRules;
+    Vector<ScopeRuleIdentifier> m_scopeRuleIdentifierForRulePosition;
+
     bool m_hasHostPseudoClassRulesMatchingInShadowTree { false };
     bool m_hasViewportDependentMediaQueries { false };
 };
