@@ -41,6 +41,7 @@
 #include "StyleRuleImport.h"
 #include "StyleScope.h"
 #include "StyleSheetContents.h"
+#include "css/CSSSelectorList.h"
 #include <wtf/CryptographicallyRandomNumber.h>
 #include <wtf/text/StringConcatenateNumbers.h>
 
@@ -109,35 +110,69 @@ void RuleSetBuilder::addChildRule(Ref<StyleRuleBase> rule)
     switch (rule->type()) {
     case StyleRuleType::StyleWithNesting:
         if (m_ruleSet)
-            addStyleRule(downcast<StyleRuleWithNesting>(rule));
+            addStyleRule(uncheckedDowncast<StyleRuleWithNesting>(rule));
         return;
 
     case StyleRuleType::Style:
         if (m_ruleSet)
-            addStyleRule(downcast<StyleRule>(rule));
+            addStyleRule(uncheckedDowncast<StyleRule>(rule));
         return;
 
+    case StyleRuleType::Scope: {
+        auto scopeRule = uncheckedDowncast<StyleRuleScope>(WTFMove(rule));
+        auto previousScopeIdentifier = m_currentScopeIdentifier;
+        if (m_ruleSet) {
+            m_ruleSet->m_scopeRules.append({ scopeRule.copyRef(), previousScopeIdentifier });
+            m_currentScopeIdentifier = m_ruleSet->m_scopeRules.size();
+        }
+
+        // https://drafts.csswg.org/css-nesting/#nesting-at-scope
+        // For the purposes of the style rules in its body and its own <scope-end> selector,
+        // the @scope rule is treated as an ancestor style rule, matching the elements matched by its <scope-start> selector.
+        if (m_shouldResolveNesting == ShouldResolveNesting::Yes) {
+            const CSSSelectorList* parentResolvedSelectorList = nullptr;
+            if (m_selectorListStack.size())
+                parentResolvedSelectorList =  m_selectorListStack.last();
+            if (!scopeRule->originalScopeStart().isEmpty())
+                scopeRule->setScopeStart(CSSSelectorParser::resolveNestingParent(scopeRule->originalScopeStart(), parentResolvedSelectorList));
+            if (!scopeRule->originalScopeEnd().isEmpty())
+                scopeRule->setScopeEnd(CSSSelectorParser::resolveNestingParent(scopeRule->originalScopeEnd(), &scopeRule->scopeStart()));
+        }
+
+        const auto& scopeStart = scopeRule->scopeStart();
+        // If <scope-start> is empty, it doesn't create a nesting context (the nesting selector might eventually be replaced by :scope)
+        if (!scopeStart.isEmpty())
+            m_selectorListStack.append(&scopeStart);
+        addChildRules(scopeRule->childRules());
+        if (!scopeStart.isEmpty())
+            m_selectorListStack.removeLast();
+
+        if (m_ruleSet)
+            m_currentScopeIdentifier = previousScopeIdentifier;
+        return;
+    }
+
     case StyleRuleType::Page:
         if (m_ruleSet)
-            m_ruleSet->addPageRule(downcast<StyleRulePage>(rule));
+            m_ruleSet->addPageRule(uncheckedDowncast<StyleRulePage>(rule));
         return;
 
     case StyleRuleType::Media: {
-        auto& mediaRule = downcast<StyleRuleMedia>(rule);
-        if (m_mediaQueryCollector.pushAndEvaluate(mediaRule.mediaQueries()))
-            addChildRules(mediaRule.childRules());
-        m_mediaQueryCollector.pop(mediaRule.mediaQueries());
+        auto mediaRule = uncheckedDowncast<StyleRuleMedia>(WTFMove(rule));
+        if (m_mediaQueryCollector.pushAndEvaluate(mediaRule->mediaQueries()))
+            addChildRules(mediaRule->childRules());
+        m_mediaQueryCollector.pop(mediaRule->mediaQueries());
         return;
     }
 
     case StyleRuleType::Container: {
-        auto& containerRule = downcast<StyleRuleContainer>(rule);
+        auto containerRule = uncheckedDowncast<StyleRuleContainer>(WTFMove(rule));
         auto previousContainerQueryIdentifier = m_currentContainerQueryIdentifier;
         if (m_ruleSet) {
-            m_ruleSet->m_containerQueries.append({ containerRule, previousContainerQueryIdentifier });
+            m_ruleSet->m_containerQueries.append({ containerRule.copyRef(), previousContainerQueryIdentifier });
             m_currentContainerQueryIdentifier = m_ruleSet->m_containerQueries.size();
         }
-        addChildRules(containerRule.childRules());
+        addChildRules(containerRule->childRules());
         if (m_ruleSet)
             m_currentContainerQueryIdentifier = previousContainerQueryIdentifier;
         return;
@@ -147,16 +182,16 @@ void RuleSetBuilder::addChildRule(Ref<StyleRuleBase> rule)
     case StyleRuleType::LayerStatement: {
         disallowDynamicMediaQueryEvaluationIfNeeded();
 
-        auto& layerRule = downcast<StyleRuleLayer>(rule);
-        if (layerRule.isStatement()) {
+        auto layerRule = uncheckedDowncast<StyleRuleLayer>(WTFMove(rule));
+        if (layerRule->isStatement()) {
             // Statement syntax just registers the layers.
-            registerLayers(layerRule.nameList());
+            registerLayers(layerRule->nameList());
             return;
         }
         // Block syntax.
-        pushCascadeLayer(layerRule.name());
-        addChildRules(layerRule.childRules());
-        popCascadeLayer(layerRule.name());
+        pushCascadeLayer(layerRule->name());
+        addChildRules(layerRule->childRules());
+        popCascadeLayer(layerRule->name());
         return;
     }
     case StyleRuleType::CounterStyle:
@@ -170,11 +205,12 @@ void RuleSetBuilder::addChildRule(Ref<StyleRuleBase> rule)
             m_collectedResolverMutatingRules.append({ rule, m_currentCascadeLayerIdentifier });
         return;
 
-    case StyleRuleType::Supports:
-        if (downcast<StyleRuleSupports>(rule).conditionIsSupported())
-            addChildRules(downcast<StyleRuleSupports>(rule).childRules());
+    case StyleRuleType::Supports: {
+        auto supportsRule = uncheckedDowncast<StyleRuleSupports>(WTFMove(rule));
+        if (supportsRule->conditionIsSupported())
+            addChildRules(supportsRule->childRules());
         return;
-
+    }
     case StyleRuleType::Import:
     case StyleRuleType::Margin:
     case StyleRuleType::Namespace:
@@ -219,16 +255,15 @@ void RuleSetBuilder::addRulesFromSheetContents(const StyleSheetContents& sheet)
 void RuleSetBuilder::resolveSelectorListWithNesting(StyleRuleWithNesting& rule)
 {
     const CSSSelectorList* parentResolvedSelectorList = nullptr;
-    if (m_styleRuleStack.size()) 
-        parentResolvedSelectorList =  m_styleRuleStack.last();
+    if (m_selectorListStack.size())
+        parentResolvedSelectorList =  m_selectorListStack.last();
 
     // If it's a top-level rule without a nesting parent selector, keep the selector list as is.
     if (!rule.originalSelectorList().hasExplicitNestingParent() && !parentResolvedSelectorList)
         return;
 
     auto resolvedSelectorList = CSSSelectorParser::resolveNestingParent(rule.originalSelectorList(), parentResolvedSelectorList);
-    ASSERT(!resolvedSelectorList.hasExplicitNestingParent());
-    rule.wrapperAdoptSelectorList(WTFMove(resolvedSelectorList));    
+    rule.wrapperAdoptSelectorList(WTFMove(resolvedSelectorList));
 }
 
 void RuleSetBuilder::addStyleRuleWithSelectorList(const CSSSelectorList& selectorList, const StyleRule& rule)
@@ -238,7 +273,7 @@ void RuleSetBuilder::addStyleRuleWithSelectorList(const CSSSelectorList& selecto
         for (size_t selectorIndex = 0; selectorIndex != notFound; selectorIndex = selectorList.indexOfNextSelectorAfter(selectorIndex)) {
             RuleData ruleData(rule, selectorIndex, selectorListIndex, m_ruleSet->ruleCount());
             m_mediaQueryCollector.addRuleIfNeeded(ruleData);
-            m_ruleSet->addRule(WTFMove(ruleData), m_currentCascadeLayerIdentifier, m_currentContainerQueryIdentifier);
+            m_ruleSet->addRule(WTFMove(ruleData), m_currentCascadeLayerIdentifier, m_currentContainerQueryIdentifier, m_currentScopeIdentifier);
             ++selectorListIndex;
         }
     }
@@ -251,12 +286,12 @@ void RuleSetBuilder::addStyleRule(StyleRuleWithNesting& rule)
 
     auto& selectorList = rule.selectorList();
     addStyleRuleWithSelectorList(selectorList, rule);
-    
+
     // Process nested rules
-    m_styleRuleStack.append(&selectorList);
+    m_selectorListStack.append(&selectorList);
     for (auto& nestedRule : rule.nestedRules())
         addChildRule(nestedRule);
-    m_styleRuleStack.removeLast();
+    m_selectorListStack.removeLast();
 }
 
 void RuleSetBuilder::addStyleRule(const StyleRule& rule)
@@ -352,10 +387,9 @@ void RuleSetBuilder::updateCascadeLayerPriorities()
 
     auto layerCount = m_ruleSet->m_cascadeLayers.size();
 
-    Vector<RuleSet::CascadeLayerIdentifier> layersInPriorityOrder;
-    layersInPriorityOrder.reserveInitialCapacity(layerCount);
-    for (RuleSet::CascadeLayerIdentifier identifier = 1; identifier <= layerCount; ++identifier)
-        layersInPriorityOrder.uncheckedAppend(identifier);
+    Vector<RuleSet::CascadeLayerIdentifier> layersInPriorityOrder(layerCount, [](size_t i) {
+        return i + 1;
+    });
 
     std::sort(layersInPriorityOrder.begin(), layersInPriorityOrder.end(), compare);
 
@@ -394,39 +428,39 @@ void RuleSetBuilder::addMutatingRulesToResolver()
             m_ruleSet->m_resolverMutatingRulesInLayers.append(collectedRule);
 
         auto& rule = collectedRule.rule;
-        if (is<StyleRuleFontFace>(rule)) {
-            m_resolver->document().fontSelector().addFontFaceRule(downcast<StyleRuleFontFace>(rule.get()), false);
+        if (auto* styleRuleFontFace = dynamicDowncast<StyleRuleFontFace>(rule.get())) {
+            m_resolver->document().fontSelector().addFontFaceRule(*styleRuleFontFace, false);
             m_resolver->invalidateMatchedDeclarationsCache();
             continue;
         }
-        if (is<StyleRuleFontPaletteValues>(rule)) {
-            m_resolver->document().fontSelector().addFontPaletteValuesRule(downcast<StyleRuleFontPaletteValues>(rule.get()));
+        if (auto* styleRuleFontPaletteValues = dynamicDowncast<StyleRuleFontPaletteValues>(rule.get())) {
+            m_resolver->document().fontSelector().addFontPaletteValuesRule(*styleRuleFontPaletteValues);
             m_resolver->invalidateMatchedDeclarationsCache();
             continue;
         }
-        if (is<StyleRuleFontFeatureValues>(rule)) {
-            m_resolver->document().fontSelector().addFontFeatureValuesRule(downcast<StyleRuleFontFeatureValues>(rule.get()));
+        if (auto* styleRuleFontFeatureValues = dynamicDowncast<StyleRuleFontFeatureValues>(rule.get())) {
+            m_resolver->document().fontSelector().addFontFeatureValuesRule(*styleRuleFontFeatureValues);
             m_resolver->invalidateMatchedDeclarationsCache();
             continue;
         }
-        if (is<StyleRuleKeyframes>(rule)) {
-            m_resolver->addKeyframeStyle(downcast<StyleRuleKeyframes>(rule.get()));
+        if (auto* styleRuleKeyframes = dynamicDowncast<StyleRuleKeyframes>(rule.get())) {
+            m_resolver->addKeyframeStyle(*styleRuleKeyframes);
             continue;
         }
-        if (is<StyleRuleCounterStyle>(rule)) {
+        if (auto* styleRuleCounterStyle = dynamicDowncast<StyleRuleCounterStyle>(rule.get())) {
             if (m_resolver->scopeType() == Resolver::ScopeType::ShadowTree)
                 continue;
             auto& registry = m_resolver->document().styleScope().counterStyleRegistry();
-            registry.addCounterStyle(downcast<StyleRuleCounterStyle>(rule.get()).descriptors());
+            registry.addCounterStyle(styleRuleCounterStyle->descriptors());
             continue;
         }
-        if (is<StyleRuleProperty>(rule)) {
+        if (auto* styleRuleProperty = dynamicDowncast<StyleRuleProperty>(rule.get())) {
             // "A @property is invalid if it occurs in a stylesheet inside of a shadow tree, and must be ignored."
             // https://drafts.css-houdini.org/css-properties-values-api/#at-property-rule
             if (m_resolver->scopeType() == Resolver::ScopeType::ShadowTree)
                 continue;
             auto& registry = m_resolver->document().styleScope().customPropertyRegistry();
-            registry.registerFromStylesheet(downcast<StyleRuleProperty>(rule.get()).descriptor());
+            registry.registerFromStylesheet(styleRuleProperty->descriptor());
             continue;
         }
     }
@@ -473,9 +507,9 @@ void RuleSetBuilder::MediaQueryCollector::pop(const MQ::MediaQueryList& mediaQue
 
     if (!dynamicContextStack.last().affectedRulePositions.isEmpty() || !collectDynamic) {
         RuleSet::DynamicMediaQueryRules rules;
-        rules.mediaQueries.reserveCapacity(rules.mediaQueries.size() + dynamicContextStack.size());
-        for (auto& context : dynamicContextStack)
-            rules.mediaQueries.uncheckedAppend(context.queries);
+        rules.mediaQueries = WTF::map(dynamicContextStack, [](auto& context) {
+            return context.queries;
+        });
 
         if (collectDynamic) {
             rules.affectedRulePositions.appendVector(dynamicContextStack.last().affectedRulePositions);
