@@ -41,15 +41,13 @@ inline void traverseRuleFeaturesInShadowTree(Element& element, TraverseFunction&
         return;
 
     auto& shadowRuleSets = element.shadowRoot()->styleScope().resolver().ruleSets();
-    bool hasHostPseudoClassRulesMatchingInShadowTree = false;
     bool hasHostPseudoClassRule = shadowRuleSets.hasMatchingUserOrAuthorStyle([&] (auto& style) {
-        hasHostPseudoClassRulesMatchingInShadowTree = style.hasHostPseudoClassRulesMatchingInShadowTree();
-        return !style.hostPseudoClassRules().isEmpty();
+        return !style.hostPseudoClassRules().isEmpty() || style.hasHostPseudoClassRulesMatchingInShadowTree();
     });
-    if (!hasHostPseudoClassRule && !hasHostPseudoClassRulesMatchingInShadowTree)
+    if (!hasHostPseudoClassRule)
         return;
 
-    function(shadowRuleSets.features(), hasHostPseudoClassRulesMatchingInShadowTree);
+    function(shadowRuleSets.features(), false);
 }
 
 template <typename TraverseFunction>
@@ -79,8 +77,6 @@ inline void traverseRuleFeatures(Element& element, TraverseFunction&& function)
                 return true;
 #endif
         }
-        if (is<HTMLSlotElement>(element) && ruleSets.hasMatchingUserOrAuthorStyle([] (auto& style) { return !style.slottedPseudoElementRules().isEmpty(); }))
-            return true;
         return false;
     };
 
@@ -90,8 +86,11 @@ inline void traverseRuleFeatures(Element& element, TraverseFunction&& function)
     traverseRuleFeaturesForSlotted(element, function);
 
     // Ensure that the containing tree resolver also exists so it doesn't get created in the middle of invalidation.
-    if (element.isInShadowTree())
-        Style::Scope::forNode(*element.containingShadowRoot()->host()).resolver();
+    if (element.isInShadowTree() && element.containingShadowRoot()) {
+        auto& host = *element.containingShadowRoot()->host();
+        if (host.isConnected())
+            Style::Scope::forNode(host).resolver();
+    }
 }
 
 }
