@@ -50,7 +50,7 @@ struct SelectorMatchingState;
 
 class Invalidator {
 public:
-    Invalidator(const Vector<StyleSheetContents*>&, const MQ::MediaQueryEvaluator&);
+    Invalidator(const Vector<Ref<StyleSheetContents>>&, const MQ::MediaQueryEvaluator&);
     Invalidator(const InvalidationRuleSetVector&);
 
     ~Invalidator();
@@ -68,6 +68,7 @@ public:
     static void invalidateWithMatchElementRuleSets(Element&, const MatchElementRuleSets&);
     static void invalidateAllStyle(Scope&);
     static void invalidateHostAndSlottedStyleIfNeeded(ShadowRoot&);
+    static void invalidateWithScopeBreakingHasPseudoClassRuleSet(Element&, const RuleSet*);
 
 private:
     enum class CheckDescendants : bool { No, Yes };
@@ -81,6 +82,7 @@ private:
     struct RuleInformation {
         bool hasSlottedPseudoElementRules { false };
         bool hasHostPseudoClassRules { false };
+        bool hasHostPseudoClassRulesMatchingInShadowTree { false };
         bool hasShadowPseudoElementRules { false };
         bool hasCuePseudoElementRules { false };
         bool hasPartPseudoElementRules { false };
