@@ -85,13 +85,10 @@ void SVGComponentTransferFunctionElement::attributeChanged(const QualifiedName&
 void SVGComponentTransferFunctionElement::svgAttributeChanged(const QualifiedName& attrName)
 {
     if (PropertyRegistry::isKnownAttribute(attrName)) {
-        RefPtr parent = parentElement();
-
-        if (parent && is<SVGFEComponentTransferElement>(*parent)) {
+        if (RefPtr transferElement = dynamicDowncast<SVGFEComponentTransferElement>(parentElement())) {
             InstanceInvalidationGuard guard(*this);
-            downcast<SVGFEComponentTransferElement>(*parent).transferFunctionAttributeChanged(*this, attrName);
+            transferElement->transferFunctionAttributeChanged(*this, attrName);
         }
-
         return;
     }
 
