@@ -24,8 +24,8 @@
 #include "SVGGeometryElement.h"
 
 #include "DOMPoint.h"
+#include "LegacyRenderSVGResource.h"
 #include "LegacyRenderSVGShape.h"
-#include "RenderSVGResource.h"
 #include "RenderSVGShape.h"
 #include "SVGDocumentExtensions.h"
 #include "SVGPathUtilities.h"
@@ -47,18 +47,18 @@ SVGGeometryElement::SVGGeometryElement(const QualifiedName& tagName, Document& d
 
 float SVGGeometryElement::getTotalLength() const
 {
-    document().updateLayoutIgnorePendingStylesheets();
+    document().updateLayoutIgnorePendingStylesheets({ LayoutOptions::ContentVisibilityForceLayout }, this);
 
     auto* renderer = this->renderer();
     if (!renderer)
         return 0;
 
-    if (is<LegacyRenderSVGShape>(renderer))
-        return downcast<LegacyRenderSVGShape>(renderer)->getTotalLength();
+    if (auto* renderSVGShape = dynamicDowncast<LegacyRenderSVGShape>(renderer))
+        return renderSVGShape->getTotalLength();
 
 #if ENABLE(LAYER_BASED_SVG_ENGINE)
-    if (is<RenderSVGShape>(renderer))
-        return downcast<RenderSVGShape>(renderer)->getTotalLength();
+    if (auto* renderSVGShape = dynamicDowncast<RenderSVGShape>(renderer))
+        return renderSVGShape->getTotalLength();
 #endif
 
     ASSERT_NOT_REACHED();
@@ -67,7 +67,7 @@ float SVGGeometryElement::getTotalLength() const
 
 ExceptionOr<Ref<SVGPoint>> SVGGeometryElement::getPointAtLength(float distance) const
 {
-    document().updateLayoutIgnorePendingStylesheets();
+    document().updateLayoutIgnorePendingStylesheets({ LayoutOptions::ContentVisibilityForceLayout }, this);
 
     // Spec: Clamp distance to [0, length].
     distance = clampTo<float>(distance, 0, getTotalLength());
@@ -76,36 +76,36 @@ ExceptionOr<Ref<SVGPoint>> SVGGeometryElement::getPointAtLength(float distance)
 
     // Spec: If current element is a non-rendered element, throw an InvalidStateError.
     if (!renderer)
-        return Exception { InvalidStateError };
+        return Exception { ExceptionCode::InvalidStateError };
 
     // Spec: Return a newly created, detached SVGPoint object.
-    if (is<LegacyRenderSVGShape>(renderer))
-        return SVGPoint::create(downcast<LegacyRenderSVGShape>(renderer)->getPointAtLength(distance));
+    if (auto* renderSVGShape = dynamicDowncast<LegacyRenderSVGShape>(renderer))
+        return SVGPoint::create(renderSVGShape->getPointAtLength(distance));
 
 #if ENABLE(LAYER_BASED_SVG_ENGINE)
-    if (is<RenderSVGShape>(renderer))
-        return SVGPoint::create(downcast<RenderSVGShape>(renderer)->getPointAtLength(distance));
+    if (auto* renderSVGShape = dynamicDowncast<RenderSVGShape>(renderer))
+        return SVGPoint::create(renderSVGShape->getPointAtLength(distance));
 #endif
 
     ASSERT_NOT_REACHED();
-    return Exception { InvalidStateError };
+    return Exception { ExceptionCode::InvalidStateError };
 }
 
 bool SVGGeometryElement::isPointInFill(DOMPointInit&& pointInit)
 {
-    document().updateLayoutIgnorePendingStylesheets();
+    document().updateLayoutIgnorePendingStylesheets({ LayoutOptions::ContentVisibilityForceLayout }, this);
 
     auto* renderer = this->renderer();
     if (!renderer)
         return false;
 
     FloatPoint point {static_cast<float>(pointInit.x), static_cast<float>(pointInit.y)};
-    if (is<LegacyRenderSVGShape>(renderer))
-        return downcast<LegacyRenderSVGShape>(renderer)->isPointInFill(point);
+    if (auto* renderSVGShape = dynamicDowncast<LegacyRenderSVGShape>(renderer))
+        return renderSVGShape->isPointInFill(point);
 
 #if ENABLE(LAYER_BASED_SVG_ENGINE)
-    if (is<RenderSVGShape>(renderer))
-        return downcast<RenderSVGShape>(renderer)->isPointInFill(point);
+    if (auto* renderSVGShape = dynamicDowncast<RenderSVGShape>(renderer))
+        return renderSVGShape->isPointInFill(point);
 #endif
 
     ASSERT_NOT_REACHED();
@@ -114,35 +114,34 @@ bool SVGGeometryElement::isPointInFill(DOMPointInit&& pointInit)
 
 bool SVGGeometryElement::isPointInStroke(DOMPointInit&& pointInit)
 {
-    document().updateLayoutIgnorePendingStylesheets();
+    document().updateLayoutIgnorePendingStylesheets({ LayoutOptions::ContentVisibilityForceLayout }, this);
 
     auto* renderer = this->renderer();
     if (!renderer)
         return false;
 
     FloatPoint point {static_cast<float>(pointInit.x), static_cast<float>(pointInit.y)};
-    if (is<LegacyRenderSVGShape>(renderer))
-        return downcast<LegacyRenderSVGShape>(renderer)->isPointInStroke(point);
+    if (auto* renderSVGShape = dynamicDowncast<LegacyRenderSVGShape>(renderer))
+        return renderSVGShape->isPointInStroke(point);
 
 #if ENABLE(LAYER_BASED_SVG_ENGINE)
-    if (is<RenderSVGShape>(renderer))
-        return downcast<RenderSVGShape>(renderer)->isPointInStroke(point);
+    if (auto* renderSVGShape = dynamicDowncast<RenderSVGShape>(renderer))
+        return renderSVGShape->isPointInStroke(point);
 #endif
 
     ASSERT_NOT_REACHED();
     return false;
 }
 
-void SVGGeometryElement::parseAttribute(const QualifiedName& name, const AtomString& value)
+void SVGGeometryElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
     if (name == SVGNames::pathLengthAttr) {
-        m_pathLength->setBaseValInternal(value.toFloat());
+        m_pathLength->setBaseValInternal(newValue.toFloat());
         if (m_pathLength->baseVal() < 0)
             document().accessSVGExtensions().reportError("A negative value for path attribute <pathLength> is not allowed"_s);
-        return;
     }
 
-    SVGGraphicsElement::parseAttribute(name, value);
+    SVGGraphicsElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
 }
 
 void SVGGeometryElement::svgAttributeChanged(const QualifiedName& attrName)
