@@ -26,8 +26,6 @@
 #include "config.h"
 #include "SWServer.h"
 
-#if ENABLE(SERVICE_WORKER)
-
 #include "BackgroundFetchEngine.h"
 #include "BackgroundFetchInformation.h"
 #include "BackgroundFetchOptions.h"
@@ -70,12 +68,6 @@ SWServer::Connection::~Connection()
         request.callback({ });
 }
 
-HashSet<SWServer*>& SWServer::allServers()
-{
-    static NeverDestroyed<HashSet<SWServer*>> servers;
-    return servers;
-}
-
 SWServer::~SWServer()
 {
     // Destroy the remaining connections before the SWServer gets destroyed since they have a raw pointer
@@ -93,8 +85,6 @@ SWServer::~SWServer()
     }
     for (auto& runningWorker : runningWorkers)
         runningWorker->terminate();
-
-    allServers().remove(this);
 }
 
 SWServerWorker* SWServer::workerByID(ServiceWorkerIdentifier identifier) const
@@ -388,7 +378,7 @@ void SWServer::Connection::removeServiceWorkerRegistrationInServer(ServiceWorker
     m_server.removeClientServiceWorkerRegistration(*this, identifier);
 }
 
-SWServer::SWServer(SWServerDelegate& delegate, UniqueRef<SWOriginStore>&& originStore, bool processTerminationDelayEnabled, String&& registrationDatabaseDirectory, PAL::SessionID sessionID, bool shouldRunServiceWorkersOnMainThreadForTesting, bool hasServiceWorkerEntitlement, std::optional<unsigned> overrideServiceWorkerRegistrationCountTestingValue)
+SWServer::SWServer(SWServerDelegate& delegate, UniqueRef<SWOriginStore>&& originStore, bool processTerminationDelayEnabled, String&& registrationDatabaseDirectory, PAL::SessionID sessionID, bool shouldRunServiceWorkersOnMainThreadForTesting, bool hasServiceWorkerEntitlement, std::optional<unsigned> overrideServiceWorkerRegistrationCountTestingValue, ServiceWorkerIsInspectable inspectable)
     : m_delegate(delegate)
     , m_originStore(WTFMove(originStore))
     , m_sessionID(sessionID)
@@ -396,6 +386,7 @@ SWServer::SWServer(SWServerDelegate& delegate, UniqueRef<SWOriginStore>&& origin
     , m_shouldRunServiceWorkersOnMainThreadForTesting(shouldRunServiceWorkersOnMainThreadForTesting)
     , m_hasServiceWorkerEntitlement(hasServiceWorkerEntitlement)
     , m_overrideServiceWorkerRegistrationCountTestingValue(overrideServiceWorkerRegistrationCountTestingValue)
+    , m_isInspectable(inspectable)
 {
     RELEASE_LOG_IF(registrationDatabaseDirectory.isEmpty(), ServiceWorker, "No path to store the service worker registrations");
 
@@ -421,7 +412,6 @@ SWServer::SWServer(SWServerDelegate& delegate, UniqueRef<SWOriginStore>&& origin
         registrationStoreImportComplete();
 
     UNUSED_PARAM(registrationDatabaseDirectory);
-    allServers().add(this);
 }
 
 unsigned SWServer::maxRegistrationCount()
@@ -451,7 +441,7 @@ void SWServer::validateRegistrationDomain(WebCore::RegistrableDomain domain, Ser
         return;
     }
 
-    m_delegate->appBoundDomains([this, weakThis = WeakPtr { *this }, domain = WTFMove(domain), jobTypeAllowed, completionHandler = WTFMove(completionHandler)](auto&& appBoundDomains) mutable {
+    m_delegate->appBoundDomains([this, weakThis = WeakPtr { *this }, domain = WTFMove(domain), jobTypeAllowed, completionHandler = WTFMove(completionHandler)](HashSet<RegistrableDomain>&& appBoundDomains) mutable {
         if (!weakThis)
             return;
         m_hasReceivedAppBoundDomains = true;
@@ -489,7 +479,7 @@ void SWServer::scheduleJob(ServiceWorkerJobData&& jobData)
             if (jobQueue.size() == 1)
                 jobQueue.runNextJob();
         } else
-            rejectJob(jobData, { TypeError, "Job rejected for non app-bound domain"_s });
+            rejectJob(jobData, { ExceptionCode::TypeError, "Job rejected for non app-bound domain"_s });
     });
 }
 
@@ -586,7 +576,7 @@ void SWServer::startScriptFetch(const ServiceWorkerJobData& jobData, SWServerReg
         // This is a soft-update job, create directly a network load to fetch the script.
         auto request = createScriptRequest(jobData.scriptURL, jobData, registration);
         request.setHTTPHeaderField(HTTPHeaderName::ServiceWorker, "script"_s);
-        m_delegate->softUpdate(ServiceWorkerJobData { jobData }, shouldRefreshCache, WTFMove(request), [weakThis = WeakPtr { *this }, jobDataIdentifier = jobData.identifier(), registrationKey = jobData.registrationKey()](auto&& result) {
+        m_delegate->softUpdate(ServiceWorkerJobData { jobData }, shouldRefreshCache, WTFMove(request), [weakThis = WeakPtr { *this }, jobDataIdentifier = jobData.identifier(), registrationKey = jobData.registrationKey()](WorkerFetchResult&& result) {
             std::optional<ProcessIdentifier> requestingProcessIdentifier;
             if (weakThis)
                 weakThis->scriptFetchFinished(jobDataIdentifier, registrationKey, requestingProcessIdentifier, WTFMove(result));
@@ -644,7 +634,7 @@ void SWServer::refreshImportedScripts(const ServiceWorkerJobData& jobData, SWSer
     bool shouldRefreshCache = registration.updateViaCache() == ServiceWorkerUpdateViaCache::None || (registration.getNewestWorker() && registration.isStale());
     auto handler = RefreshImportedScriptsHandler::create(urls.size(), WTFMove(callback));
     for (auto& url : urls) {
-        m_delegate->softUpdate(ServiceWorkerJobData { jobData }, shouldRefreshCache, createScriptRequest(url, jobData, registration), [handler, url, size = urls.size()](auto&& result) {
+        m_delegate->softUpdate(ServiceWorkerJobData { jobData }, shouldRefreshCache, createScriptRequest(url, jobData, registration), [handler, url, size = urls.size()](WorkerFetchResult&& result) {
             handler->add(url, WTFMove(result));
         });
     }
@@ -767,7 +757,7 @@ std::optional<ExceptionData> SWServer::claim(SWServerWorker& worker)
 {
     auto* registration = worker.registration();
     if (!registration || &worker != registration->activeWorker())
-        return ExceptionData { InvalidStateError, "Service worker is not active"_s };
+        return ExceptionData { ExceptionCode::InvalidStateError, "Service worker is not active"_s };
 
     auto& origin = worker.origin();
     forEachClientForOrigin(origin, [&](auto& clientData) {
@@ -867,6 +857,8 @@ void SWServer::contextConnectionCreated(SWServerToContextConnection& contextConn
     for (auto& connection : m_connections.values())
         connection->contextConnectionCreated(contextConnection);
 
+    contextConnection.setInspectable(m_isInspectable);
+
     auto pendingContextDatas = m_pendingContextDatas.take(contextConnection.registrableDomain());
     for (auto& data : pendingContextDatas) {
         m_delegate->addAllowedFirstPartyForCookies(contextConnection.webProcessIdentifier(), requestingProcessIdentifier, data.registration.key.firstPartyForCookies());
@@ -1477,12 +1469,12 @@ void SWServer::softUpdate(SWServerRegistration& registration)
     scheduleJob(WTFMove(jobData));
 }
 
-void SWServer::processPushMessage(std::optional<Vector<uint8_t>>&& data, URL&& registrationURL, CompletionHandler<void(bool)>&& callback)
+void SWServer::processPushMessage(std::optional<Vector<uint8_t>>&& data, std::optional<NotificationPayload>&& notificationPayload, URL&& registrationURL, CompletionHandler<void(bool, std::optional<NotificationPayload>&&)>&& callback)
 {
-    whenImportIsCompletedIfNeeded([this, weakThis = WeakPtr { *this }, data = WTFMove(data), registrationURL = WTFMove(registrationURL), callback = WTFMove(callback)]() mutable {
+    whenImportIsCompletedIfNeeded([this, weakThis = WeakPtr { *this }, data = WTFMove(data), notificationPayload = WTFMove(notificationPayload), registrationURL = WTFMove(registrationURL), callback = WTFMove(callback)]() mutable {
         LOG(Push, "ServiceWorker import is complete, can handle push message now");
         if (!weakThis) {
-            callback(false);
+            callback(false, WTFMove(notificationPayload));
             return;
         }
 
@@ -1491,22 +1483,21 @@ void SWServer::processPushMessage(std::optional<Vector<uint8_t>>&& data, URL&& r
         auto registration = m_scopeToRegistrationMap.get(registrationKey);
         if (!registration) {
             RELEASE_LOG_ERROR(Push, "Cannot process push message: Failed to find SW registration for scope %" SENSITIVE_LOG_STRING, registrationKey.scope().string().utf8().data());
-            callback(true);
+            callback(true, WTFMove(notificationPayload));
             return;
         }
 
         auto* worker = registration->activeWorker();
         if (!worker) {
             RELEASE_LOG_ERROR(Push, "Cannot process push message: No active worker for scope %" SENSITIVE_LOG_STRING, registrationKey.scope().string().utf8().data());
-            callback(true);
+            callback(true, WTFMove(notificationPayload));
             return;
         }
 
-        RELEASE_LOG(Push, "Firing Push event");
-
-        fireFunctionalEvent(*registration, [worker = Ref { *worker }, weakThis = WTFMove(weakThis), data = WTFMove(data), callback = WTFMove(callback)](auto&& connectionOrStatus) mutable {
+        RELEASE_LOG(Push, "Firing push event");
+        fireFunctionalEvent(*registration, [worker = Ref { *worker }, weakThis = WTFMove(weakThis), data = WTFMove(data), notificationPayload = WTFMove(notificationPayload), callback = WTFMove(callback)](auto&& connectionOrStatus) mutable {
             if (!connectionOrStatus.has_value()) {
-                callback(connectionOrStatus.error() == ShouldSkipEvent::Yes);
+                callback(connectionOrStatus.error() == ShouldSkipEvent::Yes, WTFMove(notificationPayload));
                 return;
             }
 
@@ -1514,11 +1505,11 @@ void SWServer::processPushMessage(std::optional<Vector<uint8_t>>&& data, URL&& r
 
             worker->incrementFunctionalEventCounter();
             auto terminateWorkerTimer = makeUnique<Timer>([worker] {
-                RELEASE_LOG_ERROR(ServiceWorker, "Service worker is taking too much time to process a push event");
+                RELEASE_LOG_ERROR(ServiceWorker, "Service worker is taking too much time to process a `push` event");
                 worker->decrementFunctionalEventCounter();
             });
             terminateWorkerTimer->startOneShot(weakThis && weakThis->m_isProcessTerminationDelayEnabled ? defaultTerminationDelay : defaultFunctionalEventDuration);
-            connectionOrStatus.value()->firePushEvent(serviceWorkerIdentifier, data, [callback = WTFMove(callback), terminateWorkerTimer = WTFMove(terminateWorkerTimer), worker = WTFMove(worker)](bool succeeded) mutable {
+            connectionOrStatus.value()->firePushEvent(serviceWorkerIdentifier, data, WTFMove(notificationPayload), [callback = WTFMove(callback), terminateWorkerTimer = WTFMove(terminateWorkerTimer), worker = WTFMove(worker)](bool succeeded, std::optional<NotificationPayload>&& resultPayload) mutable {
                 if (!succeeded)
                     RELEASE_LOG(Push, "Push event was not successfully handled");
                 if (terminateWorkerTimer->isActive()) {
@@ -1526,7 +1517,7 @@ void SWServer::processPushMessage(std::optional<Vector<uint8_t>>&& data, URL&& r
                     terminateWorkerTimer->stop();
                 }
 
-                callback(succeeded);
+                callback(succeeded, WTFMove(resultPayload));
             });
         });
     });
@@ -1713,27 +1704,38 @@ Vector<ServiceWorkerClientPendingMessage> SWServer::releaseServiceWorkerClientPe
     return m_clientPendingMessagesById.take(contextIdentifier);
 }
 
+void SWServer::setInspectable(ServiceWorkerIsInspectable inspectable)
+{
+    if (m_isInspectable == inspectable)
+        return;
+
+    m_isInspectable = inspectable;
+
+    for (auto* connection : m_contextConnections.values())
+        connection->setInspectable(inspectable);
+}
+
 void SWServer::Connection::startBackgroundFetch(ServiceWorkerRegistrationIdentifier registrationIdentifier, const String& backgroundFetchIdentifier, Vector<BackgroundFetchRequest>&& requests, BackgroundFetchOptions&& options, ExceptionOrBackgroundFetchInformationCallback&& callback)
 {
     auto* registration = server().getRegistration(registrationIdentifier);
     if (!registration) {
-        callback(makeUnexpected(ExceptionData { InvalidStateError, "No registration found"_s }));
+        callback(makeUnexpected(ExceptionData { ExceptionCode::InvalidStateError, "No registration found"_s }));
         return;
     }
 
     server().requestBackgroundFetchPermission({ registration->key().topOrigin(), SecurityOriginData::fromURL(registration->key().scope()) }, [server = WeakPtr { server() }, registrationIdentifier, backgroundFetchIdentifier, requests = WTFMove(requests), options = WTFMove(options), callback = WTFMove(callback)](bool result) mutable {
         if (!server || !result) {
-            callback(makeUnexpected(ExceptionData { NotAllowedError, "Background fetch permission is denied"_s }));
+            callback(makeUnexpected(ExceptionData { ExceptionCode::NotAllowedError, "Background fetch permission is denied"_s }));
             return;
         }
 
         auto* registration = server->getRegistration(registrationIdentifier);
         if (!registration) {
-            callback(makeUnexpected(ExceptionData { InvalidStateError, "No registration found"_s }));
+            callback(makeUnexpected(ExceptionData { ExceptionCode::InvalidStateError, "No registration found"_s }));
             return;
         }
         if (!registration->activeWorker()) {
-            callback(makeUnexpected(ExceptionData { TypeError, "No active worker"_s }));
+            callback(makeUnexpected(ExceptionData { ExceptionCode::TypeError, "No active worker"_s }));
             return;
         }
 
@@ -1752,7 +1754,7 @@ void SWServer::Connection::backgroundFetchInformation(ServiceWorkerRegistrationI
 {
     auto* registration = server().getRegistration(registrationIdentifier);
     if (!registration) {
-        callback(makeUnexpected(ExceptionData { InvalidStateError, "No registration found"_s }));
+        callback(makeUnexpected(ExceptionData { ExceptionCode::InvalidStateError, "No registration found"_s }));
         return;
     }
 
@@ -1803,5 +1805,3 @@ void SWServer::Connection::retrieveRecordResponseBody(BackgroundFetchRecordIdent
 }
 
 } // namespace WebCore
-
-#endif // ENABLE(SERVICE_WORKER)
