@@ -25,15 +25,16 @@
 
 #pragma once
 
-#if ENABLE(SERVICE_WORKER)
-
+#include "NavigationPreloadState.h"
 #include "SWServer.h"
-#include "ServiceWorkerClientIdentifier.h"
+#include "ScriptExecutionContextIdentifier.h"
 #include "ServiceWorkerRegistrationData.h"
 #include "ServiceWorkerTypes.h"
+#include "Timer.h"
 #include <wtf/HashCountedSet.h>
 #include <wtf/MonotonicTime.h>
 #include <wtf/WallTime.h>
+#include <wtf/WeakPtr.h>
 
 namespace WebCore {
 
@@ -43,12 +44,13 @@ enum class ServiceWorkerRegistrationState : uint8_t;
 enum class ServiceWorkerState : uint8_t;
 struct ExceptionData;
 struct ServiceWorkerContextData;
-struct ServiceWorkerFetchResult;
 
-class SWServerRegistration {
+enum class IsAppInitiated : bool { No, Yes };
+
+class SWServerRegistration : public CanMakeWeakPtr<SWServerRegistration> {
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    SWServerRegistration(SWServer&, const ServiceWorkerRegistrationKey&, ServiceWorkerUpdateViaCache, const URL& scopeURL, const URL& scriptURL);
+    SWServerRegistration(SWServer&, const ServiceWorkerRegistrationKey&, ServiceWorkerUpdateViaCache, const URL& scopeURL, const URL& scriptURL, std::optional<ScriptExecutionContextIdentifier> serviceWorkerPageIdentifier, NavigationPreloadState&&);
     ~SWServerRegistration();
 
     const ServiceWorkerRegistrationKey& key() const { return m_registrationKey; }
@@ -57,11 +59,9 @@ public:
     SWServerWorker* getNewestWorker();
     WEBCORE_EXPORT ServiceWorkerRegistrationData data() const;
 
-    bool isUninstalling() const { return m_uninstalling; }
-    void setIsUninstalling(bool);
-
     void setLastUpdateTime(WallTime);
     WallTime lastUpdateTime() const { return m_lastUpdateTime; }
+    bool isStale() const { return m_lastUpdateTime && (WallTime::now() - m_lastUpdateTime) > 86400_s; }
 
     void setUpdateViaCache(ServiceWorkerUpdateViaCache);
     ServiceWorkerUpdateViaCache updateViaCache() const { return m_updateViaCache; }
@@ -82,31 +82,49 @@ public:
     MonotonicTime creationTime() const { return m_creationTime; }
 
     bool hasClientsUsingRegistration() const { return !m_clientsUsingRegistration.isEmpty(); }
-    void addClientUsingRegistration(const ServiceWorkerClientIdentifier&);
-    void removeClientUsingRegistration(const ServiceWorkerClientIdentifier&);
+    void addClientUsingRegistration(const ScriptExecutionContextIdentifier&);
+    void removeClientUsingRegistration(const ScriptExecutionContextIdentifier&);
     void unregisterServerConnection(SWServerConnectionIdentifier);
 
     void notifyClientsOfControllerChange();
-    void controlClient(ServiceWorkerClientIdentifier);
+    void controlClient(ScriptExecutionContextIdentifier);
 
     void clear();
     bool tryClear();
     void tryActivate();
     void didFinishActivation(ServiceWorkerIdentifier);
+    
+    bool isUnregistered() const;
 
-    void forEachConnection(const WTF::Function<void(SWServer::Connection&)>&);
+    void forEachConnection(const Function<void(SWServer::Connection&)>&);
+
+    WEBCORE_EXPORT bool shouldSoftUpdate(const FetchOptions&) const;
+    WEBCORE_EXPORT void scheduleSoftUpdate(IsAppInitiated);
+    static constexpr Seconds softUpdateDelay { 1_s };
+
+    URL scopeURLWithoutFragment() const { return m_scopeURL; }
+    URL scriptURL() const { return m_scriptURL; }
+
+    bool isAppInitiated() const { return m_isAppInitiated; }
+    std::optional<ScriptExecutionContextIdentifier> serviceWorkerPageIdentifier() const { return m_serviceWorkerPageIdentifier; }
+
+    WEBCORE_EXPORT std::optional<ExceptionData> enableNavigationPreload();
+    WEBCORE_EXPORT std::optional<ExceptionData> disableNavigationPreload();
+    WEBCORE_EXPORT std::optional<ExceptionData> setNavigationPreloadHeaderValue(String&&);
+    const NavigationPreloadState& navigationPreloadState() const { return m_preloadState; }
 
 private:
     void activate();
     void handleClientUnload();
+    void softUpdate();
 
     ServiceWorkerRegistrationIdentifier m_identifier;
     ServiceWorkerRegistrationKey m_registrationKey;
     ServiceWorkerUpdateViaCache m_updateViaCache;
     URL m_scopeURL;
     URL m_scriptURL;
+    std::optional<ScriptExecutionContextIdentifier> m_serviceWorkerPageIdentifier;
 
-    bool m_uninstalling { false };
     RefPtr<SWServerWorker> m_preInstallationWorker; // Implementation detail, not part of the specification.
     RefPtr<SWServerWorker> m_installingWorker;
     RefPtr<SWServerWorker> m_waitingWorker;
@@ -118,9 +136,12 @@ private:
     SWServer& m_server;
 
     MonotonicTime m_creationTime;
-    HashMap<SWServerConnectionIdentifier, HashSet<DocumentIdentifier>> m_clientsUsingRegistration;
+    HashMap<SWServerConnectionIdentifier, HashSet<ScriptExecutionContextIdentifier>> m_clientsUsingRegistration;
+
+    WebCore::Timer m_softUpdateTimer;
+    
+    bool m_isAppInitiated { true };
+    NavigationPreloadState m_preloadState;
 };
 
 } // namespace WebCore
-
-#endif // ENABLE(SERVICE_WORKER)
