@@ -37,8 +37,6 @@
 #include <JavaScriptCore/RuntimeFlags.h>
 #include <wtf/CompletionHandler.h>
 #include <wtf/Deque.h>
-#include <wtf/ObjectIdentifier.h>
-#include <wtf/Optional.h>
 #include <wtf/URL.h>
 #include <wtf/WeakPtr.h>
 
@@ -50,7 +48,7 @@ class WorkerScriptLoader;
 
 struct WorkletParameters;
 
-enum WorkletGlobalScopeIdentifierType { };
+enum class WorkletGlobalScopeIdentifierType { };
 using WorkletGlobalScopeIdentifier = ObjectIdentifier<WorkletGlobalScopeIdentifierType>;
 
 class WorkletGlobalScope : public WorkerOrWorkletGlobalScope {
@@ -73,8 +71,6 @@ public:
 
     void evaluate();
 
-    ReferrerPolicy referrerPolicy() const final;
-
     void addConsoleMessage(std::unique_ptr<Inspector::ConsoleMessage>&&) final;
 
     SecurityOrigin& topOrigin() const final { return m_topOrigin.get(); }
@@ -87,7 +83,7 @@ public:
 
     void prepareForDestruction() override;
 
-    void fetchAndInvokeScript(const URL&, FetchRequestCredentials, CompletionHandler<void(Optional<Exception>&&)>&&);
+    void fetchAndInvokeScript(const URL&, FetchRequestCredentials, CompletionHandler<void(std::optional<Exception>&&)>&&);
 
     Document* responsibleDocument() { return m_document.get(); }
     const Document* responsibleDocument() const { return m_document.get(); }
@@ -97,9 +93,7 @@ protected:
     WorkletGlobalScope(Document&, Ref<JSC::VM>&&, ScriptSourceCode&&);
 
 private:
-#if ENABLE(INDEXED_DATABASE)
     IDBClient::IDBConnectionProxy* idbConnectionProxy() final { ASSERT_NOT_REACHED(); return nullptr; }
-#endif
 
     // EventTarget.
     EventTargetInterface eventTargetInterface() const final { return WorkletGlobalScopeEventTargetInterfaceType; }
@@ -120,13 +114,13 @@ private:
     String userAgent(const URL&) const final;
     const Settings::Values& settingsValues() const final { return m_settingsValues; }
 
-    WeakPtr<Document> m_document;
+    WeakPtr<Document, WeakPtrImplWithEventTargetData> m_document;
 
     Ref<SecurityOrigin> m_topOrigin;
 
     URL m_url;
     JSC::RuntimeFlags m_jsRuntimeFlags;
-    Optional<ScriptSourceCode> m_code;
+    std::optional<ScriptSourceCode> m_code;
 
     std::unique_ptr<WorkerMessagePortChannelProvider> m_messagePortChannelProvider;
 
