@@ -19,7 +19,6 @@
 #pragma once
 
 #include "ExceptionOr.h"
-#include "ParseFromStringOptions.h"
 #include <wtf/WeakPtr.h>
 
 namespace WebCore {
@@ -33,7 +32,7 @@ public:
     static Ref<DOMParser> create(Document& contextDocument);
     ~DOMParser();
 
-    ExceptionOr<Ref<Document>> parseFromString(const String&, const String& contentType, ParseFromStringOptions);
+    ExceptionOr<Ref<Document>> parseFromString(const String&, const String& contentType);
 
 private:
     explicit DOMParser(Document& contextDocument);
