@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000 Peter Kelly <pmk@post.com>
- * Copyright (C) 2005-2017 Apple Inc. All rights reserved.
+ * Copyright (C) 2005-2021 Apple Inc. All rights reserved.
  * Copyright (C) 2006 Alexey Proskuryakov <ap@webkit.org>
  * Copyright (C) 2007 Samuel Weinig <sam@webkit.org>
  * Copyright (C) 2008 Nokia Corporation and/or its subsidiary(-ies)
@@ -34,21 +34,21 @@
 #include "CommonAtomStrings.h"
 #include "CustomElementReactionQueue.h"
 #include "CustomElementRegistry.h"
-#include "DOMWindow.h"
 #include "Document.h"
 #include "DocumentFragment.h"
 #include "DocumentType.h"
 #include "EventLoop.h"
-#include "Frame.h"
 #include "FrameDestructionObserverInlines.h"
 #include "FrameLoader.h"
 #include "HTMLEntityParser.h"
 #include "HTMLHtmlElement.h"
-#include "HTMLParserIdioms.h"
 #include "HTMLTemplateElement.h"
 #include "HTTPParsers.h"
 #include "InlineClassicScript.h"
+#include "LocalDOMWindow.h"
+#include "LocalFrame.h"
 #include "MIMETypeRegistry.h"
+#include "OriginAccessPatterns.h"
 #include "Page.h"
 #include "PageConsoleClient.h"
 #include "PendingScript.h"
@@ -66,6 +66,7 @@
 #include "TransformSource.h"
 #include "XMLNSNames.h"
 #include "XMLDocumentParserScope.h"
+#include <libxml/parser.h>
 #include <libxml/parserInternals.h>
 #include <wtf/unicode/CharacterNames.h>
 #include <wtf/unicode/UTF8Conversion.h>
@@ -448,7 +449,9 @@ static bool shouldAllowExternalLoad(const URL& url)
     // retrieved content.  If we had more context, we could potentially allow
     // the parser to load a DTD.  As things stand, we take the conservative
     // route and allow same-origin requests only.
-    if (!XMLDocumentParserScope::currentCachedResourceLoader->document()->securityOrigin().canRequest(url)) {
+    if (!XMLDocumentParserScope::currentCachedResourceLoader || !XMLDocumentParserScope::currentCachedResourceLoader->document())
+        return false;
+    if (!XMLDocumentParserScope::currentCachedResourceLoader->document()->securityOrigin().canRequest(url, OriginAccessPatternsForWebProcess::singleton())) {
         XMLDocumentParserScope::currentCachedResourceLoader->printAccessDeniedMessage(url);
         return false;
     }
@@ -463,9 +466,9 @@ static void* openFunc(const char* uri)
 
     CachedResourceLoader& cachedResourceLoader = *XMLDocumentParserScope::currentCachedResourceLoader;
     Document* document = cachedResourceLoader.document();
-    // Same logic as HTMLBaseElement::href(). Keep them in sync.
+    // Same logic as Document::completeURL(). Keep them in sync.
     auto* encoding = (document && document->decoder()) ? document->decoder()->encodingForURLParsing() : nullptr;
-    URL url(document ? document->fallbackBaseURL() : URL(), stripLeadingAndTrailingHTMLSpaces(String::fromLatin1(uri)), encoding);
+    URL url(document ? document->fallbackBaseURL() : URL(), String::fromLatin1(uri), encoding);
 
     if (!shouldAllowExternalLoad(url))
         return &globalDescriptor;
@@ -535,6 +538,15 @@ static void errorFunc(void*, const char*, ...)
 }
 #endif
 
+static xmlExternalEntityLoader defaultEntityLoader { nullptr };
+
+static xmlParserInputPtr entityLoader(const char* url, const char* id, xmlParserCtxtPtr context)
+{
+    if (!shouldAllowExternalLoad(URL(String::fromUTF8(url))))
+        return nullptr;
+    return defaultEntityLoader(url, id, context);
+}
+
 static void initializeXMLParser()
 {
     static std::once_flag flag;
@@ -542,6 +554,8 @@ static void initializeXMLParser()
         xmlInitParser();
         xmlRegisterInputCallbacks(matchFunc, openFunc, readFunc, closeFunc);
         xmlRegisterOutputCallbacks(matchFunc, openFunc, writeFunc, closeFunc);
+        defaultEntityLoader = xmlGetExternalEntityLoader();
+        xmlSetExternalEntityLoader(entityLoader);
         libxmlLoaderThread = &Thread::current();
     });
 }
@@ -598,7 +612,7 @@ bool XMLDocumentParser::supportsXMLVersion(const String& version)
     return version == "1.0"_s;
 }
 
-XMLDocumentParser::XMLDocumentParser(Document& document, FrameView* frameView, OptionSet<ParserContentPolicy> policy)
+XMLDocumentParser::XMLDocumentParser(Document& document, LocalFrameView* frameView, OptionSet<ParserContentPolicy> policy)
     : ScriptableDocumentParser(document, policy)
     , m_view(frameView)
     , m_pendingCallbacks(makeUnique<PendingCallbacks>())
@@ -744,19 +758,6 @@ static inline bool handleElementAttributes(Vector<Attribute>& prefixedAttributes
     return true;
 }
 
-// This is a hack around https://bugzilla.gnome.org/show_bug.cgi?id=502960
-// Otherwise libxml doesn't include namespace for parsed entities, breaking entity
-// expansion for all entities containing elements.
-static inline bool hackAroundLibXMLEntityParsingBug()
-{
-#if LIBXML_VERSION >= 20704
-    // This bug has been fixed in libxml 2.7.4.
-    return false;
-#else
-    return true;
-#endif
-}
-
 void XMLDocumentParser::startElementNs(const xmlChar* xmlLocalName, const xmlChar* xmlPrefix, const xmlChar* xmlURI, int numNamespaces, const xmlChar** libxmlNamespaces, int numAttributes, int numDefaulted, const xmlChar** libxmlAttributes)
 {
     if (isStopped())
@@ -783,11 +784,6 @@ void XMLDocumentParser::startElementNs(const xmlChar* xmlLocalName, const xmlCha
             uri = m_defaultNamespaceURI;
     }
 
-    // If libxml entity parsing is broken, transfer the currentNodes' namespaceURI to the new node,
-    // if we're currently expanding elements which originate from an entity declaration.
-    if (hackAroundLibXMLEntityParsingBug() && depthTriggeringEntityExpansion() != -1 && context()->depth > depthTriggeringEntityExpansion() && uri.isNull() && prefix.isNull())
-        uri = m_currentNode->namespaceURI();
-
     bool isFirstElement = !m_sawFirstElement;
     m_sawFirstElement = true;
 
@@ -872,10 +868,6 @@ void XMLDocumentParser::endElementNs()
     RefPtr<ContainerNode> node = m_currentNode;
     node->finishParsingChildren();
 
-    // Once we reach the depth again where entity expansion started, stop executing the work-around.
-    if (hackAroundLibXMLEntityParsingBug() && context()->depth <= depthTriggeringEntityExpansion())
-        setDepthTriggeringEntityExpansion(-1);
-
     if (!scriptingContentIsAllowed(parserContentPolicy()) && is<Element>(*node) && isScriptElement(downcast<Element>(*node))) {
         popCurrentNode();
         node->remove();
@@ -906,15 +898,15 @@ void XMLDocumentParser::endElementNs()
     m_requestingScript = true;
 
     auto& scriptElement = downcastScriptElement(element);
-    if (scriptElement.prepareScript(m_scriptStartPosition, ScriptElement::AllowLegacyTypeInTypeAttribute)) {
+    if (scriptElement.prepareScript(m_scriptStartPosition)) {
         // FIXME: Script execution should be shared between
         // the libxml2 and Qt XMLDocumentParser implementations.
 
         if (scriptElement.readyToBeParserExecuted()) {
             if (scriptElement.scriptType() == ScriptType::Classic)
-                scriptElement.executeClassicScript(ScriptSourceCode(scriptElement.scriptContent(), URL(document()->url()), m_scriptStartPosition, JSC::SourceProviderSourceType::Program, InlineClassicScript::create(scriptElement)));
+                scriptElement.executeClassicScript(ScriptSourceCode(scriptElement.scriptContent(), scriptElement.sourceTaintedOrigin(), URL(document()->url()), m_scriptStartPosition, JSC::SourceProviderSourceType::Program, InlineClassicScript::create(scriptElement)));
             else
-                scriptElement.registerImportMap(ScriptSourceCode(scriptElement.scriptContent(), URL(document()->url()), m_scriptStartPosition, JSC::SourceProviderSourceType::ImportMap));
+                scriptElement.registerImportMap(ScriptSourceCode(scriptElement.scriptContent(), scriptElement.sourceTaintedOrigin(), URL(document()->url()), m_scriptStartPosition, JSC::SourceProviderSourceType::ImportMap));
         } else if (scriptElement.willBeParserExecuted() && scriptElement.loadableScript()) {
             m_pendingScript = PendingScript::create(scriptElement, *scriptElement.loadableScript());
             m_pendingScript->setClient(*this);
@@ -1082,64 +1074,33 @@ static inline XMLDocumentParser* getParser(void* closure)
     return static_cast<XMLDocumentParser*>(ctxt->_private);
 }
 
-// This is a hack around http://bugzilla.gnome.org/show_bug.cgi?id=159219
-// Otherwise libxml seems to call all the SAX callbacks twice for any replaced entity.
-static inline bool hackAroundLibXMLEntityBug(void* closure)
-{
-#if LIBXML_VERSION >= 20627
-    // This bug has been fixed in libxml 2.6.27.
-    UNUSED_PARAM(closure);
-    return false;
-#else
-    return static_cast<xmlParserCtxtPtr>(closure)->node;
-#endif
-}
-
 static void startElementNsHandler(void* closure, const xmlChar* localname, const xmlChar* prefix, const xmlChar* uri, int numNamespaces, const xmlChar** namespaces, int numAttributes, int numDefaulted, const xmlChar** libxmlAttributes)
 {
-    if (hackAroundLibXMLEntityBug(closure))
-        return;
-
     getParser(closure)->startElementNs(localname, prefix, uri, numNamespaces, namespaces, numAttributes, numDefaulted, libxmlAttributes);
 }
 
 static void endElementNsHandler(void* closure, const xmlChar*, const xmlChar*, const xmlChar*)
 {
-    if (hackAroundLibXMLEntityBug(closure))
-        return;
-
     getParser(closure)->endElementNs();
 }
 
 static void charactersHandler(void* closure, const xmlChar* s, int len)
 {
-    if (hackAroundLibXMLEntityBug(closure))
-        return;
-
     getParser(closure)->characters(s, len);
 }
 
 static void processingInstructionHandler(void* closure, const xmlChar* target, const xmlChar* data)
 {
-    if (hackAroundLibXMLEntityBug(closure))
-        return;
-
     getParser(closure)->processingInstruction(target, data);
 }
 
 static void cdataBlockHandler(void* closure, const xmlChar* s, int len)
 {
-    if (hackAroundLibXMLEntityBug(closure))
-        return;
-
     getParser(closure)->cdataBlock(s, len);
 }
 
 static void commentHandler(void* closure, const xmlChar* comment)
 {
-    if (hackAroundLibXMLEntityBug(closure))
-        return;
-
     getParser(closure)->comment(comment);
 }
 
@@ -1187,10 +1148,12 @@ static xmlEntityPtr sharedXHTMLEntity()
     return &entity;
 }
 
-static size_t convertUTF16EntityToUTF8(const UChar* utf16Entity, size_t numberOfCodeUnits, char* target, size_t targetSize)
+static size_t convertUTF16EntityToUTF8(std::span<const UChar> utf16Entity, char* target, size_t targetSize)
 {
     const char* originalTarget = target;
-    auto conversionResult = WTF::Unicode::convertUTF16ToUTF8(&utf16Entity, utf16Entity + numberOfCodeUnits, &target, target + targetSize);
+    auto start = utf16Entity.data();
+    auto end = start + utf16Entity.size();
+    auto conversionResult = WTF::Unicode::convertUTF16ToUTF8(&start, end, &target, target + targetSize);
     if (conversionResult != WTF::Unicode::ConversionResult::Success)
         return 0;
 
@@ -1202,23 +1165,24 @@ static size_t convertUTF16EntityToUTF8(const UChar* utf16Entity, size_t numberOf
 
 static xmlEntityPtr getXHTMLEntity(const xmlChar* name)
 {
-    UChar utf16DecodedEntity[4];
-    size_t numberOfCodeUnits = decodeNamedEntityToUCharArray(reinterpret_cast<const char*>(name), utf16DecodedEntity);
-    if (!numberOfCodeUnits)
-        return 0;
+    auto decodedEntity = decodeNamedHTMLEntityForXMLParser(reinterpret_cast<const char*>(name));
+    if (decodedEntity.failed())
+        return nullptr;
+
+    auto utf16DecodedEntity = decodedEntity.span();
 
     constexpr size_t kSharedXhtmlEntityResultLength = std::size(sharedXHTMLEntityResult);
     size_t entityLengthInUTF8;
     // Unlike HTML parser, XML parser parses the content of named
     // entities. So we need to escape '&' and '<'.
-    if (numberOfCodeUnits == 1 && utf16DecodedEntity[0] == '&') {
+    if (utf16DecodedEntity.size() == 1 && utf16DecodedEntity[0] == '&') {
         sharedXHTMLEntityResult[0] = '&';
         sharedXHTMLEntityResult[1] = '#';
         sharedXHTMLEntityResult[2] = '3';
         sharedXHTMLEntityResult[3] = '8';
         sharedXHTMLEntityResult[4] = ';';
         entityLengthInUTF8 = 5;
-    } else if (numberOfCodeUnits == 1 && utf16DecodedEntity[0] == '<') {
+    } else if (utf16DecodedEntity.size() == 1 && utf16DecodedEntity[0] == '<') {
         sharedXHTMLEntityResult[0] = '&';
         sharedXHTMLEntityResult[1] = '#';
         sharedXHTMLEntityResult[2] = 'x';
@@ -1226,7 +1190,7 @@ static xmlEntityPtr getXHTMLEntity(const xmlChar* name)
         sharedXHTMLEntityResult[4] = 'C';
         sharedXHTMLEntityResult[5] = ';';
         entityLengthInUTF8 = 6;
-    } else if (numberOfCodeUnits == 2 && utf16DecodedEntity[0] == '<' && utf16DecodedEntity[1] == 0x20D2) {
+    } else if (utf16DecodedEntity.size() == 2 && utf16DecodedEntity[0] == '<' && utf16DecodedEntity[1] == 0x20D2) {
         sharedXHTMLEntityResult[0] = '&';
         sharedXHTMLEntityResult[1] = '#';
         sharedXHTMLEntityResult[2] = '6';
@@ -1237,8 +1201,8 @@ static xmlEntityPtr getXHTMLEntity(const xmlChar* name)
         sharedXHTMLEntityResult[7] = 0x92;
         entityLengthInUTF8 = 8;
     } else {
-        ASSERT(numberOfCodeUnits <= 4);
-        entityLengthInUTF8 = convertUTF16EntityToUTF8(utf16DecodedEntity, numberOfCodeUnits,
+        ASSERT(utf16DecodedEntity.size() <= 4);
+        entityLengthInUTF8 = convertUTF16EntityToUTF8(utf16DecodedEntity,
             reinterpret_cast<char*>(sharedXHTMLEntityResult), kSharedXhtmlEntityResultLength);
         if (!entityLengthInUTF8)
             return 0;
@@ -1251,34 +1215,10 @@ static xmlEntityPtr getXHTMLEntity(const xmlChar* name)
     return entity;
 }
 
-static void entityDeclarationHandler(void* closure, const xmlChar* name, int type, const xmlChar* publicId, const xmlChar* systemId, xmlChar* content)
-{
-    // Prevent the next call to getEntityHandler() to record the entity expansion depth.
-    // We're parsing the entity declaration, so there's no need to record anything.
-    // We only need to record the depth, if we're actually expanding the entity, when it's referenced.
-    if (hackAroundLibXMLEntityParsingBug())
-        getParser(closure)->setIsParsingEntityDeclaration(true);
-    xmlSAX2EntityDecl(closure, name, type, publicId, systemId, content);
-}
-
 static xmlEntityPtr getEntityHandler(void* closure, const xmlChar* name)
 {
     xmlParserCtxtPtr ctxt = static_cast<xmlParserCtxtPtr>(closure);
 
-    XMLDocumentParser* parser = getParser(closure);
-    if (hackAroundLibXMLEntityParsingBug()) {
-        if (parser->isParsingEntityDeclaration()) {
-            // We're parsing the entity declarations (not an entity reference), no need to do anything special.
-            parser->setIsParsingEntityDeclaration(false);
-            ASSERT(parser->depthTriggeringEntityExpansion() == -1);
-        } else {
-            // The entity will be used and eventually expanded. Record the current parser depth
-            // so the next call to startElementNs() knows that the new element originates from
-            // an entity declaration.
-            parser->setDepthTriggeringEntityExpansion(ctxt->depth);
-        }
-    }
-
     xmlEntityPtr ent = xmlGetPredefinedEntity(name);
     if (ent) {
         ent->etype = XML_INTERNAL_PREDEFINED_ENTITY;
@@ -1286,7 +1226,7 @@ static xmlEntityPtr getEntityHandler(void* closure, const xmlChar* name)
     }
 
     ent = xmlGetDocEntity(ctxt->myDoc, name);
-    if (!ent && parser->isXHTMLDocument()) {
+    if (!ent && getParser(closure)->isXHTMLDocument()) {
         ent = getXHTMLEntity(name);
         if (ent)
             ent->etype = XML_INTERNAL_GENERAL_ENTITY;
@@ -1359,7 +1299,7 @@ void XMLDocumentParser::initializeParserContext(const CString& chunk)
     sax.internalSubset = internalSubsetHandler;
     sax.externalSubset = externalSubsetHandler;
     sax.ignorableWhitespace = ignorableWhitespaceHandler;
-    sax.entityDecl = entityDeclarationHandler;
+    sax.entityDecl = xmlSAX2EntityDecl;
     sax.initialized = XML_SAX2_MAGIC;
     DocumentParser::startParsing();
     m_sawError = false;
