@@ -39,16 +39,19 @@ enum class NodeKind : uint8_t {
     BindingAttribute,
     BuiltinAttribute,
     ConstAttribute,
+    DiagnosticAttribute,
     GroupAttribute,
     IdAttribute,
     InterpolateAttribute,
     InvariantAttribute,
     LocationAttribute,
+    MustUseAttribute,
     SizeAttribute,
     StageAttribute,
     WorkgroupSizeAttribute,
 
     Directive,
+    DiagnosticDirective,
 
     // Expression
     BinaryExpression,
@@ -71,6 +74,7 @@ enum class NodeKind : uint8_t {
     AbstractIntegerLiteral,
     BoolLiteral,
     Float32Literal,
+    Float16Literal,
     Signed32Literal,
     Unsigned32Literal,
 
@@ -100,11 +104,9 @@ enum class NodeKind : uint8_t {
 
     TypeAlias,
 
-    // TypeName
-    ArrayTypeName,
-    NamedTypeName,
-    ParameterizedTypeName,
-    ReferenceTypeName,
+    ArrayTypeExpression,
+    ElaboratedTypeExpression,
+    ReferenceTypeExpression,
 
     Variable,
 
