@@ -29,9 +29,10 @@
 #include "ASTBuilder.h"
 #include "ASTExpression.h"
 #include "ASTForward.h"
+#include "ASTFunction.h"
 #include "ASTStatement.h"
 #include "ASTStructure.h"
-#include "ASTTypeName.h"
+#include "ASTTypeAlias.h"
 #include "ASTVariable.h"
 #include "CompilationMessage.h"
 #include "Lexer.h"
@@ -49,27 +50,35 @@ public:
         : m_shaderModule(shaderModule)
         , m_builder(shaderModule.astBuilder())
         , m_lexer(lexer)
-        , m_current(lexer.lex())
+        , m_tokens(m_lexer.lex())
+        , m_current(m_tokens[0])
+        , m_currentPosition({ m_current.span.line, m_current.span.lineOffset, m_current.span.offset })
     {
     }
 
     Result<void> parseShader();
 
+    void maybeSplitToken(unsigned index);
+    void disambiguateTemplates();
+
     // AST::<type>::Ref whenever it can return multiple types.
     Result<AST::Identifier> parseIdentifier();
-    Result<void> parseGlobalDecl();
+    Result<void> parseEnableDirective();
+    Result<void> parseRequireDirective();
+    Result<AST::Declaration::Ref> parseDeclaration();
     Result<AST::Attribute::List> parseAttributes();
     Result<AST::Attribute::Ref> parseAttribute();
     Result<AST::Structure::Ref> parseStructure(AST::Attribute::List&&);
     Result<std::reference_wrapper<AST::StructureMember>> parseStructureMember();
-    Result<AST::TypeName::Ref> parseTypeName();
-    Result<AST::TypeName::Ref> parseTypeNameAfterIdentifier(AST::Identifier&&, SourcePosition start);
-    Result<AST::TypeName::Ref> parseArrayType();
+    Result<AST::Expression::Ref> parseTypeName();
+    Result<AST::Expression::Ref> parseTypeNameAfterIdentifier(AST::Identifier&&, SourcePosition start);
+    Result<AST::Expression::Ref> parseArrayType();
     Result<AST::Variable::Ref> parseVariable();
     Result<AST::Variable::Ref> parseVariableWithAttributes(AST::Attribute::List&&);
     Result<AST::VariableQualifier::Ref> parseVariableQualifier();
-    Result<AST::StorageClass> parseStorageClass();
-    Result<AST::AccessMode> parseAccessMode();
+    Result<AddressSpace> parseAddressSpace();
+    Result<AccessMode> parseAccessMode();
+    Result<AST::TypeAlias::Ref> parseTypeAlias();
     Result<AST::Function::Ref> parseFunction(AST::Attribute::List&&);
     Result<std::reference_wrapper<AST::Parameter>> parseParameter();
     Result<AST::Statement::Ref> parseStatement();
@@ -77,8 +86,12 @@ public:
     Result<AST::Statement::Ref> parseIfStatement();
     Result<AST::Statement::Ref> parseIfStatementWithAttributes(AST::Attribute::List&&, SourcePosition _startOfElementPosition);
     Result<AST::Statement::Ref> parseForStatement();
+    Result<AST::Statement::Ref> parseLoopStatement();
+    Result<AST::Statement::Ref> parseSwitchStatement();
+    Result<AST::Statement::Ref> parseWhileStatement();
     Result<AST::Statement::Ref> parseReturnStatement();
     Result<AST::Statement::Ref> parseVariableUpdatingStatement();
+    Result<AST::Statement::Ref> parseVariableUpdatingStatement(AST::Expression::Ref&&);
     Result<AST::Expression::Ref> parseShortCircuitExpression(AST::Expression::Ref&&, TokenType, AST::BinaryOperation);
     Result<AST::Expression::Ref> parseRelationalExpression();
     Result<AST::Expression::Ref> parseRelationalExpressionPostUnary(AST::Expression::Ref&& lhs);
@@ -95,6 +108,7 @@ public:
     Result<AST::Expression::Ref> parseLHSExpression();
     Result<AST::Expression::Ref> parseCoreLHSExpression();
     Result<AST::Expression::List> parseArgumentExpressionList();
+    Result<AST::Diagnostic> parseDiagnostic();
 
 private:
     Expected<Token, TokenType> consumeType(TokenType);
@@ -107,7 +121,10 @@ private:
     ShaderModule& m_shaderModule;
     AST::Builder& m_builder;
     Lexer& m_lexer;
+    Vector<Token> m_tokens;
+    unsigned m_currentTokenIndex { 0 };
     Token m_current;
+    SourcePosition m_currentPosition;
 };
 
 } // namespace WGSL
