@@ -27,6 +27,7 @@
 #include "Types.h"
 
 #include "ASTStructure.h"
+#include "TypeStore.h"
 #include <wtf/StdLibExtras.h>
 #include <wtf/StringPrintStream.h>
 #include <wtf/text/StringHash.h>
@@ -63,6 +64,18 @@ void Type::dump(PrintStream& out) const
         [&](const Struct& structure) {
             out.print(structure.structure.name());
         },
+        [&](const PrimitiveStruct& structure) {
+            out.print(structure.name, "<");
+            switch (structure.kind) {
+            case PrimitiveStruct::FrexpResult::kind:
+                out.print(*structure.values[PrimitiveStruct::FrexpResult::fract]);
+                break;
+            case PrimitiveStruct::ModfResult::kind:
+                out.print(*structure.values[PrimitiveStruct::ModfResult::fract]);
+                break;
+            }
+            out.print(">");
+        },
         [&](const Function& function) {
             out.print("(");
             bool first = true;
@@ -97,24 +110,57 @@ void Type::dump(PrintStream& out) const
             case Texture::Kind::TextureMultisampled2d:
                 out.print("texture_multisampled_2d");
                 break;
-            case Texture::Kind::TextureStorage1d:
-                out.print("texture_storage_1d");
-                break;
-            case Texture::Kind::TextureStorage2d:
-                out.print("texture_storage_2d");
-                break;
-            case Texture::Kind::TextureStorage2dArray:
-                out.print("texture_storage_2d_array");
-                break;
-            case Texture::Kind::TextureStorage3d:
-                out.print("texture_storage_3d");
-                break;
             }
             out.print("<", *texture.element, ">");
         },
+        [&](const TextureStorage& texture) {
+            switch (texture.kind) {
+            case TextureStorage::Kind::TextureStorage1d:
+                out.print("texture_storage_1d");
+                break;
+            case TextureStorage::Kind::TextureStorage2d:
+                out.print("texture_storage_2d");
+                break;
+            case TextureStorage::Kind::TextureStorage2dArray:
+                out.print("texture_storage_2d_array");
+                break;
+            case TextureStorage::Kind::TextureStorage3d:
+                out.print("texture_storage_3d");
+                break;
+            }
+            out.print("<", texture.format, ", ", texture.access, ">");
+        },
+        [&](const TextureDepth& texture) {
+            switch (texture.kind) {
+            case TextureDepth::Kind::TextureDepth2d:
+                out.print("texture_depth_2d");
+                break;
+            case TextureDepth::Kind::TextureDepth2dArray:
+                out.print("texture_depth_2d_array");
+                break;
+            case TextureDepth::Kind::TextureDepthCube:
+                out.print("texture_depth_cube");
+                break;
+            case TextureDepth::Kind::TextureDepthCubeArray:
+                out.print("texture_depth_cube_array");
+                break;
+            case TextureDepth::Kind::TextureDepthMultisampled2d:
+                out.print("texture_depth_multisampled_2d");
+                break;
+            }
+        },
         [&](const Reference& reference) {
             out.print("ref<", reference.addressSpace, ", ", *reference.element, ", ", reference.accessMode, ">");
         },
+        [&](const Pointer& reference) {
+            out.print("ptr<", reference.addressSpace, ", ", *reference.element, ", ", reference.accessMode, ">");
+        },
+        [&](const Atomic& atomic) {
+            out.print("atomic<", *atomic.element, ">");
+        },
+        [&](const TypeConstructor& constructor) {
+            out.print(constructor.name);
+        },
         [&](const Bottom&) {
             // Bottom is an implementation detail and should never leak, but we
             // keep the ability to print it in debug to help when dumping types
@@ -156,7 +202,8 @@ ConversionRank conversionRank(const Type* from, const Type* to)
         switch (primitivePair(fromPrimitive->kind, toPrimitive->kind)) {
         case primitivePair(Primitive::AbstractFloat, Primitive::F32):
             return { 1 };
-        // FIXME: AbstractFloat to f16 should return 2
+        case primitivePair(Primitive::AbstractFloat, Primitive::F16):
+            return { 2 };
         case primitivePair(Primitive::AbstractInt, Primitive::I32):
             return { 3 };
         case primitivePair(Primitive::AbstractInt, Primitive::U32):
@@ -165,7 +212,8 @@ ConversionRank conversionRank(const Type* from, const Type* to)
             return { 5 };
         case primitivePair(Primitive::AbstractInt, Primitive::F32):
             return { 6 };
-        // FIXME: AbstractInt to f16 should return 7
+        case primitivePair(Primitive::AbstractInt, Primitive::F16):
+            return { 7 };
         default:
             return std::nullopt;
         }
@@ -200,6 +248,21 @@ ConversionRank conversionRank(const Type* from, const Type* to)
         return conversionRank(fromArray->element, toArray->element);
     }
 
+    if (auto* fromPrimitiveStruct = std::get_if<PrimitiveStruct>(from)) {
+        auto* toPrimitiveStruct = std::get_if<PrimitiveStruct>(to);
+        if (!toPrimitiveStruct)
+            return std::nullopt;
+        auto kind = fromPrimitiveStruct->kind;
+        if (kind != toPrimitiveStruct->kind)
+            return std::nullopt;
+        switch (kind) {
+        case PrimitiveStruct::FrexpResult::kind:
+            return conversionRank(fromPrimitiveStruct->values[PrimitiveStruct::FrexpResult::fract], toPrimitiveStruct->values[PrimitiveStruct::FrexpResult::fract]);
+        case PrimitiveStruct::ModfResult::kind:
+            return conversionRank(fromPrimitiveStruct->values[PrimitiveStruct::ModfResult::fract], toPrimitiveStruct->values[PrimitiveStruct::ModfResult::fract]);
+        }
+    }
+
     // FIXME: add the abstract result conversion rules
     return std::nullopt;
 }
@@ -217,16 +280,23 @@ unsigned Type::size() const
     return WTF::switchOn(*this,
         [&](const Primitive& primitive) -> unsigned {
             switch (primitive.kind) {
+            case Types::Primitive::F16:
+                return 2;
             case Types::Primitive::F32:
             case Types::Primitive::I32:
             case Types::Primitive::U32:
                 return 4;
             case Types::Primitive::Bool:
+                return 1;
             case Types::Primitive::Void:
             case Types::Primitive::AbstractInt:
             case Types::Primitive::AbstractFloat:
             case Types::Primitive::Sampler:
+            case Types::Primitive::SamplerComparison:
             case Types::Primitive::TextureExternal:
+            case Types::Primitive::AccessMode:
+            case Types::Primitive::TexelFormat:
+            case Types::Primitive::AddressSpace:
                 RELEASE_ASSERT_NOT_REACHED();
             }
         },
@@ -242,19 +312,13 @@ unsigned Type::size() const
             return matrix.columns * WTF::roundUpToMultipleOf(rowAlignment, rowSize);
         },
         [&](const Array& array) -> unsigned {
-            ASSERT(array.size.has_value());
-            return *array.size * WTF::roundUpToMultipleOf(array.element->alignment(), array.element->size());
+            return array.size.value_or(1) * WTF::roundUpToMultipleOf(array.element->alignment(), array.element->size());
         },
         [&](const Struct& structure) -> unsigned {
-            unsigned alignment = 0;
-            unsigned size = 0;
-            for (auto& [_, field] : structure.fields) {
-                auto fieldAlignment = field->alignment();
-                alignment = std::max(alignment, fieldAlignment);
-                size = WTF::roundUpToMultipleOf(fieldAlignment, size);
-                size += field->size();
-            }
-            return WTF::roundUpToMultipleOf(alignment, size);
+            return structure.structure.size();
+        },
+        [&](const PrimitiveStruct&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
         },
         [&](const Function&) -> unsigned {
             RELEASE_ASSERT_NOT_REACHED();
@@ -262,9 +326,25 @@ unsigned Type::size() const
         [&](const Texture&) -> unsigned {
             RELEASE_ASSERT_NOT_REACHED();
         },
+        [&](const TextureStorage&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
+        },
+        [&](const TextureDepth&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
+        },
         [&](const Reference&) -> unsigned {
             RELEASE_ASSERT_NOT_REACHED();
         },
+        [&](const Pointer&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
+        },
+        [&](const Atomic& a) -> unsigned {
+            RELEASE_ASSERT(a.element);
+            return a.element->size();
+        },
+        [&](const TypeConstructor&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
+        },
         [&](const Bottom&) -> unsigned {
             RELEASE_ASSERT_NOT_REACHED();
         });
@@ -275,16 +355,23 @@ unsigned Type::alignment() const
     return WTF::switchOn(*this,
         [&](const Primitive& primitive) -> unsigned {
             switch (primitive.kind) {
+            case Types::Primitive::F16:
+                return 2;
             case Types::Primitive::F32:
             case Types::Primitive::I32:
             case Types::Primitive::U32:
                 return 4;
             case Types::Primitive::Bool:
+                return 1;
             case Types::Primitive::Void:
             case Types::Primitive::AbstractInt:
             case Types::Primitive::AbstractFloat:
             case Types::Primitive::Sampler:
+            case Types::Primitive::SamplerComparison:
             case Types::Primitive::TextureExternal:
+            case Types::Primitive::AccessMode:
+            case Types::Primitive::TexelFormat:
+            case Types::Primitive::AddressSpace:
                 RELEASE_ASSERT_NOT_REACHED();
             }
         },
@@ -304,10 +391,10 @@ unsigned Type::alignment() const
             return array.element->alignment();
         },
         [&](const Struct& structure) -> unsigned {
-            unsigned alignment = 0;
-            for (auto& [_, field] : structure.fields)
-                alignment = std::max(alignment, field->alignment());
-            return alignment;
+            return structure.structure.alignment();
+        },
+        [&](const PrimitiveStruct&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
         },
         [&](const Function&) -> unsigned {
             RELEASE_ASSERT_NOT_REACHED();
@@ -315,9 +402,25 @@ unsigned Type::alignment() const
         [&](const Texture&) -> unsigned {
             RELEASE_ASSERT_NOT_REACHED();
         },
+        [&](const TextureStorage&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
+        },
+        [&](const TextureDepth&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
+        },
         [&](const Reference&) -> unsigned {
             RELEASE_ASSERT_NOT_REACHED();
         },
+        [&](const Pointer&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
+        },
+        [&](const Atomic& a) -> unsigned {
+            RELEASE_ASSERT(a.element);
+            return a.element->alignment();
+        },
+        [&](const TypeConstructor&) -> unsigned {
+            RELEASE_ASSERT_NOT_REACHED();
+        },
         [&](const Bottom&) -> unsigned {
             RELEASE_ASSERT_NOT_REACHED();
         });
@@ -339,47 +442,30 @@ bool isPrimitiveReference(const Type* type, Primitive::Kind kind)
     return isPrimitive(reference->element, kind);
 }
 
+const Type* shaderTypeForTexelFormat(TexelFormat format, const TypeStore& types)
+{
+    switch (format) {
+    case TexelFormat::BGRA8unorm:
+    case TexelFormat::RGBA8unorm:
+    case TexelFormat::RGBA8snorm:
+    case TexelFormat::RGBA16float:
+    case TexelFormat::R32float:
+    case TexelFormat::RG32float:
+    case TexelFormat::RGBA32float:
+        return types.f32Type();
+    case TexelFormat::RGBA8uint:
+    case TexelFormat::RGBA16uint:
+    case TexelFormat::R32uint:
+    case TexelFormat::RG32uint:
+    case TexelFormat::RGBA32uint:
+        return types.u32Type();
+    case TexelFormat::RGBA8sint:
+    case TexelFormat::RGBA16sint:
+    case TexelFormat::R32sint:
+    case TexelFormat::RG32sint:
+    case TexelFormat::RGBA32sint:
+        return types.i32Type();
+    }
+}
+
 } // namespace WGSL
-
-namespace WTF {
-
-void printInternal(PrintStream& out, WGSL::AddressSpace addressSpace)
-{
-    switch (addressSpace) {
-    case WGSL::AddressSpace::Function:
-        out.print("function");
-        return;
-    case WGSL::AddressSpace::Private:
-        out.print("private");
-        return;
-    case WGSL::AddressSpace::Workgroup:
-        out.print("workgroup");
-        return;
-    case WGSL::AddressSpace::Uniform:
-        out.print("uniform");
-        return;
-    case WGSL::AddressSpace::Storage:
-        out.print("storage");
-        return;
-    case WGSL::AddressSpace::Handle:
-        out.print("handle");
-        return;
-    }
-}
-
-void printInternal(PrintStream& out, WGSL::AccessMode accessMode)
-{
-    switch (accessMode) {
-    case WGSL::AccessMode::Read:
-        out.print("read");
-        return;
-    case WGSL::AccessMode::Write:
-        out.print("write");
-        return;
-    case WGSL::AccessMode::ReadWrite:
-        out.print("read_write");
-        return;
-    }
-}
-
-} // namespace WTF
