@@ -27,10 +27,15 @@
 
 #import "CommandsMixin.h"
 #import <wtf/FastMalloc.h>
+#import <wtf/HashMap.h>
+#import <wtf/HashSet.h>
+#import <wtf/HashTraits.h>
 #import <wtf/Ref.h>
 #import <wtf/RefCounted.h>
 #import <wtf/Vector.h>
 
+@class TextureAndClearColor;
+
 struct WGPURenderPassEncoderImpl {
 };
 
@@ -38,6 +43,7 @@ namespace WebGPU {
 
 class BindGroup;
 class Buffer;
+class CommandEncoder;
 class Device;
 class QuerySet;
 class RenderBundle;
@@ -47,9 +53,9 @@ class RenderPipeline;
 class RenderPassEncoder : public WGPURenderPassEncoderImpl, public RefCounted<RenderPassEncoder>, public CommandsMixin {
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    static Ref<RenderPassEncoder> create(id<MTLRenderCommandEncoder> renderCommandEncoder, const WGPURenderPassDescriptor& descriptor, NSUInteger visibilityResultBufferSize, bool depthReadOnly, bool stencilReadOnly, Device& device)
+    static Ref<RenderPassEncoder> create(id<MTLRenderCommandEncoder> renderCommandEncoder, const WGPURenderPassDescriptor& descriptor, NSUInteger visibilityResultBufferSize, bool depthReadOnly, bool stencilReadOnly, CommandEncoder& parentEncoder, id<MTLBuffer> visibilityResultBuffer, Device& device)
     {
-        return adoptRef(*new RenderPassEncoder(renderCommandEncoder, descriptor, visibilityResultBufferSize, depthReadOnly, stencilReadOnly, device));
+        return adoptRef(*new RenderPassEncoder(renderCommandEncoder, descriptor, visibilityResultBufferSize, depthReadOnly, stencilReadOnly, parentEncoder, visibilityResultBuffer, device));
     }
     static Ref<RenderPassEncoder> createInvalid(Device& device)
     {
@@ -59,15 +65,13 @@ public:
     ~RenderPassEncoder();
 
     void beginOcclusionQuery(uint32_t queryIndex);
-    void beginPipelineStatisticsQuery(const QuerySet&, uint32_t queryIndex);
     void draw(uint32_t vertexCount, uint32_t instanceCount, uint32_t firstVertex, uint32_t firstInstance);
     void drawIndexed(uint32_t indexCount, uint32_t instanceCount, uint32_t firstIndex, int32_t baseVertex, uint32_t firstInstance);
     void drawIndexedIndirect(const Buffer& indirectBuffer, uint64_t indirectOffset);
     void drawIndirect(const Buffer& indirectBuffer, uint64_t indirectOffset);
     void endOcclusionQuery();
     void endPass();
-    void endPipelineStatisticsQuery();
-    void executeBundles(Vector<std::reference_wrapper<const RenderBundle>>&& bundles);
+    void executeBundles(Vector<std::reference_wrapper<RenderBundle>>&& bundles);
     void insertDebugMarker(String&& markerLabel);
     void popDebugGroup();
     void pushDebugGroup(String&& groupLabel);
@@ -86,12 +90,13 @@ public:
     bool isValid() const { return m_renderCommandEncoder; }
 
 private:
-    RenderPassEncoder(id<MTLRenderCommandEncoder>, const WGPURenderPassDescriptor&, NSUInteger, bool depthReadOnly, bool stencilReadOnly, Device&);
+    RenderPassEncoder(id<MTLRenderCommandEncoder>, const WGPURenderPassDescriptor&, NSUInteger, bool depthReadOnly, bool stencilReadOnly, CommandEncoder&, id<MTLBuffer>, Device&);
     RenderPassEncoder(Device&);
 
     bool validatePopDebugGroup() const;
 
-    void makeInvalid() { m_renderCommandEncoder = nil; }
+    void makeInvalid();
+    void executePreDrawCommands();
 
     id<MTLRenderCommandEncoder> m_renderCommandEncoder { nil };
 
@@ -111,6 +116,25 @@ private:
     NSUInteger m_visibilityResultBufferSize { 0 };
     bool m_depthReadOnly { false };
     bool m_stencilReadOnly { false };
+    Vector<uint32_t> m_vertexDynamicOffsets;
+    Vector<uint32_t> m_fragmentDynamicOffsets;
+    const RenderPipeline* m_pipeline { nullptr };
+    RefPtr<CommandEncoder> m_parentEncoder;
+    HashMap<uint32_t, Vector<uint32_t>, DefaultHash<uint32_t>, WTF::UnsignedWithZeroKeyHashTraits<uint32_t>> m_bindGroupDynamicOffsets;
+    float m_minDepth { 0.f };
+    float m_maxDepth { 1.f };
+    HashSet<uint64_t, DefaultHash<uint32_t>, WTF::UnsignedWithZeroKeyHashTraits<uint32_t>> m_queryBufferIndicesToClear;
+    id<MTLBuffer> m_visibilityResultBuffer { nil };
+    uint32_t m_renderTargetWidth { 0 };
+    uint32_t m_renderTargetHeight { 0 };
+    NSMutableDictionary<NSNumber*, TextureAndClearColor*> *m_attachmentsToClear { nil };
+    NSMutableDictionary<NSNumber*, TextureAndClearColor*> *m_allColorAttachments { nil };
+    id<MTLTexture> m_depthStencilAttachmentToClear { nil };
+    float m_depthClearValue { 0 };
+    uint32_t m_stencilClearValue { 0 };
+    bool m_clearDepthAttachment { false };
+    bool m_clearStencilAttachment { false };
+    bool m_issuedDrawCall { false };
 };
 
 } // namespace WebGPU
