@@ -30,17 +30,37 @@
 
 #include "GPUConnectionToWebProcess.h"
 #include "GPUProcess.h"
+#include "Logging.h"
 #include <WebCore/AudioUtilities.h>
 #include <wtf/ThreadSafeRefCounted.h>
 
 #if PLATFORM(COCOA)
-#include "SharedRingBufferStorage.h"
+#include "SharedCARingBuffer.h"
 #include <WebCore/AudioOutputUnitAdaptor.h>
 #include <WebCore/CAAudioStreamDescription.h>
 #include <WebCore/CARingBuffer.h>
 #include <WebCore/WebAudioBufferList.h>
 #endif
 
+#if ENABLE(IPC_TESTING_API)
+#define WEB_PROCESS_TERMINATE_CONDITION !m_gpuConnectionToWebProcess.connection().ignoreInvalidMessageForTesting()
+#else
+#define WEB_PROCESS_TERMINATE_CONDITION true
+#endif
+
+#define TERMINATE_WEB_PROCESS_WITH_MESSAGE(message) \
+    if (WEB_PROCESS_TERMINATE_CONDITION) { \
+        RELEASE_LOG_FAULT(IPC, "Requesting termination of web process %" PRIu64 " for reason: %" PUBLIC_LOG_STRING, m_gpuConnectionToWebProcess.webProcessIdentifier().toUInt64(), #message); \
+        m_gpuConnectionToWebProcess.terminateWebProcess(); \
+    }
+
+#define MESSAGE_CHECK(assertion, message) do { \
+    if (UNLIKELY(!(assertion))) { \
+        TERMINATE_WEB_PROCESS_WITH_MESSAGE(message); \
+        return; \
+    } \
+} while (0)
+
 namespace WebKit {
 
 class RemoteAudioDestination final
@@ -50,13 +70,12 @@ class RemoteAudioDestination final
 {
     WTF_MAKE_FAST_ALLOCATED;
 public:
-    RemoteAudioDestination(GPUConnectionToWebProcess&, RemoteAudioDestinationIdentifier identifier, const String& inputDeviceId, uint32_t numberOfInputChannels, uint32_t numberOfOutputChannels, float sampleRate, float hardwareSampleRate, IPC::Semaphore&& renderSemaphore)
-        : m_id(identifier)
+    RemoteAudioDestination(GPUConnectionToWebProcess&, const String& inputDeviceId, uint32_t numberOfInputChannels, uint32_t numberOfOutputChannels, float sampleRate, float hardwareSampleRate, IPC::Semaphore&& renderSemaphore)
+        : m_renderSemaphore(WTFMove(renderSemaphore))
 #if PLATFORM(COCOA)
         , m_audioOutputUnitAdaptor(*this)
-        , m_ringBuffer(makeUniqueRef<WebCore::CARingBuffer>())
+        , m_numOutputChannels(numberOfOutputChannels)
 #endif
-        , m_renderSemaphore(WTFMove(renderSemaphore))
     {
         ASSERT(isMainRunLoop());
 #if PLATFORM(COCOA)
@@ -73,9 +92,24 @@ public:
     }
 
 #if PLATFORM(COCOA)
-    void audioSamplesStorageChanged(const SharedMemory::IPCHandle& ipcHandle, const WebCore::CAAudioStreamDescription& description, uint64_t numberOfFrames)
+    void setSharedMemory(SharedMemory::Handle&& handle)
     {
-        m_ringBuffer = WebCore::CARingBuffer::adoptStorage(makeUniqueRef<ReadOnlySharedRingBufferStorage>(ipcHandle.handle), description, numberOfFrames);
+        m_frameCount = SharedMemory::map(WTFMove(handle), SharedMemory::Protection::ReadWrite);
+    }
+
+    void audioSamplesStorageChanged(ConsumerSharedCARingBuffer::Handle&& handle)
+    {
+        if (m_isPlaying) {
+            stop();
+            ASSERT(!m_isPlaying);
+            if (m_isPlaying)
+                return;
+        }
+        m_ringBuffer = ConsumerSharedCARingBuffer::map(sizeof(Float32), m_numOutputChannels, WTFMove(handle));
+        if (!m_ringBuffer)
+            return;
+        start();
+        ASSERT(m_isPlaying);
     }
 #endif
 
@@ -103,6 +137,15 @@ public:
 
 private:
 #if PLATFORM(COCOA)
+    void incrementTotalFrameCount(UInt32 numberOfFrames)
+    {
+        static_assert(std::atomic<UInt32>::is_always_lock_free, "Shared memory atomic usage assumes lock free primitives are used");
+        if (m_frameCount) {
+            RELEASE_ASSERT(m_frameCount->size() == sizeof(std::atomic<uint32_t>));
+            WTF::atomicExchangeAdd(static_cast<uint32_t*>(m_frameCount->data()), numberOfFrames);
+        }
+    }
+
     OSStatus render(double sampleTime, uint64_t hostTime, UInt32 numberOfFrames, AudioBufferList* ioData)
     {
         ASSERT(!isMainRunLoop());
@@ -113,29 +156,22 @@ private:
             status = noErr;
         }
 
-        unsigned requestedSamplesCount = m_extraRequestedFrames;
-        for (; requestedSamplesCount < numberOfFrames; requestedSamplesCount += WebCore::AudioUtilities::renderQuantumSize) {
-            // Ask the audio thread in the WebContent process to render a quantum.
-            m_renderSemaphore.signal();
-        }
-        m_extraRequestedFrames = requestedSamplesCount - numberOfFrames;
+        incrementTotalFrameCount(numberOfFrames);
+        m_renderSemaphore.signal();
 
         return status;
     }
 #endif
-
-    RemoteAudioDestinationIdentifier m_id;
+    IPC::Semaphore m_renderSemaphore;
+    bool m_isPlaying { false };
 
 #if PLATFORM(COCOA)
     WebCore::AudioOutputUnitAdaptor m_audioOutputUnitAdaptor;
-
-    UniqueRef<WebCore::CARingBuffer> m_ringBuffer;
+    RefPtr<SharedMemory> m_frameCount;
+    const uint32_t m_numOutputChannels;
+    std::unique_ptr<ConsumerSharedCARingBuffer> m_ringBuffer;
     uint64_t m_startFrame { 0 };
-    unsigned m_extraRequestedFrames { 0 };
 #endif
-    IPC::Semaphore m_renderSemaphore;
-
-    bool m_isPlaying { false };
 };
 
 RemoteAudioDestinationManager::RemoteAudioDestinationManager(GPUConnectionToWebProcess& connection)
@@ -145,18 +181,24 @@ RemoteAudioDestinationManager::RemoteAudioDestinationManager(GPUConnectionToWebP
 
 RemoteAudioDestinationManager::~RemoteAudioDestinationManager() = default;
 
-void RemoteAudioDestinationManager::createAudioDestination(const String& inputDeviceId, uint32_t numberOfInputChannels, uint32_t numberOfOutputChannels, float sampleRate, float hardwareSampleRate, IPC::Semaphore&& renderSemaphore, CompletionHandler<void(const WebKit::RemoteAudioDestinationIdentifier)>&& completionHandler)
+void RemoteAudioDestinationManager::createAudioDestination(RemoteAudioDestinationIdentifier identifier, const String& inputDeviceId, uint32_t numberOfInputChannels, uint32_t numberOfOutputChannels, float sampleRate, float hardwareSampleRate, IPC::Semaphore&& renderSemaphore, SharedMemory::Handle&& handle)
 {
-    auto newID = RemoteAudioDestinationIdentifier::generateThreadSafe();
-    auto destination = makeUniqueRef<RemoteAudioDestination>(m_gpuConnectionToWebProcess, newID, inputDeviceId, numberOfInputChannels, numberOfOutputChannels, sampleRate, hardwareSampleRate, WTFMove(renderSemaphore));
-    m_audioDestinations.add(newID, WTFMove(destination));
-    completionHandler(newID);
+    MESSAGE_CHECK(!m_gpuConnectionToWebProcess.isLockdownModeEnabled(), "Received a createAudioDestination() message from a webpage in Lockdown mode.");
+
+    auto destination = makeUniqueRef<RemoteAudioDestination>(m_gpuConnectionToWebProcess, inputDeviceId, numberOfInputChannels, numberOfOutputChannels, sampleRate, hardwareSampleRate, WTFMove(renderSemaphore));
+#if PLATFORM(COCOA)
+    destination->setSharedMemory(WTFMove(handle));
+#else
+    UNUSED_PARAM(handle);
+#endif
+    m_audioDestinations.add(identifier, WTFMove(destination));
 }
 
-void RemoteAudioDestinationManager::deleteAudioDestination(RemoteAudioDestinationIdentifier identifier, CompletionHandler<void()>&& completionHandler)
+void RemoteAudioDestinationManager::deleteAudioDestination(RemoteAudioDestinationIdentifier identifier)
 {
+    MESSAGE_CHECK(!m_gpuConnectionToWebProcess.isLockdownModeEnabled(), "Received a deleteAudioDestination() message from a webpage in Lockdown mode.");
+
     m_audioDestinations.remove(identifier);
-    completionHandler();
 
     if (allowsExitUnderMemoryPressure())
         m_gpuConnectionToWebProcess.gpuProcess().tryExitIfUnusedAndUnderMemoryPressure();
@@ -164,6 +206,8 @@ void RemoteAudioDestinationManager::deleteAudioDestination(RemoteAudioDestinatio
 
 void RemoteAudioDestinationManager::startAudioDestination(RemoteAudioDestinationIdentifier identifier, CompletionHandler<void(bool)>&& completionHandler)
 {
+    MESSAGE_CHECK(!m_gpuConnectionToWebProcess.isLockdownModeEnabled(), "Received a startAudioDestination() message from a webpage in Lockdown mode.");
+
     bool isPlaying = false;
     if (auto* item = m_audioDestinations.get(identifier)) {
         item->start();
@@ -174,6 +218,8 @@ void RemoteAudioDestinationManager::startAudioDestination(RemoteAudioDestination
 
 void RemoteAudioDestinationManager::stopAudioDestination(RemoteAudioDestinationIdentifier identifier, CompletionHandler<void(bool)>&& completionHandler)
 {
+    MESSAGE_CHECK(!m_gpuConnectionToWebProcess.isLockdownModeEnabled(), "Received a stopAudioDestination() message from a webpage in Lockdown mode.");
+
     bool isPlaying = false;
     if (auto* item = m_audioDestinations.get(identifier)) {
         item->stop();
@@ -183,10 +229,10 @@ void RemoteAudioDestinationManager::stopAudioDestination(RemoteAudioDestinationI
 }
 
 #if PLATFORM(COCOA)
-void RemoteAudioDestinationManager::audioSamplesStorageChanged(RemoteAudioDestinationIdentifier identifier, const SharedMemory::IPCHandle& ipcHandle, const WebCore::CAAudioStreamDescription& description, uint64_t numberOfFrames)
+void RemoteAudioDestinationManager::audioSamplesStorageChanged(RemoteAudioDestinationIdentifier identifier, ConsumerSharedCARingBuffer::Handle&& handle)
 {
     if (auto* item = m_audioDestinations.get(identifier))
-        item->audioSamplesStorageChanged(ipcHandle, description, numberOfFrames);
+        item->audioSamplesStorageChanged(WTFMove(handle));
 }
 #endif
 
@@ -201,4 +247,8 @@ bool RemoteAudioDestinationManager::allowsExitUnderMemoryPressure() const
 
 } // namespace WebKit
 
+#undef MESSAGE_CHECK
+#undef TERMINATE_WEB_PROCESS_WITH_MESSAGE
+#undef WEB_PROCESS_TERMINATE_CONDITION
+
 #endif // ENABLE(GPU_PROCESS) && ENABLE(WEB_AUDIO)
