@@ -51,22 +51,28 @@ const char* WebCookieManager::supplementName()
 WebCookieManager::WebCookieManager(NetworkProcess& process)
     : m_process(process)
 {
-    m_process.addMessageReceiver(Messages::WebCookieManager::messageReceiverName(), *this);
+    process.addMessageReceiver(Messages::WebCookieManager::messageReceiverName(), *this);
 }
 
 WebCookieManager::~WebCookieManager() = default;
 
+Ref<NetworkProcess> WebCookieManager::protectedProcess()
+{
+    ASSERT(RunLoop::isMain());
+    return m_process.get();
+}
+
 void WebCookieManager::getHostnamesWithCookies(PAL::SessionID sessionID, CompletionHandler<void(Vector<String>&&)>&& completionHandler)
 {
     HashSet<String> hostnames;
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         storageSession->getHostnamesWithCookies(hostnames);
     completionHandler(copyToVector(hostnames));
 }
 
 void WebCookieManager::deleteCookiesForHostnames(PAL::SessionID sessionID, const Vector<String>& hostnames, CompletionHandler<void()>&& completionHandler)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         storageSession->deleteCookiesForHostnames(hostnames, WTFMove(completionHandler));
     else
         completionHandler();
@@ -74,7 +80,7 @@ void WebCookieManager::deleteCookiesForHostnames(PAL::SessionID sessionID, const
 
 void WebCookieManager::deleteAllCookies(PAL::SessionID sessionID, CompletionHandler<void()>&& completionHandler)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         storageSession->deleteAllCookies(WTFMove(completionHandler));
     else
         completionHandler();
@@ -82,7 +88,7 @@ void WebCookieManager::deleteAllCookies(PAL::SessionID sessionID, CompletionHand
 
 void WebCookieManager::deleteCookie(PAL::SessionID sessionID, const Cookie& cookie, CompletionHandler<void()>&& completionHandler)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         storageSession->deleteCookie(cookie, WTFMove(completionHandler));
     else
         completionHandler();
@@ -90,7 +96,7 @@ void WebCookieManager::deleteCookie(PAL::SessionID sessionID, const Cookie& cook
 
 void WebCookieManager::deleteAllCookiesModifiedSince(PAL::SessionID sessionID, WallTime time, CompletionHandler<void()>&& completionHandler)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         storageSession->deleteAllCookiesModifiedSince(time, WTFMove(completionHandler));
     else
         completionHandler();
@@ -99,7 +105,7 @@ void WebCookieManager::deleteAllCookiesModifiedSince(PAL::SessionID sessionID, W
 void WebCookieManager::getAllCookies(PAL::SessionID sessionID, CompletionHandler<void(Vector<WebCore::Cookie>&&)>&& completionHandler)
 {
     Vector<Cookie> cookies;
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         cookies = storageSession->getAllCookies();
     completionHandler(WTFMove(cookies));
 }
@@ -107,14 +113,14 @@ void WebCookieManager::getAllCookies(PAL::SessionID sessionID, CompletionHandler
 void WebCookieManager::getCookies(PAL::SessionID sessionID, const URL& url, CompletionHandler<void(Vector<WebCore::Cookie>&&)>&& completionHandler)
 {
     Vector<Cookie> cookies;
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         cookies = storageSession->getCookies(url);
     completionHandler(WTFMove(cookies));
 }
 
 void WebCookieManager::setCookie(PAL::SessionID sessionID, const Vector<Cookie>& cookies, CompletionHandler<void()>&& completionHandler)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID)) {
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID)) {
         for (auto& cookie : cookies)
             storageSession->setCookie(cookie);
     }
@@ -123,7 +129,7 @@ void WebCookieManager::setCookie(PAL::SessionID sessionID, const Vector<Cookie>&
 
 void WebCookieManager::setCookies(PAL::SessionID sessionID, const Vector<Cookie>& cookies, const URL& url, const URL& mainDocumentURL, CompletionHandler<void()>&& completionHandler)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         storageSession->setCookies(cookies, url, mainDocumentURL);
     completionHandler();
 }
@@ -131,12 +137,12 @@ void WebCookieManager::setCookies(PAL::SessionID sessionID, const Vector<Cookie>
 void WebCookieManager::notifyCookiesDidChange(PAL::SessionID sessionID)
 {
     ASSERT(RunLoop::isMain());
-    m_process.send(Messages::NetworkProcessProxy::CookiesDidChange(sessionID), 0);
+    protectedProcess()->send(Messages::NetworkProcessProxy::CookiesDidChange(sessionID), 0);
 }
 
 void WebCookieManager::startObservingCookieChanges(PAL::SessionID sessionID)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID)) {
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID)) {
         WebCore::startObservingCookieChanges(*storageSession, [this, sessionID] {
             notifyCookiesDidChange(sessionID);
         });
@@ -145,13 +151,13 @@ void WebCookieManager::startObservingCookieChanges(PAL::SessionID sessionID)
 
 void WebCookieManager::stopObservingCookieChanges(PAL::SessionID sessionID)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         WebCore::stopObservingCookieChanges(*storageSession);
 }
 
 void WebCookieManager::setHTTPCookieAcceptPolicy(PAL::SessionID sessionID, HTTPCookieAcceptPolicy policy, CompletionHandler<void()>&& completionHandler)
 {
-    platformSetHTTPCookieAcceptPolicy(sessionID, policy, [policy, process = Ref { m_process }, completionHandler = WTFMove(completionHandler)] () mutable {
+    platformSetHTTPCookieAcceptPolicy(sessionID, policy, [policy, process = protectedProcess(), completionHandler = WTFMove(completionHandler)] () mutable {
         process->cookieAcceptPolicyChanged(policy);
         completionHandler();
     });
@@ -159,7 +165,7 @@ void WebCookieManager::setHTTPCookieAcceptPolicy(PAL::SessionID sessionID, HTTPC
 
 void WebCookieManager::getHTTPCookieAcceptPolicy(PAL::SessionID sessionID, CompletionHandler<void(HTTPCookieAcceptPolicy)>&& completionHandler)
 {
-    if (auto* storageSession = m_process.storageSession(sessionID))
+    if (auto* storageSession = protectedProcess()->storageSession(sessionID))
         completionHandler(storageSession->cookieAcceptPolicy());
     else
         completionHandler(HTTPCookieAcceptPolicy::Never);
