@@ -47,6 +47,12 @@ LegacyCustomProtocolManager::LegacyCustomProtocolManager(NetworkProcess& network
     m_networkProcess->addMessageReceiver(Messages::LegacyCustomProtocolManager::messageReceiverName(), *this);
 }
 
+Ref<NetworkProcess> LegacyCustomProtocolManager::protectedNetworkProcess() const
+{
+    ASSERT(RunLoop::isMain());
+    return m_networkProcess.get();
+}
+
 void LegacyCustomProtocolManager::initialize(const NetworkProcessCreationParameters& parameters)
 {
     registerProtocolClass();
@@ -71,12 +77,12 @@ void LegacyCustomProtocolManager::removeCustomProtocol(LegacyCustomProtocolID cu
 
 void LegacyCustomProtocolManager::startLoading(LegacyCustomProtocolID customProtocolID, const WebCore::ResourceRequest& request)
 {
-    m_networkProcess->send(Messages::LegacyCustomProtocolManagerProxy::StartLoading(customProtocolID, request));
+    protectedNetworkProcess()->send(Messages::LegacyCustomProtocolManagerProxy::StartLoading(customProtocolID, request));
 }
 
 void LegacyCustomProtocolManager::stopLoading(LegacyCustomProtocolID customProtocolID)
 {
-    m_networkProcess->send(Messages::LegacyCustomProtocolManagerProxy::StopLoading(customProtocolID), 0);
+    protectedNetworkProcess()->send(Messages::LegacyCustomProtocolManagerProxy::StopLoading(customProtocolID), 0);
 }
 
 } // namespace WebKit
