@@ -61,16 +61,15 @@ NetworkResourceLoadParameters::NetworkResourceLoadParameters(
     , WebCore::CrossOriginOpenerPolicy&& sourceCrossOriginOpenerPolicy
     , uint64_t navigationID
     , std::optional<WebCore::NavigationRequester>&& navigationRequester
-#if ENABLE(SERVICE_WORKER)
     , WebCore::ServiceWorkersMode serviceWorkersMode
     , std::optional<WebCore::ServiceWorkerRegistrationIdentifier> serviceWorkerRegistrationIdentifier
     , OptionSet<WebCore::HTTPHeadersToKeepFromCleaning> httpHeadersToKeep
     , std::optional<WebCore::FetchIdentifier> navigationPreloadIdentifier
-#endif
 #if ENABLE(CONTENT_EXTENSIONS)
     , URL&& mainDocumentURL
     , std::optional<UserContentControllerIdentifier> userContentControllerIdentifier
 #endif
+    , bool linkPreconnectEarlyHintsEnabled
     ) : NetworkLoadParameters(WTFMove(networkLoadParameters))
         , identifier(identifier)
         , maximumBufferingTime(maximumBufferingTime)
@@ -97,16 +96,15 @@ NetworkResourceLoadParameters::NetworkResourceLoadParameters(
         , sourceCrossOriginOpenerPolicy(WTFMove(sourceCrossOriginOpenerPolicy))
         , navigationID(navigationID)
         , navigationRequester(WTFMove(navigationRequester))
-#if ENABLE(SERVICE_WORKER)
         , serviceWorkersMode(serviceWorkersMode)
         , serviceWorkerRegistrationIdentifier(serviceWorkerRegistrationIdentifier)
         , httpHeadersToKeep(httpHeadersToKeep)
         , navigationPreloadIdentifier(navigationPreloadIdentifier)
-#endif
 #if ENABLE(CONTENT_EXTENSIONS)
         , mainDocumentURL(WTFMove(mainDocumentURL))
         , userContentControllerIdentifier(userContentControllerIdentifier)
 #endif
+        , linkPreconnectEarlyHintsEnabled(linkPreconnectEarlyHintsEnabled)
 {
     if (httpBody) {
         request.setHTTPBody(WTFMove(httpBody));
@@ -119,7 +117,7 @@ NetworkResourceLoadParameters::NetworkResourceLoadParameters(
         }
     }
     
-    if (request.url().isLocalFile()) {
+    if (request.url().protocolIsFile()) {
         if (!sandboxExtensionIflocalFile)
             RELEASE_ASSERT_NOT_REACHED_WITH_MESSAGE("NetworkResourceLoadParameters which specify a URL of a local file should have sandboxExtensionIflocalFile");
         resourceSandboxExtension = SandboxExtension::create(WTFMove(*sandboxExtensionIflocalFile));
@@ -152,7 +150,7 @@ std::optional<Vector<SandboxExtension::Handle>> NetworkResourceLoadParameters::s
 
 std::optional<SandboxExtension::Handle> NetworkResourceLoadParameters::sandboxExtensionIflocalFile() const
 {
-    if (!request.url().isLocalFile())
+    if (!request.url().protocolIsFile())
         return std::nullopt;
     
     SandboxExtension::Handle requestSandboxExtension;
