@@ -25,16 +25,26 @@
 
 #pragma once
 
+#include <wtf/spi/darwin/XPCSPI.h>
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
 // FIXME: Remove these after <rdar://problem/30772033> is fixed.
 void NetworkServiceInitializer();
-void PluginServiceInitializer();
 void WebContentServiceInitializer();
 void GPUServiceInitializer();
-void WebAuthnServiceInitializer();
+
+void ExtensionEventHandler(xpc_connection_t);
+
+// Declared in WKProcessExtension.h for use in extension targets. Must be declared in project
+//  headers because the extension targets cannot import the entire WebKit module (rdar://119162443).
+@interface WKGrant : NSObject
+@end
+
+@interface WKProcessExtension : NSObject
+@end
 
 #ifdef __cplusplus
 }
