@@ -40,8 +40,9 @@ extern "C" {
 
 #define WEBKIT2_LOG_CHANNELS(M) \
     M(ActivityState) \
-    M(PrivateClickMeasurement) \
+    M(AdvancedPrivacyProtections) \
     M(AppSSO) \
+    M(Animations) \
     M(Automation) \
     M(AutomationInteractions) \
     M(BackForward) \
@@ -86,7 +87,9 @@ extern "C" {
     M(PerformanceLogging) \
     M(Plugins) \
     M(Printing) \
+    M(PrivateClickMeasurement) \
     M(Process) \
+    M(ProcessCapabilities) \
     M(ProcessSuspension) \
     M(ProcessSwapping) \
     M(ProximityNetworking) \
@@ -107,6 +110,7 @@ extern "C" {
     M(Storage) \
     M(StorageAPI) \
     M(SystemPreview) \
+    M(Telephony) \
     M(TextInput) \
     M(TextInteraction) \
     M(Translation) \
