@@ -70,15 +70,12 @@ public:
         Win32Handle;
 #endif
 
-    SharedMemoryHandle() = default;
     SharedMemoryHandle(SharedMemoryHandle&&) = default;
     explicit SharedMemoryHandle(const SharedMemoryHandle&) = default;
     SharedMemoryHandle(SharedMemoryHandle::Type&&, size_t);
 
     SharedMemoryHandle& operator=(SharedMemoryHandle&&) = default;
 
-    bool isNull() const;
-
     size_t size() const { return m_size; }
 
     // Take/Set ownership of the memory for jetsam purposes.
