@@ -27,23 +27,26 @@
 
 #if ENABLE(GPU_PROCESS)
 
+#include "GPUProcessPreferencesForWebProcess.h"
 #include "WebCoreArgumentCoders.h"
 #include <WebCore/ProcessIdentity.h>
 #include <wtf/MachSendRight.h>
 
+#if HAVE(AUDIT_TOKEN)
+#include "CoreIPCAuditToken.h"
+#endif
+
 namespace WebKit {
 
 struct GPUProcessConnectionParameters {
     WebCore::ProcessIdentity webProcessIdentity;
-    bool isDOMRenderingEnabled { false };
+    GPUProcessPreferencesForWebProcess preferences;
     bool isLockdownModeEnabled { false };
-    bool isWebGPUEnabled { false };
 #if ENABLE(IPC_TESTING_API)
     bool ignoreInvalidMessageForTesting { false };
 #endif
-    bool allowTestOnlyIPC { false };
 #if HAVE(AUDIT_TOKEN)
-    std::optional<audit_token_t> presentingApplicationAuditToken;
+    std::optional<CoreIPCAuditToken> presentingApplicationAuditToken;
 #endif
 #if ENABLE(VP9)
     std::optional<bool> hasVP9HardwareDecoder;
