@@ -34,7 +34,7 @@
 namespace WebKit {
 using namespace WebCore;
 
-DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER(ShareableBitmap);
+DECLARE_ALLOCATOR_WITH_HEAP_IDENTIFIER_AND_EXPORT(ShareableBitmap, WTF_INTERNAL);
 DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(ShareableBitmap);
 
 ShareableBitmapConfiguration::ShareableBitmapConfiguration(const IntSize& size, std::optional<DestinationColorSpace> colorSpace, bool isOpaque)
@@ -112,7 +112,7 @@ RefPtr<ShareableBitmap> ShareableBitmap::createFromImageDraw(NativeImage& image)
     if (!context)
         return nullptr;
 
-    context->drawNativeImage(image, imageSize, FloatRect({ }, imageSize), FloatRect({ }, imageSize), { CompositeOperator::Copy });
+    context->drawNativeImage(image, FloatRect({ }, imageSize), FloatRect({ }, imageSize), { CompositeOperator::Copy });
     return bitmap;
 }
 
@@ -142,21 +142,12 @@ auto ShareableBitmap::createHandle(SharedMemory::Protection protection) const ->
     auto memoryHandle = m_sharedMemory->createHandle(protection);
     if (!memoryHandle)
         return std::nullopt;
-    Handle handle;
-    handle.m_handle = WTFMove(*memoryHandle);
-    handle.m_configuration = m_configuration;
-    return { WTFMove(handle) };
+    return { Handle(WTFMove(*memoryHandle), m_configuration) };
 }
 
 auto ShareableBitmap::createReadOnlyHandle() const -> std::optional<Handle>
 {
-    Handle handle;
-    auto memoryHandle = m_sharedMemory->createHandle(SharedMemory::Protection::ReadOnly);
-    if (!memoryHandle)
-        return std::nullopt;
-    handle.m_handle = WTFMove(*memoryHandle);
-    handle.m_configuration = m_configuration;
-    return handle;
+    return createHandle(SharedMemory::Protection::ReadOnly);
 }
 
 ShareableBitmap::ShareableBitmap(ShareableBitmapConfiguration configuration, Ref<SharedMemory>&& sharedMemory)
