@@ -39,17 +39,7 @@ class Encoder;
 
 namespace WebKit {
 
-class UpdateInfo {
-    WTF_MAKE_NONCOPYABLE(UpdateInfo);
-
-public:
-    UpdateInfo() { }
-    UpdateInfo(UpdateInfo&&) = default;
-    UpdateInfo& operator=(UpdateInfo&&) = default;
-
-    void encode(IPC::Encoder&) const;
-    static WARN_UNUSED_RETURN bool decode(IPC::Decoder&, UpdateInfo&);
-
+struct UpdateInfo {
     // The size of the web view.
     WebCore::IntSize viewSize;
     float deviceScaleFactor { 0 };
@@ -68,7 +58,7 @@ public:
     float updateScaleFactor { 0 };
 
     // The handle of the shareable bitmap containing the updates. Will be null if there are no updates.
-    ShareableBitmapHandle bitmapHandle;
+    std::optional<ShareableBitmap::Handle> bitmapHandle;
 
     // The offset in the bitmap where the rendered contents are.
     WebCore::IntPoint bitmapOffset;
