@@ -31,6 +31,7 @@
 #include "SandboxExtension.h"
 #include "SessionState.h"
 #include "UserContentControllerParameters.h"
+#include "ViewWindowCoordinates.h"
 #include "WebCoreArgumentCoders.h"
 #include "WebPageGroupData.h"
 #include "WebPageProxyIdentifier.h"
@@ -69,6 +70,10 @@
 #include "WebExtensionControllerParameters.h"
 #endif
 
+#if (PLATFORM(GTK) || PLATFORM(WPE)) && USE(GBM)
+#include "DMABufRendererBufferFormat.h"
+#endif
+
 namespace IPC {
 class Decoder;
 class Encoder;
@@ -164,6 +169,7 @@ struct WebPageCreationParameters {
     bool useFormSemanticContext { false };
     int headerBannerHeight { 0 };
     int footerBannerHeight { 0 };
+    std::optional<ViewWindowCoordinates> viewWindowCoordinates;
 #endif
 #if ENABLE(META_VIEWPORT)
     bool ignoresViewportScaleLimits;
@@ -195,6 +201,9 @@ struct WebPageCreationParameters {
 #endif
 #if HAVE(APP_ACCENT_COLORS)
     WebCore::Color accentColor;
+#if PLATFORM(MAC)
+    bool appUsesCustomAccentColor;
+#endif
 #endif
 #if USE(WPE_RENDERER)
     UnixFileDescriptor hostFileDescriptor;
@@ -287,7 +296,11 @@ struct WebPageCreationParameters {
 
     WebCore::ContentSecurityPolicyModeForExtension contentSecurityPolicyModeForExtension { WebCore::ContentSecurityPolicyModeForExtension::None };
 
-    std::optional<FrameTreeCreationParameters> subframeProcessFrameTreeCreationParameters;
+    struct SubframeProcessPageParameters {
+        URL initialMainDocumentURL;
+        FrameTreeCreationParameters frameTreeParameters;
+    };
+    std::optional<SubframeProcessPageParameters> subframeProcessPageParameters;
     std::optional<WebCore::FrameIdentifier> openerFrameIdentifier;
     std::optional<WebCore::FrameIdentifier> mainFrameIdentifier;
 
@@ -299,6 +312,10 @@ struct WebPageCreationParameters {
 #if HAVE(MACH_BOOTSTRAP_EXTENSION)
     SandboxExtension::Handle machBootstrapHandle;
 #endif
+
+#if (PLATFORM(GTK) || PLATFORM(WPE)) && USE(GBM)
+    Vector<DMABufRendererBufferFormat> preferredBufferFormats;
+#endif
 };
 
 } // namespace WebKit
