@@ -22,6 +22,7 @@
 #include "APIObject.h"
 #include "SharedMemory.h"
 #include "WebHitTestResultData.h"
+#include "WebPageProxy.h"
 #include <WebCore/DictionaryPopupInfo.h>
 #include <WebCore/FloatPoint.h>
 #include <WebCore/IntRect.h>
@@ -45,7 +46,7 @@ class WebFrame;
 
 class HitTestResult : public API::ObjectImpl<API::Object::Type::HitTestResult> {
 public:
-    static Ref<HitTestResult> create(const WebKit::WebHitTestResultData&);
+    static Ref<HitTestResult> create(const WebKit::WebHitTestResultData&, WebKit::WebPageProxy&);
 
     WTF::String absoluteImageURL() const { return m_data.absoluteImageURL; }
     WTF::String absolutePDFURL() const { return m_data.absolutePDFURL; }
@@ -55,6 +56,7 @@ public:
     WTF::String linkLabel() const { return m_data.linkLabel; }
     WTF::String linkTitle() const { return m_data.linkTitle; }
     WTF::String lookupText() const { return m_data.lookupText; }
+    WTF::String sourceImageMIMEType() const { return m_data.sourceImageMIMEType; }
 
     bool isContentEditable() const { return m_data.isContentEditable; }
 
@@ -70,13 +72,23 @@ public:
 
     bool isDownloadableMedia() const { return m_data.isDownloadableMedia; }
 
+    bool mediaIsInFullscreen() const { return m_data.mediaIsInFullscreen; }
+
+    WebKit::WebHitTestResultData::ElementType elementType() const { return m_data.elementType; }
+
+    WebKit::WebPageProxy& page() { return m_page.get(); }
+
+    const std::optional<WebKit::FrameInfoData>& frameInfo() const { return m_data.frameInfo; }
+
 private:
-    explicit HitTestResult(const WebKit::WebHitTestResultData& hitTestResultData)
+    explicit HitTestResult(const WebKit::WebHitTestResultData& hitTestResultData, WebKit::WebPageProxy& page)
         : m_data(hitTestResultData)
+        , m_page(page)
     {
     }
 
     WebKit::WebHitTestResultData m_data;
+    Ref<WebKit::WebPageProxy> m_page;
 };
 
 } // namespace API
