@@ -30,6 +30,7 @@
 
 #include "WebExtensionContextParameters.h"
 #include "WebExtensionContextProxyMessages.h"
+#include "WebPageProxy.h"
 #include <wtf/HashMap.h>
 #include <wtf/NeverDestroyed.h>
 
@@ -57,28 +58,52 @@ WebExtensionContext::WebExtensionContext()
 
 WebExtensionContextParameters WebExtensionContext::parameters() const
 {
-    return WebExtensionContextParameters {
+    return {
         identifier(),
         baseURL(),
         uniqueIdentifier(),
+        extension().serializeLocalization(),
         extension().serializeManifest(),
         extension().manifestVersion(),
-        inTestingMode()
+        inTestingMode(),
+        backgroundPageIdentifier(),
+        popupPageIdentifiers(),
+        tabPageIdentifiers()
     };
 }
 
-WeakHashSet<WebProcessProxy> WebExtensionContext::processes(WebExtensionEventListenerType type) const
+bool WebExtensionContext::pageListensForEvent(const WebPageProxy& page, WebExtensionEventListenerType type, WebExtensionContentWorldType contentWorldType) const
 {
-    WeakHashSet<WebProcessProxy> processes;
-    auto page = m_eventListenerPages.find(type);
-    if (page != m_eventListenerPages.end()) {
-        for (auto entry : page->value) {
-            auto& process = entry.key.process();
-            if (process.canSendMessage())
-                processes.add(process);
-        }
+    if (!hasAccessInPrivateBrowsing() && page.sessionID().isEphemeral())
+        return false;
+
+    auto pagesEntry = m_eventListenerPages.find({ type, contentWorldType });
+    if (pagesEntry == m_eventListenerPages.end())
+        return false;
+
+    if (!pagesEntry->value.contains(page))
+        return false;
+
+    return page.process().canSendMessage();
+}
+
+WebExtensionContext::WebProcessProxySet WebExtensionContext::processes(WebExtensionEventListenerType type, WebExtensionContentWorldType contentWorldType) const
+{
+    auto pagesEntry = m_eventListenerPages.find({ type, contentWorldType });
+    if (pagesEntry == m_eventListenerPages.end())
+        return { };
+
+    WebProcessProxySet result;
+    for (auto entry : pagesEntry->value) {
+        if (!hasAccessInPrivateBrowsing() && entry.key.sessionID().isEphemeral())
+            continue;
+
+        Ref process = entry.key.process();
+        if (process->canSendMessage())
+            result.add(WTFMove(process));
     }
-    return processes;
+
+    return result;
 }
 
 } // namespace WebKit
