@@ -51,15 +51,11 @@ namespace WebKit {
 
 class WebPageProxy;
 
-enum MediaDevicePermissionRequestIdentifierType { };
+enum class MediaDevicePermissionRequestIdentifierType { };
 using MediaDevicePermissionRequestIdentifier = ObjectIdentifier<MediaDevicePermissionRequestIdentifierType>;
 
 class UserMediaPermissionRequestManagerProxy
-#if ENABLE(MEDIA_STREAM)
-    : public WebCore::AudioCaptureFactory::ExtensiveObserver
-#else
     : public CanMakeWeakPtr<UserMediaPermissionRequestManagerProxy>
-#endif
 #if !RELEASE_LOG_DISABLED
     , private LoggerHelper
 #endif
