@@ -30,9 +30,21 @@
 #include "MessageReceiver.h"
 #include <WebCore/CredentialRequestOptions.h>
 #include <WebCore/FrameIdentifier.h>
+#include <WebCore/MediationRequirement.h>
+#include <wtf/CompletionHandler.h>
 #include <wtf/Forward.h>
 #include <wtf/Noncopyable.h>
 
+#if HAVE(WEB_AUTHN_AS_MODERN)
+OBJC_CLASS _WKASDelegate;
+OBJC_CLASS NSArray;
+OBJC_CLASS NSString;
+OBJC_CLASS ASAuthorization;
+OBJC_CLASS ASAuthorizationController;
+typedef NSString *ASAuthorizationPublicKeyCredentialUserVerificationPreference;
+typedef NSString *ASAuthorizationPublicKeyCredentialAttestationKind;
+#endif
+
 namespace WebCore {
 enum class AuthenticatorAttachment : uint8_t;
 struct ExceptionData;
@@ -55,6 +67,7 @@ class WebPageProxy;
 struct FrameInfoData;
 struct WebAuthenticationRequestData;
 
+using CapbilitiesCompletionHandler = CompletionHandler<void(const HashMap<WTF::String, bool>&)>;
 using RequestCompletionHandler = CompletionHandler<void(const WebCore::AuthenticatorResponseData&, WebCore::AuthenticatorAttachment, const WebCore::ExceptionData&)>;
 
 class WebAuthenticatorCoordinatorProxy : public IPC::MessageReceiver {
@@ -64,6 +77,11 @@ public:
     explicit WebAuthenticatorCoordinatorProxy(WebPageProxy&);
     ~WebAuthenticatorCoordinatorProxy();
 
+#if HAVE(WEB_AUTHN_AS_MODERN)
+    void pauseConditionalAssertion();
+    void unpauseConditionalAssertion();
+#endif
+
 private:
     using QueryCompletionHandler = CompletionHandler<void(bool)>;
 
@@ -71,10 +89,11 @@ private:
     void didReceiveMessage(IPC::Connection&, IPC::Decoder&) override;
 
     // Receivers.
-    void makeCredential(WebCore::FrameIdentifier, FrameInfoData&&, Vector<uint8_t>&& hash, WebCore::PublicKeyCredentialCreationOptions&&, bool processingUserGesture, RequestCompletionHandler&&);
-    void getAssertion(WebCore::FrameIdentifier, FrameInfoData&&, Vector<uint8_t>&& hash, WebCore::PublicKeyCredentialRequestOptions&&, WebCore::CredentialRequestOptions::MediationRequirement, std::optional<WebCore::SecurityOriginData>, bool processingUserGesture, RequestCompletionHandler&&);
+    void makeCredential(WebCore::FrameIdentifier, FrameInfoData&&, Vector<uint8_t>&& hash, WebCore::PublicKeyCredentialCreationOptions&&, RequestCompletionHandler&&);
+    void getAssertion(WebCore::FrameIdentifier, FrameInfoData&&, Vector<uint8_t>&& hash, WebCore::PublicKeyCredentialRequestOptions&&, WebCore::MediationRequirement, std::optional<WebCore::SecurityOriginData>, RequestCompletionHandler&&);
     void isUserVerifyingPlatformAuthenticatorAvailable(const WebCore::SecurityOriginData&, QueryCompletionHandler&&);
     void isConditionalMediationAvailable(const WebCore::SecurityOriginData&, QueryCompletionHandler&&);
+    void getClientCapabilities(const WebCore::SecurityOriginData&, CapbilitiesCompletionHandler&&);
     void cancel();
 
     void handleRequest(WebAuthenticationRequestData&&, RequestCompletionHandler&&);
@@ -82,8 +101,27 @@ private:
     WebPageProxy& m_webPageProxy;
 
 #if HAVE(UNIFIED_ASC_AUTH_UI)
+    bool isASCAvailable();
+
+#if HAVE(WEB_AUTHN_AS_MODERN)
+    RetainPtr<ASAuthorizationController> constructASController(WebAuthenticationRequestData&&);
+    RetainPtr<NSArray> requestsForRegisteration(const WebCore::PublicKeyCredentialCreationOptions&, const Vector<uint8_t>& hash);
+    RetainPtr<NSArray> requestsForAssertion(const WebCore::PublicKeyCredentialRequestOptions&, const Vector<uint8_t>& hash, std::optional<WebCore::SecurityOriginData>& parentOrigin);
+#endif
+
+    void performRequest(WebAuthenticationRequestData&&, RequestCompletionHandler&&);
+
     RetainPtr<ASCCredentialRequestContext> contextForRequest(WebAuthenticationRequestData&&);
-    void performRequest(RetainPtr<ASCCredentialRequestContext>, RequestCompletionHandler&&);
+    void performRequestLegacy(RetainPtr<ASCCredentialRequestContext>, RequestCompletionHandler&&);
+
+#if HAVE(WEB_AUTHN_AS_MODERN)
+    RequestCompletionHandler m_completionHandler;
+    RetainPtr<_WKASDelegate> m_delegate;
+    RetainPtr<ASAuthorizationController> m_controller;
+    bool m_paused { false };
+    bool m_isConditionalAssertion { false };
+#endif
+
     RetainPtr<ASCAuthorizationRemotePresenter> m_presenter;
     RetainPtr<ASCAgentProxy> m_proxy;
 #endif // HAVE(UNIFIED_ASC_AUTH_UI)
