@@ -33,13 +33,16 @@
 #include <wtf/CompletionHandler.h>
 
 namespace WebCore {
+enum class WindowProxyProperty : uint8_t;
 struct NotificationData;
+class RegistrableDomain;
 class SecurityOriginData;
 }
 
 namespace WebKit {
 
 class WebPageProxy;
+class WebsiteDataStore;
 
 class WebsiteDataStoreClient {
     WTF_MAKE_FAST_ALLOCATED;
@@ -89,7 +92,11 @@ public:
     virtual void workerUpdatedAppBadge(const WebCore::SecurityOriginData&, std::optional<uint64_t>)
     {
     }
-    
+
+    virtual void navigationToNotificationActionURL(const URL&)
+    {
+    }
+
     virtual void requestBackgroundFetchPermission(const WebCore::SecurityOriginData& topOrigin, const WebCore::SecurityOriginData& frameOrigin, CompletionHandler<void(bool)>&& completionHandler)
     {
         UNUSED_PARAM(topOrigin);
@@ -100,6 +107,10 @@ public:
     virtual void notifyBackgroundFetchChange(const String&, BackgroundFetchChange)
     {
     }
+
+    virtual void didAccessWindowProxyProperty(const WebCore::RegistrableDomain&, const WebCore::RegistrableDomain&, WebCore::WindowProxyProperty, bool)
+    {
+    }
 };
 
 } // namespace WebKit
