@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2022 Apple Inc. All rights reserved.
+ * Copyright (C) 2022-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,11 +41,25 @@ class WebExtensionAPIExtension : public WebExtensionAPIObject, public JSWebExten
     WEB_EXTENSION_DECLARE_JS_WRAPPER_CLASS(WebExtensionAPIExtension, extension);
 
 public:
-#if PLATFORM(COCOA)
-    bool isPropertyAllowed(String propertyName, WebPage*);
+    enum class ViewType : uint8_t {
+        Popup,
+        Tab,
+    };
 
-    NSURL *getURL(NSString *resourcePath, NSString **errorString);
+#if PLATFORM(COCOA)
+    bool isPropertyAllowed(ASCIILiteral propertyName, WebPage*);
+
+    bool isInIncognitoContext(WebPage*);
+    void isAllowedFileSchemeAccess(Ref<WebExtensionCallbackHandler>&&);
+    void isAllowedIncognitoAccess(Ref<WebExtensionCallbackHandler>&&);
+
+    NSURL *getURL(NSString *resourcePath, NSString **outExceptionString);
+    JSValue *getBackgroundPage(JSContextRef);
+    NSArray *getViews(JSContextRef, NSDictionary *filter, NSString **outExceptionString);
 #endif
+
+private:
+    static bool parseViewFilters(NSDictionary *, std::optional<ViewType>&, std::optional<WebExtensionTabIdentifier>&, std::optional<WebExtensionWindowIdentifier>&, NSString *sourceKey, NSString **outExceptionString);
 };
 
 } // namespace WebKit
