@@ -22,6 +22,7 @@
  * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
  * THE POSSIBILITY OF SUCH DAMAGE.
  */
+
 #include "config.h"
 #include "WebFullScreenManager.h"
 
@@ -51,7 +52,7 @@
 
 #if PLATFORM(IOS_FAMILY) || (PLATFORM(MAC) && ENABLE(VIDEO_PRESENTATION_MODE))
 #include "PlaybackSessionManager.h"
-#include "VideoFullscreenManager.h"
+#include "VideoPresentationManager.h"
 #endif
 
 namespace WebKit {
@@ -208,7 +209,7 @@ void WebFullScreenManager::enterFullScreenForElement(WebCore::Element* element)
 #if PLATFORM(IOS_FAMILY) || (PLATFORM(MAC) && ENABLE(VIDEO_PRESENTATION_MODE))
     isVideoElement = is<HTMLVideoElement>(element);
 
-    if (m_page->videoFullscreenManager().videoElementInPictureInPicture() && m_element->document().quirks().blocksEnteringStandardFullscreenFromPictureInPictureQuirk())
+    if (m_page->videoPresentationManager().videoElementInPictureInPicture() && m_element->document().quirks().blocksEnteringStandardFullscreenFromPictureInPictureQuirk())
         return;
 
     if (auto* currentPlaybackControlsElement = m_page->playbackSessionManager().currentPlaybackControlsElement())
@@ -306,7 +307,7 @@ void WebFullScreenManager::updateMainVideoElement()
             if (!rendererAndBounds)
                 continue;
 
-            auto [renderer, bounds] = *rendererAndBounds;
+            auto& [renderer, bounds] = *rendererAndBounds;
             if (!renderer || bounds.isEmpty())
                 continue;
 
@@ -343,6 +344,19 @@ void WebFullScreenManager::willExitFullScreen()
     m_page->injectedBundleFullScreenClient().beganExitFullScreen(m_page.ptr(), m_finalFrame, m_initialFrame);
 }
 
+static Vector<Ref<Element>> collectFullscreenElementsFromElement(Element* element)
+{
+    Vector<Ref<Element>> fullscreenElements;
+
+    while (element && element->document().fullscreenManager().fullscreenElement() == element) {
+        fullscreenElements.append(*element);
+        auto parentDocument = element->document().parentDocument();
+        element = parentDocument ? parentDocument->fullscreenManager().fullscreenElement() : nullptr;
+    }
+
+    return fullscreenElements;
+}
+
 void WebFullScreenManager::didExitFullScreen()
 {
     if (!m_element)
@@ -352,7 +366,17 @@ void WebFullScreenManager::didExitFullScreen()
 
     setFullscreenInsets(WebCore::FloatBoxExtent());
     setFullscreenAutoHideDuration(0_s);
+
+    auto fullscreenElements = collectFullscreenElementsFromElement(m_element.get());
+
     m_element->document().fullscreenManager().didExitFullscreen();
+
+    // Ensure the element (and all its parent fullscreen elements) that just exited fullscreen are still in view:
+    while (!fullscreenElements.isEmpty()) {
+        auto element = fullscreenElements.takeLast();
+        element->scrollIntoView();
+    }
+
     clearElement();
 }
 
@@ -377,7 +401,7 @@ void WebFullScreenManager::requestRestoreFullScreen()
     }
 
     ALWAYS_LOG(LOGIDENTIFIER, "<", element->tagName(), " id=\"", element->getIdAttribute(), "\">");
-    WebCore::UserGestureIndicator gestureIndicator(WebCore::ProcessingUserGesture, &element->document());
+    WebCore::UserGestureIndicator gestureIndicator(WebCore::IsProcessingUserGesture::Yes, &element->document());
     element->document().fullscreenManager().requestFullscreenForElement(*element, nullptr, WebCore::FullscreenManager::ExemptIFrameAllowFullscreenRequirement);
 }
 
@@ -412,8 +436,7 @@ void WebFullScreenManager::close()
 
 void WebFullScreenManager::saveScrollPosition()
 {
-    if (auto* localMainFrame = dynamicDowncast<LocalFrame>(m_page->corePage()->mainFrame()))
-        m_scrollPosition = localMainFrame->view()->scrollPosition();
+    m_scrollPosition = m_page->corePage()->mainFrame().virtualView()->scrollPosition();
 }
 
 void WebFullScreenManager::restoreScrollPosition()
