@@ -49,7 +49,7 @@ class WebAudioBufferList;
 
 namespace WebKit {
 
-class RemoteAudioDestinationProxy final : public WebCore::AudioDestinationResampler, public GPUProcessConnection::Client {
+class RemoteAudioDestinationProxy final : public WebCore::AudioDestinationResampler, public GPUProcessConnection::Client, public ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<RemoteAudioDestinationProxy> {
     WTF_MAKE_NONCOPYABLE(RemoteAudioDestinationProxy);
 public:
     using AudioIOCallback = WebCore::AudioIOCallback;
@@ -59,6 +59,10 @@ public:
     RemoteAudioDestinationProxy(AudioIOCallback&, const String& inputDeviceId, unsigned numberOfInputChannels, unsigned numberOfOutputChannels, float sampleRate);
     ~RemoteAudioDestinationProxy();
 
+    void ref() const final { return ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<RemoteAudioDestinationProxy>::ref(); }
+    void deref() const final { return ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<RemoteAudioDestinationProxy>::deref(); }
+    ThreadSafeWeakPtrControlBlock& controlBlock() const final { return ThreadSafeRefCountedAndCanMakeThreadSafeWeakPtr<RemoteAudioDestinationProxy>::controlBlock(); }
+
 private:
     void startRendering(CompletionHandler<void(bool)>&&) final;
     void stopRendering(CompletionHandler<void(bool)>&&) final;
@@ -73,9 +77,14 @@ private:
     // GPUProcessConnection::Client.
     void gpuProcessConnectionDidClose(GPUProcessConnection&) final;
 
+    uint32_t totalFrameCount() const;
+
     RemoteAudioDestinationIdentifier m_destinationID; // Call destinationID() getter to make sure the destinationID is valid.
 
-    WeakPtr<GPUProcessConnection> m_gpuProcessConnection;
+    static uint8_t s_realtimeThreadCount;
+    static constexpr uint8_t s_maximumConcurrentRealtimeThreads { 3 };
+
+    ThreadSafeWeakPtr<GPUProcessConnection> m_gpuProcessConnection;
 #if PLATFORM(COCOA)
     std::unique_ptr<ProducerSharedCARingBuffer> m_ringBuffer;
     std::unique_ptr<WebCore::WebAudioBufferList> m_audioBufferList;
@@ -88,7 +97,10 @@ private:
     float m_remoteSampleRate;
 
     RefPtr<Thread> m_renderThread;
+    RefPtr<SharedMemory> m_frameCount;
+    uint32_t m_lastFrameCount { 0 };
     std::atomic<bool> m_shouldStopThread { false };
+    bool m_isRealtimeThread { false };
 };
 
 } // namespace WebKit
