@@ -49,19 +49,19 @@ namespace WebKit {
 
 using namespace WebCore;
 
-Ref<SourceBufferPrivateRemote> SourceBufferPrivateRemote::create(GPUProcessConnection& gpuProcessConnection, RemoteSourceBufferIdentifier remoteSourceBufferIdentifier, const MediaSourcePrivateRemote& mediaSourcePrivate, const MediaPlayerPrivateRemote& mediaPlayerPrivate)
+Ref<SourceBufferPrivateRemote> SourceBufferPrivateRemote::create(GPUProcessConnection& gpuProcessConnection, RemoteSourceBufferIdentifier remoteSourceBufferIdentifier, MediaSourcePrivateRemote& mediaSourcePrivate, const MediaPlayerPrivateRemote& mediaPlayerPrivate)
 {
     return adoptRef(*new SourceBufferPrivateRemote(gpuProcessConnection, remoteSourceBufferIdentifier, mediaSourcePrivate, mediaPlayerPrivate));
 }
 
-SourceBufferPrivateRemote::SourceBufferPrivateRemote(GPUProcessConnection& gpuProcessConnection, RemoteSourceBufferIdentifier remoteSourceBufferIdentifier, const MediaSourcePrivateRemote& mediaSourcePrivate, const MediaPlayerPrivateRemote& mediaPlayerPrivate)
-    : m_gpuProcessConnection(gpuProcessConnection)
+SourceBufferPrivateRemote::SourceBufferPrivateRemote(GPUProcessConnection& gpuProcessConnection, RemoteSourceBufferIdentifier remoteSourceBufferIdentifier, MediaSourcePrivateRemote& mediaSourcePrivate, const MediaPlayerPrivateRemote& mediaPlayerPrivate)
+    : SourceBufferPrivate(mediaSourcePrivate)
+    , m_gpuProcessConnection(gpuProcessConnection)
     , m_remoteSourceBufferIdentifier(remoteSourceBufferIdentifier)
-    , m_mediaSourcePrivate(mediaSourcePrivate)
     , m_mediaPlayerPrivate(mediaPlayerPrivate)
 #if !RELEASE_LOG_DISABLED
-    , m_logger(m_mediaSourcePrivate->logger())
-    , m_logIdentifier(m_mediaSourcePrivate->nextSourceBufferLogIdentifier())
+    , m_logger(mediaSourcePrivate.logger())
+    , m_logIdentifier(mediaSourcePrivate.nextSourceBufferLogIdentifier())
 #endif
 {
     ALWAYS_LOG(LOGIDENTIFIER);
@@ -72,7 +72,6 @@ SourceBufferPrivateRemote::SourceBufferPrivateRemote(GPUProcessConnection& gpuPr
 SourceBufferPrivateRemote::~SourceBufferPrivateRemote()
 {
     ALWAYS_LOG(LOGIDENTIFIER);
-    ASSERT(!m_client);
 
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!gpuProcessConnection)
@@ -81,17 +80,24 @@ SourceBufferPrivateRemote::~SourceBufferPrivateRemote()
     gpuProcessConnection->messageReceiverMap().removeMessageReceiver(Messages::SourceBufferPrivateRemote::messageReceiverName(), m_remoteSourceBufferIdentifier.toUInt64());
 }
 
-void SourceBufferPrivateRemote::append(Ref<SharedBuffer>&& data)
+Ref<MediaPromise> SourceBufferPrivateRemote::append(Ref<SharedBuffer>&& data)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!isGPURunning())
-        return;
+        return MediaPromise::createAndReject(PlatformMediaError::IPCError);
 
-    gpuProcessConnection->connection().sendWithAsyncReply(Messages::RemoteSourceBufferProxy::Append(IPC::SharedBufferReference { WTFMove(data) }), [] (auto&& bufferHandle) {
-        // Take ownership of shared memory and mark it as media-related memory.
-        if (bufferHandle)
-            bufferHandle->takeOwnershipOfMemory(MemoryLedger::Media);
-    }, m_remoteSourceBufferIdentifier);
+    return gpuProcessConnection->connection().sendWithPromisedReply(Messages::RemoteSourceBufferProxy::Append(IPC::SharedBufferReference { WTFMove(data) }), m_remoteSourceBufferIdentifier)->whenSettled(RunLoop::current(), [weakThis = ThreadSafeWeakPtr { *this }, this](auto&& result) {
+        if (!result)
+            return MediaPromise::createAndReject(PlatformMediaError::IPCError);
+        setTimestampOffset(std::get<MediaTime>(*result));
+        return MediaPromise::createAndSettle(std::get<MediaPromise::Result>(*result));
+    });
+}
+
+void SourceBufferPrivateRemote::takeOwnershipOfMemory(WebKit::SharedMemory::Handle&& bufferHandle)
+{
+    // Take ownership of shared memory and mark it as media-related memory.
+    bufferHandle.takeOwnershipOfMemory(MemoryLedger::Media);
 }
 
 void SourceBufferPrivateRemote::abort()
@@ -115,9 +121,10 @@ void SourceBufferPrivateRemote::resetParserState()
     gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::ResetParserState(), m_remoteSourceBufferIdentifier);
 }
 
-void SourceBufferPrivateRemote::appendInternal(Ref<SharedBuffer>&&)
+Ref<MediaPromise> SourceBufferPrivateRemote::appendInternal(Ref<SharedBuffer>&&)
 {
     ASSERT_NOT_REACHED();
+    return MediaPromise::createAndReject(PlatformMediaError::IPCError);
 }
 
 void SourceBufferPrivateRemote::resetParserStateInternal()
@@ -134,36 +141,18 @@ void SourceBufferPrivateRemote::removedFromMediaSource()
     gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::RemovedFromMediaSource(), m_remoteSourceBufferIdentifier);
 }
 
-MediaPlayer::ReadyState SourceBufferPrivateRemote::readyState() const
-{
-    return m_mediaPlayerPrivate ? m_mediaPlayerPrivate->readyState() : MediaPlayer::ReadyState::HaveNothing;
-}
-
-void SourceBufferPrivateRemote::setReadyState(MediaPlayer::ReadyState state)
-{
-    if (!m_mediaSourcePrivate)
-        return;
-
-    if (m_mediaPlayerPrivate)
-        m_mediaPlayerPrivate->setReadyState(state);
-
-    auto gpuProcessConnection = m_gpuProcessConnection.get();
-    if (!isGPURunning())
-        return;
-
-    gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::SetReadyState(state), m_remoteSourceBufferIdentifier);
-}
-
 void SourceBufferPrivateRemote::setActive(bool active)
 {
-    if (!m_mediaSourcePrivate)
+    SourceBufferPrivate::setActive(active);
+
+    auto mediaSource = m_mediaSource.get();
+    if (!mediaSource)
         return;
 
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!isGPURunning())
         return;
 
-    m_isActive = active;
     gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::SetActive(active), m_remoteSourceBufferIdentifier);
 }
 
@@ -185,6 +174,7 @@ void SourceBufferPrivateRemote::setMediaSourceEnded(bool isEnded)
         return;
 
     gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::SetMediaSourceEnded(isEnded), m_remoteSourceBufferIdentifier);
+    SourceBufferPrivate::setMediaSourceEnded(isEnded);
 }
 
 void SourceBufferPrivateRemote::setMode(SourceBufferAppendMode mode)
@@ -196,53 +186,43 @@ void SourceBufferPrivateRemote::setMode(SourceBufferAppendMode mode)
     gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::SetMode(mode), m_remoteSourceBufferIdentifier);
 }
 
-void SourceBufferPrivateRemote::clientReadyStateChanged(bool sourceIsEnded)
+Ref<MediaPromise> SourceBufferPrivateRemote::removeCodedFrames(const MediaTime& start, const MediaTime& end, const MediaTime& currentMediaTime)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
-    if (isGPURunning())
-        gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::ClientReadyStateChanged(sourceIsEnded), m_remoteSourceBufferIdentifier);
-    SourceBufferPrivate::updateBufferedFromTrackBuffers(m_trackBufferRanges, sourceIsEnded);
+    if (!isGPURunning())
+        return MediaPromise::createAndReject(PlatformMediaError::IPCError);
+
+    return gpuProcessConnection->connection().sendWithPromisedReply(
+        Messages::RemoteSourceBufferProxy::RemoveCodedFrames(start, end, currentMediaTime), m_remoteSourceBufferIdentifier)->whenSettled(RunLoop::current(), [] (auto&& result) mutable {
+            if (!result)
+                return MediaPromise::createAndReject(PlatformMediaError::IPCError);
+            return MediaPromise::createAndResolve();
+        });
 }
 
-void SourceBufferPrivateRemote::removeCodedFrames(const MediaTime& start, const MediaTime& end, const MediaTime& currentMediaTime, bool isEnded, CompletionHandler<void()>&& completionHandler)
-{
-    auto gpuProcessConnection = m_gpuProcessConnection.get();
-    if (!isGPURunning()) {
-        completionHandler();
-        return;
-    }
-
-    gpuProcessConnection->connection().sendWithAsyncReply(
-        Messages::RemoteSourceBufferProxy::RemoveCodedFrames(start, end, currentMediaTime, isEnded), [this, protectedThis = Ref { *this }, completionHandler = WTFMove(completionHandler)](auto&& buffered, uint64_t totalTrackBufferSizeInBytes) mutable {
-            m_totalTrackBufferSizeInBytes = totalTrackBufferSizeInBytes;
-            setBufferedRanges(WTFMove(buffered));
-            completionHandler();
-        },
-        m_remoteSourceBufferIdentifier);
-}
-
-void SourceBufferPrivateRemote::evictCodedFrames(uint64_t newDataSize, uint64_t maximumBufferSize, const MediaTime& currentTime, bool isEnded)
+void SourceBufferPrivateRemote::evictCodedFrames(uint64_t newDataSize, uint64_t maximumBufferSize, const MediaTime& currentTime)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!isGPURunning())
         return;
 
-    auto sendResult = gpuProcessConnection->connection().sendSync(Messages::RemoteSourceBufferProxy::EvictCodedFrames(newDataSize, maximumBufferSize, currentTime, isEnded), m_remoteSourceBufferIdentifier);
+    auto sendResult = gpuProcessConnection->connection().sendSync(Messages::RemoteSourceBufferProxy::EvictCodedFrames(newDataSize, maximumBufferSize, currentTime), m_remoteSourceBufferIdentifier);
     if (sendResult.succeeded()) {
-        PlatformTimeRanges buffered;
-        std::tie(buffered, m_totalTrackBufferSizeInBytes) = sendResult.takeReply();
-        setBufferedRanges(WTFMove(buffered));
+        if (RefPtr client = this->client()) {
+            Vector<PlatformTimeRanges> trackBufferRanges;
+            std::tie(trackBufferRanges, m_totalTrackBufferSizeInBytes) = sendResult.takeReply();
+            client->sourceBufferPrivateBufferedChanged(trackBufferRanges, m_totalTrackBufferSizeInBytes);
+        }
     }
 }
 
-void SourceBufferPrivateRemote::addTrackBuffer(const AtomString& trackId, RefPtr<MediaDescription>&&)
+void SourceBufferPrivateRemote::addTrackBuffer(TrackID trackId, RefPtr<MediaDescription>&&)
 {
-    ASSERT(m_trackIdentifierMap.contains(trackId));
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!isGPURunning())
         return;
 
-    gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::AddTrackBuffer(m_trackIdentifierMap.get(trackId)), m_remoteSourceBufferIdentifier);
+    gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::AddTrackBuffer(trackId), m_remoteSourceBufferIdentifier);
 }
 
 void SourceBufferPrivateRemote::resetTrackBuffers()
@@ -355,158 +335,137 @@ void SourceBufferPrivateRemote::setAppendWindowEnd(const MediaTime& appendWindow
     gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::SetAppendWindowEnd(appendWindowEnd), m_remoteSourceBufferIdentifier);
 }
 
-void SourceBufferPrivateRemote::seekToTime(const MediaTime& mediaTime)
+Ref<SourceBufferPrivate::ComputeSeekPromise> SourceBufferPrivateRemote::computeSeekTime(const WebCore::SeekTarget& target)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!isGPURunning())
-        return;
+        return ComputeSeekPromise::createAndReject(PlatformMediaError::IPCError);
 
-    gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::SeekToTime(mediaTime), m_remoteSourceBufferIdentifier);
-}
-
-void SourceBufferPrivateRemote::updateTrackIds(Vector<std::pair<AtomString, AtomString>>&& trackIdPairs)
-{
-    auto gpuProcessConnection = m_gpuProcessConnection.get();
-    if (!isGPURunning())
-        return;
-
-    auto identifierPairs = trackIdPairs.map([this](auto& trackIdPair) {
-        ASSERT(m_prevTrackIdentifierMap.contains(trackIdPair.first));
-        ASSERT(m_trackIdentifierMap.contains(trackIdPair.second));
-        return std::pair { m_prevTrackIdentifierMap.take(trackIdPair.first), m_trackIdentifierMap.get(trackIdPair.second) };
+    return gpuProcessConnection->connection().sendWithPromisedReply(Messages::RemoteSourceBufferProxy::ComputeSeekTime(target), m_remoteSourceBufferIdentifier)->whenSettled(RunLoop::current(), [](auto&& result) {
+        return result ? ComputeSeekPromise::createAndSettle(*result) : ComputeSeekPromise::createAndReject(PlatformMediaError::IPCError);
     });
-
-    gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::UpdateTrackIds(identifierPairs), m_remoteSourceBufferIdentifier);
 }
 
-void SourceBufferPrivateRemote::bufferedSamplesForTrackId(const AtomString& trackId, CompletionHandler<void(Vector<String>&&)>&& completionHandler)
+void SourceBufferPrivateRemote::seekToTime(const MediaTime& time)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
-    if (!isGPURunning()) {
-        completionHandler({ });
+    if (!isGPURunning())
         return;
-    }
 
-    gpuProcessConnection->connection().sendWithAsyncReply(Messages::RemoteSourceBufferProxy::BufferedSamplesForTrackId(m_trackIdentifierMap.get(trackId)), [completionHandler = WTFMove(completionHandler)](auto&& samples) mutable {
-        completionHandler(WTFMove(samples));
-    }, m_remoteSourceBufferIdentifier);
+    gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::SeekToTime(time), m_remoteSourceBufferIdentifier);
 }
 
-void SourceBufferPrivateRemote::enqueuedSamplesForTrackID(const AtomString& trackId, CompletionHandler<void(Vector<String>&&)>&& completionHandler)
+void SourceBufferPrivateRemote::updateTrackIds(Vector<std::pair<TrackID, TrackID>>&& trackIdPairs)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
-    if (!isGPURunning()) {
-        completionHandler({ });
+    if (!isGPURunning())
         return;
-    }
 
-    gpuProcessConnection->connection().sendWithAsyncReply(Messages::RemoteSourceBufferProxy::EnqueuedSamplesForTrackID(m_trackIdentifierMap.get(trackId)), [completionHandler = WTFMove(completionHandler)](auto&& samples) mutable {
-        completionHandler(WTFMove(samples));
-    }, m_remoteSourceBufferIdentifier);
+    gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::UpdateTrackIds(WTFMove(trackIdPairs)), m_remoteSourceBufferIdentifier);
 }
 
-void SourceBufferPrivateRemote::sourceBufferPrivateDidReceiveInitializationSegment(InitializationSegmentInfo&& segmentInfo, CompletionHandler<void(WebCore::SourceBufferPrivateClient::ReceiveResult)>&& completionHandler)
+Ref<SourceBufferPrivate::SamplesPromise> SourceBufferPrivateRemote::bufferedSamplesForTrackId(TrackID trackId)
 {
-    if (!m_client || !m_mediaPlayerPrivate) {
-        completionHandler(WebCore::SourceBufferPrivateClient::ReceiveResult::ClientDisconnected);
+    auto gpuProcessConnection = m_gpuProcessConnection.get();
+    if (!isGPURunning())
+        return SamplesPromise::createAndResolve(Vector<String> { });
+
+    return gpuProcessConnection->connection().sendWithPromisedReply(Messages::RemoteSourceBufferProxy::BufferedSamplesForTrackId(trackId), m_remoteSourceBufferIdentifier)->whenSettled(RunLoop::current(), [](auto&& result) {
+        if (!result)
+            return SamplesPromise::createAndResolve(Vector<String> { });
+        return SamplesPromise::createAndSettle(WTFMove(*result));
+    });
+}
+
+Ref<SourceBufferPrivate::SamplesPromise> SourceBufferPrivateRemote::enqueuedSamplesForTrackID(TrackID trackId)
+{
+    auto gpuProcessConnection = m_gpuProcessConnection.get();
+    if (!isGPURunning())
+        return SamplesPromise::createAndResolve(Vector<String> { });
+
+    return gpuProcessConnection->connection().sendWithPromisedReply(Messages::RemoteSourceBufferProxy::EnqueuedSamplesForTrackID(trackId), m_remoteSourceBufferIdentifier)->whenSettled(RunLoop::current(), [](auto&& result) {
+        if (!result)
+            return SamplesPromise::createAndResolve(Vector<String> { });
+        return SamplesPromise::createAndSettle(WTFMove(*result));
+    });
+}
+
+void SourceBufferPrivateRemote::sourceBufferPrivateDidReceiveInitializationSegment(InitializationSegmentInfo&& segmentInfo, CompletionHandler<void(WebCore::MediaPromise::Result&&)>&& completionHandler)
+{
+    RefPtr client = this->client();
+    if (!client || !m_mediaPlayerPrivate) {
+        completionHandler(makeUnexpected(WebCore::PlatformMediaError::ClientDisconnected));
         return;
     }
 
     SourceBufferPrivateClient::InitializationSegment segment;
     segment.duration = segmentInfo.duration;
 
-    m_prevTrackIdentifierMap.swap(m_trackIdentifierMap);
-    segment.audioTracks.reserveInitialCapacity(segmentInfo.audioTracks.size());
-    for (auto& audioTrack : segmentInfo.audioTracks) {
-        SourceBufferPrivateClient::InitializationSegment::AudioTrackInformation info;
-        info.track = m_mediaPlayerPrivate->audioTrackPrivateRemote(audioTrack.identifier);
-        info.description = RemoteMediaDescription::create(audioTrack.description);
-        segment.audioTracks.uncheckedAppend(info);
-        m_trackIdentifierMap.add(info.track->id(), audioTrack.identifier);
-    }
+    segment.audioTracks = WTF::map(segmentInfo.audioTracks, [&](auto& audioTrack) {
+        SourceBufferPrivateClient::InitializationSegment::AudioTrackInformation info {
+            RemoteMediaDescription::create(audioTrack.description),
+            m_mediaPlayerPrivate->audioTrackPrivateRemote(audioTrack.id)
+        };
+        return info;
+    });
 
-    segment.videoTracks.reserveInitialCapacity(segmentInfo.videoTracks.size());
-    for (auto& videoTrack : segmentInfo.videoTracks) {
-        SourceBufferPrivateClient::InitializationSegment::VideoTrackInformation info;
-        info.track = m_mediaPlayerPrivate->videoTrackPrivateRemote(videoTrack.identifier);
-        info.description = RemoteMediaDescription::create(videoTrack.description);
-        segment.videoTracks.uncheckedAppend(info);
-        m_trackIdentifierMap.add(info.track->id(), videoTrack.identifier);
-    }
+    segment.videoTracks = WTF::map(segmentInfo.videoTracks, [&](auto& videoTrack) {
+        SourceBufferPrivateClient::InitializationSegment::VideoTrackInformation info {
+            RemoteMediaDescription::create(videoTrack.description),
+            m_mediaPlayerPrivate->videoTrackPrivateRemote(videoTrack.id)
+        };
+        return info;
+    });
 
-    segment.textTracks.reserveInitialCapacity(segmentInfo.textTracks.size());
-    for (auto& textTrack : segmentInfo.textTracks) {
-        SourceBufferPrivateClient::InitializationSegment::TextTrackInformation info;
-        info.track = m_mediaPlayerPrivate->textTrackPrivateRemote(textTrack.identifier);
-        info.description = RemoteMediaDescription::create(textTrack.description);
-        segment.textTracks.uncheckedAppend(info);
-        m_trackIdentifierMap.add(info.track->id(), textTrack.identifier);
-    }
+    segment.textTracks = WTF::map(segmentInfo.textTracks, [&](auto& textTrack) {
+        SourceBufferPrivateClient::InitializationSegment::TextTrackInformation info {
+            RemoteMediaDescription::create(textTrack.description),
+            m_mediaPlayerPrivate->textTrackPrivateRemote(textTrack.id)
+        };
+        return info;
+    });
 
-    m_client->sourceBufferPrivateDidReceiveInitializationSegment(WTFMove(segment), WTFMove(completionHandler));
-}
-
-void SourceBufferPrivateRemote::sourceBufferPrivateStreamEndedWithDecodeError()
-{
-    if (m_client)
-        m_client->sourceBufferPrivateStreamEndedWithDecodeError();
-}
-
-void SourceBufferPrivateRemote::sourceBufferPrivateAppendComplete(SourceBufferPrivateClient::AppendResult appendResult, uint64_t totalTrackBufferSizeInBytes, const MediaTime& timestampOffset)
-{
-    m_totalTrackBufferSizeInBytes = totalTrackBufferSizeInBytes;
-    if (m_client) {
-        setTimestampOffset(timestampOffset);
-        m_client->sourceBufferPrivateAppendComplete(appendResult);
-    }
+    client->sourceBufferPrivateDidReceiveInitializationSegment(WTFMove(segment))->whenSettled(RunLoop::current(), WTFMove(completionHandler));
 }
 
 void SourceBufferPrivateRemote::sourceBufferPrivateHighestPresentationTimestampChanged(const MediaTime& timestamp)
 {
-    if (m_client)
-        m_client->sourceBufferPrivateHighestPresentationTimestampChanged(timestamp);
+    if (RefPtr client = this->client())
+        client->sourceBufferPrivateHighestPresentationTimestampChanged(timestamp);
 }
 
 void SourceBufferPrivateRemote::sourceBufferPrivateDurationChanged(const MediaTime& duration, CompletionHandler<void()>&& completionHandler)
 {
-    if (m_client)
-        m_client->sourceBufferPrivateDurationChanged(duration, WTFMove(completionHandler));
+    if (RefPtr client = this->client())
+        client->sourceBufferPrivateDurationChanged(duration)->whenSettled(RunLoop::current(), WTFMove(completionHandler));
     else
         completionHandler();
 }
 
-void SourceBufferPrivateRemote::sourceBufferPrivateBufferedChanged(WebCore::PlatformTimeRanges&& timeRanges, CompletionHandler<void()>&& completionHandler)
+void SourceBufferPrivateRemote::sourceBufferPrivateBufferedChanged(Vector<WebCore::PlatformTimeRanges>&& trackBuffersRanges, uint64_t extraMemory, CompletionHandler<void()>&& completionHandler)
 {
-    setBufferedRanges(WTFMove(timeRanges), WTFMove(completionHandler));
-}
-
-void SourceBufferPrivateRemote::sourceBufferPrivateTrackBuffersChanged(Vector<WebCore::PlatformTimeRanges>&& trackBuffersRanges)
-{
-    m_trackBufferRanges = WTFMove(trackBuffersRanges);
+    if (RefPtr client = this->client())
+        client->sourceBufferPrivateBufferedChanged(WTFMove(trackBuffersRanges), extraMemory)->whenSettled(RunLoop::current(), WTFMove(completionHandler));
+    else
+        completionHandler();
 }
 
 void SourceBufferPrivateRemote::sourceBufferPrivateDidParseSample(double sampleDuration)
 {
-    if (m_client)
-        m_client->sourceBufferPrivateDidParseSample(sampleDuration);
+    if (RefPtr client = this->client())
+        client->sourceBufferPrivateDidParseSample(sampleDuration);
 }
 
 void SourceBufferPrivateRemote::sourceBufferPrivateDidDropSample()
 {
-    if (m_client)
-        m_client->sourceBufferPrivateDidDropSample();
+    if (RefPtr client = this->client())
+        client->sourceBufferPrivateDidDropSample();
 }
 
 void SourceBufferPrivateRemote::sourceBufferPrivateDidReceiveRenderingError(int64_t errorCode)
 {
-    if (m_client)
-        m_client->sourceBufferPrivateDidReceiveRenderingError(errorCode);
-}
-
-void SourceBufferPrivateRemote::sourceBufferPrivateReportExtraMemoryCost(uint64_t extraMemory)
-{
-    if (m_client)
-        m_client->sourceBufferPrivateReportExtraMemoryCost(extraMemory);
+    if (RefPtr client = this->client())
+        client->sourceBufferPrivateDidReceiveRenderingError(errorCode);
 }
 
 uint64_t SourceBufferPrivateRemote::totalTrackBufferSizeInBytes() const
@@ -514,21 +473,16 @@ uint64_t SourceBufferPrivateRemote::totalTrackBufferSizeInBytes() const
     return m_totalTrackBufferSizeInBytes;
 }
 
-void SourceBufferPrivateRemote::memoryPressure(uint64_t maximumBufferSize, const MediaTime& currentTime, bool isEnded)
+void SourceBufferPrivateRemote::memoryPressure(uint64_t maximumBufferSize, const MediaTime& currentTime)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!isGPURunning())
         return;
 
-    gpuProcessConnection->connection().sendWithAsyncReply(
-        Messages::RemoteSourceBufferProxy::MemoryPressure(maximumBufferSize, currentTime, isEnded), [this, protectedThis = Ref { *this }](auto&& buffer, uint64_t totalTrackBufferSizeInBytes) mutable {
-            m_totalTrackBufferSizeInBytes = totalTrackBufferSizeInBytes;
-            setBufferedRanges(WTFMove(buffer));
-        },
-        m_remoteSourceBufferIdentifier);
+    gpuProcessConnection->connection().send(Messages::RemoteSourceBufferProxy::MemoryPressure(maximumBufferSize, currentTime), m_remoteSourceBufferIdentifier);
 }
 
-MediaTime SourceBufferPrivateRemote::minimumUpcomingPresentationTimeForTrackID(const AtomString& trackID)
+MediaTime SourceBufferPrivateRemote::minimumUpcomingPresentationTimeForTrackID(TrackID trackID)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!isGPURunning())
@@ -538,7 +492,7 @@ MediaTime SourceBufferPrivateRemote::minimumUpcomingPresentationTimeForTrackID(c
     return std::get<0>(sendResult.takeReplyOr(MediaTime::invalidTime()));
 }
 
-void SourceBufferPrivateRemote::setMaximumQueueDepthForTrackID(const AtomString& trackID, uint64_t depth)
+void SourceBufferPrivateRemote::setMaximumQueueDepthForTrackID(TrackID trackID, uint64_t depth)
 {
     auto gpuProcessConnection = m_gpuProcessConnection.get();
     if (!isGPURunning())
