@@ -25,26 +25,19 @@
 
 #pragma once
 
-#if ENABLE(PDFKIT_PLUGIN)
+#if ENABLE(LEGACY_PDFKIT_PLUGIN)
 
-#include "DataReference.h"
-#include "PDFPluginIdentifier.h"
+#include "PDFPluginBase.h"
 #include "WebMouseEvent.h"
-#include <WebCore/AffineTransform.h>
-#include <WebCore/FindOptions.h>
 #include <WebCore/NetscapePlugInStreamLoader.h>
-#include <WebCore/ScrollableArea.h>
 #include <wtf/HashMap.h>
 #include <wtf/Identified.h>
 #include <wtf/Range.h>
 #include <wtf/RangeSet.h>
 #include <wtf/RefCounted.h>
-#include <wtf/RetainPtr.h>
-#include <wtf/ThreadSafeRefCounted.h>
 #include <wtf/Threading.h>
 #include <wtf/URL.h>
 #include <wtf/Vector.h>
-#include <wtf/WeakPtr.h>
 
 typedef struct objc_object* id;
 
@@ -75,7 +68,6 @@ class FloatSize;
 class FragmentedSharedBuffer;
 class GraphicsContext;
 class HTMLPlugInElement;
-class Scrollbar;
 struct PluginInfo;
 }
 
@@ -88,20 +80,14 @@ class ShareableBitmap;
 class WebFrame;
 class WebKeyboardEvent;
 class WebWheelEvent;
-
-struct FrameInfoData;
 struct WebHitTestResultData;
 
-class PDFPlugin final : public ThreadSafeRefCounted<PDFPlugin>, public WebCore::ScrollableArea {
+class PDFPlugin final : public PDFPluginBase {
 public:
+    static bool pdfKitLayerControllerIsAvailable();
+
     static Ref<PDFPlugin> create(WebCore::HTMLPlugInElement&);
-    ~PDFPlugin();
-
-    bool isBeingDestroyed() const { return m_isBeingDestroyed; }
-
-    static WebCore::PluginInfo pluginInfo();
-
-    WebCore::IntSize size() const { return m_size; }
+    virtual ~PDFPlugin() = default;
 
     void didMutatePDFDocument() { m_pdfDocumentWasMutated = true; }
 
@@ -112,13 +98,14 @@ public:
     void notifyDisplayModeChanged(int);
 
     void notifySelectionChanged(PDFSelection *);
-    void notifyCursorChanged(uint64_t /* PDFLayerControllerCursorType */);
 
-    void zoomIn();
-    void zoomOut();
-    void save(CompletionHandler<void(const String&, const URL&, const IPC::DataReference&)>&&);
-    void openWithPreview(CompletionHandler<void(const String&, FrameInfoData&&, const IPC::DataReference&, const String&)>&&);
-    PDFPluginIdentifier identifier() const { return m_identifier; }
+    // HUD Actions.
+#if ENABLE(PDF_HUD)
+    void zoomIn() final;
+    void zoomOut() final;
+    void save(CompletionHandler<void(const String&, const URL&, const IPC::DataReference&)>&&) final;
+    void openWithPreview(CompletionHandler<void(const String&, FrameInfoData&&, const IPC::DataReference&, const String&)>&&) final;
+#endif
 
     void clickedLink(NSURL *);
 
@@ -132,18 +119,8 @@ public:
 
     void attemptToUnlockPDF(const String& password);
 
-    WebCore::FloatRect convertFromPDFViewToScreen(const WebCore::FloatRect&) const;
-    WebCore::IntPoint convertFromRootViewToPDFView(const WebCore::IntPoint&) const;
-    WebCore::IntRect boundsOnScreen() const;
-
     bool showContextMenuAtPoint(const WebCore::IntPoint&);
 
-    std::tuple<String, PDFSelection *, NSDictionary *> lookupTextAtLocation(const WebCore::FloatPoint&, WebHitTestResultData&) const;
-    WebCore::FloatRect rectForSelectionInRootView(PDFSelection *) const;
-
-    CGFloat scaleFactor() const;
-    float deviceScaleFactor() const;
-
     PDFPluginAnnotation* activeAnnotation() const { return m_activeAnnotation.get(); }
     WebCore::AXObjectCache* axObjectCache() const;
 
@@ -160,123 +137,99 @@ public:
     size_t decrementThreadsWaitingOnCallback() { return --m_threadsWaitingOnCallback; }
 #endif
 
-    void setView(PluginView&);
-    void destroy();
-    void updateControlTints(WebCore::GraphicsContext&);
-    RefPtr<ShareableBitmap> snapshot();
-    CALayer *pluginLayer();
-    void geometryDidChange(const WebCore::IntSize& pluginSize, const WebCore::AffineTransform& pluginToRootViewTransform);
-    void contentsScaleFactorChanged(float);
-    void visibilityDidChange(bool);
-    void streamDidReceiveResponse(const WebCore::ResourceResponse&);
-    void streamDidReceiveData(const WebCore::SharedBuffer&);
-    void streamDidFinishLoading();
-    void streamDidFail();
-    bool handleMouseEvent(const WebMouseEvent&);
-    bool handleWheelEvent(const WebWheelEvent&);
-    bool handleMouseEnterEvent(const WebMouseEvent&);
-    bool handleMouseLeaveEvent(const WebMouseEvent&);
-    bool handleContextMenuEvent(const WebMouseEvent&);
-    bool handleKeyboardEvent(const WebKeyboardEvent&);
-    bool handleEditingCommand(StringView commandName);
-    bool isEditingCommandEnabled(StringView commandName);
-    WebCore::ScrollPosition scrollPositionForTesting() const { return scrollPosition(); }
-    WebCore::Scrollbar* horizontalScrollbar() { return m_horizontalScrollbar.get(); }
-    WebCore::Scrollbar* verticalScrollbar() { return m_verticalScrollbar.get(); }
-    RefPtr<WebCore::FragmentedSharedBuffer> liveResourceData() const;
-    void willDetachRenderer();
-
-    RetainPtr<PDFDocument> pdfDocumentForPrinting() const { return m_pdfDocument; }
-    WebCore::FloatSize pdfDocumentSizeForPrinting() const;
-    id accessibilityHitTest(const WebCore::IntPoint&) const;
-    id accessibilityObject() const;
-    id accessibilityAssociatedPluginParentForElement(WebCore::Element*) const;
-
-    unsigned countFindMatches(const String& target, WebCore::FindOptions, unsigned maxMatchCount);
-    bool findString(const String& target, WebCore::FindOptions, unsigned maxMatchCount);
-
-    bool performDictionaryLookupAtLocation(const WebCore::FloatPoint&);
-    String getSelectionString() const;
-    bool existingSelectionContainsPoint(const WebCore::FloatPoint&) const;
-
-    bool isFullFramePlugin() const;
+    WebCore::IntPoint convertFromPluginToPDFView(const WebCore::IntPoint&) const;
+    WebCore::IntPoint convertFromPDFViewToRootView(const WebCore::IntPoint&) const;
+    WebCore::IntRect convertFromPDFViewToRootView(const WebCore::IntRect&) const;
+    WebCore::IntPoint convertFromRootViewToPDFView(const WebCore::IntPoint&) const;
+    WebCore::FloatRect convertFromPDFViewToScreen(const WebCore::FloatRect&) const;
 
 private:
     explicit PDFPlugin(WebCore::HTMLPlugInElement&);
+    bool isLegacyPDFPlugin() const override { return true; }
 
     PDFSelection *nextMatchForString(const String& target, bool searchForward, bool caseSensitive, bool wrapSearch, PDFSelection *initialSelection, bool startInSelection);
 
-    // ScrollableArea functions.
-    WebCore::IntRect scrollCornerRect() const final;
-    WebCore::ScrollableArea* enclosingScrollableArea() const final;
-    bool isScrollableOrRubberbandable() final { return true; }
-    bool hasScrollableOrRubberbandableAncestor() final { return true; }
-    WebCore::IntRect scrollableAreaBoundingBox(bool* = nullptr) const final;
-    void setScrollOffset(const WebCore::ScrollOffset&) final;
-    void invalidateScrollbarRect(WebCore::Scrollbar&, const WebCore::IntRect&) final;
-    void invalidateScrollCornerRect(const WebCore::IntRect&) final;
-    WebCore::IntPoint lastKnownMousePositionInView() const final { return m_lastMousePositionInPluginCoordinates; }
-    bool isActive() const final;
-    bool isScrollCornerVisible() const final { return false; }
-    WebCore::ScrollPosition scrollPosition() const final;
-    WebCore::ScrollPosition minimumScrollPosition() const final;
-    WebCore::ScrollPosition maximumScrollPosition() const final;
-    WebCore::IntSize visibleSize() const final { return m_size; }
-    WebCore::IntSize contentsSize() const final { return m_pdfDocumentSize; }
-    WebCore::Scrollbar* horizontalScrollbar() const final { return m_horizontalScrollbar.get(); }
-    WebCore::Scrollbar* verticalScrollbar() const final { return m_verticalScrollbar.get(); }
-    bool shouldSuspendScrollAnimations() const final { return false; } // If we return true, ScrollAnimatorMac will keep cycling a timer forever, waiting for a good time to animate.
-    void scrollbarStyleChanged(WebCore::ScrollbarStyle, bool forceUpdate) final;
-    WebCore::IntRect convertFromScrollbarToContainingView(const WebCore::Scrollbar&, const WebCore::IntRect& scrollbarRect) const final;
-    WebCore::IntRect convertFromContainingViewToScrollbar(const WebCore::Scrollbar&, const WebCore::IntRect& parentRect) const final;
-    WebCore::IntPoint convertFromScrollbarToContainingView(const WebCore::Scrollbar&, const WebCore::IntPoint& scrollbarPoint) const final;
-    WebCore::IntPoint convertFromContainingViewToScrollbar(const WebCore::Scrollbar&, const WebCore::IntPoint& parentPoint) const final;
-    bool forceUpdateScrollbarsOnMainThreadForPerformanceTesting() const final;
-    bool shouldPlaceVerticalScrollbarOnLeft() const final { return false; }
-    String debugDescription() const final;
+    void didChangeScrollOffset() override;
 
-    void updateScrollbars();
-    Ref<WebCore::Scrollbar> createScrollbar(WebCore::ScrollbarOrientation);
-    void destroyScrollbar(WebCore::ScrollbarOrientation);
-    void installPDFDocument();
-    void addArchiveResource();
-    void calculateSizes();
-    void tryRunScriptsInPDFDocument();
+    void invalidateScrollbarRect(WebCore::Scrollbar&, const WebCore::IntRect&) override;
+    void invalidateScrollCornerRect(const WebCore::IntRect&) override;
+    void updateScrollbars() override;
+    Ref<WebCore::Scrollbar> createScrollbar(WebCore::ScrollbarOrientation) override;
+    void destroyScrollbar(WebCore::ScrollbarOrientation) override;
+
+    // PDFPluginBase
+    WebCore::PluginLayerHostingStrategy layerHostingStrategy() const override { return WebCore::PluginLayerHostingStrategy::PlatformLayer; }
+    PlatformLayer* platformLayer() const override;
+
+    void setView(PluginView&) override;
+    void teardown() override;
+    bool isComposited() const override { return true; }
+
+    void installPDFDocument() override;
+    void tryRunScriptsInPDFDocument() override;
+
+    CGFloat scaleFactor() const override;
+
+    RetainPtr<PDFDocument> pdfDocumentForPrinting() const override { return m_pdfDocument; }
+    WebCore::FloatSize pdfDocumentSizeForPrinting() const override;
+
+    void geometryDidChange(const WebCore::IntSize& pluginSize, const WebCore::AffineTransform& pluginToRootViewTransform) override;
+    void deviceScaleFactorChanged(float) override;
+
+    void setPageScaleFactor(double, std::optional<WebCore::IntPoint> origin) override;
+
+    WebCore::IntSize contentsSize() const override;
+    unsigned firstPageHeight() const override;
+
+    RefPtr<WebCore::FragmentedSharedBuffer> liveResourceData() const override;
+
+    bool wantsWheelEvents() const override { return true; }
+    bool handleMouseEvent(const WebMouseEvent&) override;
+    bool handleWheelEvent(const WebWheelEvent&) override;
+    bool handleMouseEnterEvent(const WebMouseEvent&) override;
+    bool handleMouseLeaveEvent(const WebMouseEvent&) override;
+    bool handleContextMenuEvent(const WebMouseEvent&) override;
+    bool handleKeyboardEvent(const WebKeyboardEvent&) override;
+    bool handleEditingCommand(StringView commandName) override;
+    bool isEditingCommandEnabled(StringView commandName) override;
+
+    String getSelectionString() const override;
+    bool existingSelectionContainsPoint(const WebCore::FloatPoint&) const override;
+    WebCore::FloatRect rectForSelectionInRootView(PDFSelection *) const override;
+
+    unsigned countFindMatches(const String& target, WebCore::FindOptions, unsigned maxMatchCount) override;
+    bool findString(const String& target, WebCore::FindOptions, unsigned maxMatchCount) override;
+
+    bool performDictionaryLookupAtLocation(const WebCore::FloatPoint&) override;
+    std::tuple<String, PDFSelection *, NSDictionary *> lookupTextAtLocation(const WebCore::FloatPoint&, WebHitTestResultData&) const override;
+
+    bool shouldCreateTransientPaintingSnapshot() const override { return true; }
+    RefPtr<ShareableBitmap> snapshot() override;
+
+    id accessibilityHitTest(const WebCore::IntPoint&) const override;
+    id accessibilityObject() const override;
+    id accessibilityAssociatedPluginParentForElement(WebCore::Element*) const override;
+
+    void incrementalPDFStreamDidReceiveData(const WebCore::SharedBuffer&) override;
+    bool incrementalPDFStreamDidFinishLoading() override;
+    void incrementalPDFStreamDidFail() override;
 
     NSEvent *nsEventForWebMouseEvent(const WebMouseEvent&);
-    WebCore::IntPoint convertFromPluginToPDFView(const WebCore::IntPoint&) const;
-    WebCore::IntPoint convertFromRootViewToPlugin(const WebCore::IntPoint&) const;
-    WebCore::IntPoint convertFromPDFViewToRootView(const WebCore::IntPoint&) const;
-    WebCore::IntRect convertFromPDFViewToRootView(const WebCore::IntRect&) const;
-    WebCore::IntRect frameForHUD() const;
-    void ensureDataBufferLength(uint64_t length);
 
     bool supportsForms();
 
-    bool handlesPageScaleFactor() const;
     void updatePageAndDeviceScaleFactors();
 
     void createPasswordEntryForm();
 
-    WebCore::IntSize pdfDocumentSize() const { return m_pdfDocumentSize; }
-    void setPDFDocumentSize(WebCore::IntSize size) { m_pdfDocumentSize = size; }
-
-#ifdef __OBJC__
     NSData *liveData() const;
-    NSData *rawData() const { return (__bridge NSData *)m_data.get(); }
-#endif
-
     JSObjectRef makeJSPDFDoc(JSContextRef);
     static JSClassRef jsPDFDocClass();
     static JSValueRef jsPDFDocPrint(JSContextRef, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception);
 
-    WeakPtr<PluginView> m_view;
-    WeakPtr<WebFrame> m_frame;
 
     bool m_pdfDocumentWasMutated { false };
 
-    WebCore::IntSize m_scrollOffset;
-
     RetainPtr<CALayer> m_containerLayer;
     RetainPtr<CALayer> m_contentLayer;
     RetainPtr<CALayer> m_horizontalScrollbarLayer;
@@ -289,35 +242,15 @@ private:
     RefPtr<PDFPluginPasswordField> m_passwordField;
     RefPtr<WebCore::Element> m_annotationContainer;
 
-    WebCore::AffineTransform m_rootViewToPluginTransform;
-    WebMouseEvent m_lastMouseEvent;
-    WebCore::IntPoint m_lastMousePositionInPluginCoordinates;
+    std::optional<WebMouseEvent> m_lastMouseEvent;
 
     String m_temporaryPDFUUID;
-
     String m_lastFoundString;
 
     RetainPtr<WKPDFLayerControllerDelegate> m_pdfLayerControllerDelegate;
 
-    WebCore::IntSize m_size;
-
     URL m_sourceURL;
 
-    String m_suggestedFilename;
-    RetainPtr<CFMutableDataRef> m_data;
-    uint64_t m_streamedBytes { 0 };
-
-    RetainPtr<PDFDocument> m_pdfDocument;
-
-    bool m_documentFinishedLoading { false };
-    bool m_isBeingDestroyed { false };
-    bool m_hasBeenDestroyed { false };
-    unsigned m_firstPageHeight { 0 };
-    WebCore::IntSize m_pdfDocumentSize; // All pages, including gaps.
-
-    RefPtr<WebCore::Scrollbar> m_horizontalScrollbar;
-    RefPtr<WebCore::Scrollbar> m_verticalScrollbar;
-
 #if HAVE(INCREMENTAL_PDF_APIS)
     void threadEntry(Ref<PDFPlugin>&&);
     void adoptBackgroundThreadDocument();
@@ -401,10 +334,12 @@ private:
 #endif
 
 #endif // HAVE(INCREMENTAL_PDF_APIS)
-
-    PDFPluginIdentifier m_identifier;
 };
 
 } // namespace WebKit
 
-#endif // ENABLE(PDFKIT_PLUGIN)
+SPECIALIZE_TYPE_TRAITS_BEGIN(WebKit::PDFPlugin)
+    static bool isType(const WebKit::PDFPluginBase& plugin) { return plugin.isLegacyPDFPlugin(); }
+SPECIALIZE_TYPE_TRAITS_END()
+
+#endif // ENABLE(LEGACY_PDFKIT_PLUGIN)
