@@ -29,6 +29,7 @@
 #if ENABLE(DRAG_SUPPORT)
 
 #include "ArgumentCodersGtk.h"
+#include "MessageSenderInlines.h"
 #include "ShareableBitmap.h"
 #include "WebPage.h"
 #include "WebPageProxyMessages.h"
@@ -49,7 +50,7 @@ static RefPtr<ShareableBitmap> convertCairoSurfaceToShareableBitmap(cairo_surfac
         return nullptr;
 
     IntSize imageSize(cairo_image_surface_get_width(surface), cairo_image_surface_get_height(surface));
-    auto bitmap = ShareableBitmap::create(imageSize, { });
+    auto bitmap = ShareableBitmap::create({ imageSize });
     auto graphicsContext = bitmap->createGraphicsContext();
 
     ASSERT(graphicsContext->hasPlatformContext());
@@ -62,19 +63,22 @@ void WebDragClient::didConcludeEditDrag()
 {
 }
 
-void WebDragClient::startDrag(DragItem item, DataTransfer& dataTransfer, Frame&)
+void WebDragClient::startDrag(DragItem dragItem, DataTransfer& dataTransfer, Frame&)
 {
-    auto& dragImage = item.image;
-    RefPtr<ShareableBitmap> bitmap = convertCairoSurfaceToShareableBitmap(dragImage.get().get());
-    ShareableBitmap::Handle handle;
+    auto& dragImage = dragItem.image;
+    auto bitmap = convertCairoSurfaceToShareableBitmap(dragImage.get().get());
+    std::optional<ShareableBitmap::Handle> handle;
 
-    // If we have a bitmap, but cannot create a handle to it, we fail early.
-    if (bitmap && !bitmap->createHandle(handle))
-        return;
+    if (bitmap) {
+        handle = bitmap->createHandle();
+
+        // If we have a bitmap, but cannot create a handle to it, we fail early.
+        if (!handle)
+            return;
+    }
 
     m_page->willStartDrag();
-
-    m_page->send(Messages::WebPageProxy::StartDrag(dataTransfer.pasteboard().selectionData(), dataTransfer.sourceOperationMask(), handle, dataTransfer.dragLocation()));
+    m_page->send(Messages::WebPageProxy::StartDrag(dataTransfer.pasteboard().selectionData(), dataTransfer.sourceOperationMask(), WTFMove(handle), dataTransfer.dragLocation()));
 }
 
 }; // namespace WebKit.
