@@ -30,19 +30,23 @@
 
 #include "DrawingAreaProxyMessages.h"
 #include "LayerTreeHost.h"
+#include "MessageSenderInlines.h"
 #include "ShareableBitmap.h"
 #include "UpdateInfo.h"
+#include "WebDisplayRefreshMonitor.h"
 #include "WebPage.h"
 #include "WebPageCreationParameters.h"
+#include "WebPageInlines.h"
 #include "WebPreferencesKeys.h"
 #include "WebProcess.h"
-#include <WebCore/Frame.h>
-#include <WebCore/FrameView.h>
 #include <WebCore/GraphicsContext.h>
+#include <WebCore/LocalFrame.h>
+#include <WebCore/LocalFrameView.h>
 #include <WebCore/Page.h>
 #include <WebCore/PageOverlayController.h>
 #include <WebCore/Region.h>
 #include <WebCore/Settings.h>
+#include <wtf/SetForScope.h>
 
 #if USE(GLIB_EVENT_LOOP)
 #include <wtf/glib/RunLoopSourcePriority.h>
@@ -53,24 +57,13 @@ using namespace WebCore;
 
 DrawingAreaCoordinatedGraphics::DrawingAreaCoordinatedGraphics(WebPage& webPage, const WebPageCreationParameters& parameters)
     : DrawingArea(DrawingAreaType::CoordinatedGraphics, parameters.drawingAreaIdentifier, webPage)
+    , m_isPaintingSuspended(!(parameters.activityState & ActivityState::IsVisible))
     , m_exitCompositingTimer(RunLoop::main(), this, &DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingMode)
-    , m_discardPreviousLayerTreeHostTimer(RunLoop::main(), this, &DrawingAreaCoordinatedGraphics::discardPreviousLayerTreeHost)
     , m_displayTimer(RunLoop::main(), this, &DrawingAreaCoordinatedGraphics::displayTimerFired)
 {
-#if USE(GLIB_EVENT_LOOP)
-    m_discardPreviousLayerTreeHostTimer.setPriority(RunLoopSourcePriority::ReleaseUnusedResourcesTimer);
-#if !PLATFORM(WPE)
+#if USE(GLIB_EVENT_LOOP) && !PLATFORM(WPE)
     m_displayTimer.setPriority(RunLoopSourcePriority::NonAcceleratedDrawingTimer);
 #endif
-#endif
-
-    updatePreferences(parameters.store);
-
-    if (m_alwaysUseCompositing) {
-        enterAcceleratedCompositingMode(nullptr);
-        if (!parameters.isProcessSwap)
-            sendEnterAcceleratedCompositingModeIfNeeded();
-    }
 }
 
 DrawingAreaCoordinatedGraphics::~DrawingAreaCoordinatedGraphics() = default;
@@ -82,7 +75,7 @@ void DrawingAreaCoordinatedGraphics::setNeedsDisplay()
         return;
     }
 
-    setNeedsDisplayInRect(m_webPage.bounds());
+    setNeedsDisplayInRect(m_webPage->bounds());
 }
 
 void DrawingAreaCoordinatedGraphics::setNeedsDisplayInRect(const IntRect& rect)
@@ -96,7 +89,7 @@ void DrawingAreaCoordinatedGraphics::setNeedsDisplayInRect(const IntRect& rect)
     }
 
     IntRect dirtyRect = rect;
-    dirtyRect.intersect(m_webPage.bounds());
+    dirtyRect.intersect(m_webPage->bounds());
     if (dirtyRect.isEmpty())
         return;
 
@@ -117,9 +110,6 @@ void DrawingAreaCoordinatedGraphics::scroll(const IntRect& scrollRect, const Int
     if (scrollRect.isEmpty())
         return;
 
-    if (m_previousLayerTreeHost)
-        m_previousLayerTreeHost->scrollNonCompositedContents(scrollRect);
-
     if (!m_scrollRect.isEmpty() && scrollRect != m_scrollRect) {
         unsigned scrollArea = scrollRect.width() * scrollRect.height();
         unsigned currentScrollArea = m_scrollRect.width() * m_scrollRect.height();
@@ -163,15 +153,12 @@ void DrawingAreaCoordinatedGraphics::scroll(const IntRect& scrollRect, const Int
 
 void DrawingAreaCoordinatedGraphics::forceRepaint()
 {
-    if (m_inUpdateBackingStoreState) {
-        m_forceRepaintAfterBackingStoreStateUpdate = true;
+    if (m_inUpdateGeometry)
         return;
-    }
-    m_forceRepaintAfterBackingStoreStateUpdate = false;
 
     if (!m_layerTreeHost) {
         m_isWaitingForDidUpdate = false;
-        m_dirtyRegion = m_webPage.bounds();
+        m_dirtyRegion = m_webPage->bounds();
         display();
         return;
     }
@@ -180,23 +167,12 @@ void DrawingAreaCoordinatedGraphics::forceRepaint()
         return;
 
     setNeedsDisplay();
-    m_webPage.layoutIfNeeded();
+    Ref { m_webPage.get() }->layoutIfNeeded();
     if (!m_layerTreeHost)
         return;
 
-    // FIXME: We need to do the same work as the layerHostDidFlushLayers function here,
-    // but clearly it doesn't make sense to call the function with that name.
-    // Consider refactoring and renaming it.
     if (m_compositingAccordingToProxyMessages)
         m_layerTreeHost->forceRepaint();
-    else {
-        // Call setShouldNotifyAfterNextScheduledLayerFlush(false) here to
-        // prevent layerHostDidFlushLayers() from being called a second time.
-        m_layerTreeHost->setShouldNotifyAfterNextScheduledLayerFlush(false);
-#if USE(COORDINATED_GRAPHICS)
-        layerHostDidFlushLayers();
-#endif
-    }
 }
 
 void DrawingAreaCoordinatedGraphics::forceRepaintAsync(WebPage& page, CompletionHandler<void()>&& completionHandler)
@@ -232,14 +208,7 @@ void DrawingAreaCoordinatedGraphics::setLayerTreeStateIsFrozen(bool isFrozen)
 
 void DrawingAreaCoordinatedGraphics::updatePreferences(const WebPreferencesStore& store)
 {
-    Settings& settings = m_webPage.corePage()->settings();
-#if PLATFORM(WAYLAND)
-    if (PlatformDisplay::sharedDisplay().type() == PlatformDisplay::Type::Wayland
-        && &PlatformDisplay::sharedDisplayForCompositing() == &PlatformDisplay::sharedDisplay()) {
-        // We failed to create the shared display for compositing, disable accelerated compositing.
-        settings.setAcceleratedCompositingEnabled(false);
-    }
-#endif
+    Settings& settings = m_webPage->corePage()->settings();
     settings.setForceCompositingMode(store.getBoolValueForKey(WebPreferencesKey::forceCompositingModeKey()));
     // Fixed position elements need to be composited and create stacking contexts
     // in order to be scrolled by the ScrollingCoordinator.
@@ -264,12 +233,10 @@ void DrawingAreaCoordinatedGraphics::updatePreferences(const WebPreferencesStore
     }
 }
 
-void DrawingAreaCoordinatedGraphics::mainFrameContentSizeChanged(const IntSize& size)
+void DrawingAreaCoordinatedGraphics::mainFrameContentSizeChanged(WebCore::FrameIdentifier, const IntSize& size)
 {
     if (m_layerTreeHost)
         m_layerTreeHost->contentsSizeChanged(size);
-    else if (m_previousLayerTreeHost)
-        m_previousLayerTreeHost->contentsSizeChanged(size);
 }
 
 #if USE(COORDINATED_GRAPHICS) || USE(TEXTURE_MAPPER)
@@ -277,19 +244,30 @@ void DrawingAreaCoordinatedGraphics::deviceOrPageScaleFactorChanged()
 {
     if (m_layerTreeHost)
         m_layerTreeHost->deviceOrPageScaleFactorChanged();
-    else if (m_previousLayerTreeHost)
-        m_previousLayerTreeHost->deviceOrPageScaleFactorChanged();
 }
 
 void DrawingAreaCoordinatedGraphics::didChangeViewportAttributes(ViewportAttributes&& attrs)
 {
     if (m_layerTreeHost)
         m_layerTreeHost->didChangeViewportAttributes(WTFMove(attrs));
-    else if (m_previousLayerTreeHost)
-        m_previousLayerTreeHost->didChangeViewportAttributes(WTFMove(attrs));
+}
+
+bool DrawingAreaCoordinatedGraphics::enterAcceleratedCompositingModeIfNeeded()
+{
+    ASSERT(!m_layerTreeHost);
+    if (!m_alwaysUseCompositing)
+        return false;
+
+    enterAcceleratedCompositingMode(nullptr);
+    return true;
 }
 #endif
 
+void DrawingAreaCoordinatedGraphics::setDeviceScaleFactor(float deviceScaleFactor)
+{
+    Ref { m_webPage.get() }->setDeviceScaleFactor(deviceScaleFactor);
+}
+
 bool DrawingAreaCoordinatedGraphics::supportsAsyncScrolling() const
 {
     return m_supportsAsyncScrolling;
@@ -297,18 +275,18 @@ bool DrawingAreaCoordinatedGraphics::supportsAsyncScrolling() const
 
 void DrawingAreaCoordinatedGraphics::registerScrollingTree()
 {
-#if ENABLE(SCROLLING_THREAD)
+#if ENABLE(ASYNC_SCROLLING) && ENABLE(SCROLLING_THREAD)
     if (m_supportsAsyncScrolling)
         WebProcess::singleton().eventDispatcher().addScrollingTreeForPage(m_webPage);
-#endif
+#endif // ENABLE(ASYNC_SCROLLING) && ENABLE(SCROLLING_THREAD)
 }
 
 void DrawingAreaCoordinatedGraphics::unregisterScrollingTree()
 {
-#if ENABLE(SCROLLING_THREAD)
+#if ENABLE(ASYNC_SCROLLING) && ENABLE(SCROLLING_THREAD)
     if (m_supportsAsyncScrolling)
         WebProcess::singleton().eventDispatcher().removeScrollingTreeForPage(m_webPage);
-#endif
+#endif // ENABLE(ASYNC_SCROLLING) && ENABLE(SCROLLING_THREAD)
 }
 
 GraphicsLayerFactory* DrawingAreaCoordinatedGraphics::graphicsLayerFactory()
@@ -318,18 +296,13 @@ GraphicsLayerFactory* DrawingAreaCoordinatedGraphics::graphicsLayerFactory()
     return m_layerTreeHost ? m_layerTreeHost->graphicsLayerFactory() : nullptr;
 }
 
-void DrawingAreaCoordinatedGraphics::setRootCompositingLayer(GraphicsLayer* graphicsLayer)
+void DrawingAreaCoordinatedGraphics::setRootCompositingLayer(WebCore::Frame&, GraphicsLayer* graphicsLayer)
 {
     if (m_layerTreeHost) {
         if (graphicsLayer) {
             // We're already in accelerated compositing mode, but the root compositing layer changed.
             m_exitCompositingTimer.stop();
             m_wantsToExitAcceleratedCompositingMode = false;
-
-            // If we haven't sent the EnterAcceleratedCompositingMode message, make sure that the
-            // layer tree host calls us back after the next layer flush so we can send it then.
-            if (!m_compositingAccordingToProxyMessages)
-                m_layerTreeHost->setShouldNotifyAfterNextScheduledLayerFlush(true);
         }
         m_layerTreeHost->setRootCompositingLayer(graphicsLayer);
 
@@ -338,7 +311,7 @@ void DrawingAreaCoordinatedGraphics::setRootCompositingLayer(GraphicsLayer* grap
             // compositing code via display() and layout.
             // If we're leaving compositing mode because of a setSize, it is safe to
             // exit accelerated compositing mode right away.
-            if (m_inUpdateBackingStoreState)
+            if (m_inUpdateGeometry)
                 exitAcceleratedCompositingMode();
             else
                 exitAcceleratedCompositingModeSoon();
@@ -364,30 +337,21 @@ void DrawingAreaCoordinatedGraphics::triggerRenderingUpdate()
         scheduleDisplay();
 }
 
-#if USE(COORDINATED_GRAPHICS) || USE(GRAPHICS_LAYER_TEXTURE_MAPPER)
-void DrawingAreaCoordinatedGraphics::layerHostDidFlushLayers()
-{
-    ASSERT(m_layerTreeHost);
-    m_layerTreeHost->forceRepaint();
-
-    if (m_shouldSendDidUpdateBackingStoreState && !exitAcceleratedCompositingModePending()) {
-        sendDidUpdateBackingStoreState();
-        return;
-    }
-}
-#endif
-
 RefPtr<DisplayRefreshMonitor> DrawingAreaCoordinatedGraphics::createDisplayRefreshMonitor(PlatformDisplayID displayID)
 {
+#if HAVE(DISPLAY_LINK)
+    return WebDisplayRefreshMonitor::create(displayID);
+#else
     if (!m_layerTreeHost || m_wantsToExitAcceleratedCompositingMode || exitAcceleratedCompositingModePending())
         return nullptr;
     return m_layerTreeHost->createDisplayRefreshMonitor(displayID);
+#endif
 }
 
-void DrawingAreaCoordinatedGraphics::activityStateDidChange(OptionSet<ActivityState::Flag> changed, ActivityStateChangeID, CompletionHandler<void()>&& completionHandler)
+void DrawingAreaCoordinatedGraphics::activityStateDidChange(OptionSet<ActivityState> changed, ActivityStateChangeID, CompletionHandler<void()>&& completionHandler)
 {
     if (changed & ActivityState::IsVisible) {
-        if (m_webPage.isVisible())
+        if (m_webPage->isVisible())
             resumePainting();
         else
             suspendPainting();
@@ -395,82 +359,45 @@ void DrawingAreaCoordinatedGraphics::activityStateDidChange(OptionSet<ActivitySt
     completionHandler();
 }
 
-void DrawingAreaCoordinatedGraphics::attachViewOverlayGraphicsLayer(GraphicsLayer* viewOverlayRootLayer)
+void DrawingAreaCoordinatedGraphics::attachViewOverlayGraphicsLayer(WebCore::FrameIdentifier, GraphicsLayer* viewOverlayRootLayer)
 {
     if (m_layerTreeHost)
         m_layerTreeHost->setViewOverlayRootLayer(viewOverlayRootLayer);
-    else if (m_previousLayerTreeHost)
-        m_previousLayerTreeHost->setViewOverlayRootLayer(viewOverlayRootLayer);
 }
 
-void DrawingAreaCoordinatedGraphics::updateBackingStoreState(uint64_t stateID, bool respondImmediately, float deviceScaleFactor, const IntSize& size, const IntSize& scrollOffset)
+void DrawingAreaCoordinatedGraphics::updateGeometry(const IntSize& size, CompletionHandler<void()>&& completionHandler)
 {
-    if (stateID != m_backingStoreStateID && !m_layerTreeHost)
+    SetForScope inUpdateGeometry(m_inUpdateGeometry, true);
+    Ref webPage = m_webPage.get();
+    webPage->setSize(size);
+    webPage->layoutIfNeeded();
+
+    if (!m_layerTreeHost)
         m_dirtyRegion = IntRect(IntPoint(), size);
 
-    ASSERT(!m_inUpdateBackingStoreState);
-    m_inUpdateBackingStoreState = true;
+    LayerTreeContext previousLayerTreeContext;
+    if (m_layerTreeHost) {
+        previousLayerTreeContext = m_layerTreeHost->layerTreeContext();
+        m_layerTreeHost->sizeDidChange(webPage->size());
+    }
 
-    ASSERT_ARG(stateID, stateID >= m_backingStoreStateID);
-    if (stateID != m_backingStoreStateID) {
-        m_backingStoreStateID = stateID;
-        m_shouldSendDidUpdateBackingStoreState = true;
-
-        m_webPage.setDeviceScaleFactor(deviceScaleFactor);
-        m_webPage.setSize(size);
-        m_webPage.updateRendering();
-        m_webPage.finalizeRenderingUpdate({ });
-        m_webPage.flushPendingEditorStateUpdate();
-        m_webPage.scrollMainFrameIfNotAtMaxScrollPosition(scrollOffset);
-        m_webPage.didUpdateRendering();
-
-        if (m_layerTreeHost)
-            m_layerTreeHost->sizeDidChange(m_webPage.size());
-        else if (m_previousLayerTreeHost)
-            m_previousLayerTreeHost->sizeDidChange(m_webPage.size());
+    if (m_layerTreeHost) {
+        auto layerTreeContext = m_layerTreeHost->layerTreeContext();
+        m_layerTreeHost->forceRepaint();
+        if (layerTreeContext != previousLayerTreeContext)
+            send(Messages::DrawingAreaProxy::UpdateAcceleratedCompositingMode(0, layerTreeContext));
     } else {
-        ASSERT(size == m_webPage.size());
-        if (!m_shouldSendDidUpdateBackingStoreState) {
-            // We've already sent a DidUpdateBackingStoreState message for this state. We have nothing more to do.
-            m_inUpdateBackingStoreState = false;
-            if (m_forceRepaintAfterBackingStoreStateUpdate)
-                forceRepaint();
-            return;
-        }
+        UpdateInfo updateInfo;
+        if (m_isPaintingSuspended) {
+            updateInfo.viewSize = webPage->size();
+            updateInfo.deviceScaleFactor = webPage->corePage()->deviceScaleFactor();
+        } else
+            display(updateInfo);
+        if (!m_layerTreeHost)
+            send(Messages::DrawingAreaProxy::Update(0, WTFMove(updateInfo)));
     }
 
-    // The UI process has updated to a new backing store state. Any Update messages we sent before
-    // this point will be ignored. We wait to set this to false until after updating the page's
-    // size so that any displays triggered by the relayout will be ignored. If we're supposed to
-    // respond to the UpdateBackingStoreState message immediately, we'll do a display anyway in
-    // sendDidUpdateBackingStoreState; otherwise we shouldn't do one right now.
-    m_isWaitingForDidUpdate = false;
-
-    if (respondImmediately) {
-        // Make sure to resume painting if we're supposed to respond immediately, otherwise we'll just
-        // send back an empty UpdateInfo struct.
-        bool wasSuspended = m_isPaintingSuspended;
-        if (m_isPaintingSuspended)
-            resumePainting();
-
-        sendDidUpdateBackingStoreState();
-        if (wasSuspended)
-            suspendPainting();
-    }
-
-    m_inUpdateBackingStoreState = false;
-
-    if (m_forceRepaintAfterBackingStoreStateUpdate)
-        forceRepaint();
-}
-
-void DrawingAreaCoordinatedGraphics::targetRefreshRateDidChange(unsigned rate)
-{
-    UNUSED_PARAM(rate);
-#if !USE(GRAPHICS_LAYER_TEXTURE_MAPPER)
-    if (m_layerTreeHost)
-        m_layerTreeHost->targetRefreshRateDidChange(rate);
-#endif
+    completionHandler();
 }
 
 void DrawingAreaCoordinatedGraphics::displayDidRefresh()
@@ -494,8 +421,11 @@ void DrawingAreaCoordinatedGraphics::displayDidRefresh()
 void DrawingAreaCoordinatedGraphics::adjustTransientZoom(double scale, FloatPoint origin)
 {
     if (!m_transientZoom) {
-        FrameView& frameView = *m_webPage.mainFrameView();
-        FloatRect unobscuredContentRect = frameView.unobscuredContentRectIncludingScrollbars();
+        RefPtr frameView = m_webPage->localMainFrameView();
+        if (!frameView)
+            return;
+
+        FloatRect unobscuredContentRect = frameView->unobscuredContentRectIncludingScrollbars();
 
         m_transientZoom = true;
         m_transientZoomInitialOrigin = unobscuredContentRect.location();
@@ -511,10 +441,11 @@ void DrawingAreaCoordinatedGraphics::adjustTransientZoom(double scale, FloatPoin
     FloatPoint unscrolledOrigin(origin);
     unscrolledOrigin.moveBy(-m_transientZoomInitialOrigin);
 
-    m_webPage.scalePage(scale / m_webPage.viewScaleFactor(), roundedIntPoint(-unscrolledOrigin));
+    Ref webPage = m_webPage.get();
+    webPage->scalePage(scale / webPage->viewScaleFactor(), roundedIntPoint(-unscrolledOrigin));
 }
 
-void DrawingAreaCoordinatedGraphics::commitTransientZoom(double scale, FloatPoint origin)
+void DrawingAreaCoordinatedGraphics::commitTransientZoom(double scale, FloatPoint origin, CompletionHandler<void()>&& completionHandler)
 {
     if (m_layerTreeHost)
         m_layerTreeHost->commitTransientZoom(scale, origin);
@@ -522,53 +453,14 @@ void DrawingAreaCoordinatedGraphics::commitTransientZoom(double scale, FloatPoin
     FloatPoint unscrolledOrigin(origin);
     unscrolledOrigin.moveBy(-m_transientZoomInitialOrigin);
 
-    m_webPage.scalePage(scale / m_webPage.viewScaleFactor(), roundedIntPoint(-unscrolledOrigin));
+    Ref webPage = m_webPage.get();
+    webPage->scalePage(scale / webPage->viewScaleFactor(), roundedIntPoint(-unscrolledOrigin));
 
     m_transientZoom = false;
+    completionHandler();
 }
 #endif
 
-void DrawingAreaCoordinatedGraphics::sendDidUpdateBackingStoreState()
-{
-    ASSERT(!m_isWaitingForDidUpdate);
-    ASSERT(m_shouldSendDidUpdateBackingStoreState);
-
-    if (!m_isPaintingSuspended && !m_layerTreeHost) {
-        UpdateInfo updateInfo;
-        display(updateInfo);
-        if (!m_layerTreeHost) {
-            m_shouldSendDidUpdateBackingStoreState = false;
-
-            LayerTreeContext layerTreeContext;
-            send(Messages::DrawingAreaProxy::DidUpdateBackingStoreState(m_backingStoreStateID, updateInfo, layerTreeContext));
-            m_compositingAccordingToProxyMessages = false;
-            return;
-        }
-    }
-
-    ASSERT(m_shouldSendDidUpdateBackingStoreState);
-    m_shouldSendDidUpdateBackingStoreState = false;
-
-    UpdateInfo updateInfo;
-    updateInfo.viewSize = m_webPage.size();
-    updateInfo.deviceScaleFactor = m_webPage.corePage()->deviceScaleFactor();
-
-    LayerTreeContext layerTreeContext;
-    if (m_layerTreeHost) {
-        layerTreeContext = m_layerTreeHost->layerTreeContext();
-
-        // We don't want the layer tree host to notify after the next scheduled
-        // layer flush because that might end up sending an EnterAcceleratedCompositingMode
-        // message back to the UI process, but the updated layer tree context
-        // will be sent back in the DidUpdateBackingStoreState message.
-        m_layerTreeHost->setShouldNotifyAfterNextScheduledLayerFlush(false);
-        m_layerTreeHost->forceRepaint();
-    }
-
-    send(Messages::DrawingAreaProxy::DidUpdateBackingStoreState(m_backingStoreStateID, updateInfo, layerTreeContext));
-    m_compositingAccordingToProxyMessages = !layerTreeContext.isEmpty();
-}
-
 void DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingModeSoon()
 {
     if (m_layerTreeStateIsFrozen) {
@@ -582,12 +474,6 @@ void DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingModeSoon()
     m_exitCompositingTimer.startOneShot(0_s);
 }
 
-void DrawingAreaCoordinatedGraphics::discardPreviousLayerTreeHost()
-{
-    m_discardPreviousLayerTreeHostTimer.stop();
-    m_previousLayerTreeHost = nullptr;
-}
-
 void DrawingAreaCoordinatedGraphics::suspendPainting()
 {
     ASSERT(!m_isPaintingSuspended);
@@ -599,7 +485,7 @@ void DrawingAreaCoordinatedGraphics::suspendPainting()
 
     m_isPaintingSuspended = true;
 
-    m_webPage.corePage()->suspendScriptedAnimations();
+    m_webPage->corePage()->suspendScriptedAnimations();
 }
 
 void DrawingAreaCoordinatedGraphics::resumePainting()
@@ -618,54 +504,46 @@ void DrawingAreaCoordinatedGraphics::resumePainting()
     // FIXME: We shouldn't always repaint everything here.
     setNeedsDisplay();
 
-    m_webPage.corePage()->resumeScriptedAnimations();
+    m_webPage->corePage()->resumeScriptedAnimations();
 }
 
 void DrawingAreaCoordinatedGraphics::enterAcceleratedCompositingMode(GraphicsLayer* graphicsLayer)
 {
 #if PLATFORM(GTK)
     if (!m_alwaysUseCompositing) {
-        m_webPage.corePage()->settings().setForceCompositingMode(true);
+        m_webPage->corePage()->settings().setForceCompositingMode(true);
         m_alwaysUseCompositing = true;
     }
 #endif
-    m_discardPreviousLayerTreeHostTimer.stop();
 
     m_exitCompositingTimer.stop();
     m_wantsToExitAcceleratedCompositingMode = false;
 
+#if !HAVE(DISPLAY_LINK)
     auto changeWindowScreen = [&] {
         // In order to ensure that we get a unique DisplayRefreshMonitor per-DrawingArea (necessary because ThreadedDisplayRefreshMonitor
         // is driven by the ThreadedCompositor of the drawing area), give each page a unique DisplayID derived from DrawingArea's unique ID.
-        m_webPage.windowScreenDidChange(m_layerTreeHost->displayID(), std::nullopt);
+        Ref { m_webPage.get() }->windowScreenDidChange(m_layerTreeHost->displayID(), std::nullopt);
     };
+#endif
 
     ASSERT(!m_layerTreeHost);
-    if (m_previousLayerTreeHost) {
-        m_layerTreeHost = WTFMove(m_previousLayerTreeHost);
-        changeWindowScreen();
-        m_layerTreeHost->setIsDiscardable(false);
-        m_layerTreeHost->resumeRendering();
-        if (!m_layerTreeStateIsFrozen)
-            m_layerTreeHost->setLayerFlushSchedulingEnabled(true);
-    } else {
-#if USE(GRAPHICS_LAYER_TEXTURE_MAPPER)
-        m_layerTreeHost = makeUnique<LayerTreeHost>(m_webPage);
+#if USE(GRAPHICS_LAYER_TEXTURE_MAPPER) || HAVE(DISPLAY_LINK)
+    m_layerTreeHost = makeUnique<LayerTreeHost>(m_webPage);
 #elif USE(COORDINATED_GRAPHICS)
-        m_layerTreeHost = makeUnique<LayerTreeHost>(m_webPage, std::numeric_limits<uint32_t>::max() - m_identifier.toUInt64());
+    m_layerTreeHost = makeUnique<LayerTreeHost>(m_webPage, std::numeric_limits<uint32_t>::max() - m_identifier.toUInt64());
 #else
-        m_layerTreeHost = nullptr;
-        return;
+    m_layerTreeHost = nullptr;
+    return;
 #endif
-        changeWindowScreen();
-        if (m_layerTreeStateIsFrozen)
-            m_layerTreeHost->setLayerFlushSchedulingEnabled(false);
-        if (m_isPaintingSuspended)
-            m_layerTreeHost->pauseRendering();
-    }
 
-    if (!m_inUpdateBackingStoreState)
-        m_layerTreeHost->setShouldNotifyAfterNextScheduledLayerFlush(true);
+#if !HAVE(DISPLAY_LINK)
+    changeWindowScreen();
+#endif
+    if (m_layerTreeStateIsFrozen)
+        m_layerTreeHost->setLayerFlushSchedulingEnabled(false);
+    if (m_isPaintingSuspended)
+        m_layerTreeHost->pauseRendering();
 
     m_layerTreeHost->setRootCompositingLayer(graphicsLayer);
 
@@ -690,7 +568,7 @@ void DrawingAreaCoordinatedGraphics::sendEnterAcceleratedCompositingModeIfNeeded
         return;
     }
 
-    send(Messages::DrawingAreaProxy::EnterAcceleratedCompositingMode(m_backingStoreStateID, m_layerTreeHost->layerTreeContext()));
+    send(Messages::DrawingAreaProxy::EnterAcceleratedCompositingMode(0, m_layerTreeHost->layerTreeContext()));
     m_compositingAccordingToProxyMessages = true;
 }
 
@@ -705,41 +583,35 @@ void DrawingAreaCoordinatedGraphics::exitAcceleratedCompositingMode()
     m_wantsToExitAcceleratedCompositingMode = false;
 
     ASSERT(m_layerTreeHost);
-    m_previousLayerTreeHost = WTFMove(m_layerTreeHost);
-    m_previousLayerTreeHost->setIsDiscardable(true);
-    m_previousLayerTreeHost->pauseRendering();
-    m_previousLayerTreeHost->setLayerFlushSchedulingEnabled(false);
-    m_discardPreviousLayerTreeHostTimer.startOneShot(5_s);
+    m_layerTreeHost = nullptr;
 
+    Ref webPage = m_webPage.get();
+#if !HAVE(DISPLAY_LINK)
     // Always use the primary display ID (0) when not in accelerated compositing mode.
-    m_webPage.windowScreenDidChange(0, std::nullopt);
+    webPage->windowScreenDidChange(0, std::nullopt);
+#endif
 
-    m_dirtyRegion = m_webPage.bounds();
+    m_dirtyRegion = webPage->bounds();
 
-    if (m_inUpdateBackingStoreState)
+    if (m_inUpdateGeometry)
         return;
 
-    if (m_shouldSendDidUpdateBackingStoreState) {
-        sendDidUpdateBackingStoreState();
-        return;
-    }
-
     UpdateInfo updateInfo;
     if (m_isPaintingSuspended) {
-        updateInfo.viewSize = m_webPage.size();
-        updateInfo.deviceScaleFactor = m_webPage.corePage()->deviceScaleFactor();
+        updateInfo.viewSize = webPage->size();
+        updateInfo.deviceScaleFactor = webPage->corePage()->deviceScaleFactor();
     } else
         display(updateInfo);
 
     // Send along a complete update of the page so we can paint the contents right after we exit the
     // accelerated compositing mode, eliminiating flicker.
     if (m_compositingAccordingToProxyMessages) {
-        send(Messages::DrawingAreaProxy::ExitAcceleratedCompositingMode(m_backingStoreStateID, updateInfo));
+        send(Messages::DrawingAreaProxy::ExitAcceleratedCompositingMode(0, WTFMove(updateInfo)));
         m_compositingAccordingToProxyMessages = false;
     } else {
         // If we left accelerated compositing mode before we sent an EnterAcceleratedCompositingMode message to the
         // UI process, we still need to let it know about the new contents, so send an Update message.
-        send(Messages::DrawingAreaProxy::Update(m_backingStoreStateID, updateInfo));
+        send(Messages::DrawingAreaProxy::Update(0, WTFMove(updateInfo)));
     }
 }
 
@@ -770,7 +642,7 @@ void DrawingAreaCoordinatedGraphics::display()
 {
     ASSERT(!m_layerTreeHost);
     ASSERT(!m_isWaitingForDidUpdate);
-    ASSERT(!m_inUpdateBackingStoreState);
+    ASSERT(!m_inUpdateGeometry);
 
     if (m_layerTreeStateIsFrozen)
         return;
@@ -778,11 +650,6 @@ void DrawingAreaCoordinatedGraphics::display()
     if (m_isPaintingSuspended)
         return;
 
-    if (m_shouldSendDidUpdateBackingStoreState) {
-        sendDidUpdateBackingStoreState();
-        return;
-    }
-
     UpdateInfo updateInfo;
     display(updateInfo);
 
@@ -796,10 +663,10 @@ void DrawingAreaCoordinatedGraphics::display()
     }
 
     if (m_compositingAccordingToProxyMessages) {
-        send(Messages::DrawingAreaProxy::ExitAcceleratedCompositingMode(m_backingStoreStateID, updateInfo));
+        send(Messages::DrawingAreaProxy::ExitAcceleratedCompositingMode(0, WTFMove(updateInfo)));
         m_compositingAccordingToProxyMessages = false;
     } else
-        send(Messages::DrawingAreaProxy::Update(m_backingStoreStateID, updateInfo));
+        send(Messages::DrawingAreaProxy::Update(0, WTFMove(updateInfo)));
     m_isWaitingForDidUpdate = true;
     m_scheduledWhileWaitingForDidUpdate = false;
 }
@@ -827,12 +694,13 @@ static bool shouldPaintBoundsRect(const IntRect& bounds, const Vector<IntRect, 1
 
 void DrawingAreaCoordinatedGraphics::display(UpdateInfo& updateInfo)
 {
-    ASSERT(!m_isPaintingSuspended);
+    ASSERT(!m_isPaintingSuspended || m_inUpdateGeometry);
     ASSERT(!m_layerTreeHost);
 
-    m_webPage.updateRendering();
-    m_webPage.finalizeRenderingUpdate({ });
-    m_webPage.flushPendingEditorStateUpdate();
+    Ref webPage = m_webPage.get();
+    webPage->updateRendering();
+    webPage->finalizeRenderingUpdate({ });
+    webPage->flushPendingEditorStateUpdate();
 
     // The layout may have put the page into accelerated compositing mode. If the LayerTreeHost is
     // in charge of displaying, we have nothing more to do.
@@ -842,16 +710,16 @@ void DrawingAreaCoordinatedGraphics::display(UpdateInfo& updateInfo)
     if (m_dirtyRegion.isEmpty())
         return;
 
-    updateInfo.viewSize = m_webPage.size();
-    updateInfo.deviceScaleFactor = m_webPage.corePage()->deviceScaleFactor();
+    updateInfo.viewSize = webPage->size();
+    updateInfo.deviceScaleFactor = webPage->corePage()->deviceScaleFactor();
 
     IntRect bounds = m_dirtyRegion.bounds();
-    ASSERT(m_webPage.bounds().contains(bounds));
+    ASSERT(webPage->bounds().contains(bounds));
 
     IntSize bitmapSize = bounds.size();
-    float deviceScaleFactor = m_webPage.corePage()->deviceScaleFactor();
+    float deviceScaleFactor = webPage->corePage()->deviceScaleFactor();
     bitmapSize.scale(deviceScaleFactor);
-    auto bitmap = ShareableBitmap::create(bitmapSize, { });
+    auto bitmap = ShareableBitmap::create({ bitmapSize });
     if (!bitmap)
         return;
 
@@ -883,15 +751,38 @@ void DrawingAreaCoordinatedGraphics::display(UpdateInfo& updateInfo)
 
     for (const auto& rect : rects) {
         if (graphicsContext)
-            m_webPage.drawRect(*graphicsContext, rect);
+            webPage->drawRect(*graphicsContext, rect);
         updateInfo.updateRects.append(rect);
     }
 
-    m_webPage.didUpdateRendering();
+    webPage->didUpdateRendering();
 
     // Layout can trigger more calls to setNeedsDisplay and we don't want to process them
     // until the UI process has painted the update, so we stop the timer here.
     m_displayTimer.stop();
 }
 
+void DrawingAreaCoordinatedGraphics::forceUpdate()
+{
+    if (m_isWaitingForDidUpdate || m_layerTreeHost)
+        return;
+
+    m_dirtyRegion = m_webPage->bounds();
+    display();
+}
+
+void DrawingAreaCoordinatedGraphics::didDiscardBackingStore()
+{
+    // Ensure the next update will cover the entire view, since the UI process discarded its backing store.
+    m_dirtyRegion = m_webPage->bounds();
+}
+
+#if PLATFORM(WPE) && USE(GBM) && ENABLE(WPE_PLATFORM)
+void DrawingAreaCoordinatedGraphics::preferredBufferFormatsDidChange()
+{
+    if (m_layerTreeHost)
+        m_layerTreeHost->preferredBufferFormatsDidChange();
+}
+#endif
+
 } // namespace WebKit
