@@ -127,7 +127,8 @@ void WebProcess::platformInitializeWebProcess(WebProcessCreationParameters& para
 #endif
 
 #if PLATFORM(WPE)
-    if (!parameters.isServiceWorkerProcess) {
+    m_dmaBufRendererBufferMode = parameters.dmaBufRendererBufferMode;
+    if (!parameters.isServiceWorkerProcess && m_dmaBufRendererBufferMode.isEmpty()) {
         auto& implementationLibraryName = parameters.implementationLibraryName;
         if (!implementationLibraryName.isNull() && implementationLibraryName.data()[0] != '\0')
             wpe_loader_init(parameters.implementationLibraryName.data());
@@ -141,6 +142,11 @@ void WebProcess::platformInitializeWebProcess(WebProcessCreationParameters& para
     WebCore::GBMDevice::singleton().initialize(parameters.renderDeviceFile);
 #endif
 
+#if PLATFORM(WPE)
+    if (!parameters.isServiceWorkerProcess && !m_dmaBufRendererBufferMode.isEmpty())
+        WebCore::PlatformDisplay::setUseDMABufForRendering(true);
+#endif
+
 #if PLATFORM(GTK) && USE(EGL)
     m_dmaBufRendererBufferMode = parameters.dmaBufRendererBufferMode;
     if (!m_dmaBufRendererBufferMode.isEmpty()) {
@@ -158,18 +164,6 @@ void WebProcess::platformInitializeWebProcess(WebProcessCreationParameters& para
     }
 #endif
 
-#if PLATFORM(WAYLAND)
-    if (PlatformDisplay::sharedDisplay().type() == PlatformDisplay::Type::Wayland && !parameters.isServiceWorkerProcess && m_dmaBufRendererBufferMode.isEmpty()) {
-        auto hostClientFileDescriptor = parameters.hostClientFileDescriptor.release();
-        if (hostClientFileDescriptor != -1) {
-            wpe_loader_init(parameters.implementationLibraryName.data());
-            m_displayForCompositing = WebCore::PlatformDisplayLibWPE::create();
-            if (!downcast<WebCore::PlatformDisplayLibWPE>(*m_displayForCompositing).initialize(hostClientFileDescriptor))
-                m_displayForCompositing = nullptr;
-        }
-    }
-#endif
-
 #if USE(GSTREAMER)
     WebCore::setGStreamerOptionsFromUIProcess(WTFMove(parameters.gstreamerOptions));
 #endif
@@ -201,6 +195,7 @@ void WebProcess::platformInitializeWebProcess(WebProcessCreationParameters& para
 
 #if PLATFORM(GTK)
     GtkSettingsManagerProxy::singleton().applySettings(WTFMove(parameters.gtkSettings));
+    WebCore::setScreenProperties(parameters.screenProperties);
 #endif
 }
 
@@ -253,6 +248,15 @@ void WebProcess::releaseSystemMallocMemory()
 #endif
 }
 
+#if PLATFORM(GTK)
+void WebProcess::setScreenProperties(const WebCore::ScreenProperties& properties)
+{
+    WebCore::setScreenProperties(properties);
+    for (auto& page : m_pageMap.values())
+        page->screenPropertiesDidChange();
+}
+#endif
+
 } // namespace WebKit
 
 #undef RELEASE_LOG_SESSION_ID
