@@ -26,6 +26,7 @@
 #include "config.h"
 
 #include "MoveOnly.h"
+#include <wtf/CrossThreadCopier.h>
 #include <wtf/HashMap.h>
 #include <wtf/HashSet.h>
 #include <wtf/ListHashSet.h>
@@ -115,8 +116,8 @@ TEST(WTF_Vector, ConstructWithFrom)
 
 TEST(WTF_Vector, ConstructWithFromString)
 {
-    String s1 = "s1";
-    String s2 = "s2";
+    String s1 = "s1"_s;
+    String s2 = "s2"_s;
     String s3 = s1;
     auto vector = Vector<String>::from(s1, s2, WTFMove(s3));
     EXPECT_EQ(3U, vector.size());
@@ -128,10 +129,10 @@ TEST(WTF_Vector, ConstructWithFromString)
     EXPECT_TRUE(s3.isNull());
 }
 
-TEST(WTF_Vector, IsolateCopy)
+TEST(WTF_Vector, IsolatedCopy)
 {
-    String s1 = "s1";
-    String s2 = "s2";
+    String s1 = "s1"_s;
+    String s2 = "s2"_s;
 
     auto vector1 = Vector<String>::from(WTFMove(s1), s2);
     EXPECT_EQ(2U, vector1.size());
@@ -140,19 +141,19 @@ TEST(WTF_Vector, IsolateCopy)
     auto* data1 = vector1[0].impl();
     auto* data2 = vector1[1].impl();
 
-    auto vector2 = vector1.isolatedCopy();
+    auto vector2 = crossThreadCopy(vector1);
 
-    EXPECT_TRUE("s1" == vector2[0]);
-    EXPECT_TRUE("s2" == vector2[1]);
+    EXPECT_TRUE("s1"_s == vector2[0]);
+    EXPECT_TRUE("s2"_s == vector2[1]);
 
     EXPECT_FALSE(data1 == vector2[0].impl());
     EXPECT_FALSE(data2 == vector2[1].impl());
 
-    auto vector3 = WTFMove(vector1).isolatedCopy();
+    auto vector3 = crossThreadCopy(WTFMove(vector1));
     EXPECT_EQ(0U, vector1.size());
 
-    EXPECT_TRUE("s1" == vector3[0]);
-    EXPECT_TRUE("s2" == vector3[1]);
+    EXPECT_TRUE("s1"_s == vector3[0]);
+    EXPECT_TRUE("s2"_s == vector3[1]);
 
     EXPECT_TRUE(data1 == vector3[0].impl());
     EXPECT_FALSE(data2 == vector3[1].impl());
@@ -317,6 +318,63 @@ TEST(WTF_Vector, CopyFromOtherMinCapacity)
     EXPECT_FALSE(vector != vectorCopy);
 }
 
+TEST(WTF_Vector, ConstructorOtherRawPointerTypeAndLength)
+{
+    const UChar uchars[] = { 'b', 'a', 'r' };
+    Vector<LChar> vector(uchars, 3);
+    EXPECT_EQ(vector.size(), 3U);
+    EXPECT_EQ(vector[0], 'b');
+    EXPECT_EQ(vector[1], 'a');
+    EXPECT_EQ(vector[2], 'r');
+}
+
+TEST(WTF_Vector, ConstructorTakingLengthAndFunctor)
+{
+    Vector<size_t> vector(5, [](size_t i) { return i; });
+    EXPECT_EQ(vector.size(), 5U);
+    EXPECT_EQ(vector[0], 0U);
+    EXPECT_EQ(vector[1], 1U);
+    EXPECT_EQ(vector[2], 2U);
+    EXPECT_EQ(vector[3], 3U);
+    EXPECT_EQ(vector[4], 4U);
+}
+
+TEST(WTF_Vector, AppendList)
+{
+    Vector<size_t> vector({ 1, 2, 3 });
+    EXPECT_EQ(vector.size(), 3U);
+    EXPECT_EQ(vector[0], 1U);
+    EXPECT_EQ(vector[1], 2U);
+    EXPECT_EQ(vector[2], 3U);
+    vector.appendList({ 4, 5, 6 });
+    EXPECT_EQ(vector.size(), 6U);
+    EXPECT_EQ(vector[0], 1U);
+    EXPECT_EQ(vector[1], 2U);
+    EXPECT_EQ(vector[2], 3U);
+    EXPECT_EQ(vector[3], 4U);
+    EXPECT_EQ(vector[4], 5U);
+    EXPECT_EQ(vector[5], 6U);
+}
+
+TEST(WTF_Vector, AppendContainerWithMapping)
+{
+    Vector<size_t> vector({ 1, 2, 3 });
+    ListHashSet<std::pair<size_t, size_t>> set;
+    set.add({ 2, 2 });
+    set.add({ 2, 3 });
+    set.add({ 1, 5 });
+    vector.appendContainerWithMapping(set, [](auto& pair) {
+        return pair.first + pair.second;
+    });
+    EXPECT_EQ(vector.size(), 6U);
+    EXPECT_EQ(vector[0], 1U);
+    EXPECT_EQ(vector[1], 2U);
+    EXPECT_EQ(vector[2], 3U);
+    EXPECT_EQ(vector[3], 4U);
+    EXPECT_EQ(vector[4], 5U);
+    EXPECT_EQ(vector[5], 6U);
+}
+
 TEST(WTF_Vector, Reverse)
 {
     Vector<int> intVector;
@@ -372,7 +430,7 @@ TEST(WTF_Vector, MoveOnly_UncheckedAppend)
     vector.reserveInitialCapacity(100);
     for (size_t i = 0; i < 100; ++i) {
         MoveOnly moveOnly(i);
-        vector.uncheckedAppend(WTFMove(moveOnly));
+        vector.append(WTFMove(moveOnly));
         EXPECT_EQ(0U, moveOnly.value());
     }
 
@@ -582,6 +640,42 @@ TEST(WTF_Vector, RemoveFirst)
     EXPECT_TRUE(v.isEmpty());
 }
 
+TEST(WTF_Vector, RemoveLast)
+{
+    Vector<int> v;
+    EXPECT_TRUE(v.isEmpty());
+    EXPECT_FALSE(v.removeLast(1));
+    EXPECT_FALSE(v.removeLast(-1));
+    EXPECT_TRUE(v.isEmpty());
+
+    v.fill(2, 10);
+    EXPECT_EQ(10U, v.size());
+    EXPECT_FALSE(v.removeLast(1));
+    EXPECT_EQ(10U, v.size());
+    v.clear();
+
+    v.fill(1, 10);
+    EXPECT_EQ(10U, v.size());
+    EXPECT_TRUE(v.removeLast(1));
+    EXPECT_TRUE(v == Vector<int>({1, 1, 1, 1, 1, 1, 1, 1, 1}));
+    EXPECT_EQ(9U, v.size());
+    EXPECT_FALSE(v.removeLast(2));
+    EXPECT_EQ(9U, v.size());
+    EXPECT_TRUE(v == Vector<int>({1, 1, 1, 1, 1, 1, 1, 1, 1}));
+
+    unsigned removed = 0;
+    while (v.removeLast(1))
+        ++removed;
+    EXPECT_EQ(9U, removed);
+    EXPECT_TRUE(v.isEmpty());
+
+    v.resize(1);
+    EXPECT_EQ(1U, v.size());
+    EXPECT_TRUE(v.removeLast(1));
+    EXPECT_EQ(0U, v.size());
+    EXPECT_TRUE(v.isEmpty());
+}
+
 TEST(WTF_Vector, RemoveAll)
 {
     // Using a memcpy-able type.
@@ -775,6 +869,39 @@ TEST(WTF_Vector, RemoveFirstMatching)
     EXPECT_EQ(7U, v.size());
 }
 
+TEST(WTF_Vector, RemoveLastMatching)
+{
+    Vector<int> v;
+    EXPECT_TRUE(v.isEmpty());
+    EXPECT_FALSE(v.removeLastMatching([] (int value) { return value > 0; }));
+    EXPECT_FALSE(v.removeLastMatching([] (int) { return true; }));
+    EXPECT_FALSE(v.removeLastMatching([] (int) { return false; }));
+
+    v = {3, 1, 1, 1, 2, 2, 1, 2, 1, 2, 1, 3};
+    EXPECT_EQ(12U, v.size());
+    EXPECT_FALSE(v.removeLastMatching([] (int) { return false; }));
+    EXPECT_EQ(12U, v.size());
+    EXPECT_FALSE(v.removeLastMatching([] (int value) { return value < 0; }));
+    EXPECT_EQ(12U, v.size());
+    EXPECT_TRUE(v.removeLastMatching([] (int value) { return value < 3; }));
+    EXPECT_EQ(11U, v.size());
+    EXPECT_TRUE(v == Vector<int>({3, 1, 1, 1, 2, 2, 1, 2, 1, 2, 3}));
+    EXPECT_TRUE(v.removeLastMatching([] (int value) { return value > 2; }));
+    EXPECT_EQ(10U, v.size());
+    EXPECT_TRUE(v == Vector<int>({3, 1, 1, 1, 2, 2, 1, 2, 1, 2}));
+    EXPECT_TRUE(v.removeLastMatching([] (int value) { return value > 2; }, 10));
+    EXPECT_EQ(9U, v.size());
+    EXPECT_TRUE(v == Vector<int>({1, 1, 1, 2, 2, 1, 2, 1, 2}));
+    EXPECT_TRUE(v.removeLastMatching([] (int value) { return value == 1; }, 7));
+    EXPECT_EQ(8U, v.size());
+    EXPECT_TRUE(v == Vector<int>({1, 1, 1, 2, 2, 1, 2, 2}));
+    EXPECT_TRUE(v.removeLastMatching([] (int value) { return value == 1; }, 4));
+    EXPECT_EQ(7U, v.size());
+    EXPECT_TRUE(v == Vector<int>({1, 1, 2, 2, 1, 2, 2}));
+    EXPECT_FALSE(v.removeLastMatching([] (int value) { return value == 2; }, 1));
+    EXPECT_EQ(7U, v.size());
+}
+
 TEST(WTF_Vector, RemoveAllMatching)
 {
     Vector<int> v;
@@ -844,7 +971,7 @@ TEST(WTF_Vector, MapStaticFunctionMoveOnly)
 
     vector.reserveInitialCapacity(3);
     for (unsigned i = 0; i < 3; ++i)
-        vector.uncheckedAppend(MoveOnly { i });
+        vector.append(MoveOnly { i });
 
     auto mapped = WTF::map(vector, multiplyByTwoMoveOnly);
 
@@ -876,7 +1003,7 @@ TEST(WTF_Vector, MapLambdaMove)
 
     vector.reserveInitialCapacity(3);
     for (unsigned i = 0; i < 3; ++i)
-        vector.uncheckedAppend(MoveOnly { i });
+        vector.append(MoveOnly { i });
 
 
     unsigned counter = 0;
@@ -894,9 +1021,9 @@ TEST(WTF_Vector, MapLambdaMove)
 TEST(WTF_Vector, MapFromHashMap)
 {
     HashMap<String, String> map;
-    map.set(String { "k1" }, String { "v1" });
-    map.set(String { "k2" }, String { "v2" });
-    map.set(String { "k3" }, String { "v3" });
+    map.set("k1"_s, "v1"_s);
+    map.set("k2"_s, "v2"_s);
+    map.set("k3"_s, "v3"_s);
 
     auto mapped = WTF::map(map, [&] (KeyValuePair<String, String>& pair) -> String {
         return pair.value;
@@ -904,9 +1031,9 @@ TEST(WTF_Vector, MapFromHashMap)
     std::sort(mapped.begin(), mapped.end(), WTF::codePointCompareLessThan);
 
     EXPECT_EQ(3U, mapped.size());
-    EXPECT_TRUE(mapped[0] == "v1");
-    EXPECT_TRUE(mapped[1] == "v2");
-    EXPECT_TRUE(mapped[2] == "v3");
+    EXPECT_TRUE(mapped[0] == "v1"_s);
+    EXPECT_TRUE(mapped[1] == "v2"_s);
+    EXPECT_TRUE(mapped[2] == "v3"_s);
 
     mapped = WTF::map(map, [&] (const auto& pair) -> String {
         return pair.key;
@@ -914,9 +1041,9 @@ TEST(WTF_Vector, MapFromHashMap)
     std::sort(mapped.begin(), mapped.end(), WTF::codePointCompareLessThan);
 
     EXPECT_EQ(3U, mapped.size());
-    EXPECT_TRUE(mapped[0] == "k1");
-    EXPECT_TRUE(mapped[1] == "k2");
-    EXPECT_TRUE(mapped[2] == "k3");
+    EXPECT_TRUE(mapped[0] == "k1"_s);
+    EXPECT_TRUE(mapped[1] == "k2"_s);
+    EXPECT_TRUE(mapped[2] == "k3"_s);
 
     mapped = WTF::map(WTFMove(map), [&] (KeyValuePair<String, String>&& pair) -> String {
         return WTFMove(pair.value);
@@ -924,17 +1051,17 @@ TEST(WTF_Vector, MapFromHashMap)
     std::sort(mapped.begin(), mapped.end(), WTF::codePointCompareLessThan);
 
     EXPECT_EQ(3U, mapped.size());
-    EXPECT_TRUE(mapped[0] == "v1");
-    EXPECT_TRUE(mapped[1] == "v2");
-    EXPECT_TRUE(mapped[2] == "v3");
+    EXPECT_TRUE(mapped[0] == "v1"_s);
+    EXPECT_TRUE(mapped[1] == "v2"_s);
+    EXPECT_TRUE(mapped[2] == "v3"_s);
 
-    EXPECT_TRUE(map.contains("k1"));
-    EXPECT_TRUE(map.contains("k2"));
-    EXPECT_TRUE(map.contains("k3"));
+    EXPECT_TRUE(map.contains("k1"_s));
+    EXPECT_TRUE(map.contains("k2"_s));
+    EXPECT_TRUE(map.contains("k3"_s));
 
-    EXPECT_TRUE(map.get("k1").isNull());
-    EXPECT_TRUE(map.get("k2").isNull());
-    EXPECT_TRUE(map.get("k3").isNull());
+    EXPECT_TRUE(map.get("k1"_s).isNull());
+    EXPECT_TRUE(map.get("k2"_s).isNull());
+    EXPECT_TRUE(map.get("k3"_s).isNull());
 
 }
 
@@ -1051,7 +1178,7 @@ TEST(WTF_Vector, CompactMapStaticFunctionReturnOptionalRefPtr)
 
 TEST(WTF_Vector, CompactMapLambdaReturnOptional)
 {
-    Vector<String> vector { "a", "b", "hello", "ciao", "world", "webkit" };
+    Vector<String> vector { "a"_s, "b"_s, "hello"_s, "ciao"_s, "world"_s, "webkit"_s };
 
     auto function = [](const String& value) -> std::optional<String> {
         if (value.length() < 5)
@@ -1216,6 +1343,106 @@ TEST(WTF_Vector, CompactMapLambdaReturnOptionalRefPtr)
     EXPECT_EQ(nullptr, mapped[4]);
 }
 
+struct CopyCountingObject {
+    constexpr CopyCountingObject(int identifier)
+        : identifier { identifier }
+    {
+    }
+
+    CopyCountingObject(const CopyCountingObject& other)
+        : identifier { other.identifier }
+    {
+        ++copyCount;
+    }
+    CopyCountingObject(CopyCountingObject&& other)
+        : identifier { other.identifier }
+    {
+    }
+    CopyCountingObject& operator=(const CopyCountingObject& other)
+    {
+        identifier = other.identifier;
+        ++copyCount;
+        return *this;
+    }
+    CopyCountingObject& operator=(CopyCountingObject&& other)
+    {
+        identifier = other.identifier;
+        return *this;
+    }
+
+    int identifier { 0 };
+    static int copyCount;
+};
+
+constexpr bool operator==(const CopyCountingObject& a, int b) { return a.identifier == b; }
+constexpr bool operator==(int a, const CopyCountingObject& b) { return a == b.identifier; }
+
+int CopyCountingObject::copyCount;
+
+TEST(WTF_Vector, MapMinimalCopy)
+{
+    std::array<CopyCountingObject, 5> array { 1, 2, 3, 4, 5 };
+    Vector<CopyCountingObject> vector { 1, 2, 3, 4, 5 };
+
+    CopyCountingObject::copyCount = 0;
+    auto allCopiedFromVector = vector.map([](const auto& object) -> CopyCountingObject {
+        return object;
+    });
+    EXPECT_EQ(5U, allCopiedFromVector.size());
+    EXPECT_EQ(1, allCopiedFromVector[0]);
+    EXPECT_EQ(2, allCopiedFromVector[1]);
+    EXPECT_EQ(3, allCopiedFromVector[2]);
+    EXPECT_EQ(4, allCopiedFromVector[3]);
+    EXPECT_EQ(5, allCopiedFromVector[4]);
+    EXPECT_EQ(5, CopyCountingObject::copyCount);
+
+    CopyCountingObject::copyCount = 0;
+    auto allCopied = WTF::map(array, [](const auto& object) -> CopyCountingObject {
+        return object;
+    });
+    EXPECT_EQ(5U, allCopied.size());
+    EXPECT_EQ(1, allCopied[0]);
+    EXPECT_EQ(2, allCopied[1]);
+    EXPECT_EQ(3, allCopied[2]);
+    EXPECT_EQ(4, allCopied[3]);
+    EXPECT_EQ(5, allCopied[4]);
+    EXPECT_EQ(5, CopyCountingObject::copyCount);
+
+    CopyCountingObject::copyCount = 0;
+    auto allMoved = WTF::map(WTFMove(array), [](CopyCountingObject&& object) -> CopyCountingObject {
+        return WTFMove(object);
+    });
+    EXPECT_EQ(5U, allMoved.size());
+    EXPECT_EQ(1, allMoved[0]);
+    EXPECT_EQ(2, allMoved[1]);
+    EXPECT_EQ(3, allMoved[2]);
+    EXPECT_EQ(4, allMoved[3]);
+    EXPECT_EQ(5, allMoved[4]);
+    EXPECT_EQ(0, CopyCountingObject::copyCount);
+
+    CopyCountingObject::copyCount = 0;
+    auto evensCopied = WTF::compactMap(array, [](const auto& object) -> std::optional<CopyCountingObject> {
+        if (object.identifier % 2)
+            return std::nullopt;
+        return object;
+    });
+    EXPECT_EQ(2U, evensCopied.size());
+    EXPECT_EQ(2, evensCopied[0]);
+    EXPECT_EQ(4, evensCopied[1]);
+    EXPECT_EQ(2, CopyCountingObject::copyCount);
+
+    CopyCountingObject::copyCount = 0;
+    auto evensMoved = WTF::compactMap(WTFMove(array), [](auto&& object) -> std::optional<CopyCountingObject> {
+        if (object.identifier % 2)
+            return std::nullopt;
+        return WTFMove(object);
+    });
+    EXPECT_EQ(2U, evensMoved.size());
+    EXPECT_EQ(2, evensMoved[0]);
+    EXPECT_EQ(4, evensMoved[1]);
+    EXPECT_EQ(0, CopyCountingObject::copyCount);
+}
+
 TEST(WTF_Vector, CopyToVector)
 {
     HashSet<int> intSet { 1, 2, 3 };
@@ -1284,8 +1511,8 @@ TEST(WTF_Vector, CopyToVectorSizeRangeIterator)
 
 TEST(WTF_Vector, StringComparison)
 {
-    Vector<String> a = {{ "a" }};
-    Vector<String> b = {{ "a" }};
+    Vector<String> a = {{ "a"_s }};
+    Vector<String> b = {{ "a"_s }};
     EXPECT_TRUE(a == b);
 }
 
@@ -1404,8 +1631,8 @@ TEST(WTF_Vector, HashKeyInlineCapacity)
 
 TEST(WTF_Vector, HashKeyString)
 {
-    Vector<String> a = { "a" };
-    Vector<String> b = { "a", "b" };
+    Vector<String> a = { "a"_s };
+    Vector<String> b = { "a"_s, "b"_s };
 
     HashSet<Vector<String>> hash;
 
@@ -1437,46 +1664,46 @@ TEST(WTF_Vector, HashKeyString)
     EXPECT_FALSE(hash.contains(a));
     EXPECT_FALSE(hash.contains(b));
 
-    for (auto i : { "a", "b", "c", "d", "e" }) {
-        for (auto j : { "a", "b", "c", "d", "e" }) {
+    for (auto i : { "a"_s, "b"_s, "c"_s, "d"_s, "e"_s }) {
+        for (auto j : { "a"_s, "b"_s, "c"_s, "d"_s, "e"_s }) {
             hash.add({ i, j });
             hash.add({ i, j, i, j });
         }
     }
 
-    for (auto i : { "a", "b", "c", "d", "e" }) {
-        for (auto j : { "a", "b", "c", "d", "e" }) {
+    for (auto i : { "a"_s, "b"_s, "c"_s, "d"_s, "e"_s }) {
+        for (auto j : { "a"_s, "b"_s, "c"_s, "d"_s, "e"_s }) {
             EXPECT_TRUE(hash.contains({ i, j }));
             EXPECT_TRUE(hash.contains({ i, j, i, j }));
             EXPECT_FALSE(hash.contains({ i, j, i }));
-            EXPECT_FALSE(hash.contains({ "x", j }));
-            EXPECT_FALSE(hash.contains({ i, j, "x", j }));
+            EXPECT_FALSE(hash.contains({ "x"_s, j }));
+            EXPECT_FALSE(hash.contains({ i, j, "x"_s, j }));
         }
     }
 
-    for (auto i : { "d", "e" }) {
-        for (auto j : { "a", "b", "c", "d", "e" }) {
+    for (auto i : { "d"_s, "e"_s }) {
+        for (auto j : { "a"_s, "b"_s, "c"_s, "d"_s, "e"_s }) {
             hash.remove({ i, j });
             hash.remove({ i, j, i, j });
         }
     }
 
-    for (auto i : { "d", "e" }) {
-        for (auto j : { "a", "b", "c", "d", "e" }) {
+    for (auto i : { "d"_s, "e"_s }) {
+        for (auto j : { "a"_s, "b"_s, "c"_s, "d"_s, "e"_s }) {
             EXPECT_FALSE(hash.contains({ i, j }));
             EXPECT_FALSE(hash.contains({ i, j, i, j }));
         }
     }
 
-    for (auto i : { "a", "b", "c" }) {
-        for (auto j : { "a", "b", "c", "d", "e" }) {
+    for (auto i : { "a"_s, "b"_s, "c"_s }) {
+        for (auto j : { "a"_s, "b"_s, "c"_s, "d"_s, "e"_s }) {
             EXPECT_TRUE(hash.contains({ i, j }));
             EXPECT_TRUE(hash.contains({ i, j, i, j }));
         }
     }
 
-    for (auto i : { "a", "b", "c" }) {
-        for (auto j : { "a", "b", "c", "d", "e" }) {
+    for (auto i : { "a"_s, "b"_s, "c"_s }) {
+        for (auto j : { "a"_s, "b"_s, "c"_s, "d"_s, "e"_s }) {
             hash.remove({ i, j });
             hash.remove({ i, j, i, j });
         }
@@ -1511,4 +1738,330 @@ TEST(WTF_Vector, MapCustomReturnType)
     EXPECT_FLOAT_EQ(output[1], 2.0f);
 }
 
+TEST(WTF_Vector, MoveConstructor)
+{
+    {
+        Vector<String> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        Vector<String> strings2(WTFMove(strings));
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 0U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 5U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 16U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String, 10> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        EXPECT_EQ(strings.capacity(), 10U);
+        Vector<String, 10> strings2(WTFMove(strings));
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 10U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 10U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 10U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String, 2> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        EXPECT_EQ(strings.capacity(), 5U);
+        Vector<String, 2> strings2(WTFMove(strings));
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 2U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 5U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 16U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+}
+
+TEST(WTF_Vector, MoveAssignmentOperator)
+{
+    {
+        Vector<String> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        EXPECT_EQ(strings.capacity(), 5U);
+        Vector<String> strings2;
+        strings2 = WTFMove(strings);
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 0U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 5U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 16U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        Vector<String> strings2({ "foo"_str, "bar"_str });
+        strings2 = WTFMove(strings);
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 0U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 5U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 16U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String, 10> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        EXPECT_EQ(strings.capacity(), 10U);
+        Vector<String, 10> strings2;
+        strings2 = WTFMove(strings);
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 10U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 10U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 10U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String, 10> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        EXPECT_EQ(strings.capacity(), 10U);
+        Vector<String, 10> strings2({ "foo"_str, "bar"_str });
+        strings2 = WTFMove(strings);
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 10U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 10U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 10U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String, 2> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        EXPECT_EQ(strings.capacity(), 5U);
+        Vector<String, 2> strings2;
+        strings2 = WTFMove(strings);
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 2U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 5U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 16U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String, 2> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        EXPECT_EQ(strings.capacity(), 5U);
+        Vector<String, 2> strings2({ "foo"_str, "bar"_str });
+        strings2 = WTFMove(strings);
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 2U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 5U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 16U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String, 2> strings({ "a"_str, "b"_str, "c"_str, "d"_str, "e"_str });
+        EXPECT_EQ(strings.size(), 5U);
+        EXPECT_EQ(strings.capacity(), 5U);
+        Vector<String, 2> strings2({ "foo"_str, "bar"_str, "baz"_str });
+        strings2 = WTFMove(strings);
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 2U);
+        EXPECT_EQ(strings2.size(), 5U);
+        EXPECT_EQ(strings2.capacity(), 5U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+        EXPECT_STREQ(strings2[3].utf8().data(), "d");
+        EXPECT_STREQ(strings2[4].utf8().data(), "e");
+        strings2.append("f"_str);
+        EXPECT_EQ(strings2.size(), 6U);
+        EXPECT_EQ(strings2.capacity(), 16U);
+        EXPECT_STREQ(strings2[5].utf8().data(), "f");
+    }
+
+    {
+        Vector<String, 2> strings({ "a"_str, "b"_str });
+        EXPECT_EQ(strings.size(), 2U);
+        EXPECT_EQ(strings.capacity(), 2U);
+        Vector<String, 2> strings2({ "foo"_str, "bar"_str, "baz"_str });
+        strings2 = WTFMove(strings);
+        EXPECT_EQ(strings.size(), 0U);
+        EXPECT_EQ(strings.capacity(), 2U);
+        EXPECT_EQ(strings2.size(), 2U);
+        EXPECT_EQ(strings2.capacity(), 2U);
+        EXPECT_STREQ(strings2[0].utf8().data(), "a");
+        EXPECT_STREQ(strings2[1].utf8().data(), "b");
+        strings2.append("c"_str);
+        EXPECT_EQ(strings2.size(), 3U);
+        EXPECT_EQ(strings2.capacity(), 16U);
+        EXPECT_STREQ(strings2[2].utf8().data(), "c");
+    }
+}
+
+static Vector<int> mapVector(Vector<int> vector)
+{
+    return vector;
+}
+
+TEST(WTF_Vector, FlatMapCopy)
+{
+    Vector<Vector<int>> vector {
+        { 1, 2 },
+        { 3, 4 },
+    };
+
+    static_assert(std::is_same<decltype(WTF::flatMap(vector, mapVector)), typename WTF::Vector<int>>::value,
+        "WTF::flatMap returns Vector<int>");
+    auto mapped = WTF::flatMap(vector, mapVector);
+
+    EXPECT_EQ(2U, vector.size());
+    EXPECT_EQ(2U, vector[0].size());
+    EXPECT_EQ(2U, vector[1].size());
+
+    EXPECT_EQ(4U, mapped.size());
+    EXPECT_EQ(1, mapped[0]);
+    EXPECT_EQ(2, mapped[1]);
+    EXPECT_EQ(3, mapped[2]);
+    EXPECT_EQ(4, mapped[3]);
+}
+
+TEST(WTF_Vector, FlatMapMove)
+{
+    Vector<Vector<int>> vector {
+        { 1, 2 },
+        { 3, 4 },
+    };
+
+    static_assert(std::is_same<decltype(WTF::flatMap(WTFMove(vector), mapVector)), typename WTF::Vector<int>>::value,
+        "WTF::flatMap returns Vector<int>");
+    auto mapped = WTF::flatMap(WTFMove(vector), mapVector);
+
+    EXPECT_EQ(2U, vector.size());
+    EXPECT_EQ(0U, vector[0].size());
+    EXPECT_EQ(0U, vector[1].size());
+
+    EXPECT_EQ(4U, mapped.size());
+    EXPECT_EQ(1, mapped[0]);
+    EXPECT_EQ(2, mapped[1]);
+    EXPECT_EQ(3, mapped[2]);
+    EXPECT_EQ(4, mapped[3]);
+}
+
+TEST(WTF_Vector, FlatMapEmptyInnerVector)
+{
+    Vector<Vector<int>> vector {
+        { },
+        { 1, 2 },
+        { },
+        { 3, 4 },
+        { },
+    };
+
+    static_assert(std::is_same<decltype(WTF::flatMap(vector, mapVector)), typename WTF::Vector<int>>::value,
+        "WTF::flatMap returns Vector<int>");
+    auto mapped = WTF::flatMap(vector, mapVector);
+
+    EXPECT_EQ(4U, mapped.size());
+    EXPECT_EQ(1, mapped[0]);
+    EXPECT_EQ(2, mapped[1]);
+    EXPECT_EQ(3, mapped[2]);
+    EXPECT_EQ(4, mapped[3]);
+}
+
+struct InnerStruct {
+    Vector<int> vector;
+};
+
+static Vector<int> mapInnerStruct(InnerStruct innerStruct)
+{
+    return innerStruct.vector;
+}
+
+TEST(WTF_Vector, FlatMapInnerStruct)
+{
+    Vector<InnerStruct> vector {
+        { { 1, 2 } },
+        { { 3, 4 } },
+    };
+
+    static_assert(std::is_same<decltype(WTF::flatMap(vector, mapInnerStruct)), typename WTF::Vector<int>>::value,
+        "WTF::flatMap returns Vector<int>");
+    auto mapped = WTF::flatMap(vector, mapInnerStruct);
+
+    EXPECT_EQ(4U, mapped.size());
+    EXPECT_EQ(1, mapped[0]);
+    EXPECT_EQ(2, mapped[1]);
+    EXPECT_EQ(3, mapped[2]);
+    EXPECT_EQ(4, mapped[3]);
+}
+
 } // namespace TestWebKitAPI
