@@ -308,6 +308,15 @@ TEST_F(GStreamerTest, harnessDecodeMP4Video)
 {
     // Process an MP4 file, expecting non-video streams will be discarded.
     GRefPtr<GstElement> element = gst_element_factory_make("decodebin3", nullptr);
+
+    // Disable internal buffering in decodebin3.
+    for (auto child : GstIteratorAdaptor<GstElement>(GUniquePtr<GstIterator>(gst_bin_iterate_recurse(GST_BIN_CAST(element.get()))))) {
+        if (g_str_has_prefix(GST_OBJECT_NAME(GST_OBJECT_CAST(child)), "multiqueue")) {
+            g_object_set(child, "max-size-buffers", 1, nullptr);
+            break;
+        }
+    }
+
     struct BufferTracker {
         uint64_t totalVideoBuffers { 0 };
     } bufferTracker;
